@@ -2,32 +2,32 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifndef BASE_ALLOCATOR_PARTITION_ALLOCATOR_SRC_PARTITION_ALLOC_PARTITION_ADDRESS_SPACE_H_
-#define BASE_ALLOCATOR_PARTITION_ALLOCATOR_SRC_PARTITION_ALLOC_PARTITION_ADDRESS_SPACE_H_
+#ifndef PARTITION_ALLOC_PARTITION_ADDRESS_SPACE_H_
+#define PARTITION_ALLOC_PARTITION_ADDRESS_SPACE_H_
 
-#include <bit>
 #include <cstddef>
 #include <utility>
 
-#include "build/build_config.h"
 #include "partition_alloc/address_pool_manager_types.h"
+#include "partition_alloc/build_config.h"
+#include "partition_alloc/buildflags.h"
 #include "partition_alloc/page_allocator_constants.h"
+#include "partition_alloc/partition_alloc_base/bits.h"
 #include "partition_alloc/partition_alloc_base/compiler_specific.h"
 #include "partition_alloc/partition_alloc_base/component_export.h"
 #include "partition_alloc/partition_alloc_base/notreached.h"
-#include "partition_alloc/partition_alloc_buildflags.h"
 #include "partition_alloc/partition_alloc_check.h"
 #include "partition_alloc/partition_alloc_config.h"
 #include "partition_alloc/partition_alloc_constants.h"
 #include "partition_alloc/partition_alloc_forward.h"
 #include "partition_alloc/thread_isolation/alignment.h"
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
 #include "partition_alloc/thread_isolation/thread_isolation.h"
 #endif
 
 // The feature is not applicable to 32-bit address space.
-#if BUILDFLAG(HAS_64_BIT_POINTERS)
+#if PA_BUILDFLAG(HAS_64_BIT_POINTERS)
 
 namespace partition_alloc {
 
@@ -41,14 +41,21 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   struct PoolInfo {
     pool_handle handle;
     uintptr_t base;
+    uintptr_t base_mask;
     uintptr_t offset;
   };
 
 #if PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
+  PA_ALWAYS_INLINE static uintptr_t BRPPoolBaseMask() {
+    return setup_.brp_pool_base_mask_;
+  }
   PA_ALWAYS_INLINE static uintptr_t RegularPoolBaseMask() {
     return setup_.regular_pool_base_mask_;
   }
 #else
+  PA_ALWAYS_INLINE static constexpr uintptr_t BRPPoolBaseMask() {
+    return kBRPPoolBaseMask;
+  }
   PA_ALWAYS_INLINE static constexpr uintptr_t RegularPoolBaseMask() {
     return kRegularPoolBaseMask;
   }
@@ -56,33 +63,41 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
 
   PA_ALWAYS_INLINE static PoolInfo GetPoolInfo(uintptr_t address) {
     // When USE_BACKUP_REF_PTR is off, BRP pool isn't used.
-#if !BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#if !PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
     PA_DCHECK(!IsInBRPPool(address));
 #endif
     pool_handle pool = kNullPoolHandle;
     uintptr_t base = 0;
-#if BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+    uintptr_t base_mask = 0;
+#if PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
     if (IsInBRPPool(address)) {
       pool = kBRPPoolHandle;
       base = setup_.brp_pool_base_address_;
+      base_mask = BRPPoolBaseMask();
     } else
-#endif  // BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#endif  // PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
       if (IsInRegularPool(address)) {
         pool = kRegularPoolHandle;
         base = setup_.regular_pool_base_address_;
+        base_mask = RegularPoolBaseMask();
       } else if (IsInConfigurablePool(address)) {
         PA_DCHECK(IsConfigurablePoolInitialized());
         pool = kConfigurablePoolHandle;
         base = setup_.configurable_pool_base_address_;
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+        base_mask = setup_.configurable_pool_base_mask_;
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
       } else if (IsInThreadIsolatedPool(address)) {
         pool = kThreadIsolatedPoolHandle;
         base = setup_.thread_isolated_pool_base_address_;
+        base_mask = kThreadIsolatedPoolBaseMask;
 #endif
       } else {
         PA_NOTREACHED();
       }
-    return PoolInfo{.handle = pool, .base = base, .offset = address - base};
+    return PoolInfo{.handle = pool,
+                    .base = base,
+                    .base_mask = base_mask,
+                    .offset = address - base};
   }
   PA_ALWAYS_INLINE static constexpr size_t ConfigurablePoolMaxSize() {
     return kConfigurablePoolMaxSize;
@@ -101,7 +116,7 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   //
   // This function must only be called from the main thread.
   static void InitConfigurablePool(uintptr_t pool_base, size_t size);
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   static void InitThreadIsolatedPool(ThreadIsolationOption thread_isolation);
   static void UninitThreadIsolatedPoolForTesting();
 #endif
@@ -125,7 +140,7 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
            kUninitializedPoolBaseAddress;
   }
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   PA_ALWAYS_INLINE static bool IsThreadIsolatedPoolInitialized() {
     return setup_.thread_isolated_pool_base_address_ !=
            kUninitializedPoolBaseAddress;
@@ -157,7 +172,17 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
     return (address & brp_pool_base_mask) == setup_.brp_pool_base_address_;
   }
 
-#if BUILDFLAG(GLUE_CORE_POOLS)
+#if PA_CONFIG(ENABLE_SHADOW_METADATA)
+  PA_ALWAYS_INLINE static uintptr_t BRPPoolBase() {
+#if PA_BUILDFLAG(GLUE_CORE_POOLS)
+    return RegularPoolBase() + RegularPoolSize();
+#else
+    return setup_.brp_pool_base_address_;
+#endif  // PA_BUILDFLAG(GLUE_CORE_POOLS)
+  }
+#endif  // PA_CONFIG(ENABLE_SHADOW_METADATA)
+
+#if PA_BUILDFLAG(GLUE_CORE_POOLS)
   // Checks whether the address belongs to either regular or BRP pool.
   // Returns false for nullptr.
   PA_ALWAYS_INLINE static bool IsInCorePools(uintptr_t address) {
@@ -183,7 +208,7 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
     return RegularPoolSize() * 2;
   }
 #endif  // PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
-#endif  // BUILDFLAG(GLUE_CORE_POOLS)
+#endif  // PA_BUILDFLAG(GLUE_CORE_POOLS)
 
   PA_ALWAYS_INLINE static uintptr_t OffsetInBRPPool(uintptr_t address) {
     PA_DCHECK(IsInBRPPool(address));
@@ -200,7 +225,7 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
     return setup_.configurable_pool_base_address_;
   }
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   // Returns false for nullptr.
   PA_ALWAYS_INLINE static bool IsInThreadIsolatedPool(uintptr_t address) {
     return (address & kThreadIsolatedPoolBaseMask) ==
@@ -209,19 +234,109 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
 #endif
 
 #if PA_CONFIG(ENABLE_SHADOW_METADATA)
-  PA_ALWAYS_INLINE static std::ptrdiff_t ShadowPoolOffset(pool_handle pool) {
-    if (pool == kRegularPoolHandle) {
-      return regular_pool_shadow_offset_;
-    } else if (pool == kBRPPoolHandle) {
-      return brp_pool_shadow_offset_;
-    } else {
-      // TODO(crbug.com/1362969): Add shadow for configurable pool as well.
-      // Shadow is not created for ConfigurablePool for now, so this part should
-      // be unreachable.
-      PA_NOTREACHED();
+  PA_ALWAYS_INLINE static bool IsShadowMetadataEnabledOnRegularPool() {
+    return regular_pool_fd_ != -1;
+  }
+
+  PA_ALWAYS_INLINE static bool IsShadowMetadataEnabledOnBRPPool() {
+    return brp_pool_fd_ != -1;
+  }
+
+  PA_ALWAYS_INLINE static bool IsShadowMetadataEnabledOnConfigurablePool() {
+    return configurable_pool_fd_ != -1;
+  }
+
+  PA_ALWAYS_INLINE static bool IsShadowMetadataEnabled(pool_handle pool) {
+    switch (pool) {
+      case kRegularPoolHandle:
+        return IsShadowMetadataEnabledOnRegularPool();
+      case kBRPPoolHandle:
+        return IsShadowMetadataEnabledOnBRPPool();
+      case kConfigurablePoolHandle:
+        return IsShadowMetadataEnabledOnConfigurablePool();
+      default:
+        return false;
     }
   }
-#endif
+
+  // To reduce the cost of address conversion (metadata address inside Regular
+  // Pool to its shadow metadata address), we will make the size of the address
+  // space of shadow metadata the same as `max(regular pool size, brp
+  // pool size, configurable pool size)` (only 1 shadow address space. Not 3)
+  // So we need to use different offset for metadata of the regular pool's
+  // SuperPages and for the brp pool's SuperPages.
+  // i.e. |kSystemPageOffsetOfRegularPoolShadow| and
+  // |kSystemPageOffsetOfBRPPoolShadow|.
+  //
+  // i: the index of SystemPage for metadata inside the regular pool's
+  // SuperPage.
+  //    (currently, the index is 1.)
+  //
+  //     i-th
+  // +------------+
+  // | SystemPage | (regular pool)
+  // +------------+
+  //       \
+  //        \ mapping
+  //         \
+  //      (i+kSystemPageOffsetOfRegularPoolShadow)-th
+  //     +------------+
+  //     | SystemPage | (shadow)
+  //     +------------+
+  //
+  // (i + kSystemPageOffsetOfRegularPoolShadow)-th SystemPage inside the matched
+  // SuperPage inside the shadow pool is used for the metadata.
+  static constexpr size_t kSystemPageOffsetOfRegularPoolShadow = 0u;
+  static constexpr size_t kSystemPageOffsetOfBRPPoolShadow = 2u;
+  static constexpr size_t kSystemPageOffsetOfConfigurablePoolShadow = 4u;
+
+  static size_t RegularPoolShadowSize();
+  static size_t BRPPoolShadowSize();
+  static size_t ConfigurablePoolShadowSize();
+
+  PA_ALWAYS_INLINE static std::ptrdiff_t RegularPoolShadowOffset() {
+    return regular_pool_shadow_offset_;
+  }
+
+  PA_ALWAYS_INLINE static std::ptrdiff_t BRPPoolShadowOffset() {
+    return brp_pool_shadow_offset_;
+  }
+
+  PA_ALWAYS_INLINE static std::ptrdiff_t ConfigurablePoolShadowOffset() {
+    return configurable_pool_shadow_offset_;
+  }
+
+  // TODO(crbug.com/40238514): Confirm we can use kConfigurablePoolMaxSize/4
+  // for iOS and confirm iOS EarlyGrey tests pass when the shadow metadata
+  // is enabled, since IIRC iOS limits virtual address space too.
+  static_assert(
+      !PA_BUILDFLAG(IS_IOS),
+      "kConfigurablePoolMaxSize is too large to run iOS EarlyGrey tests, "
+      "because the test process cannot use an extended virtual address space. "
+      "Temporarily disable ShadowMetadata feature on iOS");
+
+#if PA_BUILDFLAG(DCHECKS_ARE_ON)
+  // Check whether the given |ptr| points to an address inside the address space
+  // reserved for the regular and brp shadow. However the result |true| doesn't
+  // mean the given |ptr| is valid. Because we don't use the entire address
+  // space for the shadow. We only use 2 SystemPageSize() / kSuperPageSize(%)
+  // of the space.
+  //
+  // TODO(crbug.com/40238514) This is an unused function. Start using it in
+  // tests and/or in production code.
+  PA_ALWAYS_INLINE static bool IsInPoolShadow(const void* ptr) {
+    uintptr_t ptr_as_uintptr = reinterpret_cast<uintptr_t>(ptr);
+    return (pool_shadow_address_ <= ptr_as_uintptr &&
+            (ptr_as_uintptr < pool_shadow_address_ + RegularPoolSize() ||
+             ptr_as_uintptr < pool_shadow_address_ + BRPPoolSize() ||
+             ptr_as_uintptr < pool_shadow_address_ + kConfigurablePoolMaxSize));
+  }
+#endif  // PA_BUILDFLAG(DCHECKS_ARE_ON)
+
+  static void InitShadowMetadata(PoolHandleMask pool);
+  static void MapMetadata(uintptr_t super_page, bool copy_metadata);
+  static void UnmapShadowMetadata(uintptr_t super_page, pool_handle pool);
+#endif  // PA_CONFIG(ENABLE_SHADOW_METADATA)
 
   // PartitionAddressSpace is static_only class.
   PartitionAddressSpace() = delete;
@@ -243,7 +358,7 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   }
 #endif  // PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   PA_ALWAYS_INLINE static constexpr size_t ThreadIsolatedPoolSize() {
     return kThreadIsolatedPoolSize;
   }
@@ -270,19 +385,19 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   // ArrayBuffers be located inside of it.
   static constexpr size_t kRegularPoolSize = kPoolMaxSize;
   static constexpr size_t kBRPPoolSize = kPoolMaxSize;
-  static_assert(std::has_single_bit(kRegularPoolSize));
-  static_assert(std::has_single_bit(kBRPPoolSize));
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+  static_assert(base::bits::HasSingleBit(kRegularPoolSize));
+  static_assert(base::bits::HasSingleBit(kBRPPoolSize));
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   static constexpr size_t kThreadIsolatedPoolSize = kGiB / 4;
-  static_assert(std::has_single_bit(kThreadIsolatedPoolSize));
+  static_assert(base::bits::HasSingleBit(kThreadIsolatedPoolSize));
 #endif
   static constexpr size_t kConfigurablePoolMaxSize = kPoolMaxSize;
   static constexpr size_t kConfigurablePoolMinSize = 1 * kGiB;
   static_assert(kConfigurablePoolMinSize <= kConfigurablePoolMaxSize);
-  static_assert(std::has_single_bit(kConfigurablePoolMaxSize));
-  static_assert(std::has_single_bit(kConfigurablePoolMinSize));
+  static_assert(base::bits::HasSingleBit(kConfigurablePoolMaxSize));
+  static_assert(base::bits::HasSingleBit(kConfigurablePoolMinSize));
 
-#if BUILDFLAG(IS_IOS)
+#if PA_BUILDFLAG(IS_IOS)
 
 #if !PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
 #error iOS is only supported with a dynamically sized GigaCase.
@@ -295,9 +410,9 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   static constexpr size_t kBRPPoolSizeForIOSTestProcess = kGiB / 4;
   static_assert(kRegularPoolSizeForIOSTestProcess < kRegularPoolSize);
   static_assert(kBRPPoolSizeForIOSTestProcess < kBRPPoolSize);
-  static_assert(std::has_single_bit(kRegularPoolSizeForIOSTestProcess));
-  static_assert(std::has_single_bit(kBRPPoolSizeForIOSTestProcess));
-#endif  // BUILDFLAG(IOS_IOS)
+  static_assert(base::bits::HasSingleBit(kRegularPoolSizeForIOSTestProcess));
+  static_assert(base::bits::HasSingleBit(kBRPPoolSizeForIOSTestProcess));
+#endif  // PA_BUILDFLAG(IOS_IOS)
 
 #if !PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
   // Masks used to easy determine belonging to a pool.
@@ -309,7 +424,7 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   static constexpr uintptr_t kBRPPoolBaseMask = ~kBRPPoolOffsetMask;
 #endif  // !PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   static constexpr uintptr_t kThreadIsolatedPoolOffsetMask =
       static_cast<uintptr_t>(kThreadIsolatedPoolSize) - 1;
   static constexpr uintptr_t kThreadIsolatedPoolBaseMask =
@@ -331,23 +446,23 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
     uintptr_t regular_pool_base_address_ = kUninitializedPoolBaseAddress;
     uintptr_t brp_pool_base_address_ = kUninitializedPoolBaseAddress;
     uintptr_t configurable_pool_base_address_ = kUninitializedPoolBaseAddress;
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
     uintptr_t thread_isolated_pool_base_address_ =
         kUninitializedPoolBaseAddress;
 #endif
 #if PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
     uintptr_t regular_pool_base_mask_ = 0;
     uintptr_t brp_pool_base_mask_ = 0;
-#if BUILDFLAG(GLUE_CORE_POOLS)
+#if PA_BUILDFLAG(GLUE_CORE_POOLS)
     uintptr_t core_pools_base_mask_ = 0;
 #endif
 #endif  // PA_CONFIG(DYNAMICALLY_SELECT_POOL_SIZE)
     uintptr_t configurable_pool_base_mask_ = 0;
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
     ThreadIsolationOption thread_isolation_;
 #endif
   };
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   static_assert(sizeof(PoolSetup) % SystemPageSize() == 0,
                 "PoolSetup has to fill a page(s)");
 #else
@@ -360,14 +475,20 @@ class PA_COMPONENT_EXPORT(PARTITION_ALLOC) PartitionAddressSpace {
   // These are write-once fields, frequently accessed thereafter. Make sure they
   // don't share a cacheline with other, potentially writeable data, through
   // alignment and padding.
-  static PoolSetup setup_ PA_CONSTINIT;
+  PA_CONSTINIT static PoolSetup setup_;
 
 #if PA_CONFIG(ENABLE_SHADOW_METADATA)
   static std::ptrdiff_t regular_pool_shadow_offset_;
   static std::ptrdiff_t brp_pool_shadow_offset_;
-#endif
+  static std::ptrdiff_t configurable_pool_shadow_offset_;
+  // TODO(crbug.com/40238514): Use platform file handles instead of |int|.
+  static int regular_pool_fd_;
+  static int brp_pool_fd_;
+  static int configurable_pool_fd_;
+  static uintptr_t pool_shadow_address_;
+#endif  // PA_CONFIG(ENABLE_SHADOW_METADATA)
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
   // If we use thread isolation, we need to write-protect its metadata.
   // Allow the function to get access to the PoolSetup.
   friend void WriteProtectThreadIsolatedGlobals(ThreadIsolationOption);
@@ -387,28 +508,28 @@ PA_ALWAYS_INLINE uintptr_t OffsetInBRPPool(uintptr_t address) {
   return PartitionAddressSpace::OffsetInBRPPool(address);
 }
 
-#if PA_CONFIG(ENABLE_SHADOW_METADATA)
-PA_ALWAYS_INLINE std::ptrdiff_t ShadowPoolOffset(pool_handle pool) {
-  return PartitionAddressSpace::ShadowPoolOffset(pool);
-}
-#endif
-
 }  // namespace internal
 
 // Returns false for nullptr.
 PA_ALWAYS_INLINE bool IsManagedByPartitionAlloc(uintptr_t address) {
   // When ENABLE_BACKUP_REF_PTR_SUPPORT is off, BRP pool isn't used.
-#if !BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#if !PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
   PA_DCHECK(!internal::PartitionAddressSpace::IsInBRPPool(address));
 #endif
-  return internal::PartitionAddressSpace::IsInRegularPool(address)
-#if BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
-         || internal::PartitionAddressSpace::IsInBRPPool(address)
+
+  return
+#if PA_BUILDFLAG(GLUE_CORE_POOLS)
+      internal::PartitionAddressSpace::IsInCorePools(address)
+#else
+#if PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+      internal::PartitionAddressSpace::IsInBRPPool(address) ||
 #endif
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
-         || internal::PartitionAddressSpace::IsInThreadIsolatedPool(address)
+      internal::PartitionAddressSpace::IsInRegularPool(address)
+#endif  // PA_BUILDFLAG(GLUE_CORE_POOLS)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
+      || internal::PartitionAddressSpace::IsInThreadIsolatedPool(address)
 #endif
-         || internal::PartitionAddressSpace::IsInConfigurablePool(address);
+      || internal::PartitionAddressSpace::IsInConfigurablePool(address);
 }
 
 // Returns false for nullptr.
@@ -421,13 +542,13 @@ PA_ALWAYS_INLINE bool IsManagedByPartitionAllocBRPPool(uintptr_t address) {
   return internal::PartitionAddressSpace::IsInBRPPool(address);
 }
 
-#if BUILDFLAG(GLUE_CORE_POOLS)
+#if PA_BUILDFLAG(GLUE_CORE_POOLS)
 // Checks whether the address belongs to either regular or BRP pool.
 // Returns false for nullptr.
 PA_ALWAYS_INLINE bool IsManagedByPartitionAllocCorePools(uintptr_t address) {
   return internal::PartitionAddressSpace::IsInCorePools(address);
 }
-#endif  // BUILDFLAG(GLUE_CORE_POOLS)
+#endif  // PA_BUILDFLAG(GLUE_CORE_POOLS)
 
 // Returns false for nullptr.
 PA_ALWAYS_INLINE bool IsManagedByPartitionAllocConfigurablePool(
@@ -435,7 +556,7 @@ PA_ALWAYS_INLINE bool IsManagedByPartitionAllocConfigurablePool(
   return internal::PartitionAddressSpace::IsInConfigurablePool(address);
 }
 
-#if BUILDFLAG(ENABLE_THREAD_ISOLATION)
+#if PA_BUILDFLAG(ENABLE_THREAD_ISOLATION)
 // Returns false for nullptr.
 PA_ALWAYS_INLINE bool IsManagedByPartitionAllocThreadIsolatedPool(
     uintptr_t address) {
@@ -449,6 +570,6 @@ PA_ALWAYS_INLINE bool IsConfigurablePoolAvailable() {
 
 }  // namespace partition_alloc
 
-#endif  // BUILDFLAG(HAS_64_BIT_POINTERS)
+#endif  // PA_BUILDFLAG(HAS_64_BIT_POINTERS)
 
-#endif  // BASE_ALLOCATOR_PARTITION_ALLOCATOR_SRC_PARTITION_ALLOC_PARTITION_ADDRESS_SPACE_H_
+#endif  // PARTITION_ALLOC_PARTITION_ADDRESS_SPACE_H_
