@@ -5,12 +5,12 @@
 // Holds functions for generating OOM errors from PartitionAlloc. This is
 // distinct from oom.h in that it is meant only for use in PartitionAlloc.
 
-#ifndef BASE_ALLOCATOR_PARTITION_ALLOCATOR_SRC_PARTITION_ALLOC_PARTITION_OOM_H_
-#define BASE_ALLOCATOR_PARTITION_ALLOCATOR_SRC_PARTITION_ALLOC_PARTITION_OOM_H_
+#ifndef PARTITION_ALLOC_PARTITION_OOM_H_
+#define PARTITION_ALLOC_PARTITION_OOM_H_
 
 #include <cstddef>
 
-#include "build/build_config.h"
+#include "partition_alloc/build_config.h"
 #include "partition_alloc/partition_alloc_base/compiler_specific.h"
 #include "partition_alloc/partition_alloc_base/component_export.h"
 
@@ -26,7 +26,7 @@ extern OomFunction g_oom_handling_function;
 [[noreturn]] PA_NOINLINE PA_COMPONENT_EXPORT(
     PARTITION_ALLOC) void PartitionExcessiveAllocationSize(size_t size);
 
-#if !defined(ARCH_CPU_64_BITS)
+#if !PA_BUILDFLAG(PA_ARCH_CPU_64_BITS)
 [[noreturn]] PA_NOINLINE void PartitionOutOfMemoryWithLotsOfUncommitedPages(
     size_t size);
 [[noreturn]] PA_NOINLINE void PartitionOutOfMemoryWithLargeVirtualSize(
@@ -37,4 +37,4 @@ extern OomFunction g_oom_handling_function;
 
 }  // namespace partition_alloc
 
-#endif  // BASE_ALLOCATOR_PARTITION_ALLOCATOR_SRC_PARTITION_ALLOC_PARTITION_OOM_H_
+#endif  // PARTITION_ALLOC_PARTITION_OOM_H_
