@@ -11,6 +11,8 @@
 #include <jni.h>
 
 #include "base/android/jni_string.h"
+
+// Must come after all headers that specialize FromJniType() / ToJniType().
 #include "base/base_jni/IntStringCallback_jni.h"
 
 namespace base {
@@ -19,7 +21,7 @@ namespace android {
 void RunIntStringCallbackAndroid(const JavaRef<jobject>& callback,
                                  int int_arg,
                                  const std::string& str_arg) {
-  JNIEnv* env = AttachCurrentThread();
+  JNIEnv* env = jni_zero::AttachCurrentThread();
   Java_IntStringCallback_onResult(env, callback, int_arg,
                                   ConvertUTF8ToJavaString(env, str_arg));
 }
