@@ -259,28 +259,11 @@ public class FileUtilsTest {
         assertFileList("");
     }
 
-    @Test
-    public void testBatchDeleteFiles() throws IOException {
-        // Batch delete files specified as path names.
-        prepareMixedFilesTestCase();
-        assertFileList("a1/; a1/b1/; a1/b1/c; a1/b1/c2; a1/b2/; a1/b2/c/; a1/b3; a2/; c");
-        FileUtils.batchDeleteFiles(getPathNames("a1/b1", "c", "nonexistent"), null);
-        assertFileList("a1/; a1/b2/; a1/b2/c/; a1/b3; a2/");
-        // Note that "b2" is not "a1/b2".
-        FileUtils.batchDeleteFiles(getPathNames("b2", "a1/b2/c"), null);
-        assertFileList("a1/; a1/b2/; a1/b3; a2/");
-        FileUtils.batchDeleteFiles(getPathNames("a1/b3", "a1", "a2", "a2", "a1", "a1/b2"), null);
-        assertFileList("");
-
-        // Omit testing content URL deletion.
-    }
-
     @Test
     public void testGetFileSize() throws IOException {
         Function<byte[], Boolean> runCase =
                 (byte[] inputBytes) -> {
                     ByteArrayInputStream inputStream = new ByteArrayInputStream(inputBytes);
-                    byte[] fileBytes;
                     long size;
                     try {
                         File tempFile = temporaryFolder.newFile();
@@ -341,7 +324,6 @@ public class FileUtilsTest {
                 (byte[] inputBytes) -> {
                     ByteArrayInputStream inputStream = new ByteArrayInputStream(inputBytes);
                     ByteArrayOutputStream verifyStream = new ByteArrayOutputStream();
-                    byte[] fileBytes;
                     try {
                         File tempFile = temporaryFolder.newFile();
                         FileUtils.copyStreamToFile(inputStream, tempFile);
@@ -389,10 +371,10 @@ public class FileUtilsTest {
 
     @Test
     public void testGetUriForFileWithContentUri() {
-        // ContentUriUtils needs to be initialized for "content://" URL to work. Use a fake
+        // FileProviderUtils needs to be initialized for "content://" URL to work. Use a fake
         // version to avoid dealing with Android innards, and to provide consistent results.
-        ContentUriUtils.setFileProviderUtil(
-                new ContentUriUtils.FileProviderUtil() {
+        FileProviderUtils.setFileProviderUtil(
+                new FileProviderUtils.FileProviderUtil() {
                     @Override
                     public Uri getContentUriFromFile(File file) {
                         Uri.Builder builder = new Uri.Builder();
@@ -423,7 +405,7 @@ public class FileUtilsTest {
 
     @Test
     public void testGetUriForFileWithoutContentUri() {
-        // Assumes contentUriUtils.setFileProviderUtil() is not called yet.
+        // Assumes FileProviderUtils.setFileProviderUtil() is not called yet.
         // Only test using absolute path. Otherwise cwd would be included into results.
         assertEquals("file:///", FileUtils.getUriForFile(new File("/")).toString());
         assertEquals("file:///foo.bar", FileUtils.getUriForFile(new File("/foo.bar")).toString());
