@@ -12,6 +12,8 @@ import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 
+import android.content.SharedPreferences;
+
 import androidx.test.filters.SmallTest;
 
 import org.junit.Before;
@@ -566,4 +568,30 @@ public class SharedPreferencesManagerTest {
         mSubject.removeKeysWithPrefix(TEST_PREFIX);
         verify(mChecker, times(7)).checkIsPrefixInUse(eq(TEST_PREFIX));
     }
+
+    @Test
+    @SmallTest
+    public void testCheckerIsCalledInEditor() {
+        final SharedPreferences.Editor ed = mSubject.getEditor();
+        ed.putInt("int_key", 123);
+        verify(mChecker).checkIsKeyInUse(eq("int_key"));
+
+        ed.putBoolean("bool_key", true);
+        verify(mChecker).checkIsKeyInUse(eq("bool_key"));
+
+        ed.putString("string_key", "foo");
+        verify(mChecker).checkIsKeyInUse(eq("string_key"));
+
+        ed.putLong("long_key", 999L);
+        verify(mChecker).checkIsKeyInUse(eq("long_key"));
+
+        ed.putFloat("float_key", 2.5f);
+        verify(mChecker).checkIsKeyInUse(eq("float_key"));
+
+        ed.putStringSet("string_set_key", new HashSet<>());
+        verify(mChecker).checkIsKeyInUse(eq("string_set_key"));
+
+        ed.remove("some_key");
+        verify(mChecker).checkIsKeyInUse(eq("some_key"));
+    }
 }
