@@ -2,17 +2,22 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <string>
+#include <string_view>
 #include <tuple>
 
 #include "base/check_deref.h"
 #include "base/check_version_internal.h"
+#include "base/dcheck_is_on.h"
 #include "base/debug/dump_without_crashing.h"
-#include "base/features.h"
 #include "base/functional/bind.h"
 #include "base/functional/callback.h"
 #include "base/logging.h"
 #include "base/macros/concat.h"
-#include "base/strings/string_piece.h"
+#include "base/notimplemented.h"
+#include "base/notreached.h"
+#include "base/strings/cstring_view.h"
+#include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/test/gtest_util.h"
 #include "base/test/scoped_feature_list.h"
@@ -20,6 +25,14 @@
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if BUILDFLAG(IS_POSIX)
+#include <errno.h>
+#endif
+
+#if BUILDFLAG(IS_WIN)
+#include <windows.h>
+#endif
+
 namespace {
 
 int g_dump_without_crashing_count = 0;
@@ -79,7 +92,7 @@ MATCHER_P2(LogErrorMatches, line, expected_msg, "") {
   } while (0)
 #else
 #define EXPECT_CHECK(msg, check_expr) \
-  EXPECT_DEATH_IF_SUPPORTED(check_expr, CHECK_MATCHER(__LINE__, msg))
+  BASE_EXPECT_DEATH(check_expr, CHECK_MATCHER(__LINE__, msg))
 #endif  // !CHECK_WILL_STREAM()
 
 // Macro which expects a DCHECK to fire if DCHECKs are enabled.
@@ -88,7 +101,7 @@ MATCHER_P2(LogErrorMatches, line, expected_msg, "") {
 #define EXPECT_DCHECK(msg, check_expr)                                         \
   do {                                                                         \
     if (DCHECK_IS_ON() && logging::LOGGING_DCHECK == logging::LOGGING_FATAL) { \
-      EXPECT_DEATH_IF_SUPPORTED(check_expr, CHECK_MATCHER(__LINE__, msg));     \
+      BASE_EXPECT_DEATH(check_expr, CHECK_MATCHER(__LINE__, msg));             \
     } else if (DCHECK_IS_ON()) {                                               \
       ScopedExpectDumpWithoutCrashing expect_dump;                             \
       check_expr;                                                              \
@@ -109,7 +122,7 @@ MATCHER_P2(LogErrorMatches, line, expected_msg, "") {
       got_log_message = true;                                                  \
       EXPECT_EQ(severity, logging::LOGGING_ERROR);                             \
       EXPECT_EQ(str.substr(message_start), (msg));                             \
-      if (base::StringPiece(expected_file) != "") {                            \
+      if (std::string_view(expected_file) != "") {                             \
         EXPECT_STREQ(expected_file, file);                                     \
       }                                                                        \
       if (expected_line != -1) {                                               \
@@ -138,6 +151,7 @@ MATCHER_P2(LogErrorMatches, line, expected_msg, "") {
     logging::SetLogMessageHandler(nullptr);                                    \
   } while (0)
 
+#if defined(OFFICIAL_BUILD)
 #if DCHECK_IS_ON()
 #define EXPECT_DUMP_WILL_BE_CHECK EXPECT_DCHECK
 #else
@@ -149,6 +163,9 @@ MATCHER_P2(LogErrorMatches, line, expected_msg, "") {
                                    statement, expected_string "\n");        \
   } while (0)
 #endif  // DCHECK_IS_ON()
+#else
+#define EXPECT_DUMP_WILL_BE_CHECK EXPECT_CHECK
+#endif  // defined(OFFICIAL_BUILD)
 
 TEST(CheckDeathTest, Basics) {
   EXPECT_CHECK("Check failed: false. ", CHECK(false));
@@ -225,6 +242,37 @@ TEST(CheckDeathTest, CheckOp) {
                             DUMP_WILL_BE_CHECK_GT(a, b));
 }
 
+TEST(CheckDeathTest, CheckOpStrings) {
+  std::string_view sv = "1";
+  base::cstring_view csv = "2";
+  std::string s = "3";
+
+  EXPECT_CHECK("Check failed: sv == csv (1 vs. 2)", CHECK_EQ(sv, csv));
+  EXPECT_CHECK("Check failed: csv == s (2 vs. 3)", CHECK_EQ(csv, s));
+  EXPECT_CHECK("Check failed: sv == s (1 vs. 3)", CHECK_EQ(sv, s));
+
+  EXPECT_DCHECK("Check failed: sv == csv (1 vs. 2)", DCHECK_EQ(sv, csv));
+  EXPECT_DCHECK("Check failed: csv == s (2 vs. 3)", DCHECK_EQ(csv, s));
+  EXPECT_DCHECK("Check failed: sv == s (1 vs. 3)", DCHECK_EQ(sv, s));
+}
+
+TEST(CheckDeathTest, CheckOpPointers) {
+  uint8_t arr[] = {3, 2, 1, 0};
+  uint8_t* arr_start = &arr[0];
+  // Print pointers and not the binary data in `arr`.
+#if BUILDFLAG(IS_WIN)
+  EXPECT_CHECK(
+      "=~Check failed: arr_start != arr_start \\([0-9A-F]+ vs. "
+      "[0-9A-F]+\\)",
+      CHECK_NE(arr_start, arr_start));
+#else
+  EXPECT_CHECK(
+      "=~Check failed: arr_start != arr_start \\(0x[0-9a-f]+ vs. "
+      "0x[0-9a-f]+\\)",
+      CHECK_NE(arr_start, arr_start));
+#endif
+}
+
 TEST(CheckTest, CheckStreamsAreLazy) {
   int called_count = 0;
   int not_called_count = 0;
@@ -493,58 +541,24 @@ TEST(CheckDeathTest, OstreamVsToString) {
                CHECK_EQ(g, h));
 }
 
-// This non-void function is here to make sure that NOTREACHED_NORETURN() is
-// properly annotated as [[noreturn]] and does not require a return statement.
-int NotReachedNoreturnInFunction() {
-  NOTREACHED_NORETURN();
+// This non-void function is here to make sure that NOTREACHED() is properly
+// annotated as [[noreturn]] and does not require a return statement.
+int NotReachedInFunction() {
+  NOTREACHED();
   // No return statement here.
 }
 
 TEST(CheckDeathTest, NotReached) {
-#if BUILDFLAG(DCHECK_IS_CONFIGURABLE)
-  // This specific death test relies on LOGGING_DCHECK not being FATAL, even
-  // when run as part of a death test.
-  ScopedDcheckSeverity dcheck_severity(logging::LOGGING_ERROR);
-#endif
-
-#if DCHECK_IS_ON()
-  // Expect a DCHECK with streamed params intact.
-  EXPECT_DCHECK("Check failed: false. foo", NOTREACHED() << "foo");
-#elif CHECK_WILL_STREAM() || BUILDFLAG(ENABLE_LOG_ERROR_NOT_REACHED)
-  // This block makes sure that base::Location::Current() returns non-dummy
-  // values for file_name() and line_number(). This is necessary to avoid a
-  // false negative inside EXPECT_LOG_ERROR_WITH_FILENAME() where we exhonorate
-  // the NOTREACHED() macro below even though it didn't provide the expected
-  // filename and line numbers.
-  // See EXPECT_LOG_ERROR_WITH_FILENAME() for the exclusion of "" and -1.
-  ASSERT_NE(base::Location::Current().file_name(), nullptr);
-  EXPECT_STRNE(base::Location::Current().file_name(), "");
-  EXPECT_NE(base::Location::Current().line_number(), -1);
-  // Expect LOG(ERROR) that looks like CHECK(false) with streamed params intact.
-  // Note that this implementation uses base::Location::Current() which doesn't
-  // match __FILE__ (strips ../../ prefix) and __LINE__ (uses __builtin_LINE()).
-  EXPECT_LOG_ERROR_WITH_FILENAME(base::Location::Current().file_name(),
-                                 base::Location::Current().line_number(),
-                                 NOTREACHED() << "foo",
-                                 "Check failed: false. foo\n");
-#else
-  // Expect LOG(ERROR) that looks like CHECK(false) without file or line intact.
-  // We use `""` and `-1` to not expect a specific filename or line number.
-  // The actual location comes from
-  // logging::NotReachedError::TriggerNotReached() but we have no good way of
-  // asserting what that filename or line number is from here.
-  EXPECT_LOG_ERROR_WITH_FILENAME("", -1, NOTREACHED() << "foo",
-                                 "Check failed: false. NOTREACHED log messages "
-                                 "are omitted in official builds. Sorry!\n");
-#endif
-  EXPECT_DEATH_IF_SUPPORTED(NotReachedNoreturnInFunction(),
+  // Expect to be CHECK fatal but with a different error message.
+  EXPECT_CHECK("NOTREACHED hit. foo", NOTREACHED() << "foo");
+  // This call can't use EXPECT_CHECK as the NOTREACHED happens on a different
+  // line.
+  EXPECT_DEATH_IF_SUPPORTED(NotReachedInFunction(),
                             CHECK_WILL_STREAM() ? "NOTREACHED hit. " : "");
 }
 
-TEST(CheckDeathTest, NotReachedFatalExperiment) {
-  base::test::ScopedFeatureList feature_list(
-      base::features::kNotReachedIsFatal);
-  EXPECT_CHECK_DEATH(NOTREACHED());
+TEST(CheckDeathTest, NotReachedInMigration) {
+  EXPECT_CHECK_DEATH(NOTREACHED_IN_MIGRATION());
 }
 
 TEST(CheckDeathTest, DumpWillBeCheck) {
@@ -555,8 +569,8 @@ TEST(CheckDeathTest, DumpWillBeCheck) {
 }
 
 TEST(CheckDeathTest, DumpWillBeNotReachedNoreturn) {
-  EXPECT_DUMP_WILL_BE_CHECK("NOTREACHED hit. foo",
-                            DUMP_WILL_BE_NOTREACHED_NORETURN() << "foo");
+  EXPECT_DUMP_WILL_BE_CHECK("NOTREACHED hit. foo", DUMP_WILL_BE_NOTREACHED()
+                                                       << "foo");
 }
 
 static const std::string kNotImplementedMessage = "Not implemented reached in ";
@@ -642,7 +656,7 @@ TEST(CheckTest, CheckDerefOfPointer) {
 TEST(CheckDeathTest, CheckDerefOfNullPointer) {
   std::string* null_pointer = nullptr;
   EXPECT_CHECK("Check failed: null_pointer != nullptr. ",
-               CHECK_DEREF(null_pointer));
+               std::ignore = CHECK_DEREF(null_pointer));
 }
 
 // Test CHECK_DEREF of `const T*`
@@ -660,7 +674,7 @@ TEST(CheckTest, CheckDerefOfConstPointer) {
 TEST(CheckDeathTest, CheckDerefOfConstNullPointer) {
   std::string* const_null_pointer = nullptr;
   EXPECT_CHECK("Check failed: const_null_pointer != nullptr. ",
-               CHECK_DEREF(const_null_pointer));
+               std::ignore = CHECK_DEREF(const_null_pointer));
 }
 
 TEST(CheckDeathTest, CheckNotFatalUntil) {
@@ -735,4 +749,50 @@ TEST(CheckDeathTest, NotReachedNotFatalUntil) {
                                                << "foo");
 }
 
+TEST(CheckDeathTest, CorrectSystemErrorUsed) {
+#if BUILDFLAG(DCHECK_IS_CONFIGURABLE)
+  // DCHECKs are enabled, and LOGGING_DCHECK is mutable, but defaults to
+  // non-fatal. Set it to LOGGING_FATAL to get the expected behavior from the
+  // rest of this test.
+  ScopedDcheckSeverity dcheck_severity(logging::LOGGING_FATAL);
+#endif  // BUILDFLAG(DCHECK_IS_CONFIGURABLE)
+  const logging::SystemErrorCode kTestError = 28;
+  const std::string kExpectedCheckMessageRegex = base::StrCat(
+      {" Check failed: false. ", base::NumberToString(kTestError)});
+  const std::string kExpectedPCheckMessageRegex =
+      base::StrCat({" Check failed: false. ", base::NumberToString(kTestError),
+                    ": ", logging::SystemErrorCodeToString(kTestError)});
+  const std::string kExpectedNotreachedMessageRegex =
+      base::StrCat({" NOTREACHED hit. ", base::NumberToString(kTestError)});
+
+  auto set_last_error = [](logging::SystemErrorCode error) {
+#if BUILDFLAG(IS_WIN)
+    ::SetLastError(error);
+#else
+    errno = error;
+#endif
+  };
+
+  // Test that the last system error code was used as expected.
+  set_last_error(kTestError);
+  EXPECT_CHECK(kExpectedCheckMessageRegex,
+               CHECK(false) << logging::GetLastSystemErrorCode());
+
+  set_last_error(kTestError);
+  EXPECT_DCHECK(kExpectedCheckMessageRegex,
+                DCHECK(false) << logging::GetLastSystemErrorCode());
+
+  set_last_error(kTestError);
+  EXPECT_CHECK(kExpectedPCheckMessageRegex,
+               PCHECK(false) << logging::GetLastSystemErrorCode());
+
+  set_last_error(kTestError);
+  EXPECT_DCHECK(kExpectedPCheckMessageRegex,
+                DPCHECK(false) << logging::GetLastSystemErrorCode());
+
+  set_last_error(kTestError);
+  EXPECT_CHECK(kExpectedNotreachedMessageRegex,
+               NOTREACHED() << logging::GetLastSystemErrorCode());
+}
+
 }  // namespace
