@@ -2,20 +2,26 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/command_line.h"
 
 #include <ostream>
+#include <string_view>
 
 #include "base/check_op.h"
 #include "base/containers/contains.h"
 #include "base/containers/span.h"
+#include "base/debug/debugging_buildflags.h"
 #include "base/files/file_path.h"
 #include "base/logging.h"
 #include "base/notreached.h"
 #include "base/numerics/checked_math.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/strcat.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_tokenizer.h"
 #include "base/strings/string_util.h"
@@ -48,10 +54,10 @@ constexpr CommandLine::CharType kSwitchValueSeparator[] =
 // By putting slash last, we can control whether it is treaded as a switch
 // value by changing the value of switch_prefix_count to be one less than
 // the array size.
-constexpr CommandLine::StringPieceType kSwitchPrefixes[] = {L"--", L"-", L"/"};
+constexpr CommandLine::StringViewType kSwitchPrefixes[] = {L"--", L"-", L"/"};
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 // Unixes don't use slash as a switch.
-constexpr CommandLine::StringPieceType kSwitchPrefixes[] = {"--", "-"};
+constexpr CommandLine::StringViewType kSwitchPrefixes[] = {"--", "-"};
 #endif
 size_t switch_prefix_count = std::size(kSwitchPrefixes);
 
@@ -66,7 +72,7 @@ constexpr CommandLine::CharType kSingleArgument[] =
     FILE_PATH_LITERAL("single-argument");
 #endif  // BUILDFLAG(IS_WIN)
 
-size_t GetSwitchPrefixLength(CommandLine::StringPieceType string) {
+size_t GetSwitchPrefixLength(CommandLine::StringViewType string) {
   for (size_t i = 0; i < switch_prefix_count; ++i) {
     CommandLine::StringType prefix(kSwitchPrefixes[i]);
     if (string.substr(0, prefix.length()) == prefix)
@@ -95,8 +101,8 @@ bool IsSwitch(const CommandLine::StringType& string,
 
 // Returns true iff |string| represents a switch with key
 // |switch_key_without_prefix|, regardless of value.
-bool IsSwitchWithKey(CommandLine::StringPieceType string,
-                     CommandLine::StringPieceType switch_key_without_prefix) {
+bool IsSwitchWithKey(CommandLine::StringViewType string,
+                     CommandLine::StringViewType switch_key_without_prefix) {
   size_t prefix_length = GetSwitchPrefixLength(string);
   if (prefix_length == 0 || prefix_length == string.length())
     return false;
@@ -187,9 +193,37 @@ CommandLine::CommandLine(const StringVector& argv)
 }
 
 CommandLine::CommandLine(const CommandLine& other) = default;
-
+CommandLine::CommandLine(CommandLine&& other) noexcept
+    :
+#if BUILDFLAG(IS_WIN)
+      raw_command_line_string_(
+          std::exchange(other.raw_command_line_string_, StringViewType())),
+      has_single_argument_switch_(
+          std::exchange(other.has_single_argument_switch_, false)),
+#endif  // BUILDFLAG(IS_WIN)
+      argv_(std::exchange(other.argv_, StringVector(1))),
+      switches_(std::move(other.switches_)),
+      begin_args_(std::exchange(other.begin_args_, 1)) {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  other.sequence_checker_.Detach();
+#endif
+}
 CommandLine& CommandLine::operator=(const CommandLine& other) = default;
-
+CommandLine& CommandLine::operator=(CommandLine&& other) noexcept {
+#if BUILDFLAG(IS_WIN)
+  raw_command_line_string_ =
+      std::exchange(other.raw_command_line_string_, StringViewType());
+  has_single_argument_switch_ =
+      std::exchange(other.has_single_argument_switch_, false);
+#endif  // BUILDFLAG(IS_WIN)
+  argv_ = std::exchange(other.argv_, StringVector(1));
+  switches_ = std::move(other.switches_);
+  begin_args_ = std::exchange(other.begin_args_, 1);
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  other.sequence_checker_.Detach();
+#endif
+  return *this;
+}
 CommandLine::~CommandLine() = default;
 
 #if BUILDFLAG(IS_WIN)
@@ -260,9 +294,16 @@ bool CommandLine::InitializedForCurrentProcess() {
   return !!current_process_commandline_;
 }
 
+// static
+CommandLine CommandLine::FromArgvWithoutProgram(const StringVector& argv) {
+  CommandLine cmd(NO_PROGRAM);
+  cmd.AppendSwitchesAndArguments(argv);
+  return cmd;
+}
+
 #if BUILDFLAG(IS_WIN)
 // static
-CommandLine CommandLine::FromString(StringPieceType command_line) {
+CommandLine CommandLine::FromString(StringViewType command_line) {
   CommandLine cmd(NO_PROGRAM);
   cmd.ParseFromString(command_line);
   return cmd;
@@ -282,7 +323,9 @@ void CommandLine::InitFromArgv(const StringVector& argv) {
   switches_.clear();
   begin_args_ = 1;
   SetProgram(argv.empty() ? FilePath() : FilePath(argv[0]));
-  AppendSwitchesAndArguments(argv);
+  if (!argv.empty()) {
+    AppendSwitchesAndArguments(make_span(argv).subspan(1));
+  }
 }
 
 FilePath CommandLine::GetProgram() const {
@@ -290,6 +333,9 @@ FilePath CommandLine::GetProgram() const {
 }
 
 void CommandLine::SetProgram(const FilePath& program) {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  sequence_checker_.Check();
+#endif
 #if BUILDFLAG(IS_WIN)
   argv_[0] = StringType(TrimWhitespace(program.value(), TRIM_ALL));
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
@@ -299,16 +345,17 @@ void CommandLine::SetProgram(const FilePath& program) {
 #endif
 }
 
-bool CommandLine::HasSwitch(StringPiece switch_string) const {
+bool CommandLine::HasSwitch(std::string_view switch_string) const {
   DCHECK_EQ(ToLowerASCII(switch_string), switch_string);
   return Contains(switches_, switch_string);
 }
 
 bool CommandLine::HasSwitch(const char switch_constant[]) const {
-  return HasSwitch(StringPiece(switch_constant));
+  return HasSwitch(std::string_view(switch_constant));
 }
 
-std::string CommandLine::GetSwitchValueASCII(StringPiece switch_string) const {
+std::string CommandLine::GetSwitchValueASCII(
+    std::string_view switch_string) const {
   StringType value = GetSwitchValueNative(switch_string);
 #if BUILDFLAG(IS_WIN)
   if (!IsStringASCII(base::AsStringPiece16(value))) {
@@ -325,33 +372,36 @@ std::string CommandLine::GetSwitchValueASCII(StringPiece switch_string) const {
 #endif
 }
 
-FilePath CommandLine::GetSwitchValuePath(StringPiece switch_string) const {
+FilePath CommandLine::GetSwitchValuePath(std::string_view switch_string) const {
   return FilePath(GetSwitchValueNative(switch_string));
 }
 
 CommandLine::StringType CommandLine::GetSwitchValueNative(
-    StringPiece switch_string) const {
+    std::string_view switch_string) const {
   DCHECK_EQ(ToLowerASCII(switch_string), switch_string);
   auto result = switches_.find(switch_string);
   return result == switches_.end() ? StringType() : result->second;
 }
 
-void CommandLine::AppendSwitch(StringPiece switch_string) {
+void CommandLine::AppendSwitch(std::string_view switch_string) {
   AppendSwitchNative(switch_string, StringType());
 }
 
-void CommandLine::AppendSwitchPath(StringPiece switch_string,
+void CommandLine::AppendSwitchPath(std::string_view switch_string,
                                    const FilePath& path) {
   AppendSwitchNative(switch_string, path.value());
 }
 
-void CommandLine::AppendSwitchNative(StringPiece switch_string,
-                                     CommandLine::StringPieceType value) {
+void CommandLine::AppendSwitchNative(std::string_view switch_string,
+                                     CommandLine::StringViewType value) {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  sequence_checker_.Check();
+#endif
 #if BUILDFLAG(IS_WIN)
   const std::string switch_key = ToLowerASCII(switch_string);
   StringType combined_switch_string(UTF8ToWide(switch_key));
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
-  StringPiece switch_key = switch_string;
+  std::string_view switch_key = switch_string;
   StringType combined_switch_string(switch_key);
 #endif
   size_t prefix_length = GetSwitchPrefixLength(combined_switch_string);
@@ -375,8 +425,8 @@ void CommandLine::AppendSwitchNative(StringPiece switch_string,
   begin_args_ = (CheckedNumeric(begin_args_) + 1).ValueOrDie();
 }
 
-void CommandLine::AppendSwitchASCII(StringPiece switch_string,
-                                    StringPiece value_string) {
+void CommandLine::AppendSwitchASCII(std::string_view switch_string,
+                                    std::string_view value_string) {
 #if BUILDFLAG(IS_WIN)
   AppendSwitchNative(switch_string, UTF8ToWide(value_string));
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
@@ -386,7 +436,10 @@ void CommandLine::AppendSwitchASCII(StringPiece switch_string,
 #endif
 }
 
-void CommandLine::RemoveSwitch(base::StringPiece switch_key_without_prefix) {
+void CommandLine::RemoveSwitch(std::string_view switch_key_without_prefix) {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  sequence_checker_.Check();
+#endif
 #if BUILDFLAG(IS_WIN)
   StringType switch_key_native = UTF8ToWide(switch_key_without_prefix);
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
@@ -412,18 +465,17 @@ void CommandLine::RemoveSwitch(base::StringPiece switch_key_without_prefix) {
                                });
   if (expell == argv_switches_end) {
     NOTREACHED();
-    return;
   }
   begin_args_ -= argv_switches_end - expell;
   argv_.erase(expell, argv_switches_end);
 }
 
 void CommandLine::CopySwitchesFrom(const CommandLine& source,
-                                   const char* const switches[],
-                                   size_t count) {
-  for (size_t i = 0; i < count; ++i) {
-    if (source.HasSwitch(switches[i]))
-      AppendSwitchNative(switches[i], source.GetSwitchValueNative(switches[i]));
+                                   span<const char* const> switches) {
+  for (const char* entry : switches) {
+    if (source.HasSwitch(entry)) {
+      AppendSwitchNative(entry, source.GetSwitchValueNative(entry));
+    }
   }
 }
 
@@ -437,7 +489,7 @@ CommandLine::StringVector CommandLine::GetArgs() const {
   return args;
 }
 
-void CommandLine::AppendArg(StringPiece value) {
+void CommandLine::AppendArg(std::string_view value) {
 #if BUILDFLAG(IS_WIN)
   DCHECK(IsStringUTF8(value));
   AppendArgNative(UTF8ToWide(value));
@@ -452,7 +504,10 @@ void CommandLine::AppendArgPath(const FilePath& path) {
   AppendArgNative(path.value());
 }
 
-void CommandLine::AppendArgNative(StringPieceType value) {
+void CommandLine::AppendArgNative(StringViewType value) {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  sequence_checker_.Check();
+#endif
   argv_.push_back(StringType(value));
 }
 
@@ -460,22 +515,28 @@ void CommandLine::AppendArguments(const CommandLine& other,
                                   bool include_program) {
   if (include_program)
     SetProgram(other.GetProgram());
-  AppendSwitchesAndArguments(other.argv());
+  if (!other.argv().empty()) {
+    AppendSwitchesAndArguments(make_span(other.argv()).subspan(1));
+  }
 }
 
-void CommandLine::PrependWrapper(StringPieceType wrapper) {
+void CommandLine::PrependWrapper(StringViewType wrapper) {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  sequence_checker_.Check();
+#endif
   if (wrapper.empty())
     return;
   // Split the wrapper command based on whitespace (with quoting).
-  // StringPieceType does not currently work directly with StringTokenizerT.
+  // StringViewType does not currently work directly with StringTokenizerT.
   using CommandLineTokenizer =
       StringTokenizerT<StringType, StringType::const_iterator>;
   StringType wrapper_string(wrapper);
   CommandLineTokenizer tokenizer(wrapper_string, FILE_PATH_LITERAL(" "));
   tokenizer.set_quote_chars(FILE_PATH_LITERAL("'\""));
   std::vector<StringType> wrapper_argv;
-  while (tokenizer.GetNext())
-    wrapper_argv.emplace_back(tokenizer.token());
+  while (std::optional<StringViewType> token = tokenizer.GetNextTokenView()) {
+    wrapper_argv.emplace_back(token.value());
+  }
 
   // Prepend the wrapper and update the switches/arguments |begin_args_|.
   argv_.insert(argv_.begin(), wrapper_argv.begin(), wrapper_argv.end());
@@ -483,7 +544,7 @@ void CommandLine::PrependWrapper(StringPieceType wrapper) {
 }
 
 #if BUILDFLAG(IS_WIN)
-void CommandLine::ParseFromString(StringPieceType command_line) {
+void CommandLine::ParseFromString(StringViewType command_line) {
   command_line = TrimWhitespace(command_line, TRIM_ALL);
   if (command_line.empty())
     return;
@@ -492,7 +553,7 @@ void CommandLine::ParseFromString(StringPieceType command_line) {
   int num_args = 0;
   wchar_t** args = NULL;
   // When calling CommandLineToArgvW, use the apiset if available.
-  // Doing so will bypass loading shell32.dll on Win8+.
+  // Doing so will bypass loading shell32.dll on Windows.
   HMODULE downlevel_shell32_dll =
       ::LoadLibraryEx(L"api-ms-win-downlevel-shell32-l1-1-0.dll", nullptr,
                       LOAD_LIBRARY_SEARCH_SYSTEM32);
@@ -512,7 +573,7 @@ void CommandLine::ParseFromString(StringPieceType command_line) {
                          << command_line;
   StringVector argv(args, args + num_args);
   InitFromArgv(argv);
-  raw_command_line_string_ = StringPieceType();
+  raw_command_line_string_ = StringViewType();
   LocalFree(args);
 
   if (downlevel_shell32_dll)
@@ -521,14 +582,12 @@ void CommandLine::ParseFromString(StringPieceType command_line) {
 
 #endif  // BUILDFLAG(IS_WIN)
 
-void CommandLine::AppendSwitchesAndArguments(
-    const CommandLine::StringVector& argv) {
+void CommandLine::AppendSwitchesAndArguments(span<const StringType> argv) {
   bool parse_switches = true;
 #if BUILDFLAG(IS_WIN)
   const bool is_parsed_from_string = !raw_command_line_string_.empty();
 #endif
-  for (size_t i = 1; i < argv.size(); ++i) {
-    CommandLine::StringType arg = argv[i];
+  for (StringType arg : argv) {
 #if BUILDFLAG(IS_WIN)
     arg = CommandLine::StringType(TrimWhitespace(arg, TRIM_ALL));
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
@@ -682,11 +741,17 @@ void CommandLine::ParseAsSingleArgument(
   if (arg_position >= raw_command_line_string_.length())
     return;
   has_single_argument_switch_ = true;
-  const StringPieceType arg = raw_command_line_string_.substr(arg_position);
+  const StringViewType arg = raw_command_line_string_.substr(arg_position);
   if (!arg.empty()) {
     AppendArgNative(arg);
   }
 }
 #endif  // BUILDFLAG(IS_WIN)
 
+void CommandLine::DetachFromCurrentSequence() {
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+  sequence_checker_.Detach();
+#endif  // BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+}
+
 }  // namespace base
