@@ -17,6 +17,7 @@
 #define BASE_COMMAND_LINE_H_
 
 #include <stddef.h>
+
 #include <functional>
 #include <map>
 #include <memory>
@@ -27,7 +28,6 @@
 #include "base/base_export.h"
 #include "base/containers/span.h"
 #include "base/debug/debugging_buildflags.h"
-#include "base/strings/string_piece.h"
 #include "build/build_config.h"
 
 #if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
@@ -49,8 +49,8 @@ class BASE_EXPORT CommandLine {
 #endif
 
   using CharType = StringType::value_type;
-  using StringPieceType = std::basic_string_view<CharType>;
   using StringVector = std::vector<StringType>;
+  using StringViewType = std::basic_string_view<CharType>;
   using SwitchMap = std::map<std::string, StringType, std::less<>>;
 
   // Returns CommandLine object constructed with switches and keys alone.
@@ -59,7 +59,7 @@ class BASE_EXPORT CommandLine {
   static CommandLine FromArgvWithoutProgram(const StringVector& argv);
 
 #if BUILDFLAG(IS_WIN)
-  static CommandLine FromString(StringPieceType command_line);
+  static CommandLine FromString(StringViewType command_line);
 #endif
 
   // A constructor for CommandLines that only carry switches and arguments.
@@ -80,6 +80,9 @@ class BASE_EXPORT CommandLine {
   CommandLine(const CommandLine& other);
   CommandLine& operator=(const CommandLine& other);
 
+  CommandLine(CommandLine&& other) noexcept;
+  CommandLine& operator=(CommandLine&& other) noexcept;
+
   ~CommandLine();
 
 #if BUILDFLAG(IS_WIN)
@@ -187,30 +190,31 @@ class BASE_EXPORT CommandLine {
   // Switch names must be lowercase.
   // The second override provides an optimized version to avoid inlining codegen
   // at every callsite to find the length of the constant and construct a
-  // StringPiece.
-  bool HasSwitch(StringPiece switch_string) const;
+  // std::string_view.
+  bool HasSwitch(std::string_view switch_string) const;
   bool HasSwitch(const char switch_constant[]) const;
 
   // Returns the value associated with the given switch. If the switch has no
   // value or isn't present, this method returns the empty string.
   // Switch names must be lowercase.
-  std::string GetSwitchValueASCII(StringPiece switch_string) const;
-  FilePath GetSwitchValuePath(StringPiece switch_string) const;
-  StringType GetSwitchValueNative(StringPiece switch_string) const;
+  std::string GetSwitchValueASCII(std::string_view switch_string) const;
+  FilePath GetSwitchValuePath(std::string_view switch_string) const;
+  StringType GetSwitchValueNative(std::string_view switch_string) const;
 
   // Get a copy of all switches, along with their values.
   const SwitchMap& GetSwitches() const { return switches_; }
 
   // Append a switch [with optional value] to the command line.
   // Note: Switches will precede arguments regardless of appending order.
-  void AppendSwitch(StringPiece switch_string);
-  void AppendSwitchPath(StringPiece switch_string, const FilePath& path);
-  void AppendSwitchNative(StringPiece switch_string, StringPieceType value);
-  void AppendSwitchASCII(StringPiece switch_string, StringPiece value);
+  void AppendSwitch(std::string_view switch_string);
+  void AppendSwitchPath(std::string_view switch_string, const FilePath& path);
+  void AppendSwitchNative(std::string_view switch_string, StringViewType value);
+  void AppendSwitchASCII(std::string_view switch_string,
+                         std::string_view value);
 
   // Removes the switch that matches |switch_key_without_prefix|, regardless of
   // prefix and value. If no such switch is present, this has no effect.
-  void RemoveSwitch(const base::StringPiece switch_key_without_prefix);
+  void RemoveSwitch(std::string_view switch_key_without_prefix);
 
   // Copies a set of switches (and any values) from another command line.
   // Commonly used when launching a subprocess.
@@ -225,9 +229,9 @@ class BASE_EXPORT CommandLine {
   // properly such that it is interpreted as one argument to the target command.
   // AppendArg is primarily for ASCII; non-ASCII input is interpreted as UTF-8.
   // Note: Switches will precede arguments regardless of appending order.
-  void AppendArg(StringPiece value);
+  void AppendArg(std::string_view value);
   void AppendArgPath(const FilePath& value);
-  void AppendArgNative(StringPieceType value);
+  void AppendArgNative(StringViewType value);
 
   // Append the switches and arguments from another command line to this one.
   // If `include_program` is true, program will be overwritten by other's.
@@ -235,12 +239,12 @@ class BASE_EXPORT CommandLine {
 
   // Insert a command before the current command.
   // Common for debuggers, like "gdb --args".
-  void PrependWrapper(StringPieceType wrapper);
+  void PrependWrapper(StringViewType wrapper);
 
 #if BUILDFLAG(IS_WIN)
   // Initialize by parsing the given command line string.
   // The program name is assumed to be the first item in the string.
-  void ParseFromString(StringPieceType command_line);
+  void ParseFromString(StringViewType command_line);
 
   // Returns true if the command line had the --single-argument switch, and
   // thus likely came from a Windows shell registration. This is only set if the
@@ -271,10 +275,10 @@ class BASE_EXPORT CommandLine {
       return *this;
     }
 
-    // Disallow move.
-    InstanceBoundSequenceChecker(InstanceBoundSequenceChecker&&) = delete;
+    // Allow move as per SequenceChecker.
+    InstanceBoundSequenceChecker(InstanceBoundSequenceChecker&&) = default;
     InstanceBoundSequenceChecker& operator=(InstanceBoundSequenceChecker&&) =
-        delete;
+        default;
 
     void Detach() { DETACH_FROM_SEQUENCE(sequence_checker_); }
     void Check() { DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_); }
@@ -312,7 +316,7 @@ class BASE_EXPORT CommandLine {
   // The string returned by GetCommandLineW(), to be parsed via
   // ParseFromString(). Empty if this command line was not parsed from a string,
   // or if ParseFromString() has finished executing.
-  StringPieceType raw_command_line_string_;
+  StringViewType raw_command_line_string_;
 
   // Set to true if the command line had --single-argument when initially
   // parsed. It does not change if the command line mutates after initial
@@ -340,8 +344,8 @@ class BASE_EXPORT CommandLine {
 class BASE_EXPORT DuplicateSwitchHandler {
  public:
   // out_value contains the existing value of the switch
-  virtual void ResolveDuplicate(base::StringPiece key,
-                                CommandLine::StringPieceType new_value,
+  virtual void ResolveDuplicate(std::string_view key,
+                                CommandLine::StringViewType new_value,
                                 CommandLine::StringType& out_value) = 0;
   virtual ~DuplicateSwitchHandler() = default;
 };
