@@ -2,9 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/debug/test_elf_image_builder.h"
 
 #include <cstring>
+#include <string_view>
 #include <type_traits>
 #include <utility>
 
@@ -61,7 +67,7 @@ TestElfImageBuilder& TestElfImageBuilder::AddLoadSegment(Word flags,
 
 TestElfImageBuilder& TestElfImageBuilder::AddNoteSegment(
     Word type,
-    StringPiece name,
+    std::string_view name,
     span<const uint8_t> desc) {
   const size_t name_with_null_size = name.size() + 1;
   std::vector<uint8_t> buffer(
@@ -89,7 +95,7 @@ TestElfImageBuilder& TestElfImageBuilder::AddNoteSegment(
   return *this;
 }
 
-TestElfImageBuilder& TestElfImageBuilder::AddSoName(StringPiece soname) {
+TestElfImageBuilder& TestElfImageBuilder::AddSoName(std::string_view soname) {
   DCHECK(!soname_.has_value());
   soname_.emplace(soname);
   return *this;
