@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,28 +7,24 @@
 
 #include <stdint.h>
 
+#include <optional>
 #include <string>
 
 #include "base/base_export.h"
+#include "base/compiler_specific.h"
 #include "base/containers/span.h"
 #include "base/files/file_path.h"
 #include "base/files/file_tracing.h"
 #include "base/files/platform_file.h"
 #include "base/time/time.h"
+#include "base/trace_event/base_tracing_forward.h"
 #include "build/build_config.h"
 
-#if defined(OS_POSIX) || defined(OS_FUCHSIA)
-#include <sys/stat.h>
-#endif
+struct stat;
 
 namespace base {
 
-#if defined(OS_BSD) || defined(OS_APPLE) || defined(OS_NACL) || \
-    defined(OS_FUCHSIA) || (defined(OS_ANDROID) && __ANDROID_API__ < 21)
-typedef struct stat stat_wrapper_t;
-#elif defined(OS_POSIX)
-typedef struct stat64 stat_wrapper_t;
-#endif
+using stat_wrapper_t = struct stat;
 
 // Thin wrapper around an OS-level file.
 // Note that this class does not provide any support for asynchronous IO, other
@@ -40,6 +36,9 @@ typedef struct stat64 stat_wrapper_t;
 // obvious non-modifying way are marked as const. Any method that forward calls
 // to the OS is not considered const, even if there is no apparent change to
 // member variables.
+//
+// On POSIX, if the given file is a symbolic link, most of the methods apply to
+// the file that the symbolic link resolves to.
 class BASE_EXPORT File {
  public:
   // FLAG_(OPEN|CREATE).* are mutually exclusive. You should specify exactly one
@@ -47,9 +46,7 @@ class BASE_EXPORT File {
   // a file.
   // FLAG_(WRITE|APPEND) are mutually exclusive. This is so that APPEND behavior
   // will be consistent with O_APPEND on POSIX.
-  // FLAG_EXCLUSIVE_(READ|WRITE) only grant exclusive access to the file on
-  // creation on POSIX; for existing files, consider using Lock().
-  enum Flags {
+  enum Flags : uint32_t {
     FLAG_OPEN = 1 << 0,            // Opens a file, only if it exists.
     FLAG_CREATE = 1 << 1,          // Creates a new file, only if it does not
                                    // already exist.
@@ -60,21 +57,26 @@ class BASE_EXPORT File {
     FLAG_READ = 1 << 5,
     FLAG_WRITE = 1 << 6,
     FLAG_APPEND = 1 << 7,
-    FLAG_EXCLUSIVE_READ = 1 << 8,  // EXCLUSIVE is opposite of Windows SHARE.
-    FLAG_EXCLUSIVE_WRITE = 1 << 9,
+    FLAG_WIN_EXCLUSIVE_READ = 1 << 8,   // Windows only. Opposite of SHARE.
+    FLAG_WIN_EXCLUSIVE_WRITE = 1 << 9,  // Windows only. Opposite of SHARE.
     FLAG_ASYNC = 1 << 10,
-    FLAG_TEMPORARY = 1 << 11,  // Used on Windows only.
-    FLAG_HIDDEN = 1 << 12,     // Used on Windows only.
+    FLAG_WIN_TEMPORARY = 1 << 11,  // Windows only.
+    FLAG_WIN_HIDDEN = 1 << 12,     // Windows only.
     FLAG_DELETE_ON_CLOSE = 1 << 13,
-    FLAG_WRITE_ATTRIBUTES = 1 << 14,     // Used on Windows only.
-    FLAG_SHARE_DELETE = 1 << 15,         // Used on Windows only.
-    FLAG_TERMINAL_DEVICE = 1 << 16,      // Serial port flags.
-    FLAG_BACKUP_SEMANTICS = 1 << 17,     // Used on Windows only.
-    FLAG_EXECUTE = 1 << 18,              // Used on Windows only.
-    FLAG_SEQUENTIAL_SCAN = 1 << 19,      // Used on Windows only.
+    FLAG_WRITE_ATTRIBUTES = 1 << 14,  // File opened in a mode allowing writing
+                                      // attributes, such as with SetTimes().
+    FLAG_WIN_SHARE_DELETE = 1 << 15,  // Windows only.
+    FLAG_TERMINAL_DEVICE = 1 << 16,   // Serial port flags.
+    FLAG_WIN_BACKUP_SEMANTICS = 1 << 17,  // Windows only.
+    FLAG_WIN_EXECUTE = 1 << 18,           // Windows only.
+    FLAG_WIN_SEQUENTIAL_SCAN = 1 << 19,   // Windows only.
     FLAG_CAN_DELETE_ON_CLOSE = 1 << 20,  // Requests permission to delete a file
                                          // via DeleteOnClose() (Windows only).
                                          // See DeleteOnClose() for details.
+    FLAG_WIN_NO_EXECUTE =
+        1 << 21,  // Windows only. Marks the file with a deny ACE that prevents
+                  // opening the file with EXECUTE access. Cannot be used with
+                  // FILE_WIN_EXECUTE flag. See also PreventExecuteMapping.
   };
 
   // This enum has been recorded in multiple histograms using PlatformFileError
@@ -121,7 +123,7 @@ class BASE_EXPORT File {
   struct BASE_EXPORT Info {
     Info();
     ~Info();
-#if defined(OS_POSIX) || defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
     // Fills this struct with values from |stat_info|.
     void FromStat(const stat_wrapper_t& stat_info);
 #endif
@@ -178,8 +180,8 @@ class BASE_EXPORT File {
   void Initialize(const FilePath& path, uint32_t flags);
 
   // Returns |true| if the handle / fd wrapped by this object is valid.  This
-  // method doesn't interact with the file system (and is safe to be called from
-  // ThreadRestrictions::SetIOAllowed(false) threads).
+  // method doesn't interact with the file system and is thus safe to be called
+  // from threads that disallow blocking.
   bool IsValid() const;
 
   // Returns true if a new file was created (or an old one truncated to zero
@@ -205,13 +207,11 @@ class BASE_EXPORT File {
   // (relative to the start) or -1 in case of error.
   int64_t Seek(Whence whence, int64_t offset);
 
-  // Simplified versions of Read() and friends (see below) that check the int
+  // Simplified versions of Read() and friends (see below) that check the
   // return value and just return a boolean. They return true if and only if
-  // the function read in / wrote out exactly |size| bytes of data.
+  // the function read in exactly |data.size()| bytes of data.
   bool ReadAndCheck(int64_t offset, span<uint8_t> data);
   bool ReadAtCurrentPosAndCheck(span<uint8_t> data);
-  bool WriteAndCheck(int64_t offset, span<const uint8_t> data);
-  bool WriteAtCurrentPosAndCheck(span<const uint8_t> data);
 
   // Reads the given number of bytes (or until EOF is reached) starting with the
   // given offset. Returns the number of bytes read, or -1 on error. Note that
@@ -219,18 +219,31 @@ class BASE_EXPORT File {
   // is not intended for stream oriented files but instead for cases when the
   // normal expectation is that actually |size| bytes are read unless there is
   // an error.
-  int Read(int64_t offset, char* data, int size);
+  UNSAFE_BUFFER_USAGE int Read(int64_t offset, char* data, int size);
+  std::optional<size_t> Read(int64_t offset, base::span<uint8_t> data);
 
   // Same as above but without seek.
-  int ReadAtCurrentPos(char* data, int size);
+  UNSAFE_BUFFER_USAGE int ReadAtCurrentPos(char* data, int size);
+  std::optional<size_t> ReadAtCurrentPos(base::span<uint8_t> data);
 
   // Reads the given number of bytes (or until EOF is reached) starting with the
   // given offset, but does not make any effort to read all data on all
-  // platforms. Returns the number of bytes read, or -1 on error.
-  int ReadNoBestEffort(int64_t offset, char* data, int size);
+  // platforms. Returns the number of bytes read, or -1/std::nullopt on error.
+  UNSAFE_BUFFER_USAGE int ReadNoBestEffort(int64_t offset,
+                                           char* data,
+                                           int size);
+  std::optional<size_t> ReadNoBestEffort(int64_t offset,
+                                         base::span<uint8_t> data);
 
   // Same as above but without seek.
-  int ReadAtCurrentPosNoBestEffort(char* data, int size);
+  UNSAFE_BUFFER_USAGE int ReadAtCurrentPosNoBestEffort(char* data, int size);
+  std::optional<size_t> ReadAtCurrentPosNoBestEffort(base::span<uint8_t> data);
+
+  // Simplified versions of Write() and friends (see below) that check the
+  // return value and just return a boolean. They return true if and only if
+  // the function wrote out exactly |data.size()| bytes of data.
+  bool WriteAndCheck(int64_t offset, span<const uint8_t> data);
+  bool WriteAtCurrentPosAndCheck(span<const uint8_t> data);
 
   // Writes the given buffer into the file at the given offset, overwritting any
   // data that was previously there. Returns the number of bytes written, or -1
@@ -238,17 +251,23 @@ class BASE_EXPORT File {
   // all platforms. |data| can be nullptr when |size| is 0.
   // Ignores the offset and writes to the end of the file if the file was opened
   // with FLAG_APPEND.
-  int Write(int64_t offset, const char* data, int size);
+  UNSAFE_BUFFER_USAGE int Write(int64_t offset, const char* data, int size);
+  std::optional<size_t> Write(int64_t offset, base::span<const uint8_t> data);
 
   // Save as above but without seek.
-  int WriteAtCurrentPos(const char* data, int size);
+  UNSAFE_BUFFER_USAGE int WriteAtCurrentPos(const char* data, int size);
+  std::optional<size_t> WriteAtCurrentPos(base::span<const uint8_t> data);
 
   // Save as above but does not make any effort to write all data on all
-  // platforms. Returns the number of bytes written, or -1 on error.
-  int WriteAtCurrentPosNoBestEffort(const char* data, int size);
+  // platforms. Returns the number of bytes written, or -1/std::nullopt
+  // on error.
+  UNSAFE_BUFFER_USAGE int WriteAtCurrentPosNoBestEffort(const char* data,
+                                                        int size);
+  std::optional<size_t> WriteAtCurrentPosNoBestEffort(
+      base::span<const uint8_t> data);
 
   // Returns the current size of this file, or a negative number on failure.
-  int64_t GetLength();
+  int64_t GetLength() const;
 
   // Truncates the file to the given length. If |length| is greater than the
   // current size of the file, the file is extended with zeros. If the file
@@ -275,7 +294,8 @@ class BASE_EXPORT File {
   // Returns some basic information for the given file.
   bool GetInfo(Info* info);
 
-#if !defined(OS_FUCHSIA)  // Fuchsia's POSIX API does not support file locking.
+#if !BUILDFLAG( \
+    IS_FUCHSIA)  // Fuchsia's POSIX API does not support file locking.
   enum class LockMode {
     kShared,
     kExclusive,
@@ -306,7 +326,7 @@ class BASE_EXPORT File {
   // Unlock a file previously locked.
   Error Unlock();
 
-#endif  // !defined(OS_FUCHSIA)
+#endif  // !BUILDFLAG(IS_FUCHSIA)
 
   // Returns a new object referencing this file for use within the current
   // process. Handling of FLAG_DELETE_ON_CLOSE varies by OS. On POSIX, the File
@@ -317,7 +337,16 @@ class BASE_EXPORT File {
 
   bool async() const { return async_; }
 
-#if defined(OS_WIN)
+  // Serialise this object into a trace.
+  void WriteIntoTrace(perfetto::TracedValue context) const;
+
+#if BUILDFLAG(IS_APPLE)
+  // Initializes experiments. Must be invoked early in process startup, but
+  // after `FeatureList` initialization.
+  static void InitializeFeatures();
+#endif  // BUILDFLAG(IS_APPLE)
+
+#if BUILDFLAG(IS_WIN)
   // Sets or clears the DeleteFile disposition on the file. Returns true if
   // the disposition was set or cleared, as indicated by |delete_on_close|.
   //
@@ -331,11 +360,11 @@ class BASE_EXPORT File {
   //   calling DeleteOnClose(true).
   //
   // In all cases, all pre-existing handles to the file must have been opened
-  // with FLAG_SHARE_DELETE. Once the disposition has been set by any of the
+  // with FLAG_WIN_SHARE_DELETE. Once the disposition has been set by any of the
   // above means, no new File objects can be created for the file.
   //
   // So:
-  // - Use FLAG_SHARE_DELETE when creating/opening a file to allow another
+  // - Use FLAG_WIN_SHARE_DELETE when creating/opening a file to allow another
   //   entity on the system to cause it to be deleted when it is closed. (Note:
   //   another entity can delete the file the moment after it is closed, so not
   //   using this permission doesn't provide any protections.)
@@ -350,30 +379,46 @@ class BASE_EXPORT File {
   //   deleted in the event of untimely process termination, and then clearing
   //   this state once the file is suitable for persistence.
   bool DeleteOnClose(bool delete_on_close);
-#endif
 
-#if defined(OS_WIN)
+  // Precondition: last_error is not 0, also known as ERROR_SUCCESS.
   static Error OSErrorToFileError(DWORD last_error);
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
+  // Precondition: saved_errno is not 0.
   static Error OSErrorToFileError(int saved_errno);
 #endif
 
   // Gets the last global error (errno or GetLastError()) and converts it to the
-  // closest base::File::Error equivalent via OSErrorToFileError(). The returned
-  // value is only trustworthy immediately after another base::File method
-  // fails. base::File never resets the global error to zero.
+  // closest base::File::Error equivalent via OSErrorToFileError(). It should
+  // therefore only be called immediately after another base::File method fails.
+  // base::File never resets the global error to zero.
   static Error GetLastFileError();
 
   // Converts an error value to a human-readable form. Used for logging.
   static std::string ErrorToString(Error error);
 
-#if defined(OS_POSIX) || defined(OS_FUCHSIA)
-  // Wrapper for stat() or stat64().
-  static int Stat(const char* path, stat_wrapper_t* sb);
+#if BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
+  // Wrapper for stat().
+  static int Stat(const FilePath& path, stat_wrapper_t* sb);
+  // Wrapper for fstat().
   static int Fstat(int fd, stat_wrapper_t* sb);
-  static int Lstat(const char* path, stat_wrapper_t* sb);
+  // Wrapper for lstat().
+  static int Lstat(const FilePath& path, stat_wrapper_t* sb);
 #endif
 
+  // This function can be used to augment `flags` with the correct flags
+  // required to create a File that can be safely passed to an untrusted
+  // process. It must be called if the File is intended to be transferred to an
+  // untrusted process, but can still be safely called even if the File is not
+  // intended to be transferred.
+  static constexpr uint32_t AddFlagsForPassingToUntrustedProcess(
+      uint32_t flags) {
+    if (flags & File::FLAG_WRITE || flags & File::FLAG_APPEND ||
+        flags & File::FLAG_WRITE_ATTRIBUTES) {
+      flags |= File::FLAG_WIN_NO_EXECUTE;
+    }
+    return flags;
+  }
+
  private:
   friend class FileTracing::ScopedTrace;
 
