@@ -1,26 +1,43 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/files/file.h"
 
+// The only 32-bit platform that uses this file is Android. On Android APIs
+// >= 21, this standard define is the right way to express that you want a
+// 64-bit offset in struct stat, and the stat64 struct and functions aren't
+// useful.
+#define _FILE_OFFSET_BITS 64
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
+static_assert(sizeof(base::stat_wrapper_t::st_size) >= 8);
+
+#include <atomic>
+#include <optional>
+
 #include "base/check_op.h"
-#include "base/metrics/histogram_functions.h"
+#include "base/feature_list.h"
+#include "base/metrics/field_trial_params.h"
+#include "base/notimplemented.h"
 #include "base/notreached.h"
+#include "base/numerics/safe_conversions.h"
 #include "base/posix/eintr_wrapper.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/scoped_blocking_call.h"
 #include "build/build_config.h"
-#include "build/chromeos_buildflags.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 #include "base/os_compat_android.h"
 #endif
 
@@ -33,15 +50,45 @@ static_assert(File::FROM_BEGIN == SEEK_SET && File::FROM_CURRENT == SEEK_CUR &&
 
 namespace {
 
+#if BUILDFLAG(IS_APPLE)
+// When enabled, `F_FULLFSYNC` is not used in `File::Flush`. Instead,
+// `F_BARRIERFSYNC` or `flush()` is used (depending on the
+// "MacEfficientFileFlushUseBarrier" param). The feature exists to measure the
+// cost of `F_FULLFSYNC` compared to other solutions (not ready to enable by
+// default as-is). See
+// https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fsync.2.html
+BASE_FEATURE(kMacEfficientFileFlush,
+             "MacEfficientFileFlush",
+             base::FEATURE_DISABLED_BY_DEFAULT);
+
+const FeatureParam<bool> kMacEfficientFileFlushUseBarrier{
+    &kMacEfficientFileFlush, "MacEfficientFileFlushUseBarrier", true};
+
+enum class MacFileFlushMechanism {
+  kFlush,
+  kFullFsync,
+  kBarrierFsync,
+};
+
+std::atomic<MacFileFlushMechanism> g_mac_file_flush_mechanism{
+    MacFileFlushMechanism::kFullFsync};
+#endif  // BUILDFLAG(IS_APPLE)
+
 // NaCl doesn't provide the following system calls, so either simulate them or
 // wrap them in order to minimize the number of #ifdef's in this file.
-#if !defined(OS_NACL) && !defined(OS_AIX)
+#if !BUILDFLAG(IS_NACL) && !BUILDFLAG(IS_AIX)
 bool IsOpenAppend(PlatformFile file) {
   return (fcntl(file, F_GETFL) & O_APPEND) != 0;
 }
 
 int CallFtruncate(PlatformFile file, int64_t length) {
+#if BUILDFLAG(IS_BSD) || BUILDFLAG(IS_APPLE) || BUILDFLAG(IS_FUCHSIA)
+  static_assert(sizeof(off_t) >= sizeof(int64_t),
+                "off_t is not a 64-bit integer");
   return HANDLE_EINTR(ftruncate(file, length));
+#else
+  return HANDLE_EINTR(ftruncate64(file, length));
+#endif
 }
 
 int CallFutimes(PlatformFile file, const struct timeval times[2]) {
@@ -61,8 +108,8 @@ int CallFutimes(PlatformFile file, const struct timeval times[2]) {
 #endif
 }
 
-#if !defined(OS_FUCHSIA)
-short FcntlFlockType(absl::optional<File::LockMode> mode) {
+#if !BUILDFLAG(IS_FUCHSIA)
+short FcntlFlockType(std::optional<File::LockMode> mode) {
   if (!mode.has_value())
     return F_UNLCK;
   switch (mode.value()) {
@@ -75,7 +122,7 @@ short FcntlFlockType(absl::optional<File::LockMode> mode) {
 }
 
 File::Error CallFcntlFlock(PlatformFile file,
-                           absl::optional<File::LockMode> mode) {
+                           std::optional<File::LockMode> mode) {
   struct flock lock;
   lock.l_type = FcntlFlockType(std::move(mode));
   lock.l_whence = SEEK_SET;
@@ -87,7 +134,7 @@ File::Error CallFcntlFlock(PlatformFile file,
 }
 #endif
 
-#else   // defined(OS_NACL) && !defined(OS_AIX)
+#else   // BUILDFLAG(IS_NACL) && !BUILDFLAG(IS_AIX)
 
 bool IsOpenAppend(PlatformFile file) {
   // NaCl doesn't implement fcntl. Since NaCl's write conforms to the POSIX
@@ -107,11 +154,11 @@ int CallFutimes(PlatformFile file, const struct timeval times[2]) {
 }
 
 File::Error CallFcntlFlock(PlatformFile file,
-                           absl::optional<File::LockMode> mode) {
+                           std::optional<File::LockMode> mode) {
   NOTIMPLEMENTED();  // NaCl doesn't implement flock struct.
   return File::FILE_ERROR_INVALID_OPERATION;
 }
-#endif  // defined(OS_NACL)
+#endif  // BUILDFLAG(IS_NACL)
 
 }  // namespace
 
@@ -127,28 +174,28 @@ void File::Info::FromStat(const stat_wrapper_t& stat_info) {
   // creation time. However, other than on Mac & iOS where the actual file
   // creation time is included as st_birthtime, the rest of POSIX platforms have
   // no portable way to get the creation time.
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_FUCHSIA)
   time_t last_modified_sec = stat_info.st_mtim.tv_sec;
   int64_t last_modified_nsec = stat_info.st_mtim.tv_nsec;
   time_t last_accessed_sec = stat_info.st_atim.tv_sec;
   int64_t last_accessed_nsec = stat_info.st_atim.tv_nsec;
   time_t creation_time_sec = stat_info.st_ctim.tv_sec;
   int64_t creation_time_nsec = stat_info.st_ctim.tv_nsec;
-#elif defined(OS_ANDROID)
+#elif BUILDFLAG(IS_ANDROID)
   time_t last_modified_sec = stat_info.st_mtime;
   int64_t last_modified_nsec = stat_info.st_mtime_nsec;
   time_t last_accessed_sec = stat_info.st_atime;
   int64_t last_accessed_nsec = stat_info.st_atime_nsec;
   time_t creation_time_sec = stat_info.st_ctime;
   int64_t creation_time_nsec = stat_info.st_ctime_nsec;
-#elif defined(OS_APPLE)
+#elif BUILDFLAG(IS_APPLE)
   time_t last_modified_sec = stat_info.st_mtimespec.tv_sec;
   int64_t last_modified_nsec = stat_info.st_mtimespec.tv_nsec;
   time_t last_accessed_sec = stat_info.st_atimespec.tv_sec;
   int64_t last_accessed_nsec = stat_info.st_atimespec.tv_nsec;
   time_t creation_time_sec = stat_info.st_birthtimespec.tv_sec;
   int64_t creation_time_nsec = stat_info.st_birthtimespec.tv_nsec;
-#elif defined(OS_BSD)
+#elif BUILDFLAG(IS_BSD)
   time_t last_modified_sec = stat_info.st_mtimespec.tv_sec;
   int64_t last_modified_nsec = stat_info.st_mtimespec.tv_nsec;
   time_t last_accessed_sec = stat_info.st_atimespec.tv_sec;
@@ -166,18 +213,15 @@ void File::Info::FromStat(const stat_wrapper_t& stat_info) {
 
   last_modified =
       Time::FromTimeT(last_modified_sec) +
-      TimeDelta::FromMicroseconds(last_modified_nsec /
-                                  Time::kNanosecondsPerMicrosecond);
+      Microseconds(last_modified_nsec / Time::kNanosecondsPerMicrosecond);
 
   last_accessed =
       Time::FromTimeT(last_accessed_sec) +
-      TimeDelta::FromMicroseconds(last_accessed_nsec /
-                                  Time::kNanosecondsPerMicrosecond);
+      Microseconds(last_accessed_nsec / Time::kNanosecondsPerMicrosecond);
 
   creation_time =
       Time::FromTimeT(creation_time_sec) +
-      TimeDelta::FromMicroseconds(creation_time_nsec /
-                                  Time::kNanosecondsPerMicrosecond);
+      Microseconds(creation_time_nsec / Time::kNanosecondsPerMicrosecond);
 }
 
 bool File::IsValid() const {
@@ -207,7 +251,7 @@ int64_t File::Seek(Whence whence, int64_t offset) {
 
   SCOPED_FILE_TRACE_WITH_SIZE("Seek", offset);
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   static_assert(sizeof(int64_t) == sizeof(off64_t), "off64_t must be 64 bits");
   return lseek64(file_.get(), static_cast<off64_t>(offset),
                  static_cast<int>(whence));
@@ -221,23 +265,24 @@ int64_t File::Seek(Whence whence, int64_t offset) {
 int File::Read(int64_t offset, char* data, int size) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
   DCHECK(IsValid());
-  if (size < 0)
+  if (size < 0 || !IsValueInRangeForNumericType<off_t>(offset + size - 1))
     return -1;
 
   SCOPED_FILE_TRACE_WITH_SIZE("Read", size);
 
   int bytes_read = 0;
-  int rv;
+  long rv;
   do {
     rv = HANDLE_EINTR(pread(file_.get(), data + bytes_read,
-                            size - bytes_read, offset + bytes_read));
+                            static_cast<size_t>(size - bytes_read),
+                            static_cast<off_t>(offset + bytes_read)));
     if (rv <= 0)
       break;
 
     bytes_read += rv;
   } while (bytes_read < size);
 
-  return bytes_read ? bytes_read : rv;
+  return bytes_read ? bytes_read : checked_cast<int>(rv);
 }
 
 int File::ReadAtCurrentPos(char* data, int size) {
@@ -249,23 +294,29 @@ int File::ReadAtCurrentPos(char* data, int size) {
   SCOPED_FILE_TRACE_WITH_SIZE("ReadAtCurrentPos", size);
 
   int bytes_read = 0;
-  int rv;
+  long rv;
   do {
-    rv = HANDLE_EINTR(read(file_.get(), data + bytes_read, size - bytes_read));
+    rv = HANDLE_EINTR(read(file_.get(), data + bytes_read,
+                           static_cast<size_t>(size - bytes_read)));
     if (rv <= 0)
       break;
 
     bytes_read += rv;
   } while (bytes_read < size);
 
-  return bytes_read ? bytes_read : rv;
+  return bytes_read ? bytes_read : checked_cast<int>(rv);
 }
 
 int File::ReadNoBestEffort(int64_t offset, char* data, int size) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
   DCHECK(IsValid());
+  if (size < 0 || !IsValueInRangeForNumericType<off_t>(offset))
+    return -1;
+
   SCOPED_FILE_TRACE_WITH_SIZE("ReadNoBestEffort", size);
-  return HANDLE_EINTR(pread(file_.get(), data, size, offset));
+  return checked_cast<int>(
+      HANDLE_EINTR(pread(file_.get(), data, static_cast<size_t>(size),
+                         static_cast<off_t>(offset))));
 }
 
 int File::ReadAtCurrentPosNoBestEffort(char* data, int size) {
@@ -275,7 +326,8 @@ int File::ReadAtCurrentPosNoBestEffort(char* data, int size) {
     return -1;
 
   SCOPED_FILE_TRACE_WITH_SIZE("ReadAtCurrentPosNoBestEffort", size);
-  return HANDLE_EINTR(read(file_.get(), data, size));
+  return checked_cast<int>(
+      HANDLE_EINTR(read(file_.get(), data, static_cast<size_t>(size))));
 }
 
 int File::Write(int64_t offset, const char* data, int size) {
@@ -291,18 +343,20 @@ int File::Write(int64_t offset, const char* data, int size) {
   SCOPED_FILE_TRACE_WITH_SIZE("Write", size);
 
   int bytes_written = 0;
-  int rv;
+  long rv;
   do {
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
     // In case __USE_FILE_OFFSET64 is not used, we need to call pwrite64()
     // instead of pwrite().
     static_assert(sizeof(int64_t) == sizeof(off64_t),
                   "off64_t must be 64 bits");
     rv = HANDLE_EINTR(pwrite64(file_.get(), data + bytes_written,
-                               size - bytes_written, offset + bytes_written));
+                               static_cast<size_t>(size - bytes_written),
+                               offset + bytes_written));
 #else
     rv = HANDLE_EINTR(pwrite(file_.get(), data + bytes_written,
-                             size - bytes_written, offset + bytes_written));
+                             static_cast<size_t>(size - bytes_written),
+                             offset + bytes_written));
 #endif
     if (rv <= 0)
       break;
@@ -310,7 +364,7 @@ int File::Write(int64_t offset, const char* data, int size) {
     bytes_written += rv;
   } while (bytes_written < size);
 
-  return bytes_written ? bytes_written : rv;
+  return bytes_written ? bytes_written : checked_cast<int>(rv);
 }
 
 int File::WriteAtCurrentPos(const char* data, int size) {
@@ -322,17 +376,17 @@ int File::WriteAtCurrentPos(const char* data, int size) {
   SCOPED_FILE_TRACE_WITH_SIZE("WriteAtCurrentPos", size);
 
   int bytes_written = 0;
-  int rv;
+  long rv;
   do {
     rv = HANDLE_EINTR(write(file_.get(), data + bytes_written,
-                            size - bytes_written));
+                            static_cast<size_t>(size - bytes_written)));
     if (rv <= 0)
       break;
 
     bytes_written += rv;
   } while (bytes_written < size);
 
-  return bytes_written ? bytes_written : rv;
+  return bytes_written ? bytes_written : checked_cast<int>(rv);
 }
 
 int File::WriteAtCurrentPosNoBestEffort(const char* data, int size) {
@@ -342,10 +396,11 @@ int File::WriteAtCurrentPosNoBestEffort(const char* data, int size) {
     return -1;
 
   SCOPED_FILE_TRACE_WITH_SIZE("WriteAtCurrentPosNoBestEffort", size);
-  return HANDLE_EINTR(write(file_.get(), data, size));
+  return checked_cast<int>(
+      HANDLE_EINTR(write(file_.get(), data, static_cast<size_t>(size))));
 }
 
-int64_t File::GetLength() {
+int64_t File::GetLength() const {
   DCHECK(IsValid());
 
   SCOPED_FILE_TRACE("GetLength");
@@ -391,7 +446,7 @@ bool File::GetInfo(Info* info) {
   return true;
 }
 
-#if !defined(OS_FUCHSIA)
+#if !BUILDFLAG(IS_FUCHSIA)
 File::Error File::Lock(File::LockMode mode) {
   SCOPED_FILE_TRACE("Lock");
   return CallFcntlFlock(file_.get(), mode);
@@ -399,7 +454,7 @@ File::Error File::Lock(File::LockMode mode) {
 
 File::Error File::Unlock() {
   SCOPED_FILE_TRACE("Unlock");
-  return CallFcntlFlock(file_.get(), absl::optional<File::LockMode>());
+  return CallFcntlFlock(file_.get(), std::optional<File::LockMode>());
 }
 #endif
 
@@ -416,6 +471,22 @@ File File::Duplicate() const {
   return File(std::move(other_fd), async());
 }
 
+#if BUILDFLAG(IS_APPLE)
+void File::InitializeFeatures() {
+  if (FeatureList::IsEnabled(kMacEfficientFileFlush)) {
+    // "relaxed" because there is no dependency between these memory operations
+    // and other memory operations.
+    if (kMacEfficientFileFlushUseBarrier.Get()) {
+      g_mac_file_flush_mechanism.store(MacFileFlushMechanism::kBarrierFsync,
+                                       std::memory_order_relaxed);
+    } else {
+      g_mac_file_flush_mechanism.store(MacFileFlushMechanism::kFlush,
+                                       std::memory_order_relaxed);
+    }
+  }
+}
+#endif  // BUILDFLAG(IS_APPLE)
+
 // Static.
 File::Error File::OSErrorToFileError(int saved_errno) {
   switch (saved_errno) {
@@ -425,7 +496,7 @@ File::Error File::OSErrorToFileError(int saved_errno) {
     case EPERM:
       return FILE_ERROR_ACCESS_DENIED;
     case EBUSY:
-#if !defined(OS_NACL)  // ETXTBSY not defined by NaCl.
+#if !BUILDFLAG(IS_NACL)  // ETXTBSY not defined by NaCl.
     case ETXTBSY:
 #endif
       return FILE_ERROR_IN_USE;
@@ -445,9 +516,6 @@ File::Error File::OSErrorToFileError(int saved_errno) {
     case ENOTDIR:
       return FILE_ERROR_NOT_A_DIRECTORY;
     default:
-#if !defined(OS_NACL)  // NaCl build has no metrics code.
-      UmaHistogramSparse("PlatformFile.UnknownErrors.Posix", saved_errno);
-#endif
       // This function should only be called for errors.
       DCHECK_NE(0, saved_errno);
       return FILE_ERROR_FAILED;
@@ -455,7 +523,7 @@ File::Error File::OSErrorToFileError(int saved_errno) {
 }
 
 // NaCl doesn't implement system calls to open files directly.
-#if !defined(OS_NACL)
+#if !BUILDFLAG(IS_NACL)
 // TODO(erikkay): does it make sense to support FLAG_EXCLUSIVE_* here?
 void File::DoInitialize(const FilePath& path, uint32_t flags) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
@@ -481,19 +549,17 @@ void File::DoInitialize(const FilePath& path, uint32_t flags) {
 
   if (!open_flags && !(flags & FLAG_OPEN) && !(flags & FLAG_OPEN_ALWAYS)) {
     NOTREACHED();
-    errno = EOPNOTSUPP;
-    error_details_ = FILE_ERROR_FAILED;
-    return;
   }
 
   if (flags & FLAG_WRITE && flags & FLAG_READ) {
     open_flags |= O_RDWR;
   } else if (flags & FLAG_WRITE) {
     open_flags |= O_WRONLY;
-  } else if (!(flags & FLAG_READ) &&
-             !(flags & FLAG_WRITE_ATTRIBUTES) &&
-             !(flags & FLAG_APPEND) &&
-             !(flags & FLAG_OPEN_ALWAYS)) {
+  } else if (!(flags & FLAG_READ) && !(flags & FLAG_WRITE_ATTRIBUTES) &&
+             !(flags & FLAG_APPEND) && !(flags & FLAG_OPEN_ALWAYS)) {
+    // Note: For FLAG_WRITE_ATTRIBUTES and no other read/write flags, we'll
+    // open the file in O_RDONLY mode (== 0, see static_assert below), so that
+    // we get a fd that can be used for SetTimes().
     NOTREACHED();
   }
 
@@ -507,8 +573,8 @@ void File::DoInitialize(const FilePath& path, uint32_t flags) {
 
   static_assert(O_RDONLY == 0, "O_RDONLY must equal zero");
 
-  int mode = S_IRUSR | S_IWUSR;
-#if BUILDFLAG(IS_CHROMEOS_ASH) || BUILDFLAG(IS_CHROMEOS_LACROS)
+  mode_t mode = S_IRUSR | S_IWUSR;
+#if BUILDFLAG(IS_CHROMEOS)
   mode |= S_IRGRP | S_IROTH;
 #endif
 
@@ -517,9 +583,6 @@ void File::DoInitialize(const FilePath& path, uint32_t flags) {
   if (flags & FLAG_OPEN_ALWAYS) {
     if (descriptor < 0) {
       open_flags |= O_CREAT;
-      if (flags & FLAG_EXCLUSIVE_READ || flags & FLAG_EXCLUSIVE_WRITE)
-        open_flags |= O_EXCL;   // together with O_CREAT implies O_NOFOLLOW
-
       descriptor = HANDLE_EINTR(open(path.value().c_str(), open_flags, mode));
       if (descriptor >= 0)
         created_ = true;
@@ -541,35 +604,58 @@ void File::DoInitialize(const FilePath& path, uint32_t flags) {
   error_details_ = FILE_OK;
   file_.reset(descriptor);
 }
-#endif  // !defined(OS_NACL)
+#endif  // !BUILDFLAG(IS_NACL)
 
 bool File::Flush() {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
   DCHECK(IsValid());
   SCOPED_FILE_TRACE("Flush");
 
-#if defined(OS_NACL)
+#if BUILDFLAG(IS_NACL)
   NOTIMPLEMENTED();  // NaCl doesn't implement fsync.
   return true;
-#elif defined(OS_ANDROID) || defined(OS_CHROMEOS) || defined(OS_FUCHSIA) || \
-    defined(OS_LINUX)
+#elif BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_CHROMEOS) || \
+    BUILDFLAG(IS_FUCHSIA) || BUILDFLAG(IS_LINUX)
   return !HANDLE_EINTR(fdatasync(file_.get()));
-#elif defined(OS_APPLE)
+#elif BUILDFLAG(IS_APPLE)
   // On macOS and iOS, fsync() is guaranteed to send the file's data to the
   // underlying storage device, but may return before the device actually writes
   // the data to the medium. When used by database systems, this may result in
-  // unexpected data loss.
-  // https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fsync.2.html
-  if (!HANDLE_EINTR(fcntl(file_.get(), F_FULLFSYNC)))
-    return true;
-
-  // Some filesystms do not support fcntl(F_FULLFSYNC). We handle these cases by
-  // falling back to fsync(). Unfortunately, lack of F_FULLFSYNC support results
-  // in various error codes, so we cannot use the error code as a definitive
-  // indicator that F_FULLFSYNC was not supported. So, if fcntl() errors out for
-  // any reason, we may end up making an unnecessary system call.
+  // unexpected data loss. Depending on experiment state, this function may use
+  // F_BARRIERFSYNC or F_FULLFSYNC to provide stronger guarantees than fsync().
   //
-  // See the CL description at https://crrev.com/c/1400159 for details.
+  // See documentation:
+  // https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fsync.2.html
+  //
+  // "relaxed" because there is no dependency between this memory operation and
+  // other memory operations.
+  switch (g_mac_file_flush_mechanism.load(std::memory_order_relaxed)) {
+    case MacFileFlushMechanism::kBarrierFsync: {
+      if (!HANDLE_EINTR(fcntl(file_.get(), F_BARRIERFSYNC))) {
+        return true;
+      }
+      // Fall back to `fsync()` in case of failure.
+      break;
+    }
+    case MacFileFlushMechanism::kFullFsync: {
+      if (!HANDLE_EINTR(fcntl(file_.get(), F_FULLFSYNC))) {
+        return true;
+      }
+      // Fall back to `fsync()` in case of failure.
+      break;
+    }
+    case MacFileFlushMechanism::kFlush: {
+      // Fall back to `fsync()`.
+      break;
+    }
+  }
+
+  // `fsync()` if `F_BARRIERFSYNC` or `F_FULLFSYNC` failed, or if the mechanism
+  // is `kFlush`. Some file systems do not support `F_FULLFSYNC` /
+  // `F_BARRIERFSYNC` but we cannot use the error code as a definitive indicator
+  // that it's the case, so we'll keep trying `F_FULLFSYNC` / `F_BARRIERFSYNC`
+  // for every call to this method when it's the case. See the CL description at
+  // https://crrev.com/c/1400159 for details.
   return !HANDLE_EINTR(fsync(file_.get()));
 #else
   return !HANDLE_EINTR(fsync(file_.get()));
@@ -586,33 +672,17 @@ File::Error File::GetLastFileError() {
   return base::File::OSErrorToFileError(errno);
 }
 
-#if defined(OS_BSD) || defined(OS_APPLE) || defined(OS_NACL) || \
-    defined(OS_FUCHSIA) || (defined(OS_ANDROID) && __ANDROID_API__ < 21)
-int File::Stat(const char* path, stat_wrapper_t* sb) {
+int File::Stat(const FilePath& path, stat_wrapper_t* sb) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  return stat(path, sb);
+  return stat(path.value().c_str(), sb);
 }
 int File::Fstat(int fd, stat_wrapper_t* sb) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
   return fstat(fd, sb);
 }
-int File::Lstat(const char* path, stat_wrapper_t* sb) {
+int File::Lstat(const FilePath& path, stat_wrapper_t* sb) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  return lstat(path, sb);
+  return lstat(path.value().c_str(), sb);
 }
-#else
-int File::Stat(const char* path, stat_wrapper_t* sb) {
-  ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  return stat64(path, sb);
-}
-int File::Fstat(int fd, stat_wrapper_t* sb) {
-  ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  return fstat64(fd, sb);
-}
-int File::Lstat(const char* path, stat_wrapper_t* sb) {
-  ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  return lstat64(path, sb);
-}
-#endif
 
 }  // namespace base
