@@ -1,7 +1,12 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/files/file.h"
 
 #include <stdint.h>
@@ -21,8 +26,12 @@
 #include "third_party/perfetto/include/perfetto/test/traced_value_test_support.h"  // no-presubmit-check nogncheck
 #endif  // BUILDFLAG(ENABLE_BASE_TRACING)
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #include <windows.h>
+
+#include "base/environment.h"
+#include "base/strings/utf_string_conversions.h"
+#include "base/test/gtest_util.h"
 #endif
 
 using base::File;
@@ -151,9 +160,8 @@ TEST(FileTest, DeleteOpenFile) {
   FilePath file_path = temp_dir.GetPath().AppendASCII("create_file_1");
 
   // Create a file.
-  File file(file_path,
-            base::File::FLAG_OPEN_ALWAYS | base::File::FLAG_READ |
-                base::File::FLAG_SHARE_DELETE);
+  File file(file_path, base::File::FLAG_OPEN_ALWAYS | base::File::FLAG_READ |
+                           base::File::FLAG_WIN_SHARE_DELETE);
   EXPECT_TRUE(file.IsValid());
   EXPECT_TRUE(file.created());
   EXPECT_EQ(base::File::FILE_OK, file.error_details());
@@ -219,10 +227,14 @@ TEST(FileTest, ReadWrite) {
     EXPECT_EQ(data_to_write[i], data_read_1[i]);
 
   // Read again, but using the trivial native wrapper.
-  bytes_read = file.ReadNoBestEffort(0, data_read_1, kTestDataSize);
-  EXPECT_LE(bytes_read, kTestDataSize);
-  for (int i = 0; i < bytes_read; i++)
+  std::optional<size_t> maybe_bytes_read =
+      file.ReadNoBestEffort(0, base::as_writable_byte_span(data_read_1)
+                                   .first(static_cast<size_t>(kTestDataSize)));
+  ASSERT_TRUE(maybe_bytes_read.has_value());
+  EXPECT_LE(maybe_bytes_read.value(), static_cast<size_t>(kTestDataSize));
+  for (size_t i = 0; i < maybe_bytes_read.value(); i++) {
     EXPECT_EQ(data_to_write[i], data_read_1[i]);
+  }
 
   // Write past the end of the file.
   const int kOffsetBeyondEndOfFile = 10;
@@ -248,8 +260,87 @@ TEST(FileTest, ReadWrite) {
     EXPECT_EQ(data_to_write[i - kOffsetBeyondEndOfFile], data_read_2[i]);
 }
 
+TEST(FileTest, ReadWriteSpans) {
+  base::ScopedTempDir temp_dir;
+  ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
+  FilePath file_path = temp_dir.GetPath().AppendASCII("read_write_file");
+  File file(file_path, base::File::FLAG_CREATE | base::File::FLAG_READ |
+                           base::File::FLAG_WRITE);
+  ASSERT_TRUE(file.IsValid());
+
+  // Write 0 bytes to the file.
+  std::optional<size_t> bytes_written = file.Write(0, base::span<uint8_t>());
+  ASSERT_TRUE(bytes_written.has_value());
+  EXPECT_EQ(0u, bytes_written.value());
+
+  // Write "test" to the file.
+  std::string data_to_write("test");
+  bytes_written = file.Write(0, base::as_byte_span(data_to_write));
+  ASSERT_TRUE(bytes_written.has_value());
+  EXPECT_EQ(data_to_write.size(), bytes_written.value());
+
+  // Read from EOF.
+  uint8_t data_read_1[32];
+  std::optional<size_t> bytes_read =
+      file.Read(bytes_written.value(), data_read_1);
+  ASSERT_TRUE(bytes_read.has_value());
+  EXPECT_EQ(0u, bytes_read.value());
+
+  // Read from somewhere in the middle of the file.
+  const int kPartialReadOffset = 1;
+  bytes_read = file.Read(kPartialReadOffset, data_read_1);
+  ASSERT_TRUE(bytes_read.has_value());
+  EXPECT_EQ(bytes_written.value() - kPartialReadOffset, bytes_read.value());
+  for (size_t i = 0; i < bytes_read.value(); i++) {
+    EXPECT_EQ(data_to_write[i + kPartialReadOffset], data_read_1[i]);
+  }
+
+  // Read 0 bytes.
+  bytes_read = file.Read(0, base::span<uint8_t>());
+  ASSERT_TRUE(bytes_read.has_value());
+  EXPECT_EQ(0u, bytes_read.value());
+
+  // Read the entire file.
+  bytes_read = file.Read(0, data_read_1);
+  ASSERT_TRUE(bytes_read.has_value());
+  EXPECT_EQ(data_to_write.size(), bytes_read.value());
+  for (int i = 0; i < bytes_read; i++) {
+    EXPECT_EQ(data_to_write[i], data_read_1[i]);
+  }
+
+  // Write past the end of the file.
+  const size_t kOffsetBeyondEndOfFile = 10;
+  const size_t kPartialWriteLength = 2;
+  bytes_written =
+      file.Write(kOffsetBeyondEndOfFile,
+                 base::as_byte_span(data_to_write).first(kPartialWriteLength));
+  ASSERT_TRUE(bytes_written.has_value());
+  EXPECT_EQ(kPartialWriteLength, bytes_written.value());
+
+  // Make sure the file was extended.
+  int64_t file_size = 0;
+  EXPECT_TRUE(GetFileSize(file_path, &file_size));
+  EXPECT_EQ(static_cast<int64_t>(kOffsetBeyondEndOfFile + kPartialWriteLength),
+            file_size);
+
+  // Make sure the file was zero-padded.
+  uint8_t data_read_2[32];
+  bytes_read = file.Read(0, data_read_2);
+  ASSERT_TRUE(bytes_read.has_value());
+  EXPECT_EQ(file_size, static_cast<int64_t>(bytes_read.value()));
+  for (size_t i = 0; i < data_to_write.size(); i++) {
+    EXPECT_EQ(data_to_write[i], data_read_2[i]);
+  }
+  for (size_t i = data_to_write.size(); i < kOffsetBeyondEndOfFile; i++) {
+    EXPECT_EQ(0, data_read_2[i]);
+  }
+  for (size_t i = 0; i < kPartialWriteLength; i++) {
+    EXPECT_EQ(data_to_write[i], data_read_2[i + kOffsetBeyondEndOfFile]);
+  }
+}
+
 TEST(FileTest, GetLastFileError) {
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   ::SetLastError(ERROR_ACCESS_DENIED);
 #else
   errno = EACCES;
@@ -365,6 +456,15 @@ TEST(FileTest, Length) {
   for (int i = 0; i < file_size; i++)
     EXPECT_EQ(data_to_write[i], data_read[i]);
 
+#if !BUILDFLAG(IS_FUCHSIA)  // Fuchsia doesn't seem to support big files.
+  // Expand the file past the 4 GB limit.
+  const int64_t kBigFileLength = 5'000'000'000;
+  EXPECT_TRUE(file.SetLength(kBigFileLength));
+  EXPECT_EQ(kBigFileLength, file.GetLength());
+  EXPECT_TRUE(GetFileSize(file_path, &file_size));
+  EXPECT_EQ(kBigFileLength, file_size);
+#endif
+
   // Close the file and reopen with base::File::FLAG_CREATE_ALWAYS, and make
   // sure the file is empty (old file was overridden).
   file.Close();
@@ -374,7 +474,7 @@ TEST(FileTest, Length) {
 }
 
 // Flakily fails: http://crbug.com/86494
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 TEST(FileTest, TouchGetInfo) {
 #else
 TEST(FileTest, DISABLED_TouchGetInfo) {
@@ -392,7 +492,7 @@ TEST(FileTest, DISABLED_TouchGetInfo) {
 
   // Add 2 seconds to account for possible rounding errors on
   // filesystems that use a 1s or 2s timestamp granularity.
-  base::Time now = base::Time::Now() + base::TimeDelta::FromSeconds(2);
+  base::Time now = base::Time::Now() + base::Seconds(2);
   EXPECT_EQ(0, info.size);
   EXPECT_FALSE(info.is_directory);
   EXPECT_FALSE(info.is_symbolic_link);
@@ -411,10 +511,8 @@ TEST(FileTest, DISABLED_TouchGetInfo) {
   // It's best to add values that are multiples of 2 (in seconds)
   // to the current last_accessed and last_modified times, because
   // FATxx uses a 2s timestamp granularity.
-  base::Time new_last_accessed =
-      info.last_accessed + base::TimeDelta::FromSeconds(234);
-  base::Time new_last_modified =
-      info.last_modified + base::TimeDelta::FromMinutes(567);
+  base::Time new_last_accessed = info.last_accessed + base::Seconds(234);
+  base::Time new_last_modified = info.last_modified + base::Minutes(567);
 
   EXPECT_TRUE(file.SetTimes(new_last_accessed, new_last_modified));
 
@@ -425,7 +523,7 @@ TEST(FileTest, DISABLED_TouchGetInfo) {
   EXPECT_FALSE(info.is_symbolic_link);
 
   // ext2/ext3 and HPS/HPS+ seem to have a timestamp granularity of 1s.
-#if defined(OS_POSIX)
+#if BUILDFLAG(IS_POSIX)
   EXPECT_EQ(info.last_accessed.ToTimeVal().tv_sec,
             new_last_accessed.ToTimeVal().tv_sec);
   EXPECT_EQ(info.last_modified.ToTimeVal().tv_sec,
@@ -489,6 +587,38 @@ TEST(FileTest, ReadAtCurrentPosition) {
   EXPECT_EQ(std::string(buffer, buffer + kDataSize), std::string(kData));
 }
 
+TEST(FileTest, ReadAtCurrentPositionSpans) {
+  base::ScopedTempDir temp_dir;
+  ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
+  FilePath file_path =
+      temp_dir.GetPath().AppendASCII("read_at_current_position");
+  File file(file_path, base::File::FLAG_CREATE | base::File::FLAG_READ |
+                           base::File::FLAG_WRITE);
+  EXPECT_TRUE(file.IsValid());
+
+  std::string data("test");
+  std::optional<size_t> result = file.Write(0, base::as_byte_span(data));
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(data.size(), result.value());
+
+  EXPECT_EQ(0, file.Seek(base::File::FROM_BEGIN, 0));
+
+  uint8_t buffer[4];
+  size_t first_chunk_size = 2;
+  result =
+      file.ReadAtCurrentPos(base::make_span(buffer).first(first_chunk_size));
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(first_chunk_size, result.value());
+
+  result =
+      file.ReadAtCurrentPos(base::make_span(buffer).subspan(first_chunk_size));
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(first_chunk_size, result.value());
+  for (size_t i = 0; i < data.size(); i++) {
+    EXPECT_EQ(data[i], static_cast<char>(buffer[i]));
+  }
+}
+
 TEST(FileTest, WriteAtCurrentPosition) {
   base::ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
@@ -513,6 +643,33 @@ TEST(FileTest, WriteAtCurrentPosition) {
   EXPECT_EQ(std::string(buffer, buffer + kDataSize), std::string(kData));
 }
 
+TEST(FileTest, WriteAtCurrentPositionSpans) {
+  base::ScopedTempDir temp_dir;
+  ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
+  FilePath file_path =
+      temp_dir.GetPath().AppendASCII("write_at_current_position");
+  File file(file_path, base::File::FLAG_CREATE | base::File::FLAG_READ |
+                           base::File::FLAG_WRITE);
+  EXPECT_TRUE(file.IsValid());
+
+  std::string data("test");
+  size_t first_chunk_size = data.size() / 2;
+  std::optional<size_t> result =
+      file.WriteAtCurrentPos(base::as_byte_span(data).first(first_chunk_size));
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(first_chunk_size, result.value());
+
+  result = file.WriteAtCurrentPos(
+      base::as_byte_span(data).subspan(first_chunk_size));
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(first_chunk_size, result.value());
+
+  const int kDataSize = 4;
+  char buffer[kDataSize];
+  EXPECT_EQ(kDataSize, file.Read(0, buffer, kDataSize));
+  EXPECT_EQ(std::string(buffer, buffer + kDataSize), data);
+}
+
 TEST(FileTest, Seek) {
   base::ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
@@ -589,7 +746,7 @@ TEST(FileTest, TracedValueSupport) {
 }
 #endif  // BUILDFLAG(ENABLE_BASE_TRACING)
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Flakily times out on Windows, see http://crbug.com/846276.
 #define MAYBE_WriteDataToLargeOffset DISABLED_WriteDataToLargeOffset
 #else
@@ -616,7 +773,21 @@ TEST(FileTest, MAYBE_WriteDataToLargeOffset) {
   ASSERT_EQ(kDataLen, file.Write(kLargeFileOffset + 1, kData, kDataLen));
 }
 
-#if defined(OS_WIN)
+TEST(FileTest, AddFlagsForPassingToUntrustedProcess) {
+  {
+    uint32_t flags = base::File::FLAG_OPEN | base::File::FLAG_READ;
+    flags = base::File::AddFlagsForPassingToUntrustedProcess(flags);
+    EXPECT_EQ(flags, base::File::FLAG_OPEN | base::File::FLAG_READ);
+  }
+  {
+    uint32_t flags = base::File::FLAG_OPEN | base::File::FLAG_WRITE;
+    flags = base::File::AddFlagsForPassingToUntrustedProcess(flags);
+    EXPECT_EQ(flags, base::File::FLAG_OPEN | base::File::FLAG_WRITE |
+                         base::File::FLAG_WIN_NO_EXECUTE);
+  }
+}
+
+#if BUILDFLAG(IS_WIN)
 TEST(FileTest, GetInfoForDirectory) {
   base::ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
@@ -692,11 +863,11 @@ TEST(FileTest, IrrevokableDeleteOnClose) {
   FilePath file_path = temp_dir.GetPath().AppendASCII("file");
 
   // DELETE_ON_CLOSE cannot be revoked by this opener.
-  File file(
-      file_path,
-      (base::File::FLAG_CREATE | base::File::FLAG_READ |
-       base::File::FLAG_WRITE | base::File::FLAG_DELETE_ON_CLOSE |
-       base::File::FLAG_SHARE_DELETE | base::File::FLAG_CAN_DELETE_ON_CLOSE));
+  File file(file_path,
+            (base::File::FLAG_CREATE | base::File::FLAG_READ |
+             base::File::FLAG_WRITE | base::File::FLAG_DELETE_ON_CLOSE |
+             base::File::FLAG_WIN_SHARE_DELETE |
+             base::File::FLAG_CAN_DELETE_ON_CLOSE));
   ASSERT_TRUE(file.IsValid());
   // https://msdn.microsoft.com/library/windows/desktop/aa364221.aspx says that
   // setting the dispositon has no effect if the handle was opened with
@@ -714,17 +885,17 @@ TEST(FileTest, IrrevokableDeleteOnCloseOther) {
   FilePath file_path = temp_dir.GetPath().AppendASCII("file");
 
   // DELETE_ON_CLOSE cannot be revoked by another opener.
-  File file(
-      file_path,
-      (base::File::FLAG_CREATE | base::File::FLAG_READ |
-       base::File::FLAG_WRITE | base::File::FLAG_DELETE_ON_CLOSE |
-       base::File::FLAG_SHARE_DELETE | base::File::FLAG_CAN_DELETE_ON_CLOSE));
+  File file(file_path,
+            (base::File::FLAG_CREATE | base::File::FLAG_READ |
+             base::File::FLAG_WRITE | base::File::FLAG_DELETE_ON_CLOSE |
+             base::File::FLAG_WIN_SHARE_DELETE |
+             base::File::FLAG_CAN_DELETE_ON_CLOSE));
   ASSERT_TRUE(file.IsValid());
 
-  File file2(
-      file_path,
-      (base::File::FLAG_OPEN | base::File::FLAG_READ | base::File::FLAG_WRITE |
-       base::File::FLAG_SHARE_DELETE | base::File::FLAG_CAN_DELETE_ON_CLOSE));
+  File file2(file_path,
+             (base::File::FLAG_OPEN | base::File::FLAG_READ |
+              base::File::FLAG_WRITE | base::File::FLAG_WIN_SHARE_DELETE |
+              base::File::FLAG_CAN_DELETE_ON_CLOSE));
   ASSERT_TRUE(file2.IsValid());
 
   file2.DeleteOnClose(false);
@@ -762,7 +933,7 @@ TEST(FileTest, UnsharedDeleteOnClose) {
   File file2(
       file_path,
       (base::File::FLAG_OPEN | base::File::FLAG_READ | base::File::FLAG_WRITE |
-       base::File::FLAG_DELETE_ON_CLOSE | base::File::FLAG_SHARE_DELETE));
+       base::File::FLAG_DELETE_ON_CLOSE | base::File::FLAG_WIN_SHARE_DELETE));
   ASSERT_FALSE(file2.IsValid());
 
   file.Close();
@@ -806,4 +977,31 @@ TEST(FileTest, UseSyncApiWithAsyncFile) {
 
   ASSERT_EQ(lying_file.WriteAtCurrentPos("12345", 5), -1);
 }
-#endif  // defined(OS_WIN)
+
+TEST(FileDeathTest, InvalidFlags) {
+  EXPECT_CHECK_DEATH_WITH(
+      {
+        // When this test is running as Admin, TMP gets ignored and temporary
+        // files/folders are created in %ProgramFiles%. This means that the
+        // temporary folder created by the death test never gets deleted, as it
+        // crashes before the `base::ScopedTempDir` goes out of scope and also
+        // does not get automatically cleaned by by the test runner.
+        //
+        // To avoid this from happening, this death test explicitly creates the
+        // temporary folder in TMP, which is set by the test runner parent
+        // process to a temporary folder for the test. This means that the
+        // folder created here is always deleted during test runner cleanup.
+        std::string tmp_folder;
+        ASSERT_TRUE(base::Environment::Create()->GetVar("TMP", &tmp_folder));
+        base::ScopedTempDir temp_dir;
+        ASSERT_TRUE(temp_dir.CreateUniqueTempDirUnderPath(
+            base::FilePath(base::UTF8ToWide(tmp_folder))));
+        FilePath file_path = temp_dir.GetPath().AppendASCII("file");
+
+        File file(file_path,
+                  base::File::FLAG_CREATE | base::File::FLAG_WIN_EXECUTE |
+                      base::File::FLAG_READ | base::File::FLAG_WIN_NO_EXECUTE);
+      },
+      "FLAG_WIN_NO_EXECUTE");
+}
+#endif  // BUILDFLAG(IS_WIN)
