@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/functional/bind.h"
 
 #include <functional>
@@ -12,10 +17,6 @@
 
 #include "base/allocator/partition_alloc_features.h"
 #include "base/allocator/partition_alloc_support.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/dangling_raw_ptr_checks.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_alloc_buildflags.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_alloc_for_testing.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_root.h"
 #include "base/functional/callback.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/raw_ptr.h"
@@ -27,6 +28,10 @@
 #include "base/test/gtest_util.h"
 #include "base/test/scoped_feature_list.h"
 #include "build/build_config.h"
+#include "partition_alloc/buildflags.h"
+#include "partition_alloc/dangling_raw_ptr_checks.h"
+#include "partition_alloc/partition_alloc_for_testing.h"
+#include "partition_alloc/partition_root.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -523,7 +528,7 @@ TEST_F(BindTest, IgnoreResultForRepeatingCallback) {
   std::string s;
   RepeatingCallback<int(int)> cb = BindRepeating(
       [](std::string* s, int i) {
-        *s += "Run" + base::NumberToString(i);
+        *s += "Run" + NumberToString(i);
         return 5;
       },
       &s);
@@ -536,7 +541,7 @@ TEST_F(BindTest, IgnoreResultForOnceCallback) {
   std::string s;
   OnceCallback<int(int)> cb = BindOnce(
       [](std::string* s, int i) {
-        *s += "Run" + base::NumberToString(i);
+        *s += "Run" + NumberToString(i);
         return 5;
       },
       &s);
@@ -569,26 +574,26 @@ TEST_F(BindTest, BindOnceWithNonConstRef) {
 
   // Everything past here following will make a copy of the argument. The copy
   // will be mutated and leave `v` unmodified.
-  auto cb3 = BindOnce(SetFromRef, base::OwnedRef(v));
+  auto cb3 = BindOnce(SetFromRef, OwnedRef(v));
   std::move(cb3).Run();
   EXPECT_EQ(v, 1);
 
   int& ref = v;
-  auto cb4 = BindOnce(SetFromRef, base::OwnedRef(ref));
+  auto cb4 = BindOnce(SetFromRef, OwnedRef(ref));
   std::move(cb4).Run();
   EXPECT_EQ(v, 1);
 
   const int cv = 1;
-  auto cb5 = BindOnce(SetFromRef, base::OwnedRef(cv));
+  auto cb5 = BindOnce(SetFromRef, OwnedRef(cv));
   std::move(cb5).Run();
   EXPECT_EQ(cv, 1);
 
   const int& cref = v;
-  auto cb6 = BindOnce(SetFromRef, base::OwnedRef(cref));
+  auto cb6 = BindOnce(SetFromRef, OwnedRef(cref));
   std::move(cb6).Run();
   EXPECT_EQ(cref, 1);
 
-  auto cb7 = BindOnce(SetFromRef, base::OwnedRef(1));
+  auto cb7 = BindOnce(SetFromRef, OwnedRef(1));
   std::move(cb7).Run();
 }
 
@@ -610,26 +615,26 @@ TEST_F(BindTest, BindRepeatingWithNonConstRef) {
 
   // Everything past here following will make a copy of the argument. The copy
   // will be mutated and leave `v` unmodified.
-  auto cb3 = BindRepeating(SetFromRef, base::OwnedRef(v));
+  auto cb3 = BindRepeating(SetFromRef, OwnedRef(v));
   std::move(cb3).Run();
   EXPECT_EQ(v, 1);
 
   int& ref = v;
-  auto cb4 = BindRepeating(SetFromRef, base::OwnedRef(ref));
+  auto cb4 = BindRepeating(SetFromRef, OwnedRef(ref));
   std::move(cb4).Run();
   EXPECT_EQ(v, 1);
 
   const int cv = 1;
-  auto cb5 = BindRepeating(SetFromRef, base::OwnedRef(cv));
+  auto cb5 = BindRepeating(SetFromRef, OwnedRef(cv));
   std::move(cb5).Run();
   EXPECT_EQ(cv, 1);
 
   const int& cref = v;
-  auto cb6 = BindRepeating(SetFromRef, base::OwnedRef(cref));
+  auto cb6 = BindRepeating(SetFromRef, OwnedRef(cref));
   std::move(cb6).Run();
   EXPECT_EQ(cref, 1);
 
-  auto cb7 = BindRepeating(SetFromRef, base::OwnedRef(1));
+  auto cb7 = BindRepeating(SetFromRef, OwnedRef(1));
   std::move(cb7).Run();
 }
 
@@ -1293,18 +1298,27 @@ TEST_F(BindTest, BindMoveOnlyVector) {
   v.push_back(std::make_unique<int>(12345));
 
   // Early binding should work:
-  base::RepeatingCallback<MoveOnlyVector()> bound_cb =
-      base::BindRepeating(&AcceptAndReturnMoveOnlyVector, Passed(&v));
+  RepeatingCallback<MoveOnlyVector()> bound_cb =
+      BindRepeating(&AcceptAndReturnMoveOnlyVector, Passed(&v));
   MoveOnlyVector intermediate_result = bound_cb.Run();
   VerifyVector(intermediate_result);
 
   // As should passing it as an argument to Run():
-  base::RepeatingCallback<MoveOnlyVector(MoveOnlyVector)> unbound_cb =
-      base::BindRepeating(&AcceptAndReturnMoveOnlyVector);
+  RepeatingCallback<MoveOnlyVector(MoveOnlyVector)> unbound_cb =
+      BindRepeating(&AcceptAndReturnMoveOnlyVector);
   MoveOnlyVector final_result = unbound_cb.Run(std::move(intermediate_result));
   VerifyVector(final_result);
 }
 
+// Using Passed() on a functor should not cause a compile error.
+TEST_F(BindTest, PassedFunctor) {
+  struct S {
+    void operator()() const {}
+  };
+
+  BindRepeating(Passed(S())).Run();
+}
+
 // Argument copy-constructor usage for non-reference copy-only parameters.
 //   - Bound arguments are only copied once.
 //   - Forwarded arguments are only copied once.
@@ -1474,27 +1488,11 @@ TEST_F(BindTest, RepeatingWithoutPassed) {
 }
 
 TEST_F(BindTest, CapturelessLambda) {
-  EXPECT_FALSE(internal::IsCallableObject<void>);
-  EXPECT_FALSE(internal::IsCallableObject<int>);
-  EXPECT_FALSE(internal::IsCallableObject<void (*)()>);
-  EXPECT_FALSE(internal::IsCallableObject<void (NoRef::*)()>);
-
-  auto f = [] {};
-  EXPECT_TRUE(internal::IsCallableObject<decltype(f)>);
-
-  int i = 0;
-  auto g = [i] { (void)i; };
-  EXPECT_TRUE(internal::IsCallableObject<decltype(g)>);
-
-  auto h = [](int, double) { return 'k'; };
-  EXPECT_TRUE((std::is_same_v<char(int, double),
-                              internal::ExtractCallableRunType<decltype(h)>>));
-
   EXPECT_EQ(42, BindRepeating([] { return 42; }).Run());
   EXPECT_EQ(42, BindRepeating([](int i) { return i * 7; }, 6).Run());
 
   int x = 1;
-  base::RepeatingCallback<void(int)> cb =
+  RepeatingCallback<void(int)> cb =
       BindRepeating([](int* x, int i) { *x *= i; }, Unretained(&x));
   cb.Run(6);
   EXPECT_EQ(6, x);
@@ -1516,9 +1514,7 @@ TEST_F(BindTest, EmptyFunctor) {
     int operator()() const { return 42; }
   };
 
-  EXPECT_TRUE(internal::IsCallableObject<NonEmptyFunctor>);
-  EXPECT_TRUE(internal::IsCallableObject<EmptyFunctor>);
-  EXPECT_TRUE(internal::IsCallableObject<EmptyFunctorConst>);
+  EXPECT_EQ(42, BindLambdaForTesting(NonEmptyFunctor()).Run());
   EXPECT_EQ(42, BindOnce(EmptyFunctor()).Run());
   EXPECT_EQ(42, BindOnce(EmptyFunctorConst()).Run());
   EXPECT_EQ(42, BindRepeating(EmptyFunctorConst()).Run());
@@ -1685,12 +1681,12 @@ TEST_F(BindTest, WindowsCallingConventions) {
 
   MethodHolder obj;
   auto stdcall_method_cb =
-      BindRepeating(&MethodHolder::Func, base::Unretained(&obj), 1);
+      BindRepeating(&MethodHolder::Func, Unretained(&obj), 1);
   EXPECT_EQ(1, stdcall_method_cb.Run());
 
   const MethodHolder const_obj;
   auto stdcall_const_method_cb =
-      BindRepeating(&MethodHolder::ConstFunc, base::Unretained(&const_obj), 1);
+      BindRepeating(&MethodHolder::ConstFunc, Unretained(&const_obj), 1);
   EXPECT_EQ(-1, stdcall_const_method_cb.Run());
 }
 #endif
@@ -1738,15 +1734,12 @@ TEST_F(BindTest, UnwrapPassed) {
 }
 
 TEST_F(BindTest, BindNoexcept) {
-  EXPECT_EQ(42, base::BindOnce(&Noexcept).Run());
-  EXPECT_EQ(
-      42,
-      base::BindOnce(&BindTest::NoexceptMethod, base::Unretained(this)).Run());
-  EXPECT_EQ(
-      42, base::BindOnce(&BindTest::ConstNoexceptMethod, base::Unretained(this))
-              .Run());
-  EXPECT_EQ(42, base::BindOnce(NoexceptFunctor()).Run());
-  EXPECT_EQ(42, base::BindOnce(ConstNoexceptFunctor()).Run());
+  EXPECT_EQ(42, BindOnce(&Noexcept).Run());
+  EXPECT_EQ(42, BindOnce(&BindTest::NoexceptMethod, Unretained(this)).Run());
+  EXPECT_EQ(42,
+            BindOnce(&BindTest::ConstNoexceptMethod, Unretained(this)).Run());
+  EXPECT_EQ(42, BindOnce(NoexceptFunctor()).Run());
+  EXPECT_EQ(42, BindOnce(ConstNoexceptFunctor()).Run());
 }
 
 int PingPong(int* i_ptr) {
@@ -1757,7 +1750,7 @@ TEST_F(BindTest, BindAndCallbacks) {
   int i = 123;
   raw_ptr<int> p = &i;
 
-  auto callback = base::BindOnce(PingPong, base::Unretained(p));
+  auto callback = BindOnce(PingPong, Unretained(p));
   int res = std::move(callback).Run();
   EXPECT_EQ(123, res);
 }
@@ -1780,13 +1773,64 @@ TEST_F(BindTest, ConvertibleArgs) {
   // Ensure it's possible to pass a T to a function expecting an S.
   void (*foo)(S) = +[](S) {};
   const T t;
-  auto callback = base::BindOnce(foo, t);
+  auto callback = BindOnce(foo, t);
   std::move(callback).Run();
 }
 
+TEST_F(BindTest, OverloadedOperator) {
+  // Bind should be able to pick the correct `operator()()` to invoke on a
+  // functor from the supplied args.
+  struct S {
+    int operator()(int x) { return x; }
+    std::string operator()(std::string s) { return s; }
+  } s;
+
+  EXPECT_EQ(42, BindOnce(s, 42).Run());
+  EXPECT_EQ("Hello", BindOnce(s, "Hello").Run());
+}
+
+TEST_F(BindTest, OverloadedOperatorQualifiers) {
+  // Bind should be able to pick the correct `operator()()` to invoke on a
+  // functor when the only difference between the overloads is their qualifiers.
+  struct S {
+    int operator()() const& { return 1; }
+    int operator()() && { return 2; }
+  } s;
+
+  // `BindRepeating()` normally stores a value and passes a const ref to the
+  // invoked method, regardless of whether lvalue or rvalue was originally
+  // provided.
+  EXPECT_EQ(1, BindRepeating(s).Run());
+  EXPECT_EQ(1, BindRepeating(S()).Run());
+
+  // The exception is if `Passed()` is used, which tells `BindRepeating()` to
+  // move the specified argument during invocation.
+  EXPECT_EQ(2, BindRepeating(Passed(S())).Run());
+
+  // `BindOnce()` also stores a value, but it always moves that value during
+  // invocation, regardless of whether lvalue or rvalue was originally provided.
+  EXPECT_EQ(2, BindOnce(s).Run());
+  EXPECT_EQ(2, BindOnce(S()).Run());
+}
+
+TEST_F(BindTest, OverloadedOperatorInexactMatch) {
+  // The Bind machinery guesses signatures for overloaded `operator()()`s based
+  // on the decay_t<>s of the bound args. But as long as all args are bound and
+  // are convertible to exactly one overload's params, everything should work,
+  // even if the guess is slightly incorrect.
+  struct S {
+    int operator()(int x) { return x; }
+    // Machinery will guess that param type here is `std::string`.
+    std::string operator()(const std::string& s) { return s; }
+  } s;
+
+  EXPECT_EQ(42, BindOnce(s, 42).Run());
+  EXPECT_EQ("Hello", BindOnce(s, "Hello").Run());
+}
+
 }  // namespace
 
-// This simulates a race weak pointer that, unlike our `base::WeakPtr<>`,
+// This simulates a race weak pointer that, unlike our `WeakPtr<>`,
 // may become invalidated between `operator bool()` is tested and `Lock()`
 // is called in the implementation of `Unwrap()`.
 template <typename T>
@@ -1818,43 +1862,44 @@ namespace {
 TEST_F(BindTest, BindRacyWeakPtrTest) {
   MockRacyWeakPtr<NoRef> weak(&no_ref_);
 
-  RepeatingClosure cb = base::BindRepeating(&NoRef::VoidMethod0, weak);
+  RepeatingClosure cb = BindRepeating(&NoRef::VoidMethod0, weak);
   cb.Run();
 }
 
 // Test null callbacks cause a DCHECK.
 TEST(BindDeathTest, NullCallback) {
-  base::RepeatingCallback<void(int)> null_cb;
+  RepeatingCallback<void(int)> null_cb;
   ASSERT_TRUE(null_cb.is_null());
-  EXPECT_CHECK_DEATH(base::BindRepeating(null_cb, 42));
+  EXPECT_CHECK_DEATH(BindRepeating(null_cb, 42));
 }
 
 TEST(BindDeathTest, NullFunctionPointer) {
   void (*null_function)(int) = nullptr;
-  EXPECT_DCHECK_DEATH(base::BindRepeating(null_function, 42));
+  EXPECT_DCHECK_DEATH(BindRepeating(null_function, 42));
 }
 
 TEST(BindDeathTest, NullCallbackWithoutBoundArgs) {
-  base::OnceCallback<void(int)> null_cb;
+  OnceCallback<void(int)> null_cb;
   ASSERT_TRUE(null_cb.is_null());
-  EXPECT_CHECK_DEATH(base::BindOnce(std::move(null_cb)));
+  EXPECT_CHECK_DEATH(BindOnce(std::move(null_cb)));
 }
 
 TEST(BindDeathTest, BanFirstOwnerOfRefCountedType) {
   StrictMock<HasRef> has_ref;
   EXPECT_DCHECK_DEATH({
     EXPECT_CALL(has_ref, HasAtLeastOneRef()).WillOnce(Return(false));
-    base::BindOnce(&HasRef::VoidMethod0, &has_ref);
+    BindOnce(&HasRef::VoidMethod0, &has_ref);
   });
 
   EXPECT_DCHECK_DEATH({
     raw_ptr<HasRef> rawptr(&has_ref);
     EXPECT_CALL(has_ref, HasAtLeastOneRef()).WillOnce(Return(false));
-    base::BindOnce(&HasRef::VoidMethod0, rawptr);
+    BindOnce(&HasRef::VoidMethod0, rawptr);
   });
 }
 
-#if BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#if PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT) && \
+    PA_BUILDFLAG(USE_RAW_PTR_BACKUP_REF_IMPL)
 
 void HandleOOM(size_t unused_size) {
   LOG(FATAL) << "Out of memory";
@@ -1942,14 +1987,14 @@ class ClassWithWeakPtr {
 
 TEST_F(BindUnretainedDanglingTest, UnretainedNoDanglingPtr) {
   raw_ptr<int> p = Alloc<int>(3);
-  auto callback = base::BindOnce(PingPong, base::Unretained(p));
+  auto callback = BindOnce(PingPong, Unretained(p));
   EXPECT_EQ(std::move(callback).Run(), 3);
   Free(p);
 }
 
 TEST_F(BindUnretainedDanglingTest, UnsafeDanglingPtr) {
   raw_ptr<int> p = Alloc<int>(3);
-  auto callback = base::BindOnce(MayBeDanglingCheckFn, base::UnsafeDangling(p));
+  auto callback = BindOnce(MayBeDanglingCheckFn, UnsafeDangling(p));
   Free(p);
   EXPECT_EQ(std::move(callback).Run(), true);
 }
@@ -1957,8 +2002,8 @@ TEST_F(BindUnretainedDanglingTest, UnsafeDanglingPtr) {
 TEST_F(BindUnretainedDanglingTest, UnsafeDanglingPtrWithDummyTrait) {
   raw_ptr<int, RawPtrTraits::kDummyForTest> p =
       Alloc<int, RawPtrTraits::kDummyForTest>(3);
-  auto callback = base::BindOnce(MayBeDanglingAndDummyTraitCheckFn,
-                                 base::UnsafeDangling(p));
+  auto callback =
+      BindOnce(MayBeDanglingAndDummyTraitCheckFn, UnsafeDangling(p));
   Free(p);
   EXPECT_EQ(std::move(callback).Run(), true);
 }
@@ -1967,8 +2012,8 @@ TEST_F(BindUnretainedDanglingTest,
        UnsafeDanglingPtrWithDummyAndDanglingTraits) {
   raw_ptr<int, RawPtrTraits::kDummyForTest | RawPtrTraits::kMayDangle> p =
       Alloc<int, RawPtrTraits::kDummyForTest | RawPtrTraits::kMayDangle>(3);
-  auto callback = base::BindOnce(MayBeDanglingAndDummyTraitCheckFn,
-                                 base::UnsafeDangling(p));
+  auto callback =
+      BindOnce(MayBeDanglingAndDummyTraitCheckFn, UnsafeDangling(p));
   Free(p);
   EXPECT_EQ(std::move(callback).Run(), true);
 }
@@ -1976,16 +2021,15 @@ TEST_F(BindUnretainedDanglingTest,
 TEST_F(BindUnretainedDanglingTest, UnsafeDanglingPtrNoRawPtrReceiver) {
   std::unique_ptr<ClassWithWeakPtr> r = std::make_unique<ClassWithWeakPtr>();
   int val = 0;
-  auto callback =
-      base::BindOnce(&ClassWithWeakPtr::RawPtrArg,
-                     base::UnsafeDangling(r.get()), base::Unretained(&val));
+  auto callback = BindOnce(&ClassWithWeakPtr::RawPtrArg,
+                           UnsafeDangling(r.get()), Unretained(&val));
   std::move(callback).Run();
   EXPECT_EQ(val, 123);
 }
 
 TEST_F(BindUnretainedDanglingTest, UnsafeDanglingUntriagedPtr) {
   raw_ptr<int> p = Alloc<int>(3);
-  auto callback = base::BindOnce(PtrCheckFn, base::UnsafeDanglingUntriaged(p));
+  auto callback = BindOnce(PtrCheckFn, UnsafeDanglingUntriaged(p));
   Free(p);
   EXPECT_EQ(std::move(callback).Run(), true);
 }
@@ -1993,8 +2037,8 @@ TEST_F(BindUnretainedDanglingTest, UnsafeDanglingUntriagedPtr) {
 TEST_F(BindUnretainedDanglingTest, UnretainedWeakReceiverValidNoDangling) {
   raw_ptr<int> p = Alloc<int>(3);
   std::unique_ptr<ClassWithWeakPtr> r = std::make_unique<ClassWithWeakPtr>();
-  auto callback = base::BindOnce(&ClassWithWeakPtr::RawPtrArg, r->GetWeakPtr(),
-                                 base::Unretained(p));
+  auto callback =
+      BindOnce(&ClassWithWeakPtr::RawPtrArg, r->GetWeakPtr(), Unretained(p));
   std::move(callback).Run();
   EXPECT_EQ(*p, 123);
   Free(p);
@@ -2004,8 +2048,8 @@ TEST_F(BindUnretainedDanglingTest, UnretainedRefWeakReceiverValidNoDangling) {
   raw_ptr<int> p = Alloc<int>(3);
   int& ref = *p;
   std::unique_ptr<ClassWithWeakPtr> r = std::make_unique<ClassWithWeakPtr>();
-  auto callback = base::BindOnce(&ClassWithWeakPtr::RawRefArg, r->GetWeakPtr(),
-                                 std::ref(ref));
+  auto callback =
+      BindOnce(&ClassWithWeakPtr::RawRefArg, r->GetWeakPtr(), std::ref(ref));
   std::move(callback).Run();
   EXPECT_EQ(*p, 123);
   Free(p);
@@ -2014,8 +2058,8 @@ TEST_F(BindUnretainedDanglingTest, UnretainedRefWeakReceiverValidNoDangling) {
 TEST_F(BindUnretainedDanglingTest, UnretainedWeakReceiverInvalidNoDangling) {
   raw_ptr<int> p = Alloc<int>(3);
   std::unique_ptr<ClassWithWeakPtr> r = std::make_unique<ClassWithWeakPtr>();
-  auto callback = base::BindOnce(&ClassWithWeakPtr::RawPtrArg, r->GetWeakPtr(),
-                                 base::Unretained(p));
+  auto callback =
+      BindOnce(&ClassWithWeakPtr::RawPtrArg, r->GetWeakPtr(), Unretained(p));
   r.reset();
   Free(p);
   std::move(callback).Run();
@@ -2027,8 +2071,8 @@ TEST_F(BindUnretainedDanglingTest, UnretainedRefWeakReceiverInvalidNoDangling) {
   raw_ptr<int> p = Alloc<int>(3);
   int& ref = *p;
   std::unique_ptr<ClassWithWeakPtr> r = std::make_unique<ClassWithWeakPtr>();
-  auto callback = base::BindOnce(&ClassWithWeakPtr::RawRefArg, r->GetWeakPtr(),
-                                 std::ref(ref));
+  auto callback =
+      BindOnce(&ClassWithWeakPtr::RawRefArg, r->GetWeakPtr(), std::ref(ref));
   r.reset();
   Free(p);
   std::move(callback).Run();
@@ -2039,8 +2083,7 @@ TEST_F(BindUnretainedDanglingTest, UnretainedRefWeakReceiverInvalidNoDangling) {
 TEST_F(BindUnretainedDanglingTest, UnretainedRefUnsafeDangling) {
   raw_ptr<int> p = Alloc<int>(3);
   int& ref = *p;
-  auto callback =
-      base::BindOnce(RefCheckFn, base::UnsafeDangling(base::raw_ref<int>(ref)));
+  auto callback = BindOnce(RefCheckFn, UnsafeDangling(raw_ref<int>(ref)));
   Free(p);
   EXPECT_EQ(std::move(callback).Run(), true);
   // Should reach this point without crashing; there is a dangling pointer, but
@@ -2050,8 +2093,8 @@ TEST_F(BindUnretainedDanglingTest, UnretainedRefUnsafeDangling) {
 TEST_F(BindUnretainedDanglingTest, UnretainedRefUnsafeDanglingUntriaged) {
   raw_ptr<int> p = Alloc<int>(3);
   int& ref = *p;
-  auto callback = base::BindOnce(
-      RefCheckFn, base::UnsafeDanglingUntriaged(base::raw_ref<const int>(ref)));
+  auto callback =
+      BindOnce(RefCheckFn, UnsafeDanglingUntriaged(raw_ref<const int>(ref)));
   Free(p);
   EXPECT_EQ(std::move(callback).Run(), true);
   // Should reach this point without crashing; there is a dangling pointer, but
@@ -2067,7 +2110,7 @@ int FuncWithRefArgument(int& i_ptr) {
 
 TEST_F(BindUnretainedDanglingDeathTest, UnretainedDanglingPtr) {
   raw_ptr<int> p = Alloc<int>(3);
-  auto callback = base::BindOnce(PingPong, base::Unretained(p));
+  auto callback = BindOnce(PingPong, Unretained(p));
   Free(p);
   EXPECT_DEATH(std::move(callback).Run(), "");
 }
@@ -2075,7 +2118,7 @@ TEST_F(BindUnretainedDanglingDeathTest, UnretainedDanglingPtr) {
 TEST_F(BindUnretainedDanglingDeathTest, UnretainedRefDanglingPtr) {
   raw_ptr<int> p = Alloc<int>(3);
   int& ref = *p;
-  auto callback = base::BindOnce(FuncWithRefArgument, std::ref(ref));
+  auto callback = BindOnce(FuncWithRefArgument, std::ref(ref));
   Free(p);
   EXPECT_DEATH(std::move(callback).Run(), "");
 }
@@ -2084,8 +2127,7 @@ TEST_F(BindUnretainedDanglingDeathTest,
        UnretainedRefWithManualUnretainedDanglingPtr) {
   raw_ptr<int> p = Alloc<int>(3);
   int& ref = *p;
-  auto callback = base::BindOnce(FuncWithRefArgument,
-                                 base::Unretained(base::raw_ref<int>(ref)));
+  auto callback = BindOnce(FuncWithRefArgument, Unretained(raw_ref<int>(ref)));
   Free(p);
   EXPECT_DEATH(std::move(callback).Run(), "");
 }
@@ -2093,15 +2135,16 @@ TEST_F(BindUnretainedDanglingDeathTest,
 TEST_F(BindUnretainedDanglingDeathTest, UnretainedWeakReceiverDangling) {
   raw_ptr<int> p = Alloc<int>(3);
   std::unique_ptr<ClassWithWeakPtr> r = std::make_unique<ClassWithWeakPtr>();
-  auto callback = base::BindOnce(&ClassWithWeakPtr::RawPtrArg, r->GetWeakPtr(),
-                                 base::Unretained(p));
+  auto callback =
+      BindOnce(&ClassWithWeakPtr::RawPtrArg, r->GetWeakPtr(), Unretained(p));
   Free(p);
   EXPECT_DEATH(std::move(callback).Run(), "");
 }
 
 #endif  // defined(GTEST_HAS_DEATH_TEST) && !BUILDFLAG(IS_ANDROID)
 
-#endif  // BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#endif  // PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT) &&
+        // PA_BUILDFLAG(USE_RAW_PTR_BACKUP_REF_IMPL)
 
 }  // namespace
 }  // namespace base
