@@ -1,13 +1,18 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/i18n/icu_util.h"
 
-#if defined(OS_WIN)
+#include "build/build_config.h"
+
+#if BUILDFLAG(IS_WIN)
 #include <windows.h>
 #endif
 
+#include <string.h>
+
+#include <memory>
 #include <string>
 
 #include "base/debug/alias.h"
@@ -20,43 +25,41 @@
 #include "base/metrics/metrics_hashes.h"
 #include "base/path_service.h"
 #include "base/strings/string_util.h"
-#include "base/strings/sys_string_conversions.h"
-#include "build/build_config.h"
 #include "build/chromecast_buildflags.h"
 #include "third_party/icu/source/common/unicode/putil.h"
+#include "third_party/icu/source/common/unicode/uclean.h"
 #include "third_party/icu/source/common/unicode/udata.h"
 #include "third_party/icu/source/common/unicode/utrace.h"
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 #include "base/android/apk_assets.h"
 #include "base/android/timezone_utils.h"
 #endif
 
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
 #include "base/ios/ios_util.h"
 #endif
 
-#if defined(OS_APPLE)
-#include "base/mac/foundation_util.h"
+#if BUILDFLAG(IS_APPLE)
+#include "base/apple/foundation_util.h"
 #endif
 
-#if defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_FUCHSIA)
 #include "base/fuchsia/intl_profile_watcher.h"
 #endif
 
-#if defined(OS_ANDROID) || defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_FUCHSIA)
 #include "third_party/icu/source/common/unicode/unistr.h"
 #endif
 
-#if defined(OS_ANDROID) || defined(OS_FUCHSIA) || \
-    ((defined(OS_LINUX) || defined(OS_CHROMEOS)) && !BUILDFLAG(IS_CHROMECAST))
+#if BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_FUCHSIA) || \
+    BUILDFLAG(IS_CHROMEOS) || (BUILDFLAG(IS_LINUX) && !BUILDFLAG(IS_CASTOS))
 #include "third_party/icu/source/i18n/unicode/timezone.h"
 #endif
 
-namespace base {
-namespace i18n {
+namespace base::i18n {
 
-#if !defined(OS_NACL)
+#if !BUILDFLAG(IS_NACL)
 namespace {
 
 #if DCHECK_IS_ON()
@@ -74,20 +77,19 @@ int g_debug_icu_last_error;
 int g_debug_icu_load;
 int g_debug_icu_pf_error_details;
 int g_debug_icu_pf_last_error;
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 wchar_t g_debug_icu_pf_filename[_MAX_PATH];
-#endif  // OS_WIN
+#endif  // BUILDFLAG(IS_WIN)
 // Use an unversioned file name to simplify a icu version update down the road.
 // No need to change the filename in multiple places (gyp files, windows
 // build pkg configurations, etc). 'l' stands for Little Endian.
 // This variable is exported through the header file.
 const char kIcuDataFileName[] = "icudtl.dat";
-const char kIcuExtraDataFileName[] = "icudtl_extra.dat";
 
 // Time zone data loading.
 // For now, only Fuchsia has a meaningful use case for this feature, so it is
 // only implemented for OS_FUCHSIA.
-#if defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_FUCHSIA)
 // The environment variable used to point the ICU data loader to the directory
 // containing time zone data. This is available from ICU version 54. The env
 // variable approach is antiquated by today's standards (2019), but is the
@@ -96,19 +98,23 @@ const char kIcuExtraDataFileName[] = "icudtl_extra.dat";
 // See for details: http://userguide.icu-project.org/datetime/timezone
 const char kIcuTimeZoneEnvVariable[] = "ICU_TIMEZONE_FILES_DIR";
 
-// We assume that Fuchsia will provide time zone data at this path for Chromium
-// to load, and that the path will be timely updated when Fuchsia needs to
-// uprev the ICU version it is using. There are unit tests that will fail at
-// Fuchsia roll time in case either Chromium or Fuchsia get upgraded to
-// mutually incompatible ICU versions. That should be enough to alert the
-// developers of the need to keep ICU library versions in ICU and Fuchsia in
-// reasonable sync.
-const char kIcuTimeZoneDataDir[] = "/config/data/tzdata/icu/44/le";
-#endif  // defined(OS_FUCHSIA)
+// Up-to-date time zone data MUST be provided by the system as a
+// directory offered to Chromium components at /config/tzdata.  Chromium
+// components "use" the `tzdata` directory capability, specifying the
+// "/config/tzdata" path. Chromium components will crash if this capability
+// is not available.
+//
+// TimeZoneDataTest.* tests verify that external timezone data is correctly
+// loaded from the system, to alert developers if the platform and Chromium
+// versions are no longer compatible versions.
+// LINT.IfChange(icu_time_zone_data_path)
+const char kIcuTimeZoneDataDir[] = "/config/tzdata/icu/44/le";
+// LINT.ThenChange(//sandbox/policy.fuchsia/sandbox_policy_fuchsia.cc:icu_time_zone_data_path)
+#endif  // BUILDFLAG(IS_FUCHSIA)
 
-#if defined(OS_ANDROID)
-const char kAssetsPathPrefix[] = "assets/";
-#endif  // defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
+const char kAndroidAssetsIcuDataFileName[] = "assets/icudtl.dat";
+#endif  // BUILDFLAG(IS_ANDROID)
 
 // File handle intentionally never closed. Not using File here because its
 // Windows implementation guards against two instances owning the same
@@ -116,124 +122,99 @@ const char kAssetsPathPrefix[] = "assets/";
 PlatformFile g_icudtl_pf = kInvalidPlatformFile;
 MemoryMappedFile* g_icudtl_mapped_file = nullptr;
 MemoryMappedFile::Region g_icudtl_region;
-PlatformFile g_icudtl_extra_pf = kInvalidPlatformFile;
-MemoryMappedFile* g_icudtl_extra_mapped_file = nullptr;
-MemoryMappedFile::Region g_icudtl_extra_region;
 
-#if defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_FUCHSIA)
 // The directory from which the ICU data loader will be configured to load time
 // zone data. It is only changed by SetIcuTimeZoneDataDirForTesting().
 const char* g_icu_time_zone_data_dir = kIcuTimeZoneDataDir;
-#endif  // defined(OS_FUCHSIA)
+#endif  // BUILDFLAG(IS_FUCHSIA)
 
-struct PfRegion {
- public:
-  PlatformFile pf;
-  MemoryMappedFile::Region region;
-};
-
-std::unique_ptr<PfRegion> OpenIcuDataFile(const std::string& filename,
-                                          const std::string& split_name) {
-  auto result = std::make_unique<PfRegion>();
-#if defined(OS_ANDROID)
-  result->pf = android::OpenApkAsset(kAssetsPathPrefix + filename, split_name,
-                                     &result->region);
-  if (result->pf != -1) {
-    return result;
+void LazyInitIcuDataFile() {
+  if (g_icudtl_pf != kInvalidPlatformFile) {
+    return;
   }
-#endif  // defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
+  int fd =
+      android::OpenApkAsset(kAndroidAssetsIcuDataFileName, &g_icudtl_region);
+  g_icudtl_pf = fd;
+  if (fd != -1) {
+    return;
+  }
+#endif  // BUILDFLAG(IS_ANDROID)
 #if defined(TOOLKIT_QT)
   FilePath data_path;
   PathService::Get(base::DIR_QT_LIBRARY_DATA, &data_path);
   data_path = data_path.AppendASCII(kIcuDataFileName);
-#elif !defined(OS_APPLE)
+#elif !BUILDFLAG(IS_APPLE)
   // For unit tests, data file is located on disk, so try there as a fallback.
   FilePath data_path;
   if (!PathService::Get(DIR_ASSETS, &data_path)) {
-    LOG(ERROR) << "Can't find " << filename;
-    return nullptr;
+    LOG(ERROR) << "Can't find " << kIcuDataFileName;
+    return;
   }
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   // TODO(brucedawson): http://crbug.com/445616
   wchar_t tmp_buffer[_MAX_PATH] = {0};
   wcscpy_s(tmp_buffer, data_path.value().c_str());
   debug::Alias(tmp_buffer);
 #endif
-  data_path = data_path.AppendASCII(filename);
+  data_path = data_path.AppendASCII(kIcuDataFileName);
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   // TODO(brucedawson): http://crbug.com/445616
   wchar_t tmp_buffer2[_MAX_PATH] = {0};
   wcscpy_s(tmp_buffer2, data_path.value().c_str());
   debug::Alias(tmp_buffer2);
 #endif
 
-#else  // !defined(OS_APPLE)
+#else  // !BUILDFLAG(IS_APPLE)
   // Assume it is in the framework bundle's Resources directory.
-  ScopedCFTypeRef<CFStringRef> data_file_name(SysUTF8ToCFStringRef(filename));
-  FilePath data_path = mac::PathForFrameworkBundleResource(data_file_name);
-#if defined(OS_IOS)
+  FilePath data_path = apple::PathForFrameworkBundleResource(kIcuDataFileName);
+#if BUILDFLAG(IS_IOS)
   FilePath override_data_path = ios::FilePathOfEmbeddedICU();
   if (!override_data_path.empty()) {
     data_path = override_data_path;
   }
-#endif  // !defined(OS_IOS)
+#endif  // !BUILDFLAG(IS_IOS)
   if (data_path.empty()) {
-    LOG(ERROR) << filename << " not found in bundle";
-    return nullptr;
+    LOG(ERROR) << kIcuDataFileName << " not found in bundle";
+    return;
   }
-#endif  // !defined(OS_APPLE)
+#endif  // !BUILDFLAG(IS_APPLE)
   File file(data_path, File::FLAG_OPEN | File::FLAG_READ);
   if (file.IsValid()) {
     // TODO(brucedawson): http://crbug.com/445616.
     g_debug_icu_pf_last_error = 0;
     g_debug_icu_pf_error_details = 0;
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
     g_debug_icu_pf_filename[0] = 0;
-#endif  // OS_WIN
+#endif  // BUILDFLAG(IS_WIN)
 
-    result->pf = file.TakePlatformFile();
-    result->region = MemoryMappedFile::Region::kWholeFile;
+    g_icudtl_pf = file.TakePlatformFile();
+    g_icudtl_region = MemoryMappedFile::Region::kWholeFile;
   }
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   else {
     // TODO(brucedawson): http://crbug.com/445616.
     g_debug_icu_pf_last_error = ::GetLastError();
     g_debug_icu_pf_error_details = file.error_details();
     wcscpy_s(g_debug_icu_pf_filename, data_path.value().c_str());
   }
-#endif  // OS_WIN
-
-  return result;
-}
-
-void LazyOpenIcuDataFile() {
-  if (g_icudtl_pf != kInvalidPlatformFile) {
-    return;
-  }
-  auto pf_region = OpenIcuDataFile(kIcuDataFileName, std::string());
-  if (!pf_region) {
-    return;
-  }
-  g_icudtl_pf = pf_region->pf;
-  g_icudtl_region = pf_region->region;
+#endif  // BUILDFLAG(IS_WIN)
 }
 
 // Configures ICU to load external time zone data, if appropriate.
 void InitializeExternalTimeZoneData() {
-#if defined(OS_FUCHSIA)
-  if (!base::DirectoryExists(base::FilePath(g_icu_time_zone_data_dir))) {
-    // TODO(https://crbug.com/1061262): Make this FATAL unless expected.
-    PLOG(WARNING) << "Could not open: '" << g_icu_time_zone_data_dir
-                  << "'. Using built-in timezone database";
-    return;
-  }
-
+#if BUILDFLAG(IS_FUCHSIA)
   // Set the environment variable to override the location used by ICU.
   // Loading can still fail if the directory is empty or its data is invalid.
   std::unique_ptr<base::Environment> env = base::Environment::Create();
+  if (!base::DirectoryExists(base::FilePath(g_icu_time_zone_data_dir))) {
+    PLOG(FATAL) << "Could not open directory: '" << g_icu_time_zone_data_dir
+                << "'";
+  }
   env->SetVar(kIcuTimeZoneEnvVariable, g_icu_time_zone_data_dir);
-#endif  // defined(OS_FUCHSIA)
+#endif  // BUILDFLAG(IS_FUCHSIA)
 }
 
 int LoadIcuData(PlatformFile data_fd,
@@ -247,7 +228,7 @@ int LoadIcuData(PlatformFile data_fd,
     return 1;  // To debug http://crbug.com/445616.
   }
 
-  out_mapped_data_file->reset(new MemoryMappedFile());
+  *out_mapped_data_file = std::make_unique<MemoryMappedFile>();
   if (!(*out_mapped_data_file)->Initialize(File(data_fd), data_region)) {
     LOG(ERROR) << "Couldn't mmap icu data file";
     return 2;  // To debug http://crbug.com/445616.
@@ -296,15 +277,15 @@ bool InitializeICUFromDataFile() {
   // it is needed.  This can fail if the process is sandboxed at that time.
   // Instead, we map the file in and hand off the data so the sandbox won't
   // cause any problems.
-  LazyOpenIcuDataFile();
+  LazyInitIcuDataFile();
   bool result =
       InitializeICUWithFileDescriptorInternal(g_icudtl_pf, g_icudtl_region);
 
-#if defined(OS_WIN)
   int debug_icu_load = g_debug_icu_load;
   debug::Alias(&debug_icu_load);
   int debug_icu_last_error = g_debug_icu_last_error;
   debug::Alias(&debug_icu_last_error);
+#if BUILDFLAG(IS_WIN)
   int debug_icu_pf_last_error = g_debug_icu_pf_last_error;
   debug::Alias(&debug_icu_pf_last_error);
   int debug_icu_pf_error_details = g_debug_icu_pf_error_details;
@@ -312,8 +293,13 @@ bool InitializeICUFromDataFile() {
   wchar_t debug_icu_pf_filename[_MAX_PATH] = {0};
   wcscpy_s(debug_icu_pf_filename, g_debug_icu_pf_filename);
   debug::Alias(&debug_icu_pf_filename);
-  CHECK(result);  // TODO(brucedawson): http://crbug.com/445616
-#endif            // defined(OS_WIN)
+#endif            // BUILDFLAG(IS_WIN)
+  // Excluding Chrome OS from this CHECK due to b/289684640.
+#if !BUILDFLAG(IS_CHROMEOS)
+  // https://crbug.com/445616
+  // https://crbug.com/1449816
+  CHECK(result);
+#endif
 
   return result;
 }
@@ -323,17 +309,17 @@ bool InitializeICUFromDataFile() {
 // On some platforms, the time zone must be explicitly initialized zone rather
 // than relying on ICU's internal initialization.
 void InitializeIcuTimeZone() {
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   // On Android, we can't leave it up to ICU to set the default time zone
   // because ICU's time zone detection does not work in many time zones (e.g.
   // Australia/Sydney, Asia/Seoul, Europe/Paris ). Use JNI to detect the host
   // time zone and set the ICU default time zone accordingly in advance of
   // actual use. See crbug.com/722821 and
   // https://ssl.icu-project.org/trac/ticket/13208 .
-  string16 zone_id = android::GetDefaultTimeZoneId();
+  std::u16string zone_id = android::GetDefaultTimeZoneId();
   icu::TimeZone::adoptDefault(icu::TimeZone::createTimeZone(
       icu::UnicodeString(false, zone_id.data(), zone_id.length())));
-#elif defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_FUCHSIA)
   // The platform-specific mechanisms used by ICU's detectHostTimeZone() to
   // determine the default time zone will not work on Fuchsia. Therefore,
   // proactively set the default system.
@@ -346,17 +332,14 @@ void InitializeIcuTimeZone() {
       FuchsiaIntlProfileWatcher::GetPrimaryTimeZoneIdForIcuInitialization();
   icu::TimeZone::adoptDefault(
       icu::TimeZone::createTimeZone(icu::UnicodeString::fromUTF8(zone_id)));
-#elif (defined(OS_LINUX) || defined(OS_CHROMEOS)) && !BUILDFLAG(IS_CHROMECAST)
+#elif BUILDFLAG(IS_CHROMEOS) || (BUILDFLAG(IS_LINUX) && !BUILDFLAG(IS_CASTOS))
   // To respond to the time zone change properly, the default time zone
   // cache in ICU has to be populated on starting up.
   // See TimeZoneMonitorLinux::NotifyClientsFromImpl().
   std::unique_ptr<icu::TimeZone> zone(icu::TimeZone::createDefault());
-#endif  // defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_ANDROID)
 }
 
-const char kICUDataFile[] = "ICU.DataFile";
-const char kICUCreateInstance[] = "ICU.CreateInstance";
-
 enum class ICUCreateInstance {
   kCharacterBreakIterator = 0,
   kWordBreakIterator = 1,
@@ -375,121 +358,6 @@ enum class ICUCreateInstance {
   kMaxValue = kChineseJapaneseBreakEngine
 };
 
-// Callback functions to report the opening of ICU Data File, and creation of
-// key objects to UMA. This help us to understand what built-in ICU data files
-// are rarely used in the user's machines and the distribution of ICU usage.
-static void U_CALLCONV TraceICUEntry(const void*, int32_t fn_number) {
-#if !defined(USING_SYSTEM_ICU) || U_ICU_VERSION_MAJOR_NUM >= 67
-  switch (fn_number) {
-    case UTRACE_UBRK_CREATE_CHARACTER:
-      base::UmaHistogramEnumeration(kICUCreateInstance,
-                                    ICUCreateInstance::kCharacterBreakIterator);
-      break;
-    case UTRACE_UBRK_CREATE_SENTENCE:
-      base::UmaHistogramEnumeration(kICUCreateInstance,
-                                    ICUCreateInstance::kSentenceBreakIterator);
-      break;
-    case UTRACE_UBRK_CREATE_TITLE:
-      base::UmaHistogramEnumeration(kICUCreateInstance,
-                                    ICUCreateInstance::kTitleBreakIterator);
-      break;
-    case UTRACE_UBRK_CREATE_WORD:
-      base::UmaHistogramEnumeration(kICUCreateInstance,
-                                    ICUCreateInstance::kWordBreakIterator);
-      break;
-    default:
-      return;
-  }
-#endif
-}
-
-static void U_CALLCONV TraceICUData(const void* context,
-                                    int32_t fn_number,
-                                    int32_t level,
-                                    const char* fmt,
-                                    va_list args) {
-  switch (fn_number) {
-    case UTRACE_UDATA_DATA_FILE: {
-      std::string icu_data_file_name(va_arg(args, const char*));
-      va_end(args);
-      // Skip icu version specified prefix if exist.
-      // path is prefixed with icu version prefix such as "icudt65l-".
-      // Histogram only the part after the -.
-      if (icu_data_file_name.find("icudt") == 0) {
-        size_t dash = icu_data_file_name.find("-");
-        if (dash != std::string::npos) {
-          icu_data_file_name = icu_data_file_name.substr(dash + 1);
-        }
-      }
-      // UmaHistogramSparse should track less than 100 values.
-      // We currently have about total 55 built-in data files inside ICU
-      // so it fit the UmaHistogramSparse usage.
-      int hash = base::HashMetricName(icu_data_file_name);
-      base::UmaHistogramSparse(kICUDataFile, hash);
-      return;
-    }
-#if !defined(USING_SYSTEM_ICU) || U_ICU_VERSION_MAJOR_NUM >= 67
-    case UTRACE_UBRK_CREATE_LINE: {
-      const char* lb_type = va_arg(args, const char*);
-      va_end(args);
-      ICUCreateInstance value;
-      switch (lb_type[0]) {
-        case '\0':
-          value = ICUCreateInstance::kLineBreakIterator;
-          break;
-        case 'l':
-          DCHECK(strcmp(lb_type, "loose") == 0);
-          value = ICUCreateInstance::kLineBreakIteratorTypeLoose;
-          break;
-        case 'n':
-          DCHECK(strcmp(lb_type, "normal") == 0);
-          value = ICUCreateInstance::kLineBreakIteratorTypeNormal;
-          break;
-        case 's':
-          DCHECK(strcmp(lb_type, "strict") == 0);
-          value = ICUCreateInstance::kLineBreakIteratorTypeStrict;
-          break;
-        default:
-          return;
-      }
-      base::UmaHistogramEnumeration(kICUCreateInstance, value);
-      return;
-    }
-    case UTRACE_UBRK_CREATE_BREAK_ENGINE: {
-      const char* script = va_arg(args, const char*);
-      va_end(args);
-      ICUCreateInstance value;
-      switch (script[0]) {
-        case 'H':
-          DCHECK(strcmp(script, "Hani") == 0);
-          value = ICUCreateInstance::kChineseJapaneseBreakEngine;
-          break;
-        case 'K':
-          DCHECK(strcmp(script, "Khmr") == 0);
-          value = ICUCreateInstance::kKhmerBreakEngine;
-          break;
-        case 'L':
-          DCHECK(strcmp(script, "Laoo") == 0);
-          value = ICUCreateInstance::kLaoBreakEngine;
-          break;
-        case 'M':
-          DCHECK(strcmp(script, "Mymr") == 0);
-          value = ICUCreateInstance::kBurmeseBreakEngine;
-          break;
-        case 'T':
-          DCHECK(strcmp(script, "Thai") == 0);
-          value = ICUCreateInstance::kThaiBreakEngine;
-          break;
-        default:
-          return;
-      }
-      base::UmaHistogramEnumeration(kICUCreateInstance, value);
-      return;
-    }
-#endif
-  }
-}
-
 // Common initialization to run regardless of how ICU is initialized.
 // There are multiple exposed InitializeIcu* functions. This should be called
 // as at the end of (the last functions in the sequence of) these functions.
@@ -499,8 +367,6 @@ bool DoCommonInitialization() {
   // when requested.
   InitializeIcuTimeZone();
 
-  const void* context = nullptr;
-  utrace_setFunctions(context, TraceICUEntry, nullptr, TraceICUData);
   utrace_setLevel(UTRACE_VERBOSE);
   return true;
 }
@@ -508,23 +374,6 @@ bool DoCommonInitialization() {
 }  // namespace
 
 #if (ICU_UTIL_DATA_IMPL == ICU_UTIL_DATA_FILE)
-bool InitializeExtraICUWithFileDescriptor(
-    PlatformFile data_fd,
-    const MemoryMappedFile::Region& data_region) {
-  if (g_icudtl_pf != kInvalidPlatformFile) {
-    // Must call InitializeExtraICUWithFileDescriptor() before
-    // InitializeICUWithFileDescriptor().
-    return false;
-  }
-  std::unique_ptr<MemoryMappedFile> mapped_file;
-  UErrorCode err;
-  if (LoadIcuData(data_fd, data_region, &mapped_file, &err) != 0) {
-    return false;
-  }
-  g_icudtl_extra_mapped_file = mapped_file.release();
-  return true;
-}
-
 bool InitializeICUWithFileDescriptor(
     PlatformFile data_fd,
     const MemoryMappedFile::Region& data_region) {
@@ -544,51 +393,27 @@ PlatformFile GetIcuDataFileHandle(MemoryMappedFile::Region* out_region) {
   return g_icudtl_pf;
 }
 
-PlatformFile GetIcuExtraDataFileHandle(MemoryMappedFile::Region* out_region) {
-  if (g_icudtl_extra_pf == kInvalidPlatformFile) {
-    return kInvalidPlatformFile;
-  }
-  *out_region = g_icudtl_extra_region;
-  return g_icudtl_extra_pf;
-}
-
-bool InitializeExtraICU(const std::string& split_name) {
-  if (g_icudtl_pf != kInvalidPlatformFile) {
-    // Must call InitializeExtraICU() before InitializeICU().
-    return false;
-  }
-  auto pf_region = OpenIcuDataFile(kIcuExtraDataFileName, split_name);
-  if (!pf_region) {
-    return false;
-  }
-  g_icudtl_extra_pf = pf_region->pf;
-  g_icudtl_extra_region = pf_region->region;
-  std::unique_ptr<MemoryMappedFile> mapped_file;
-  UErrorCode err;
-  if (LoadIcuData(g_icudtl_extra_pf, g_icudtl_extra_region, &mapped_file,
-                  &err) != 0) {
-    return false;
-  }
-  g_icudtl_extra_mapped_file = mapped_file.release();
-  return true;
-}
-
 void ResetGlobalsForTesting() {
+  // Reset ICU library internal state before tearing-down the mapped data
+  // file, or handle.
+  u_cleanup();
+
+  // `g_icudtl_pf` does not actually own the FD once ICU is initialized, so
+  // don't try to close it here.
   g_icudtl_pf = kInvalidPlatformFile;
-  g_icudtl_mapped_file = nullptr;
-  g_icudtl_extra_pf = kInvalidPlatformFile;
-  g_icudtl_extra_mapped_file = nullptr;
-#if defined(OS_FUCHSIA)
+  delete std::exchange(g_icudtl_mapped_file, nullptr);
+
+#if BUILDFLAG(IS_FUCHSIA)
   g_icu_time_zone_data_dir = kIcuTimeZoneDataDir;
-#endif  // defined(OS_FUCHSIA)
+#endif  // BUILDFLAG(IS_FUCHSIA)
 }
 
-#if defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_FUCHSIA)
 // |dir| must remain valid until ResetGlobalsForTesting() is called.
 void SetIcuTimeZoneDataDirForTesting(const char* dir) {
   g_icu_time_zone_data_dir = dir;
 }
-#endif  // defined(OS_FUCHSIA)
+#endif  // BUILDFLAG(IS_FUCHSIA)
 #endif  // (ICU_UTIL_DATA_IMPL == ICU_UTIL_DATA_FILE)
 
 bool InitializeICU() {
@@ -615,7 +440,6 @@ void AllowMultipleInitializeCallsForTesting() {
 #endif
 }
 
-#endif  // !defined(OS_NACL)
+#endif  // !BUILDFLAG(IS_NACL)
 
-}  // namespace i18n
-}  // namespace base
+}  // namespace base::i18n
