@@ -8,12 +8,11 @@
 
 #include <memory>
 #include <string>
+#include <string_view>
 
 #include "base/i18n/unicodestring.h"
-#include "base/logging.h"
 #include "base/notreached.h"
 #include "base/numerics/safe_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/time/time.h"
 #include "build/chromeos_buildflags.h"
@@ -31,7 +30,7 @@ namespace base {
 namespace {
 
 UDate ToUDate(const Time& time) {
-  // TODO(crbug.com/1392437): Consider using the `...IgnoringNull` variant and
+  // TODO(crbug.com/40247732): Consider using the `...IgnoringNull` variant and
   // adding a `CHECK(!time.is_null())`; trying to format a null Time as a string
   // is almost certainly an indication that the caller has made a mistake.
   return time.InMillisecondsFSinceUnixEpoch();
@@ -63,7 +62,7 @@ std::u16string TimeFormatWithoutAmPm(const icu::DateFormat* formatter,
 }
 
 icu::SimpleDateFormat CreateSimpleDateFormatter(
-    StringPiece pattern,
+    std::string_view pattern,
     bool generate_pattern = true,
     const icu::Locale& locale = icu::Locale::getDefault()) {
   UErrorCode status = U_ZERO_ERROR;
@@ -96,7 +95,6 @@ UMeasureFormatWidth DurationWidthToMeasureWidth(DurationFormatWidth width) {
     case DURATION_WIDTH_NUMERIC: return UMEASFMT_WIDTH_NUMERIC;
   }
   NOTREACHED();
-  return UMEASFMT_WIDTH_COUNT;
 }
 
 const char* DateFormatToString(DateFormat format) {
@@ -107,7 +105,6 @@ const char* DateFormatToString(DateFormat format) {
       return UDAT_MONTH_WEEKDAY_DAY;
   }
   NOTREACHED();
-  return UDAT_YEAR_MONTH_DAY;
 }
 
 }  // namespace
@@ -197,12 +194,12 @@ std::u16string TimeFormatFriendlyDate(const Time& time) {
 }
 
 std::u16string LocalizedTimeFormatWithPattern(const Time& time,
-                                              StringPiece pattern) {
-  return TimeFormat(CreateSimpleDateFormatter(std::move(pattern)), time);
+                                              std::string_view pattern) {
+  return TimeFormat(CreateSimpleDateFormatter(pattern), time);
 }
 
 std::string UnlocalizedTimeFormatWithPattern(const Time& time,
-                                             StringPiece pattern,
+                                             std::string_view pattern,
                                              const icu::TimeZone* time_zone) {
   icu::SimpleDateFormat formatter =
       CreateSimpleDateFormatter({}, false, icu::Locale("en_US"));
@@ -211,7 +208,8 @@ std::string UnlocalizedTimeFormatWithPattern(const Time& time,
   }
 
   // Formats `time` according to `pattern`.
-  const auto format_time = [&formatter](const Time& time, StringPiece pattern) {
+  const auto format_time = [&formatter](const Time& time,
+                                        std::string_view pattern) {
     formatter.applyPattern(
         icu::UnicodeString(pattern.data(), pattern.length()));
     return base::UTF16ToUTF8(TimeFormat(formatter, time));
@@ -226,7 +224,7 @@ std::string UnlocalizedTimeFormatWithPattern(const Time& time,
           Time::kMicrosecondsPerMillisecond) {
     // Adds digits to `output` for each 'S' at the start of `pattern`.
     const auto format_microseconds = [&output](int64_t mutable_micros,
-                                               StringPiece pattern) {
+                                               std::string_view pattern) {
       size_t i = 0;
       for (; i < pattern.length() && pattern[i] == 'S'; ++i) {
         output += static_cast<char>('0' + mutable_micros / 100);
@@ -261,7 +259,7 @@ std::string UnlocalizedTimeFormatWithPattern(const Time& time,
 
   // Format any remaining pattern.
   if (!pattern.empty()) {
-    output += format_time(time, std::move(pattern));
+    output += format_time(time, pattern);
   }
   return output;
 }
@@ -286,35 +284,15 @@ bool TimeDurationFormat(TimeDelta time,
   const int minutes = total_minutes % 60;
   UMeasureFormatWidth u_width = DurationWidthToMeasureWidth(width);
 
-  // TODO(derat): Delete the |status| checks and LOG(ERROR) calls throughout
-  // this function once the cause of http://crbug.com/677043 is tracked down.
   const icu::Measure measures[] = {
       icu::Measure(hours, icu::MeasureUnit::createHour(status), status),
       icu::Measure(minutes, icu::MeasureUnit::createMinute(status), status)};
-  if (U_FAILURE(status)) {
-    LOG(ERROR) << "Creating MeasureUnit or Measure for " << hours << "h"
-               << minutes << "m failed: " << u_errorName(status);
-    return false;
-  }
-
   icu::MeasureFormat measure_format(icu::Locale::getDefault(), u_width, status);
-  if (U_FAILURE(status)) {
-    LOG(ERROR) << "Creating MeasureFormat for "
-               << icu::Locale::getDefault().getName()
-               << " failed: " << u_errorName(status);
-    return false;
-  }
-
   icu::UnicodeString formatted;
   icu::FieldPosition ignore(icu::FieldPosition::DONT_CARE);
   measure_format.formatMeasures(measures, 2, formatted, ignore, status);
-  if (U_FAILURE(status)) {
-    LOG(ERROR) << "formatMeasures failed: " << u_errorName(status);
-    return false;
-  }
-
   *out = i18n::UnicodeStringToString16(formatted);
-  return true;
+  return U_SUCCESS(status);
 }
 
 bool TimeDurationFormatWithSeconds(TimeDelta time,
@@ -342,6 +320,42 @@ bool TimeDurationFormatWithSeconds(TimeDelta time,
   return U_SUCCESS(status);
 }
 
+bool TimeDurationCompactFormatWithSeconds(TimeDelta time,
+                                          DurationFormatWidth width,
+                                          std::u16string* out) {
+  DCHECK(out);
+  UErrorCode status = U_ZERO_ERROR;
+  const int64_t total_seconds = ClampRound<int64_t>(time.InSecondsF());
+  const int64_t hours = total_seconds / base::Time::kSecondsPerHour;
+  const int64_t minutes =
+      (total_seconds - hours * base::Time::kSecondsPerHour) /
+      base::Time::kSecondsPerMinute;
+  const int64_t seconds = total_seconds % base::Time::kSecondsPerMinute;
+  UMeasureFormatWidth u_width = DurationWidthToMeasureWidth(width);
+  const icu::Measure hours_measure =
+      icu::Measure(hours, icu::MeasureUnit::createHour(status), status);
+  const icu::Measure minutes_measure =
+      icu::Measure(minutes, icu::MeasureUnit::createMinute(status), status);
+  const icu::Measure seconds_measure =
+      icu::Measure(seconds, icu::MeasureUnit::createSecond(status), status);
+  icu::MeasureFormat measure_format(icu::Locale::getDefault(), u_width, status);
+  icu::UnicodeString formatted;
+  icu::FieldPosition ignore(icu::FieldPosition::DONT_CARE);
+  if (hours != 0 || width == DurationFormatWidth::DURATION_WIDTH_NUMERIC) {
+    icu::Measure input_measures[3]{hours_measure, minutes_measure,
+                                   seconds_measure};
+    measure_format.formatMeasures(input_measures, 3, formatted, ignore, status);
+  } else if (minutes != 0) {
+    icu::Measure input_measures[2]{minutes_measure, seconds_measure};
+    measure_format.formatMeasures(input_measures, 2, formatted, ignore, status);
+  } else {
+    icu::Measure input_measures[1]{seconds_measure};
+    measure_format.formatMeasures(input_measures, 1, formatted, ignore, status);
+  }
+  *out = i18n::UnicodeStringToString16(formatted);
+  return U_SUCCESS(status);
+}
+
 std::u16string DateIntervalFormat(const Time& begin_time,
                                   const Time& end_time,
                                   DateFormat format) {
