@@ -2,10 +2,16 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/json/json_parser.h"
 
 #include <cmath>
 #include <iterator>
+#include <string_view>
 #include <utility>
 #include <vector>
 
@@ -18,13 +24,11 @@
 #include "base/numerics/safe_conversions.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversion_utils.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/third_party/icu/icu_utf.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace base {
 namespace internal {
@@ -58,10 +62,9 @@ std::string ErrorCodeToString(JSONParser::JsonParseError error_code) {
     case JSONParser::JSON_UNREPRESENTABLE_NUMBER:
       return JSONParser::kUnrepresentableNumber;
     case JSONParser::JSON_PARSE_ERROR_COUNT:
-      break;
+      NOTREACHED();
   }
   NOTREACHED();
-  return std::string();
 }
 
 const int32_t kExtendedASCIIStart = 0x80;
@@ -70,7 +73,7 @@ constexpr base_icu::UChar32 kUnicodeReplacementPoint = 0xFFFD;
 // UnprefixedHexStringToInt acts like |HexStringToInt|, but enforces that the
 // input consists purely of hex digits. I.e. no "0x" nor "OX" prefix is
 // permitted.
-bool UnprefixedHexStringToInt(StringPiece input, int* output) {
+bool UnprefixedHexStringToInt(std::string_view input, int* output) {
   for (size_t i = 0; i < input.size(); i++) {
     if (!IsHexDigit(input[i])) {
       return false;
@@ -87,7 +90,8 @@ enum class ChromiumJsonExtension {
   kXEscape,
   kVerticalTabEscape,
   kControlCharacter,
-  kMaxValue = kControlCharacter,
+  kNewlineInString,
+  kMaxValue = kNewlineInString,
 };
 
 const char kExtensionHistogramName[] =
@@ -127,7 +131,7 @@ JSONParser::JSONParser(int options, size_t max_depth)
 
 JSONParser::~JSONParser() = default;
 
-absl::optional<Value> JSONParser::Parse(StringPiece input) {
+std::optional<Value> JSONParser::Parse(std::string_view input) {
   input_ = input;
   index_ = 0;
   // Line and column counting is 1-based, but |index_| is 0-based. For example,
@@ -154,14 +158,14 @@ absl::optional<Value> JSONParser::Parse(StringPiece input) {
   ConsumeIfMatch("\xEF\xBB\xBF");
 
   // Parse the first and any nested tokens.
-  absl::optional<Value> root(ParseNextToken());
+  std::optional<Value> root(ParseNextToken());
   if (!root)
-    return absl::nullopt;
+    return std::nullopt;
 
   // Make sure the input stream is at an end.
   if (GetNextToken() != T_END_OF_INPUT) {
     ReportError(JSON_UNEXPECTED_DATA_AFTER_ROOT, 0);
-    return absl::nullopt;
+    return std::nullopt;
   }
 
   return root;
@@ -184,79 +188,44 @@ int JSONParser::error_column() const {
   return error_column_;
 }
 
-// StringBuilder ///////////////////////////////////////////////////////////////
-
-JSONParser::StringBuilder::StringBuilder() : StringBuilder(nullptr) {}
-
-JSONParser::StringBuilder::StringBuilder(const char* pos)
-    : pos_(pos), length_(0) {}
-
-JSONParser::StringBuilder::~StringBuilder() = default;
-
-JSONParser::StringBuilder& JSONParser::StringBuilder::operator=(
-    StringBuilder&& other) = default;
-
-void JSONParser::StringBuilder::Append(base_icu::UChar32 point) {
-  DCHECK(IsValidCodepoint(point));
-
-  if (point < kExtendedASCIIStart) {
-    if (!string_) {
-      DCHECK_EQ(static_cast<char>(point), pos_[length_]);
-      ++length_;
-    } else {
-      string_->push_back(static_cast<char>(point));
-    }
-  } else {
-    Convert();
-    if (UNLIKELY(point == kUnicodeReplacementPoint)) {
-      string_->append(kUnicodeReplacementString);
-    } else {
-      WriteUnicodeCharacter(point, &*string_);
-    }
-  }
-}
-
-void JSONParser::StringBuilder::Convert() {
-  if (string_)
-    return;
-  string_.emplace(pos_, length_);
-}
-
-std::string JSONParser::StringBuilder::DestructiveAsString() {
-  if (string_)
-    return std::move(*string_);
-  return std::string(pos_, length_);
-}
-
 // JSONParser private //////////////////////////////////////////////////////////
 
-absl::optional<StringPiece> JSONParser::PeekChars(size_t count) {
-  if (index_ + count > input_.length())
-    return absl::nullopt;
-  // Using StringPiece::substr() is significantly slower (according to
-  // base_perftests) than constructing a substring manually.
-  return StringPiece(input_.data() + index_, count);
+std::optional<std::string_view> JSONParser::PeekChars(size_t count) {
+  if (count > input_.length() - index_) {
+    return std::nullopt;
+  }
+  // Using string_view::substr() was historically significantly slower
+  // (according to base_perftests) than constructing a substring manually.
+  //
+  // TODO(crbug.com/40284755): Is this still the case? Ideally the bounds check
+  // performed by substr would be deleted by the optimizer for being redundant
+  // with the runtime check above. However, to do so, the compiler would need
+  // to know `index_ <= input_.length()` is a class invariant. If we
+  // restructured the code so that we only stored the remaining data, that
+  // would avoid this, but it would prevent rewinding (the places in this file
+  // which look at `input_[index_ - 1]`.)
+  return std::string_view(input_.data() + index_, count);
 }
 
-absl::optional<char> JSONParser::PeekChar() {
-  absl::optional<StringPiece> chars = PeekChars(1);
+std::optional<char> JSONParser::PeekChar() {
+  std::optional<std::string_view> chars = PeekChars(1);
   if (chars)
     return (*chars)[0];
-  return absl::nullopt;
+  return std::nullopt;
 }
 
-absl::optional<StringPiece> JSONParser::ConsumeChars(size_t count) {
-  absl::optional<StringPiece> chars = PeekChars(count);
+std::optional<std::string_view> JSONParser::ConsumeChars(size_t count) {
+  std::optional<std::string_view> chars = PeekChars(count);
   if (chars)
     index_ += count;
   return chars;
 }
 
-absl::optional<char> JSONParser::ConsumeChar() {
-  absl::optional<StringPiece> chars = ConsumeChars(1);
+std::optional<char> JSONParser::ConsumeChar() {
+  std::optional<std::string_view> chars = ConsumeChars(1);
   if (chars)
     return (*chars)[0];
-  return absl::nullopt;
+  return std::nullopt;
 }
 
 const char* JSONParser::pos() {
@@ -267,7 +236,7 @@ const char* JSONParser::pos() {
 JSONParser::Token JSONParser::GetNextToken() {
   EatWhitespaceAndComments();
 
-  absl::optional<char> c = PeekChar();
+  std::optional<char> c = PeekChar();
   if (!c)
     return T_END_OF_INPUT;
 
@@ -310,7 +279,7 @@ JSONParser::Token JSONParser::GetNextToken() {
 }
 
 void JSONParser::EatWhitespaceAndComments() {
-  while (absl::optional<char> c = PeekChar()) {
+  while (std::optional<char> c = PeekChar()) {
     switch (*c) {
       case '\r':
       case '\n':
@@ -335,7 +304,7 @@ void JSONParser::EatWhitespaceAndComments() {
 }
 
 bool JSONParser::EatComment() {
-  absl::optional<StringPiece> comment_start = PeekChars(2);
+  std::optional<std::string_view> comment_start = PeekChars(2);
   if (!comment_start)
     return false;
 
@@ -351,7 +320,7 @@ bool JSONParser::EatComment() {
 
     ConsumeChars(2);
     // Single line comment, read to newline.
-    while (absl::optional<char> c = PeekChar()) {
+    while (std::optional<char> c = PeekChar()) {
       if (c == '\n' || c == '\r')
         return true;
       ConsumeChar();
@@ -367,7 +336,7 @@ bool JSONParser::EatComment() {
     ConsumeChars(2);
     char previous_char = '\0';
     // Block comment, read until end marker.
-    while (absl::optional<char> c = PeekChar()) {
+    while (std::optional<char> c = PeekChar()) {
       if (previous_char == '*' && c == '/') {
         // EatWhitespaceAndComments will inspect pos(), which will still be on
         // the last / of the comment, so advance once more (which may also be
@@ -384,11 +353,11 @@ bool JSONParser::EatComment() {
   return false;
 }
 
-absl::optional<Value> JSONParser::ParseNextToken() {
+std::optional<Value> JSONParser::ParseNextToken() {
   return ParseToken(GetNextToken());
 }
 
-absl::optional<Value> JSONParser::ParseToken(Token token) {
+std::optional<Value> JSONParser::ParseToken(Token token) {
   switch (token) {
     case T_OBJECT_BEGIN:
       return ConsumeDictionary();
@@ -404,20 +373,20 @@ absl::optional<Value> JSONParser::ParseToken(Token token) {
       return ConsumeLiteral();
     default:
       ReportError(JSON_UNEXPECTED_TOKEN, 0);
-      return absl::nullopt;
+      return std::nullopt;
   }
 }
 
-absl::optional<Value> JSONParser::ConsumeDictionary() {
+std::optional<Value> JSONParser::ConsumeDictionary() {
   if (ConsumeChar() != '{') {
     ReportError(JSON_UNEXPECTED_TOKEN, 0);
-    return absl::nullopt;
+    return std::nullopt;
   }
 
   StackMarker depth_check(max_depth_, &stack_depth_);
   if (depth_check.IsTooDeep()) {
     ReportError(JSON_TOO_MUCH_NESTING, -1);
-    return absl::nullopt;
+    return std::nullopt;
   }
 
   std::vector<std::pair<std::string, Value>> values;
@@ -426,31 +395,31 @@ absl::optional<Value> JSONParser::ConsumeDictionary() {
   while (token != T_OBJECT_END) {
     if (token != T_STRING) {
       ReportError(JSON_UNQUOTED_DICTIONARY_KEY, 0);
-      return absl::nullopt;
+      return std::nullopt;
     }
 
     // First consume the key.
-    StringBuilder key;
-    if (!ConsumeStringRaw(&key)) {
-      return absl::nullopt;
+    std::optional<std::string> key = ConsumeStringRaw();
+    if (!key) {
+      return std::nullopt;
     }
 
     // Read the separator.
     token = GetNextToken();
     if (token != T_OBJECT_PAIR_SEPARATOR) {
       ReportError(JSON_SYNTAX_ERROR, 0);
-      return absl::nullopt;
+      return std::nullopt;
     }
 
     // The next token is the value. Ownership transfers to |dict|.
     ConsumeChar();
-    absl::optional<Value> value = ParseNextToken();
+    std::optional<Value> value = ParseNextToken();
     if (!value) {
       // ReportError from deeper level.
-      return absl::nullopt;
+      return std::nullopt;
     }
 
-    values.emplace_back(key.DestructiveAsString(), std::move(*value));
+    values.emplace_back(std::move(*key), std::move(*value));
 
     token = GetNextToken();
     if (token == T_LIST_SEPARATOR) {
@@ -458,11 +427,11 @@ absl::optional<Value> JSONParser::ConsumeDictionary() {
       token = GetNextToken();
       if (token == T_OBJECT_END && !(options_ & JSON_ALLOW_TRAILING_COMMAS)) {
         ReportError(JSON_TRAILING_COMMA, 0);
-        return absl::nullopt;
+        return std::nullopt;
       }
     } else if (token != T_OBJECT_END) {
       ReportError(JSON_SYNTAX_ERROR, 0);
-      return absl::nullopt;
+      return std::nullopt;
     }
   }
 
@@ -474,26 +443,26 @@ absl::optional<Value> JSONParser::ConsumeDictionary() {
                            std::make_move_iterator(values.end())));
 }
 
-absl::optional<Value> JSONParser::ConsumeList() {
+std::optional<Value> JSONParser::ConsumeList() {
   if (ConsumeChar() != '[') {
     ReportError(JSON_UNEXPECTED_TOKEN, 0);
-    return absl::nullopt;
+    return std::nullopt;
   }
 
   StackMarker depth_check(max_depth_, &stack_depth_);
   if (depth_check.IsTooDeep()) {
     ReportError(JSON_TOO_MUCH_NESTING, -1);
-    return absl::nullopt;
+    return std::nullopt;
   }
 
   Value::List list;
 
   Token token = GetNextToken();
   while (token != T_ARRAY_END) {
-    absl::optional<Value> item = ParseToken(token);
+    std::optional<Value> item = ParseToken(token);
     if (!item) {
       // ReportError from deeper level.
-      return absl::nullopt;
+      return std::nullopt;
     }
 
     list.Append(std::move(*item));
@@ -504,11 +473,11 @@ absl::optional<Value> JSONParser::ConsumeList() {
       token = GetNextToken();
       if (token == T_ARRAY_END && !(options_ & JSON_ALLOW_TRAILING_COMMAS)) {
         ReportError(JSON_TRAILING_COMMA, 0);
-        return absl::nullopt;
+        return std::nullopt;
       }
     } else if (token != T_ARRAY_END) {
       ReportError(JSON_SYNTAX_ERROR, 0);
-      return absl::nullopt;
+      return std::nullopt;
     }
   }
 
@@ -517,175 +486,217 @@ absl::optional<Value> JSONParser::ConsumeList() {
   return Value(std::move(list));
 }
 
-absl::optional<Value> JSONParser::ConsumeString() {
-  StringBuilder string;
-  if (!ConsumeStringRaw(&string))
-    return absl::nullopt;
-  return Value(string.DestructiveAsString());
+std::optional<Value> JSONParser::ConsumeString() {
+  std::optional<std::string> string = ConsumeStringRaw();
+  if (!string) {
+    return std::nullopt;
+  }
+  return Value(std::move(*string));
 }
 
-bool JSONParser::ConsumeStringRaw(StringBuilder* out) {
+std::optional<std::string> JSONParser::ConsumeStringRaw() {
   if (ConsumeChar() != '"') {
     ReportError(JSON_UNEXPECTED_TOKEN, 0);
-    return false;
+    return std::nullopt;
   }
 
-  // StringBuilder will internally build a StringPiece unless a UTF-16
-  // conversion occurs, at which point it will perform a copy into a
-  // std::string.
-  StringBuilder string(pos());
+  std::string string;
+  for (;;) {
+    auto [result, consumed] = ConsumeStringPart();
+    switch (result) {
+      case StringResult::kError:
+        return std::nullopt;
 
-  while (absl::optional<char> c = PeekChar()) {
-    base_icu::UChar32 next_char = 0;
-    if (static_cast<unsigned char>(*c) < kExtendedASCIIStart) {
-      // Fast path for ASCII.
-      next_char = *c;
-    } else if (!ReadUnicodeCharacter(input_.data(), input_.length(), &index_,
-                                     &next_char) ||
-               !IsValidCodepoint(next_char)) {
-      if ((options_ & JSON_REPLACE_INVALID_CHARACTERS) == 0) {
-        ReportError(JSON_UNSUPPORTED_ENCODING, 0);
-        return false;
+      case StringResult::kDone:
+        // This is the last time we're appending, so pre-reserve the desired
+        // size, to prevent `+=` from overallocating. (In other cases, the
+        // overallocating is desirable for amortization.) In particular,
+        // the common case is that `string` is empty and we return in one step.
+        string.reserve(string.size() + consumed.size());
+        string += consumed;
+        return std::move(string);
+
+      case StringResult::kReplacementCharacter:
+        string += consumed;
+        string += kUnicodeReplacementString;
+        break;  // Keep parsing.
+
+      case StringResult::kEscape:
+        string += consumed;
+        std::optional<char> escape_char = ConsumeChar();
+        if (!escape_char) {
+          ReportError(JSON_INVALID_ESCAPE, -1);
+          return std::nullopt;
+        }
+
+        switch (*escape_char) {
+          // Allowed esape sequences:
+          case 'x': {  // UTF-8 sequence.
+            // UTF-8 \x escape sequences are not allowed in the spec, but they
+            // are supported here for backwards-compatiblity with the old
+            // parser.
+            UmaHistogramEnumeration(kExtensionHistogramName,
+                                    ChromiumJsonExtension::kXEscape);
+            if (!(options_ & JSON_ALLOW_X_ESCAPES)) {
+              ReportError(JSON_INVALID_ESCAPE, -1);
+              return std::nullopt;
+            }
+
+            std::optional<std::string_view> escape_sequence = ConsumeChars(2);
+            if (!escape_sequence) {
+              ReportError(JSON_INVALID_ESCAPE, -3);
+              return std::nullopt;
+            }
+
+            int hex_digit = 0;
+            if (!UnprefixedHexStringToInt(*escape_sequence, &hex_digit)) {
+              ReportError(JSON_INVALID_ESCAPE, -3);
+              return std::nullopt;
+            }
+
+            // A two-character hex sequence is at most 0xff and all codepoints
+            // up to 0xff are valid.
+            DCHECK_LE(hex_digit, 0xff);
+            DCHECK(IsValidCharacter(hex_digit));
+            WriteUnicodeCharacter(hex_digit, &string);
+            break;
+          }
+          case 'u': {  // UTF-16 sequence.
+            // UTF units are of the form \uXXXX.
+            base_icu::UChar32 code_point;
+            if (!DecodeUTF16(&code_point)) {
+              ReportError(JSON_INVALID_ESCAPE, -1);
+              return std::nullopt;
+            }
+            WriteUnicodeCharacter(code_point, &string);
+            break;
+          }
+          case '"':
+            string.push_back('"');
+            break;
+          case '\\':
+            string.push_back('\\');
+            break;
+          case '/':
+            string.push_back('/');
+            break;
+          case 'b':
+            string.push_back('\b');
+            break;
+          case 'f':
+            string.push_back('\f');
+            break;
+          case 'n':
+            string.push_back('\n');
+            break;
+          case 'r':
+            string.push_back('\r');
+            break;
+          case 't':
+            string.push_back('\t');
+            break;
+          case 'v':  // Not listed as valid escape sequence in the RFC.
+            UmaHistogramEnumeration(kExtensionHistogramName,
+                                    ChromiumJsonExtension::kVerticalTabEscape);
+            if (!(options_ & JSON_ALLOW_VERT_TAB)) {
+              ReportError(JSON_INVALID_ESCAPE, -1);
+              return std::nullopt;
+            }
+            string.push_back('\v');
+            break;
+          // All other escape squences are illegal.
+          default:
+            ReportError(JSON_INVALID_ESCAPE, -1);
+            return std::nullopt;
+        }
+        break;  // Keep parsing.
+    }
+  }
+}
+
+std::pair<JSONParser::StringResult, std::string_view>
+JSONParser::ConsumeStringPart() {
+  const size_t start_index = index_;
+  while (std::optional<char> c = PeekChar()) {
+    // Handle non-ASCII characters, which never trigger any special handling
+    // beyond needing to be valid UTF-8. ASCII characters will be handled
+    // separately below.
+    if (static_cast<unsigned char>(*c) >= kExtendedASCIIStart) {
+      base_icu::UChar32 next_char = 0;
+      size_t last_index = index_;
+      if (!ReadUnicodeCharacter(input_.data(), input_.length(), &index_,
+                                &next_char)) {
+        if ((options_ & JSON_REPLACE_INVALID_CHARACTERS) == 0) {
+          ReportError(JSON_UNSUPPORTED_ENCODING, 0);
+          // No need to return consumed data.
+          return {StringResult::kError, {}};
+        }
+        ConsumeChar();
+        return {StringResult::kReplacementCharacter,
+                input_.substr(start_index, last_index - start_index)};
       }
+
+      // Valid UTF-8 will be copied as-is into the output, so keep processing.
+      DCHECK_GE(next_char, kExtendedASCIIStart);
       ConsumeChar();
-      string.Append(kUnicodeReplacementPoint);
       continue;
     }
 
-    if (next_char == '"') {
+    if (*c == '"') {
+      std::string_view ret = input_.substr(start_index, index_ - start_index);
       ConsumeChar();
-      *out = std::move(string);
-      return true;
+      return {StringResult::kDone, ret};
     }
-    if (next_char != '\\') {
-      // Per Section 7, "All Unicode characters may be placed within the
-      // quotation marks, except for the characters that MUST be escaped:
-      // quotation mark, reverse solidus, and the control characters (U+0000
-      // through U+001F)".
-      if (next_char <= 0x1F) {
-        UmaHistogramEnumeration(kExtensionHistogramName,
-                                ChromiumJsonExtension::kControlCharacter);
-        if (!(options_ & JSON_ALLOW_CONTROL_CHARS)) {
-          ReportError(JSON_UNSUPPORTED_ENCODING, -1);
-          return false;
-        }
-      }
-
-      // If this character is not an escape sequence, track any line breaks and
-      // copy next_char to the StringBuilder. The JSON spec forbids unescaped
-      // ASCII control characters within a string, including '\r' and '\n', but
-      // this implementation is more lenient.
-      if ((next_char == '\r') || (next_char == '\n')) {
-        index_last_line_ = index_;
-        // Don't increment line_number_ twice for "\r\n". We are guaranteed
-        // that (index_ > 0) because we are consuming a string, so we must have
-        // seen an opening '"' quote character.
-        if ((next_char == '\r') || (input_[index_ - 1] != '\r')) {
-          ++line_number_;
-        }
-      }
+    if (*c == '\\') {
+      std::string_view ret = input_.substr(start_index, index_ - start_index);
       ConsumeChar();
-      string.Append(next_char);
-    } else {
-      // And if it is an escape sequence, the input string will be adjusted
-      // (either by combining the two characters of an encoded escape sequence,
-      // or with a UTF conversion), so using StringPiece isn't possible -- force
-      // a conversion.
-      string.Convert();
+      return {StringResult::kEscape, ret};
+    }
 
-      // Read past the escape '\' and ensure there's a character following.
-      absl::optional<StringPiece> escape_sequence = ConsumeChars(2);
-      if (!escape_sequence) {
-        ReportError(JSON_INVALID_ESCAPE, -1);
-        return false;
+    // Per Section 7, "All Unicode characters may be placed within the
+    // quotation marks, except for the characters that MUST be escaped:
+    // quotation mark, reverse solidus, and the control characters (U+0000
+    // through U+001F)".
+    if (*c == '\n' || *c == '\r') {
+      UmaHistogramEnumeration(kExtensionHistogramName,
+                              ChromiumJsonExtension::kNewlineInString);
+      if (!(options_ &
+            (JSON_ALLOW_NEWLINES_IN_STRINGS | JSON_ALLOW_CONTROL_CHARS))) {
+        ReportError(JSON_UNSUPPORTED_ENCODING, -1);
+        return {StringResult::kError, {}};  // No need to return consumed data.
       }
-
-      switch ((*escape_sequence)[1]) {
-        // Allowed esape sequences:
-        case 'x': {  // UTF-8 sequence.
-          // UTF-8 \x escape sequences are not allowed in the spec, but they
-          // are supported here for backwards-compatiblity with the old parser.
-          UmaHistogramEnumeration(kExtensionHistogramName,
-                                  ChromiumJsonExtension::kXEscape);
-          if (!(options_ & JSON_ALLOW_X_ESCAPES)) {
-            ReportError(JSON_INVALID_ESCAPE, -1);
-            return false;
-          }
-
-          escape_sequence = ConsumeChars(2);
-          if (!escape_sequence) {
-            ReportError(JSON_INVALID_ESCAPE, -3);
-            return false;
-          }
-
-          int hex_digit = 0;
-          if (!UnprefixedHexStringToInt(*escape_sequence, &hex_digit) ||
-              !IsValidCharacter(hex_digit)) {
-            ReportError(JSON_INVALID_ESCAPE, -3);
-            return false;
-          }
-
-          string.Append(hex_digit);
-          break;
-        }
-        case 'u': {  // UTF-16 sequence.
-          // UTF units are of the form \uXXXX.
-          base_icu::UChar32 code_point;
-          if (!DecodeUTF16(&code_point)) {
-            ReportError(JSON_INVALID_ESCAPE, -1);
-            return false;
-          }
-          string.Append(code_point);
-          break;
-        }
-        case '"':
-          string.Append('"');
-          break;
-        case '\\':
-          string.Append('\\');
-          break;
-        case '/':
-          string.Append('/');
-          break;
-        case 'b':
-          string.Append('\b');
-          break;
-        case 'f':
-          string.Append('\f');
-          break;
-        case 'n':
-          string.Append('\n');
-          break;
-        case 'r':
-          string.Append('\r');
-          break;
-        case 't':
-          string.Append('\t');
-          break;
-        case 'v':  // Not listed as valid escape sequence in the RFC.
-          UmaHistogramEnumeration(kExtensionHistogramName,
-                                  ChromiumJsonExtension::kVerticalTabEscape);
-          if (!(options_ & JSON_ALLOW_VERT_TAB)) {
-            ReportError(JSON_INVALID_ESCAPE, -1);
-            return false;
-          }
-          string.Append('\v');
-          break;
-        // All other escape squences are illegal.
-        default:
-          ReportError(JSON_INVALID_ESCAPE, -1);
-          return false;
+    } else if (*c <= 0x1F) {
+      UmaHistogramEnumeration(kExtensionHistogramName,
+                              ChromiumJsonExtension::kControlCharacter);
+      if (!(options_ & JSON_ALLOW_CONTROL_CHARS)) {
+        ReportError(JSON_UNSUPPORTED_ENCODING, -1);
+        return {StringResult::kError, {}};  // No need to return consumed data.
       }
     }
+
+    // If this character is not an escape sequence, track any line breaks and
+    // keep parsing. The JSON spec forbids unescaped ASCII control characters
+    // within a string, including '\r' and '\n', but this implementation is more
+    // lenient.
+    if (*c == '\r' || *c == '\n') {
+      index_last_line_ = index_;
+      // Don't increment line_number_ twice for "\r\n". We are guaranteed that
+      // (index_ > 0) because we are consuming a string, so we must have seen an
+      // opening '"' quote character.
+      if ((*c == '\r') || (input_[index_ - 1] != '\r')) {
+        ++line_number_;
+      }
+    }
+    ConsumeChar();
   }
 
   ReportError(JSON_SYNTAX_ERROR, -1);
-  return false;
+  return {StringResult::kError, {}};  // No need to return consumed data.
 }
 
 // Entry is at the first X in \uXXXX.
 bool JSONParser::DecodeUTF16(base_icu::UChar32* out_code_point) {
-  absl::optional<StringPiece> escape_sequence = ConsumeChars(4);
+  std::optional<std::string_view> escape_sequence = ConsumeChars(4);
   if (!escape_sequence)
     return false;
 
@@ -743,7 +754,7 @@ bool JSONParser::DecodeUTF16(base_icu::UChar32* out_code_point) {
   return true;
 }
 
-absl::optional<Value> JSONParser::ConsumeNumber() {
+std::optional<Value> JSONParser::ConsumeNumber() {
   const char* num_start = pos();
   const size_t start_index = index_;
   size_t end_index = start_index;
@@ -753,7 +764,7 @@ absl::optional<Value> JSONParser::ConsumeNumber() {
 
   if (!ReadInt(false)) {
     ReportError(JSON_SYNTAX_ERROR, 0);
-    return absl::nullopt;
+    return std::nullopt;
   }
   end_index = index_;
 
@@ -762,13 +773,13 @@ absl::optional<Value> JSONParser::ConsumeNumber() {
     ConsumeChar();
     if (!ReadInt(true)) {
       ReportError(JSON_SYNTAX_ERROR, 0);
-      return absl::nullopt;
+      return std::nullopt;
     }
     end_index = index_;
   }
 
   // Optional exponent part.
-  absl::optional<char> c = PeekChar();
+  std::optional<char> c = PeekChar();
   if (c == 'e' || c == 'E') {
     ConsumeChar();
     if (PeekChar() == '-' || PeekChar() == '+') {
@@ -776,40 +787,19 @@ absl::optional<Value> JSONParser::ConsumeNumber() {
     }
     if (!ReadInt(true)) {
       ReportError(JSON_SYNTAX_ERROR, 0);
-      return absl::nullopt;
+      return std::nullopt;
     }
     end_index = index_;
   }
 
-  // ReadInt is greedy because numbers have no easily detectable sentinel,
-  // so save off where the parser should be on exit (see Consume invariant at
-  // the top of the header), then make sure the next token is one which is
-  // valid.
-  size_t exit_index = index_;
-
-  switch (GetNextToken()) {
-    case T_OBJECT_END:
-    case T_ARRAY_END:
-    case T_LIST_SEPARATOR:
-    case T_END_OF_INPUT:
-      break;
-    default:
-      ReportError(JSON_SYNTAX_ERROR, 0);
-      return absl::nullopt;
-  }
-
-  index_ = exit_index;
-
-  StringPiece num_string(num_start, end_index - start_index);
+  std::string_view num_string(num_start, end_index - start_index);
 
   int num_int;
   if (StringToInt(num_string, &num_int)) {
     // StringToInt will treat `-0` as zero, losing the significance of the
     // negation.
     if (num_int == 0 && num_string.starts_with('-')) {
-      if (base::FeatureList::IsEnabled(features::kJsonNegativeZero)) {
-        return Value(-0.0);
-      }
+      return Value(-0.0);
     }
     return Value(num_int);
   }
@@ -820,14 +810,14 @@ absl::optional<Value> JSONParser::ConsumeNumber() {
   }
 
   ReportError(JSON_UNREPRESENTABLE_NUMBER, 0);
-  return absl::nullopt;
+  return std::nullopt;
 }
 
 bool JSONParser::ReadInt(bool allow_leading_zeros) {
   size_t len = 0;
   char first = 0;
 
-  while (absl::optional<char> c = PeekChar()) {
+  while (std::optional<char> c = PeekChar()) {
     if (!IsAsciiDigit(c))
       break;
 
@@ -847,7 +837,7 @@ bool JSONParser::ReadInt(bool allow_leading_zeros) {
   return true;
 }
 
-absl::optional<Value> JSONParser::ConsumeLiteral() {
+std::optional<Value> JSONParser::ConsumeLiteral() {
   if (ConsumeIfMatch("true"))
     return Value(true);
   if (ConsumeIfMatch("false"))
@@ -855,10 +845,10 @@ absl::optional<Value> JSONParser::ConsumeLiteral() {
   if (ConsumeIfMatch("null"))
     return Value(Value::Type::NONE);
   ReportError(JSON_SYNTAX_ERROR, 0);
-  return absl::nullopt;
+  return std::nullopt;
 }
 
-bool JSONParser::ConsumeIfMatch(StringPiece match) {
+bool JSONParser::ConsumeIfMatch(std::string_view match) {
   if (match == PeekChars(match.size())) {
     ConsumeChars(match.size());
     return true;
