@@ -2,11 +2,17 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/json/json_reader.h"
 
 #include <stddef.h>
 
 #include <cmath>
+#include <string_view>
 #include <utility>
 
 #include "base/base_paths.h"
@@ -15,7 +21,6 @@
 #include "base/logging.h"
 #include "base/path_service.h"
 #include "base/rust_buildflags.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/gmock_expected_support.h"
@@ -25,19 +30,18 @@
 #include "build/build_config.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace {
 
 // MSan will do a better job detecting over-read errors if the input is not
 // nul-terminated on the heap. This will copy |input| to a new buffer owned by
-// |owner|, returning a base::StringPiece to |owner|.
-base::StringPiece MakeNotNullTerminatedInput(const char* input,
-                                             std::unique_ptr<char[]>* owner) {
+// |owner|, returning a std::string_view to |owner|.
+std::string_view MakeNotNullTerminatedInput(const char* input,
+                                            std::unique_ptr<char[]>* owner) {
   size_t str_len = strlen(input);
   owner->reset(new char[str_len]);
   memcpy(owner->get(), input, str_len);
-  return base::StringPiece(owner->get(), str_len);
+  return std::string_view(owner->get(), str_len);
 }
 
 }  // namespace
@@ -57,7 +61,7 @@ class JSONReaderTest : public testing::TestWithParam<bool> {
 };
 
 TEST_P(JSONReaderTest, Whitespace) {
-  absl::optional<Value> root = JSONReader::Read("   null   ");
+  std::optional<Value> root = JSONReader::Read("   null   ");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_none());
 }
@@ -73,7 +77,7 @@ TEST_P(JSONReaderTest, SimpleBool) {
 #if BUILDFLAG(BUILD_RUST_JSON_READER)
   base::HistogramTester histograms;
 #endif  // BUILDFLAG(BUILD_RUST_JSON_READER)
-  absl::optional<Value> root = JSONReader::Read("true  ");
+  std::optional<Value> root = JSONReader::Read("true  ");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_bool());
 #if BUILDFLAG(BUILD_RUST_JSON_READER)
@@ -82,7 +86,7 @@ TEST_P(JSONReaderTest, SimpleBool) {
 }
 
 TEST_P(JSONReaderTest, EmbeddedComments) {
-  absl::optional<Value> root = JSONReader::Read("/* comment */null");
+  std::optional<Value> root = JSONReader::Read("/* comment */null");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_none());
   root = JSONReader::Read("40 /* comment */");
@@ -138,7 +142,7 @@ TEST_P(JSONReaderTest, EmbeddedComments) {
 }
 
 TEST_P(JSONReaderTest, Ints) {
-  absl::optional<Value> root = JSONReader::Read("43");
+  std::optional<Value> root = JSONReader::Read("43");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_int());
   EXPECT_EQ(43, root->GetInt());
@@ -154,7 +158,7 @@ TEST_P(JSONReaderTest, NonDecimalNumbers) {
 TEST_P(JSONReaderTest, NumberZero) {
   // Test 0 (which needs to be special cased because of the leading zero
   // clause).
-  absl::optional<Value> root = JSONReader::Read("0");
+  std::optional<Value> root = JSONReader::Read("0");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_int());
   EXPECT_EQ(0, root->GetInt());
@@ -163,7 +167,7 @@ TEST_P(JSONReaderTest, NumberZero) {
 TEST_P(JSONReaderTest, LargeIntPromotion) {
   // Numbers that overflow ints should succeed, being internally promoted to
   // storage as doubles
-  absl::optional<Value> root = JSONReader::Read("2147483648");
+  std::optional<Value> root = JSONReader::Read("2147483648");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_double());
   EXPECT_DOUBLE_EQ(2147483648.0, root->GetDouble());
@@ -180,7 +184,7 @@ TEST_P(JSONReaderTest, LargerIntIsLossy) {
   // In this case, parsing is lossy.
   const char* etc807 = "9223372036854775807";
   const char* etc808 = "9223372036854775808.000000";
-  absl::optional<Value> root = JSONReader::Read(etc807);
+  std::optional<Value> root = JSONReader::Read(etc807);
   ASSERT_TRUE(root);
   ASSERT_FALSE(root->is_int());
   ASSERT_TRUE(root->is_double());
@@ -191,7 +195,7 @@ TEST_P(JSONReaderTest, LargerIntIsLossy) {
 }
 
 TEST_P(JSONReaderTest, Doubles) {
-  absl::optional<Value> root = JSONReader::Read("43.1");
+  std::optional<Value> root = JSONReader::Read("43.1");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_double());
   EXPECT_DOUBLE_EQ(43.1, root->GetDouble());
@@ -268,7 +272,7 @@ TEST_P(JSONReaderTest, InvalidNumbers) {
 }
 
 TEST_P(JSONReaderTest, Zeroes) {
-  absl::optional<Value> root = JSONReader::Read("0");
+  std::optional<Value> root = JSONReader::Read("0");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_int());
   EXPECT_DOUBLE_EQ(0, root->GetInt());
@@ -293,30 +297,30 @@ TEST_P(JSONReaderTest, Zeroes) {
 }
 
 TEST_P(JSONReaderTest, SimpleString) {
-  absl::optional<Value> root = JSONReader::Read("\"hello world\"");
+  std::optional<Value> root = JSONReader::Read("\"hello world\"");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_string());
   EXPECT_EQ("hello world", root->GetString());
 }
 
 TEST_P(JSONReaderTest, EmptyString) {
-  absl::optional<Value> root = JSONReader::Read("\"\"");
+  std::optional<Value> root = JSONReader::Read("\"\"");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_string());
   EXPECT_EQ("", root->GetString());
 }
 
 TEST_P(JSONReaderTest, BasicStringEscapes) {
-  absl::optional<Value> root =
-      JSONReader::Read("\" \\\"\\\\\\/\\b\\f\\n\\r\\t\\v\"");
+  std::optional<Value> root =
+      JSONReader::Read("\" \\\"\\\\\\/\\b\\f\\n\\r\\t\"");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_string());
-  EXPECT_EQ(" \"\\/\b\f\n\r\t\v", root->GetString());
+  EXPECT_EQ(" \"\\/\b\f\n\r\t", root->GetString());
 }
 
 TEST_P(JSONReaderTest, UnicodeEscapes) {
   // Test hex and unicode escapes including the null character.
-  absl::optional<Value> root =
+  std::optional<Value> root =
       JSONReader::Read("\"\\x41\\xFF\\x00\\u1234\\u0000\"");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_string());
@@ -341,21 +345,21 @@ TEST_P(JSONReaderTest, InvalidStrings) {
 }
 
 TEST_P(JSONReaderTest, BasicArray) {
-  absl::optional<Value> root = JSONReader::Read("[true, false, null]");
+  std::optional<Value> root = JSONReader::Read("[true, false, null]");
   ASSERT_TRUE(root);
   Value::List* list = root->GetIfList();
   ASSERT_TRUE(list);
   EXPECT_EQ(3U, list->size());
 
   // Test with trailing comma.  Should be parsed the same as above.
-  absl::optional<Value> root2 =
+  std::optional<Value> root2 =
       JSONReader::Read("[true, false, null, ]", JSON_ALLOW_TRAILING_COMMAS);
   ASSERT_TRUE(root2);
   EXPECT_EQ(*list, *root2);
 }
 
 TEST_P(JSONReaderTest, EmptyArray) {
-  absl::optional<Value> value = JSONReader::Read("[]");
+  std::optional<Value> value = JSONReader::Read("[]");
   ASSERT_TRUE(value);
   Value::List* list = value->GetIfList();
   ASSERT_TRUE(list);
@@ -363,7 +367,7 @@ TEST_P(JSONReaderTest, EmptyArray) {
 }
 
 TEST_P(JSONReaderTest, CompleteArray) {
-  absl::optional<Value> value = JSONReader::Read("[\"a\", 3, 4.56, null]");
+  std::optional<Value> value = JSONReader::Read("[\"a\", 3, 4.56, null]");
   ASSERT_TRUE(value);
   Value::List* list = value->GetIfList();
   ASSERT_TRUE(list);
@@ -371,7 +375,7 @@ TEST_P(JSONReaderTest, CompleteArray) {
 }
 
 TEST_P(JSONReaderTest, NestedArrays) {
-  absl::optional<Value> value = JSONReader::Read(
+  std::optional<Value> value = JSONReader::Read(
       "[[true], [], {\"smell\": \"nice\",\"taste\": \"yummy\" }, [false, [], "
       "[null]], null]");
   ASSERT_TRUE(value);
@@ -380,7 +384,7 @@ TEST_P(JSONReaderTest, NestedArrays) {
   EXPECT_EQ(5U, list->size());
 
   // Lots of trailing commas.
-  absl::optional<Value> root2 = JSONReader::Read(
+  std::optional<Value> root2 = JSONReader::Read(
       "[[true], [], {\"smell\": \"nice\",\"taste\": \"yummy\" }, [false, [], "
       "[null, ]  , ], null,]",
       JSON_ALLOW_TRAILING_COMMAS);
@@ -405,7 +409,7 @@ TEST_P(JSONReaderTest, InvalidArrays) {
 
 TEST_P(JSONReaderTest, ArrayTrailingComma) {
   // Valid if we set |allow_trailing_comma| to true.
-  absl::optional<Value> value =
+  std::optional<Value> value =
       JSONReader::Read("[true,]", JSON_ALLOW_TRAILING_COMMAS);
   ASSERT_TRUE(value);
   Value::List* list = value->GetIfList();
@@ -426,13 +430,13 @@ TEST_P(JSONReaderTest, ArrayTrailingCommaNoEmptyElements) {
 }
 
 TEST_P(JSONReaderTest, EmptyDictionary) {
-  absl::optional<Value> dict_val = JSONReader::Read("{}");
+  std::optional<Value> dict_val = JSONReader::Read("{}");
   ASSERT_TRUE(dict_val);
   ASSERT_TRUE(dict_val->is_dict());
 }
 
 TEST_P(JSONReaderTest, CompleteDictionary) {
-  absl::optional<Value> root1 = JSONReader::Read(
+  std::optional<Value> root1 = JSONReader::Read(
       "{\"number\":9.87654321, \"null\":null , \"\\x53\" : \"str\", \"bool\": "
       "false, \"more\": {} }");
   ASSERT_TRUE(root1);
@@ -451,7 +455,7 @@ TEST_P(JSONReaderTest, CompleteDictionary) {
   ASSERT_TRUE(bool_val);
   ASSERT_FALSE(*bool_val);
 
-  absl::optional<Value> root2 = JSONReader::Read(
+  std::optional<Value> root2 = JSONReader::Read(
       "{\"number\":9.87654321, \"null\":null , \"\\x53\" : \"str\", \"bool\": "
       "false, \"more\": {},}",
       JSON_PARSE_CHROMIUM_EXTENSIONS | JSON_ALLOW_TRAILING_COMMAS);
@@ -491,7 +495,7 @@ TEST_P(JSONReaderTest, CompleteDictionary) {
 }
 
 TEST_P(JSONReaderTest, NestedDictionaries) {
-  absl::optional<Value> root1 = JSONReader::Read(
+  std::optional<Value> root1 = JSONReader::Read(
       "{\"inner\":{\"array\":[true, 3, 4.56, null]},\"false\":false,\"d\":{}}");
   ASSERT_TRUE(root1);
   const base::Value::Dict* root1_dict = root1->GetIfDict();
@@ -507,7 +511,7 @@ TEST_P(JSONReaderTest, NestedDictionaries) {
   inner_dict = root1_dict->FindDict("d");
   EXPECT_TRUE(inner_dict);
 
-  absl::optional<Value> root2 = JSONReader::Read(
+  std::optional<Value> root2 = JSONReader::Read(
       "{\"inner\": {\"array\":[true, 3, 4.56, null] , "
       "},\"false\":false,\"d\":{},}",
       JSON_ALLOW_TRAILING_COMMAS);
@@ -516,7 +520,7 @@ TEST_P(JSONReaderTest, NestedDictionaries) {
 }
 
 TEST_P(JSONReaderTest, DictionaryKeysWithPeriods) {
-  absl::optional<Value> root =
+  std::optional<Value> root =
       JSONReader::Read("{\"a.b\":3,\"c\":2,\"d.e.f\":{\"g.h.i.j\":1}}");
   ASSERT_TRUE(root);
   Value::Dict* root_dict = root->GetIfDict();
@@ -548,7 +552,7 @@ TEST_P(JSONReaderTest, DictionaryKeysWithPeriods) {
 }
 
 TEST_P(JSONReaderTest, DuplicateKeys) {
-  absl::optional<Value> root = JSONReader::Read("{\"x\":1,\"x\":2,\"y\":3}");
+  std::optional<Value> root = JSONReader::Read("{\"x\":1,\"x\":2,\"y\":3}");
   ASSERT_TRUE(root);
   const Value::Dict* root_dict = root->GetIfDict();
   ASSERT_TRUE(root_dict);
@@ -599,7 +603,7 @@ TEST_P(JSONReaderTest, StackOverflow) {
   for (int i = 0; i < 5000; ++i)
     not_evil.append("[],");
   not_evil.append("[]]");
-  absl::optional<Value> value = JSONReader::Read(not_evil);
+  std::optional<Value> value = JSONReader::Read(not_evil);
   ASSERT_TRUE(value);
   Value::List* list = value->GetIfList();
   ASSERT_TRUE(list);
@@ -607,7 +611,7 @@ TEST_P(JSONReaderTest, StackOverflow) {
 }
 
 TEST_P(JSONReaderTest, UTF8Input) {
-  absl::optional<Value> root = JSONReader::Read("\"\xe7\xbd\x91\xe9\xa1\xb5\"");
+  std::optional<Value> root = JSONReader::Read("\"\xe7\xbd\x91\xe9\xa1\xb5\"");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_string());
   const std::string& str_val = root->GetString();
@@ -677,7 +681,7 @@ TEST_P(JSONReaderTest, InvalidUTF8Input) {
 }
 
 TEST_P(JSONReaderTest, UTF16Escapes) {
-  absl::optional<Value> root = JSONReader::Read("\"\\u20ac3,14\"");
+  std::optional<Value> root = JSONReader::Read("\"\\u20ac3,14\"");
   ASSERT_TRUE(root);
   ASSERT_TRUE(root->is_string());
   EXPECT_EQ(
@@ -706,7 +710,7 @@ TEST_P(JSONReaderTest, InvalidUTF16Escapes) {
       "\"\\ud83d\\u1\"",     // No lower surrogate.
       "\"\\ud83\\u1\"",      // Invalid upper surrogate.
   };
-  absl::optional<Value> root;
+  std::optional<Value> root;
   for (auto* i : cases) {
     root = JSONReader::Read(i);
     EXPECT_FALSE(root) << i;
@@ -714,7 +718,7 @@ TEST_P(JSONReaderTest, InvalidUTF16Escapes) {
 }
 
 TEST_P(JSONReaderTest, LiteralRoots) {
-  absl::optional<Value> root = JSONReader::Read("null");
+  std::optional<Value> root = JSONReader::Read("null");
   ASSERT_TRUE(root);
   EXPECT_TRUE(root->is_none());
 
@@ -759,7 +763,7 @@ TEST_P(JSONReaderTest, StringOptimizations) {
   Value list_value_1;
 
   {
-    absl::optional<Value> root = JSONReader::Read(
+    std::optional<Value> root = JSONReader::Read(
         "{"
         "  \"test\": {"
         "    \"foo\": true,"
@@ -852,13 +856,24 @@ TEST_P(JSONReaderTest, ASCIIControlCodes) {
   // rejected. RFC 8259 section 7 says "the characters that MUST be escaped
   // [include]... the control characters (U+0000 through U+001F)".
   //
-  // Nonetheless, we accept them, for backwards compatibility.
-  const char json[] = {'"', 'a', '\0', 'b', '\n', 'c', '"'};
-  absl::optional<Value> root =
-      JSONReader::Read(std::string(json, sizeof(json)));
-  ASSERT_TRUE(root);
-  ASSERT_TRUE(root->is_string());
-  EXPECT_EQ(5u, root->GetString().length());
+  // Currently, we accept \r and \n in JSON strings because they are widely used
+  // and somewhat useful (especially when nesting JSON messages), but reject all
+  // other control characters.
+  {
+    const char json[] = "\"a\rn\nc\"";
+    auto root = JSONReader::Read(json);
+    ASSERT_TRUE(root);
+    ASSERT_TRUE(root->is_string());
+    EXPECT_EQ(5u, root->GetString().length());
+  }
+
+  {
+    // Replace the \r with a disallowed \f, and require parsing to fail:
+    const char json[] = "\"a\fn\nc\"";
+    auto root = JSONReader::ReadAndReturnValueWithError(json);
+    EXPECT_FALSE(root.has_value());
+    EXPECT_NE("", root.error().message);
+  }
 }
 
 TEST_P(JSONReaderTest, MaxNesting) {
@@ -872,7 +887,7 @@ TEST_P(JSONReaderTest, Decode4ByteUtf8Char) {
   // should be able to handle. The UTF-8 encoding of U+1F607 SMILING FACE WITH
   // HALO is "\xF0\x9F\x98\x87".
   const char kUtf8Data[] = "[\"😇\",[],[],[],{\"google:suggesttype\":[]}]";
-  absl::optional<Value> root = JSONReader::Read(kUtf8Data, JSON_PARSE_RFC);
+  std::optional<Value> root = JSONReader::Read(kUtf8Data, JSON_PARSE_RFC);
   ASSERT_TRUE(root);
   Value::List* list = root->GetIfList();
   ASSERT_TRUE(list);
@@ -932,7 +947,7 @@ TEST_P(JSONReaderTest, DecodeNegativeEscapeSequence) {
 TEST_P(JSONReaderTest, ReplaceInvalidCharacters) {
   // U+D800 is a lone high surrogate.
   const std::string invalid_high = "\"\xED\xA0\x80\"";
-  absl::optional<Value> value =
+  std::optional<Value> value =
       JSONReader::Read(invalid_high, JSON_REPLACE_INVALID_CHARACTERS);
   ASSERT_TRUE(value);
   ASSERT_TRUE(value->is_string());
@@ -951,7 +966,7 @@ TEST_P(JSONReaderTest, ReplaceInvalidCharacters) {
 TEST_P(JSONReaderTest, ReplaceInvalidUTF16EscapeSequence) {
   // U+D800 is a lone high surrogate.
   const std::string invalid_high = "\"_\\uD800_\"";
-  absl::optional<Value> value =
+  std::optional<Value> value =
       JSONReader::Read(invalid_high, JSON_REPLACE_INVALID_CHARACTERS);
   ASSERT_TRUE(value);
   ASSERT_TRUE(value->is_string());
@@ -989,10 +1004,10 @@ TEST_P(JSONReaderTest, ParseNumberErrors) {
     SCOPED_TRACE(StringPrintf("case %u: \"%s\"", i, test_case.input));
 
     std::unique_ptr<char[]> input_owner;
-    StringPiece input =
+    std::string_view input =
         MakeNotNullTerminatedInput(test_case.input, &input_owner);
 
-    absl::optional<Value> result = JSONReader::Read(input);
+    std::optional<Value> result = JSONReader::Read(input);
     EXPECT_EQ(test_case.parse_success, result.has_value());
 
     if (!result)
@@ -1030,7 +1045,8 @@ TEST_P(JSONReaderTest, UnterminatedInputs) {
     SCOPED_TRACE(StringPrintf("case %u: \"%s\"", i, test_case));
 
     std::unique_ptr<char[]> input_owner;
-    StringPiece input = MakeNotNullTerminatedInput(test_case, &input_owner);
+    std::string_view input =
+        MakeNotNullTerminatedInput(test_case, &input_owner);
 
     EXPECT_FALSE(JSONReader::Read(input));
   }
@@ -1102,7 +1118,7 @@ TEST_P(JSONReaderTest, LineColumnCounting) {
     SCOPED_TRACE(StringPrintf("case %u: \"%s\"", i, test_case.input));
 
     auto root = JSONReader::ReadAndReturnValueWithError(
-        test_case.input, JSON_PARSE_RFC | JSON_ALLOW_CONTROL_CHARS);
+        test_case.input, JSON_PARSE_RFC | JSON_ALLOW_NEWLINES_IN_STRINGS);
     EXPECT_FALSE(root.has_value());
     EXPECT_EQ(test_case.error_line, root.error().line);
     EXPECT_EQ(test_case.error_column, root.error().column);
@@ -1121,13 +1137,8 @@ TEST_P(JSONReaderTest, ChromiumExtensions) {
       {"{ /* comment */ \"foo\": 3 }", JSON_ALLOW_COMMENTS},
       {"{ // comment\n \"foo\": 3 }", JSON_ALLOW_COMMENTS},
       {"[\"\\xAB\"]", JSON_ALLOW_X_ESCAPES},
-      {"[\"\b\"]", JSON_ALLOW_CONTROL_CHARS},
-      {"[\"\f\"]", JSON_ALLOW_CONTROL_CHARS},
-      {"[\"\n\"]", JSON_ALLOW_CONTROL_CHARS},
-      {"[\"\r\"]", JSON_ALLOW_CONTROL_CHARS},
-      {"[\"\t\"]", JSON_ALLOW_CONTROL_CHARS},
-      {"[\"\v\"]", JSON_ALLOW_CONTROL_CHARS},
-      {"[\"\\v\"]", JSON_ALLOW_VERT_TAB},
+      {"[\"\n\"]", JSON_ALLOW_NEWLINES_IN_STRINGS},
+      {"[\"\r\"]", JSON_ALLOW_NEWLINES_IN_STRINGS},
   };
 
   for (size_t i = 0; i < std::size(kCases); ++i) {
@@ -1152,6 +1163,32 @@ TEST_P(JSONReaderTest, ChromiumExtensions) {
   }
 }
 
+// For every control character, place it unescaped in a string and ensure that:
+// a) It doesn't parse with JSON_PARSE_RFC
+// b) It doesn't parse with JSON_PARSE_CHROMIUM_EXTENSIONS
+// c) It does parse with JSON_ALLOW_CONTROL_CHARS
+TEST_P(JSONReaderTest, UnescapedControls) {
+  std::string input = "\"foo\"";
+  // ECMA-404 (JSON standard) section 9: characters from 0x00 to 0x1f must be
+  // escaped.
+  for (char c = 0x00; c <= 0x1f; c++) {
+    input[1] = c;
+
+    auto result = JSONReader::Read(input, JSON_PARSE_RFC);
+    EXPECT_FALSE(result.has_value());
+
+    bool should_parse_with_extensions = (c == '\r' || c == '\n');
+    result = JSONReader::Read(input, JSON_PARSE_CHROMIUM_EXTENSIONS);
+    EXPECT_EQ(should_parse_with_extensions, result.has_value());
+
+    result = JSONReader::Read(input, JSON_ALLOW_CONTROL_CHARS);
+    ASSERT_TRUE(result.has_value());
+    ASSERT_TRUE(result->is_string());
+    EXPECT_EQ(result->GetString().length(), input.length() - 2);
+    EXPECT_EQ(result->GetString()[0], c);
+  }
+}
+
 TEST_P(JSONReaderTest, UsingRust) {
   ASSERT_EQ(JSONReader::UsingRust(), using_rust_);
 }
