@@ -18,6 +18,19 @@
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace base {
+
+namespace subtle {
+
+class BindWeakPtrFactoryForTesting {
+ public:
+  template <typename T>
+  static void BindToCurrentSequence(WeakPtrFactory<T>& factory) {
+    factory.BindToCurrentSequence(BindWeakPtrFactoryPassKey());
+  }
+};
+
+}  // namespace subtle
+
 namespace {
 
 WeakPtr<int> PassThru(WeakPtr<int> ptr) {
@@ -51,8 +64,13 @@ struct Base {
 struct Derived : public Base {};
 
 struct TargetBase {};
-struct Target : public TargetBase, public SupportsWeakPtr<Target> {
+
+struct Target : public TargetBase {
   virtual ~Target() = default;
+  WeakPtr<Target> AsWeakPtr() { return weak_ptr_factory_.GetWeakPtr(); }
+
+ private:
+  WeakPtrFactory<Target> weak_ptr_factory_{this};
 };
 
 struct DerivedTarget : public Target {};
@@ -156,6 +174,15 @@ class BackgroundThread : public Thread {
     return result;
   }
 
+  void BindToCurrentSequence(TargetWithFactory* target_with_factory) {
+    WaitableEvent completion(WaitableEvent::ResetPolicy::MANUAL,
+                             WaitableEvent::InitialState::NOT_SIGNALED);
+    task_runner()->PostTask(
+        FROM_HERE, base::BindOnce(&BackgroundThread::DoBindToCurrentSequence,
+                                  target_with_factory, &completion));
+    completion.Wait();
+  }
+
  protected:
   static void DoCreateArrowFromArrow(Arrow** arrow,
                                      const Arrow* other,
@@ -208,6 +235,13 @@ class BackgroundThread : public Thread {
     delete object;
     completion->Signal();
   }
+
+  static void DoBindToCurrentSequence(TargetWithFactory* target_with_factory,
+                                      WaitableEvent* completion) {
+    subtle::BindWeakPtrFactoryForTesting::BindToCurrentSequence(
+        target_with_factory->factory);
+    completion->Signal();
+  }
 };
 
 }  // namespace
@@ -299,43 +333,6 @@ TEST(WeakPtrTest, ConstructFromNullptr) {
   EXPECT_EQ(nullptr, ptr.get());
 }
 
-TEST(WeakPtrTest, SupportsWeakPtr) {
-  Target target;
-  WeakPtr<Target> ptr = target.AsWeakPtr();
-  EXPECT_EQ(&target, ptr.get());
-}
-
-TEST(WeakPtrTest, DerivedTarget) {
-  DerivedTarget target;
-  WeakPtr<DerivedTarget> ptr = AsWeakPtr(&target);
-  EXPECT_EQ(&target, ptr.get());
-}
-
-TEST(WeakPtrTest, DerivedTargetWithNestedBase) {
-  DerivedTargetWithNestedBase target;
-  WeakPtr<DerivedTargetWithNestedBase> ptr = AsWeakPtr(&target);
-  EXPECT_EQ(&target, ptr.get());
-}
-
-TEST(WeakPtrTest, DerivedTargetMultipleInheritance) {
-  DerivedTargetMultipleInheritance derived_target;
-  Target& target = derived_target;
-  EXPECT_NE(static_cast<void*>(&derived_target), static_cast<void*>(&target));
-
-  WeakPtr<Target> target_weak_ptr = AsWeakPtr(&target);
-  EXPECT_EQ(target_weak_ptr.get(), &target);
-
-  WeakPtr<DerivedTargetMultipleInheritance> derived_target_weak_ptr =
-      AsWeakPtr(&derived_target);
-  EXPECT_EQ(derived_target_weak_ptr.get(), &derived_target);
-
-  target_weak_ptr = derived_target_weak_ptr;
-  EXPECT_EQ(target_weak_ptr.get(), &target);
-
-  target_weak_ptr = std::move(derived_target_weak_ptr);
-  EXPECT_EQ(target_weak_ptr.get(), &target);
-}
-
 TEST(WeakPtrFactoryTest, BooleanTesting) {
   int data;
   WeakPtrFactory<int> factory(&data);
@@ -771,6 +768,27 @@ TEST(WeakPtrTest, GetMutableWeakPtr) {
   EXPECT_EQ(test_struct.member, 1);
 }
 
+TEST(WeakPtrDeathTest, BindToCurrentSequence) {
+  BackgroundThread background;
+  background.Start();
+
+  TargetWithFactory target_with_factory;
+  Arrow arrow{
+      .target = target_with_factory.factory.GetWeakPtr(),
+  };
+
+  // WeakPtr can be accessed on main thread.
+  EXPECT_TRUE(arrow.target.get());
+
+  background.BindToCurrentSequence(&target_with_factory);
+
+  // Now WeakPtr can be accessed on background thread.
+  EXPECT_TRUE(background.DeRef(&arrow));
+
+  // WeakPtr can no longer be accessed on main thread.
+  EXPECT_DCHECK_DEATH(arrow.target.get());
+}
+
 TEST(WeakPtrDeathTest, WeakPtrCopyDoesNotChangeThreadBinding) {
   // The default style "fast" does not support multi-threaded tests
   // (introduces deadlock on Linux).
