@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 // The FieldTrial class handles the lower level configuration of running A/B
 // tests.
 //
@@ -76,6 +81,7 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include "base/atomicops.h"
@@ -89,13 +95,17 @@
 #include "base/memory/shared_memory_mapping.h"
 #include "base/metrics/persistent_memory_allocator.h"
 #include "base/pickle.h"
-#include "base/strings/string_piece.h"
 #include "base/synchronization/lock.h"
 #include "base/types/expected.h"
 #include "base/types/pass_key.h"
 #include "build/blink_buildflags.h"
 #include "build/build_config.h"
 
+#if BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_APPLE)
+#include "base/files/platform_file.h"
+#include "base/posix/global_descriptors.h"
+#endif
+
 namespace base {
 
 namespace test {
@@ -106,6 +116,12 @@ class CompareActiveGroupToFieldTrialMatcher;
 class FieldTrialList;
 struct LaunchOptions;
 
+#if BUILDFLAG(USE_BLINK)
+namespace shared_memory {
+enum class SharedMemoryError;
+}  // namespace shared_memory
+#endif
+
 class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
  public:
   typedef int Probability;  // Probability type for being selected in a trial.
@@ -121,7 +137,7 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
     // used in preference to |trial_name| for generating the entropy by entropy
     // providers that support it. A given instance should always return the same
     // value given the same input |trial_name| and |randomization_seed| values.
-    virtual double GetEntropyForTrial(StringPiece trial_name,
+    virtual double GetEntropyForTrial(std::string_view trial_name,
                                       uint32_t randomization_seed) const = 0;
 
     // Returns a pseudorandom integer in [0, output_range).
@@ -131,8 +147,8 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
 
   // Separate type from FieldTrial::PickleState so that it can use StringPieces.
   struct State {
-    StringPiece trial_name;
-    StringPiece group_name;
+    std::string_view trial_name;
+    std::string_view group_name;
     bool activated = false;
     // Whether the trial was overridden, see `FieldTrial::SetOverridden()`.
     bool is_overridden = false;
@@ -148,8 +164,8 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
 
   // Represents a FieldTrial, its selected group, whether it's active, and
   // whether it's overridden. String members are pointers to the underlying
-  // strings owned by the FieldTrial object. Does not use StringPiece to avoid
-  // conversions back to std::string.
+  // strings owned by the FieldTrial object. Does not use std::string_view to
+  // avoid conversions back to std::string.
   struct BASE_EXPORT PickleState {
     raw_ptr<const std::string> trial_name = nullptr;
     raw_ptr<const std::string> group_name = nullptr;
@@ -172,9 +188,11 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
     static constexpr size_t kExpectedInstanceSize = 16;
 
     // Return a pointer to the data area immediately following the entry.
-    char* GetPickledDataPtr() { return reinterpret_cast<char*>(this + 1); }
-    const char* GetPickledDataPtr() const {
-      return reinterpret_cast<const char*>(this + 1);
+    uint8_t* GetPickledDataPtr() {
+      return reinterpret_cast<uint8_t*>(this + 1);
+    }
+    const uint8_t* GetPickledDataPtr() const {
+      return reinterpret_cast<const uint8_t*>(this + 1);
     }
 
     // Whether or not this field trial is activated. This is really just a
@@ -196,8 +214,8 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
     // Calling this is only valid when the entry is initialized. That is, it
     // resides in shared memory and has a pickle containing the trial name,
     // group name, and is_overridden.
-    bool GetState(StringPiece& trial_name,
-                  StringPiece& group_name,
+    bool GetState(std::string_view& trial_name,
+                  std::string_view& group_name,
                   bool& is_overridden) const;
 
     // Calling this is only valid when the entry is initialized as well. Reads
@@ -212,14 +230,14 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
     // Takes the iterator and writes out the first two items into |trial_name|
     // and |group_name|.
     bool ReadStringPair(PickleIterator* iter,
-                        StringPiece* trial_name,
-                        StringPiece* group_name) const;
+                        std::string_view* trial_name,
+                        std::string_view* group_name) const;
 
     // Reads the field trial header, which includes the name of the trial and
     // group, and the is_overridden bool.
     bool ReadHeader(PickleIterator& iter,
-                    StringPiece& trial_name,
-                    StringPiece& group_name,
+                    std::string_view& trial_name,
+                    std::string_view& group_name,
                     bool& is_overridden) const;
   };
 
@@ -284,10 +302,11 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
   //
   // The ownership of the returned FieldTrial is transfered to the caller which
   // is responsible for deref'ing it (e.g. by using scoped_refptr<FieldTrial>).
-  static FieldTrial* CreateSimulatedFieldTrial(StringPiece trial_name,
-                                               Probability total_probability,
-                                               StringPiece default_group_name,
-                                               double entropy_value);
+  static FieldTrial* CreateSimulatedFieldTrial(
+      std::string_view trial_name,
+      Probability total_probability,
+      std::string_view default_group_name,
+      double entropy_value);
 
   // Parses a '--force-fieldtrials' formatted string into entries.
   // Returns true if the string was parsed correctly. On failure, the |entries|
@@ -296,10 +315,9 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
   // Note that currently, States returned here have is_overridden=false, but we
   // are in the process of migrating to marking field trials set manually by
   // command line as overridden. See b/284986126.
-  static bool ParseFieldTrialsString(
-      const base::StringPiece field_trials_string,
-      bool override_trials,
-      std::vector<State>& entries);
+  static bool ParseFieldTrialsString(std::string_view field_trials_string,
+                                     bool override_trials,
+                                     std::vector<State>& entries);
 
   // Returns a '--force-fieldtrials' formatted string representing the list of
   // provided trial states.
@@ -308,8 +326,8 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
 
   // Whether this field trial is low anonymity or not (see
   // |FieldTrialListIncludingLowAnonymity|).
-  // TODO(crbug.com/1431156): remove this once all call sites have been properly
-  // migrated to use an appropriate observer.
+  // TODO(crbug.com/40263398): remove this once all call sites have been
+  // properly migrated to use an appropriate observer.
   bool is_low_anonymity() const { return is_low_anonymity_; }
 
  private:
@@ -357,9 +375,9 @@ class BASE_EXPORT FieldTrial : public RefCounted<FieldTrial> {
 
   // Creates a field trial with the specified parameters. Group assignment will
   // be done based on |entropy_value|, which must have a range of [0, 1).
-  FieldTrial(StringPiece trial_name,
+  FieldTrial(std::string_view trial_name,
              Probability total_probability,
-             StringPiece default_group_name,
+             std::string_view default_group_name,
              double entropy_value,
              bool is_low_anonymity,
              bool is_overridden);
@@ -510,9 +528,9 @@ class BASE_EXPORT FieldTrialList {
   // This static method can be used to get a startup-randomized FieldTrial or a
   // previously created forced FieldTrial.
   static FieldTrial* FactoryGetFieldTrial(
-      StringPiece trial_name,
+      std::string_view trial_name,
       FieldTrial::Probability total_probability,
-      StringPiece default_group_name,
+      std::string_view default_group_name,
       const FieldTrial::EntropyProvider& entropy_provider,
       uint32_t randomization_seed = 0,
       bool is_low_anonymity = false,
@@ -520,7 +538,7 @@ class BASE_EXPORT FieldTrialList {
 
   // The Find() method can be used to test to see if a named trial was already
   // registered, or to retrieve a pointer to it from the global map.
-  static FieldTrial* Find(StringPiece trial_name);
+  static FieldTrial* Find(std::string_view trial_name);
 
   // Returns the group name chosen for the named trial, or the empty string if
   // the trial does not exist. The first call of this function on a given field
@@ -528,13 +546,13 @@ class BASE_EXPORT FieldTrialList {
   // metrics, crashes, etc.
   // Note: Direct use of this function and related FieldTrial functions is
   // generally discouraged - instead please use base::Feature when possible.
-  static std::string FindFullName(StringPiece trial_name);
+  static std::string FindFullName(std::string_view trial_name);
 
   // Returns true if the named trial has been registered.
-  static bool TrialExists(StringPiece trial_name);
+  static bool TrialExists(std::string_view trial_name);
 
   // Returns true if the named trial exists and has been activated.
-  static bool IsTrialActive(StringPiece trial_name);
+  static bool IsTrialActive(std::string_view trial_name);
 
   // Creates a persistent representation of all FieldTrial instances for
   // resurrection in another process. This allows randomization to be done in
@@ -590,14 +608,7 @@ class BASE_EXPORT FieldTrialList {
   // via PopulateLaunchOptionsWithFieldTrialState() in the parent process.
   // Trials are retrieved from a shared memory segment that has been shared with
   // the child process.
-  //
-  // `fd_key` is used on non-Mac POSIX platforms to access the shared memory
-  // segment and ignored on other platforms. The argument is needed here since
-  // //base can't depend on //content. On other platforms, we expect the
-  // `cmd_line` switch for kFieldTrialHandle to contain the shared memory handle
-  // that contains the field trial allocator.
-  static void CreateTrialsInChildProcess(const CommandLine& cmd_line,
-                                         uint32_t fd_key);
+  static void CreateTrialsInChildProcess(const CommandLine& cmd_line);
 
   // Creates base::Feature overrides in a child process using shared memory.
   // Requires CreateTrialsInChildProcess() to have been called first which
@@ -609,18 +620,14 @@ class BASE_EXPORT FieldTrialList {
   // line arguments necessary for a child process to inherit the shared-memory
   // object containing the FieldTrial configuration.
   static void PopulateLaunchOptionsWithFieldTrialState(
+#if BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_APPLE)
+      GlobalDescriptors::Key descriptor_key,
+      ScopedFD& descriptor_to_share,
+#endif  // BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_APPLE)
       CommandLine* command_line,
       LaunchOptions* launch_options);
 #endif  // !BUILDFLAG(USE_BLINK)
 
-#if BUILDFLAG(USE_BLINK) && BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_APPLE)
-  // On POSIX, we also need to explicitly pass down this file descriptor that
-  // should be shared with the child process. Returns -1 if it was not
-  // initialized properly. The current process remains the owner of the passed
-  // descriptor.
-  static int GetFieldTrialDescriptor();
-#endif  // BUILDFLAG(USE_BLINK) && BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_APPLE)
-
   static ReadOnlySharedMemoryRegion DuplicateFieldTrialSharedMemoryForTesting();
 
   // Create a FieldTrial with the given |name| and using 100% probability for
@@ -633,8 +640,8 @@ class BASE_EXPORT FieldTrialList {
   // Visibility of field trials with |is_low_anonymity| set to true is
   // restricted to specific callers only, see
   // |FieldTrialListIncludingLowAnonymity|.
-  static FieldTrial* CreateFieldTrial(StringPiece name,
-                                      StringPiece group_name,
+  static FieldTrial* CreateFieldTrial(std::string_view name,
+                                      std::string_view group_name,
                                       bool is_low_anonymity = false,
                                       bool is_overridden = false);
 
@@ -705,7 +712,7 @@ class BASE_EXPORT FieldTrialList {
   static void RestoreInstanceForTesting(FieldTrialList* instance);
 
   // Creates a list of FieldTrial::State for all FieldTrial instances.
-  // StringPiece members are bound to the lifetime of the corresponding
+  // std::string_view members are bound to the lifetime of the corresponding
   // FieldTrial.
   static std::vector<FieldTrial::State> GetAllFieldTrialStates(
       PassKey<test::ScopedFeatureList>);
@@ -757,38 +764,12 @@ class BASE_EXPORT FieldTrialList {
       const ReadOnlySharedMemoryRegion& shm,
       LaunchOptions* launch_options);
 
-  // Indicates failure modes of deserializing the shared memory handle.
-  enum class SharedMemError {
-    kNoError,
-    kUnexpectedTokensCount,
-    kParseInt0Failed,
-    kParseInt4Failed,
-    kUnexpectedHandleType,
-    kInvalidHandle,
-    kGetFDFailed,
-    kDeserializeGUIDFailed,
-    kDeserializeFailed,
-    kCreateTrialsFailed,
-  };
-
-  // Deserialization instantiates the shared memory region for FieldTrials from
-  // the serialized information contained in |switch_value|. Returns an invalid
-  // ReadOnlySharedMemoryRegion on failure.
-  // |fd_key| is used on non-Mac POSIX platforms as the file descriptor passed
-  // down to the child process for the shared memory region.
-  static expected<ReadOnlySharedMemoryRegion, SharedMemError>
-  DeserializeSharedMemoryRegionMetadata(const std::string& switch_value,
-                                        uint32_t fd_key);
-
   // Takes in |handle_switch| from the command line which represents the shared
   // memory handle for field trials, parses it, and creates the field trials.
   // Returns true on success, false on failure.
   // |switch_value| also contains the serialized GUID.
-  // |fd_key| is used on non-Mac POSIX platforms as the file descriptor passed
-  // down to the child process for the shared memory region.
-  static SharedMemError CreateTrialsFromSwitchValue(
-      const std::string& switch_value,
-      uint32_t fd_key);
+  static base::shared_memory::SharedMemoryError CreateTrialsFromSwitchValue(
+      const std::string& switch_value);
 #endif  // BUILDFLAG(USE_BLINK)
 
   // Takes an unmapped ReadOnlySharedMemoryRegion, maps it with the correct size
@@ -822,7 +803,8 @@ class BASE_EXPORT FieldTrialList {
   typedef std::map<std::string, FieldTrial*, std::less<>> RegistrationMap;
 
   // Helper function should be called only while holding lock_.
-  FieldTrial* PreLockedFind(StringPiece name) EXCLUSIVE_LOCKS_REQUIRED(lock_);
+  FieldTrial* PreLockedFind(std::string_view name)
+      EXCLUSIVE_LOCKS_REQUIRED(lock_);
 
   // Register() stores a pointer to the given trial in a global map.
   // This method also AddRef's the indicated trial.
