@@ -70,6 +70,7 @@
 #include <map>
 #include <memory>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include "base/base_export.h"
@@ -81,7 +82,6 @@
 #include "base/metrics/bucket_ranges.h"
 #include "base/metrics/histogram_base.h"
 #include "base/metrics/histogram_samples.h"
-#include "base/strings/string_piece.h"
 #include "base/time/time.h"
 #include "base/values.h"
 
@@ -123,6 +123,25 @@ class BASE_EXPORT Histogram : public HistogramBase {
   // buckets <= (maximum - minimum + 2) - this is to ensure that we don't have
   // more buckets than the range of numbers; having more buckets than 1 per
   // value in the range would be nonsensical.
+  static HistogramBase* FactoryGet(std::string_view name,
+                                   Sample minimum,
+                                   Sample maximum,
+                                   size_t bucket_count,
+                                   int32_t flags);
+  static HistogramBase* FactoryTimeGet(std::string_view name,
+                                       base::TimeDelta minimum,
+                                       base::TimeDelta maximum,
+                                       size_t bucket_count,
+                                       int32_t flags);
+  static HistogramBase* FactoryMicrosecondsTimeGet(std::string_view name,
+                                                   base::TimeDelta minimum,
+                                                   base::TimeDelta maximum,
+                                                   size_t bucket_count,
+                                                   int32_t flags);
+
+  // Overloads of the above functions that take a const std::string& or const
+  // char* |name| param, to avoid code bloat from the std::string constructor
+  // being inlined into call sites.
   static HistogramBase* FactoryGet(const std::string& name,
                                    Sample minimum,
                                    Sample maximum,
@@ -139,9 +158,6 @@ class BASE_EXPORT Histogram : public HistogramBase {
                                                    size_t bucket_count,
                                                    int32_t flags);
 
-  // Overloads of the above functions that take a const char* |name| param, to
-  // avoid code bloat from the std::string constructor being inlined into call
-  // sites.
   static HistogramBase* FactoryGet(const char* name,
                                    Sample minimum,
                                    Sample maximum,
@@ -202,7 +218,7 @@ class BASE_EXPORT Histogram : public HistogramBase {
   // function on non-dcheck builds without crashing.
   // Note. Currently it allow some bad input, e.g. 0 as minimum, but silently
   // converts it to good input: 1.
-  static bool InspectConstructionArguments(StringPiece name,
+  static bool InspectConstructionArguments(std::string_view name,
                                            Sample* minimum,
                                            Sample* maximum,
                                            size_t* bucket_count);
@@ -329,6 +345,20 @@ class BASE_EXPORT LinearHistogram : public Histogram {
 
   /* minimum should start from 1. 0 is as minimum is invalid. 0 is an implicit
      default underflow bucket. */
+  static HistogramBase* FactoryGet(std::string_view name,
+                                   Sample minimum,
+                                   Sample maximum,
+                                   size_t bucket_count,
+                                   int32_t flags);
+  static HistogramBase* FactoryTimeGet(std::string_view name,
+                                       TimeDelta minimum,
+                                       TimeDelta maximum,
+                                       size_t bucket_count,
+                                       int32_t flags);
+
+  // Overloads of the above two functions that take a const std::string& or
+  // const char* |name| param, to avoid code bloat from the std::string
+  // constructor being inlined into call sites.
   static HistogramBase* FactoryGet(const std::string& name,
                                    Sample minimum,
                                    Sample maximum,
@@ -340,9 +370,6 @@ class BASE_EXPORT LinearHistogram : public Histogram {
                                        size_t bucket_count,
                                        int32_t flags);
 
-  // Overloads of the above two functions that take a const char* |name| param,
-  // to avoid code bloat from the std::string constructor being inlined into
-  // call sites.
   static HistogramBase* FactoryGet(const char* name,
                                    Sample minimum,
                                    Sample maximum,
@@ -443,6 +470,16 @@ class BASE_EXPORT ScaledLinearHistogram {
  public:
   // Currently only works with "exact" linear histograms: minimum=1, maximum=N,
   // and bucket_count=N+1.
+  ScaledLinearHistogram(std::string_view name,
+                        Sample minimum,
+                        Sample maximum,
+                        size_t bucket_count,
+                        int32_t scale,
+                        int32_t flags);
+
+  // Overload of the above function that take a const std::string& or const
+  // char* |name| param, to avoid code bloat from the std::string constructor
+  // being inlined into call sites.
   ScaledLinearHistogram(const char* name,
                         Sample minimum,
                         Sample maximum,
@@ -490,11 +527,12 @@ class BASE_EXPORT ScaledLinearHistogram {
 // BooleanHistogram is a histogram for booleans.
 class BASE_EXPORT BooleanHistogram : public LinearHistogram {
  public:
-  static HistogramBase* FactoryGet(const std::string& name, int32_t flags);
+  static HistogramBase* FactoryGet(std::string_view name, int32_t flags);
 
-  // Overload of the above function that takes a const char* |name| param,
-  // to avoid code bloat from the std::string constructor being inlined into
-  // call sites.
+  // Overload of the above function that take a const std::string& or const
+  // char* |name| param, to avoid code bloat from the std::string constructor
+  // being inlined into call sites.
+  static HistogramBase* FactoryGet(const std::string& name, int32_t flags);
   static HistogramBase* FactoryGet(const char* name, int32_t flags);
 
   BooleanHistogram(const BooleanHistogram&) = delete;
@@ -540,13 +578,16 @@ class BASE_EXPORT CustomHistogram : public Histogram {
   // > 0 and < kSampleType_MAX. (Currently 0 is still accepted for backward
   // compatibility). The limits can be unordered or contain duplication, but
   // client should not depend on this.
-  static HistogramBase* FactoryGet(const std::string& name,
+  static HistogramBase* FactoryGet(std::string_view name,
                                    const std::vector<Sample>& custom_ranges,
                                    int32_t flags);
 
-  // Overload of the above function that takes a const char* |name| param,
-  // to avoid code bloat from the std::string constructor being inlined into
-  // call sites.
+  // Overload of the above function that take a const std::string& or const
+  // char* |name| param, to avoid code bloat from the std::string constructor
+  // being inlined into call sites.
+  static HistogramBase* FactoryGet(const std::string& name,
+                                   const std::vector<Sample>& custom_ranges,
+                                   int32_t flags);
   static HistogramBase* FactoryGet(const char* name,
                                    const std::vector<Sample>& custom_ranges,
                                    int32_t flags);
@@ -602,6 +643,28 @@ class BASE_EXPORT CustomHistogram : public Histogram {
   static bool ValidateCustomRanges(const std::vector<Sample>& custom_ranges);
 };
 
+namespace internal {
+
+// Controls whether invocations of UMA_HISTOGRAM_SPLIT_BY_PROCESS_PRIORITY in
+// this process log to their ".BestEffort" suffix or not. Timing metrics
+// reported through UMA_HISTOGRAM_SPLIT_BY_PROCESS_PRIORITY which overlap a
+// best-effort range will be suffixed with ".BestEffort".
+BASE_EXPORT void SetSharedLastForegroundTimeForMetrics(
+    const std::atomic<TimeTicks>* last_foreground_time_ref);
+
+// Reports whether the interval [`now - range`, `now`] overlaps with a period
+// where this process was running at Process::Priority::kBestEffort. Defaults to
+// false if `last_foreground_time_ref` was never set (e.g. in processes not
+// affected by priorities) but otherwise defaults to true if there's ambiguity
+// (might have overlapped a best-effort range; as the reported timing might have
+// been affected and shouldn't be reported as "definitely measured in
+// foreground").
+// This method is atomic and suitable for performance critical histogram
+// samples.
+BASE_EXPORT bool OverlapsBestEffortRange(TimeTicks now, TimeDelta range);
+
+}  // namespace internal
+
 }  // namespace base
 
 #endif  // BASE_METRICS_HISTOGRAM_H_
