@@ -7,18 +7,23 @@
 
 #include <stdint.h>
 
+#include <string_view>
+
 #include "base/base_export.h"
-#include "base/strings/string_piece.h"
 
 namespace base {
 
 // Computes a uint64_t hash of a given string based on its MD5 hash. Suitable
 // for metric names.
-BASE_EXPORT uint64_t HashMetricName(base::StringPiece name);
+BASE_EXPORT uint64_t HashMetricName(std::string_view name);
 
 // Computes a uint32_t hash of a given string based on its MD5 hash. This
 // can be more suitable for contexts where memory use is a concern.
-BASE_EXPORT uint32_t HashMetricNameAs32Bits(base::StringPiece name);
+BASE_EXPORT uint32_t HashMetricNameAs32Bits(std::string_view name);
+
+// Computes a uint32_t hash of a given string based on its SHA1 hash. Suitable
+// for uniquely identifying field trial names and group names.
+BASE_EXPORT uint32_t HashFieldTrialName(std::string_view name);
 
 }  // namespace base
 
