@@ -29,7 +29,7 @@ class BASE_EXPORT SparseHistogram : public HistogramBase {
  public:
   // If there's one with same name, return the existing one. If not, create a
   // new one.
-  static HistogramBase* FactoryGet(const std::string& name, int32_t flags);
+  static HistogramBase* FactoryGet(std::string_view name, int32_t flags);
 
   // Create a histogram using data in persistent storage. The allocator must
   // live longer than the created sparse histogram.
@@ -44,7 +44,7 @@ class BASE_EXPORT SparseHistogram : public HistogramBase {
 
   ~SparseHistogram() override;
 
-  // HistogramBase implementation:
+  // HistogramBase:
   uint64_t name_hash() const override;
   HistogramType GetHistogramType() const override;
   bool HasConstructionArguments(Sample expected_minimum,
@@ -62,7 +62,7 @@ class BASE_EXPORT SparseHistogram : public HistogramBase {
   base::Value::Dict ToGraphDict() const override;
 
  protected:
-  // HistogramBase implementation:
+  // HistogramBase:
   void SerializeInfoImpl(base::Pickle* pickle) const override;
 
  private:
