@@ -15,6 +15,7 @@
 #include <atomic>  // For std::memory_order_*.
 #include <memory>
 #include <string>
+#include <string_view>
 #include <unordered_map>
 #include <vector>
 
@@ -28,7 +29,6 @@
 #include "base/metrics/ranges_manager.h"
 #include "base/metrics/record_histogram_checker.h"
 #include "base/observer_list_threadsafe.h"
-#include "base/strings/string_piece.h"
 #include "base/synchronization/lock.h"
 #include "base/thread_annotations.h"
 #include "base/types/pass_key.h"
@@ -83,7 +83,8 @@ class BASE_EXPORT StatisticsRecorder {
   // name and the callback to be invoked. The class starts observing on
   // construction and removes itself from the observer list on destruction. The
   // clients are always notified on the same sequence in which they were
-  // registered.
+  // registered. This will not get a notification if created while sending out
+  // that notification.
   class BASE_EXPORT ScopedHistogramSampleObserver {
    public:
     // Constructor. Called with the desired histogram name and the callback to
@@ -184,7 +185,7 @@ class BASE_EXPORT StatisticsRecorder {
   // if a matching histogram is not found.
   //
   // This method is thread safe.
-  static HistogramBase* FindHistogram(base::StringPiece name);
+  static HistogramBase* FindHistogram(std::string_view name);
 
   // Imports histograms from providers. If |async| is true, the providers may do
   // the work asynchronously (though this is not guaranteed and it is up to the
@@ -259,7 +260,7 @@ class BASE_EXPORT StatisticsRecorder {
   // memory is being released.
   //
   // This method is thread safe.
-  static void ForgetHistogramForTesting(base::StringPiece name);
+  static void ForgetHistogramForTesting(std::string_view name);
 
   // Creates a temporary StatisticsRecorder object for testing purposes. All new
   // histograms will be registered in it until it is destructed or pushed aside
@@ -330,12 +331,13 @@ class BASE_EXPORT StatisticsRecorder {
   // Note: |name| is only used in DCHECK builds to assert that there was no
   // collision (i.e. different histograms with the same hash).
   HistogramBase* FindHistogramByHashInternal(uint64_t hash,
-                                             StringPiece name) const
+                                             std::string_view name) const
       EXCLUSIVE_LOCKS_REQUIRED(GetLock());
 
-  // Adds an observer to be notified when a new sample is recorded on the
-  // histogram referred to by |histogram_name|. Can be called before or after
-  // the histogram is created.
+  // Adds an observer to be notified when a new sample is recorded on
+  // the histogram referred to by |histogram_name|. Observers added
+  // while sending out notification are not notified. Can be called
+  // before or after the histogram is created.
   //
   // This method is thread safe.
   static void AddHistogramSampleObserver(
