@@ -20,12 +20,14 @@ void PortProvider::RemoveObserver(Observer* observer) {
   observer_list_->RemoveObserver(observer);
 }
 
-void PortProvider::NotifyObservers(ProcessHandle process) {
-  observer_list_->Notify(FROM_HERE, &Observer::OnReceivedTaskPort, process);
+void PortProvider::NotifyObservers(ProcessHandle process_handle) {
+  observer_list_->Notify(FROM_HERE, &Observer::OnReceivedTaskPort,
+                         process_handle);
 }
 
-mach_port_t SelfPortProvider::TaskForPid(base::ProcessHandle process) const {
-  DCHECK(base::Process(process).is_current());
+mach_port_t SelfPortProvider::TaskForHandle(
+    base::ProcessHandle process_handle) const {
+  DCHECK(base::Process(process_handle).is_current());
   return mach_task_self();
 }
 
