@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/process/process_metrics.h"
 
 #include <dirent.h>
@@ -10,32 +15,35 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <sys/stat.h>
+#include <sys/sysmacros.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <optional>
+#include <string_view>
 #include <utility>
 
+#include "base/containers/span.h"
 #include "base/cpu.h"
 #include "base/files/dir_reader_posix.h"
 #include "base/files/file_util.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/notreached.h"
+#include "base/numerics/clamped_math.h"
 #include "base/numerics/safe_conversions.h"
 #include "base/process/internal_linux.h"
-#include "base/process/process_metrics_iocounters.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_tokenizer.h"
 #include "base/strings/string_util.h"
 #include "base/system/sys_info.h"
 #include "base/threading/thread_restrictions.h"
+#include "base/types/expected.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "third_party/abseil-cpp/absl/strings/ascii.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace base {
 
@@ -57,24 +65,27 @@ uint64_t ReadFileToUint64(const FilePath& file) {
 }
 #endif
 
-// Get the total CPU from a proc stat buffer.  Return value is number of jiffies
-// on success or 0 if parsing failed.
-int64_t ParseTotalCPUTimeFromStats(const std::vector<std::string>& proc_stats) {
-  return internal::GetProcStatsFieldAsInt64(proc_stats, internal::VM_UTIME) +
-         internal::GetProcStatsFieldAsInt64(proc_stats, internal::VM_STIME);
-}
-
-// Get the total CPU of a single process.  Return value is number of jiffies
-// on success or -1 on error.
-int64_t GetProcessCPU(pid_t pid) {
-  std::string buffer;
-  std::vector<std::string> proc_stats;
-  if (!internal::ReadProcStats(pid, &buffer) ||
-      !internal::ParseProcStats(buffer, &proc_stats)) {
-    return -1;
+// Get the total CPU from a proc stat buffer. Return value is a TimeDelta
+// converted from a number of jiffies on success or an error code if parsing
+// failed.
+base::expected<TimeDelta, ProcessCPUUsageError> ParseTotalCPUTimeFromStats(
+    base::span<const std::string> proc_stats) {
+  const std::optional<int64_t> utime =
+      internal::GetProcStatsFieldAsOptionalInt64(proc_stats,
+                                                 internal::VM_UTIME);
+  if (utime.value_or(-1) < 0) {
+    return base::unexpected(ProcessCPUUsageError::kSystemError);
   }
-
-  return ParseTotalCPUTimeFromStats(proc_stats);
+  const std::optional<int64_t> stime =
+      internal::GetProcStatsFieldAsOptionalInt64(proc_stats,
+                                                 internal::VM_STIME);
+  if (stime.value_or(-1) < 0) {
+    return base::unexpected(ProcessCPUUsageError::kSystemError);
+  }
+  const TimeDelta cpu_time = internal::ClockTicksToTimeDelta(
+      base::ClampAdd(utime.value(), stime.value()));
+  CHECK(!cpu_time.is_negative());
+  return base::ok(cpu_time);
 }
 
 }  // namespace
@@ -90,8 +101,16 @@ size_t ProcessMetrics::GetResidentSetSize() const {
          checked_cast<size_t>(getpagesize());
 }
 
-TimeDelta ProcessMetrics::GetCumulativeCPUUsage() {
-  return internal::ClockTicksToTimeDelta(GetProcessCPU(process_));
+base::expected<TimeDelta, ProcessCPUUsageError>
+ProcessMetrics::GetCumulativeCPUUsage() {
+  std::string buffer;
+  std::vector<std::string> proc_stats;
+  if (!internal::ReadProcStats(process_, &buffer) ||
+      !internal::ParseProcStats(buffer, &proc_stats)) {
+    return base::unexpected(ProcessCPUUsageError::kSystemError);
+  }
+
+  return ParseTotalCPUTimeFromStats(proc_stats);
 }
 
 bool ProcessMetrics::GetCumulativeCPUUsagePerThread(
@@ -110,45 +129,16 @@ bool ProcessMetrics::GetCumulativeCPUUsagePerThread(
           return;
         }
 
-        TimeDelta thread_time = internal::ClockTicksToTimeDelta(
-            ParseTotalCPUTimeFromStats(proc_stats));
-        cpu_per_thread.emplace_back(tid, thread_time);
+        const base::expected<TimeDelta, ProcessCPUUsageError> thread_time =
+            ParseTotalCPUTimeFromStats(proc_stats);
+        if (thread_time.has_value()) {
+          cpu_per_thread.emplace_back(tid, thread_time.value());
+        }
       });
 
   return !cpu_per_thread.empty();
 }
 
-// For the /proc/self/io file to exist, the Linux kernel must have
-// CONFIG_TASK_IO_ACCOUNTING enabled.
-bool ProcessMetrics::GetIOCounters(IoCounters* io_counters) const {
-  StringPairs pairs;
-  if (!internal::ReadProcFileToTrimmedStringPairs(process_, "io", &pairs)) {
-    return false;
-  }
-
-  io_counters->OtherOperationCount = 0;
-  io_counters->OtherTransferCount = 0;
-
-  for (const auto& pair : pairs) {
-    const std::string& key = pair.first;
-    const std::string& value_str = pair.second;
-    uint64_t* target_counter = nullptr;
-    if (key == "syscr")
-      target_counter = &io_counters->ReadOperationCount;
-    else if (key == "syscw")
-      target_counter = &io_counters->WriteOperationCount;
-    else if (key == "rchar")
-      target_counter = &io_counters->ReadTransferCount;
-    else if (key == "wchar")
-      target_counter = &io_counters->WriteTransferCount;
-    if (!target_counter)
-      continue;
-    bool converted = StringToUint64(value_str, target_counter);
-    DCHECK(converted);
-  }
-  return true;
-}
-
 #if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
 uint64_t ProcessMetrics::GetVmSwapBytes() const {
   return internal::ReadProcStatusAndGetKbFieldAsSizeT(process_, "VmSwap") *
@@ -228,11 +218,19 @@ size_t GetSystemCommitCharge() {
   SystemMemoryInfoKB meminfo;
   if (!GetSystemMemoryInfo(&meminfo))
     return 0;
-  return checked_cast<size_t>(meminfo.total - meminfo.free - meminfo.buffers -
-                              meminfo.cached);
+  return GetSystemCommitChargeFromMeminfo(meminfo);
 }
 
-int ParseProcStatCPU(StringPiece input) {
+size_t GetSystemCommitChargeFromMeminfo(const SystemMemoryInfoKB& meminfo) {
+  // TODO(crbug.com/315988925): This math is incorrect: `cached` can be very
+  // large so that `free` + `buffers` + `cached` > `total`. Replace this with a
+  // more meaningful metric or remove it. In the meantime, convert underflows to
+  // 0 instead of crashing.
+  return ClampedNumeric<size_t>(meminfo.total) - meminfo.free -
+         meminfo.buffers - meminfo.cached;
+}
+
+int ParseProcStatCPU(std::string_view input) {
   // |input| may be empty if the process disappeared somehow.
   // e.g. http://crbug.com/145811.
   if (input.empty())
@@ -350,7 +348,8 @@ Value::Dict SystemMemoryInfoKB::ToDict() const {
   return res;
 }
 
-bool ParseProcMeminfo(StringPiece meminfo_data, SystemMemoryInfoKB* meminfo) {
+bool ParseProcMeminfo(std::string_view meminfo_data,
+                      SystemMemoryInfoKB* meminfo) {
   // The format of /proc/meminfo is:
   //
   // MemTotal:      8235324 kB
@@ -365,9 +364,9 @@ bool ParseProcMeminfo(StringPiece meminfo_data, SystemMemoryInfoKB* meminfo) {
   // least non-zero. So start off with a zero total.
   meminfo->total = 0;
 
-  for (const StringPiece& line : SplitStringPiece(
+  for (std::string_view line : SplitStringPiece(
            meminfo_data, "\n", KEEP_WHITESPACE, SPLIT_WANT_NONEMPTY)) {
-    std::vector<StringPiece> tokens = SplitStringPiece(
+    std::vector<std::string_view> tokens = SplitStringPiece(
         line, kWhitespaceASCII, TRIM_WHITESPACE, SPLIT_WANT_NONEMPTY);
     // HugePages_* only has a number and no suffix so there may not be exactly 3
     // tokens.
@@ -420,7 +419,7 @@ bool ParseProcMeminfo(StringPiece meminfo_data, SystemMemoryInfoKB* meminfo) {
   return meminfo->total > 0;
 }
 
-bool ParseProcVmstat(StringPiece vmstat_data, VmStatInfo* vmstat) {
+bool ParseProcVmstat(std::string_view vmstat_data, VmStatInfo* vmstat) {
   // The format of /proc/vmstat is:
   //
   // nr_free_pages 299878
@@ -442,10 +441,10 @@ bool ParseProcVmstat(StringPiece vmstat_data, VmStatInfo* vmstat) {
   bool has_oom_kill = false;
   vmstat->oom_kill = 0;
 
-  for (const StringPiece& line : SplitStringPiece(
+  for (std::string_view line : SplitStringPiece(
            vmstat_data, "\n", KEEP_WHITESPACE, SPLIT_WANT_NONEMPTY)) {
-    std::vector<StringPiece> tokens = SplitStringPiece(
-        line, " ", KEEP_WHITESPACE, SPLIT_WANT_NONEMPTY);
+    std::vector<std::string_view> tokens =
+        SplitStringPiece(line, " ", KEEP_WHITESPACE, SPLIT_WANT_NONEMPTY);
     if (tokens.size() != 2)
       continue;
 
@@ -496,7 +495,7 @@ bool GetSystemMemoryInfo(SystemMemoryInfoKB* meminfo) {
 
 Value::Dict VmStatInfo::ToDict() const {
   Value::Dict res;
-  // TODO(crbug.com/1334256): Make base::Value able to hold uint64_t and remove
+  // TODO(crbug.com/40228085): Make base::Value able to hold uint64_t and remove
   // casts below.
   res.Set("pswpin", static_cast<int>(pswpin));
   res.Set("pswpout", static_cast<int>(pswpout));
@@ -559,7 +558,7 @@ Value::Dict SystemDiskInfo::ToDict() const {
   return res;
 }
 
-bool IsValidDiskName(StringPiece candidate) {
+bool IsValidDiskName(std::string_view candidate) {
   if (candidate.length() < 3)
     return false;
 
@@ -598,7 +597,7 @@ bool GetSystemDiskInfo(SystemDiskInfo* diskinfo) {
     return false;
   }
 
-  std::vector<StringPiece> diskinfo_lines = SplitStringPiece(
+  std::vector<std::string_view> diskinfo_lines = SplitStringPiece(
       diskinfo_data, "\n", KEEP_WHITESPACE, SPLIT_WANT_NONEMPTY);
   if (diskinfo_lines.empty()) {
     DLOG(WARNING) << "No lines found";
@@ -629,8 +628,8 @@ bool GetSystemDiskInfo(SystemDiskInfo* diskinfo) {
   uint64_t io_time = 0;
   uint64_t weighted_io_time = 0;
 
-  for (const StringPiece& line : diskinfo_lines) {
-    std::vector<StringPiece> disk_fields = SplitStringPiece(
+  for (std::string_view line : diskinfo_lines) {
+    std::vector<std::string_view> disk_fields = SplitStringPiece(
         line, kWhitespaceASCII, TRIM_WHITESPACE, SPLIT_WANT_NONEMPTY);
 
     // Fields may have overflowed and reset to zero.
@@ -697,7 +696,7 @@ Value::Dict GraphicsMemoryInfoKB::ToDict() const {
   return res;
 }
 
-bool ParseZramMmStat(StringPiece mm_stat_data, SwapInfo* swap_info) {
+bool ParseZramMmStat(std::string_view mm_stat_data, SwapInfo* swap_info) {
   // There are 7 columns in /sys/block/zram0/mm_stat,
   // split by several spaces. The first three columns
   // are orig_data_size, compr_data_size and mem_used_total.
@@ -707,7 +706,7 @@ bool ParseZramMmStat(StringPiece mm_stat_data, SwapInfo* swap_info) {
   // For more details:
   // https://www.kernel.org/doc/Documentation/blockdev/zram.txt
 
-  std::vector<StringPiece> tokens = SplitStringPiece(
+  std::vector<std::string_view> tokens = SplitStringPiece(
       mm_stat_data, kWhitespaceASCII, TRIM_WHITESPACE, SPLIT_WANT_NONEMPTY);
   if (tokens.size() < 7) {
     DLOG(WARNING) << "zram mm_stat: tokens: " << tokens.size()
@@ -725,7 +724,7 @@ bool ParseZramMmStat(StringPiece mm_stat_data, SwapInfo* swap_info) {
   return true;
 }
 
-bool ParseZramStat(StringPiece stat_data, SwapInfo* swap_info) {
+bool ParseZramStat(std::string_view stat_data, SwapInfo* swap_info) {
   // There are 11 columns in /sys/block/zram0/stat,
   // split by several spaces. The first column is read I/Os
   // and fifth column is write I/Os.
@@ -735,7 +734,7 @@ bool ParseZramStat(StringPiece stat_data, SwapInfo* swap_info) {
   // For more details:
   // https://www.kernel.org/doc/Documentation/blockdev/zram.txt
 
-  std::vector<StringPiece> tokens = SplitStringPiece(
+  std::vector<std::string_view> tokens = SplitStringPiece(
       stat_data, kWhitespaceASCII, TRIM_WHITESPACE, SPLIT_WANT_NONEMPTY);
   if (tokens.size() < 11) {
     DLOG(WARNING) << "zram stat: tokens: " << tokens.size()
@@ -790,7 +789,7 @@ bool GetSwapInfoImpl(SwapInfo* swap_info) {
   // Since ZRAM update, it shows the usage data in different places.
   // If file "/sys/block/zram0/mm_stat" exists, use the new way, otherwise,
   // use the old way.
-  static absl::optional<bool> use_new_zram_interface;
+  static std::optional<bool> use_new_zram_interface;
   FilePath zram_mm_stat_file("/sys/block/zram0/mm_stat");
   if (!use_new_zram_interface.has_value()) {
     use_new_zram_interface = PathExists(zram_mm_stat_file);
@@ -837,7 +836,155 @@ bool GetSwapInfo(SwapInfo* swap_info) {
   return true;
 }
 
+namespace {
+
+size_t ParseSize(const std::string& value) {
+  size_t pos = value.find(' ');
+  std::string base = value.substr(0, pos);
+  std::string units = value.substr(pos + 1);
+
+  size_t ret = 0;
+
+  base::StringToSizeT(base, &ret);
+
+  if (units == "KiB") {
+    ret *= 1024;
+  } else if (units == "MiB") {
+    ret *= 1024 * 1024;
+  }
+
+  return ret;
+}
+
+struct DrmFdInfo {
+  size_t memory_total;
+  size_t memory_shared;
+};
+
+void GetFdInfoFromPid(pid_t pid,
+                      std::map<unsigned int, struct DrmFdInfo>& fdinfo_table) {
+  const FilePath pid_path =
+      FilePath("/proc").AppendASCII(base::NumberToString(pid));
+  const FilePath fd_path = pid_path.AppendASCII("fd");
+  DirReaderPosix dir_reader(fd_path.value().c_str());
+
+  if (!dir_reader.IsValid()) {
+    return;
+  }
+
+  for (; dir_reader.Next();) {
+    const char* name = dir_reader.name();
+
+    if (strcmp(name, ".") == 0 || strcmp(name, "..") == 0) {
+      continue;
+    }
+
+    struct stat stat;
+    int err = fstatat(dir_reader.fd(), name, &stat, 0);
+    if (err) {
+      continue;
+    }
+
+    /* Skip fd's that are not drm device files: */
+    if (!S_ISCHR(stat.st_mode) || major(stat.st_rdev) != 226) {
+      continue;
+    }
+
+    const FilePath fdinfo_path =
+        pid_path.AppendASCII("fdinfo").AppendASCII(name);
+
+    std::string fdinfo_data;
+    if (!ReadFileToStringNonBlocking(fdinfo_path, &fdinfo_data)) {
+      continue;
+    }
+
+    std::stringstream ss(fdinfo_data);
+    std::string line;
+    struct DrmFdInfo fdinfo = {};
+    unsigned int client_id = 0;
+
+    while (std::getline(ss, line, '\n')) {
+      size_t pos = line.find(':');
+
+      if (pos == std::string::npos) {
+        continue;
+      }
+
+      std::string key = line.substr(0, pos);
+      std::string value = line.substr(pos + 1);
+
+      /* trim leading space from the value: */
+      value = value.substr(value.find_first_not_of(" \t"));
+
+      if (key == "drm-client-id") {
+        base::StringToUint(value, &client_id);
+      } else if (key == "drm-total-memory") {
+        fdinfo.memory_total = ParseSize(value);
+      } else if (key == "drm-shared-memory") {
+        fdinfo.memory_shared = ParseSize(value);
+      }
+    }
+
+    /* The compositor only imports buffers.. so shared==total.  Skip this
+     * as it is not interesting:
+     */
+    if (client_id && fdinfo.memory_shared != fdinfo.memory_total) {
+      fdinfo_table[client_id] = fdinfo;
+    }
+  }
+}
+
+bool GetGraphicsMemoryInfoFdInfo(GraphicsMemoryInfoKB* gpu_meminfo) {
+  // First parse clients file to get the tgid's of processes using the GPU
+  // so that we don't need to parse *all* processes:
+  const FilePath clients_path("/run/debugfs_gpu/clients");
+  std::string clients_data;
+  std::map<unsigned int, struct DrmFdInfo> fdinfo_table;
+
+  if (!ReadFileToStringNonBlocking(clients_path, &clients_data)) {
+    return false;
+  }
+
+  // This has been the format since kernel commit:
+  // 50d47cb318ed ("drm: Include task->name and master status in debugfs clients
+  // info")
+  //
+  // comm pid dev  master auth uid magic
+  // %20s %5d %3d   %c    %c %5d %10u\n
+  //
+  // In practice comm rarely contains spaces, but it can in fact contain
+  // any character.  So we parse based on the 20 char limit (plus one
+  // space):
+  std::istringstream clients_stream(clients_data);
+  std::string line;
+  while (std::getline(clients_stream, line)) {
+    pid_t pid;
+    int num_res = sscanf(&line.c_str()[21], "%5d", &pid);
+    if (num_res == 1) {
+      GetFdInfoFromPid(pid, fdinfo_table);
+    }
+  }
+
+  if (fdinfo_table.size() == 0) {
+    return false;
+  }
+
+  gpu_meminfo->gpu_memory_size = 0;
+
+  for (auto const& p : fdinfo_table) {
+    gpu_meminfo->gpu_memory_size += p.second.memory_total;
+    /* TODO it would be nice to also be able to report shared */
+  }
+
+  return true;
+}
+
+}  // namespace
+
 bool GetGraphicsMemoryInfo(GraphicsMemoryInfoKB* gpu_meminfo) {
+  if (GetGraphicsMemoryInfoFdInfo(gpu_meminfo)) {
+    return true;
+  }
 #if defined(ARCH_CPU_X86_FAMILY)
   // Reading i915_gem_objects on intel platform with kernel 5.4 is slow and is
   // prohibited.
