@@ -6,6 +6,7 @@
 
 #include <memory>
 #include <string>
+#include <string_view>
 #include <utility>
 
 #include "base/at_exit.h"
@@ -33,15 +34,19 @@
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/test/scoped_feature_list.h"
 #include "base/test/task_environment.h"
 #include "base/time/time.h"
 #endif  // BUILDFLAG(IS_CHROMEOS)
 
-#if BUILDFLAG(IS_WIN)
-#include "base/win/base_win_buildflags.h"
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_WIN)
+#include "base/test/scoped_feature_list.h"
+#endif
 
+#if BUILDFLAG(IS_WIN)
 #include <windows.h>
+
+#include "base/win/base_win_buildflags.h"
+#include "base/win/windows_version.h"
 #endif
 
 namespace {
@@ -58,7 +63,7 @@ constexpr int kDummyExitCode = 42;
 // Fake port provider that returns the calling process's
 // task port, ignoring its argument.
 class FakePortProvider : public base::PortProvider {
-  mach_port_t TaskForPid(base::ProcessHandle process) const override {
+  mach_port_t TaskForHandle(base::ProcessHandle process_handle) const override {
     return mach_task_self();
   }
 };
@@ -78,10 +83,10 @@ std::string GetProcessCpuCgroup(const base::Process& process) {
     return std::string();
   }
 
-  std::vector<base::StringPiece> lines = SplitStringPiece(
+  std::vector<std::string_view> lines = SplitStringPiece(
       proc, "\n", base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
   for (const auto& line : lines) {
-    std::vector<base::StringPiece> fields = SplitStringPiece(
+    std::vector<std::string_view> fields = SplitStringPiece(
         line, ":", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
     if (fields.size() != 3U) {
       continue;
@@ -182,7 +187,7 @@ MULTIPROCESS_TEST_MAIN(SleepyChildProcess) {
   return 0;
 }
 
-// TODO(https://crbug.com/726484): Enable these tests on Fuchsia when
+// TODO(crbug.com/42050607): Enable these tests on Fuchsia when
 // CreationTime() is implemented.
 TEST_F(ProcessTest, CreationTimeCurrentProcess) {
   // The current process creation time should be less than or equal to the
@@ -376,6 +381,37 @@ TEST_F(ProcessTest, SetProcessPriority) {
   EXPECT_TRUE(process.SetPriority(base::Process::Priority::kUserBlocking));
   EXPECT_EQ(process.GetPriority(), Process::Priority::kUserBlocking);
 #endif
+
+#if BUILDFLAG(IS_WIN)
+  EXPECT_TRUE(process.SetPriority(base::Process::Priority::kUserVisible));
+  // Eco QoS level read & write are not supported prior to WIN11_22H2,
+  // Priority::kUserVisible has same behavior as Priority::kUserBlocking, and
+  // is translated as Priority::kUserBlocking.
+  if (base::win::OSInfo::GetInstance()->version() >=
+      base::win::Version::WIN11_22H2) {
+    EXPECT_EQ(process.GetPriority(), Process::Priority::kUserVisible);
+  } else {
+    EXPECT_EQ(process.GetPriority(), Process::Priority::kUserBlocking);
+  }
+
+  EXPECT_TRUE(process.SetPriority(base::Process::Priority::kBestEffort));
+  EXPECT_EQ(process.GetPriority(), Process::Priority::kBestEffort);
+  EXPECT_TRUE(process.SetPriority(base::Process::Priority::kUserVisible));
+  if (base::win::OSInfo::GetInstance()->version() >=
+      base::win::Version::WIN11_22H2) {
+    EXPECT_EQ(process.GetPriority(), Process::Priority::kUserVisible);
+  } else {
+    EXPECT_EQ(process.GetPriority(), Process::Priority::kUserBlocking);
+  }
+  EXPECT_TRUE(process.SetPriority(base::Process::Priority::kUserBlocking));
+  EXPECT_EQ(process.GetPriority(), Process::Priority::kUserBlocking);
+#elif !BUILDFLAG(IS_APPLE)
+  // On other platforms, Process::Priority::kUserVisible is translated as
+  // Process::Priority::kUserBlocking.
+  EXPECT_TRUE(process.SetPriority(base::Process::Priority::kUserVisible));
+  EXPECT_EQ(process.GetPriority(), Process::Priority::kUserBlocking);
+#endif
+
   int new_os_priority = process.GetOSPriority();
   EXPECT_EQ(old_os_priority, new_os_priority);
 }
@@ -427,7 +463,7 @@ class FunctionTestThread : public PlatformThread::Delegate {
   FunctionTestThread& operator=(const FunctionTestThread&) = delete;
 
   void ThreadMain() override {
-    PlatformThread::SetCurrentThreadType(ThreadType::kCompositing);
+    PlatformThread::SetCurrentThreadType(ThreadType::kDisplayCritical);
     while (true) {
       PlatformThread::Sleep(Milliseconds(100));
     }
@@ -459,7 +495,7 @@ class RTDisplayFunctionTestThread : public PlatformThread::Delegate {
       delete;
 
   void ThreadMain() override {
-    PlatformThread::SetCurrentThreadType(ThreadType::kCompositing);
+    PlatformThread::SetCurrentThreadType(ThreadType::kDisplayCritical);
     while (true) {
       PlatformThread::Sleep(Milliseconds(100));
     }
@@ -508,8 +544,8 @@ MULTIPROCESS_TEST_MAIN(ProcessThreadBackgroundingMain) {
   PlatformThreadHandle handle1, handle2, handle3;
   FunctionTestThread thread1, thread2, thread3;
   base::test::ScopedFeatureList scoped_feature_list(kSetThreadBgForBgProcess);
-  PlatformThreadChromeOS::InitFeaturesPostFieldTrial();
-  PlatformThread::SetCurrentThreadType(ThreadType::kCompositing);
+  PlatformThreadChromeOS::InitializeFeatures();
+  PlatformThread::SetCurrentThreadType(ThreadType::kDisplayCritical);
 
   // Register signal handler to be notified to create threads after backgrounding.
   signal(SIGUSR1, sig_create_threads_after_bg);
@@ -550,12 +586,12 @@ MULTIPROCESS_TEST_MAIN(ProcessThreadBackgroundingMain) {
 // that the threads in the process are backgrounded correctly.
 TEST_F(ProcessTest, ProcessThreadBackgrounding) {
   if (!PlatformThread::CanChangeThreadType(ThreadType::kDefault,
-                                           ThreadType::kCompositing)) {
+                                           ThreadType::kDisplayCritical)) {
     return;
   }
 
   base::test::ScopedFeatureList scoped_feature_list(kSetThreadBgForBgProcess);
-  PlatformThreadChromeOS::InitFeaturesPostFieldTrial();
+  PlatformThreadChromeOS::InitializeFeatures();
 
   // Register signal handlers to be notified of events in child process.
   signal(SIGUSR1, sig_prebg_threads_created_handler);
@@ -571,7 +607,7 @@ TEST_F(ProcessTest, ProcessThreadBackgrounding) {
   }
 
   // Verify that the threads are initially in the foreground.
-  AssertThreadsType(process.Pid(), ThreadType::kCompositing);
+  AssertThreadsType(process.Pid(), ThreadType::kDisplayCritical);
   AssertThreadsBgState(process.Pid(), false);
 
   EXPECT_TRUE(process.SetPriority(Process::Priority::kBestEffort));
@@ -585,14 +621,14 @@ TEST_F(ProcessTest, ProcessThreadBackgrounding) {
   }
 
   // Verify that the threads are backgrounded.
-  AssertThreadsType(process.Pid(), ThreadType::kCompositing);
+  AssertThreadsType(process.Pid(), ThreadType::kDisplayCritical);
   AssertThreadsBgState(process.Pid(), true);
 
   EXPECT_TRUE(process.SetPriority(Process::Priority::kUserBlocking));
   EXPECT_TRUE(process.GetPriority() == base::Process::Priority::kUserBlocking);
 
   // Verify that the threads are foregrounded.
-  AssertThreadsType(process.Pid(), ThreadType::kCompositing);
+  AssertThreadsType(process.Pid(), ThreadType::kDisplayCritical);
   AssertThreadsBgState(process.Pid(), false);
 }
 
@@ -600,7 +636,7 @@ MULTIPROCESS_TEST_MAIN(ProcessRTAudioBgMain) {
   PlatformThreadHandle handle1;
   RTAudioFunctionTestThread thread1;
   base::test::ScopedFeatureList scoped_feature_list(kSetThreadBgForBgProcess);
-  PlatformThreadChromeOS::InitFeaturesPostFieldTrial();
+  PlatformThreadChromeOS::InitializeFeatures();
   PlatformThread::SetCurrentThreadType(ThreadType::kRealtimeAudio);
 
   if (!PlatformThread::Create(0, &thread1, &handle1)) {
@@ -619,12 +655,12 @@ MULTIPROCESS_TEST_MAIN(ProcessRTAudioBgMain) {
 // Test the property of kRealTimeAudio threads in a backgrounded process.
 TEST_F(ProcessTest, ProcessRTAudioBg) {
   if (!PlatformThread::CanChangeThreadType(ThreadType::kDefault,
-                                           ThreadType::kCompositing)) {
+                                           ThreadType::kDisplayCritical)) {
     return;
   }
 
   base::test::ScopedFeatureList scoped_feature_list(kSetThreadBgForBgProcess);
-  PlatformThreadChromeOS::InitFeaturesPostFieldTrial();
+  PlatformThreadChromeOS::InitializeFeatures();
 
   // Register signal handler to check if RT thread was created by child process.
   signal(SIGUSR1, sig_audio_rt_threads_created_handler);
@@ -664,9 +700,9 @@ MULTIPROCESS_TEST_MAIN(ProcessRTDisplayBgMain) {
   base::test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatures(
       {kSetThreadBgForBgProcess, kSetRtForDisplayThreads}, {});
-  PlatformThreadChromeOS::InitFeaturesPostFieldTrial();
+  PlatformThreadChromeOS::InitializeFeatures();
 
-  PlatformThread::SetCurrentThreadType(ThreadType::kCompositing);
+  PlatformThread::SetCurrentThreadType(ThreadType::kDisplayCritical);
 
   if (!PlatformThread::Create(0, &thread1, &handle1)) {
     ADD_FAILURE() << "ProcessRTDisplayBgMain: Failed to create thread1";
@@ -681,17 +717,17 @@ MULTIPROCESS_TEST_MAIN(ProcessRTDisplayBgMain) {
   }
 }
 
-// Test the property of kCompositing threads in a backgrounded process.
+// Test the property of kDisplayCritical threads in a backgrounded process.
 TEST_F(ProcessTest, ProcessRTDisplayBg) {
   if (!PlatformThread::CanChangeThreadType(ThreadType::kDefault,
-                                           ThreadType::kCompositing)) {
+                                           ThreadType::kDisplayCritical)) {
     return;
   }
 
   base::test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatures(
       {kSetThreadBgForBgProcess, kSetRtForDisplayThreads}, {});
-  PlatformThreadChromeOS::InitFeaturesPostFieldTrial();
+  PlatformThreadChromeOS::InitializeFeatures();
 
   // Register signal handler to check if RT thread was created by child process.
   signal(SIGUSR1, sig_display_rt_threads_created_handler);
@@ -705,7 +741,7 @@ TEST_F(ProcessTest, ProcessRTDisplayBg) {
   }
 
   AssertThreadsRT(process.Pid(), true);
-  AssertThreadsType(process.Pid(), ThreadType::kCompositing);
+  AssertThreadsType(process.Pid(), ThreadType::kDisplayCritical);
   AssertThreadsBgState(process.Pid(), false);
 
   EXPECT_TRUE(process.SetPriority(Process::Priority::kBestEffort));
@@ -714,7 +750,7 @@ TEST_F(ProcessTest, ProcessRTDisplayBg) {
   // Verify that the threads transitioned away from RT when process is
   // kBestEffort
   AssertThreadsRT(process.Pid(), false);
-  AssertThreadsType(process.Pid(), ThreadType::kCompositing);
+  AssertThreadsType(process.Pid(), ThreadType::kDisplayCritical);
   AssertThreadsBgState(process.Pid(), true);
 
   EXPECT_TRUE(process.SetPriority(Process::Priority::kUserBlocking));
@@ -722,7 +758,7 @@ TEST_F(ProcessTest, ProcessRTDisplayBg) {
 
   // Verify that it is back to RT when process is kUserBlocking
   AssertThreadsRT(process.Pid(), true);
-  AssertThreadsType(process.Pid(), ThreadType::kCompositing);
+  AssertThreadsType(process.Pid(), ThreadType::kDisplayCritical);
   AssertThreadsBgState(process.Pid(), false);
 }
 
