@@ -23,8 +23,4 @@ void RandBytes(span<uint8_t> output) {
   }
 }
 
-void RandBytes(void* output, size_t output_length) {
-  RandBytes(make_span(reinterpret_cast<uint8_t*>(output), output_length));
-}
-
 }  // namespace base
