@@ -13,7 +13,6 @@
 #include "base/allocator/dispatcher/tls.h"
 #include "base/check.h"
 #include "base/compiler_specific.h"
-#include "base/memory/raw_ptr.h"
 #include "base/no_destructor.h"
 #include "base/rand_util.h"
 #include "base/ranges/algorithm.h"
@@ -172,6 +171,14 @@ PoissonAllocationSampler::ScopedMuteHookedSamplesForTesting::
   ResetProfilingStateFlag(ProfilingStateFlag::kHookedSamplesMutedForTesting);
 }
 
+PoissonAllocationSampler::ScopedMuteHookedSamplesForTesting::
+    ScopedMuteHookedSamplesForTesting(ScopedMuteHookedSamplesForTesting&&) =
+        default;
+
+PoissonAllocationSampler::ScopedMuteHookedSamplesForTesting&
+PoissonAllocationSampler::ScopedMuteHookedSamplesForTesting::operator=(
+    ScopedMuteHookedSamplesForTesting&&) = default;
+
 // static
 ABSL_CONST_INIT std::atomic<PoissonAllocationSampler::ProfilingStateFlagMask>
     PoissonAllocationSampler::profiling_state_{0};
@@ -205,7 +212,7 @@ size_t PoissonAllocationSampler::SamplingInterval() const {
 
 // static
 size_t PoissonAllocationSampler::GetNextSampleInterval(size_t interval) {
-  if (UNLIKELY(g_deterministic)) {
+  if (g_deterministic) [[unlikely]] {
     return interval;
   }
 
@@ -226,10 +233,10 @@ size_t PoissonAllocationSampler::GetNextSampleInterval(size_t interval) {
   // huge gaps in the sampling stream. Probability of the upper bound gets hit
   // is exp(-20) ~ 2e-9, so it should not skew the distribution.
   size_t max_value = interval * 20;
-  if (UNLIKELY(value < min_value)) {
+  if (value < min_value) [[unlikely]] {
     return min_value;
   }
-  if (UNLIKELY(value > max_value)) {
+  if (value > max_value) [[unlikely]] {
     return max_value;
   }
   return static_cast<size_t>(value);
@@ -245,11 +252,11 @@ void PoissonAllocationSampler::DoRecordAllocation(
 
   thread_local_data->accumulated_bytes += size;
   intptr_t accumulated_bytes = thread_local_data->accumulated_bytes;
-  if (LIKELY(accumulated_bytes < 0)) {
+  if (accumulated_bytes < 0) [[likely]] {
     return;
   }
 
-  if (UNLIKELY(!(state & ProfilingStateFlag::kIsRunning))) {
+  if (!(state & ProfilingStateFlag::kIsRunning)) [[unlikely]] {
     // Sampling was in fact disabled when the hook was called. Reset the state
     // of the sampler. We do this check off the fast-path, because it's quite a
     // rare state when the sampler is stopped after it's started. (The most
@@ -261,12 +268,12 @@ void PoissonAllocationSampler::DoRecordAllocation(
   }
 
   // Failed allocation? Skip the sample.
-  if (UNLIKELY(!address)) {
+  if (!address) [[unlikely]] {
     return;
   }
 
   size_t mean_interval = g_sampling_interval.load(std::memory_order_relaxed);
-  if (UNLIKELY(!thread_local_data->sampling_interval_initialized)) {
+  if (!thread_local_data->sampling_interval_initialized) [[unlikely]] {
     thread_local_data->sampling_interval_initialized = true;
     // This is the very first allocation on the thread. It always makes it
     // passing the condition at |RecordAlloc|, because accumulated_bytes
@@ -292,12 +299,12 @@ void PoissonAllocationSampler::DoRecordAllocation(
 
   thread_local_data->accumulated_bytes = accumulated_bytes;
 
-  if (UNLIKELY(ScopedMuteThreadSamples::IsMuted())) {
+  if (ScopedMuteThreadSamples::IsMuted()) [[unlikely]] {
     return;
   }
 
   ScopedMuteThreadSamples no_reentrancy_scope;
-  std::vector<raw_ptr<SamplesObserver, VectorExperimental>> observers_copy;
+  std::vector<SamplesObserver*> observers_copy;
   {
     AutoLock lock(mutex_);
 
@@ -324,7 +331,7 @@ void PoissonAllocationSampler::DoRecordFree(void* address) {
   // thus reenter DoRecordAlloc. However the call chain won't build up further
   // as RecordAlloc accesses are guarded with pthread TLS-based ReentryGuard.
   ScopedMuteThreadSamples no_reentrancy_scope;
-  std::vector<raw_ptr<SamplesObserver, VectorExperimental>> observers_copy;
+  std::vector<SamplesObserver*> observers_copy;
   {
     AutoLock lock(mutex_);
     observers_copy = observers_;
@@ -425,7 +432,7 @@ void PoissonAllocationSampler::RemoveSamplesObserver(
   ScopedMuteThreadSamples no_reentrancy_scope;
   AutoLock lock(mutex_);
   auto it = ranges::find(observers_, observer);
-  DCHECK(it != observers_.end());
+  CHECK(it != observers_.end(), base::NotFatalUntil::M125);
   observers_.erase(it);
 
   // Stop the profiler if there are no more observers. Setting/resetting
