@@ -14,6 +14,8 @@
 #include "base/functional/callback_helpers.h"
 #include "base/sequence_checker_impl.h"
 #include "base/sequence_token.h"
+#include "base/synchronization/lock.h"
+#include "base/synchronization/lock_subtle.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/task/thread_pool.h"
 #include "base/test/bind.h"
@@ -322,4 +324,210 @@ TEST(SequenceCheckerTest, FromThreadDestruction) {
   }
 }
 
+// Verifies sequence checking while holding the same locks from different
+// sequences.
+//
+// Note: This is only supported in DCHECK builds.
+#if DCHECK_IS_ON()
+TEST(SequenceCheckerTest, LockBasic) {
+  test::TaskEnvironment task_environment;
+  WaitableEvent thread_pool_done;
+  Lock lock;
+
+  // Create sequence checker while holding lock.
+  ReleasableAutoLock releasable_auto_lock(&lock,
+                                          subtle::LockTracking::kEnabled);
+  SequenceCheckerImpl sequence_checker;
+  releasable_auto_lock.Release();
+
+  ThreadPool::PostTask(BindLambdaForTesting([&]() {
+    // Check sequencing while holding the lock.
+    {
+      AutoLock auto_lock(lock, subtle::LockTracking::kEnabled);
+      EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+    }
+
+    thread_pool_done.Signal();
+  }));
+
+  thread_pool_done.Wait();
+
+  // Check sequencing from the creation sequence, without holding the lock.
+  // Sequencing is *not* valid because sequencing now depends on the lock.
+  EXPECT_FALSE(sequence_checker.CalledOnValidSequence());
+
+  // Check sequencing from the creation sequence while holding the lock.
+  {
+    AutoLock auto_lock(lock, subtle::LockTracking::kEnabled);
+    EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+  }
+}
+
+TEST(SequenceCheckerTest, ManyLocks) {
+  test::TaskEnvironment task_environment;
+  WaitableEvent thread_pool_done;
+  Lock lock_a;
+  Lock lock_b;
+  Lock lock_c;
+
+  ReleasableAutoLock releasable_auto_lock_a(&lock_a,
+                                            subtle::LockTracking::kEnabled);
+  ReleasableAutoLock releasable_auto_lock_b(&lock_b,
+                                            subtle::LockTracking::kEnabled);
+  ReleasableAutoLock releasable_auto_lock_c(&lock_c,
+                                            subtle::LockTracking::kEnabled);
+  SequenceCheckerImpl sequence_checker;
+  releasable_auto_lock_c.Release();
+  releasable_auto_lock_b.Release();
+  releasable_auto_lock_a.Release();
+
+  ThreadPool::PostTask(BindLambdaForTesting([&]() {
+    {
+      AutoLock auto_lock_a(lock_a, subtle::LockTracking::kEnabled);
+      AutoLock auto_lock_b(lock_b, subtle::LockTracking::kEnabled);
+      AutoLock auto_lock_c(lock_c, subtle::LockTracking::kEnabled);
+      EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+    }
+
+    {
+      AutoLock auto_lock_a(lock_a, subtle::LockTracking::kEnabled);
+      AutoLock auto_lock_b(lock_b, subtle::LockTracking::kEnabled);
+      EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+    }
+
+    {
+      AutoLock auto_lock_c(lock_c, subtle::LockTracking::kEnabled);
+      EXPECT_FALSE(sequence_checker.CalledOnValidSequence());
+    }
+
+    {
+      AutoLock auto_lock_b(lock_b, subtle::LockTracking::kEnabled);
+      EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+    }
+
+    {
+      AutoLock auto_lock_b(lock_a, subtle::LockTracking::kEnabled);
+      EXPECT_FALSE(sequence_checker.CalledOnValidSequence());
+    }
+
+    thread_pool_done.Signal();
+  }));
+
+  thread_pool_done.Wait();
+
+  EXPECT_FALSE(sequence_checker.CalledOnValidSequence());
+}
+
+TEST(SequenceCheckerTest, LockAndSequence) {
+  test::TaskEnvironment task_environment;
+  WaitableEvent thread_pool_done;
+  Lock lock;
+
+  // Create sequence checker while holding lock.
+  ReleasableAutoLock releasable_auto_lock(&lock,
+                                          subtle::LockTracking::kEnabled);
+  SequenceCheckerImpl sequence_checker;
+  releasable_auto_lock.Release();
+
+  // Check sequencing without holding the lock.
+  EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+
+  ThreadPool::PostTask(BindLambdaForTesting([&]() {
+    // Check sequencing while holding the lock. This is not valid because
+    // `CalledOnValidSequence()` previously returned true while the lock wasn't
+    // held.
+    {
+      AutoLock auto_lock(lock, subtle::LockTracking::kEnabled);
+      EXPECT_FALSE(sequence_checker.CalledOnValidSequence());
+    }
+
+    thread_pool_done.Signal();
+  }));
+
+  thread_pool_done.Wait();
+}
+
+TEST(SequenceCheckerTest, LockDetachFromSequence) {
+  test::TaskEnvironment task_environment;
+  WaitableEvent thread_pool_done;
+  Lock lock;
+
+  // Create sequence checker and detach while holding lock.
+  ReleasableAutoLock releasable_auto_lock(&lock,
+                                          subtle::LockTracking::kEnabled);
+  SequenceCheckerImpl sequence_checker;
+  sequence_checker.DetachFromSequence();
+  releasable_auto_lock.Release();
+
+  // Re-bind without holding the lock.
+  EXPECT_TRUE(sequence_checker.CalledOnValidSequence());
+
+  ThreadPool::PostTask(BindLambdaForTesting([&]() {
+    // Check sequencing while holding the lock. This is not valid because the
+    // sequence checker was detached and re-bound without the lock.
+    {
+      AutoLock auto_lock(lock, subtle::LockTracking::kEnabled);
+      EXPECT_FALSE(sequence_checker.CalledOnValidSequence());
+    }
+
+    thread_pool_done.Signal();
+  }));
+
+  thread_pool_done.Wait();
+}
+
+TEST(SequenceCheckerTest, LockMoveConstruction) {
+  test::TaskEnvironment task_environment;
+  WaitableEvent thread_pool_done;
+  Lock lock;
+
+  // Create sequence checker and move-construct while holding a lock.
+  ReleasableAutoLock releasable_auto_lock(&lock,
+                                          subtle::LockTracking::kEnabled);
+  SequenceCheckerImpl sequence_checker;
+  SequenceCheckerImpl other_sequence_checker(std::move(sequence_checker));
+  releasable_auto_lock.Release();
+
+  ThreadPool::PostTask(BindLambdaForTesting([&]() {
+    // Check sequencing while holding the lock.
+    {
+      AutoLock auto_lock(lock, subtle::LockTracking::kEnabled);
+      EXPECT_TRUE(other_sequence_checker.CalledOnValidSequence());
+    }
+
+    thread_pool_done.Signal();
+  }));
+
+  thread_pool_done.Wait();
+}
+
+TEST(SequenceCheckerTest, LockMoveAssignment) {
+  test::TaskEnvironment task_environment;
+  WaitableEvent thread_pool_done;
+  Lock lock;
+
+  SequenceCheckerImpl other_sequence_checker;
+
+  // Create sequence checker and move-assign it to `other_sequence_checker`
+  // while holding a lock.
+  ReleasableAutoLock releasable_auto_lock(&lock,
+                                          subtle::LockTracking::kEnabled);
+  SequenceCheckerImpl sequence_checker;
+  other_sequence_checker = std::move(sequence_checker);
+  releasable_auto_lock.Release();
+
+  ThreadPool::PostTask(BindLambdaForTesting([&]() {
+    // Check sequencing while holding the lock.
+    {
+      AutoLock auto_lock(lock, subtle::LockTracking::kEnabled);
+      EXPECT_TRUE(other_sequence_checker.CalledOnValidSequence());
+    }
+
+    thread_pool_done.Signal();
+  }));
+
+  thread_pool_done.Wait();
+}
+#endif  // DCHECK_IS_ON()
+
 }  // namespace base::internal
