@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,38 +6,45 @@
 #define BASE_TASK_SEQUENCE_MANAGER_SEQUENCE_MANAGER_IMPL_H_
 
 #include <deque>
-#include <list>
 #include <map>
 #include <memory>
-#include <random>
+#include <optional>
 #include <set>
 #include <string>
-#include <unordered_map>
 #include <utility>
 
 #include "base/atomic_sequence_num.h"
-#include "base/cancelable_callback.h"
+#include "base/base_export.h"
+#include "base/callback_list.h"
 #include "base/containers/circular_deque.h"
 #include "base/debug/crash_logging.h"
+#include "base/feature_list.h"
+#include "base/functional/callback_forward.h"
+#include "base/memory/raw_ptr.h"
+#include "base/memory/raw_ptr_exclusion.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/message_loop/message_pump_type.h"
+#include "base/observer_list.h"
 #include "base/pending_task.h"
+#include "base/rand_util.h"
 #include "base/run_loop.h"
-#include "base/sequenced_task_runner.h"
-#include "base/single_thread_task_runner.h"
 #include "base/synchronization/lock.h"
-#include "base/task/common/task_annotator.h"
 #include "base/task/current_thread.h"
 #include "base/task/sequence_manager/associated_thread_id.h"
 #include "base/task/sequence_manager/enqueue_order.h"
 #include "base/task/sequence_manager/enqueue_order_generator.h"
 #include "base/task/sequence_manager/sequence_manager.h"
+#include "base/task/sequence_manager/task_queue.h"
 #include "base/task/sequence_manager/task_queue_impl.h"
 #include "base/task/sequence_manager/task_queue_selector.h"
 #include "base/task/sequence_manager/thread_controller.h"
+#include "base/task/sequence_manager/work_tracker.h"
+#include "base/task/sequenced_task_runner.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/threading/thread_checker.h"
 #include "base/time/default_tick_clock.h"
+#include "base/types/pass_key.h"
 #include "base/values.h"
 #include "build/build_config.h"
 
@@ -48,6 +55,10 @@ class WebContentsAdapter;
 
 namespace base {
 
+namespace internal {
+class SequenceManagerThreadDelegate;
+}
+
 namespace trace_event {
 class ConvertableToTraceFormat;
 }  // namespace trace_event
@@ -61,10 +72,18 @@ class TimeDomain;
 
 namespace internal {
 
-class RealTimeDomain;
 class TaskQueueImpl;
+class DefaultWakeUpQueue;
+class SequenceManagerImpl;
 class ThreadControllerImpl;
 
+// A private factory method for SequenceManagerThreadDelegate which is
+// equivalent to sequence_manager::CreateUnboundSequenceManager() but returns
+// the underlying impl.
+std::unique_ptr<SequenceManagerImpl> CreateUnboundSequenceManagerImpl(
+    PassKey<base::internal::SequenceManagerThreadDelegate>,
+    SequenceManager::Settings settings);
+
 // The task queue manager provides N task queues and a selector interface for
 // choosing which task queue to service next. Each task queue consists of two
 // sub queues:
@@ -89,19 +108,8 @@ class BASE_EXPORT SequenceManagerImpl
   SequenceManagerImpl& operator=(const SequenceManagerImpl&) = delete;
   ~SequenceManagerImpl() override;
 
-  // Assume direct control over current thread and create a SequenceManager.
-  // This function should be called only once per thread.
-  // This function assumes that a task execution environment is already
-  // initialized for the current thread.
-  static std::unique_ptr<SequenceManagerImpl> CreateOnCurrentThread(
-      SequenceManager::Settings settings = SequenceManager::Settings());
-
-  // Create an unbound SequenceManager (typically for a future thread). The
-  // SequenceManager can be initialized on the current thread and then needs to
-  // be bound and initialized on the target thread by calling one of the Bind*()
-  // methods.
-  static std::unique_ptr<SequenceManagerImpl> CreateUnbound(
-      SequenceManager::Settings settings);
+  // Initializes features for this class. See `base::features::Init()`.
+  static void InitializeFeatures();
 
   // SequenceManager implementation:
   void BindToCurrentThread() override;
@@ -110,9 +118,8 @@ class BASE_EXPORT SequenceManagerImpl
   void SetObserver(Observer* observer) override;
   void AddTaskTimeObserver(TaskTimeObserver* task_time_observer) override;
   void RemoveTaskTimeObserver(TaskTimeObserver* task_time_observer) override;
-  void RegisterTimeDomain(TimeDomain* time_domain) override;
-  void UnregisterTimeDomain(TimeDomain* time_domain) override;
-  TimeDomain* GetRealTimeDomain() const override;
+  void SetTimeDomain(TimeDomain* time_domain) override;
+  void ResetTimeDomain() override;
   const TickClock* GetTickClock() const override;
   TimeTicks NowTicks() const override;
   void SetDefaultTaskRunner(
@@ -120,63 +127,62 @@ class BASE_EXPORT SequenceManagerImpl
   void ReclaimMemory() override;
   bool GetAndClearSystemIsQuiescentBit() override;
   void SetWorkBatchSize(int work_batch_size) override;
-  void SetTimerSlack(TimerSlack timer_slack) override;
   void EnableCrashKeys(const char* async_stack_crash_key) override;
   const MetricRecordingSettings& GetMetricRecordingSettings() const override;
   size_t GetPendingTaskCountForTesting() const override;
-  scoped_refptr<TaskQueue> CreateTaskQueue(
-      const TaskQueue::Spec& spec) override;
+  TaskQueue::Handle CreateTaskQueue(const TaskQueue::Spec& spec) override;
   std::string DescribeAllPendingTasks() const override;
-  std::unique_ptr<NativeWorkHandle> OnNativeWorkPending(
-      TaskQueue::QueuePriority priority) override;
+  void PrioritizeYieldingToNative(base::TimeTicks prioritize_until) override;
   void AddTaskObserver(TaskObserver* task_observer) override;
   void RemoveTaskObserver(TaskObserver* task_observer) override;
+  std::optional<WakeUp> GetNextDelayedWakeUp() const override;
+  TaskQueue::QueuePriority GetPriorityCount() const override;
 
   // SequencedTaskSource implementation:
-  Task* SelectNextTask(
+  void SetRunTaskSynchronouslyAllowed(
+      bool can_run_tasks_synchronously) override;
+  std::optional<SelectedTask> SelectNextTask(
+      LazyNow& lazy_now,
       SelectTaskOption option = SelectTaskOption::kDefault) override;
-  void DidRunTask() override;
-  TimeDelta DelayTillNextTask(
+  void DidRunTask(LazyNow& lazy_now) override;
+  std::optional<WakeUp> GetPendingWakeUp(
       LazyNow* lazy_now,
-      SelectTaskOption option = SelectTaskOption::kDefault) const override;
+      SelectTaskOption option = SelectTaskOption::kDefault) override;
   bool HasPendingHighResolutionTasks() override;
-  bool OnSystemIdle() override;
+  void OnBeginWork() override;
+  bool OnIdle() override;
+  void MaybeEmitTaskDetails(
+      perfetto::EventContext& ctx,
+      const SequencedTaskSource::SelectedTask& selected_task) const override;
 
   void AddDestructionObserver(
       CurrentThread::DestructionObserver* destruction_observer);
   void RemoveDestructionObserver(
       CurrentThread::DestructionObserver* destruction_observer);
-  // TODO(alexclarke): Remove this as part of https://crbug.com/825327.
+  [[nodiscard]] CallbackListSubscription RegisterOnNextIdleCallback(
+      OnceClosure on_next_idle_callback);
+
+  // Sets / returns the default TaskRunner. Thread-safe.
   void SetTaskRunner(scoped_refptr<SingleThreadTaskRunner> task_runner);
-  // TODO(alexclarke): Remove this as part of https://crbug.com/825327.
   scoped_refptr<SingleThreadTaskRunner> GetTaskRunner();
+
   bool IsBoundToCurrentThread() const;
   MessagePump* GetMessagePump() const;
   bool IsType(MessagePumpType type) const;
   void SetAddQueueTimeToTasks(bool enable);
-  void SetTaskExecutionAllowed(bool allowed);
-  bool IsTaskExecutionAllowed() const;
-#if defined(OS_IOS)
+  void SetTaskExecutionAllowedInNativeNestedLoop(bool allowed);
+  bool IsTaskExecutionAllowedInNativeNestedLoop() const;
+#if BUILDFLAG(IS_IOS)
   void AttachToMessagePump();
 #endif
   bool IsIdleForTesting() override;
-  void BindToCurrentThread(std::unique_ptr<MessagePump> pump);
-  void DeletePendingTasks();
-  bool HasTasks();
-  MessagePumpType GetType() const;
+  void EnableMessagePumpTimeKeeperMetrics(
+      const char* thread_name,
+      bool wall_time_based_metrics_enabled_for_testing = false);
 
   // Requests that a task to process work is scheduled.
   void ScheduleWork();
 
-  // Requests that a delayed task to process work is posted on the main task
-  // runner. These delayed tasks are de-duplicated. Must be called on the thread
-  // this class was created on.
-
-  // Schedules next wake-up at the given time, cancels any previous requests.
-  // Use TimeTicks::Max() to cancel a wake-up.
-  // Must be called from a TimeDomain only.
-  void SetNextDelayedDoWork(LazyNow* lazy_now, TimeTicks run_time);
-
   // Returns the currently executing TaskQueue if any. Must be called on the
   // thread this class was created on.
   internal::TaskQueueImpl* currently_executing_task_queue() const;
@@ -186,11 +192,7 @@ class BASE_EXPORT SequenceManagerImpl
   void UnregisterTaskQueueImpl(
       std::unique_ptr<internal::TaskQueueImpl> task_queue);
 
-  // Schedule a call to UnregisterTaskQueueImpl as soon as it's safe to do so.
-  void ShutdownTaskQueueGracefully(
-      std::unique_ptr<internal::TaskQueueImpl> task_queue);
-
-  const scoped_refptr<AssociatedThreadId>& associated_thread() const {
+  scoped_refptr<AssociatedThreadId> associated_thread() const {
     return associated_thread_;
   }
 
@@ -199,8 +201,7 @@ class BASE_EXPORT SequenceManagerImpl
   WeakPtr<SequenceManagerImpl> GetWeakPtr();
 
   // How frequently to perform housekeeping tasks (sweeping canceled tasks etc).
-  static constexpr TimeDelta kReclaimMemoryInterval =
-      TimeDelta::FromSeconds(30);
+  static constexpr TimeDelta kReclaimMemoryInterval = Seconds(30);
 
  protected:
   static std::unique_ptr<ThreadControllerImpl>
@@ -212,19 +213,47 @@ class BASE_EXPORT SequenceManagerImpl
                       SequenceManager::Settings settings = Settings());
 
   friend class internal::TaskQueueImpl;
+  friend class internal::DefaultWakeUpQueue;
   friend class ::base::sequence_manager::SequenceManagerForTest;
   friend class QtWebEngineCore::MessagePumpForUIQt;
   friend class QtWebEngineCore::WebContentsAdapter;
 
  private:
-  class NativeWorkHandleImpl;
-
   // Returns the SequenceManager running the
   // current thread. It must only be used on the thread it was obtained.
   // Only to be used by CurrentThread for the moment
   static SequenceManagerImpl* GetCurrent();
   friend class ::base::CurrentThread;
 
+  // Factory friends to call into private creation methods.
+  friend std::unique_ptr<SequenceManager>
+      sequence_manager::CreateSequenceManagerOnCurrentThread(
+          SequenceManager::Settings);
+  friend std::unique_ptr<SequenceManager>
+  sequence_manager::CreateSequenceManagerOnCurrentThreadWithPump(
+      std::unique_ptr<MessagePump> message_pump,
+      SequenceManager::Settings);
+  friend std::unique_ptr<SequenceManager>
+      sequence_manager::CreateUnboundSequenceManager(SequenceManager::Settings);
+  friend std::unique_ptr<SequenceManagerImpl>
+      sequence_manager::internal::CreateUnboundSequenceManagerImpl(
+          PassKey<base::internal::SequenceManagerThreadDelegate>,
+          SequenceManager::Settings);
+
+  // Assume direct control over current thread and create a SequenceManager.
+  // This function should be called only once per thread.
+  // This function assumes that a task execution environment is already
+  // initialized for the current thread.
+  static std::unique_ptr<SequenceManagerImpl> CreateOnCurrentThread(
+      SequenceManager::Settings settings);
+
+  // Create an unbound SequenceManager (typically for a future thread). The
+  // SequenceManager can be initialized on the current thread and then needs to
+  // be bound and initialized on the target thread by calling one of the Bind*()
+  // methods.
+  static std::unique_ptr<SequenceManagerImpl> CreateUnbound(
+      SequenceManager::Settings settings);
+
   enum class ProcessTaskResult {
     kDeferred,
     kExecuted,
@@ -246,15 +275,18 @@ class BASE_EXPORT SequenceManagerImpl
                   TaskQueue::TaskTiming task_timing)
         : pending_task(std::move(task)),
           task_queue(task_queue),
-          task_queue_name(task_queue->GetName()),
+          task_queue_name(task_queue->GetProtoName()),
           task_timing(task_timing),
           priority(task_queue->GetQueuePriority()),
           task_type(pending_task.task_type) {}
 
     Task pending_task;
-    internal::TaskQueueImpl* task_queue = nullptr;
+
+    // `task_queue` is not a raw_ptr<...> for performance reasons (based on
+    // analysis of sampling profiler data and tab_search:top100:2020).
+    RAW_PTR_EXCLUSION internal::TaskQueueImpl* task_queue = nullptr;
     // Save task_queue_name as the task queue can be deleted within the task.
-    const char* task_queue_name;
+    QueueName task_queue_name;
     TaskQueue::TaskTiming task_timing;
     // Save priority as it might change after running a task.
     TaskQueue::QueuePriority priority;
@@ -265,28 +297,34 @@ class BASE_EXPORT SequenceManagerImpl
 
   struct MainThreadOnly {
     explicit MainThreadOnly(
+        SequenceManagerImpl* sequence_manager,
         const scoped_refptr<AssociatedThreadId>& associated_thread,
-        const SequenceManager::Settings& settings);
+        const SequenceManager::Settings& settings,
+        const base::TickClock* clock);
     ~MainThreadOnly();
 
     int nesting_depth = 0;
     NonNestableTaskDeque non_nestable_task_queue;
     // TODO(altimin): Switch to instruction pointer crash key when it's
     // available.
-    debug::CrashKeyString* file_name_crash_key = nullptr;
-    debug::CrashKeyString* function_name_crash_key = nullptr;
-    debug::CrashKeyString* async_stack_crash_key = nullptr;
+    raw_ptr<debug::CrashKeyString> file_name_crash_key = nullptr;
+    raw_ptr<debug::CrashKeyString> function_name_crash_key = nullptr;
+    raw_ptr<debug::CrashKeyString> async_stack_crash_key = nullptr;
     std::array<char, static_cast<size_t>(debug::CrashKeySize::Size64)>
         async_stack_buffer = {};
 
-    std::mt19937_64 random_generator;
-    std::uniform_real_distribution<double> uniform_distribution;
+    std::optional<base::MetricsSubSampler> metrics_subsampler;
 
     internal::TaskQueueSelector selector;
-    ObserverList<TaskObserver>::Unchecked task_observers;
-    ObserverList<TaskTimeObserver>::Unchecked task_time_observers;
-    std::set<TimeDomain*> time_domains;
-    std::unique_ptr<internal::RealTimeDomain> real_time_domain;
+    // RAW_PTR_EXCLUSION: Performance reasons(based on analysis of
+    // speedometer3).
+    ObserverList<TaskObserver>::UncheckedAndRawPtrExcluded task_observers;
+    ObserverList<TaskTimeObserver> task_time_observers;
+    const raw_ptr<const base::TickClock> default_clock;
+    raw_ptr<TimeDomain> time_domain = nullptr;
+
+    std::unique_ptr<WakeUpQueue> wake_up_queue;
+    std::unique_ptr<WakeUpQueue> non_waking_wake_up_queue;
 
     // If true MaybeReclaimMemory will attempt to reclaim memory.
     bool memory_reclaim_scheduled = false;
@@ -295,56 +333,70 @@ class BASE_EXPORT SequenceManagerImpl
     TimeTicks next_time_to_reclaim_memory;
 
     // List of task queues managed by this SequenceManager.
-    // - active_queues contains queues that are still running tasks.
-    //   Most often they are owned by relevant TaskQueues, but
-    //   queues_to_gracefully_shutdown_ are included here too.
-    // - queues_to_gracefully_shutdown contains queues which should be deleted
-    //   when they become empty.
+    // - active_queues contains queues that are still running tasks, which are
+    //   are owned by relevant TaskQueues.
     // - queues_to_delete contains soon-to-be-deleted queues, because some
     //   internal scheduling code does not expect queues to be pulled
     //   from underneath.
 
-    std::set<internal::TaskQueueImpl*> active_queues;
+    // RAW_PTR_EXCLUSION: Performance reasons (based on analysis of
+    // speedometer3).
+    RAW_PTR_EXCLUSION std::set<internal::TaskQueueImpl*> active_queues;
 
-    std::map<internal::TaskQueueImpl*, std::unique_ptr<internal::TaskQueueImpl>>
-        queues_to_gracefully_shutdown;
     std::map<internal::TaskQueueImpl*, std::unique_ptr<internal::TaskQueueImpl>>
         queues_to_delete;
 
     bool task_was_run_on_quiescence_monitored_queue = false;
     bool nesting_observer_registered_ = false;
 
-    // Due to nested runloops more than one task can be executing concurrently.
-    // Note that this uses std::deque for pointer stability, since pointers to
-    // objects in this container are stored in TLS.
+    // Use std::deque() so that references returned by SelectNextTask() remain
+    // valid until the matching call to DidRunTask(), even when nested RunLoops
+    // cause tasks to be pushed on the stack in-between. This is needed because
+    // references are kept in local variables by calling code between
+    // SelectNextTask()/DidRunTask().
     std::deque<ExecutingTask> task_execution_stack;
 
-    Observer* observer = nullptr;  // NOT OWNED
+    raw_ptr<Observer> observer = nullptr;  // NOT OWNED
 
-    ObserverList<CurrentThread::DestructionObserver>::Unchecked
-        destruction_observers;
+    ObserverList<CurrentThread::DestructionObserver>::
+        UncheckedAndDanglingUntriaged destruction_observers;
 
-    // By default native work is not prioritized at all.
-    std::multiset<TaskQueue::QueuePriority> pending_native_work{
-        TaskQueue::kBestEffortPriority};
+    // Notified the next time `OnIdle()` completes without scheduling additional
+    // work.
+    OnceClosureList on_next_idle_callbacks;
   };
 
   void CompleteInitializationOnBoundThread();
 
   // TaskQueueSelector::Observer:
   void OnTaskQueueEnabled(internal::TaskQueueImpl* queue) override;
+  void OnWorkAvailable() override;
 
   // RunLoop::NestingObserver:
   void OnBeginNestedRunLoop() override;
   void OnExitNestedRunLoop() override;
 
-  // Called by the task queue to inform this SequenceManager of a task that's
-  // about to be queued. This SequenceManager may use this opportunity to add
-  // metadata to |pending_task| before it is moved into the queue.
-  void WillQueueTask(Task* pending_task, const char* task_queue_name);
+  // Schedules next wake-up at the given time, canceling any previous requests.
+  // Use std::nullopt to cancel a wake-up. Must be called on the thread this
+  // class was created on.
+  void SetNextWakeUp(LazyNow* lazy_now, std::optional<WakeUp> wake_up);
 
-  // Delayed Tasks with run_times <= Now() are enqueued onto the work queue and
-  // reloads any empty work queues.
+  // Called before TaskQueue requests to reload its empty immediate work queue.
+  void WillRequestReloadImmediateWorkQueue();
+
+  // Returns a valid `SyncWorkAuthorization` if a call to `RunOrPostTask` on a
+  // `SequencedTaskRunner` bound to this `SequenceManager` may run its task
+  // synchronously.
+  SyncWorkAuthorization TryAcquireSyncWorkAuthorization();
+
+  // Called when a task is about to be queued. May add metadata to the task and
+  // emit trace events.
+  void WillQueueTask(Task* pending_task);
+
+  // Enqueues onto delayed WorkQueues all delayed tasks which must run now
+  // (cannot be postponed) and possibly some delayed tasks which can run now but
+  // could be postponed (due to how tasks are stored, it is not possible to
+  // retrieve all such tasks efficiently) and reloads any empty work queues.
   void MoveReadyDelayedTasksToWorkQueues(LazyNow* lazy_now);
 
   void NotifyWillProcessTask(ExecutingTask* task, LazyNow* time_before_task);
@@ -357,8 +409,9 @@ class BASE_EXPORT SequenceManagerImpl
   std::unique_ptr<trace_event::ConvertableToTraceFormat>
   AsValueWithSelectorResultForTracing(internal::WorkQueue* selected_work_queue,
                                       bool force_verbose) const;
-  Value AsValueWithSelectorResult(internal::WorkQueue* selected_work_queue,
-                                  bool force_verbose) const;
+  Value::Dict AsValueWithSelectorResult(
+      internal::WorkQueue* selected_work_queue,
+      bool force_verbose) const;
 
   // Used in construction of TaskQueueImpl to obtain an AtomicFlag which it can
   // use to request reload by ReloadEmptyWorkQueues. The lifetime of
@@ -370,10 +423,10 @@ class BASE_EXPORT SequenceManagerImpl
 
   // Calls |TakeImmediateIncomingQueueTasks| on all queues with their reload
   // flag set in |empty_queues_to_reload_|.
-  void ReloadEmptyWorkQueues() const;
+  void ReloadEmptyWorkQueues();
 
   std::unique_ptr<internal::TaskQueueImpl> CreateTaskQueueImpl(
-      const TaskQueue::Spec& spec) override;
+      const TaskQueue::Spec& spec);
 
   // Periodically reclaims memory by sweeping away canceled tasks and shrinking
   // buffers.
@@ -382,24 +435,36 @@ class BASE_EXPORT SequenceManagerImpl
   // Deletes queues marked for deletion and empty queues marked for shutdown.
   void CleanUpQueues();
 
-  void RemoveAllCanceledTasksFromFrontOfWorkQueues();
+  // Removes canceled delayed tasks from the front of wake up queue.
+  void RemoveAllCanceledDelayedTasksFromFront(LazyNow* lazy_now);
 
   TaskQueue::TaskTiming::TimeRecordingPolicy ShouldRecordTaskTiming(
       const internal::TaskQueueImpl* task_queue);
   bool ShouldRecordCPUTimeForTask();
+
+  // Write the async stack trace onto a crash key as whitespace-delimited hex
+  // addresses.
   void RecordCrashKeys(const PendingTask&);
 
   // Helper to terminate all scoped trace events to allow starting new ones
   // in SelectNextTask().
-  Task* SelectNextTaskImpl(SelectTaskOption option);
+  std::optional<SelectedTask> SelectNextTaskImpl(LazyNow& lazy_now,
+                                                 SelectTaskOption option);
 
-  // Check if a task of priority |priority| should run given the pending set of
-  // native work.
-  bool ShouldRunTaskOfPriority(TaskQueue::QueuePriority priority) const;
+  // Returns a wake-up for the next delayed task which is not ripe for
+  // execution, or nullopt if `option` is `kSkipDelayedTask` or there
+  // are no such tasks (immediate tasks don't count).
+  std::optional<WakeUp> GetNextDelayedWakeUpWithOption(
+      SelectTaskOption option) const;
 
-  // Ignores any immediate work.
-  TimeDelta GetDelayTillNextDelayedTask(LazyNow* lazy_now,
-                                        SelectTaskOption option) const;
+  // Given a `wake_up` describing when the next delayed task should run, returns
+  // a wake up that should be scheduled on the thread. `is_immediate()` if the
+  // wake up should run immediately. `nullopt` if no wake up is required because
+  // `wake_up` is `nullopt` or a `time_domain` is used.
+  std::optional<WakeUp> AdjustWakeUp(std::optional<WakeUp> wake_up,
+                                     LazyNow* lazy_now) const;
+
+  void MaybeAddLeewayToTask(Task& task) const;
 
 #if DCHECK_IS_ON()
   void LogTaskDebugInfo(const internal::WorkQueue* work_queue) const;
@@ -410,7 +475,7 @@ class BASE_EXPORT SequenceManagerImpl
   TaskQueue::TaskTiming InitializeTaskTiming(
       internal::TaskQueueImpl* task_queue);
 
-  scoped_refptr<AssociatedThreadId> associated_thread_;
+  const scoped_refptr<AssociatedThreadId> associated_thread_;
 
   EnqueueOrderGenerator enqueue_order_generator_;
 
@@ -419,17 +484,13 @@ class BASE_EXPORT SequenceManagerImpl
 
   const MetricRecordingSettings metric_recording_settings_;
 
+  WorkTracker work_tracker_;
+
   // Whether to add the queue time to tasks.
   base::subtle::Atomic32 add_queue_time_to_tasks_;
 
   AtomicFlagSet empty_queues_to_reload_;
 
-  // A check to bail out early during memory corruption.
-  // https://crbug.com/757940
-  bool Validate();
-
-  volatile int32_t memory_corruption_sentinel_;
-
   MainThreadOnly main_thread_only_;
   MainThreadOnly& main_thread_only() {
     DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
@@ -440,6 +501,23 @@ class BASE_EXPORT SequenceManagerImpl
     return main_thread_only_;
   }
 
+  // |clock_| either refers to the TickClock representation of |time_domain|
+  // (same object) if any, or to |default_clock| otherwise. It is maintained as
+  // an atomic pointer here for multi-threaded usage.
+  std::atomic<const base::TickClock*> clock_;
+  const base::TickClock* main_thread_clock() const {
+    DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
+    return clock_.load(std::memory_order_relaxed);
+  }
+  const base::TickClock* any_thread_clock() const {
+    // |memory_order_acquire| matched by |memory_order_release| in
+    // SetTimeDomain() to ensure all data used by |clock_| is visible when read
+    // from the current thread. A thread might try to access a stale |clock_|
+    // but that's not an issue since |time_domain| contractually outlives
+    // SequenceManagerImpl even if it's reset.
+    return clock_.load(std::memory_order_acquire);
+  }
+
   WeakPtrFactory<SequenceManagerImpl> weak_factory_{this};
 };
 
