@@ -4,18 +4,57 @@
 
 #include "base/task/sequence_manager/thread_controller.h"
 
+#include <atomic>
+#include <string_view>
+
 #include "base/check.h"
+#include "base/feature_list.h"
 #include "base/metrics/histogram.h"
 #include "base/metrics/histogram_base.h"
+#include "base/metrics/histogram_functions.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/notreached.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_util.h"
 #include "base/time/tick_clock.h"
+#include "base/time/time.h"
 #include "base/trace_event/base_tracing.h"
 
 namespace base {
 namespace sequence_manager {
 namespace internal {
 
+namespace {
+// Enable sample metadata recording in this class, if it's currently disabled.
+// Note that even if `kThreadControllerSetsProfilerMetadata` is disabled, sample
+// metadata may still be recorded.
+BASE_FEATURE(kThreadControllerSetsProfilerMetadata,
+             "ThreadControllerSetsProfilerMetadata",
+             base::FEATURE_DISABLED_BY_DEFAULT);
+
+// Thread safe copy to be updated once feature list is available. This
+// defaults to true to make sure that no metadata is lost on clients that
+// need to record. This leads to some overeporting before feature list
+// initialization on other clients but that's still way better than the current
+// situation which is reporting all the time.
+std::atomic<bool> g_thread_controller_sets_profiler_metadata{true};
+
+// ThreadController interval metrics are mostly of interest for intervals that
+// are not trivially short. Under a certain threshold it's unlikely that
+// intervention from developers would move metrics. Log with suffix for
+// intervals under a threshold chosen via tracing data. To validate the
+// threshold makes sense and does not filter out too many samples
+// ThreadController.ActiveIntervalDuration can be used.
+constexpr TimeDelta kNonTrivialActiveIntervalLength = Milliseconds(1);
+constexpr TimeDelta kMediumActiveIntervalLength = Milliseconds(100);
+
+std::string MakeSuffix(std::string_view time_suffix,
+                       std::string_view thread_name) {
+  return base::StrCat({".", time_suffix, ".", thread_name});
+}
+
+}  // namespace
+
 ThreadController::ThreadController(const TickClock* time_source)
     : associated_thread_(AssociatedThreadId::CreateUnbound()),
       time_source_(time_source) {}
@@ -38,32 +77,86 @@ ThreadController::RunLevelTracker::~RunLevelTracker() {
   DCHECK_EQ(run_levels_.size(), 0u);
 }
 
+// static
+void ThreadController::InitializeFeatures(
+    features::EmitThreadControllerProfilerMetadata emit_profiler_metadata) {
+  g_thread_controller_sets_profiler_metadata.store(
+      emit_profiler_metadata ==
+              features::EmitThreadControllerProfilerMetadata::kForce ||
+          base::FeatureList::IsEnabled(kThreadControllerSetsProfilerMetadata),
+      std::memory_order_relaxed);
+}
+
+bool ThreadController::RunLevelTracker::RunLevel::ShouldRecordSampleMetadata() {
+  return g_thread_controller_sets_profiler_metadata.load(
+      std::memory_order_relaxed);
+}
+
+std::string_view ThreadController::RunLevelTracker::RunLevel::GetThreadName() {
+  std::string_view thread_name = "Other";
+  if (!time_keeper_->thread_name().empty()) {
+    thread_name = time_keeper_->thread_name();
+  }
+  return thread_name;
+}
+
+std::string
+ThreadController::RunLevelTracker::RunLevel::GetSuffixForCatchAllHistogram() {
+  return MakeSuffix("Any", GetThreadName());
+}
+
+std::string ThreadController::RunLevelTracker::RunLevel::GetSuffixForHistogram(
+    TimeDelta duration) {
+  std::string_view time_suffix;
+  if (duration < kNonTrivialActiveIntervalLength) {
+    time_suffix = "Short";
+  } else if (duration < kMediumActiveIntervalLength) {
+    time_suffix = "Medium";
+  }
+  return MakeSuffix(time_suffix, GetThreadName());
+}
+
 void ThreadController::EnableMessagePumpTimeKeeperMetrics(
-    const char* thread_name) {
+    const char* thread_name,
+    bool wall_time_based_metrics_enabled_for_testing) {
   // MessagePump runs too fast, a low-res clock would result in noisy metrics.
   if (!base::TimeTicks::IsHighResolution())
     return;
 
-  run_level_tracker_.EnableTimeKeeperMetrics(thread_name);
+  run_level_tracker_.EnableTimeKeeperMetrics(
+      thread_name, wall_time_based_metrics_enabled_for_testing);
 }
 
 void ThreadController::RunLevelTracker::EnableTimeKeeperMetrics(
-    const char* thread_name) {
-  time_keeper_.EnableRecording(thread_name);
+    const char* thread_name,
+    bool wall_time_based_metrics_enabled_for_testing) {
+  time_keeper_.EnableRecording(thread_name,
+                               wall_time_based_metrics_enabled_for_testing);
 }
 
 void ThreadController::RunLevelTracker::TimeKeeper::EnableRecording(
-    const char* thread_name) {
+    const char* thread_name,
+    bool wall_time_based_metrics_enabled_for_testing) {
   DCHECK(!histogram_);
+  thread_name_ = thread_name;
+  wall_time_based_metrics_enabled_for_testing_ =
+      wall_time_based_metrics_enabled_for_testing;
+
   histogram_ = LinearHistogram::FactoryGet(
       JoinString({"Scheduling.MessagePumpTimeKeeper", thread_name}, "."), 1,
       Phase::kLastPhase, Phase::kLastPhase + 1,
       base::HistogramBase::kUmaTargetedHistogramFlag);
 
 #if BUILDFLAG(ENABLE_BASE_TRACING)
-  perfetto_track_.emplace(reinterpret_cast<uint64_t>(this),
-                          perfetto::ThreadTrack::Current());
-  // TODO(1006541): Use Perfetto library to name this Track.
+  perfetto_track_.emplace(
+      reinterpret_cast<uint64_t>(this),
+      // TODO(crbug.com/42050015): Replace with ThreadTrack::Current() after SDK
+      // migration.
+      // In the non-SDK version, ThreadTrack::Current() returns a different
+      // track id on some platforms (for example Mac OS), which results in
+      // async tracks not being associated with their thread.
+      perfetto::ThreadTrack::ForThread(base::PlatformThread::CurrentId()));
+  // TODO(crbug.com/42050015): Use Perfetto library to name this Track.
   // auto desc = perfetto_track_->Serialize();
   // desc.set_name(JoinString({"MessagePumpPhases", thread_name}, " "));
   // perfetto::internal::TrackEventDataSource::SetTrackDescriptor(
@@ -76,12 +169,7 @@ void ThreadController::RunLevelTracker::OnRunLoopStarted(State initial_state,
   DCHECK_CALLED_ON_VALID_THREAD(outer_->associated_thread_->thread_checker);
 
   const bool is_nested = !run_levels_.empty();
-  run_levels_.emplace(initial_state, is_nested, time_keeper_, lazy_now
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-                      ,
-                      terminating_wakeup_lambda_
-#endif
-  );
+  run_levels_.emplace(initial_state, is_nested, time_keeper_, lazy_now);
 
   // In unit tests, RunLoop::Run() acts as the initial wake-up.
   if (!is_nested && initial_state != kIdle)
@@ -117,12 +205,7 @@ void ThreadController::RunLevelTracker::OnWorkStarted(LazyNow& lazy_now) {
   // Already running a work item? => #work-in-work-implies-nested
   if (run_levels_.top().state() == kRunningWorkItem) {
     run_levels_.emplace(kRunningWorkItem, /*nested=*/true, time_keeper_,
-                        lazy_now
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-                        ,
-                        terminating_wakeup_lambda_
-#endif
-    );
+                        lazy_now);
   } else {
     if (run_levels_.top().state() == kIdle) {
       time_keeper_.RecordWakeUp(lazy_now);
@@ -131,7 +214,7 @@ void ThreadController::RunLevelTracker::OnWorkStarted(LazyNow& lazy_now) {
     }
 
     // Going from kIdle or kInBetweenWorkItems to kRunningWorkItem.
-    run_levels_.top().UpdateState(kRunningWorkItem);
+    run_levels_.top().UpdateState(kRunningWorkItem, lazy_now);
   }
 }
 
@@ -151,13 +234,15 @@ void ThreadController::RunLevelTracker::OnApplicationTaskSelected(
   time_keeper_.OnApplicationTaskSelected(queue_time, lazy_now);
 }
 
-void ThreadController::RunLevelTracker::OnWorkEnded(LazyNow& lazy_now) {
+void ThreadController::RunLevelTracker::OnWorkEnded(LazyNow& lazy_now,
+                                                    int run_level_depth) {
   DCHECK_CALLED_ON_VALID_THREAD(outer_->associated_thread_->thread_checker);
   if (run_levels_.empty())
     return;
 
-  // #done-work-while-not-running-implies-done-nested
-  if (run_levels_.top().state() != kRunningWorkItem) {
+  // #done-work-at-lower-runlevel-implies-done-nested
+  if (run_level_depth != static_cast<int>(num_run_levels())) {
+    DCHECK_EQ(run_level_depth + 1, static_cast<int>(num_run_levels()));
     run_levels_.top().set_exit_lazy_now(&lazy_now);
     run_levels_.pop();
   } else {
@@ -167,7 +252,7 @@ void ThreadController::RunLevelTracker::OnWorkEnded(LazyNow& lazy_now) {
   // Whether we exited a nested run-level or not: the current run-level is now
   // transitioning from kRunningWorkItem to kInBetweenWorkItems.
   DCHECK_EQ(run_levels_.top().state(), kRunningWorkItem);
-  run_levels_.top().UpdateState(kInBetweenWorkItems);
+  run_levels_.top().UpdateState(kInBetweenWorkItems, lazy_now);
 }
 
 void ThreadController::RunLevelTracker::OnIdle(LazyNow& lazy_now) {
@@ -177,7 +262,7 @@ void ThreadController::RunLevelTracker::OnIdle(LazyNow& lazy_now) {
 
   DCHECK_NE(run_levels_.top().state(), kRunningWorkItem);
   time_keeper_.RecordEndOfPhase(kIdleWork, lazy_now);
-  run_levels_.top().UpdateState(kIdle);
+  run_levels_.top().UpdateState(kIdle, lazy_now);
 }
 
 void ThreadController::RunLevelTracker::RecordScheduleWork() {
@@ -202,49 +287,40 @@ void ThreadController::RunLevelTracker::SetTraceObserverForTesting(
 ThreadController::RunLevelTracker::TraceObserverForTesting*
     ThreadController::RunLevelTracker::trace_observer_for_testing_ = nullptr;
 
-ThreadController::RunLevelTracker::RunLevel::RunLevel(
-    State initial_state,
-    bool is_nested,
-    TimeKeeper& time_keeper,
-    LazyNow& lazy_now
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-    ,
-    TerminatingFlowLambda& terminating_wakeup_flow_lambda
-#endif
-    )
+ThreadController::RunLevelTracker::RunLevel::RunLevel(State initial_state,
+                                                      bool is_nested,
+                                                      TimeKeeper& time_keeper,
+                                                      LazyNow& lazy_now)
     : is_nested_(is_nested),
       time_keeper_(time_keeper),
       thread_controller_sample_metadata_("ThreadController active",
-                                         base::SampleMetadataScope::kThread)
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-      ,
-      terminating_wakeup_flow_lambda_(terminating_wakeup_flow_lambda)
-#endif
-{
+                                         base::SampleMetadataScope::kThread) {
   if (is_nested_) {
     // Stop the current kWorkItem phase now, it will resume after the kNested
     // phase ends.
     time_keeper_->RecordEndOfPhase(kWorkItemSuspendedOnNested, lazy_now);
   }
-  UpdateState(initial_state);
+  UpdateState(initial_state, lazy_now);
 }
 
 ThreadController::RunLevelTracker::RunLevel::~RunLevel() {
   if (!was_moved_) {
     DCHECK(exit_lazy_now_);
-    UpdateState(kIdle);
+    UpdateState(kIdle, *exit_lazy_now_);
     if (is_nested_) {
       // Attribute the entire time in this nested RunLevel to kNested phase. If
       // this wasn't the last nested RunLevel, this is ignored and will be
       // applied on the final pop().
       time_keeper_->RecordEndOfPhase(kNested, *exit_lazy_now_);
 
-      // Intentionally ordered after UpdateState(kIdle), reinstantiates
-      // thread_controller_sample_metadata_ when yielding back to a parent
-      // RunLevel (which is active by definition as it is currently running this
-      // one).
-      thread_controller_sample_metadata_.Set(
-          static_cast<int64_t>(++thread_controller_active_id_));
+      if (ShouldRecordSampleMetadata()) {
+        // Intentionally ordered after UpdateState(kIdle), reinstantiates
+        // thread_controller_sample_metadata_ when yielding back to a parent
+        // RunLevel (which is active by definition as it is currently running
+        // this one).
+        thread_controller_sample_metadata_.Set(
+            static_cast<int64_t>(++thread_controller_active_id_));
+      }
     }
   }
 }
@@ -252,7 +328,147 @@ ThreadController::RunLevelTracker::RunLevel::~RunLevel() {
 ThreadController::RunLevelTracker::RunLevel::RunLevel(RunLevel&& other) =
     default;
 
-void ThreadController::RunLevelTracker::RunLevel::UpdateState(State new_state) {
+void ThreadController::RunLevelTracker::RunLevel::LogPercentageMetric(
+    const char* name,
+    int percentage) {
+  UmaHistogramPercentage(base::StrCat({name, ".", GetThreadName()}),
+                         percentage);
+}
+
+void ThreadController::RunLevelTracker::RunLevel::LogPercentageMetric(
+    const char* name,
+    int percentage,
+    base::TimeDelta interval_duration) {
+  UmaHistogramPercentage(base::StrCat({name, GetSuffixForCatchAllHistogram()}),
+                         percentage);
+  UmaHistogramPercentage(
+      base::StrCat({name, GetSuffixForHistogram(interval_duration)}),
+      percentage);
+}
+
+void ThreadController::RunLevelTracker::RunLevel::LogIntervalMetric(
+    const char* name,
+    base::TimeDelta value,
+    base::TimeDelta interval_duration) {
+  // Log towards "Any" time suffix first.
+  UmaHistogramTimes(base::StrCat({name, GetSuffixForCatchAllHistogram()}),
+                    value);
+  if (interval_duration < kNonTrivialActiveIntervalLength) {
+    UmaHistogramCustomMicrosecondsTimes(
+        base::StrCat({name, GetSuffixForHistogram(interval_duration)}), value,
+        base::Microseconds(1), kNonTrivialActiveIntervalLength, 100);
+  } else if (interval_duration < kMediumActiveIntervalLength) {
+    UmaHistogramCustomTimes(
+        base::StrCat({name, GetSuffixForHistogram(interval_duration)}), value,
+        kNonTrivialActiveIntervalLength, kMediumActiveIntervalLength, 100);
+  }
+}
+
+void ThreadController::RunLevelTracker::RunLevel::LogOnActiveMetrics(
+    LazyNow& lazy_now) {
+  CHECK(last_active_start_.is_null());
+  CHECK(last_active_threadtick_start_.is_null());
+
+  if (!last_active_end_.is_null()) {
+    const base::TimeDelta idle_time = lazy_now.Now() - last_active_end_;
+    LogIntervalMetric("Scheduling.ThreadController.IdleDuration", idle_time,
+                      idle_time);
+    last_active_end_ = base::TimeTicks();
+    accumulated_idle_time_ += idle_time;
+  }
+
+  // Taking thread ticks can be expensive. Make sure to do it rarely enough to
+  // not have a discernible impact on performance.
+  static const bool thread_ticks_supported = ThreadTicks::IsSupported();
+  // Disable subsampling to support wall-time based metrics. Only supported for
+  // testing purposes. By default, the subsampling probability is 0.1%.
+  const double probability =
+      time_keeper_->wall_time_based_metrics_enabled_for_testing() ? 1.0 : 0.001;
+  if (thread_ticks_supported &&
+      metrics_sub_sampler_.ShouldSample(probability)) {
+    last_active_start_ = lazy_now.Now();
+    last_active_threadtick_start_ = ThreadTicks::Now();
+  }
+}
+
+void ThreadController::RunLevelTracker::RunLevel::LogOnIdleMetrics(
+    LazyNow& lazy_now) {
+  if (!last_active_start_.is_null()) {
+    const base::TimeDelta elapsed_ticks = lazy_now.Now() - last_active_start_;
+    base::TimeDelta elapsed_thread_ticks =
+        ThreadTicks::Now() - last_active_threadtick_start_;
+
+    // Round to 100% in case of clock imprecisions making it look like
+    // there's impossibly more ThreadTicks than TimeTicks elapsed.
+    elapsed_thread_ticks = std::min(elapsed_thread_ticks, elapsed_ticks);
+
+    LogIntervalMetric("Scheduling.ThreadController.ActiveIntervalDuration",
+                      elapsed_ticks, elapsed_ticks);
+    LogIntervalMetric(
+        "Scheduling.ThreadController.ActiveIntervalOffCpuDuration",
+        elapsed_ticks - elapsed_thread_ticks, elapsed_ticks);
+    LogIntervalMetric("Scheduling.ThreadController.ActiveIntervalOnCpuDuration",
+                      elapsed_thread_ticks, elapsed_ticks);
+
+    // If the interval was shorter than a tick, 100% on-cpu time is assumed.
+    int active_interval_cpu_percentage =
+        elapsed_ticks.is_zero()
+            ? 100
+            : static_cast<int>(
+                  (elapsed_thread_ticks * 100).IntDiv(elapsed_ticks));
+
+    LogPercentageMetric(
+        "Scheduling.ThreadController.ActiveIntervalOnCpuPercentage",
+        active_interval_cpu_percentage, elapsed_ticks);
+
+    if (time_keeper_->wall_time_based_metrics_enabled_for_testing()) {
+      accumulated_active_time_ += elapsed_ticks;
+      accumulated_active_on_cpu_time_ += elapsed_thread_ticks;
+      accumulated_active_off_cpu_time_ +=
+          (elapsed_ticks - elapsed_thread_ticks);
+
+      // Accumulated wall-time since last wall-time based metric was stored.
+      const base::TimeDelta accumulated_wall_time =
+          accumulated_active_time_ + accumulated_idle_time_;
+
+      // Add wall-time based ratio metrics (in percent) when the total sum of
+      // active and idle times is larger than one second.
+      if (accumulated_wall_time > Seconds(1)) {
+        const int active_vs_wall_time_percentage = checked_cast<int>(
+            (accumulated_active_time_ * 100).IntDiv(accumulated_wall_time));
+        LogPercentageMetric(
+            "Scheduling.ThreadController.ActiveVsWallTimePercentage",
+            active_vs_wall_time_percentage);
+        const int active_on_cpu_vs_wall_time_percentage =
+            checked_cast<int>((accumulated_active_on_cpu_time_ * 100)
+                                  .IntDiv(accumulated_wall_time));
+        LogPercentageMetric(
+            "Scheduling.ThreadController.ActiveOnCpuVsWallTimePercentage",
+            active_on_cpu_vs_wall_time_percentage);
+        const int active_off_cpu_vs_wall_time_percentage =
+            checked_cast<int>((accumulated_active_off_cpu_time_ * 100)
+                                  .IntDiv(accumulated_wall_time));
+        LogPercentageMetric(
+            "Scheduling.ThreadController.ActiveOffCpuVsWallTimePercentage",
+            active_off_cpu_vs_wall_time_percentage);
+
+        accumulated_idle_time_ = base::TimeDelta();
+        accumulated_active_time_ = base::TimeDelta();
+        accumulated_active_on_cpu_time_ = base::TimeDelta();
+        accumulated_active_off_cpu_time_ = base::TimeDelta();
+      }
+    }
+
+    // Reset timings.
+    last_active_start_ = base::TimeTicks();
+    last_active_threadtick_start_ = base::ThreadTicks();
+    last_active_end_ = lazy_now.Now();
+  }
+}
+
+void ThreadController::RunLevelTracker::RunLevel::UpdateState(
+    State new_state,
+    LazyNow& lazy_now) {
   // The only state that can be redeclared is idle, anything else should be a
   // transition.
   DCHECK(state_ != new_state || new_state == kIdle)
@@ -267,19 +483,29 @@ void ThreadController::RunLevelTracker::RunLevel::UpdateState(State new_state) {
 
   // Change of state.
   if (is_active) {
+    LogOnActiveMetrics(lazy_now);
+
     // Flow emission is found at
     // ThreadController::RunLevelTracker::RecordScheduleWork.
-    TRACE_EVENT_BEGIN("base", "ThreadController active",
-                      *terminating_wakeup_flow_lambda_);
-    // Overriding the annotation from the previous RunLevel is intentional. Only
-    // the top RunLevel is ever updated, which holds the relevant state.
-    thread_controller_sample_metadata_.Set(
-        static_cast<int64_t>(++thread_controller_active_id_));
+    TRACE_EVENT_BEGIN("base", "ThreadController active", lazy_now.Now(),
+                      [&](perfetto::EventContext& ctx) {
+                        time_keeper_->MaybeEmitIncomingWakeupFlow(ctx);
+                      });
+
+    if (ShouldRecordSampleMetadata()) {
+      // Overriding the annotation from the previous RunLevel is intentional.
+      // Only the top RunLevel is ever updated, which holds the relevant state.
+      thread_controller_sample_metadata_.Set(
+          static_cast<int64_t>(++thread_controller_active_id_));
+    }
   } else {
-    thread_controller_sample_metadata_.Remove();
-    TRACE_EVENT_END("base");
-    // TODO(crbug.com/1021571): Remove this once fixed.
-    PERFETTO_INTERNAL_ADD_EMPTY_EVENT();
+    if (ShouldRecordSampleMetadata()) {
+      thread_controller_sample_metadata_.Remove();
+    }
+
+    LogOnIdleMetrics(lazy_now);
+
+    TRACE_EVENT_END("base", lazy_now.Now());
   }
 
   if (trace_observer_for_testing_) {
@@ -382,8 +608,8 @@ void ThreadController::RunLevelTracker::TimeKeeper::RecordEndOfPhase(
   if (is_tracing_enabled) {
     if (!was_tracing_enabled_) {
       // The first event name on the track hackily names the track...
-      // TODO(1006541): Use the Perfetto library to properly name this Track in
-      // EnableRecording above.
+      // TODO(crbug.com/42050015): Use the Perfetto library to properly name
+      // this Track in EnableRecording above.
       TRACE_EVENT_INSTANT(TRACE_DISABLED_BY_DEFAULT("base"),
                           "MessagePumpPhases", *perfetto_track_,
                           last_phase_end_ - Seconds(1));
@@ -402,6 +628,20 @@ void ThreadController::RunLevelTracker::TimeKeeper::RecordEndOfPhase(
   last_phase_end_ = phase_end;
 }
 
+void ThreadController::RunLevelTracker::TimeKeeper::MaybeEmitIncomingWakeupFlow(
+    perfetto::EventContext& ctx) {
+#if BUILDFLAG(ENABLE_BASE_TRACING)
+  static const uint8_t* flow_enabled =
+      TRACE_EVENT_API_GET_CATEGORY_GROUP_ENABLED("wakeup.flow");
+  if (!*flow_enabled) {
+    return;
+  }
+
+  perfetto::TerminatingFlow::ProcessScoped(
+      reinterpret_cast<uint64_t>(&(outer_.get())))(ctx);
+#endif
+}
+
 bool ThreadController::RunLevelTracker::TimeKeeper::ShouldRecordNow(
     ShouldRecordReqs reqs) {
   DCHECK_CALLED_ON_VALID_THREAD(
@@ -411,7 +651,7 @@ bool ThreadController::RunLevelTracker::TimeKeeper::ShouldRecordNow(
   // cycle in which `histogram_` is enabled. Only start recording from there.
   // Ignore any nested phases. `reqs` may indicate exceptions to this.
   //
-  // TODO(crbug.com/1329717): In a follow-up, we could probably always be
+  // TODO(crbug.com/40226913): In a follow-up, we could probably always be
   // tracking the phases of the pump and merely ignore the reporting if
   // `histogram_` isn't set.
   switch (reqs) {
@@ -480,7 +720,6 @@ const char* ThreadController::RunLevelTracker::TimeKeeper::PhaseToEventName(
       // kWorkItemSuspendedOnNested should be transformed into kNativeWork or
       // kApplicationTask before this point.
       NOTREACHED();
-      return "";
   }
 }
 
