@@ -5,16 +5,21 @@
 #ifndef BASE_TASK_SEQUENCE_MANAGER_THREAD_CONTROLLER_H_
 #define BASE_TASK_SEQUENCE_MANAGER_THREAD_CONTROLLER_H_
 
+#include <optional>
 #include <stack>
+#include <string>
+#include <string_view>
 #include <vector>
 
 #include "base/base_export.h"
 #include "base/check.h"
+#include "base/features.h"
 #include "base/memory/raw_ptr.h"
 #include "base/memory/raw_ref.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/message_loop/message_pump.h"
 #include "base/profiler/sample_metadata.h"
+#include "base/rand_util.h"
 #include "base/run_loop.h"
 #include "base/task/common/lazy_now.h"
 #include "base/task/sequence_manager/associated_thread_id.h"
@@ -25,7 +30,6 @@
 #include "base/trace_event/base_tracing.h"
 #include "base/tracing_buildflags.h"
 #include "build/build_config.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace base {
 
@@ -102,7 +106,7 @@ class BASE_EXPORT ThreadController {
   // immediate work into account.
   // TODO(kraynov): Remove |lazy_now| parameter.
   virtual void SetNextDelayedDoWork(LazyNow* lazy_now,
-                                    absl::optional<WakeUp> wake_up) = 0;
+                                    std::optional<WakeUp> wake_up) = 0;
 
   // Sets the sequenced task source from which to take tasks after
   // a Schedule*Work() call is made.
@@ -117,7 +121,7 @@ class BASE_EXPORT ThreadController {
 
   // Explicitly allow or disallow task execution. Implicitly disallowed when
   // entering a nested runloop.
-  virtual void SetTaskExecutionAllowed(bool allowed) = 0;
+  virtual void SetTaskExecutionAllowedInNativeNestedLoop(bool allowed) = 0;
 
   // Whether task execution is allowed or not.
   virtual bool IsTaskExecutionAllowed() const = 0;
@@ -141,12 +145,16 @@ class BASE_EXPORT ThreadController {
   virtual void DetachFromMessagePump() = 0;
 #endif
 
-  // Initializes the state of all the thread controller features. Must be
-  // invoked after FeatureList initialization.
-  static void InitializeFeatures();
+  // Initializes features for this class. See `base::features::Init()`.
+  static void InitializeFeatures(
+      features::EmitThreadControllerProfilerMetadata emit_profiler_metadata);
 
   // Enables TimeKeeper metrics. `thread_name` will be used as a suffix.
-  void EnableMessagePumpTimeKeeperMetrics(const char* thread_name);
+  // Setting `wall_time_based_metrics_enabled_for_testing` adds wall-time
+  // based metrics for this thread. It also also disables subsampling.
+  void EnableMessagePumpTimeKeeperMetrics(
+      const char* thread_name,
+      bool wall_time_based_metrics_enabled_for_testing);
 
   // Currently only overridden on ThreadControllerWithMessagePumpImpl.
   //
@@ -188,6 +196,9 @@ class BASE_EXPORT ThreadController {
   // would result in an inconsistent set of DCHECKs...
   raw_ptr<const TickClock> time_source_;  // Not owned.
 
+  // Whether or not wall-time based metrics are enabled.
+  bool wall_time_based_metrics_enabled_for_testing_;
+
   // Tracks the state of each run-level (main and nested ones) in its associated
   // ThreadController. It does so using two high-level principles:
   //  1) #work-in-work-implies-nested :
@@ -266,7 +277,9 @@ class BASE_EXPORT ThreadController {
     // RunLevelTracker.
     void RecordScheduleWork();
 
-    void EnableTimeKeeperMetrics(const char* thread_name);
+    void EnableTimeKeeperMetrics(
+        const char* thread_name,
+        bool wall_time_based_metrics_enabled_for_testing);
 
     // Observes changes of state sent as trace-events so they can be tested.
     class TraceObserverForTesting {
@@ -282,12 +295,6 @@ class BASE_EXPORT ThreadController {
         TraceObserverForTesting* trace_observer_for_testing);
 
    private:
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-    using TerminatingFlowLambda =
-        std::invoke_result<decltype(perfetto::TerminatingFlow::FromPointer),
-                           void*>::type;
-#endif
-
     // Keeps track of the time spent in various Phases (ignores idle), reports
     // via UMA to the corresponding phase every time one reaches >= 100ms of
     // cumulative time, resulting in a metric of relative time spent in each
@@ -298,7 +305,8 @@ class BASE_EXPORT ThreadController {
      public:
       explicit TimeKeeper(const RunLevelTracker& outer);
 
-      void EnableRecording(const char* thread_name);
+      void EnableRecording(const char* thread_name,
+                           bool wall_time_based_metrics_enabled_for_testing);
 
       // Records the start time of the first phase out-of-idle. The kScheduled
       // phase will be attributed the time before this point once its
@@ -318,6 +326,17 @@ class BASE_EXPORT ThreadController {
       // for it.
       void RecordEndOfPhase(Phase phase, LazyNow& lazy_now);
 
+      // If recording is enabled: If the `wakeup.flow` category is enabled,
+      // record a TerminatingFlow into the current "ThreadController Active"
+      // track event.
+      void MaybeEmitIncomingWakeupFlow(perfetto::EventContext& ctx);
+
+      const std::string& thread_name() const { return thread_name_; }
+
+      bool wall_time_based_metrics_enabled_for_testing() const {
+        return wall_time_based_metrics_enabled_for_testing_;
+      }
+
      private:
       enum class ShouldRecordReqs {
         // Regular should-record requirements.
@@ -340,6 +359,9 @@ class BASE_EXPORT ThreadController {
 
       static const char* PhaseToEventName(Phase phase);
 
+      std::string thread_name_;
+      // Whether or not wall-time based metrics are reported.
+      bool wall_time_based_metrics_enabled_for_testing_ = false;
       // Cumulative time deltas for each phase, reported and reset when >=100ms.
       std::array<TimeDelta, Phase::kLastPhase + 1> deltas_ = {};
       // Set at the start of the first work item out-of-idle. Consumed from the
@@ -350,7 +372,7 @@ class BASE_EXPORT ThreadController {
       TimeTicks last_phase_end_;
       // The end of the last kIdleWork phase. Used as a minimum for the next
       // kScheduled phase's begin (as it's possible that the next wake-up is
-      // scheduled during DoIdleWork adn we don't want overlapping phases).
+      // scheduled during DoIdleWork and we don't want overlapping phases).
       TimeTicks last_sleep_;
       // Assumes each kWorkItem is native unless OnApplicationTaskSelected() is
       // invoked in a given [OnWorkStarted, OnWorkEnded].
@@ -359,7 +381,7 @@ class BASE_EXPORT ThreadController {
       // non-null when recording is enabled.
       raw_ptr<HistogramBase> histogram_ = nullptr;
 #if BUILDFLAG(ENABLE_BASE_TRACING)
-      absl::optional<perfetto::Track> perfetto_track_;
+      std::optional<perfetto::Track> perfetto_track_;
 
       // True if tracing was enabled during the last pass of RecordTimeInPhase.
       bool was_tracing_enabled_ = false;
@@ -372,12 +394,7 @@ class BASE_EXPORT ThreadController {
       RunLevel(State initial_state,
                bool is_nested,
                TimeKeeper& time_keeper,
-               LazyNow& lazy_now
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-               ,
-               TerminatingFlowLambda& terminating_flow_lambda
-#endif
-      );
+               LazyNow& lazy_now);
       ~RunLevel();
 
       // Move-constructible for STL compat. Flags `other.was_moved_` so it noops
@@ -386,7 +403,7 @@ class BASE_EXPORT ThreadController {
       RunLevel(RunLevel&& other);
       RunLevel& operator=(RunLevel&&) = delete;
 
-      void UpdateState(State new_state);
+      void UpdateState(State new_state, LazyNow& lazy_now);
 
       State state() const { return state_; }
 
@@ -397,20 +414,43 @@ class BASE_EXPORT ThreadController {
       }
 
      private:
+      void LogPercentageMetric(const char* name, int value);
+      void LogPercentageMetric(const char* name,
+                               int value,
+                               base::TimeDelta interval_duration);
+      void LogIntervalMetric(const char* name,
+                             base::TimeDelta value,
+                             base::TimeDelta interval_duration);
+      void LogOnActiveMetrics(LazyNow& lazy_now);
+      void LogOnIdleMetrics(LazyNow& lazy_now);
+
+      base::TimeTicks last_active_end_;
+      base::TimeTicks last_active_start_;
+      base::ThreadTicks last_active_threadtick_start_;
+      base::TimeDelta accumulated_idle_time_;
+      base::TimeDelta accumulated_active_time_;
+      base::TimeDelta accumulated_active_on_cpu_time_;
+      base::TimeDelta accumulated_active_off_cpu_time_;
+      MetricsSubSampler metrics_sub_sampler_;
+
       State state_ = kIdle;
       bool is_nested_;
 
       bool ShouldRecordSampleMetadata();
 
+      // Get full suffix for histogram logging purposes. |duration| should equal
+      // TimeDelta() when not applicable.
+      std::string GetSuffixForHistogram(TimeDelta duration);
+
+      std::string GetSuffixForCatchAllHistogram();
+      std::string_view GetThreadName();
+
       const raw_ref<TimeKeeper> time_keeper_;
       // Must be set shortly before ~RunLevel.
       raw_ptr<LazyNow> exit_lazy_now_ = nullptr;
 
       SampleMetadata thread_controller_sample_metadata_;
       size_t thread_controller_active_id_ = 0;
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-      const raw_ref<TerminatingFlowLambda> terminating_wakeup_flow_lambda_;
-#endif
 
       // Toggles to true when used as RunLevel&& input to construct another
       // RunLevel. This RunLevel's destructor will then no-op.
@@ -431,11 +471,6 @@ class BASE_EXPORT ThreadController {
 
     [[maybe_unused]] const raw_ref<const ThreadController> outer_;
 
-#if BUILDFLAG(ENABLE_BASE_TRACING)
-    TerminatingFlowLambda terminating_wakeup_lambda_{
-        perfetto::TerminatingFlow::FromPointer(const_cast<RunLevelTracker*>(this))};
-#endif
-
     std::stack<RunLevel, std::vector<RunLevel>> run_levels_
         GUARDED_BY_CONTEXT(outer_->associated_thread_->thread_checker);
 
