@@ -4,6 +4,8 @@
 
 #include "base/task/sequence_manager/thread_controller_with_message_pump_impl.h"
 
+#include <array>
+#include <optional>
 #include <queue>
 #include <string>
 #include <utility>
@@ -12,6 +14,7 @@
 #include "base/functional/bind.h"
 #include "base/memory/raw_ptr.h"
 #include "base/memory/scoped_refptr.h"
+#include "base/message_loop/message_pump.h"
 #include "base/task/sequence_manager/task_queue.h"
 #include "base/task/sequence_manager/thread_controller_power_monitor.h"
 #include "base/task/single_thread_task_runner.h"
@@ -24,7 +27,6 @@
 #include "build/build_config.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 using testing::_;
 using testing::Invoke;
@@ -73,11 +75,13 @@ class ThreadControllerForTest : public ThreadControllerWithMessagePumpImpl {
     // EnableMessagePumpTimeKeeperMetrics is a no-op on hardware which lacks
     // high-res clocks.
     ASSERT_TRUE(TimeTicks::IsHighResolution());
-    EnableMessagePumpTimeKeeperMetrics("TestMainThread");
+    EnableMessagePumpTimeKeeperMetrics(
+        "TestMainThread",
+        /*wall_time_based_metrics_enabled_for_testing=*/false);
   }
 
   // Optionally emplaced, strict from then on.
-  absl::optional<testing::StrictMock<MockTraceObserver>> trace_observer_;
+  std::optional<testing::StrictMock<MockTraceObserver>> trace_observer_;
 };
 
 class MockMessagePump : public MessagePump {
@@ -96,7 +100,7 @@ class MockMessagePump : public MessagePump {
   }
 };
 
-// TODO(crbug.com/901373): Deduplicate FakeTaskRunners.
+// TODO(crbug.com/40600768): Deduplicate FakeTaskRunners.
 class FakeTaskRunner : public SingleThreadTaskRunner {
  public:
   bool PostDelayedTask(const Location& from_here,
@@ -125,16 +129,15 @@ class FakeSequencedTaskSource : public SequencedTaskSource {
   void SetRunTaskSynchronouslyAllowed(
       bool can_run_tasks_synchronously) override {}
 
-  absl::optional<SelectedTask> SelectNextTask(
-      LazyNow& lazy_now,
-      SelectTaskOption option) override {
+  std::optional<SelectedTask> SelectNextTask(LazyNow& lazy_now,
+                                             SelectTaskOption option) override {
     if (tasks_.empty())
-      return absl::nullopt;
+      return std::nullopt;
     if (tasks_.front().delayed_run_time > clock_->NowTicks())
-      return absl::nullopt;
+      return std::nullopt;
     if (option == SequencedTaskSource::SelectTaskOption::kSkipDelayedTask &&
         !tasks_.front().delayed_run_time.is_null()) {
-      return absl::nullopt;
+      return std::nullopt;
     }
     task_execution_stack_.push_back(std::move(tasks_.front()));
     tasks_.pop();
@@ -149,13 +152,13 @@ class FakeSequencedTaskSource : public SequencedTaskSource {
     task_execution_stack_.pop_back();
   }
 
-  absl::optional<WakeUp> GetPendingWakeUp(LazyNow* lazy_now,
-                                          SelectTaskOption option) override {
+  std::optional<WakeUp> GetPendingWakeUp(LazyNow* lazy_now,
+                                         SelectTaskOption option) override {
     if (tasks_.empty())
-      return absl::nullopt;
+      return std::nullopt;
     if (option == SequencedTaskSource::SelectTaskOption::kSkipDelayedTask &&
         !tasks_.front().delayed_run_time.is_null()) {
-      return absl::nullopt;
+      return std::nullopt;
     }
     if (tasks_.front().delayed_run_time.is_null())
       return WakeUp{};
@@ -658,7 +661,7 @@ TEST_F(ThreadControllerWithMessagePumpTest, PrioritizeYieldingToNative) {
         EXPECT_EQ(next_work_item.delayed_run_time, delayed_time);
         EXPECT_TRUE(next_work_item.yield_to_native);
 
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
       }));
 
   run_loop.Run();
@@ -713,9 +716,9 @@ TEST_F(ThreadControllerWithMessagePumpTest, NativeNestedMessageLoop) {
         testing::Mock::VerifyAndClearExpectations(message_pump_);
 
         EXPECT_FALSE(thread_controller_.IsTaskExecutionAllowed());
-        // SetTaskExecutionAllowed(true) should ScheduleWork.
+        // SetTaskExecutionAllowedInNativeNestedLoop(true) should ScheduleWork.
         EXPECT_CALL(*message_pump_, ScheduleWork());
-        thread_controller_.SetTaskExecutionAllowed(true);
+        thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(true);
         testing::Mock::VerifyAndClearExpectations(message_pump_);
 
         // There's no pending work so the native loop should go
@@ -732,7 +735,7 @@ TEST_F(ThreadControllerWithMessagePumpTest, NativeNestedMessageLoop) {
         thread_controller_.ScheduleWork();
         testing::Mock::VerifyAndClearExpectations(message_pump_);
 
-        thread_controller_.SetTaskExecutionAllowed(false);
+        thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(false);
 
         // Simulate a subsequent PostTask by the chromium task after
         // we've left the native loop. This should not ScheduleWork
@@ -782,7 +785,7 @@ TEST_F(ThreadControllerWithMessagePumpTest, RunWithTimeout) {
                   TimeTicks::Max());
 
         EXPECT_CALL(*message_pump_, Quit());
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
       }));
   thread_controller_.Run(true, Seconds(15));
 }
@@ -804,13 +807,13 @@ TEST_F(ThreadControllerWithMessagePumpTest, SetHighResolutionTimer) {
 
         // Ensures timer resolution is set to high resolution.
         task_source_.SetHasPendingHighResolutionTasks(true);
-        EXPECT_FALSE(delegate->DoIdleWork());
+        delegate->DoIdleWork();
         EXPECT_TRUE(
             thread_controller_.MainThreadOnlyForTesting().in_high_res_mode);
 
         // Ensures time resolution is set back to low resolution.
         task_source_.SetHasPendingHighResolutionTasks(false);
-        EXPECT_FALSE(delegate->DoIdleWork());
+        delegate->DoIdleWork();
         EXPECT_FALSE(
             thread_controller_.MainThreadOnlyForTesting().in_high_res_mode);
 
@@ -845,7 +848,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
 
         // The timer resolution should NOT be updated during power suspend.
         task_source_.SetHasPendingHighResolutionTasks(true);
-        EXPECT_FALSE(delegate->DoIdleWork());
+        delegate->DoIdleWork();
         EXPECT_FALSE(
             thread_controller_.MainThreadOnlyForTesting().in_high_res_mode);
 
@@ -853,7 +856,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
         thread_controller_.ThreadControllerPowerMonitorForTesting()->OnResume();
 
         // Ensures timer resolution is set to high resolution.
-        EXPECT_FALSE(delegate->DoIdleWork());
+        delegate->DoIdleWork();
         EXPECT_TRUE(
             thread_controller_.MainThreadOnlyForTesting().in_high_res_mode);
 
@@ -970,7 +973,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                       OnPhaseRecorded(ThreadController::kIdleWork));
           EXPECT_CALL(*thread_controller_.trace_observer_,
                       OnThreadControllerActiveEnd);
-          EXPECT_FALSE(thread_controller_.DoIdleWork());
+          thread_controller_.DoIdleWork();
 
           testing::Mock::VerifyAndClearExpectations(
               &*thread_controller_.trace_observer_);
@@ -994,7 +997,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
               OnThreadControllerActiveBegin);
   EXPECT_CALL(*message_pump_, Run(_))
       .WillOnce(Invoke([&](MessagePump::Delegate* delegate) {
-        MockCallback<OnceClosure> tasks[5];
+        std::array<MockCallback<OnceClosure>, 5> tasks;
         // Post 5 tasks, run them, go idle. Expected to only exit
         // "ThreadController active" state at the end.
         for (auto& t : tasks)
@@ -1024,7 +1027,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
       }));
 
   RunLoop().Run();
@@ -1074,7 +1077,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                       OnPhaseRecorded(ThreadController::kIdleWork));
           EXPECT_CALL(*thread_controller_.trace_observer_,
                       OnThreadControllerActiveEnd);
-          EXPECT_FALSE(thread_controller_.DoIdleWork());
+          thread_controller_.DoIdleWork();
 
           testing::Mock::VerifyAndClearExpectations(
               &*thread_controller_.trace_observer_);
@@ -1199,7 +1202,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                 // E:
                 EXPECT_CALL(*thread_controller_.trace_observer_,
                             OnThreadControllerActiveEnd);
-                EXPECT_FALSE(thread_controller_.DoIdleWork());
+                thread_controller_.DoIdleWork();
                 testing::Mock::VerifyAndClearExpectations(
                     &*thread_controller_.trace_observer_);
 
@@ -1253,7 +1256,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
 
@@ -1276,7 +1279,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1325,7 +1328,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           // C:
           EXPECT_FALSE(thread_controller_.IsTaskExecutionAllowed());
           EXPECT_CALL(*message_pump_, ScheduleWork());
-          thread_controller_.SetTaskExecutionAllowed(true);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(true);
           // i.e. simulate that something runs code within the scope of a
           // ScopedAllowApplicationTasksInNativeNestedLoop and ends up entering
           // a nested native loop which would invoke OnBeginWorkItem()
@@ -1353,7 +1356,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           // F:
           EXPECT_CALL(*thread_controller_.trace_observer_,
                       OnThreadControllerActiveEnd);
-          EXPECT_FALSE(thread_controller_.DoIdleWork());
+          thread_controller_.DoIdleWork();
           testing::Mock::VerifyAndClearExpectations(
               &*thread_controller_.trace_observer_);
 
@@ -1372,7 +1375,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           // doesn't resume as task (B) is done.
           EXPECT_CALL(*thread_controller_.trace_observer_,
                       OnPhaseRecorded(ThreadController::kNested));
-          thread_controller_.SetTaskExecutionAllowed(false);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(false);
         }));
 
         // B:
@@ -1386,7 +1389,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1430,10 +1433,10 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           // C:
           EXPECT_FALSE(thread_controller_.IsTaskExecutionAllowed());
           EXPECT_CALL(*message_pump_, ScheduleWork());
-          thread_controller_.SetTaskExecutionAllowed(true);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(true);
 
           // D:
-          thread_controller_.SetTaskExecutionAllowed(false);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(false);
           EXPECT_CALL(*thread_controller_.trace_observer_,
                       OnPhaseRecorded(ThreadController::kApplicationTask));
         }));
@@ -1462,7 +1465,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1547,7 +1550,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1596,7 +1599,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
             // C & F:
             EXPECT_FALSE(thread_controller_.IsTaskExecutionAllowed());
             EXPECT_CALL(*message_pump_, ScheduleWork());
-            thread_controller_.SetTaskExecutionAllowed(true);
+            thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(true);
 
             // D & G:
             if (i == 0) {
@@ -1612,7 +1615,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
             thread_controller_.OnEndWorkItem(run_level_depth + 1);
 
             // E & H:
-            thread_controller_.SetTaskExecutionAllowed(false);
+            thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(false);
             testing::Mock::VerifyAndClearExpectations(
                 &*thread_controller_.trace_observer_);
           }
@@ -1635,7 +1638,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1671,8 +1674,9 @@ TEST_F(ThreadControllerWithMessagePumpTest,
         // I: Go idle (exit active)
         //
         // This exercises the heuristic in
-        // ThreadControllerWithMessagePumpImpl::SetTaskExecutionAllowed() to
-        // detect the end of a nested native loop before the end of the task
+        // ThreadControllerWithMessagePumpImpl::
+        //    SetTaskExecutionAllowedInNativeNestedLoop()
+        // to detect the end of a nested native loop before the end of the task
         // that triggered it. When application tasks are not allowed however,
         // there's nothing we can do detect and two native nested loops in a
         // row. They may look like a single one if the first one is quit before
@@ -1690,7 +1694,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           // C:
           EXPECT_FALSE(thread_controller_.IsTaskExecutionAllowed());
           EXPECT_CALL(*message_pump_, ScheduleWork());
-          thread_controller_.SetTaskExecutionAllowed(true);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(true);
 
           // D:
           // kChromeTask phase is suspended when the nested loop is entered.
@@ -1719,7 +1723,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           thread_controller_.OnEndWorkItem(run_level_depth + 1);
 
           // G:
-          thread_controller_.SetTaskExecutionAllowed(false);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(false);
 
           // H:
           EXPECT_CALL(*thread_controller_.trace_observer_,
@@ -1739,7 +1743,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1786,7 +1790,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
           // C:
           EXPECT_FALSE(thread_controller_.IsTaskExecutionAllowed());
           EXPECT_CALL(*message_pump_, ScheduleWork());
-          thread_controller_.SetTaskExecutionAllowed(true);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(true);
 
           // D:
           // kChromeTask phase is suspended when the nested loop is entered.
@@ -1808,7 +1812,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
               &*thread_controller_.trace_observer_);
 
           // F + G:
-          thread_controller_.SetTaskExecutionAllowed(false);
+          thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(false);
 
           // H:
           EXPECT_CALL(*thread_controller_.trace_observer_,
@@ -1826,7 +1830,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1858,7 +1862,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
 
@@ -1911,7 +1915,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -1921,10 +1925,10 @@ TEST_F(ThreadControllerWithMessagePumpTest,
 
 // Same as ThreadControllerActiveNestedWithinNativeAllowsApplicationTasks but
 // with a dummy ScopedAllowApplicationTasksInNativeNestedLoop that is a
-// true=>true no-op for SetTaskExecutionAllowed(). This is a regression test
-// against another discussed implementation for RunLevelTracker which
-// would have used ScopedAllowApplicationTasksInNativeNestedLoop as a hint of
-// nested native loops. Doing so would have been incorrect because it assumes
+// true=>true no-op for SetTaskExecutionAllowedInNativeNestedLoop(). This is a
+// regression test against another discussed implementation for RunLevelTracker
+// which would have used ScopedAllowApplicationTasksInNativeNestedLoop as a hint
+// of nested native loops. Doing so would have been incorrect because it assumes
 // that ScopedAllowApplicationTasksInNativeNestedLoop always toggles the
 // allowance away-from and back-to |false|.
 TEST_F(ThreadControllerWithMessagePumpTest,
@@ -1946,7 +1950,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
 
@@ -1958,7 +1962,8 @@ TEST_F(ThreadControllerWithMessagePumpTest,
         //   C: Enter dummy ScopedAllowApplicationTasksInNativeNestedLoop
         //   D: Enter a native nested loop (application tasks still allowed)
         //     E: Run the application task (enter nested active)
-        //   F: Exit dummy scope (SetTaskExecutionAllowed(true)).
+        //   F: Exit dummy scope
+        //   (SetTaskExecutionAllowedInNativeNestedLoop(true)).
         // G: End the native task (exit nested active)
         // H: Go idle (exit active)
 
@@ -1971,7 +1976,8 @@ TEST_F(ThreadControllerWithMessagePumpTest,
               // C + D:
               EXPECT_TRUE(thread_controller_.IsTaskExecutionAllowed());
               EXPECT_CALL(*message_pump_, ScheduleWork());
-              thread_controller_.SetTaskExecutionAllowed(true);
+              thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(
+                  true);
               testing::Mock::VerifyAndClearExpectations(
                   &*thread_controller_.trace_observer_);
 
@@ -1989,7 +1995,8 @@ TEST_F(ThreadControllerWithMessagePumpTest,
 
               // F:
               EXPECT_CALL(*message_pump_, ScheduleWork());
-              thread_controller_.SetTaskExecutionAllowed(true);
+              thread_controller_.SetTaskExecutionAllowedInNativeNestedLoop(
+                  true);
             }));
 
         // B:
@@ -2009,7 +2016,7 @@ TEST_F(ThreadControllerWithMessagePumpTest,
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
       }));
@@ -2037,7 +2044,7 @@ TEST_F(ThreadControllerWithMessagePumpTest, MessagePumpPhasesWithQueuingTime) {
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
 
@@ -2082,7 +2089,7 @@ TEST_F(ThreadControllerWithMessagePumpTest, MessagePumpPhasesWithQueuingTime) {
                     OnPhaseRecorded(ThreadController::kIdleWork));
         EXPECT_CALL(*thread_controller_.trace_observer_,
                     OnThreadControllerActiveEnd);
-        EXPECT_FALSE(thread_controller_.DoIdleWork());
+        thread_controller_.DoIdleWork();
 
         testing::Mock::VerifyAndClearExpectations(
             &*thread_controller_.trace_observer_);
@@ -2091,4 +2098,140 @@ TEST_F(ThreadControllerWithMessagePumpTest, MessagePumpPhasesWithQueuingTime) {
   RunLoop().Run();
 }
 
+TEST_F(ThreadControllerWithMessagePumpNoBatchesTest,
+       WorkIdIncrementedForEveryWorkItem) {
+  SingleThreadTaskRunner::CurrentDefaultHandle handle(
+      MakeRefCounted<FakeTaskRunner>());
+  WorkIdProvider* work_id_provider = WorkIdProvider::GetForCurrentThread();
+
+  work_id_provider->SetCurrentWorkIdForTesting(0u);
+
+  EXPECT_CALL(*message_pump_, Run(_))
+      .WillOnce([&](MessagePump::Delegate* delegate) {
+        // Each task will increment work id by 2, once on begin work and another
+        // on end work.
+        delegate->DoWork();
+        EXPECT_EQ(work_id_provider->GetWorkId(), 2u);
+        delegate->DoWork();
+        EXPECT_EQ(work_id_provider->GetWorkId(), 4u);
+      });
+
+  for (int task_count = 0; task_count < 2; task_count++) {
+    task_source_.AddTask(FROM_HERE, DoNothing(), TimeTicks());
+  }
+
+  RunLoop run_loop;
+  run_loop.Run();
+  testing::Mock::VerifyAndClearExpectations(message_pump_);
+}
+
+TEST_F(ThreadControllerWithMessagePumpTest,
+       WorkIdIncrementedForEveryWorkItemInBatches) {
+  SingleThreadTaskRunner::CurrentDefaultHandle handle(
+      MakeRefCounted<FakeTaskRunner>());
+  thread_controller_.SetWorkBatchSize(2);
+
+  WorkIdProvider* work_id_provider = WorkIdProvider::GetForCurrentThread();
+  work_id_provider->SetCurrentWorkIdForTesting(0u);
+
+  EXPECT_CALL(*message_pump_, Run(_))
+      .WillOnce([&](MessagePump::Delegate* delegate) {
+        delegate->DoWork();
+        // Each task will increment work id by 2, once on begin work and another
+        // on end work.
+        EXPECT_EQ(work_id_provider->GetWorkId(), 4u);
+      });
+
+  for (int task_count = 0; task_count < 2; task_count++) {
+    task_source_.AddTask(FROM_HERE, DoNothing(), TimeTicks());
+  }
+
+  RunLoop run_loop;
+  run_loop.Run();
+  testing::Mock::VerifyAndClearExpectations(message_pump_);
+}
+
+TEST_F(ThreadControllerWithMessagePumpTest, WorkIdIncrementedForIdleWork) {
+  SingleThreadTaskRunner::CurrentDefaultHandle handle(
+      MakeRefCounted<FakeTaskRunner>());
+  WorkIdProvider* work_id_provider = WorkIdProvider::GetForCurrentThread();
+
+  work_id_provider->SetCurrentWorkIdForTesting(0u);
+
+  EXPECT_CALL(*message_pump_, Run(_))
+      .WillOnce([&](MessagePump::Delegate* delegate) {
+        delegate->DoIdleWork();
+        EXPECT_EQ(work_id_provider->GetWorkId(), 1u);
+      });
+
+  task_source_.AddTask(FROM_HERE, DoNothing());
+
+  RunLoop run_loop;
+  run_loop.Run();
+  testing::Mock::VerifyAndClearExpectations(message_pump_);
+}
+
+TEST_F(ThreadControllerWithMessagePumpTest, WorkIdIncrementedScopedDoWorkItem) {
+  SingleThreadTaskRunner::CurrentDefaultHandle handle(
+      MakeRefCounted<FakeTaskRunner>());
+  WorkIdProvider* work_id_provider = WorkIdProvider::GetForCurrentThread();
+
+  work_id_provider->SetCurrentWorkIdForTesting(0u);
+
+  EXPECT_CALL(*message_pump_, Run(_))
+      .WillOnce([&](MessagePump::Delegate* delegate) {
+        MessagePump::Delegate::ScopedDoWorkItem scoped_do_work_item =
+            delegate->BeginWorkItem();
+        // ScopedDoWorkItem will increment work id by 1 on construction and
+        // another on destruction.
+        EXPECT_EQ(work_id_provider->GetWorkId(), 1u);
+      });
+
+  RunLoop run_loop;
+  run_loop.Run();
+  testing::Mock::VerifyAndClearExpectations(message_pump_);
+  // Delegate::Run() itself will increment work id to account for pump overhead.
+  EXPECT_EQ(work_id_provider->GetWorkId(), 3u);
+}
+
+TEST_F(ThreadControllerWithMessagePumpTest,
+       WorkIdIncrementedDelegateBeforeWait) {
+  SingleThreadTaskRunner::CurrentDefaultHandle handle(
+      MakeRefCounted<FakeTaskRunner>());
+  WorkIdProvider* work_id_provider = WorkIdProvider::GetForCurrentThread();
+
+  work_id_provider->SetCurrentWorkIdForTesting(0u);
+
+  EXPECT_CALL(*message_pump_, Run(_))
+      .WillOnce([&](MessagePump::Delegate* delegate) {
+        // Delegate::BeforeWait will increment work id by 1 before waiting for
+        // work.
+        delegate->BeforeWait();
+        EXPECT_EQ(work_id_provider->GetWorkId(), 1u);
+      });
+
+  RunLoop run_loop;
+  run_loop.Run();
+  testing::Mock::VerifyAndClearExpectations(message_pump_);
+}
+
+TEST_F(ThreadControllerWithMessagePumpTest, WorkIdIncrementedDelegateRun) {
+  SingleThreadTaskRunner::CurrentDefaultHandle handle(
+      MakeRefCounted<FakeTaskRunner>());
+  WorkIdProvider* work_id_provider = WorkIdProvider::GetForCurrentThread();
+
+  work_id_provider->SetCurrentWorkIdForTesting(0u);
+
+  EXPECT_CALL(*message_pump_, Run(_))
+      .WillOnce([&](MessagePump::Delegate* delegate) {
+        EXPECT_EQ(work_id_provider->GetWorkId(), 0u);
+      });
+
+  RunLoop run_loop;
+  run_loop.Run();
+  testing::Mock::VerifyAndClearExpectations(message_pump_);
+  // Delegate::Run() itself will increment work id to account for pump overhead.
+  EXPECT_EQ(work_id_provider->GetWorkId(), 1u);
+}
+
 }  // namespace base::sequence_manager::internal
