@@ -9,6 +9,7 @@
 #include <algorithm>
 #include <atomic>
 #include <memory>
+#include <optional>
 #include <unordered_set>
 #include <utility>
 #include <vector>
@@ -51,7 +52,6 @@
 #include "base/timer/timer.h"
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace base {
 namespace internal {
@@ -102,9 +102,9 @@ class ThreadGroupImplImplTestBase : public ThreadGroup::Delegate {
   void StartThreadGroup(
       TimeDelta suggested_reclaim_time,
       size_t max_tasks,
-      absl::optional<int> max_best_effort_tasks = absl::nullopt,
+      std::optional<int> max_best_effort_tasks = std::nullopt,
       WorkerThreadObserver* worker_observer = nullptr,
-      absl::optional<TimeDelta> may_block_threshold = absl::nullopt) {
+      std::optional<TimeDelta> may_block_threshold = std::nullopt) {
     ASSERT_TRUE(thread_group_);
     thread_group_->Start(
         max_tasks,
@@ -117,9 +117,9 @@ class ThreadGroupImplImplTestBase : public ThreadGroup::Delegate {
   void CreateAndStartThreadGroup(
       TimeDelta suggested_reclaim_time = TimeDelta::Max(),
       size_t max_tasks = kMaxTasks,
-      absl::optional<int> max_best_effort_tasks = absl::nullopt,
+      std::optional<int> max_best_effort_tasks = std::nullopt,
       WorkerThreadObserver* worker_observer = nullptr,
-      absl::optional<TimeDelta> may_block_threshold = absl::nullopt) {
+      std::optional<TimeDelta> may_block_threshold = std::nullopt) {
     CreateThreadGroup();
     StartThreadGroup(suggested_reclaim_time, max_tasks, max_best_effort_tasks,
                      worker_observer, may_block_threshold);
@@ -319,7 +319,13 @@ TEST_F(ThreadGroupImplImplTest, ShouldYieldFloodedUserVisible) {
   scoped_refptr<JobTaskSource> task_source =
       job_task->GetJobTaskSource(FROM_HERE, {TaskPriority::USER_VISIBLE},
                                  &mock_pooled_task_runner_delegate_);
-  task_source->NotifyConcurrencyIncrease();
+
+  auto registered_task_source = task_tracker_.RegisterTaskSource(task_source);
+  ASSERT_TRUE(registered_task_source);
+  static_cast<ThreadGroup*>(thread_group_.get())
+      ->PushTaskSourceAndWakeUpWorkers(
+          RegisteredTaskSourceAndTransaction::FromTaskSource(
+              std::move(registered_task_source)));
 
   threads_running.Wait();
 
@@ -524,9 +530,9 @@ class BackgroundThreadGroupImplTest : public ThreadGroupImplImplTest {
   void CreateAndStartThreadGroup(
       TimeDelta suggested_reclaim_time = TimeDelta::Max(),
       size_t max_tasks = kMaxTasks,
-      absl::optional<int> max_best_effort_tasks = absl::nullopt,
+      std::optional<int> max_best_effort_tasks = std::nullopt,
       WorkerThreadObserver* worker_observer = nullptr,
-      absl::optional<TimeDelta> may_block_threshold = absl::nullopt) {
+      std::optional<TimeDelta> may_block_threshold = std::nullopt) {
     if (!CanUseBackgroundThreadTypeForWorkerThread())
       return;
     CreateThreadGroup(ThreadType::kBackground);
@@ -1070,7 +1076,7 @@ TEST_F(ThreadGroupImplBlockingTest, ThreadBlockUnblockPremature) {
   // MAY_BLOCK ScopedBlockingCall never increases the max tasks.
   CreateAndStartThreadGroup(TimeDelta::Max(),   // |suggested_reclaim_time|
                             kMaxTasks,          // |max_tasks|
-                            absl::nullopt,      // |max_best_effort_tasks|
+                            std::nullopt,       // |max_best_effort_tasks|
                             nullptr,            // |worker_observer|
                             TimeDelta::Max());  // |may_block_threshold|
 
@@ -1528,7 +1534,7 @@ TEST_F(ThreadGroupImplImplStartInBodyTest,
   constexpr size_t kNumWorkers = 2U;
   StartThreadGroup(TimeDelta::Max(),  // |suggested_reclaim_time|
                    kNumWorkers,       // |max_tasks|
-                   absl::nullopt);    // |max_best_effort_tasks|
+                   std::nullopt);     // |max_best_effort_tasks|
   const scoped_refptr<TaskRunner> runner = test::CreatePooledTaskRunner(
       {MayBlock()}, &mock_pooled_task_runner_delegate_);
 
@@ -1571,7 +1577,9 @@ class ThreadGroupImplBlockingCallAndMaxBestEffortTasksTest
     CreateThreadGroup();
     thread_group_->Start(kMaxTasks, kMaxBestEffortTasks, base::TimeDelta::Max(),
                          service_thread_.task_runner(), nullptr,
-                         ThreadGroup::WorkerEnvironment::NONE);
+                         ThreadGroup::WorkerEnvironment::NONE,
+                         /*synchronous_thread_start_for_testing=*/false,
+                         /*may_block_threshold=*/{});
   }
 
   void TearDown() override { ThreadGroupImplImplTestBase::CommonTearDown(); }
@@ -1647,7 +1655,9 @@ TEST_F(ThreadGroupImplImplStartInBodyTest, RacyCleanup) {
   thread_group_->Start(kLocalMaxTasks, kLocalMaxTasks,
                        kReclaimTimeForRacyCleanupTest,
                        service_thread_.task_runner(), nullptr,
-                       ThreadGroup::WorkerEnvironment::NONE);
+                       ThreadGroup::WorkerEnvironment::NONE,
+                       /*synchronous_thread_start_for_testing=*/false,
+                       /*may_block_threshold=*/{});
 
   scoped_refptr<TaskRunner> task_runner = test::CreatePooledTaskRunner(
       {WithBaseSyncPrimitives()}, &mock_pooled_task_runner_delegate_);
