@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -20,6 +20,7 @@ struct MockFoo {
   MOCK_METHOD(void, ByRef, (MoveOnly&), ());
   MOCK_METHOD(void, ByVal, (MoveOnly), ());
   MOCK_METHOD(void, TwiceByRef, (MoveOnly&, MoveOnly&), ());
+  MOCK_METHOD(bool, ByValWithReturnValue, (MoveOnly), ());
 };
 }  // namespace
 
@@ -58,3 +59,14 @@ TEST(GmockMoveSupportTest, MoveArgsTwiceByRef) {
   EXPECT_THAT(result1, Pointee(123));
   EXPECT_THAT(result2, Pointee(456));
 }
+
+TEST(GmockMoveSupportTest, MoveArgAndReturn) {
+  MoveOnly result;
+
+  MockFoo foo;
+  EXPECT_CALL(foo, ByValWithReturnValue)
+      .WillOnce(MoveArgAndReturn(&result, true));
+  EXPECT_TRUE(foo.ByValWithReturnValue(std::make_unique<int>(123)));
+
+  EXPECT_THAT(result, Pointee(123));
+}
