@@ -4,11 +4,12 @@
 
 #include "base/threading/platform_thread_win.h"
 
+#include <windows.h>
+
 #include <stddef.h>
 
 #include <string>
 
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_alloc_buildflags.h"
 #include "base/debug/alias.h"
 #include "base/debug/crash_logging.h"
 #include "base/debug/profiler.h"
@@ -28,12 +29,10 @@
 #include "base/win/scoped_handle.h"
 #include "base/win/windows_version.h"
 #include "build/build_config.h"
+#include "partition_alloc/buildflags.h"
 
-#include <windows.h>
-
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(USE_STARSCAN)
-#include "base/allocator/partition_allocator/src/partition_alloc/starscan/pcscan.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/starscan/stack/stack.h"
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#include "partition_alloc/stack/stack.h"
 #endif
 
 namespace base {
@@ -45,10 +44,6 @@ BASE_FEATURE(kAboveNormalCompositingBrowserWin,
              "AboveNormalCompositingBrowserWin",
              base::FEATURE_ENABLED_BY_DEFAULT);
 
-BASE_FEATURE(kBackgroundThreadNormalMemoryPriorityWin,
-             "BackgroundThreadNormalMemoryPriorityWin",
-             base::FEATURE_DISABLED_BY_DEFAULT);
-
 namespace {
 
 // Flag used to set thread priority to |THREAD_PRIORITY_LOWEST| for
@@ -57,9 +52,6 @@ std::atomic<bool> g_use_thread_priority_lowest{false};
 // Flag used to map Compositing ThreadType |THREAD_PRIORITY_ABOVE_NORMAL| on the
 // UI thread for |kAboveNormalCompositingBrowserWin| Feature.
 std::atomic<bool> g_above_normal_compositing_browser{true};
-// Flag used to set thread memory priority to |MEMORY_PRIORITY_NORMAL| on
-// background threads for |kThreadNormalMemoryPriorityWin| Feature.
-std::atomic<bool> g_background_thread_normal_memory_priority_win{false};
 
 // These values are sometimes returned by ::GetThreadPriority().
 constexpr int kWinDisplayPriority1 = 5;
@@ -124,9 +116,8 @@ DWORD __stdcall ThreadFunc(void* params) {
                                 FALSE,
                                 DUPLICATE_SAME_ACCESS);
 
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(USE_STARSCAN)
-  partition_alloc::internal::PCScan::NotifyThreadCreated(
-      partition_alloc::internal::GetStackPointer());
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+  partition_alloc::internal::StackTopRegistry::Get().NotifyThreadCreated();
 #endif
 
   win::ScopedHandle scoped_platform_handle;
@@ -145,8 +136,8 @@ DWORD __stdcall ThreadFunc(void* params) {
                                                    PlatformThread::CurrentId());
   }
 
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(USE_STARSCAN)
-  partition_alloc::internal::PCScan::NotifyThreadDestroyed();
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+  partition_alloc::internal::StackTopRegistry::Get().NotifyThreadDestroyed();
 #endif
 
   // Ensure thread priority is at least NORMAL before initiating thread
@@ -381,10 +372,10 @@ namespace {
 
 void SetCurrentThreadPriority(ThreadType thread_type,
                               MessagePumpType pump_type_hint) {
-  if (thread_type == ThreadType::kCompositing &&
+  if (thread_type == ThreadType::kDisplayCritical &&
       pump_type_hint == MessagePumpType::UI &&
       !g_above_normal_compositing_browser) {
-    // Ignore kCompositing thread type for UI thread as Windows has a
+    // Ignore kDisplayCritical thread type for UI thread as Windows has a
     // priority boost mechanism. See
     // https://docs.microsoft.com/en-us/windows/win32/procthread/priority-boosts
     return;
@@ -424,7 +415,6 @@ void SetCurrentThreadPriority(ThreadType thread_type,
     case ThreadType::kDefault:
       desired_priority = THREAD_PRIORITY_NORMAL;
       break;
-    case ThreadType::kCompositing:
     case ThreadType::kDisplayCritical:
       desired_priority = THREAD_PRIORITY_ABOVE_NORMAL;
       break;
@@ -439,8 +429,7 @@ void SetCurrentThreadPriority(ThreadType thread_type,
   DPLOG_IF(ERROR, !cpu_priority_success)
       << "Failed to set thread priority to " << desired_priority;
 
-  if (g_background_thread_normal_memory_priority_win &&
-      desired_priority == THREAD_MODE_BACKGROUND_BEGIN) {
+  if (desired_priority == THREAD_MODE_BACKGROUND_BEGIN) {
     // Override the memory priority.
     MEMORY_PRIORITY_INFORMATION memory_priority{.MemoryPriority =
                                                     MEMORY_PRIORITY_NORMAL};
@@ -476,7 +465,6 @@ void SetCurrentThreadQualityOfService(ThreadType thread_type) {
       desire_ecoqos = true;
       break;
     case ThreadType::kDefault:
-    case ThreadType::kCompositing:
     case ThreadType::kDisplayCritical:
     case ThreadType::kRealtimeAudio:
       desire_ecoqos = false;
@@ -567,7 +555,6 @@ ThreadPriorityForTest PlatformThread::GetCurrentThreadPriorityForTest() {
   }
 
   NOTREACHED() << "::GetThreadPriority returned " << priority << ".";
-  return ThreadPriorityForTest::kNormal;
 }
 
 void InitializePlatformThreadFeatures() {
@@ -577,9 +564,6 @@ void InitializePlatformThreadFeatures() {
   g_above_normal_compositing_browser.store(
       FeatureList::IsEnabled(kAboveNormalCompositingBrowserWin),
       std::memory_order_relaxed);
-  g_background_thread_normal_memory_priority_win.store(
-      FeatureList::IsEnabled(kBackgroundThreadNormalMemoryPriorityWin),
-      std::memory_order_relaxed);
 }
 
 // static
