@@ -2,18 +2,18 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/trace_event/malloc_dump_provider.h"
 
 #include <stddef.h>
 
 #include <unordered_map>
 
-#include "base/allocator/allocator_extension.h"
 #include "base/allocator/buildflags.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_alloc_buildflags.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_alloc_config.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_bucket_lookup.h"
-#include "base/allocator/partition_allocator/src/partition_alloc/shim/nonscannable_allocator.h"
 #include "base/debug/profiler.h"
 #include "base/format_macros.h"
 #include "base/metrics/histogram_functions.h"
@@ -22,6 +22,9 @@
 #include "base/trace_event/process_memory_dump.h"
 #include "base/trace_event/traced_value.h"
 #include "build/build_config.h"
+#include "partition_alloc/buildflags.h"
+#include "partition_alloc/partition_alloc_config.h"
+#include "partition_alloc/partition_bucket_lookup.h"
 
 #if BUILDFLAG(IS_APPLE)
 #include <malloc/malloc.h>
@@ -36,12 +39,13 @@
 #include <features.h>
 #endif
 
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
-#include "base/allocator/partition_allocator/src/partition_alloc/shim/allocator_shim_default_dispatch_to_partition_alloc.h"
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#include "base/no_destructor.h"
+#include "partition_alloc/shim/allocator_shim_default_dispatch_to_partition_alloc.h"
 #endif
 
 #if PA_CONFIG(THREAD_CACHE_ALLOC_STATS)
-#include "base/allocator/partition_allocator/src/partition_alloc/partition_alloc_constants.h"
+#include "partition_alloc/partition_alloc_constants.h"
 #endif
 
 namespace base {
@@ -110,7 +114,7 @@ void ReportWinHeapStats(MemoryDumpLevelOfDetail level_of_detail,
 }
 #endif  // BUILDFLAG(IS_WIN)
 
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
 void ReportPartitionAllocStats(ProcessMemoryDump* pmd,
                                MemoryDumpLevelOfDetail level_of_detail,
                                size_t* total_virtual_size,
@@ -133,30 +137,22 @@ void ReportPartitionAllocStats(ProcessMemoryDump* pmd,
     original_allocator->DumpStats("original", is_light_dump,
                                   &partition_stats_dumper);
   }
-  auto& nonscannable_allocator =
-      allocator_shim::NonScannableAllocator::Instance();
-  if (auto* root = nonscannable_allocator.root())
-    root->DumpStats("nonscannable", is_light_dump, &partition_stats_dumper);
-  auto& nonquarantinable_allocator =
-      allocator_shim::NonQuarantinableAllocator::Instance();
-  if (auto* root = nonquarantinable_allocator.root())
-    root->DumpStats("nonquarantinable", is_light_dump, &partition_stats_dumper);
 
   *total_virtual_size += partition_stats_dumper.total_resident_bytes();
   *resident_size += partition_stats_dumper.total_resident_bytes();
   *allocated_objects_size += partition_stats_dumper.total_active_bytes();
   *allocated_objects_count += partition_stats_dumper.total_active_count();
   *syscall_count += partition_stats_dumper.syscall_count();
-#if BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#if PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
   *cumulative_brp_quarantined_size +=
       partition_stats_dumper.cumulative_brp_quarantined_bytes();
   *cumulative_brp_quarantined_count +=
       partition_stats_dumper.cumulative_brp_quarantined_count();
-#endif  // BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#endif  // PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
 }
-#endif  // BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#endif  // PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
 
-#if !BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(IS_APPLE)
+#if !PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(IS_APPLE)
 void ReportAppleAllocStats(size_t* total_virtual_size,
                            size_t* resident_size,
                            size_t* allocated_objects_size) {
@@ -180,8 +176,8 @@ void ReportAppleAllocStats(size_t* total_virtual_size,
 }
 #endif
 
-#if (BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(IS_ANDROID)) || \
-    (!BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && !BUILDFLAG(IS_WIN) &&    \
+#if (PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && BUILDFLAG(IS_ANDROID)) || \
+    (!PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC) && !BUILDFLAG(IS_WIN) &&    \
      !BUILDFLAG(IS_APPLE) && !BUILDFLAG(IS_FUCHSIA))
 void ReportMallinfoStats(ProcessMemoryDump* pmd,
                          size_t* total_virtual_size,
@@ -218,7 +214,7 @@ void ReportMallinfoStats(ProcessMemoryDump* pmd,
 }
 #endif
 
-#if BUILDFLAG(USE_PARTITION_ALLOC)
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC)
 void ReportPartitionAllocThreadCacheStats(
     ProcessMemoryDump* pmd,
     MemoryAllocatorDump* dump,
@@ -252,40 +248,40 @@ void ReportPartitionAllocThreadCacheStats(
   dump->AddScalar("metadata_overhead", MemoryAllocatorDump::kUnitsBytes,
                   stats.metadata_overhead);
 
-  if (stats.alloc_count) {
-    int hit_rate_percent =
-        static_cast<int>((100 * stats.alloc_hits) / stats.alloc_count);
-    base::UmaHistogramPercentage(
-        "Memory.PartitionAlloc.ThreadCache.HitRate" + metrics_suffix,
-        hit_rate_percent);
-    int batch_fill_rate_percent =
-        static_cast<int>((100 * stats.batch_fill_count) / stats.alloc_count);
-    base::UmaHistogramPercentage(
-        "Memory.PartitionAlloc.ThreadCache.BatchFillRate" + metrics_suffix,
-        batch_fill_rate_percent);
-
 #if PA_CONFIG(THREAD_CACHE_ALLOC_STATS)
-    if (detailed) {
-      partition_alloc::internal::BucketIndexLookup lookup{};
-      std::string name = dump->absolute_name();
-      for (size_t i = 0; i < partition_alloc::kNumBuckets; i++) {
-        size_t bucket_size = lookup.bucket_sizes()[i];
-        if (bucket_size == partition_alloc::kInvalidBucketSize)
-          continue;
-        // Covers all normal buckets, that is up to ~1MiB, so 7 digits.
-        std::string dump_name =
-            base::StringPrintf("%s/buckets_alloc/%07d", name.c_str(),
-                               static_cast<int>(bucket_size));
-        auto* buckets_alloc_dump = pmd->CreateAllocatorDump(dump_name);
-        buckets_alloc_dump->AddScalar("count",
-                                      MemoryAllocatorDump::kUnitsObjects,
-                                      stats.allocs_per_bucket_[i]);
+  if (stats.alloc_count && detailed) {
+    partition_alloc::internal::BucketIndexLookup lookup{};
+    std::string name = dump->absolute_name();
+    for (size_t i = 0; i < partition_alloc::kNumBuckets; i++) {
+      size_t bucket_size = lookup.bucket_sizes()[i];
+      if (bucket_size == partition_alloc::kInvalidBucketSize) {
+        continue;
       }
+      // Covers all normal buckets, that is up to ~1MiB, so 7 digits.
+      std::string dump_name = base::StringPrintf(
+          "%s/buckets_alloc/%07d", name.c_str(), static_cast<int>(bucket_size));
+      auto* buckets_alloc_dump = pmd->CreateAllocatorDump(dump_name);
+      buckets_alloc_dump->AddScalar("count", MemoryAllocatorDump::kUnitsObjects,
+                                    stats.allocs_per_bucket_[i]);
     }
-#endif  // PA_CONFIG(THREAD_CACHE_ALLOC_STATS)
   }
+#endif  // PA_CONFIG(THREAD_CACHE_ALLOC_STATS)
 }
-#endif  // BUILDFLAG(USE_PARTITION_ALLOC)
+
+void ReportPartitionAllocLightweightQuarantineStats(
+    MemoryAllocatorDump* dump,
+    const partition_alloc::LightweightQuarantineStats& stats) {
+  dump->AddScalar("count", MemoryAllocatorDump::kUnitsObjects, stats.count);
+  dump->AddScalar("size_in_bytes", MemoryAllocatorDump::kUnitsBytes,
+                  stats.size_in_bytes);
+  dump->AddScalar("cumulative_count", MemoryAllocatorDump::kUnitsObjects,
+                  stats.cumulative_count);
+  dump->AddScalar("cumulative_size_in_bytes", MemoryAllocatorDump::kUnitsBytes,
+                  stats.cumulative_size_in_bytes);
+  dump->AddScalar("quarantine_miss_count", MemoryAllocatorDump::kUnitsObjects,
+                  stats.quarantine_miss_count);
+}
+#endif  // PA_BUILDFLAG(USE_PARTITION_ALLOC)
 
 }  // namespace
 
@@ -298,6 +294,25 @@ MallocDumpProvider* MallocDumpProvider::GetInstance() {
                    LeakySingletonTraits<MallocDumpProvider>>::get();
 }
 
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+// static
+void MallocDumpProvider::SetExtremeLUDGetStatsCallback(
+    ExtremeLUDGetStatsCallback callback) {
+  DCHECK(!callback.is_null());
+  auto& extreme_lud_get_stats_callback = GetExtremeLUDGetStatsCallback();
+  DCHECK(extreme_lud_get_stats_callback.is_null());
+  extreme_lud_get_stats_callback = std::move(callback);
+}
+
+// static
+MallocDumpProvider::ExtremeLUDGetStatsCallback&
+MallocDumpProvider::GetExtremeLUDGetStatsCallback() {
+  static NoDestructor<MallocDumpProvider::ExtremeLUDGetStatsCallback>
+      extreme_lud_get_stats_callback;
+  return *extreme_lud_get_stats_callback;
+}
+#endif  // PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+
 MallocDumpProvider::MallocDumpProvider() = default;
 MallocDumpProvider::~MallocDumpProvider() = default;
 
@@ -319,12 +334,12 @@ bool MallocDumpProvider::OnMemoryDump(const MemoryDumpArgs& args,
   uint64_t syscall_count = 0;
   size_t cumulative_brp_quarantined_size = 0;
   size_t cumulative_brp_quarantined_count = 0;
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
   uint64_t pa_only_resident_size;
   uint64_t pa_only_allocated_objects_size;
 #endif
 
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
   ReportPartitionAllocStats(
       pmd, args.level_of_detail, &total_virtual_size, &resident_size,
       &allocated_objects_size, &allocated_objects_count, &syscall_count,
@@ -374,12 +389,6 @@ bool MallocDumpProvider::OnMemoryDump(const MemoryDumpArgs& args,
                           allocated_objects_count);
   }
 
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
-  base::trace_event::MemoryAllocatorDump* partitions_dump =
-      pmd->CreateAllocatorDump("malloc/partitions");
-  pmd->AddOwnershipEdge(inner_dump->guid(), partitions_dump->guid());
-#endif  // BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
-
   int64_t waste = static_cast<int64_t>(resident_size - allocated_objects_size);
 
   // With PartitionAlloc, reported size under malloc/partitions is the resident
@@ -389,7 +398,7 @@ bool MallocDumpProvider::OnMemoryDump(const MemoryDumpArgs& args,
   //
   // Still report waste, as on some platforms, PartitionAlloc doesn't capture
   // all of malloc()'s memory footprint.
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
   int64_t pa_waste = static_cast<int64_t>(pa_only_resident_size -
                                           pa_only_allocated_objects_size);
   waste -= pa_waste;
@@ -405,14 +414,37 @@ bool MallocDumpProvider::OnMemoryDump(const MemoryDumpArgs& args,
                           static_cast<uint64_t>(waste));
   }
 
-  ReportPerMinuteStats(syscall_count, cumulative_brp_quarantined_size,
-                       cumulative_brp_quarantined_count, outer_dump,
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
-                       partitions_dump
-#else
-                       nullptr
-#endif
-  );
+  base::trace_event::MemoryAllocatorDump* partitions_dump = nullptr;
+  base::trace_event::MemoryAllocatorDump* elud_dump_for_small_objects = nullptr;
+  ExtremeLUDStats elud_stats_for_small_objects;
+  base::trace_event::MemoryAllocatorDump* elud_dump_for_large_objects = nullptr;
+  ExtremeLUDStats elud_stats_for_large_objects;
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+  partitions_dump = pmd->CreateAllocatorDump("malloc/partitions");
+  pmd->AddOwnershipEdge(inner_dump->guid(), partitions_dump->guid());
+
+  auto& extreme_lud_get_stats_callback = GetExtremeLUDGetStatsCallback();
+  if (!extreme_lud_get_stats_callback.is_null()) {
+    // The Extreme LUD is enabled.
+    elud_dump_for_small_objects =
+        pmd->CreateAllocatorDump("malloc/extreme_lud/small_objects");
+    elud_dump_for_large_objects =
+        pmd->CreateAllocatorDump("malloc/extreme_lud/large_objects");
+    const auto elud_stats_set = extreme_lud_get_stats_callback.Run();
+    elud_stats_for_small_objects = elud_stats_set.for_small_objects;
+    elud_stats_for_large_objects = elud_stats_set.for_large_objects;
+    ReportPartitionAllocLightweightQuarantineStats(
+        elud_dump_for_small_objects, elud_stats_for_small_objects.lq_stats);
+    ReportPartitionAllocLightweightQuarantineStats(
+        elud_dump_for_large_objects, elud_stats_for_large_objects.lq_stats);
+  }
+#endif  // PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+
+  ReportPerMinuteStats(
+      syscall_count, cumulative_brp_quarantined_size,
+      cumulative_brp_quarantined_count, elud_stats_for_small_objects,
+      elud_stats_for_large_objects, outer_dump, partitions_dump,
+      elud_dump_for_small_objects, elud_dump_for_large_objects);
 
   return true;
 }
@@ -421,9 +453,13 @@ void MallocDumpProvider::ReportPerMinuteStats(
     uint64_t syscall_count,
     size_t cumulative_brp_quarantined_bytes,
     size_t cumulative_brp_quarantined_count,
+    const ExtremeLUDStats& elud_stats_for_small_objects,
+    const ExtremeLUDStats& elud_stats_for_large_objects,
     MemoryAllocatorDump* malloc_dump,
-    MemoryAllocatorDump* partition_alloc_dump) {
-#if BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+    MemoryAllocatorDump* partition_alloc_dump,
+    MemoryAllocatorDump* elud_dump_for_small_objects,
+    MemoryAllocatorDump* elud_dump_for_large_objects) {
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
   uint64_t new_syscalls = syscall_count - last_syscall_count_;
   size_t new_brp_quarantined_bytes =
       cumulative_brp_quarantined_bytes - last_cumulative_brp_quarantined_bytes_;
@@ -431,14 +467,15 @@ void MallocDumpProvider::ReportPerMinuteStats(
       cumulative_brp_quarantined_count - last_cumulative_brp_quarantined_count_;
   base::TimeDelta time_since_last_dump =
       base::TimeTicks::Now() - last_memory_dump_time_;
-  uint64_t syscalls_per_minute = static_cast<uint64_t>(
-      (60 * new_syscalls) / time_since_last_dump.InSecondsF());
+  auto seconds_since_last_dump = time_since_last_dump.InSecondsF();
+  uint64_t syscalls_per_minute =
+      static_cast<uint64_t>((60 * new_syscalls) / seconds_since_last_dump);
   malloc_dump->AddScalar("syscalls_per_minute", "count", syscalls_per_minute);
   if (partition_alloc_dump) {
     size_t brp_quarantined_bytes_per_minute =
-        (60 * new_brp_quarantined_bytes) / time_since_last_dump.InSecondsF();
+        (60 * new_brp_quarantined_bytes) / seconds_since_last_dump;
     size_t brp_quarantined_count_per_minute =
-        (60 * new_brp_quarantined_count) / time_since_last_dump.InSecondsF();
+        (60 * new_brp_quarantined_count) / seconds_since_last_dump;
     partition_alloc_dump->AddScalar("brp_quarantined_bytes_per_minute",
                                     MemoryAllocatorDump::kUnitsBytes,
                                     brp_quarantined_bytes_per_minute);
@@ -447,14 +484,76 @@ void MallocDumpProvider::ReportPerMinuteStats(
                                     brp_quarantined_count_per_minute);
   }
 
+  auto report_elud_per_minute_stats = [time_since_last_dump,
+                                       seconds_since_last_dump](
+                                          const ExtremeLUDStats& elud_stats,
+                                          CumulativeEludStats&
+                                              last_cumulative_elud_stats,
+                                          MemoryAllocatorDump* elud_dump) {
+    size_t bytes = elud_stats.lq_stats.cumulative_size_in_bytes -
+                   last_cumulative_elud_stats.quarantined_bytes;
+    size_t count = elud_stats.lq_stats.cumulative_count -
+                   last_cumulative_elud_stats.quarantined_count;
+    size_t miss_count = elud_stats.lq_stats.quarantine_miss_count -
+                        last_cumulative_elud_stats.miss_count;
+    elud_dump->AddScalar("bytes_per_minute", MemoryAllocatorDump::kUnitsBytes,
+                         60ull * bytes / seconds_since_last_dump);
+    elud_dump->AddScalar("count_per_minute",
+                         MemoryAllocatorDump::kNameObjectCount,
+                         60ull * count / seconds_since_last_dump);
+    elud_dump->AddScalar("miss_count_per_minute",
+                         MemoryAllocatorDump::kNameObjectCount,
+                         60ull * miss_count / seconds_since_last_dump);
+    // Given the following three:
+    //   capacity := the quarantine storage space
+    //   time     := the elapsed time since the last dump
+    //   bytes    := the consumed/used bytes since the last dump
+    // We can define/calculate the following.
+    //   speed    := the consuming speed of the quarantine
+    //            = bytes / time
+    //   quarantined_time
+    //            := the time to use up the capacity
+    //               (near to how long an object may be quarantined)
+    //            = capacity / speed
+    //            = capacity / (bytes / time)
+    //            = time * capacity / bytes
+    //
+    // Note that objects in the quarantine are randomly evicted. So objects may
+    // stay in the qurantine longer or shorter depending on object sizes,
+    // allocation/deallocation patterns, etc. in addition to pure randomness.
+    // So, this is just a rough estimation, not necessarily to be the average.
+    if (bytes > 0) {
+      elud_dump->AddScalar(
+          "quarantined_time", "msec",
+          static_cast<uint64_t>(time_since_last_dump.InMilliseconds()) *
+              elud_stats.capacity_in_bytes / bytes);
+    }
+    last_cumulative_elud_stats.quarantined_bytes =
+        elud_stats.lq_stats.cumulative_size_in_bytes;
+    last_cumulative_elud_stats.quarantined_count =
+        elud_stats.lq_stats.cumulative_count;
+    last_cumulative_elud_stats.miss_count =
+        elud_stats.lq_stats.quarantine_miss_count;
+  };
+  if (elud_dump_for_small_objects) {
+    report_elud_per_minute_stats(elud_stats_for_small_objects,
+                                 last_cumulative_elud_stats_for_small_objects_,
+                                 elud_dump_for_small_objects);
+  }
+  if (elud_dump_for_large_objects) {
+    report_elud_per_minute_stats(elud_stats_for_large_objects,
+                                 last_cumulative_elud_stats_for_large_objects_,
+                                 elud_dump_for_large_objects);
+  }
+
   last_memory_dump_time_ = base::TimeTicks::Now();
   last_syscall_count_ = syscall_count;
   last_cumulative_brp_quarantined_bytes_ = cumulative_brp_quarantined_bytes;
   last_cumulative_brp_quarantined_count_ = cumulative_brp_quarantined_count;
-#endif  // BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
+#endif  // PA_BUILDFLAG(USE_PARTITION_ALLOC_AS_MALLOC)
 }
 
-#if BUILDFLAG(USE_PARTITION_ALLOC)
+#if PA_BUILDFLAG(USE_PARTITION_ALLOC)
 std::string GetPartitionDumpName(const char* root_name,
                                  const char* partition_name) {
   return base::StringPrintf("%s/%s/%s", root_name,
@@ -478,12 +577,12 @@ void MemoryDumpPartitionStatsDumper::PartitionDumpTotals(
   total_active_bytes_ += memory_stats->total_active_bytes;
   total_active_count_ += memory_stats->total_active_count;
   syscall_count_ += memory_stats->syscall_count;
-#if BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#if PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
   cumulative_brp_quarantined_bytes_ +=
       memory_stats->cumulative_brp_quarantined_bytes;
   cumulative_brp_quarantined_count_ +=
       memory_stats->cumulative_brp_quarantined_count;
-#endif  // BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#endif  // PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
 
   std::string dump_name = GetPartitionDumpName(root_name_, partition_name);
   MemoryAllocatorDump* allocator_dump =
@@ -523,13 +622,13 @@ void MemoryDumpPartitionStatsDumper::PartitionDumpTotals(
   allocator_dump->AddScalar("discardable_size",
                             MemoryAllocatorDump::kUnitsBytes,
                             memory_stats->total_discardable_bytes);
-#if BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#if PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
   allocator_dump->AddScalar("brp_quarantined_size",
                             MemoryAllocatorDump::kUnitsBytes,
                             memory_stats->total_brp_quarantined_bytes);
   allocator_dump->AddScalar("brp_quarantined_count", "count",
                             memory_stats->total_brp_quarantined_count);
-#endif  // BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
+#endif  // PA_BUILDFLAG(ENABLE_BACKUP_REF_PTR_SUPPORT)
   allocator_dump->AddScalar("syscall_count", "count",
                             memory_stats->syscall_count);
   allocator_dump->AddScalar("syscall_total_time_ms", "ms",
@@ -552,6 +651,15 @@ void MemoryDumpPartitionStatsDumper::PartitionDumpTotals(
                                          all_thread_caches_stats, "",
                                          detailed_);
   }
+
+  if (memory_stats->has_scheduler_loop_quarantine) {
+    MemoryAllocatorDump* quarantine_dump_total =
+        memory_dump_->CreateAllocatorDump(dump_name +
+                                          "/scheduler_loop_quarantine");
+    ReportPartitionAllocLightweightQuarantineStats(
+        quarantine_dump_total,
+        memory_stats->scheduler_loop_quarantine_stats_total);
+  }
 }
 
 void MemoryDumpPartitionStatsDumper::PartitionsDumpBucketStats(
@@ -600,7 +708,7 @@ void MemoryDumpPartitionStatsDumper::PartitionsDumpBucketStats(
                             MemoryAllocatorDump::kUnitsObjects,
                             memory_stats->num_decommitted_slot_spans);
 }
-#endif  // BUILDFLAG(USE_PARTITION_ALLOC)
+#endif  // PA_BUILDFLAG(USE_PARTITION_ALLOC)
 
 }  // namespace trace_event
 }  // namespace base
