@@ -7,6 +7,8 @@
 #include <stddef.h>
 
 #include <memory>
+#include <optional>
+#include <string_view>
 
 #include "base/memory/aligned_memory.h"
 #include "base/memory/ptr_util.h"
@@ -19,24 +21,23 @@
 #include "base/trace_event/traced_value.h"
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 #if BUILDFLAG(IS_WIN)
 #include <windows.h>
+
 #include "winbase.h"
 #elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 #include <sys/mman.h>
 #endif
 
-namespace base {
-namespace trace_event {
+namespace base::trace_event {
 
 namespace {
 
 const MemoryDumpArgs kDetailedDumpArgs = {MemoryDumpLevelOfDetail::kDetailed};
-const char* const kTestDumpNameAllowlist[] = {
+constexpr std::string_view kTestDumpNameAllowlist[] = {
     "Allowlisted/TestName", "Allowlisted/TestName_0x?",
-    "Allowlisted/0x?/TestName", "Allowlisted/0x?", nullptr};
+    "Allowlisted/0x?/TestName", "Allowlisted/0x?"};
 
 void* Map(size_t size) {
 #if BUILDFLAG(IS_WIN)
@@ -58,6 +59,18 @@ void Unmap(void* addr, size_t size) {
 #endif
 }
 
+std::optional<size_t> CountResidentBytesInSharedMemory(
+    const WritableSharedMemoryMapping& mapping) {
+  // SAFETY: We need the actual mapped memory size here. There's no public
+  // method to get this as a span, so we need to construct it unsafely. The
+  // mapped_size() is larger than `mem.size()` but represents the actual memory
+  // segment size in the SharedMemoryMapping.
+  auto mapped =
+      UNSAFE_BUFFERS(base::span(mapping.data(), mapping.mapped_size()));
+  return ProcessMemoryDump::CountResidentBytesInSharedMemory(mapped.data(),
+                                                             mapped.size());
+}
+
 }  // namespace
 
 TEST(ProcessMemoryDumpTest, MoveConstructor) {
@@ -477,7 +490,8 @@ TEST(ProcessMemoryDumpTest, GuidsTest) {
 
 #if defined(COUNT_RESIDENT_BYTES_SUPPORTED)
 #if BUILDFLAG(IS_FUCHSIA)
-// TODO(crbug.com/851760): Counting resident bytes is not supported on Fuchsia.
+// TODO(crbug.com/42050620): Counting resident bytes is not supported on
+// Fuchsia.
 #define MAYBE_CountResidentBytes DISABLED_CountResidentBytes
 #else
 #define MAYBE_CountResidentBytes CountResidentBytes
@@ -489,7 +503,7 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytes) {
   const size_t size1 = 5 * page_size;
   void* memory1 = Map(size1);
   memset(memory1, 0, size1);
-  absl::optional<size_t> res1 =
+  std::optional<size_t> res1 =
       ProcessMemoryDump::CountResidentBytes(memory1, size1);
   ASSERT_TRUE(res1.has_value());
   ASSERT_EQ(res1.value(), size1);
@@ -499,7 +513,7 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytes) {
   const size_t kVeryLargeMemorySize = 15 * 1024 * 1024;
   void* memory2 = Map(kVeryLargeMemorySize);
   memset(memory2, 0, kVeryLargeMemorySize);
-  absl::optional<size_t> res2 =
+  std::optional<size_t> res2 =
       ProcessMemoryDump::CountResidentBytes(memory2, kVeryLargeMemorySize);
   ASSERT_TRUE(res2.has_value());
   ASSERT_EQ(res2.value(), kVeryLargeMemorySize);
@@ -507,7 +521,8 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytes) {
 }
 
 #if BUILDFLAG(IS_FUCHSIA)
-// TODO(crbug.com/851760): Counting resident bytes is not supported on Fuchsia.
+// TODO(crbug.com/42050620): Counting resident bytes is not supported on
+// Fuchsia.
 #define MAYBE_CountResidentBytesInSharedMemory \
   DISABLED_CountResidentBytesInSharedMemory
 #else
@@ -521,10 +536,9 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytesInSharedMemory) {
     const size_t kDirtyMemorySize = 5 * page_size;
     auto region = base::WritableSharedMemoryRegion::Create(kDirtyMemorySize);
     base::WritableSharedMemoryMapping mapping = region.Map();
-    memset(mapping.memory(), 0, kDirtyMemorySize);
-    absl::optional<size_t> res1 =
-        ProcessMemoryDump::CountResidentBytesInSharedMemory(
-            mapping.memory(), mapping.mapped_size());
+    base::span<uint8_t> mapping_mem(mapping);
+    std::ranges::fill(mapping_mem, 0u);
+    std::optional<size_t> res1 = CountResidentBytesInSharedMemory(mapping);
     ASSERT_TRUE(res1.has_value());
     ASSERT_EQ(res1.value(), kDirtyMemorySize);
   }
@@ -536,10 +550,9 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytesInSharedMemory) {
         base::WritableSharedMemoryRegion::Create(kDirtyMemorySize + page_size);
     base::WritableSharedMemoryMapping mapping =
         region.MapAt(page_size / 2, kDirtyMemorySize);
-    memset(mapping.memory(), 0, kDirtyMemorySize);
-    absl::optional<size_t> res1 =
-        ProcessMemoryDump::CountResidentBytesInSharedMemory(
-            mapping.memory(), mapping.mapped_size());
+    base::span<uint8_t> mapping_mem(mapping);
+    std::ranges::fill(mapping_mem, 0u);
+    std::optional<size_t> res1 = CountResidentBytesInSharedMemory(mapping);
     ASSERT_TRUE(res1.has_value());
     ASSERT_EQ(res1.value(), kDirtyMemorySize + page_size);
   }
@@ -550,10 +563,9 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytesInSharedMemory) {
     auto region =
         base::WritableSharedMemoryRegion::Create(kVeryLargeMemorySize);
     base::WritableSharedMemoryMapping mapping = region.Map();
-    memset(mapping.memory(), 0, kVeryLargeMemorySize);
-    absl::optional<size_t> res2 =
-        ProcessMemoryDump::CountResidentBytesInSharedMemory(
-            mapping.memory(), mapping.mapped_size());
+    base::span<uint8_t> mapping_mem(mapping);
+    std::ranges::fill(mapping_mem, 0u);
+    std::optional<size_t> res2 = CountResidentBytesInSharedMemory(mapping);
     ASSERT_TRUE(res2.has_value());
     ASSERT_EQ(res2.value(), kVeryLargeMemorySize);
   }
@@ -563,15 +575,13 @@ TEST(ProcessMemoryDumpTest, MAYBE_CountResidentBytesInSharedMemory) {
     const size_t kTouchedMemorySize = 7 * 1024 * 1024;
     auto region = base::WritableSharedMemoryRegion::Create(kTouchedMemorySize);
     base::WritableSharedMemoryMapping mapping = region.Map();
-    memset(mapping.memory(), 0, kTouchedMemorySize);
-    absl::optional<size_t> res3 =
-        ProcessMemoryDump::CountResidentBytesInSharedMemory(
-            mapping.memory(), mapping.mapped_size());
+    base::span<uint8_t> mapping_mem(mapping);
+    std::ranges::fill(mapping_mem, 0u);
+    std::optional<size_t> res3 = CountResidentBytesInSharedMemory(mapping);
     ASSERT_TRUE(res3.has_value());
     ASSERT_EQ(res3.value(), kTouchedMemorySize);
   }
 }
 #endif  // defined(COUNT_RESIDENT_BYTES_SUPPORTED)
 
-}  // namespace trace_event
-}  // namespace base
+}  // namespace base::trace_event
