@@ -1,13 +1,20 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/trace_event/trace_arguments.h"
 
 #include <gtest/gtest.h>
 #include <limits>
 #include <string>
 
+#include "base/memory/raw_ptr.h"
+
 namespace base {
 namespace trace_event {
 
@@ -28,7 +35,7 @@ class MyConvertable : public ConvertableToTraceFormat {
 
  private:
   const char* text_;
-  bool* destroy_flag_;
+  raw_ptr<bool> destroy_flag_;
 };
 
 }  // namespace
@@ -215,7 +222,7 @@ TEST(TraceArguments, ConstructorSinglePointer) {
       }
 
      private:
-      bool* destroy_flag_;
+      raw_ptr<bool> destroy_flag_;
     };
     auto foo = std::make_unique<Foo>(&destroy_flag);
     EXPECT_FALSE(destroy_flag);
