@@ -7,8 +7,11 @@
 #include <stddef.h>
 
 #include <algorithm>
+#include <optional>
+#include <string_view>
 #include <utility>
 
+#include "base/containers/contains.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
 #include "base/logging.h"
@@ -18,11 +21,8 @@
 #include "base/trace_event/memory_dump_manager.h"
 #include "base/trace_event/memory_dump_request_args.h"
 #include "base/trace_event/trace_event.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 #include "third_party/perfetto/protos/perfetto/config/track_event/track_event_config.gen.h"  // nogncheck
-#endif
 
 namespace base::trace_event {
 
@@ -253,7 +253,7 @@ bool TraceConfig::EventFilterConfig::GetArgAsSet(
 }
 
 bool TraceConfig::EventFilterConfig::IsCategoryGroupEnabled(
-    const StringPiece& category_group_name) const {
+    std::string_view category_group_name) const {
   return category_filter_.IsCategoryGroupEnabled(category_group_name);
 }
 
@@ -276,12 +276,12 @@ TraceConfig::TraceConfig() {
   InitializeDefault();
 }
 
-TraceConfig::TraceConfig(StringPiece category_filter_string,
-                         StringPiece trace_options_string) {
+TraceConfig::TraceConfig(std::string_view category_filter_string,
+                         std::string_view trace_options_string) {
   InitializeFromStrings(category_filter_string, trace_options_string);
 }
 
-TraceConfig::TraceConfig(StringPiece category_filter_string,
+TraceConfig::TraceConfig(std::string_view category_filter_string,
                          TraceRecordMode record_mode) {
   InitializeFromStrings(category_filter_string,
                         TraceConfig::TraceRecordModeToStr(record_mode));
@@ -291,7 +291,7 @@ TraceConfig::TraceConfig(const Value::Dict& config) {
   InitializeFromConfigDict(config);
 }
 
-TraceConfig::TraceConfig(StringPiece config_string) {
+TraceConfig::TraceConfig(std::string_view config_string) {
   if (!config_string.empty())
     InitializeFromConfigString(config_string);
   else
@@ -370,7 +370,7 @@ std::string TraceConfig::ToCategoryFilterString() const {
 }
 
 bool TraceConfig::IsCategoryGroupEnabled(
-    const StringPiece& category_group_name) const {
+    std::string_view category_group_name) const {
   // TraceLog should call this method only as part of enabling/disabling
   // categories.
   return category_filter_.IsCategoryGroupEnabled(category_group_name);
@@ -479,16 +479,16 @@ void TraceConfig::InitializeFromConfigDict(const Value::Dict& dict) {
   }
 }
 
-void TraceConfig::InitializeFromConfigString(StringPiece config_string) {
-  absl::optional<Value> dict = JSONReader::Read(config_string);
+void TraceConfig::InitializeFromConfigString(std::string_view config_string) {
+  std::optional<Value> dict = JSONReader::Read(config_string);
   if (dict && dict->is_dict())
     InitializeFromConfigDict(dict->GetDict());
   else
     InitializeDefault();
 }
 
-void TraceConfig::InitializeFromStrings(StringPiece category_filter_string,
-                                        StringPiece trace_options_string) {
+void TraceConfig::InitializeFromStrings(std::string_view category_filter_string,
+                                        std::string_view trace_options_string) {
   if (!category_filter_string.empty())
     category_filter_.InitializeFromString(category_filter_string);
 
@@ -571,7 +571,7 @@ void TraceConfig::SetMemoryDumpConfigFromConfigDict(
       const Value::Dict& trigger_dict = trigger.GetDict();
 
       MemoryDumpConfig::Trigger dump_config;
-      absl::optional<int> interval = trigger_dict.FindInt(kMinTimeBetweenDumps);
+      std::optional<int> interval = trigger_dict.FindInt(kMinTimeBetweenDumps);
       if (!interval) {
         // If "min_time_between_dumps_ms" param was not given, then the trace
         // config uses old format where only periodic dumps are supported.
@@ -601,7 +601,7 @@ void TraceConfig::SetMemoryDumpConfigFromConfigDict(
   const Value::Dict* heap_profiler_options =
       memory_dump_config.FindDict(kHeapProfilerOptions);
   if (heap_profiler_options) {
-    absl::optional<int> min_size_bytes =
+    std::optional<int> min_size_bytes =
         heap_profiler_options->FindInt(kBreakdownThresholdBytes);
     if (min_size_bytes && *min_size_bytes >= 0) {
       memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes =
@@ -781,14 +781,20 @@ std::string TraceConfig::ToTraceOptionsString() const {
   return ret;
 }
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 std::string TraceConfig::ToPerfettoTrackEventConfigRaw(
     bool privacy_filtering_enabled) const {
   perfetto::protos::gen::TrackEventConfig te_cfg;
-  // If no categories are explicitly enabled, enable the default ones.
-  // Otherwise only matching categories are enabled.
-  if (!category_filter_.included_categories().empty())
-    te_cfg.add_disabled_categories("*");
+  if (!base::Contains(category_filter_.excluded_categories(), "*") &&
+      !base::Contains(category_filter_.included_categories(), "*")) {
+    // In the case when the default behavior is not specified, apply the
+    // following rule: if no categories are explicitly enabled, enable the
+    // default ones; otherwise only enable matching categories.
+    if (category_filter_.included_categories().empty()) {
+      te_cfg.add_enabled_categories("*");
+    } else {
+      te_cfg.add_disabled_categories("*");
+    }
+  }
   for (const auto& excluded : category_filter_.excluded_categories()) {
     te_cfg.add_disabled_categories(excluded);
   }
@@ -808,6 +814,5 @@ std::string TraceConfig::ToPerfettoTrackEventConfigRaw(
   }
   return te_cfg.SerializeAsString();
 }
-#endif
 
 }  // namespace base::trace_event
