@@ -2,11 +2,17 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/trace_event/trace_log.h"
 
 #include <cmath>
 #include <limits>
 #include <memory>
+#include <string_view>
 #include <unordered_set>
 #include <utility>
 
@@ -21,15 +27,16 @@
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/raw_ptr.h"
-#include "base/memory/raw_ptr_exclusion.h"
 #include "base/memory/ref_counted_memory.h"
+#include "base/memory/stack_allocated.h"
 #include "base/no_destructor.h"
 #include "base/notreached.h"
+#include "base/numerics/safe_conversions.h"
 #include "base/process/process.h"
 #include "base/process/process_metrics.h"
 #include "base/ranges/algorithm.h"
+#include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_tokenizer.h"
 #include "base/strings/stringprintf.h"
@@ -38,6 +45,7 @@
 #include "base/task/sequenced_task_runner.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/task/thread_pool.h"
+#include "base/task/thread_pool/thread_pool_instance.h"
 #include "base/threading/platform_thread.h"
 #include "base/threading/thread_id_name_manager.h"
 #include "base/time/time.h"
@@ -48,13 +56,8 @@
 #include "base/trace_event/process_memory_dump.h"
 #include "base/trace_event/trace_buffer.h"
 #include "base/trace_event/trace_event.h"
-#include "build/build_config.h"
-
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-#include "base/numerics/safe_conversions.h"
-#include "base/run_loop.h"
-#include "base/task/thread_pool/thread_pool_instance.h"
 #include "base/tracing/perfetto_platform.h"
+#include "build/build_config.h"
 #include "third_party/perfetto/include/perfetto/ext/trace_processor/export_json.h"  // nogncheck
 #include "third_party/perfetto/include/perfetto/trace_processor/trace_processor_storage.h"  // nogncheck
 #include "third_party/perfetto/include/perfetto/tracing/console_interceptor.h"
@@ -62,7 +65,6 @@
 #include "third_party/perfetto/protos/perfetto/config/interceptor_config.gen.h"  // nogncheck
 #include "third_party/perfetto/protos/perfetto/trace/track_event/process_descriptor.gen.h"  // nogncheck
 #include "third_party/perfetto/protos/perfetto/trace/track_event/thread_descriptor.gen.h"  // nogncheck
-#endif
 
 #if BUILDFLAG(IS_WIN)
 #include "base/trace_event/trace_event_etw_export_win.h"
@@ -101,11 +103,7 @@ const size_t kEchoToConsoleTraceEventBufferChunks = 256;
 
 const size_t kTraceEventBufferSizeInBytes = 100 * 1024;
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 bool g_perfetto_initialized_by_tracelog = false;
-#else
-constexpr TimeDelta kThreadFlushTimeout = Seconds(3);
-#endif  // BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 
 TraceLog* g_trace_log_for_testing = nullptr;
 
@@ -165,7 +163,6 @@ bool DefaultIsTraceEventArgsAllowlisted(
   return false;
 }
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 class PerfettoProtoAppenderTL
     : public base::trace_event::ConvertableToTraceFormat::ProtoAppender {
  public:
@@ -249,7 +246,6 @@ void WriteDebugAnnotations(base::trace_event::TraceEvent* trace_event,
 
       default:
         NOTREACHED() << "Don't know how to serialize this value";
-        break;
     }
   }
 }
@@ -260,19 +256,24 @@ void OnAddLegacyTraceEvent(TraceEvent* trace_event,
   perfetto::DynamicCategory category(
       TraceLog::GetInstance()->GetCategoryGroupName(
           trace_event->category_group_enabled()));
-  auto write_args = [trace_event](perfetto::EventContext ctx) {
+
+  auto phase = trace_event->phase();
+  if (phase == TRACE_EVENT_PHASE_COMPLETE) {
+    phase = TRACE_EVENT_PHASE_BEGIN;
+  }
+
+  auto write_args = [trace_event, phase](perfetto::EventContext ctx) {
     WriteDebugAnnotations(trace_event, ctx.event());
     uint32_t id_flags = trace_event->flags() & (TRACE_EVENT_FLAG_HAS_ID |
                                                 TRACE_EVENT_FLAG_HAS_LOCAL_ID |
                                                 TRACE_EVENT_FLAG_HAS_GLOBAL_ID);
     if (!id_flags &&
-        perfetto::internal::TrackEventLegacy::PhaseToType(
-            trace_event->phase()) !=
+        perfetto::internal::TrackEventLegacy::PhaseToType(phase) !=
             perfetto::protos::pbzero::TrackEvent::TYPE_UNSPECIFIED) {
       return;
     }
     auto* legacy_event = ctx.event()->set_legacy_event();
-    legacy_event->set_phase(trace_event->phase());
+    legacy_event->set_phase(phase);
     switch (id_flags) {
       case TRACE_EVENT_FLAG_HAS_ID:
         legacy_event->set_unscoped_id(trace_event->id());
@@ -288,14 +289,11 @@ void OnAddLegacyTraceEvent(TraceEvent* trace_event,
     }
   };
 
-  auto phase = trace_event->phase();
   auto flags = trace_event->flags();
   base::TimeTicks timestamp = trace_event->timestamp().is_null()
                                   ? TRACE_TIME_TICKS_NOW()
                                   : trace_event->timestamp();
-  if (phase == TRACE_EVENT_PHASE_COMPLETE) {
-    phase = TRACE_EVENT_PHASE_BEGIN;
-  } else if (phase == TRACE_EVENT_PHASE_INSTANT) {
+  if (phase == TRACE_EVENT_PHASE_INSTANT) {
     auto scope = flags & TRACE_EVENT_FLAG_SCOPE_MASK;
     switch (scope) {
       case TRACE_EVENT_SCOPE_GLOBAL:
@@ -350,7 +348,6 @@ void OnUpdateLegacyTraceEventDuration(
       phase, category, name,
       perfetto::internal::TrackEventInternal::kDefaultTrack, timestamp);
 }
-#endif  // BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 
 }  // namespace
 
@@ -372,7 +369,7 @@ class JsonStringOutputWriter
                          TraceLog::OutputCallback flush_callback)
       : flush_task_runner_(flush_task_runner),
         flush_callback_(std::move(flush_callback)) {
-    buffer_->data().reserve(kBufferReserveCapacity);
+    buffer_->as_string().reserve(kBufferReserveCapacity);
   }
 
   ~JsonStringOutputWriter() override { Flush(/*has_more=*/false); }
@@ -383,22 +380,22 @@ class JsonStringOutputWriter
       DCHECK_EQ(string, kJsonPrefix);
       did_strip_prefix_ = true;
       return perfetto::trace_processor::util::OkStatus();
-    } else if (buffer_->data().empty() &&
+    } else if (buffer_->as_string().empty() &&
                !strncmp(string.c_str(), kJsonJoiner, strlen(kJsonJoiner))) {
       // We only remove the leading joiner comma for the first chunk in a buffer
       // since the consumer is expected to insert commas between the buffers we
       // provide.
-      buffer_->data() += string.substr(strlen(kJsonJoiner));
+      buffer_->as_string() += string.substr(strlen(kJsonJoiner));
     } else if (!strncmp(string.c_str(), kJsonSuffix, strlen(kJsonSuffix))) {
       return perfetto::trace_processor::util::OkStatus();
     } else {
-      buffer_->data() += string;
+      buffer_->as_string() += string;
     }
-    if (buffer_->data().size() > kBufferLimitInBytes) {
+    if (buffer_->as_string().size() > kBufferLimitInBytes) {
       Flush(/*has_more=*/true);
       // Reset the buffer_ after moving it above.
       buffer_ = new RefCountedString();
-      buffer_->data().reserve(kBufferReserveCapacity);
+      buffer_->as_string().reserve(kBufferReserveCapacity);
     }
     return perfetto::trace_processor::util::OkStatus();
   }
@@ -430,6 +427,8 @@ class JsonStringOutputWriter
 // A helper class that allows the lock to be acquired in the middle of the scope
 // and unlocks at the end of scope if locked.
 class TraceLog::OptionalAutoLock {
+  STACK_ALLOCATED();
+
  public:
   explicit OptionalAutoLock(Lock* lock) : lock_(lock) {}
 
@@ -451,8 +450,7 @@ class TraceLog::OptionalAutoLock {
   }
 
  private:
-  // This field is not a raw_ptr<> because it is needed for lock annotations.
-  RAW_PTR_EXCLUSION Lock* lock_;
+  Lock* lock_;
   bool locked_ = false;
 };
 
@@ -616,27 +614,12 @@ TraceLog* TraceLog::GetInstance() {
 
 // static
 void TraceLog::ResetForTesting() {
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   auto* self = GetInstance();
   AutoLock lock(self->observers_lock_);
   self->enabled_state_observers_.clear();
   self->owned_enabled_state_observer_copy_.clear();
   self->async_observers_.clear();
   self->InitializePerfettoIfNeeded();
-#else   // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-  if (!g_trace_log_for_testing)
-    return;
-  {
-    AutoLock lock(g_trace_log_for_testing->lock_);
-    CategoryRegistry::ResetForTesting();
-  }
-  // Don't reset the generation value back to 0. TraceLog is normally
-  // supposed to be a singleton and the value of generation is never
-  // supposed to decrease.
-  const int generation = g_trace_log_for_testing->generation() + 1;
-  g_trace_log_for_testing->~TraceLog();
-  new (g_trace_log_for_testing) TraceLog(generation);
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
 TraceLog::TraceLog(int generation)
@@ -660,7 +643,6 @@ TraceLog::TraceLog(int generation)
 
   MemoryDumpManager::GetInstance()->RegisterDumpProvider(this, "TraceLog",
                                                          nullptr);
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   TrackEvent::AddSessionObserver(this);
   // When using the Perfetto client library, TRACE_EVENT macros will bypass
   // TraceLog entirely. However, trace event embedders which haven't been ported
@@ -669,14 +651,11 @@ TraceLog::TraceLog(int generation)
   // override is also used to capture internal metadata events.
   SetAddTraceEventOverrides(&OnAddLegacyTraceEvent, nullptr,
                             &OnUpdateLegacyTraceEventDuration);
-#endif  // BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   g_trace_log_for_testing = this;
 }
 
 TraceLog::~TraceLog() {
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   TrackEvent::RemoveSessionObserver(this);
-#endif  // BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
 void TraceLog::InitializeThreadLocalEventBufferIfSupported() {
@@ -720,39 +699,12 @@ bool TraceLog::OnMemoryDump(const MemoryDumpArgs& args,
 
 const unsigned char* TraceLog::GetCategoryGroupEnabled(
     const char* category_group) {
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   return TRACE_EVENT_API_GET_CATEGORY_GROUP_ENABLED(category_group);
-#else   // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-  TraceLog* tracelog = GetInstance();
-  if (!tracelog) {
-    DCHECK(!CategoryRegistry::kCategoryAlreadyShutdown->is_enabled());
-    return CategoryRegistry::kCategoryAlreadyShutdown->state_ptr();
-  }
-  TraceCategory* category = CategoryRegistry::GetCategoryByName(category_group);
-  if (!category) {
-    // Slow path: in the case of a new category we have to repeat the check
-    // holding the lock, as multiple threads might have reached this point
-    // at the same time.
-    auto category_initializer = [](TraceCategory* category) {
-      TraceLog::GetInstance()->UpdateCategoryState(category);
-    };
-    AutoLock lock(tracelog->lock_);
-    CategoryRegistry::GetOrCreateCategoryLocked(
-        category_group, category_initializer, &category);
-  }
-  DCHECK(category->state_ptr());
-  return category->state_ptr();
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
 const char* TraceLog::GetCategoryGroupName(
     const unsigned char* category_group_enabled) {
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   return TRACE_EVENT_API_GET_CATEGORY_GROUP_NAME(category_group_enabled);
-#else   // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-  return CategoryRegistry::GetCategoryByStatePtr(category_group_enabled)
-      ->name();
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
 void TraceLog::UpdateCategoryState(TraceCategory* category) {
@@ -816,7 +768,6 @@ void TraceLog::SetEnabled(const TraceConfig& trace_config,
            disabled.find("*") == disabled.size() - 1);
   }
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   DCHECK(!trace_config.IsArgumentFilterEnabled());
 
   // TODO(khokhlov): Avoid duplication between this code and
@@ -879,70 +830,14 @@ void TraceLog::SetEnabled(const TraceConfig& trace_config,
   perfetto_config.mutable_incremental_state_config()->set_clear_period_ms(500);
 
   SetEnabledImpl(trace_config, perfetto_config);
-#else   // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-  // Can't enable tracing when Flush() is in progress.
-  DCHECK(!flush_task_runner_);
-
-  InternalTraceOptions new_options =
-      GetInternalOptionsFromTraceConfig(trace_config);
-
-  InternalTraceOptions old_options = trace_options();
-
-  if (dispatching_to_observers_) {
-    // TODO(ssid): Change to NOTREACHED after fixing crbug.com/625170.
-    DLOG(ERROR)
-        << "Cannot manipulate TraceLog::Enabled state from an observer.";
-    return;
-  }
-
-  // Update trace config for recording.
-  const bool already_recording = enabled_;
-  if (already_recording) {
-    trace_config_.Merge(trace_config);
-  } else {
-    trace_config_ = trace_config;
-  }
-
-  enabled_ = true;
-  UpdateCategoryRegistry();
-
-  // Do not notify observers or create trace buffer if only enabled for
-  // filtering or if recording was already enabled.
-  if (already_recording)
-    return;
-
-  // Discard events if new trace options are different. Reducing trace buffer
-  // size is not supported while already recording, so only replace trace
-  // buffer if we were not already recording.
-  if (new_options != old_options ||
-      (trace_config_.GetTraceBufferSizeInEvents() && !already_recording)) {
-    trace_options_.store(new_options, std::memory_order_relaxed);
-    UseNextTraceBuffer();
-  }
-
-  num_traces_recorded_++;
-
-  UpdateCategoryRegistry();
-
-  dispatching_to_observers_ = true;
-  {
-    // Notify observers outside of the thread events lock, so they can trigger
-    // trace events.
-    AutoUnlock unlock(lock_);
-    AutoLock lock2(observers_lock_);
-    for (EnabledStateObserver* observer : enabled_state_observers_)
-      observer->OnTraceLogEnabled();
-    for (const auto& it : async_observers_) {
-      it.second.task_runner->PostTask(
-          FROM_HERE, BindOnce(&AsyncEnabledStateObserver::OnTraceLogEnabled,
-                              it.second.observer));
-    }
-  }
-  dispatching_to_observers_ = false;
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
+std::vector<TraceLog::TrackEventSession> TraceLog::GetTrackEventSessions()
+    const {
+  AutoLock lock(track_event_lock_);
+  return track_event_sessions_;
+}
+
 perfetto::DataSourceConfig TraceLog::GetCurrentTrackEventDataSourceConfig()
     const {
   AutoLock lock(track_event_lock_);
@@ -1002,7 +897,6 @@ void TraceLog::SetEnabledImpl(const TraceConfig& trace_config,
   tracing_session_->Setup(perfetto_config);
   tracing_session_->StartBlocking();
 }
-#endif  // BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 
 void TraceLog::SetArgumentFilterPredicate(
     const ArgumentFilterPredicate& argument_filter_predicate) {
@@ -1054,18 +948,12 @@ TraceLog::InternalTraceOptions TraceLog::GetInternalOptionsFromTraceConfig(
       return ret | kInternalRecordAsMuchAsPossible;
   }
   NOTREACHED();
-  return kInternalNone;
 }
 
 TraceConfig TraceLog::GetCurrentTraceConfig() const {
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   const auto chrome_config =
       GetCurrentTrackEventDataSourceConfig().chrome_config();
   return TraceConfig(chrome_config.trace_config());
-#else   // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-  AutoLock lock(lock_);
-  return trace_config_;
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
 void TraceLog::SetDisabled() {
@@ -1080,7 +968,6 @@ void TraceLog::SetDisabled(uint8_t modes_to_disable) {
 
 void TraceLog::SetDisabledWhileLocked(uint8_t modes_to_disable) {
   DCHECK(modes_to_disable == RECORDING_MODE);
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   if (!tracing_session_)
     return;
 
@@ -1103,41 +990,6 @@ void TraceLog::SetDisabledWhileLocked(uint8_t modes_to_disable) {
   } else {
     tracing_session_->StopBlocking();
   }
-#else   // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-
-  if (dispatching_to_observers_) {
-    // TODO(ssid): Change to NOTREACHED after fixing crbug.com/625170.
-    DLOG(ERROR)
-        << "Cannot manipulate TraceLog::Enabled state from an observer.";
-    return;
-  }
-
-  enabled_ = false;
-  trace_config_.Clear();
-  UpdateCategoryRegistry();
-
-  AddMetadataEventsWhileLocked();
-
-  // Remove metadata events so they will not get added to a subsequent trace.
-  metadata_events_.clear();
-
-  dispatching_to_observers_ = true;
-  {
-    // Release trace events lock, so observers can trigger trace events.
-    AutoUnlock unlock(lock_);
-    AutoLock lock2(observers_lock_);
-    for (base::trace_event::TraceLog::EnabledStateObserver* it :
-         enabled_state_observers_) {
-      it->OnTraceLogDisabled();
-    }
-    for (const auto& it : async_observers_) {
-      it.second.task_runner->PostTask(
-          FROM_HERE, BindOnce(&AsyncEnabledStateObserver::OnTraceLogDisabled,
-                              it.second.observer));
-    }
-  }
-  dispatching_to_observers_ = false;
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 }
 
 int TraceLog::GetNumTracesRecorded() {
@@ -1214,13 +1066,6 @@ TraceLogStatus TraceLog::GetStatus() const {
   return result;
 }
 
-#if !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-bool TraceLog::BufferIsFull() const {
-  AutoLock lock(lock_);
-  return logged_events_->IsFull();
-}
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-
 TraceEvent* TraceLog::AddEventToThreadSharedChunkWhileLocked(
     TraceEventHandle* handle,
     bool check_buffer_is_full) {
@@ -1323,59 +1168,10 @@ void TraceLog::FlushInternal(const TraceLog::OutputCallback& cb,
     auto data = tracing_session_->ReadTraceBlocking();
     OnTraceData(data.data(), data.size(), /*has_more=*/false);
   }
-#elif BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
+#else
   // Trace processor isn't enabled so we can't convert the resulting trace into
   // JSON.
   CHECK(false) << "JSON tracing isn't supported";
-#else
-  if (IsEnabled()) {
-    // Can't flush when tracing is enabled because otherwise PostTask would
-    // - generate more trace events;
-    // - deschedule the calling thread on some platforms causing inaccurate
-    //   timing of the trace events.
-    scoped_refptr<RefCountedString> empty_result = new RefCountedString;
-    if (!cb.is_null())
-      cb.Run(empty_result, false);
-    LOG(WARNING) << "Ignored TraceLog::Flush called when tracing is enabled";
-    return;
-  }
-
-  int gen = generation();
-  // Copy of thread_task_runners_ to be used without locking.
-  std::vector<scoped_refptr<SingleThreadTaskRunner>> task_runners;
-  {
-    AutoLock lock(lock_);
-    DCHECK(!flush_task_runner_);
-    flush_task_runner_ = SequencedTaskRunner::HasCurrentDefault()
-                             ? SequencedTaskRunner::GetCurrentDefault()
-                             : nullptr;
-    DCHECK(thread_task_runners_.empty() || flush_task_runner_);
-    flush_output_callback_ = cb;
-
-    if (thread_shared_chunk_) {
-      logged_events_->ReturnChunk(thread_shared_chunk_index_,
-                                  std::move(thread_shared_chunk_));
-    }
-
-    for (const auto& it : thread_task_runners_)
-      task_runners.push_back(it.second);
-  }
-
-  if (!task_runners.empty()) {
-    for (auto& task_runner : task_runners) {
-      task_runner->PostTask(
-          FROM_HERE, BindOnce(&TraceLog::FlushCurrentThread, Unretained(this),
-                              gen, discard_events));
-    }
-    flush_task_runner_->PostDelayedTask(
-        FROM_HERE,
-        BindOnce(&TraceLog::OnFlushTimeout, Unretained(this), gen,
-                 discard_events),
-        kThreadFlushTimeout);
-    return;
-  }
-
-  FinishFlush(gen, discard_events);
 #endif  // BUILDFLAG(USE_PERFETTO_TRACE_PROCESSOR)
 }
 
@@ -1384,7 +1180,7 @@ void TraceLog::OnTraceData(const char* data, size_t size, bool has_more) {
   if (proto_output_callback_) {
     scoped_refptr<RefCountedString> chunk = new RefCountedString();
     if (size)
-      chunk->data().assign(data, size);
+      chunk->as_string().assign(data, size);
     proto_output_callback_.Run(std::move(chunk), has_more);
     if (!has_more) {
       proto_output_callback_.Reset();
@@ -1400,9 +1196,11 @@ void TraceLog::OnTraceData(const char* data, size_t size, bool has_more) {
   }
   if (has_more)
     return;
-  trace_processor_->NotifyEndOfFile();
 
-  auto status = perfetto::trace_processor::json::ExportJson(
+  auto status = trace_processor_->NotifyEndOfFile();
+  DCHECK(status.ok()) << status.message();
+
+  status = perfetto::trace_processor::json::ExportJson(
       trace_processor_.get(), json_output_writer_.get());
   DCHECK(status.ok()) << status.message();
   trace_processor_.reset();
@@ -1424,18 +1222,18 @@ void TraceLog::ConvertTraceEventsToTraceFormat(
   // to let the caller know the completion of flush.
   scoped_refptr<RefCountedString> json_events_str_ptr = new RefCountedString();
   const size_t kReserveCapacity = kTraceEventBufferSizeInBytes * 5 / 4;
-  json_events_str_ptr->data().reserve(kReserveCapacity);
+  json_events_str_ptr->as_string().reserve(kReserveCapacity);
   while (const TraceBufferChunk* chunk = logged_events->NextChunk()) {
     for (size_t j = 0; j < chunk->size(); ++j) {
       size_t size = json_events_str_ptr->size();
       if (size > kTraceEventBufferSizeInBytes) {
         flush_output_callback.Run(json_events_str_ptr, true);
         json_events_str_ptr = new RefCountedString();
-        json_events_str_ptr->data().reserve(kReserveCapacity);
+        json_events_str_ptr->as_string().reserve(kReserveCapacity);
       } else if (size) {
-        json_events_str_ptr->data().append(",\n");
+        json_events_str_ptr->as_string().append(",\n");
       }
-      chunk->GetEventAt(j)->AppendAsJSON(&(json_events_str_ptr->data()),
+      chunk->GetEventAt(j)->AppendAsJSON(&(json_events_str_ptr->as_string()),
                                          argument_filter_predicate);
     }
   }
@@ -1600,7 +1398,7 @@ bool TraceLog::ShouldAddAfterUpdatingState(
       } else {
         // This is a thread id that we've seen before, but potentially with a
         // new name.
-        std::vector<StringPiece> existing_names = base::SplitStringPiece(
+        std::vector<std::string_view> existing_names = base::SplitStringPiece(
             existing_name->second, ",", base::KEEP_WHITESPACE,
             base::SPLIT_WANT_NONEMPTY);
         if (!Contains(existing_names, new_name)) {
@@ -2012,7 +1810,7 @@ void TraceLog::AddMetadataEventsWhileLocked() {
 #endif
 
   if (!process_labels_.empty()) {
-    std::vector<base::StringPiece> labels;
+    std::vector<std::string_view> labels;
     for (const auto& it : process_labels_)
       labels.push_back(it.second);
     AddMetadataEventWhileLocked(current_thread_id, "process_labels", "labels",
@@ -2088,7 +1886,6 @@ void TraceLog::SetProcessSortIndex(int sort_index) {
 }
 
 void TraceLog::OnSetProcessName(const std::string& process_name) {
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   if (perfetto::Tracing::IsInitialized()) {
     auto track = perfetto::ProcessTrack::Current();
     auto desc = track.Serialize();
@@ -2096,7 +1893,6 @@ void TraceLog::OnSetProcessName(const std::string& process_name) {
     desc.mutable_process()->set_pid(static_cast<int>(process_id_));
     TrackEvent::SetTrackDescriptor(track, std::move(desc));
   }
-#endif
 }
 
 int TraceLog::GetNewProcessLabelId() {
@@ -2109,14 +1905,12 @@ void TraceLog::UpdateProcessLabel(int label_id,
   if (!current_label.length())
     return RemoveProcessLabel(label_id);
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
   if (perfetto::Tracing::IsInitialized()) {
     auto track = perfetto::ProcessTrack::Current();
     auto desc = track.Serialize();
     desc.mutable_process()->add_process_labels(current_label);
     TrackEvent::SetTrackDescriptor(track, std::move(desc));
   }
-#endif
 
   AutoLock lock(lock_);
   process_labels_[label_id] = current_label;
@@ -2132,12 +1926,6 @@ void TraceLog::SetThreadSortIndex(PlatformThreadId thread_id, int sort_index) {
   thread_sort_indices_[thread_id] = sort_index;
 }
 
-#if !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-void TraceLog::SetTimeOffset(TimeDelta offset) {
-  time_offset_ = offset;
-}
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-
 size_t TraceLog::GetObserverCountForTest() const {
   AutoLock lock(observers_lock_);
   return enabled_state_observers_.size();
@@ -2195,11 +1983,10 @@ void TraceLog::SetTraceBufferForTesting(
   logged_events_ = std::move(trace_buffer);
 }
 
-#if BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 void TraceLog::OnSetup(const perfetto::DataSourceBase::SetupArgs& args) {
   AutoLock lock(track_event_lock_);
-  track_event_sessions_.emplace_back(args.internal_instance_index, *args.config,
-                                     args.backend_type);
+  track_event_sessions_.push_back({args.internal_instance_index, *args.config,
+                                     args.backend_type});
 }
 
 void TraceLog::OnStart(const perfetto::DataSourceBase::StartArgs&) {
@@ -2251,7 +2038,6 @@ void TraceLog::OnStop(const perfetto::DataSourceBase::StopArgs& args) {
                             it.second.observer));
   }
 }
-#endif  // BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
 
 void ConvertableToTraceFormat::EstimateTraceMemoryOverhead(
     TraceEventMemoryOverhead* overhead) {
@@ -2385,33 +2171,4 @@ void UpdateTraceEventDurationExplicit(
                                          thread_now);
 }
 
-#if !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-ScopedTraceBinaryEfficient::ScopedTraceBinaryEfficient(
-    const char* category_group,
-    const char* name) {
-  // The single atom works because for now the category_group can only be "gpu".
-  DCHECK_EQ(strcmp(category_group, "gpu"), 0);
-  static TRACE_EVENT_API_ATOMIC_WORD atomic = 0;
-  INTERNAL_TRACE_EVENT_GET_CATEGORY_INFO_CUSTOM_VARIABLES(
-      category_group, atomic, category_group_enabled_);
-  name_ = name;
-  if (*category_group_enabled_) {
-    event_handle_ =
-        TRACE_EVENT_API_ADD_TRACE_EVENT_WITH_THREAD_ID_AND_TIMESTAMP(
-            TRACE_EVENT_PHASE_COMPLETE, category_group_enabled_, name,
-            trace_event_internal::kGlobalScope,  // scope
-            trace_event_internal::kNoId,         // id
-            base::PlatformThread::CurrentId(),   // thread_id
-            TRACE_TIME_TICKS_NOW(), nullptr, TRACE_EVENT_FLAG_NONE);
-  }
-}
-
-ScopedTraceBinaryEfficient::~ScopedTraceBinaryEfficient() {
-  if (*category_group_enabled_) {
-    TRACE_EVENT_API_UPDATE_TRACE_EVENT_DURATION(category_group_enabled_, name_,
-                                                event_handle_);
-  }
-}
-#endif  // !BUILDFLAG(USE_PERFETTO_CLIENT_LIBRARY)
-
 }  // namespace trace_event_internal
