@@ -1,15 +1,21 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/types/token_type.h"
 
+#include "base/test/gtest_util.h"
 #include "base/unguessable_token.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace base {
 
 using FooToken = TokenType<class Foo>;
+static_assert(std::is_trivially_copy_constructible_v<FooToken>);
+static_assert(std::is_trivially_copy_assignable_v<FooToken>);
+static_assert(std::is_trivially_move_constructible_v<FooToken>);
+static_assert(std::is_trivially_move_assignable_v<FooToken>);
+static_assert(std::is_trivially_destructible_v<FooToken>);
 
 TEST(TokenType, TokenApi) {
   // Test default initialization.
@@ -43,4 +49,8 @@ TEST(TokenType, TokenApi) {
   EXPECT_EQ(token2.ToString(), token2.value().ToString());
 }
 
+TEST(TokenType, TokenFromNullUnguessableToken) {
+  EXPECT_CHECK_DEATH({ FooToken{UnguessableToken::Null()}; });
+}
+
 }  // namespace base
