@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,7 @@
 #define CC_ANIMATION_ANIMATION_DELEGATE_H_
 
 #include <memory>
+#include <optional>
 
 #include "base/time/time.h"
 #include "cc/animation/animation_export.h"
@@ -32,7 +33,7 @@ class CC_ANIMATION_EXPORT AnimationDelegate {
       base::TimeTicks animation_start_time,
       std::unique_ptr<gfx::AnimationCurve> curve) = 0;
   virtual void NotifyLocalTimeUpdated(
-      base::Optional<base::TimeDelta> local_time) = 0;
+      std::optional<base::TimeDelta> local_time) = 0;
 
  protected:
   virtual ~AnimationDelegate() {}
