@@ -4,12 +4,17 @@
 
 #include "cc/input/input_handler.h"
 
+#include <string>
 #include <utility>
 #include <vector>
 
+#include "base/feature_list.h"
 #include "base/notreached.h"
+#include "base/types/optional_ref.h"
 #include "build/build_config.h"
 #include "cc/base/features.h"
+#include "cc/input/browser_controls_offset_manager.h"
+#include "cc/input/browser_controls_offset_tags_info.h"
 #include "cc/input/scroll_elasticity_helper.h"
 #include "cc/input/scroll_utils.h"
 #include "cc/input/scrollbar_controller.h"
@@ -33,11 +38,38 @@ namespace {
 
 enum SlowScrollMetricThread { MAIN_THREAD, CC_THREAD };
 
+InputHandlerClient::ScrollEventDispatchMode GetScrollEventDispatchMode() {
+  const std::string mode_name = ::features::kScrollEventDispatchMode.Get();
+  if (mode_name ==
+      ::features::kScrollEventDispatchModeDispatchScrollEventsImmediately) {
+    return InputHandlerClient::ScrollEventDispatchMode::
+        kDispatchScrollEventsImmediately;
+  } else if (mode_name ==
+             ::features::
+                 kScrollEventDispatchModeUseScrollPredictorForEmptyQueue) {
+    return InputHandlerClient::ScrollEventDispatchMode::
+        kUseScrollPredictorForEmptyQueue;
+  } else if (mode_name ==
+             ::features::
+                 kScrollEventDispatchModeUseScrollPredictorForDeadline) {
+    return InputHandlerClient::ScrollEventDispatchMode::
+        kUseScrollPredictorForDeadline;
+  }
+
+  return InputHandlerClient::ScrollEventDispatchMode::kEnqueueScrollEvents;
+}
+
 }  // namespace
 
 InputHandlerCommitData::InputHandlerCommitData() = default;
 InputHandlerCommitData::~InputHandlerCommitData() = default;
 
+// The minimum amount of scroll delta that must be consumed before we consider
+// a scroll to have happened.
+// TODO(tdresser): Use a more rational epsilon. See crbug.com/510550 for
+// details.
+const float kScrollEpsilon = 0.1f;
+
 // static
 base::WeakPtr<InputHandler> InputHandler::Create(
     CompositorDelegateForInput& compositor_delegate) {
@@ -66,6 +98,8 @@ void InputHandler::BindToClient(InputHandlerClient* client) {
   DCHECK(input_handler_client_ == nullptr);
   input_handler_client_ = client;
   input_handler_client_->SetPrefersReducedMotion(prefers_reduced_motion_);
+  input_handler_client_->SetScrollEventDispatchMode(
+      GetScrollEventDispatchMode());
 }
 
 InputHandler::ScrollStatus InputHandler::ScrollBegin(ScrollState* scroll_state,
@@ -105,6 +139,8 @@ InputHandler::ScrollStatus InputHandler::ScrollBegin(ScrollState* scroll_state,
   // TODO(bokan): ClearCurrentlyScrollingNode shouldn't happen in ScrollBegin,
   // this should only happen in ScrollEnd. We should DCHECK here that the state
   // is cleared instead. https://crbug.com/1016229
+  //
+  // TODO(b/329346768): Validate that this is no longer needed.
   ClearCurrentlyScrollingNode();
 
   ElementId target_element_id = scroll_state->target_element_id();
@@ -148,15 +184,9 @@ InputHandler::ScrollStatus InputHandler::ScrollBegin(ScrollState* scroll_state,
           gfx::ScalePoint(gfx::PointF(viewport_point),
                           compositor_delegate_->DeviceScaleFactor());
 
-      if (scroll_state->main_thread_hit_tested_reasons()) {
-        // The client should have discarded the scroll when the hit test came
-        // back with an invalid element id. If we somehow get here, we should
-        // drop the scroll as continuing could cause us to infinitely bounce
-        // back and forth between here and hit testing on the main thread.
-        NOTREACHED();
-        scroll_status.thread = InputHandler::ScrollThread::kScrollIgnored;
-        return scroll_status;
-      }
+      // The client should have discarded the scroll when the hit test came back
+      // with an invalid element id.
+      CHECK(!scroll_state->main_thread_hit_tested_reasons());
 
       ScrollHitTestResult scroll_hit_test =
           HitTestScrollNode(device_viewport_point);
@@ -234,8 +264,9 @@ InputHandler::ScrollStatus InputHandler::ScrollBegin(ScrollState* scroll_state,
   // oopif.
   if (GetViewport().ShouldScroll(*CurrentlyScrollingNode())) {
     outer_viewport_consumed_delta_ = false;
+    inner_viewport_consumed_delta_ = false;
     if (!GetViewport().CanScroll(*CurrentlyScrollingNode(), *scroll_state)) {
-      // TODO(crbug.com/1155758): This is a temporary workaround for GuestViews
+      // TODO(crbug.com/40735567): This is a temporary workaround for GuestViews
       // as they create viewport nodes and want to bubble scroll if the
       // viewport cannot scroll in the given delta directions. There should be
       // a parameter to ThreadInputHandler to specify whether unused delta is
@@ -269,49 +300,47 @@ InputHandler::ScrollStatus InputHandler::RootScrollBegin(
 }
 
 InputHandlerScrollResult InputHandler::ScrollUpdate(
-    ScrollState* scroll_state,
+    ScrollState scroll_state,
     base::TimeDelta delayed_by) {
-  DCHECK(scroll_state);
-
   // The current_native_scrolling_element should only be set for ScrollBegin.
-  DCHECK(!scroll_state->data()->current_native_scrolling_element());
-  TRACE_EVENT2("cc", "InputHandler::ScrollUpdate", "dx",
-               scroll_state->delta_x(), "dy", scroll_state->delta_y());
+  DCHECK(!scroll_state.data()->current_native_scrolling_element());
+  TRACE_EVENT2("cc", "InputHandler::ScrollUpdate", "dx", scroll_state.delta_x(),
+               "dy", scroll_state.delta_y());
 
   if (!CurrentlyScrollingNode())
     return InputHandlerScrollResult();
 
   const ScrollNode& scroll_node = *CurrentlyScrollingNode();
-  last_scroll_update_state_ = *scroll_state;
+  last_scroll_update_state_ = scroll_state;
 
   // Snap on update if interacting with the scrollbar track or arrow buttons.
   // Interactions with the scrollbar thumb have kScrollByPrecisePixel
   // granularity.
-  if (scroll_state->is_scrollbar_interaction() &&
-      scroll_state->delta_granularity() !=
+  if (scroll_state.is_scrollbar_interaction() &&
+      scroll_state.delta_granularity() !=
           ui::ScrollGranularity::kScrollByPrecisePixel) {
     AdjustScrollDeltaForScrollbarSnap(scroll_state);
   }
 
   gfx::Vector2dF resolvedScrollDelta = ResolveScrollGranularityToPixels(
       scroll_node,
-      gfx::Vector2dF(scroll_state->delta_x(), scroll_state->delta_y()),
-      scroll_state->delta_granularity());
+      gfx::Vector2dF(scroll_state.delta_x(), scroll_state.delta_y()),
+      scroll_state.delta_granularity());
 
-  scroll_state->data()->delta_x = resolvedScrollDelta.x();
-  scroll_state->data()->delta_y = resolvedScrollDelta.y();
+  scroll_state.data()->delta_x = resolvedScrollDelta.x();
+  scroll_state.data()->delta_y = resolvedScrollDelta.y();
   // The decision of whether or not we'll animate a scroll comes down to
   // whether the granularity is specified in precise pixels or not. Thus we
   // need to preserve a precise granularity if that's what was specified; all
   // others are animated and so can be resolved to regular pixels.
-  if (scroll_state->delta_granularity() !=
+  if (scroll_state.delta_granularity() !=
       ui::ScrollGranularity::kScrollByPrecisePixel) {
-    scroll_state->data()->delta_granularity =
+    scroll_state.data()->delta_granularity =
         ui::ScrollGranularity::kScrollByPixel;
   }
 
   compositor_delegate_->AccumulateScrollDeltaForTracing(
-      gfx::Vector2dF(scroll_state->delta_x(), scroll_state->delta_y()));
+      gfx::Vector2dF(scroll_state.delta_x(), scroll_state.delta_y()));
 
   compositor_delegate_->WillScrollContent(scroll_node.element_id);
 
@@ -321,14 +350,14 @@ InputHandlerScrollResult InputHandler::ScrollUpdate(
 
   ScrollLatchedScroller(scroll_state, delayed_by);
 
-  bool did_scroll_x = scroll_state->caused_scroll_x();
-  bool did_scroll_y = scroll_state->caused_scroll_y();
+  bool did_scroll_x = scroll_state.caused_scroll_x();
+  bool did_scroll_y = scroll_state.caused_scroll_y();
 
   delta_consumed_for_scroll_gesture_ |=
-      scroll_state->delta_consumed_for_scroll_sequence();
+      scroll_state.delta_consumed_for_scroll_sequence();
   bool did_scroll_content = did_scroll_x || did_scroll_y;
   if (did_scroll_content) {
-    bool is_animated_scroll = ShouldAnimateScroll(*scroll_state);
+    bool is_animated_scroll = ShouldAnimateScroll(scroll_state);
     compositor_delegate_->DidScrollContent(scroll_node.element_id,
                                            is_animated_scroll);
   }
@@ -344,7 +373,7 @@ InputHandlerScrollResult InputHandler::ScrollUpdate(
   gfx::Vector2dF unused_root_delta;
   if (GetViewport().ShouldScroll(scroll_node)) {
     unused_root_delta =
-        gfx::Vector2dF(scroll_state->delta_x(), scroll_state->delta_y());
+        gfx::Vector2dF(scroll_state.delta_x(), scroll_state.delta_y());
   }
 
   // When inner viewport is unscrollable, disable overscrolls.
@@ -366,7 +395,7 @@ InputHandlerScrollResult InputHandler::ScrollUpdate(
   scroll_result.accumulated_root_overscroll = accumulated_root_overscroll_;
   scroll_result.unused_scroll_delta = unused_root_delta;
   scroll_result.overscroll_behavior =
-      scroll_state->is_scroll_chain_cut()
+      scroll_state.is_scroll_chain_cut()
           ? OverscrollBehavior(OverscrollBehavior::Type::kNone)
           : ActiveTree().overscroll_behavior();
 
@@ -380,7 +409,7 @@ InputHandlerScrollResult InputHandler::ScrollUpdate(
   float scale_factor = ActiveTree().page_scale_factor_for_scroll();
   scroll_result.current_visual_offset.Scale(scale_factor);
 
-  if (!GetScrollTree().CanRealizeScrollsOnCompositor(scroll_node)) {
+  if (GetScrollTree().ShouldRealizeScrollsOnMain(scroll_node)) {
     scroll_result.needs_main_thread_repaint = true;
   }
 
@@ -396,7 +425,7 @@ InputHandlerScrollResult InputHandler::ScrollUpdate(
 }
 
 void InputHandler::AdjustScrollDeltaForScrollbarSnap(
-    ScrollState* scroll_state) {
+    ScrollState& scroll_state) {
   ScrollNode* scroll_node = CurrentlyScrollingNode();
   if (!scroll_node || !scroll_node->snap_container_data)
     return;
@@ -404,7 +433,7 @@ void InputHandler::AdjustScrollDeltaForScrollbarSnap(
   // Ideally, scrollbar track and arrow interactions would have
   // kScrollByPage and kScrollByLine, respectively. Currently, both have
   // kScrollByPixel granularity.
-  // TODO(crbug.com/959441): Update snap strategy once the granularity is
+  // TODO(crbug.com/41456637): Update snap strategy once the granularity is
   // properly set. Currently, track and arrow scrolls both use a direction
   // strategy; however, the track should be using an "end and direction"
   // strategy.
@@ -413,16 +442,15 @@ void InputHandler::AdjustScrollDeltaForScrollbarSnap(
   std::unique_ptr<SnapSelectionStrategy> strategy =
       SnapSelectionStrategy::CreateForDirection(
           current_position,
-          gfx::Vector2dF(scroll_state->delta_x(), scroll_state->delta_y()),
-          true);
+          gfx::Vector2dF(scroll_state.delta_x(), scroll_state.delta_y()), true);
 
   SnapPositionData snap = data.FindSnapPosition(*strategy);
   if (snap.type == SnapPositionData::Type::kNone) {
     return;
   }
 
-  scroll_state->data()->delta_x = snap.position.x() - current_position.x();
-  scroll_state->data()->delta_y = snap.position.y() - current_position.y();
+  scroll_state.data()->delta_x = snap.position.x() - current_position.x();
+  scroll_state.data()->delta_y = snap.position.y() - current_position.y();
 }
 
 void InputHandler::ScrollEnd(bool should_snap) {
@@ -474,7 +502,7 @@ void InputHandler::RecordScrollBegin(
   //   (2) the scroll is driven by the compositor, but blocked on the main
   //       thread.
   // Otherwise, the compositor-thread is the 'scrolling thread'.
-  // TODO(crbug.com/1060712): We should also count 'main thread' as the
+  // TODO(crbug.com/40122138): We should also count 'main thread' as the
   // 'scrolling thread' if the layer being scrolled has scroll-event handlers.
   FrameInfo::SmoothEffectDrivingThread scrolling_thread;
   switch (scroll_start_state) {
@@ -546,7 +574,9 @@ InputHandlerPointerResult InputHandler::MouseMoveAt(
 }
 
 PointerResultType InputHandler::HitTest(const gfx::PointF& viewport_point) {
-  return scrollbar_controller_->HitTest(viewport_point);
+  return scrollbar_controller_->HitTest(viewport_point)
+             ? PointerResultType::kScrollbarScroll
+             : PointerResultType::kUnhandled;
 }
 
 InputHandlerPointerResult InputHandler::MouseDown(
@@ -1023,6 +1053,9 @@ void InputHandler::ProcessCommitDeltas(
     commit_data->scroll_end_data.gesture_affects_outer_viewport_scroll =
         outer_viewport_consumed_delta_;
     outer_viewport_consumed_delta_ = false;
+    commit_data->scroll_end_data.gesture_affects_inner_viewport_scroll =
+        inner_viewport_consumed_delta_;
+    inner_viewport_consumed_delta_ = false;
   }
 }
 
@@ -1073,6 +1106,21 @@ void InputHandler::DidActivatePendingTree() {
   UpdateRootLayerStateForSynchronousInputHandler();
 }
 
+void InputHandler::DidFinishImplFrame() {
+  if (input_handler_client_) {
+    input_handler_client_->DidFinishImplFrame();
+  }
+}
+
+void InputHandler::OnBeginImplFrameDeadline() {
+  if (!IsCurrentlyScrolling()) {
+    return;
+  }
+  if (input_handler_client_) {
+    input_handler_client_->DeliverInputForDeadline();
+  }
+}
+
 void InputHandler::RootLayerStateMayHaveChanged() {
   UpdateRootLayerStateForSynchronousInputHandler();
 }
@@ -1160,6 +1208,13 @@ bool InputHandler::IsCurrentScrollMainRepainted() const {
   return repaint_reasons != MainThreadScrollingReason::kNotScrollingOnMain;
 }
 
+bool InputHandler::HasQueuedInput() const {
+  if (input_handler_client_) {
+    return input_handler_client_->HasQueuedInput();
+  }
+  return false;
+}
+
 ScrollNode* InputHandler::CurrentlyScrollingNode() {
   return GetScrollTree().CurrentlyScrollingNode();
 }
@@ -1269,11 +1324,16 @@ InputHandler::ScrollHitTestResult InputHandler::HitTestScrollNode(
       ActiveTree().FindLayersUpToFirstScrollableOrOpaqueToHitTest(
           device_viewport_point);
 
-  const LayerImpl* first_scrollable_or_opaque_to_hit_test_layer =
-      !layers.empty() && (layers.back()->IsScrollerOrScrollbar() ||
-                          layers.back()->OpaqueToHitTest())
-          ? layers.back()
-          : nullptr;
+  const LayerImpl* first_scrollable_or_opaque_to_hit_test_layer = nullptr;
+  if (!layers.empty()) {
+    if (compositor_delegate_->GetSettings().enable_hit_test_opaqueness) {
+      if (layers.back()->OpaqueToHitTest()) {
+        first_scrollable_or_opaque_to_hit_test_layer = layers.back();
+      }
+    } else if (layers.back()->IsScrollerOrScrollbar()) {
+      first_scrollable_or_opaque_to_hit_test_layer = layers.back();
+    }
+  }
   ScrollNode* node_to_scroll = nullptr;
 
   // Go through each layer up to (and including) the scroller. Any may block
@@ -1289,16 +1349,23 @@ InputHandler::ScrollHitTestResult InputHandler::HitTestScrollNode(
       return result;
     }
 
-    // If we hit a non-fast scrollable region, that means there's some reason we
-    // can't scroll in this region. Primarily, because there's another scroller
-    // there that isn't composited and we don't know about so we'll return
-    // failure.
-    if (ActiveTree().PointHitsNonFastScrollableRegion(device_viewport_point,
-                                                      *layer_impl)) {
+    // If we hit a main thread hit test region, that means there's some reason
+    // we can't scroll in this region. Primarily, because there's another
+    // scroller there that isn't composited and we don't know about so we'll
+    // return failure.
+    if (ActiveTree().PointHitsMainThreadScrollHitTestRegion(
+            device_viewport_point, *layer_impl)) {
       result.main_thread_hit_test_reasons =
-          MainThreadScrollingReason::kNonFastScrollableRegion;
+          MainThreadScrollingReason::kMainThreadScrollHitTestRegion;
       return result;
     }
+
+    if (ElementId scroll_element_id = ActiveTree().PointHitsNonCompositedScroll(
+            device_viewport_point, *layer_impl)) {
+      node_to_scroll = GetScrollTree().FindNodeFromElementId(scroll_element_id);
+      CHECK(node_to_scroll);
+      break;
+    }
   }
 
   // It's theoretically possible to hit no layers or only non-scrolling layers.
@@ -1319,6 +1386,12 @@ InputHandler::ScrollHitTestResult InputHandler::HitTestScrollNode(
 }
 
 ScrollNode* InputHandler::GetNodeToScroll(ScrollNode* node) const {
+  // The root and the secondary root are sentinel nodes and don't contribute to
+  // scrolling.
+  if (node->id <= kSecondaryRootPropertyNodeId) {
+    return nullptr;
+  }
+
   // Blink has a notion of a "root scroller", which is the scroller in a page
   // that is considered to host the main content. Typically this will be the
   // document/LayoutView contents; however, in some situations Blink may choose
@@ -1350,61 +1423,50 @@ ScrollNode* InputHandler::GetNodeToScroll(ScrollNode* node) const {
   return node;
 }
 
+ScrollNode* InputHandler::GetNodeToScrollForLayer(
+    const LayerImpl* layer) const {
+  if (layer->IsScrollbarLayer()) {
+    // If we hit a scrollbar layer, get the ScrollNode from its associated
+    // scrolling layer, rather than directly from the scrollbar layer. The
+    // latter would return the parent scroller's ScrollNode.
+    if (auto* scroll_node = GetScrollTree().FindNodeFromElementId(
+            ToScrollbarLayer(layer)->scroll_element_id())) {
+      return GetNodeToScroll(scroll_node);
+    }
+    return nullptr;
+  }
+  return GetNodeToScroll(GetScrollTree().Node(layer->scroll_tree_index()));
+}
+
 bool InputHandler::IsInitialScrollHitTestReliable(
     const LayerImpl* layer_impl,
     const LayerImpl* first_scrollable_or_opaque_to_hit_test_layer,
     ScrollNode*& out_node_to_scroll) const {
-  // Hit tests directly on a composited scrollbar are always reliable.
-  if (layer_impl->IsScrollbarLayer()) {
-    DCHECK(layer_impl == first_scrollable_or_opaque_to_hit_test_layer);
-    // If we hit a scrollbar layer, get the ScrollNode from its associated
-    // scrolling layer, rather than directly from the scrollbar layer. The
-    // latter would return the parent scroller's ScrollNode.
-    out_node_to_scroll = GetScrollTree().FindNodeFromElementId(
-        ToScrollbarLayer(layer_impl)->scroll_element_id());
-    if (out_node_to_scroll) {
-      out_node_to_scroll = GetNodeToScroll(out_node_to_scroll);
-    }
-    return true;
-  }
+  ScrollNode* scroll_node = GetNodeToScrollForLayer(layer_impl);
 
-  ScrollNode* closest_scroll_node = nullptr;
-  auto& scroll_tree = GetScrollTree();
-  ScrollNode* scroll_node = scroll_tree.Node(layer_impl->scroll_tree_index());
-  for (; scroll_tree.parent(scroll_node);
-       scroll_node = scroll_tree.parent(scroll_node)) {
-    // TODO(crbug.com/1413877): This is inconsistent with the condition in
-    // LayerTreeImpl::FindLayersUpToFirstScrollableOrOpaqueToHitTest(), and
-    // may be a reason for kFailedHitTest main thread scrolling. Can we change
-    // FindLayersUpToFirstScrollableOrOpaqueToHitTest() not to return
-    // a non-scrollable scroller, or not to check scrollable here? We may also
-    // need to consider overscroll behavior of a non-scrollable scroller.
-    if (scroll_node->scrollable) {
-      closest_scroll_node = GetNodeToScroll(scroll_node);
-      break;
-    }
+  if (layer_impl == first_scrollable_or_opaque_to_hit_test_layer) {
+    out_node_to_scroll = scroll_node;
+    return true;
   }
 
   // If there's a scrolling layer, we should also have a closest scroll node,
   // and vice versa. Otherwise, the hit test is not reliable.
-  if ((first_scrollable_or_opaque_to_hit_test_layer && !closest_scroll_node) ||
-      (closest_scroll_node && !first_scrollable_or_opaque_to_hit_test_layer)) {
+  if ((first_scrollable_or_opaque_to_hit_test_layer && !scroll_node) ||
+      (scroll_node && !first_scrollable_or_opaque_to_hit_test_layer)) {
     return false;
   }
-  if (!first_scrollable_or_opaque_to_hit_test_layer && !closest_scroll_node) {
+  if (!first_scrollable_or_opaque_to_hit_test_layer && !scroll_node) {
     // It's ok if we have neither.
     out_node_to_scroll = nullptr;
     return true;
   }
 
-  // If `first_scrollable_or_opaque_to_hit_test_layer` is not a scrollbar, and
-  // its scroll tree index corresponds to the first scrollable ancestor for
-  // `layer_impl`, the hit test has not escaped to other areas of the scroll
-  // tree and is reliable.
-  if (!first_scrollable_or_opaque_to_hit_test_layer->IsScrollbarLayer() &&
-      closest_scroll_node->id ==
-          first_scrollable_or_opaque_to_hit_test_layer->scroll_tree_index()) {
-    out_node_to_scroll = closest_scroll_node;
+  // If `first_scrollable_or_opaque_to_hit_test_layer` and `layer_impl` will
+  // scroll the same scroll node, the hit test has not escaped to other areas
+  // of the scroll tree and is reliable so far.
+  if (scroll_node ==
+      GetNodeToScrollForLayer(first_scrollable_or_opaque_to_hit_test_layer)) {
+    out_node_to_scroll = scroll_node;
     return true;
   }
 
@@ -1575,25 +1637,21 @@ gfx::Vector2dF InputHandler::ScrollSingleNode(const ScrollNode& scroll_node,
   return ScrollNodeWithLocalDelta(scroll_node, adjusted_delta);
 }
 
-void InputHandler::ScrollLatchedScroller(ScrollState* scroll_state,
+void InputHandler::ScrollLatchedScroller(ScrollState& scroll_state,
                                          base::TimeDelta delayed_by) {
   DCHECK(CurrentlyScrollingNode());
-  DCHECK(scroll_state);
   DCHECK(latched_scroll_type_.has_value());
 
   ScrollNode& scroll_node = *CurrentlyScrollingNode();
-  const gfx::Vector2dF delta(scroll_state->delta_x(), scroll_state->delta_y());
+  const gfx::Vector2dF delta(scroll_state.delta_x(), scroll_state.delta_y());
   TRACE_EVENT2("cc", "InputHandler::ScrollLatchedScroller", "delta_x",
                delta.x(), "delta_y", delta.y());
   gfx::Vector2dF applied_delta;
   gfx::Vector2dF delta_applied_to_content;
   std::optional<gfx::PointF> snap_strategy_offset;
-  // TODO(tdresser): Use a more rational epsilon. See crbug.com/510550 for
-  // details.
-  const float kEpsilon = 0.1f;
 
-  if (ShouldAnimateScroll(*scroll_state)) {
-    DCHECK(!scroll_state->is_in_inertial_phase());
+  if (ShouldAnimateScroll(scroll_state)) {
+    DCHECK(!scroll_state.is_in_inertial_phase());
 
     if (ElementId id = compositor_delegate_->GetImplDeprecated()
                            .mutator_host()
@@ -1635,10 +1693,7 @@ void InputHandler::ScrollLatchedScroller(ScrollState* scroll_state,
       if (scroll_node.scrolls_outer_viewport) {
         auto result = GetViewport().ScrollAnimated(delta, delayed_by);
         applied_delta = result.consumed_delta;
-        if (std::abs(result.outer_viewport_scrolled_delta.x()) > kEpsilon ||
-            std::abs(result.outer_viewport_scrolled_delta.y()) > kEpsilon) {
-          outer_viewport_consumed_delta_ = true;
-        }
+        SetViewportConsumedDelta(result);
       } else {
         applied_delta = ComputeScrollDelta(scroll_node, delta);
         compositor_delegate_->GetImplDeprecated().ScrollAnimationCreate(
@@ -1653,8 +1708,8 @@ void InputHandler::ScrollLatchedScroller(ScrollState* scroll_state,
     // browser controls).
     delta_applied_to_content = delta;
   } else {
-    gfx::Point viewport_point(scroll_state->position_x(),
-                              scroll_state->position_y());
+    gfx::Point viewport_point(scroll_state.position_x(),
+                              scroll_state.position_y());
     if (GetViewport().ShouldScroll(scroll_node)) {
       // |scrolls_outer_viewport| will only ever be false if the scroll chains
       // up to the viewport without going through the outer viewport scroll
@@ -1664,34 +1719,31 @@ void InputHandler::ScrollLatchedScroller(ScrollState* scroll_state,
       // want to scroll *only* the inner viewport -- to allow panning while
       // zoomed -- but still use Viewport::ScrollBy to also move browser
       // controls if needed.
-      Viewport::ScrollResult result = GetViewport().ScrollBy(
-          delta, viewport_point, scroll_state->is_direct_manipulation(),
+      ViewportScrollResult result = GetViewport().ScrollBy(
+          delta, viewport_point, scroll_state.is_direct_manipulation(),
           latched_scroll_type_ != ui::ScrollInputType::kWheel,
           scroll_node.scrolls_outer_viewport);
 
       applied_delta = result.consumed_delta;
       delta_applied_to_content = result.content_scrolled_delta;
-      if (std::abs(result.outer_viewport_scrolled_delta.x()) > kEpsilon ||
-          std::abs(result.outer_viewport_scrolled_delta.y()) > kEpsilon) {
-        outer_viewport_consumed_delta_ = true;
-      }
+      SetViewportConsumedDelta(result);
     } else {
       applied_delta = ScrollSingleNode(scroll_node, delta, viewport_point,
-                                       scroll_state->is_direct_manipulation());
+                                       scroll_state.is_direct_manipulation());
     }
     snap_strategy_offset = GetVisualScrollOffset(scroll_node);
   }
   overscroll_delta_for_main_thread_ += delta - applied_delta;
 
   // If the layer wasn't able to move, try the next one in the hierarchy.
-  bool scrolled = std::abs(applied_delta.x()) > kEpsilon;
-  scrolled = scrolled || std::abs(applied_delta.y()) > kEpsilon;
+  bool scrolled = std::abs(applied_delta.x()) > kScrollEpsilon;
+  scrolled = scrolled || std::abs(applied_delta.y()) > kScrollEpsilon;
   if (!scrolled) {
     // TODO(bokan): This preserves existing behavior by not allowing tiny
     // scrolls to produce overscroll but is inconsistent in how delta gets
     // chained up. We need to clean this up.
     if (scroll_node.scrolls_outer_viewport)
-      scroll_state->ConsumeDelta(applied_delta.x(), applied_delta.y());
+      scroll_state.ConsumeDelta(applied_delta.x(), applied_delta.y());
     return;
   }
 
@@ -1711,20 +1763,28 @@ void InputHandler::ScrollLatchedScroller(ScrollState* scroll_state,
     delta_applied_to_content = applied_delta;
   }
 
-  scroll_state->set_caused_scroll(
-      std::abs(delta_applied_to_content.x()) > kEpsilon,
-      std::abs(delta_applied_to_content.y()) > kEpsilon);
-  scroll_state->ConsumeDelta(applied_delta.x(), applied_delta.y());
+  scroll_state.set_caused_scroll(
+      std::abs(delta_applied_to_content.x()) > kScrollEpsilon,
+      std::abs(delta_applied_to_content.y()) > kScrollEpsilon);
+  scroll_state.ConsumeDelta(applied_delta.x(), applied_delta.y());
 
-  did_scroll_x_for_scroll_gesture_ |= scroll_state->caused_scroll_x();
-  did_scroll_y_for_scroll_gesture_ |= scroll_state->caused_scroll_y();
+  did_scroll_x_for_scroll_gesture_ |= scroll_state.caused_scroll_x();
+  did_scroll_y_for_scroll_gesture_ |= scroll_state.caused_scroll_y();
 
-  if (snap_strategy_offset && !scroll_state->is_in_inertial_phase()) {
+  if (snap_strategy_offset && !scroll_state.is_in_inertial_phase()) {
     // We use |last_scroll_update_state_| instead of |scroll_state| as that more
     // closely matches what InputHandler::SnapAtScrollend would use.
-    snap_strategy_ = CreateSnapStrategy(
-        last_scroll_update_state_.value(), snap_strategy_offset.value(),
-        SnapReason::kScrollOffsetAnimationFinished);
+    //
+    // We validate that `last_scroll_update_state_` exists before using it. As
+    // we have seen rare crashes with it null. We do not use
+    // `std::optional::value_or` here as that performs a copy of the
+    // alternative. Which will rarely ever be needed.
+    snap_strategy_ =
+        CreateSnapStrategy(last_scroll_update_state_.has_value()
+                               ? last_scroll_update_state_.value()
+                               : scroll_state,
+                           snap_strategy_offset.value(),
+                           SnapReason::kScrollOffsetAnimationFinished);
   }
 }
 
@@ -1751,8 +1811,10 @@ ScrollNode* InputHandler::FindNodeToLatch(ScrollState* scroll_state,
       break;
     }
 
-    if (!cur_node->scrollable)
+    if (!cur_node->user_scrollable_horizontal &&
+        !cur_node->user_scrollable_vertical) {
       continue;
+    }
 
     if (!first_scrollable_node) {
       first_scrollable_node = cur_node;
@@ -1875,9 +1937,12 @@ bool InputHandler::SnapAtScrollEnd(SnapReason reason) {
   SnapContainerData& data = scroll_node->snap_container_data.value();
   gfx::PointF current_position = GetVisualScrollOffset(*scroll_node);
 
-  if (!snap_strategy_) {
-    // You might think that if a scroll never received a scroll update we could
-    // just drop the snap. However, if the GSB+GSE arrived while we were
+  if (!snap_strategy_ || snap_fling_state_ == kConstrainedNativeFling) {
+    // If this was a constrained native fling, SnapFlingController would not
+    // have had the correct final scroll position with which to create the snap
+    // strategy.
+    // Also, you might think that if a scroll never received a scroll update we
+    // could just drop the snap. However, if the GSB+GSE arrived while we were
     // mid-snap from a previous gesture, this would leave the scroller at a
     // non-snap-point.
     DCHECK(last_scroll_update_state_ || last_scroll_begin_state_);
@@ -1895,6 +1960,15 @@ bool InputHandler::SnapAtScrollEnd(SnapReason reason) {
   SnapPositionData snap = data.FindSnapPositionWithViewportAdjustment(
       *snap_strategy_, snapport_height_adjustment);
   if (snap.type == SnapPositionData::Type::kNone) {
+    // Ensure we retain the ids of any element we were previously snapped to and
+    // are still snapped to in case of scrolls in an axis where no snapping
+    // happens.
+    if (reason == SnapReason::kScrollOffsetAnimationFinished) {
+      scroll_animating_snap_target_ids_ = snap.target_element_ids;
+    } else if (data.SetTargetSnapAreaElementIds(snap.target_element_ids)) {
+      updated_snapped_elements_[scroll_node->element_id] =
+          snap.target_element_ids;
+    }
     return false;
   }
 
@@ -1908,11 +1982,10 @@ bool InputHandler::SnapAtScrollEnd(SnapReason reason) {
   if (scroll_node->scrolls_outer_viewport) {
     gfx::Vector2dF scaled_delta(delta);
     scaled_delta.Scale(compositor_delegate_->PageScaleFactor());
-    gfx::Vector2dF consumed_delta =
-        GetViewport()
-            .ScrollAnimated(scaled_delta, base::TimeDelta())
-            .consumed_delta;
+    auto result = GetViewport().ScrollAnimated(scaled_delta, base::TimeDelta());
+    gfx::Vector2dF consumed_delta = result.consumed_delta;
     did_animate = !consumed_delta.IsZero();
+    SetViewportConsumedDelta(result);
   } else {
     did_animate =
         compositor_delegate_->GetImplDeprecated().ScrollAnimationCreate(
@@ -2009,7 +2082,8 @@ void InputHandler::UpdateScrollSourceInfo(const ScrollState& scroll_state,
 // Return true if scrollable node for 'ancestor' is the same as 'child' or an
 // ancestor along the scroll tree.
 bool InputHandler::IsScrolledBy(LayerImpl* child, ScrollNode* ancestor) {
-  DCHECK(ancestor && ancestor->scrollable);
+  DCHECK(ancestor && (ancestor->user_scrollable_horizontal ||
+                      ancestor->user_scrollable_vertical));
   if (!child)
     return false;
   DCHECK_EQ(child->layer_tree_impl(), &ActiveTree());
@@ -2042,11 +2116,13 @@ void InputHandler::SetDeferBeginMainFrame(bool defer_begin_main_frame) const {
   compositor_delegate_->SetDeferBeginMainFrame(defer_begin_main_frame);
 }
 
-void InputHandler::UpdateBrowserControlsState(BrowserControlsState constraints,
-                                              BrowserControlsState current,
-                                              bool animate) {
+void InputHandler::UpdateBrowserControlsState(
+    BrowserControlsState constraints,
+    BrowserControlsState current,
+    bool animate,
+    base::optional_ref<const BrowserControlsOffsetTagsInfo> offset_tags_info) {
   compositor_delegate_->UpdateBrowserControlsState(constraints, current,
-                                                   animate);
+                                                   animate, offset_tags_info);
 }
 
 void InputHandler::SetIsHandlingTouchSequence(bool is_handling_touch_sequence) {
@@ -2081,7 +2157,7 @@ std::unique_ptr<SnapSelectionStrategy> InputHandler::CreateSnapStrategy(
     // Note: gesture scroll end is delayed in anticipation of future wheel
     // scrolls so it is fired well after the scroll ends as opposed to precise
     // touch devices where we fire it as soon as the user lifts their finger.
-    // TODO(crbug.com/1201678): The directional scroll should probably be
+    // TODO(crbug.com/40762499): The directional scroll should probably be
     // triggered at gesture scroll begin to improve responsiveness.
     return SnapSelectionStrategy::CreateForDirection(current_offset,
                                                      scroll_delta, true);
@@ -2092,4 +2168,16 @@ std::unique_ptr<SnapSelectionStrategy> InputHandler::CreateSnapStrategy(
   }
 }
 
+void InputHandler::SetViewportConsumedDelta(
+    const ViewportScrollResult& result) {
+  if (std::abs(result.outer_viewport_scrolled_delta.x()) > kScrollEpsilon ||
+      std::abs(result.outer_viewport_scrolled_delta.y()) > kScrollEpsilon) {
+    outer_viewport_consumed_delta_ = true;
+  }
+  if (std::abs(result.inner_viewport_scrolled_delta.x()) > kScrollEpsilon ||
+      std::abs(result.inner_viewport_scrolled_delta.y()) > kScrollEpsilon) {
+    inner_viewport_consumed_delta_ = true;
+  }
+}
+
 }  // namespace cc
