@@ -11,7 +11,9 @@
 #include "base/memory/raw_ptr.h"
 #include "base/memory/raw_ref.h"
 #include "base/time/time.h"
+#include "base/types/optional_ref.h"
 #include "cc/cc_export.h"
+#include "cc/input/browser_controls_offset_tags_info.h"
 #include "cc/input/browser_controls_state.h"
 #include "cc/input/compositor_input_interfaces.h"
 #include "cc/input/event_listener_properties.h"
@@ -113,6 +115,30 @@ struct CC_EXPORT InputHandlerScrollResult {
 
 class CC_EXPORT InputHandlerClient {
  public:
+  enum class ScrollEventDispatchMode {
+    // Scroll events arriving will be enqueued to be dispatched during the next
+    // `DeliverInputForBeginFrame`.
+    kEnqueueScrollEvents,
+
+    // Scroll events arriving will be dispatched immediately, if
+    // `DeliverInputForBeginFrame` was called while scrolling, with no input
+    // events in the queue. This will occur until frame production has started,
+    // or completed.
+    kDispatchScrollEventsImmediately,
+
+    // If there are no queued events when `DeliverInputForBeginFrame` is called,
+    // while we are scrolling. We will generate a new prediction, and then
+    // dispatch a synthetic `GestureScrollUpdate` using the prediction.
+    kUseScrollPredictorForEmptyQueue,
+
+    // Will perform as `kDispatchScrollEventsImmediately` until the deadline.
+    // Instead of immediately resuming frame production, we will first attempt
+    // to generate a new prediction to dispatch. As in
+    // `kUseScrollPredictorForEmptyQueue`. After which we will resume frame
+    // production and enqueuing input.
+    kUseScrollPredictorForDeadline,
+  };
+
   InputHandlerClient(const InputHandlerClient&) = delete;
   virtual ~InputHandlerClient() = default;
 
@@ -131,6 +157,10 @@ class CC_EXPORT InputHandlerClient {
       float max_page_scale_factor) = 0;
   virtual void DeliverInputForBeginFrame(const viz::BeginFrameArgs& args) = 0;
   virtual void DeliverInputForHighLatencyMode() = 0;
+  virtual void DeliverInputForDeadline() = 0;
+  virtual void DidFinishImplFrame() = 0;
+  virtual bool HasQueuedInput() const = 0;
+  virtual void SetScrollEventDispatchMode(ScrollEventDispatchMode mode) = 0;
 
  protected:
   InputHandlerClient() = default;
@@ -228,7 +258,7 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
     uint32_t main_thread_repaint_reasons =
         MainThreadScrollingReason::kNotScrollingOnMain;
 
-    // TODO(crbug.com/1155758): This is a temporary workaround for GuestViews
+    // TODO(crbug.com/40735567): This is a temporary workaround for GuestViews
     // as they create viewport nodes and want to bubble scroll if the
     // viewport cannot scroll in the given delta directions. There should be
     // a parameter to ThreadInputHandler to specify whether unused delta is
@@ -236,6 +266,23 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
     bool viewport_cannot_scroll = false;
   };
 
+  // ViewportScrollResult records, for a scroll gesture affecting a page's
+  // viewport:
+  // - the amount from the scroll gesture's delta that actually resulted in
+  //   scrolling: |consumed_delta|,
+  // - the amount from the scroll gesture's delta that applied to the content of
+  //   the page, i.e. excluding movement of browser controls.
+  // - the distribution of the scroll gesture's delta between the inner and
+  //   outer viewports, {inner,outer}_viewport_consumed_delta_
+  // TODO(tdresser): eventually |consumed_delta| should equal
+  // |content_scrolled_delta|. See crbug.com/510045 for details.
+  struct ViewportScrollResult {
+    gfx::Vector2dF consumed_delta;
+    gfx::Vector2dF content_scrolled_delta;
+    gfx::Vector2dF outer_viewport_scrolled_delta;
+    gfx::Vector2dF inner_viewport_scrolled_delta;
+  };
+
   enum class TouchStartOrMoveEventListenerType {
     kNoHandler,
     kHandler,
@@ -279,7 +326,7 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   // taken into account when determining the duration of the animation if one
   // is created.
   virtual InputHandlerScrollResult ScrollUpdate(
-      ScrollState* scroll_state,
+      ScrollState scroll_state,
       base::TimeDelta delayed_by = base::TimeDelta());
 
   // Stop scrolling the selected layer. Must be called only if ScrollBegin()
@@ -410,9 +457,11 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   // compositor thread has had a chance to update the scroll offset.
   virtual void SetDeferBeginMainFrame(bool defer_begin_main_frame) const;
 
-  virtual void UpdateBrowserControlsState(BrowserControlsState constraints,
-                                          BrowserControlsState current,
-                                          bool animate);
+  virtual void UpdateBrowserControlsState(
+      BrowserControlsState constraints,
+      BrowserControlsState current,
+      bool animate,
+      base::optional_ref<const BrowserControlsOffsetTagsInfo> offset_tags_info);
 
   virtual void SetIsHandlingTouchSequence(bool is_handling_touch_sequence);
 
@@ -470,6 +519,10 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
     return snap_strategy_;
   }
 
+  // Detects whether or not the scroll generating the |result| affected the
+  // inner or outer viewports.
+  void SetViewportConsumedDelta(const ViewportScrollResult& result);
+
   // =========== InputDelegateForCompositor Interface - This section implements
   // the interface that LayerTreeHostImpl uses to communicate with the input
   // system.
@@ -482,6 +535,8 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   void WillBeginImplFrame(const viz::BeginFrameArgs& args) override;
   void DidCommit() override;
   void DidActivatePendingTree() override;
+  void DidFinishImplFrame() override;
+  void OnBeginImplFrameDeadline() override;
   void RootLayerStateMayHaveChanged() override;
   void DidRegisterScrollbar(ElementId scroll_element_id,
                             ScrollbarOrientation orientation) override;
@@ -493,6 +548,7 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   ActivelyScrollingType GetActivelyScrollingType() const override;
   bool IsHandlingTouchSequence() const override;
   bool IsCurrentScrollMainRepainted() const override;
+  bool HasQueuedInput() const override;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(LayerTreeHostImplTest,
@@ -538,14 +594,14 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   // compositor side. The information gets shared by the main thread as part of
   // the begin_main_frame_state. Finally Use counters are updated in the main
   // thread side to keep track of the frequency of scrolling with different
-  // sources per page load. TODO(crbug.com/691886): Use GRC API to plumb the
+  // sources per page load. TODO(crbug.com/40506330): Use GRC API to plumb the
   // scroll source info for Use Counters.
   void UpdateScrollSourceInfo(const ScrollState& scroll_state,
                               ui::ScrollInputType type);
 
   // Applies the scroll_state to the currently latched scroller. See comment in
   // InputHandler::ScrollUpdate declaration for the meaning of |delayed_by|.
-  void ScrollLatchedScroller(ScrollState* scroll_state,
+  void ScrollLatchedScroller(ScrollState& scroll_state,
                              base::TimeDelta delayed_by);
 
   enum class SnapReason { kGestureScrollEnd, kScrollOffsetAnimationFinished };
@@ -572,6 +628,7 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   // Returns the ScrollNode we should use to scroll, accounting for viewport
   // scroll chaining rules.
   ScrollNode* GetNodeToScroll(ScrollNode* node) const;
+  ScrollNode* GetNodeToScrollForLayer(const LayerImpl* layer) const;
 
   // Given a starting node (determined by hit-test), walks up the scroll tree
   // looking for the first node that can consume scroll from the given
@@ -631,7 +688,7 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   gfx::Vector2dF UserScrollableDelta(const ScrollNode& node,
                                      const gfx::Vector2dF& delta) const;
 
-  void AdjustScrollDeltaForScrollbarSnap(ScrollState* scroll_state);
+  void AdjustScrollDeltaForScrollbarSnap(ScrollState& scroll_state);
 
   FrameSequenceTrackerType GetTrackerTypeForScroll(
       ui::ScrollInputType input_type) const;
@@ -737,6 +794,10 @@ class CC_EXPORT InputHandler : public InputDelegateForCompositor {
   // applied to the layout viewport.
   bool outer_viewport_consumed_delta_ = false;
 
+  // True if any of the non-zero deltas in a begin/update/end sequence was
+  // applied to the visual viewport.
+  bool inner_viewport_consumed_delta_ = false;
+
   // TODO(bokan): Mac doesn't yet have smooth scrolling for wheel; however, to
   // allow consistency in tests we use this bit to override that decision.
   // https://crbug.com/574283.
