@@ -31,6 +31,14 @@ HeadsUpDisplayLayer::HeadsUpDisplayLayer()
 
 HeadsUpDisplayLayer::~HeadsUpDisplayLayer() = default;
 
+void HeadsUpDisplayLayer::SetLayerTreeHost(LayerTreeHost* host) {
+  if (host && host != layer_tree_host()) {
+    paused_debugger_message_ =
+        host->client()->GetPausedDebuggerLocalizedMessage();
+  }
+  Layer::SetLayerTreeHost(host);
+}
+
 void HeadsUpDisplayLayer::UpdateLocationAndSize(
     const gfx::Size& device_viewport,
     float device_scale_factor) {
@@ -46,16 +54,9 @@ void HeadsUpDisplayLayer::UpdateLocationAndSize(
   constexpr int kDefaultHUDSize = 256;
   bounds_in_dips.SetSize(kDefaultHUDSize, kDefaultHUDSize);
 
-  if (layer_tree_host()->GetDebugState().ShowDebugRects()) {
+  if (layer_tree_host()->GetDebugState().ShowDebugRects() ||
+      layer_tree_host()->GetDebugState().debugger_paused) {
     bounds_in_dips = device_viewport_in_dips;
-  } else if (layer_tree_host()->GetDebugState().show_web_vital_metrics ||
-             layer_tree_host()->GetDebugState().show_smoothness_metrics) {
-    // If the HUD is used to display performance metrics (which is on the right
-    // hand side_, make sure the bounds has the correct width, with a fixed
-    // height.
-    bounds_in_dips.set_width(device_viewport_in_dips.width());
-    // Increase HUD layer height to make sure all the metrics are showing.
-    bounds_in_dips.set_height(kDefaultHUDSize * 2);
   }
 
   // DIPs are layout coordinates if painted dsf is 1. If it's not 1, then layout
@@ -72,7 +73,8 @@ bool HeadsUpDisplayLayer::HasDrawableContent() const {
 
 std::unique_ptr<LayerImpl> HeadsUpDisplayLayer::CreateLayerImpl(
     LayerTreeImpl* tree_impl) const {
-  return HeadsUpDisplayLayerImpl::Create(tree_impl, id());
+  return HeadsUpDisplayLayerImpl::Create(tree_impl, id(),
+                                         paused_debugger_message_);
 }
 
 const std::vector<gfx::Rect>& HeadsUpDisplayLayer::LayoutShiftRects() const {
@@ -84,10 +86,6 @@ void HeadsUpDisplayLayer::SetLayoutShiftRects(
   layout_shift_rects_.Write(*this) = rects;
 }
 
-void HeadsUpDisplayLayer::UpdateWebVitalMetrics(
-    std::unique_ptr<WebVitalMetrics> web_vital_metrics) {
-  web_vital_metrics_.Write(*this) = std::move(web_vital_metrics);
-}
 
 void HeadsUpDisplayLayer::PushPropertiesTo(
     LayerImpl* layer,
@@ -101,9 +99,6 @@ void HeadsUpDisplayLayer::PushPropertiesTo(
   layer_impl->SetHUDTypeface(typeface_.Write(*this));
   layer_impl->SetLayoutShiftRects(LayoutShiftRects());
   layout_shift_rects_.Write(*this).clear();
-  auto& metrics = web_vital_metrics_.Write(*this);
-  if (metrics && metrics->HasValue())
-    layer_impl->SetWebVitalMetrics(std::move(metrics));
 }
 
 }  // namespace cc
