@@ -7,6 +7,7 @@
 
 #include <stddef.h>
 #include <stdint.h>
+
 #include <array>
 #include <memory>
 #include <set>
@@ -24,6 +25,7 @@
 #include "cc/input/hit_test_opaqueness.h"
 #include "cc/input/scroll_snap_data.h"
 #include "cc/layers/layer_collections.h"
+#include "cc/layers/scroll_hit_test_rect.h"
 #include "cc/layers/touch_action_region.h"
 #include "cc/paint/element_id.h"
 #include "cc/paint/filter_operations.h"
@@ -473,31 +475,32 @@ class CC_EXPORT Layer : public base::RefCounted<Layer>,
 
   virtual bool IsScrollbarLayerForTesting() const;
 
-  // For layer tree mode only.
-  // Set or get if this layer is able to be scrolled along each axis. These are
-  // independent of the scrollable state, or size of the scrollable area
-  // specified in SetScrollable(), as these may be enabled or disabled
-  // dynamically, while SetScrollable() defines what would be possible if these
-  // are enabled.
-  // When disabled, overscroll elasticity will not be used if the scroll offset
-  // ends up past the maximum range. And when enabled, with overlay scrollbars,
-  // the scrollbars will be shown when the scroll offset changes if these are
-  // set to true.
-  void SetUserScrollable(bool horizontal, bool vertical);
-  bool GetUserScrollableHorizontal() const;
-  bool GetUserScrollableVertical() const;
-
-  // Set or get an area of this layer within which initiating a scroll can not
-  // be done from the compositor thread. Within this area, if the user attempts
-  // to start a scroll, the events must be sent to the main thread and processed
+  // For layer list mode only.
+  // Set or get an area of this layer within which a scroll hit-test can not be
+  // done from the compositor thread. Within this area, if the user attempts to
+  // start a scroll, the events must be sent to the main thread and processed
   // there.
-  void SetNonFastScrollableRegion(const Region& non_fast_scrollable_region);
-  const Region& non_fast_scrollable_region() const {
+  void SetMainThreadScrollHitTestRegion(
+      const Region& main_thread_scroll_hit_test_region);
+  const Region& main_thread_scroll_hit_test_region() const {
     if (const auto& rare_inputs = inputs_.Read(*this).rare_inputs)
-      return rare_inputs->non_fast_scrollable_region;
+      return rare_inputs->main_thread_scroll_hit_test_region;
     return Region::Empty();
   }
 
+  // For layer list mode only.
+  // A scroll in any of the rects but not in non_fast_scrollable_region can
+  // start on the compositor thread. The scroll node is determined by checking
+  // non_composited_scroll_hit_test_rects in reversed order.
+  void SetNonCompositedScrollHitTestRects(std::vector<ScrollHitTestRect> rects);
+  const std::vector<ScrollHitTestRect>* non_composited_scroll_hit_test_rects()
+      const {
+    if (const auto& rare_inputs = inputs_.Read(*this).rare_inputs) {
+      return &rare_inputs->non_composited_scroll_hit_test_rects;
+    }
+    return nullptr;
+  }
+
   // Set or get the set of touch actions allowed across each point of this
   // layer. The |touch_action_region| can specify, for any number of areas,
   // which touch actions are allowed in each area. The result is the
@@ -981,8 +984,12 @@ class CC_EXPORT Layer : public base::RefCounted<Layer>,
   // generally speaking in <10% of use cases. When adding new values to this
   // struct, consider the memory implications versus simply adding to Inputs.
   struct RareInputs {
+    RareInputs();
+    ~RareInputs();
+
     viz::RegionCaptureBounds capture_bounds;
-    Region non_fast_scrollable_region;
+    Region main_thread_scroll_hit_test_region;
+    std::vector<ScrollHitTestRect> non_composited_scroll_hit_test_rects;
     Region wheel_event_region;
   };
 
@@ -1044,9 +1051,6 @@ class CC_EXPORT Layer : public base::RefCounted<Layer>,
     // surface on itself if possible.
     bool is_fast_rounded_corner : 1 = false;
 
-    bool user_scrollable_horizontal : 1 = true;
-    bool user_scrollable_vertical : 1 = true;
-
     bool trilinear_filtering : 1 = false;
 
     bool hide_layer_and_subtree : 1 = false;
