@@ -9,6 +9,7 @@
 #include <unordered_map>
 #include <vector>
 
+#include "base/memory/raw_ptr_exclusion.h"
 #include "base/memory/ref_counted.h"
 #include "cc/cc_export.h"
 
@@ -19,8 +20,9 @@ class RenderSurfaceImpl;
 
 using LayerList = std::vector<scoped_refptr<Layer>>;
 using OwnedLayerImplList = std::vector<std::unique_ptr<LayerImpl>>;
-using LayerImplList = std::vector<LayerImpl*>;
-using RenderSurfaceList = std::vector<RenderSurfaceImpl*>;
+// RAW_PTR_EXCLUSION: Renderer performance: visible in sampling profiler stacks.
+using LayerImplList = RAW_PTR_EXCLUSION std::vector<LayerImpl*>;
+using RenderSurfaceList = RAW_PTR_EXCLUSION std::vector<RenderSurfaceImpl*>;
 using OwnedLayerImplMap = std::unordered_map<int, std::unique_ptr<LayerImpl>>;
 using LayerImplMap = std::unordered_map<int, LayerImpl*>;
 
