@@ -1117,12 +1117,11 @@ TEST_F(LayerTest, CheckPropertyChangeCausesCorrectBehavior) {
   // We can use any layer pointer here since we aren't syncing for real.
   EXPECT_SET_NEEDS_COMMIT_WAS_CALLED(
       test_layer->SetScrollable(gfx::Size(1, 1)));
-  EXPECT_SET_NEEDS_COMMIT_WAS_CALLED(
-      test_layer->SetUserScrollable(true, false));
   EXPECT_SET_NEEDS_COMMIT_WAS_CALLED(
       test_layer->SetScrollOffset(gfx::PointF(10, 10)));
   EXPECT_SET_NEEDS_COMMIT_WAS_CALLED(
-      test_layer->SetNonFastScrollableRegion(Region(gfx::Rect(1, 1, 2, 2))));
+      test_layer->SetMainThreadScrollHitTestRegion(
+          Region(gfx::Rect(1, 1, 2, 2))));
   EXPECT_SET_NEEDS_COMMIT_WAS_CALLED(
       test_layer->SetTransform(gfx::Transform::MakeScale(0.0)));
   TouchActionRegion touch_action_region;
