@@ -6,10 +6,10 @@
 #define CC_LAYERS_TEXTURE_LAYER_IMPL_H_
 
 #include <memory>
+#include <optional>
 #include <string>
 #include <vector>
 
-#include <optional>
 #include "base/containers/flat_map.h"
 #include "base/functional/callback.h"
 #include "base/memory/ptr_util.h"
@@ -33,6 +33,7 @@ class CC_EXPORT TextureLayerImpl : public LayerImpl {
 
   TextureLayerImpl& operator=(const TextureLayerImpl&) = delete;
 
+  mojom::LayerType GetLayerType() const override;
   std::unique_ptr<LayerImpl> CreateLayerImpl(
       LayerTreeImpl* layer_tree_impl) const override;
   bool IsSnappedToPixelGridInTarget() override;
@@ -62,6 +63,7 @@ class CC_EXPORT TextureLayerImpl : public LayerImpl {
 
   void SetTransferableResource(const viz::TransferableResource& resource,
                                viz::ReleaseCallback release_callback);
+  bool NeedSetTransferableResource() const;
 
   // These methods notify the display compositor, through the
   // CompositorFrameSink, of the existence of a SharedBitmapId and its
@@ -76,11 +78,16 @@ class CC_EXPORT TextureLayerImpl : public LayerImpl {
   void RegisterSharedBitmapId(viz::SharedBitmapId id,
                               scoped_refptr<CrossThreadSharedBitmap> bitmap);
   void UnregisterSharedBitmapId(viz::SharedBitmapId id);
+  void SetInInvisibleLayerTree() override;
+  // Whether the resource may be evicted in background. If it returns true, main
+  // is responsible for making sure that the resource is imported again after a
+  // visibility change.
+  static bool MayEvictResourceInBackground(
+      viz::TransferableResource::ResourceSource source);
 
  private:
   TextureLayerImpl(LayerTreeImpl* tree_impl, int id);
 
-  const char* LayerTypeAsString() const override;
   void FreeTransferableResource();
   void OnResourceEvicted();
 
@@ -91,7 +98,6 @@ class CC_EXPORT TextureLayerImpl : public LayerImpl {
   bool nearest_neighbor_ = false;
   gfx::PointF uv_top_left_ = gfx::PointF();
   gfx::PointF uv_bottom_right_ = gfx::PointF(1.f, 1.f);
-  gfx::HDRMetadata hdr_metadata_;
 
   // True while the |transferable_resource_| is owned by this layer, and
   // becomes false once it is passed to another layer or to the
