@@ -28,13 +28,13 @@ TEST(TextureLayerImplTest, VisibleOpaqueRegion) {
   LayerTreeImplTestBase impl;
 
   auto resource = viz::TransferableResource::MakeGpu(
-      gpu::Mailbox::GenerateForSharedImage(), GL_TEXTURE_2D,
+      gpu::Mailbox::Generate(), GL_TEXTURE_2D,
       gpu::SyncToken(gpu::CommandBufferNamespace::GPU_IO,
                      gpu::CommandBufferId::FromUnsafeValue(0x234), 0x456),
       layer_bounds, viz::SinglePlaneFormat::kRGBA_8888,
       false /* is_overlay_candidate */);
 
-  TextureLayerImpl* layer = impl.AddLayer<TextureLayerImpl>();
+  TextureLayerImpl* layer = impl.AddLayerInActiveTree<TextureLayerImpl>();
   layer->SetBounds(layer_bounds);
   layer->draw_properties().visible_layer_rect = layer_rect;
   layer->SetBlendBackgroundColor(true);
@@ -62,13 +62,14 @@ TEST(TextureLayerImplTest, Occlusion) {
   LayerTreeImplTestBase impl;
 
   auto resource = viz::TransferableResource::MakeGpu(
-      gpu::Mailbox::GenerateForSharedImage(), GL_TEXTURE_2D,
+      gpu::Mailbox::Generate(), GL_TEXTURE_2D,
       gpu::SyncToken(gpu::CommandBufferNamespace::GPU_IO,
                      gpu::CommandBufferId::FromUnsafeValue(0x234), 0x456),
       layer_size, viz::SinglePlaneFormat::kRGBA_8888,
       false /* is_overlay_candidate */);
 
-  TextureLayerImpl* texture_layer_impl = impl.AddLayer<TextureLayerImpl>();
+  TextureLayerImpl* texture_layer_impl =
+      impl.AddLayerInActiveTree<TextureLayerImpl>();
   texture_layer_impl->SetBounds(layer_size);
   texture_layer_impl->SetDrawsContent(true);
   texture_layer_impl->SetTransferableResource(resource,
