@@ -8,15 +8,14 @@
 #include <bitset>
 #include <deque>
 #include <memory>
+#include <optional>
 #include <queue>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <optional>
 #include "base/memory/raw_ptr.h"
 #include "base/memory/raw_ptr_exclusion.h"
-#include "base/rand_util.h"
 #include "base/time/default_tick_clock.h"
 #include "base/time/time.h"
 #include "cc/base/devtools_instrumentation.h"
@@ -48,8 +47,6 @@ struct GlobalMetricsTrackers {
   RAW_PTR_EXCLUSION LatencyUkmReporter* latency_ukm_reporter = nullptr;
   RAW_PTR_EXCLUSION FrameSequenceTrackerCollection* frame_sequence_trackers =
       nullptr;
-  // TODO(crbug.com/1489080): This member was marked `DanglingUntriaged`
-  // before being unrewritten.
   RAW_PTR_EXCLUSION EventLatencyTracker* event_latency_tracker = nullptr;
   RAW_PTR_EXCLUSION PredictorJankTracker* predictor_jank_tracker = nullptr;
   RAW_PTR_EXCLUSION ScrollJankDroppedFrameTracker*
@@ -121,6 +118,8 @@ class CC_EXPORT CompositorFrameReporter {
   // earlier in `VizBreakdown`) for traces to record them correctly. The only
   // exception is `kSwapStartToSwapEnd` and its breakdowns as we either record
   // the former or the latter in a trace, but not both.
+  // These values are persisted to logs. Entries should not be renumbered and
+  // numeric values should never be reused.
   enum class VizBreakdown {
     kSubmitToReceiveCompositorFrame = 0,
     kReceivedCompositorFrameToStartDraw = 1,
@@ -135,7 +134,8 @@ class CC_EXPORT CompositorFrameReporter {
     kLatchToSwapEnd = 7,
 
     kSwapEndToPresentationCompositorFrame = 8,
-    kBreakdownCount
+    kBreakdownCount,
+    kMaxValue = kBreakdownCount
   };
 
   enum class BlinkBreakdown {
@@ -153,6 +153,20 @@ class CC_EXPORT CompositorFrameReporter {
     kBreakdownCount
   };
 
+  // These numbers are used for indexing UMA histograms. The order should be
+  // preserved, and entries should not be deleted.
+  //
+  // These represent ratios of stages in EventMetrics::DispatchStage to the
+  // VSync time when the event originally arrived. This can be different than
+  // the frame where this event was eventually presented.
+  enum class VSyncRatioType {
+    kArrivedInRendererVsVSyncRatioAfterVSync = 0,
+    kArrivedInRendererVsVSyncRatioBeforeVSync = 1,
+    kGenerationVsVsyncRatioAfterVSync = 2,
+    kGenerationVsVsyncRatioBeforeVSync = 3,
+    kVSyncRatioTypeCount
+  };
+
   // To distinguish between impl and main reporter
   enum class ReporterType { kImpl = 0, kMain = 1 };
 
@@ -367,8 +381,15 @@ class CC_EXPORT CompositorFrameReporter {
     tick_clock_ = tick_clock;
   }
 
-  void set_has_missing_content(bool has_missing_content) {
-    has_missing_content_ = has_missing_content;
+  void set_checkerboarded_needs_raster(bool checkerboarded_needs_raster) {
+    checkerboarded_needs_raster_ = checkerboarded_needs_raster;
+  }
+  void set_checkerboarded_needs_record(bool checkerboarded_needs_record) {
+    checkerboarded_needs_record_ = checkerboarded_needs_record;
+  }
+
+  void set_top_controls_moved(bool top_controls_moved) {
+    top_controls_moved_ = top_controls_moved;
   }
 
   void SetPartialUpdateDecider(CompositorFrameReporter* decider);
@@ -559,9 +580,13 @@ class CC_EXPORT CompositorFrameReporter {
   const SmoothThread smooth_thread_;
   const int layer_tree_host_id_;
 
-  // Indicates whether the submitted frame had any missing content (i.e. content
-  // with checkerboarding).
-  bool has_missing_content_ = false;
+  // Indicates whether the submitted frame had any missing or incomplete
+  // content (i.e. content with checkerboarding), due to rasterization and
+  // recording, respectively.
+  bool checkerboarded_needs_raster_ = false;
+  bool checkerboarded_needs_record_ = false;
+
+  bool top_controls_moved_ = false;
 
   // Indicates whether the frame is forked (i.e. a PipelineReporter event starts
   // at the same frame sequence as another PipelineReporter).
@@ -595,8 +620,6 @@ class CC_EXPORT CompositorFrameReporter {
 
   ReporterType reporter_type_;
 
-  mutable base::MetricsSubSampler metrics_subsampler_;
-
   base::WeakPtrFactory<CompositorFrameReporter> weak_factory_{this};
 };
 
