@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/351564777): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "cc/metrics/events_metrics_manager.h"
 
 #include <utility>
@@ -80,19 +85,22 @@ TEST_F(EventsMetricsManagerTest, EventsMetricsSaved) {
   std::pair<std::unique_ptr<EventMetrics>, Behavior> events[] = {
       // An interesting event type for which SaveActiveEventMetrics() is not
       // called.
-      {CreateEventMetrics(ui::ET_MOUSE_PRESSED), Behavior::kDoNotSave},
+      {CreateEventMetrics(ui::EventType::kMousePressed), Behavior::kDoNotSave},
 
       // An interesting event type for which SaveActiveEventMetrics() is called
       // inside its monitor scope.
-      {CreateEventMetrics(ui::ET_MOUSE_PRESSED), Behavior::kSaveInsideScope},
+      {CreateEventMetrics(ui::EventType::kMousePressed),
+       Behavior::kSaveInsideScope},
 
       // An interesting event type for which SaveActiveEventMetrics() is called
       // after its monitor scope is finished.
-      {CreateEventMetrics(ui::ET_MOUSE_PRESSED), Behavior::kSaveOutsideScope},
+      {CreateEventMetrics(ui::EventType::kMousePressed),
+       Behavior::kSaveOutsideScope},
 
       // A non-interesting event type for which SaveActiveEventMetrics() is
       // called inside its monitor scope.
-      {CreateEventMetrics(ui::ET_MOUSE_ENTERED), Behavior::kSaveInsideScope},
+      {CreateEventMetrics(ui::EventType::kMouseEntered),
+       Behavior::kSaveInsideScope},
   };
   EXPECT_NE(events[0].first, nullptr);
   EXPECT_NE(events[1].first, nullptr);
@@ -131,14 +139,14 @@ TEST_F(EventsMetricsManagerTest, EventsMetricsSaved) {
 // different configurations.
 TEST_F(EventsMetricsManagerTest, NestedEventsMetrics) {
   struct {
-    // Type of event to use for the outer scope. `ui::EventType::ET_UNKNOWN` if
+    // Type of event to use for the outer scope. `ui::EventType::kUnknown` if
     // no event should be used.
     ui::EventType outer_event_type;
 
     // Whether to save the outer scope metrics before starting the inner scope.
     bool save_outer_metrics_before_inner;
 
-    // Type of event to use for the inner scope. `ui::EventType::ET_UNKNOWN` if
+    // Type of event to use for the inner scope. `ui::EventType::kUnknown` if
     // no event should be used.
     ui::EventType inner_event_type;
 
@@ -150,45 +158,45 @@ TEST_F(EventsMetricsManagerTest, NestedEventsMetrics) {
   } configs[] = {
       // Config #0.
       {
-          /*outer_event_type=*/ui::EventType::ET_MOUSE_PRESSED,
+          /*outer_event_type=*/ui::EventType::kMousePressed,
           /*save_outer_metrics_before_inner=*/true,
-          /*inner_event_type=*/ui::EventType::ET_MOUSE_RELEASED,
+          /*inner_event_type=*/ui::EventType::kMouseReleased,
           /*save_inner_metrics=*/true,
           /*save_outer_metrics_after_inner=*/false,
       },
 
       // Config #1.
       {
-          /*outer_event_type=*/ui::EventType::ET_MOUSE_PRESSED,
+          /*outer_event_type=*/ui::EventType::kMousePressed,
           /*save_outer_metrics_before_inner=*/false,
-          /*inner_event_type=*/ui::EventType::ET_MOUSE_RELEASED,
+          /*inner_event_type=*/ui::EventType::kMouseReleased,
           /*save_inner_metrics=*/true,
           /*save_outer_metrics_after_inner=*/true,
       },
 
       // Config #2.
       {
-          /*outer_event_type=*/ui::EventType::ET_MOUSE_PRESSED,
+          /*outer_event_type=*/ui::EventType::kMousePressed,
           /*save_outer_metrics_before_inner=*/true,
-          /*inner_event_type=*/ui::EventType::ET_MOUSE_RELEASED,
+          /*inner_event_type=*/ui::EventType::kMouseReleased,
           /*save_inner_metrics=*/true,
           /*save_outer_metrics_after_inner=*/true,
       },
 
       // Config #3.
       {
-          /*outer_event_type=*/ui::EventType::ET_MOUSE_PRESSED,
+          /*outer_event_type=*/ui::EventType::kMousePressed,
           /*save_outer_metrics_before_inner=*/false,
-          /*inner_event_type=*/ui::EventType::ET_UNKNOWN,
+          /*inner_event_type=*/ui::EventType::kUnknown,
           /*save_inner_metrics=*/false,
           /*save_outer_metrics_after_inner=*/true,
       },
 
       // Config #4.
       {
-          /*outer_event_type=*/ui::EventType::ET_UNKNOWN,
+          /*outer_event_type=*/ui::EventType::kUnknown,
           /*save_outer_metrics_before_inner=*/false,
-          /*inner_event_type=*/ui::EventType::ET_MOUSE_PRESSED,
+          /*inner_event_type=*/ui::EventType::kMousePressed,
           /*save_inner_metrics=*/true,
           /*save_outer_metrics_after_inner=*/false,
       },
@@ -200,7 +208,7 @@ TEST_F(EventsMetricsManagerTest, NestedEventsMetrics) {
 
     {  // Start outer scope.
       std::unique_ptr<EventMetrics> outer_metrics;
-      if (config.outer_event_type != ui::EventType::ET_UNKNOWN) {
+      if (config.outer_event_type != ui::EventType::kUnknown) {
         outer_metrics = CreateEventMetrics(config.outer_event_type);
         DCHECK_NE(outer_metrics, nullptr);
         expected_saved_metrics.push_back(outer_metrics.get());
@@ -212,7 +220,7 @@ TEST_F(EventsMetricsManagerTest, NestedEventsMetrics) {
 
       {  // Start inner scope.
         std::unique_ptr<EventMetrics> inner_metrics;
-        if (config.inner_event_type != ui::EventType::ET_UNKNOWN) {
+        if (config.inner_event_type != ui::EventType::kUnknown) {
           inner_metrics = CreateEventMetrics(config.inner_event_type);
           DCHECK_NE(inner_metrics, nullptr);
           expected_saved_metrics.push_back(inner_metrics.get());
