@@ -6,9 +6,11 @@
 
 #include <climits>
 #include <memory>
+#include <utility>
 
 #include "base/rand_util.h"
-#include "cc/trees/ukm_manager.h"
+#include "cc/metrics/ukm_manager.h"
+#include "services/metrics/public/cpp/ukm_recorder.h"
 
 namespace cc {
 
@@ -116,4 +118,15 @@ void LatencyUkmReporter::ReportEventLatencyUkm(
   }
 }
 
+void LatencyUkmReporter::InitializeUkmManager(
+    std::unique_ptr<ukm::UkmRecorder> recorder) {
+  ukm_manager_ = std::make_unique<UkmManager>(std::move(recorder));
+}
+
+void LatencyUkmReporter::SetSourceId(ukm::SourceId source_id) {
+  if (ukm_manager_) {
+    ukm_manager_->SetSourceId(source_id);
+  }
+}
+
 }  // namespace cc
