@@ -8,7 +8,7 @@
 
 #include "base/containers/adapters.h"
 #include "base/functional/callback_helpers.h"
-#include "base/metrics/histogram_macros.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/numerics/safe_conversions.h"
 #include "components/viz/common/quads/compositor_frame_metadata.h"
 
@@ -23,6 +23,21 @@ constexpr int max_worst_windows_size() {
   return size;
 }
 
+void RecordUmaVideoFrameSubmitter(bool is_media_stream,
+                                  base::TimeDelta time_since_decode) {
+  if (is_media_stream) {
+    base::UmaHistogramTimes("Media.VideoFrameSubmitter.Rtc.PresentationDelay",
+                            time_since_decode);
+  } else {
+    base::UmaHistogramTimes("Media.VideoFrameSubmitter.Video.PresentationDelay",
+                            time_since_decode);
+  }
+
+  // TODO(crbug.com/364352012): This will be removed once expired, kept for now
+  // due to internal dependencies.
+  base::UmaHistogramTimes("Media.VideoFrameSubmitter", time_since_decode);
+}
+
 }  // namespace
 
 namespace cc {
@@ -88,7 +103,7 @@ void VideoPlaybackRoughnessReporter::FramePresented(TokenType token,
     if (token == frame.token) {
       if (frame.decode_time.has_value()) {
         auto time_since_decode = timestamp - frame.decode_time.value();
-        UMA_HISTOGRAM_TIMES("Media.VideoFrameSubmitter", time_since_decode);
+        RecordUmaVideoFrameSubmitter(is_media_stream_, time_since_decode);
       }
 
       if (reliable_timestamp)
