@@ -16,6 +16,7 @@
 #include "base/trace_event/typed_macros.h"
 #include "build/build_config.h"
 #include "cc/base/histograms.h"
+#include "cc/mojo_embedder/viz_layer_context.h"
 #include "cc/trees/layer_tree_frame_sink_client.h"
 #include "components/viz/common/features.h"
 #include "components/viz/common/frame_sinks/begin_frame_args.h"
@@ -61,7 +62,7 @@ AsyncLayerTreeFrameSink::UnboundMessagePipes::UnboundMessagePipes(
 AsyncLayerTreeFrameSink::AsyncLayerTreeFrameSink(
     scoped_refptr<viz::RasterContextProvider> context_provider,
     scoped_refptr<RasterContextProviderWrapper> worker_context_provider_wrapper,
-    std::unique_ptr<gpu::ClientSharedImageInterface> shared_image_interface,
+    scoped_refptr<gpu::ClientSharedImageInterface> shared_image_interface,
     InitParams* params)
     : LayerTreeFrameSink(std::move(context_provider),
                          std::move(worker_context_provider_wrapper),
@@ -78,6 +79,7 @@ AsyncLayerTreeFrameSink::AsyncLayerTreeFrameSink(
       pipes_(std::move(params->pipes)),
       wants_animate_only_begin_frames_(params->wants_animate_only_begin_frames),
       auto_needs_begin_frame_(params->auto_needs_begin_frame),
+      wants_begin_frame_acks_(params->wants_begin_frame_acks),
       use_begin_frame_presentation_feedback_(
           params->use_begin_frame_presentation_feedback) {
   DETACH_FROM_THREAD(thread_checker_);
@@ -124,9 +126,12 @@ bool AsyncLayerTreeFrameSink::BindToClient(LayerTreeFrameSinkClient* client) {
     client->SetBeginFrameSource(begin_frame_source_.get());
   }
 
-  if (wants_animate_only_begin_frames_)
+  if (wants_animate_only_begin_frames_) {
     compositor_frame_sink_->SetWantsAnimateOnlyBeginFrames();
-  compositor_frame_sink_ptr_->SetWantsBeginFrameAcks();
+  }
+  if (wants_begin_frame_acks_) {
+    compositor_frame_sink_ptr_->SetWantsBeginFrameAcks();
+  }
   if (auto_needs_begin_frame_) {
     compositor_frame_sink_ptr_->SetAutoNeedsBeginFrame();
   }
@@ -191,6 +196,7 @@ void AsyncLayerTreeFrameSink::SubmitCompositorFrame(
                            StepName::STEP_SUBMIT_COMPOSITOR_FRAME);
         local_surface_id_.WriteIntoTrace(
             ctx.Wrap(data->set_local_surface_id()));
+        data->set_display_trace_id(frame.metadata.begin_frame_ack.trace_id);
       });
   if (local_surface_id_ == last_submitted_local_surface_id_) {
     DCHECK_EQ(last_submitted_device_scale_factor_, frame.device_scale_factor());
@@ -270,10 +276,18 @@ void AsyncLayerTreeFrameSink::DidNotProduceFrame(const viz::BeginFrameAck& ack,
         data->set_step(perfetto::protos::pbzero::ChromeGraphicsPipeline::
                            StepName::STEP_DID_NOT_PRODUCE_FRAME);
         data->set_frame_skipped_reason(to_proto_enum(reason));
+        data->set_display_trace_id(ack.trace_id);
       });
   compositor_frame_sink_ptr_->DidNotProduceFrame(ack);
 }
 
+std::unique_ptr<LayerContext> AsyncLayerTreeFrameSink::CreateLayerContext(
+    LayerTreeHostImpl& host_impl) {
+  CHECK(compositor_frame_sink_ptr_);
+  return std::make_unique<VizLayerContext>(*compositor_frame_sink_ptr_,
+                                           host_impl);
+}
+
 void AsyncLayerTreeFrameSink::DidAllocateSharedBitmap(
     base::ReadOnlySharedMemoryRegion region,
     const viz::SharedBitmapId& id) {
@@ -301,6 +315,24 @@ void AsyncLayerTreeFrameSink::OnBeginFrame(
     std::vector<viz::ReturnedResource> resources) {
   viz::BeginFrameArgs adjusted_args = args;
   adjusted_args.client_arrival_time = base::TimeTicks::Now();
+
+  TRACE_EVENT(
+      "viz,benchmark,graphics.pipeline", "Graphics.Pipeline",
+      perfetto::Flow::Global(adjusted_args.trace_id),
+      [&](perfetto::EventContext ctx) {
+        auto* event = ctx.event<perfetto::protos::pbzero::ChromeTrackEvent>();
+        auto* data = event->set_chrome_graphics_pipeline();
+        data->set_step(needs_begin_frames_
+                           ? perfetto::protos::pbzero::ChromeGraphicsPipeline::
+                                 StepName::STEP_RECEIVE_BEGIN_FRAME
+                           : perfetto::protos::pbzero::ChromeGraphicsPipeline::
+                                 StepName::STEP_RECEIVE_BEGIN_FRAME_DISCARD);
+        if (needs_begin_frames_) {
+          data->set_frame_sequence(adjusted_args.frame_id.sequence_number);
+        }
+        data->set_display_trace_id(adjusted_args.trace_id);
+      });
+
   if (features::IsOnBeginFrameAcksEnabled()) {
     if (frame_ack) {
       DidReceiveCompositorFrameAck(std::move(resources));
@@ -320,15 +352,6 @@ void AsyncLayerTreeFrameSink::OnBeginFrame(
   }
 
   if (!needs_begin_frames_) {
-    TRACE_EVENT(
-        "viz,benchmark,graphics.pipeline", "Graphics.Pipeline",
-        perfetto::Flow::Global(adjusted_args.trace_id),
-        [&](perfetto::EventContext ctx) {
-          auto* event = ctx.event<perfetto::protos::pbzero::ChromeTrackEvent>();
-          auto* data = event->set_chrome_graphics_pipeline();
-          data->set_step(perfetto::protos::pbzero::ChromeGraphicsPipeline::
-                             StepName::STEP_RECEIVE_BEGIN_FRAME_DISCARD);
-        });
     // We had a race with SetNeedsBeginFrame(false) and still need to let the
     // sink know that we didn't use this BeginFrame. OnBeginFrame() can also be
     // called to deliver presentation feedback.
@@ -336,16 +359,6 @@ void AsyncLayerTreeFrameSink::OnBeginFrame(
                        FrameSkippedReason::kNoDamage);
     return;
   }
-  TRACE_EVENT(
-      "viz,benchmark,graphics.pipeline", "Graphics.Pipeline",
-      perfetto::Flow::Global(adjusted_args.trace_id),
-      [&](perfetto::EventContext ctx) {
-        auto* event = ctx.event<perfetto::protos::pbzero::ChromeTrackEvent>();
-        auto* data = event->set_chrome_graphics_pipeline();
-        data->set_step(perfetto::protos::pbzero::ChromeGraphicsPipeline::
-                           StepName::STEP_RECEIVE_BEGIN_FRAME);
-        data->set_frame_sequence(adjusted_args.frame_id.sequence_number);
-      });
 
   if (begin_frame_source_)
     begin_frame_source_->OnBeginFrame(adjusted_args);
