@@ -2,11 +2,18 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/351564777): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #ifndef CC_PAINT_PAINT_OP_H_
 #define CC_PAINT_PAINT_OP_H_
 
 #include <stdint.h>
 
+#include <cmath>
+#include <iosfwd>
 #include <limits>
 #include <memory>
 #include <string>
@@ -15,15 +22,18 @@
 #include <vector>
 
 #include "base/check_op.h"
+#include "base/containers/span.h"
 #include "base/debug/alias.h"
 #include "base/functional/callback.h"
 #include "base/memory/raw_ptr.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/notreached.h"
 #include "cc/base/math_util.h"
+#include "cc/paint/element_id.h"
 #include "cc/paint/node_id.h"
 #include "cc/paint/paint_canvas.h"
 #include "cc/paint/paint_export.h"
+#include "cc/paint/paint_filter.h"
 #include "cc/paint/paint_flags.h"
 #include "cc/paint/paint_record.h"
 #include "cc/paint/refcounted_buffer.h"
@@ -39,6 +49,7 @@
 #include "third_party/skia/include/core/SkRect.h"
 #include "third_party/skia/include/core/SkRefCnt.h"
 #include "third_party/skia/include/core/SkScalar.h"
+#include "ui/gfx/display_color_spaces.h"
 #include "ui/gfx/geometry/rect.h"
 
 class SkImage;
@@ -49,6 +60,7 @@ class Slug;
 
 namespace cc {
 
+class DisplayItemList;
 class PaintOpWriter;
 class PaintOpReader;
 
@@ -80,17 +92,21 @@ enum class PaintOpType : uint8_t {
   kClipRRect,
   kConcat,
   kCustomData,
+  kDrawArc,
+  kDrawArcLite,
   kDrawColor,
   kDrawDRRect,
   kDrawImage,
   kDrawImageRect,
   kDrawIRect,
   kDrawLine,
+  kDrawLineLite,
   kDrawOval,
   kDrawPath,
   kDrawRecord,
   kDrawRect,
   kDrawRRect,
+  kDrawScrollingContents,
   kDrawSkottie,
   kDrawSlug,
   kDrawTextBlob,
@@ -101,6 +117,7 @@ enum class PaintOpType : uint8_t {
   kSave,
   kSaveLayer,
   kSaveLayerAlpha,
+  kSaveLayerFilters,
   kScale,
   kSetMatrix,
   kSetNodeId,
@@ -114,7 +131,6 @@ CC_PAINT_EXPORT std::ostream& operator<<(std::ostream&, PaintOpType);
 class CC_PAINT_EXPORT PaintOp {
  public:
   uint8_t type;
-  uint16_t aligned_size;
 
   using SerializeOptions = PaintOpBuffer::SerializeOptions;
   using DeserializeOptions = PaintOpBuffer::DeserializeOptions;
@@ -130,6 +146,7 @@ class CC_PAINT_EXPORT PaintOp {
   void Raster(SkCanvas* canvas, const PlaybackParams& params) const;
   bool IsDrawOp() const { return g_is_draw_op[type]; }
   bool IsPaintOpWithFlags() const { return g_has_paint_flags[type]; }
+  uint16_t AlignedSize() const { return g_type_to_aligned_size[type]; }
 
   bool EqualsForTesting(const PaintOp& other) const;
 
@@ -195,6 +212,9 @@ class CC_PAINT_EXPORT PaintOp {
   // generated images.
   static bool OpHasDiscardableImages(const PaintOp& op);
 
+  // Gets the maximum content color usage of all images in this PaintOp.
+  gfx::ContentColorUsage GetContentColorUsage() const;
+
   // Returns true if the given op type has PaintFlags.
   static bool TypeHasFlags(PaintOpType type);
 
@@ -209,8 +229,16 @@ class CC_PAINT_EXPORT PaintOp {
   // by the flags for kSaveLayerAlpha to preserving LCD text.
   bool HasEffectsPreventingLCDTextForSaveLayerAlpha() const { return false; }
 
-  bool HasDiscardableImages() const { return false; }
-  bool HasDiscardableImagesFromFlags() const { return false; }
+  // If `content_color_usage` is not null, the functions should update
+  // `*content_color_usage` to be
+  // max(*content_color_usage, max_content_color_usage_of_this_op).
+  bool HasDiscardableImages(gfx::ContentColorUsage* content_color_usage) const {
+    return false;
+  }
+  bool HasDiscardableImagesFromFlags(
+      gfx::ContentColorUsage* content_color_usage) const {
+    return false;
+  }
 
   // Returns the number of bytes used by this op in referenced sub records
   // and display lists.  This doesn't count other objects like paths or blobs.
@@ -254,6 +282,7 @@ class CC_PAINT_EXPORT PaintOp {
       static_cast<size_t>(PaintOpType::kLastPaintOpType) + 1;
   static bool g_is_draw_op[kNumOpTypes];
   static bool g_has_paint_flags[kNumOpTypes];
+  static uint16_t g_type_to_aligned_size[kNumOpTypes];
 
   static constexpr bool kIsDrawOp = false;
   static constexpr bool kHasPaintFlags = false;
@@ -274,7 +303,8 @@ class CC_PAINT_EXPORT PaintOpWithFlags : public PaintOp {
 
   int CountSlowPathsFromFlags() const { return flags.getPathEffect() ? 1 : 0; }
   bool HasNonAAPaint() const { return !flags.isAntiAlias(); }
-  bool HasDiscardableImagesFromFlags() const;
+  bool HasDiscardableImagesFromFlags(
+      gfx::ContentColorUsage* content_color_usage) const;
 
   void RasterWithFlags(SkCanvas* canvas,
                        const PaintFlags* flags,
@@ -483,11 +513,11 @@ class CC_PAINT_EXPORT DrawImageOp final : public PaintOpWithFlags {
                               SkCanvas* canvas,
                               const PlaybackParams& params);
   bool IsValid() const {
-    return flags.IsValid() && SkScalarIsFinite(scale_adjustment.width()) &&
-           SkScalarIsFinite(scale_adjustment.height());
+    return flags.IsValid() && std::isfinite(scale_adjustment.width()) &&
+           std::isfinite(scale_adjustment.height());
   }
   bool EqualsForTesting(const DrawImageOp& other) const;
-  bool HasDiscardableImages() const;
+  bool HasDiscardableImages(gfx::ContentColorUsage* content_color_usage) const;
   bool HasNonAAPaint() const { return false; }
   HAS_SERIALIZATION_FUNCTIONS();
 
@@ -525,11 +555,11 @@ class CC_PAINT_EXPORT DrawImageRectOp final : public PaintOpWithFlags {
                               const PlaybackParams& params);
   bool IsValid() const {
     return flags.IsValid() && src.isFinite() && dst.isFinite() &&
-           SkScalarIsFinite(scale_adjustment.width()) &&
-           SkScalarIsFinite(scale_adjustment.height());
+           std::isfinite(scale_adjustment.width()) &&
+           std::isfinite(scale_adjustment.height());
   }
   bool EqualsForTesting(const DrawImageRectOp& other) const;
-  bool HasDiscardableImages() const;
+  bool HasDiscardableImages(gfx::ContentColorUsage* content_color_usage) const;
   HAS_SERIALIZATION_FUNCTIONS();
 
   PaintImage image;
@@ -606,6 +636,109 @@ class CC_PAINT_EXPORT DrawLineOp final : public PaintOpWithFlags {
   DrawLineOp() : PaintOpWithFlags(kType) {}
 };
 
+// TODO(crbug.com/340122178): figure out a better way to unify types.
+class CC_PAINT_EXPORT DrawLineLiteOp final : public PaintOp {
+ public:
+  static constexpr PaintOpType kType = PaintOpType::kDrawLineLite;
+  static constexpr bool kIsDrawOp = true;
+  DrawLineLiteOp(SkScalar x0,
+                 SkScalar y0,
+                 SkScalar x1,
+                 SkScalar y1,
+                 const CorePaintFlags& core_paint_flags)
+      : PaintOp(kType),
+        x0(x0),
+        y0(y0),
+        x1(x1),
+        y1(y1),
+        core_paint_flags(core_paint_flags) {}
+  static void Raster(const DrawLineLiteOp* op,
+                     SkCanvas* canvas,
+                     const PlaybackParams& params);
+  bool IsValid() const { return core_paint_flags.IsValid(); }
+  bool EqualsForTesting(const DrawLineLiteOp& other) const;
+  HAS_SERIALIZATION_FUNCTIONS();
+
+  int CountSlowPaths() const { return 0; }
+
+  SkScalar x0;
+  SkScalar y0;
+  SkScalar x1;
+  SkScalar y1;
+  CorePaintFlags core_paint_flags;
+
+ private:
+  DrawLineLiteOp() : PaintOp(kType) {}
+};
+
+// TODO(crbug.com/340122178): figure out a better way to unify types.
+class CC_PAINT_EXPORT DrawArcLiteOp final : public PaintOp {
+ public:
+  static constexpr PaintOpType kType = PaintOpType::kDrawArcLite;
+  static constexpr bool kIsDrawOp = true;
+  DrawArcLiteOp(const SkRect& oval,
+                SkScalar start_angle_degrees,
+                SkScalar sweep_angle_degrees,
+                const CorePaintFlags& core_paint_flags)
+      : PaintOp(kType),
+        oval(oval),
+        start_angle_degrees(start_angle_degrees),
+        sweep_angle_degrees(sweep_angle_degrees),
+        core_paint_flags(core_paint_flags) {}
+  static void Raster(const DrawArcLiteOp* op,
+                     SkCanvas* canvas,
+                     const PlaybackParams& params);
+  bool IsValid() const {
+    return core_paint_flags.IsValid() && oval.isFinite() &&
+           std::isfinite(start_angle_degrees) &&
+           std::isfinite(sweep_angle_degrees);
+  }
+  bool EqualsForTesting(const DrawArcLiteOp& other) const;
+  HAS_SERIALIZATION_FUNCTIONS();
+
+  SkRect oval;
+  SkScalar start_angle_degrees;
+  SkScalar sweep_angle_degrees;
+  CorePaintFlags core_paint_flags;
+
+ private:
+  DrawArcLiteOp() : PaintOp(kType) {}
+};
+
+class CC_PAINT_EXPORT DrawArcOp final : public PaintOpWithFlags {
+ public:
+  static constexpr PaintOpType kType = PaintOpType::kDrawArc;
+  static constexpr bool kIsDrawOp = true;
+  DrawArcOp(const SkRect& oval,
+            SkScalar start_angle_degrees,
+            SkScalar sweep_angle_degrees,
+            const PaintFlags& flags)
+      : PaintOpWithFlags(kType, flags),
+        oval(oval),
+        start_angle_degrees(start_angle_degrees),
+        sweep_angle_degrees(sweep_angle_degrees) {}
+  static void RasterWithFlags(const DrawArcOp* op,
+                              const PaintFlags* flags,
+                              SkCanvas* canvas,
+                              const PlaybackParams& params);
+  // Actual implementation for rastering.
+  void RasterWithFlagsImpl(const PaintFlags* flags, SkCanvas* canvas) const;
+  bool IsValid() const {
+    return flags.IsValid() && oval.isFinite() &&
+           std::isfinite(start_angle_degrees) &&
+           std::isfinite(sweep_angle_degrees);
+  }
+  bool EqualsForTesting(const DrawArcOp& other) const;
+  HAS_SERIALIZATION_FUNCTIONS();
+
+  SkRect oval;
+  SkScalar start_angle_degrees;
+  SkScalar sweep_angle_degrees;
+
+ private:
+  DrawArcOp() : PaintOpWithFlags(kType) {}
+};
+
 class CC_PAINT_EXPORT DrawOvalOp final : public PaintOpWithFlags {
  public:
   static constexpr PaintOpType kType = PaintOpType::kDrawOval;
@@ -616,10 +749,7 @@ class CC_PAINT_EXPORT DrawOvalOp final : public PaintOpWithFlags {
                               const PaintFlags* flags,
                               SkCanvas* canvas,
                               const PlaybackParams& params);
-  bool IsValid() const {
-    // Reproduce SkRRect::isValid without converting.
-    return flags.IsValid() && oval.isFinite() && oval.isSorted();
-  }
+  bool IsValid() const { return flags.IsValid() && oval.isFinite(); }
   bool EqualsForTesting(const DrawOvalOp& other) const;
   HAS_SERIALIZATION_FUNCTIONS();
 
@@ -666,7 +796,7 @@ class CC_PAINT_EXPORT DrawRecordOp final : public PaintOp {
  public:
   static constexpr PaintOpType kType = PaintOpType::kDrawRecord;
   static constexpr bool kIsDrawOp = true;
-  explicit DrawRecordOp(PaintRecord record);
+  explicit DrawRecordOp(PaintRecord record, bool local_ctm = true);
   ~DrawRecordOp();
   static void Raster(const DrawRecordOp* op,
                      SkCanvas* canvas,
@@ -675,7 +805,7 @@ class CC_PAINT_EXPORT DrawRecordOp final : public PaintOp {
   bool EqualsForTesting(const DrawRecordOp& other) const;
   size_t AdditionalBytesUsed() const;
   size_t AdditionalOpCount() const;
-  bool HasDiscardableImages() const;
+  bool HasDiscardableImages(gfx::ContentColorUsage* content_color_usage) const;
   int CountSlowPaths() const;
   bool HasNonAAPaint() const;
   bool HasDrawTextOps() const;
@@ -685,6 +815,16 @@ class CC_PAINT_EXPORT DrawRecordOp final : public PaintOp {
   HAS_SERIALIZATION_FUNCTIONS();
 
   PaintRecord record;
+
+  // If `local_ctm` is `true`, the transform operations in `record` are local to
+  // that recording: any transform changes done by `record` are undone before
+  // this `DrawRecordOp` completes and `SetMatrixOp` acts relatively to the
+  // transform set on the destination record (to "anchor" `SetMatrixOp` and
+  // other multiplicative matrix transforms on the same base transform). If
+  // `local_ctm` is `false`, matrix changes done by `record` act as if part of
+  // the parent record: transform changes are preserved after this
+  // `DrawRecordOp` is rasterized and `SetMatrixOp` ignores parent transforms.
+  bool local_ctm = true;
 };
 
 class CC_PAINT_EXPORT DrawRectOp final : public PaintOpWithFlags {
@@ -727,6 +867,39 @@ class CC_PAINT_EXPORT DrawRRectOp final : public PaintOpWithFlags {
   DrawRRectOp() : PaintOpWithFlags(kType) {}
 };
 
+// This is used to draw non-composited scrolling contents. The display item
+// list should contain painted results beyond the current scroll port like
+// composited scrolling contents. During rasterization or serialization, the
+// current clip of the canvas and the current scroll offset and will be applied
+// to the display item list. This PaintOp doesn't apply the overflow clip of
+// the scroller, but the client should emit ClipRectOp.
+class CC_PAINT_EXPORT DrawScrollingContentsOp final : public PaintOp {
+ public:
+  static constexpr PaintOpType kType = PaintOpType::kDrawScrollingContents;
+  static constexpr bool kIsDrawOp = true;
+  DrawScrollingContentsOp(ElementId scroll_element_id,
+                          scoped_refptr<DisplayItemList> display_item_list);
+  ~DrawScrollingContentsOp();
+  static void Raster(const DrawScrollingContentsOp* op,
+                     SkCanvas* canvas,
+                     const PlaybackParams& params);
+  bool IsValid() const { return scroll_element_id && display_item_list; }
+  bool EqualsForTesting(const DrawScrollingContentsOp& other) const;
+  size_t AdditionalBytesUsed() const;
+  size_t AdditionalOpCount() const;
+  bool HasDiscardableImages(gfx::ContentColorUsage* content_color_usage) const;
+  int CountSlowPaths() const;
+  bool HasNonAAPaint() const;
+  bool HasDrawTextOps() const;
+  bool HasSaveLayerOps() const;
+  bool HasSaveLayerAlphaOps() const;
+  bool HasEffectsPreventingLCDTextForSaveLayerAlpha() const;
+  HAS_SERIALIZATION_FUNCTIONS();
+
+  ElementId scroll_element_id;
+  scoped_refptr<DisplayItemList> display_item_list;
+};
+
 class CC_PAINT_EXPORT DrawVerticesOp final : public PaintOpWithFlags {
  public:
   static constexpr PaintOpType kType = PaintOpType::kDrawVertices;
@@ -780,7 +953,7 @@ class CC_PAINT_EXPORT DrawSkottieOp final : public PaintOp {
            t <= 1.f;
   }
   bool EqualsForTesting(const DrawSkottieOp& other) const;
-  bool HasDiscardableImages() const;
+  bool HasDiscardableImages(gfx::ContentColorUsage* content_color_usage) const;
   HAS_SERIALIZATION_FUNCTIONS();
 
   scoped_refptr<SkottieWrapper> skottie;
@@ -976,6 +1149,29 @@ class CC_PAINT_EXPORT SaveLayerAlphaOp final : public PaintOp {
   SaveLayerAlphaOp() : PaintOp(kType) {}
 };
 
+class CC_PAINT_EXPORT SaveLayerFiltersOp final : public PaintOpWithFlags {
+ public:
+  static constexpr PaintOpType kType = PaintOpType::kSaveLayerFilters;
+  explicit SaveLayerFiltersOp(base::span<sk_sp<PaintFilter>> filters,
+                              const PaintFlags& flags);
+  ~SaveLayerFiltersOp();
+  static void RasterWithFlags(const SaveLayerFiltersOp* op,
+                              const PaintFlags* flags,
+                              SkCanvas* canvas,
+                              const PlaybackParams& params);
+  bool IsValid() const {
+    return flags.IsValid() && (!flags.getImageFilter() || filters.empty());
+  }
+  bool EqualsForTesting(const SaveLayerFiltersOp& other) const;
+  bool HasSaveLayerOps() const { return true; }
+  HAS_SERIALIZATION_FUNCTIONS();
+
+  std::vector<sk_sp<PaintFilter>> filters;
+
+ private:
+  SaveLayerFiltersOp();
+};
+
 class CC_PAINT_EXPORT ScaleOp final : public PaintOp {
  public:
   static constexpr PaintOpType kType = PaintOpType::kScale;
@@ -1066,6 +1262,13 @@ using LargestPaintOp =
 inline constexpr size_t kLargestPaintOpAlignedSize =
     PaintOpBuffer::ComputeOpAlignedSize<LargestPaintOp>();
 
+// This is declared here for use in gtest-based unit tests but is defined in
+// the //cc:test_support target. Depend on that to use this in your unit test.
+// This should not be used in production code.
+void PrintTo(const PaintOp& rect, std::ostream* os);
+
+CC_PAINT_EXPORT bool AreLiteOpsEnabled();
+
 }  // namespace cc
 
 #endif  // CC_PAINT_PAINT_OP_H_
