@@ -3,17 +3,46 @@
 // found in the LICENSE file.
 
 #include "cc/test/paint_op_helper.h"
+
+#include <array>
+
 #include "cc/paint/paint_canvas.h"
+#include "cc/paint/paint_filter.h"
+#include "cc/paint/paint_flags.h"
+#include "cc/paint/paint_op.h"
 #include "cc/paint/paint_op_buffer.h"
 #include "cc/test/skia_common.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/skia/include/core/SkRefCnt.h"
 #include "third_party/skia/include/core/SkTextBlob.h"
+#include "third_party/skia/include/core/SkTileMode.h"
 #include "third_party/skia/include/effects/SkLumaColorFilter.h"
 #include "third_party/skia/include/private/chromium/Slug.h"
 
 namespace cc {
 namespace {
 
+TEST(PaintOpHelper, PaintRecordEmptyToString) {
+  PaintOpBuffer buffer;
+  EXPECT_EQ(PaintOpHelper::ToString(buffer.ReleaseAsRecord()),
+            "<PaintRecord>[]");
+}
+
+TEST(PaintOpHelper, PaintRecordOneOpToString) {
+  PaintOpBuffer buffer;
+  buffer.push<SaveOp>();
+  EXPECT_EQ(PaintOpHelper::ToString(buffer.ReleaseAsRecord()),
+            "<PaintRecord>[SaveOp()]");
+}
+
+TEST(PaintOpHelper, PaintRecordMultipleOpsToString) {
+  PaintOpBuffer buffer;
+  buffer.push<SaveOp>();
+  buffer.push<RotateOp>(360.0f);
+  EXPECT_EQ(PaintOpHelper::ToString(buffer.ReleaseAsRecord()),
+            "<PaintRecord>[SaveOp(), RotateOp(degrees=360.000)]");
+}
+
 TEST(PaintOpHelper, AnnotateToString) {
   AnnotateOp op(PaintCanvas::AnnotationType::kUrl, SkRect::MakeXYWH(1, 2, 3, 4),
                 nullptr);
@@ -79,7 +108,7 @@ TEST(PaintOpHelper, DrawDRRectToString) {
       "filterQuality=kNone_SkFilterQuality, "
       "strokeWidth=0.000, strokeMiter=4.000, strokeCap=kButt_Cap, "
       "strokeJoin=kMiter_Join, colorFilter=(nil), "
-      "maskFilter=(nil), shader=(nil), hasShader=false, shaderIsOpaque=false, "
+      "shader=(nil), hasShader=false, shaderIsOpaque=false, "
       "pathEffect=(nil), imageFilter=(nil), drawLooper=(nil), "
       "supportsFoldingAlpha=true, isValid=true, hasDiscardableImages=false])");
 }
@@ -93,7 +122,7 @@ TEST(PaintOpHelper, DrawImageToString) {
       "flags=[color=rgba(0, 0, 0, 255), blendMode=kSrcOver, isAntiAlias=false, "
       "isDither=false, filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), "
+      "colorFilter=(nil), shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
       "isValid=true, hasDiscardableImages=false])");
@@ -111,7 +140,7 @@ TEST(PaintOpHelper, DrawImageRectToString) {
       "flags=[color=rgba(0, 0, 0, 255), blendMode=kSrcOver, isAntiAlias=false, "
       "isDither=false, filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), "
+      "colorFilter=(nil), shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
       "isValid=true, hasDiscardableImages=false])");
@@ -125,7 +154,7 @@ TEST(PaintOpHelper, DrawIRectToString) {
             "blendMode=kSrcOver, isAntiAlias=false, isDither=false, "
             "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
             "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-            "colorFilter=(nil), maskFilter=(nil), "
+            "colorFilter=(nil), "
             "shader=(nil), hasShader=false, shaderIsOpaque=false, "
             "pathEffect=(nil), imageFilter=(nil), drawLooper=(nil), "
             "supportsFoldingAlpha=true, isValid=true, "
@@ -141,7 +170,7 @@ TEST(PaintOpHelper, DrawLineToString) {
       "0, 0, 255), blendMode=kSrcOver, isAntiAlias=false, isDither=false, "
       "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), "
+      "colorFilter=(nil), shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
       "isValid=true, hasDiscardableImages=false])");
@@ -156,7 +185,7 @@ TEST(PaintOpHelper, DrawOvalToString) {
       "0, 0, 255), blendMode=kSrcOver, isAntiAlias=false, isDither=false, "
       "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), "
+      "colorFilter=(nil), shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
       "isValid=true, hasDiscardableImages=false])");
@@ -171,7 +200,7 @@ TEST(PaintOpHelper, DrawPathToString) {
             "blendMode=kSrcOver, isAntiAlias=false, isDither=false, "
             "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
             "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-            "colorFilter=(nil), maskFilter=(nil), "
+            "colorFilter=(nil), "
             "shader=(nil), hasShader=false, shaderIsOpaque=false, "
             "pathEffect=(nil), imageFilter=(nil), drawLooper=(nil), "
             "supportsFoldingAlpha=true, isValid=true, "
@@ -181,7 +210,7 @@ TEST(PaintOpHelper, DrawPathToString) {
 TEST(PaintOpHelper, DrawRecordToString) {
   DrawRecordOp op((PaintRecord()));
   std::string str = PaintOpHelper::ToString(op);
-  EXPECT_EQ(str, "DrawRecordOp(record=(empty))");
+  EXPECT_EQ(str, "DrawRecordOp(record=<PaintRecord>[])");
 }
 
 TEST(PaintOpHelper, DrawRectToString) {
@@ -194,7 +223,7 @@ TEST(PaintOpHelper, DrawRectToString) {
       "isDither=false, filterQuality=kNone_SkFilterQuality, "
       "strokeWidth=0.000, strokeMiter=4.000, strokeCap=kButt_Cap, "
       "strokeJoin=kMiter_Join, colorFilter=(nil), "
-      "maskFilter=(nil), shader=(nil), hasShader=false, shaderIsOpaque=false, "
+      "shader=(nil), hasShader=false, shaderIsOpaque=false, "
       "pathEffect=(nil), imageFilter=(nil), drawLooper=(nil), "
       "supportsFoldingAlpha=true, isValid=true, hasDiscardableImages=false])");
 }
@@ -209,7 +238,7 @@ TEST(PaintOpHelper, DrawRRectToString) {
       "flags=[color=rgba(0, 0, 0, 255), blendMode=kSrcOver, isAntiAlias=false, "
       "isDither=false, filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), "
+      "colorFilter=(nil), shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
       "isValid=true, hasDiscardableImages=false])");
@@ -223,7 +252,7 @@ TEST(PaintOpHelper, DrawSlugToString) {
       "DrawSlugOp(flags=[color=rgba(0, 0, 0, 255), blendMode=kSrcOver, "
       "isAntiAlias=false, isDither=false, filterQuality=kNone_SkFilterQuality, "
       "strokeWidth=0.000, strokeMiter=4.000, strokeCap=kButt_Cap, "
-      "strokeJoin=kMiter_Join, colorFilter=(nil), maskFilter=(nil), "
+      "strokeJoin=kMiter_Join, colorFilter=(nil), "
       "shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
@@ -239,7 +268,7 @@ TEST(PaintOpHelper, DrawTextBlobToString) {
       "0, 0, 255), blendMode=kSrcOver, isAntiAlias=false, isDither=false, "
       "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), "
+      "colorFilter=(nil), shader=(nil), "
       "hasShader=false, shaderIsOpaque=false, pathEffect=(nil), "
       "imageFilter=(nil), drawLooper=(nil), supportsFoldingAlpha=true, "
       "isValid=true, hasDiscardableImages=false])");
@@ -260,7 +289,7 @@ TEST(PaintOpHelper, DrawVerticesToString) {
       "isAntiAlias=false, isDither=false, "
       "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
       "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
-      "colorFilter=(nil), maskFilter=(nil), shader=(nil), hasShader=false, "
+      "colorFilter=(nil), shader=(nil), hasShader=false, "
       "shaderIsOpaque=false, pathEffect=(nil), imageFilter=(nil), "
       "drawLooper=(nil), supportsFoldingAlpha=true, isValid=true, "
       "hasDiscardableImages=false])");
@@ -300,7 +329,7 @@ TEST(PaintOpHelper, SaveLayerToString) {
       "filterQuality=kNone_SkFilterQuality, "
       "strokeWidth=0.000, strokeMiter=4.000, strokeCap=kButt_Cap, "
       "strokeJoin=kMiter_Join, colorFilter=(nil), "
-      "maskFilter=(nil), shader=(nil), hasShader=false, shaderIsOpaque=false, "
+      "shader=(nil), hasShader=false, shaderIsOpaque=false, "
       "pathEffect=(nil), imageFilter=(nil), drawLooper=(nil), "
       "supportsFoldingAlpha=true, isValid=true, hasDiscardableImages=false])");
 }
@@ -319,7 +348,7 @@ TEST(PaintOpHelper, SaveLayerWithFilterToString) {
       "filterQuality=kNone_SkFilterQuality, "
       "strokeWidth=0.000, strokeMiter=4.000, strokeCap=kButt_Cap, "
       "strokeJoin=kMiter_Join, colorFilter=(nil), "
-      "maskFilter=(nil), shader=(nil), hasShader=false, shaderIsOpaque=false, "
+      "shader=(nil), hasShader=false, shaderIsOpaque=false, "
       "pathEffect=(nil), imageFilter=DropShadowPaintFilter(dx=0.000, dy=0.000, "
       "sigma_x=0.000, sigma_y=0.000, color=rgba(0.000000, 0.000000, 0.000000, "
       "0.000000), shadow_mode=kDrawShadowAndForeground, input=(nil), "
@@ -334,6 +363,27 @@ TEST(PaintOpHelper, SaveLayerAlphaToString) {
             "SaveLayerAlphaOp(bounds=[1.000,2.000 3.000x4.000], alpha=1.000)");
 }
 
+TEST(PaintOpHelper, SaveLayerFiltersToString) {
+  PaintFlags flags;
+  SaveLayerFiltersOp op(
+      std::array<sk_sp<PaintFilter>, 2>{
+          sk_make_sp<BlurPaintFilter>(1.0f, 2.0f, SkTileMode::kRepeat,
+                                      /*input=*/nullptr),
+          nullptr},
+      flags);
+  EXPECT_EQ(PaintOpHelper::ToString(op),
+            "SaveLayerFiltersOp(flags=[color=rgba(0, 0, 0, 255), "
+            "blendMode=kSrcOver, isAntiAlias=false, isDither=false, "
+            "filterQuality=kNone_SkFilterQuality, strokeWidth=0.000, "
+            "strokeMiter=4.000, strokeCap=kButt_Cap, strokeJoin=kMiter_Join, "
+            "colorFilter=(nil), shader=(nil), hasShader=false, "
+            "shaderIsOpaque=false, pathEffect=(nil), imageFilter=(nil), "
+            "drawLooper=(nil), supportsFoldingAlpha=true, isValid=true, "
+            "hasDiscardableImages=false], "
+            "filters={BlurPaintFilter(sigma_x=1.000, sigma_y=2.000, "
+            "tile_mode=kRepeat, input=(nil), crop_rect=(nil)), (nil)})");
+}
+
 TEST(PaintOpHelper, ScaleToString) {
   ScaleOp op(12, 13.9f);
   std::string str = PaintOpHelper::ToString(op);
@@ -485,7 +535,7 @@ TEST(PaintOpHelperFilters, RecordPaintFilter) {
                            /*raster_scale=*/{0.5f, 0.8f},
                            RecordPaintFilter::ScalingBehavior::kFixedScale);
   EXPECT_EQ(PaintOpHelper::ToString(filter),
-            "RecordPaintFilter(record=<paint record>, "
+            "RecordPaintFilter(record=<PaintRecord>[SaveOp()], "
             "record_bounds=[0.000,0.000 100.000x100.000], "
             "raster_scale=[0.5x0.8], scaling_behavior=kFixedScale, "
             "crop_rect=(nil))");
