@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/351564777): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include <memory>
 #include <utility>
 #include <vector>
@@ -24,7 +29,7 @@ TEST(SkottieTransferCacheEntryTest, SerializationDeserialization) {
           kLottieDataWithoutAssets1.length());
 
   scoped_refptr<SkottieWrapper> skottie =
-      SkottieWrapper::CreateSerializable(std::move(a_data));
+      SkottieWrapper::UnsafeCreateSerializable(std::move(a_data));
 
   // Serialize
   auto client_entry(std::make_unique<ClientSkottieTransferCacheEntry>(skottie));
