@@ -7,13 +7,13 @@
 
 #include <stdint.h>
 
+#include "base/tracing/protos/chrome_track_event.pbzero.h"
 #include "cc/cc_export.h"
 #include "cc/scheduler/commit_earlyout_reason.h"
 #include "cc/scheduler/draw_result.h"
 #include "cc/scheduler/scheduler_settings.h"
 #include "cc/tiles/tile_priority.h"
 #include "components/viz/common/frame_sinks/begin_frame_args.h"
-#include "third_party/perfetto/protos/perfetto/trace/track_event/chrome_compositor_scheduler_state.pbzero.h"
 
 namespace cc {
 
@@ -49,8 +49,8 @@ class CC_EXPORT SchedulerStateMachine {
     WAITING_FOR_FIRST_COMMIT,
     WAITING_FOR_FIRST_ACTIVATION,
   };
-  static perfetto::protos::pbzero::ChromeCompositorStateMachine::MajorState::
-      LayerTreeFrameSinkState
+  static perfetto::protos::pbzero::ChromeCompositorStateMachineV2::
+      MajorStateV2::LayerTreeFrameSinkState
       LayerTreeFrameSinkStateToProtozeroEnum(LayerTreeFrameSinkState state);
 
   // Note: BeginImplFrameState does not cycle through these states in a fixed
@@ -60,35 +60,32 @@ class CC_EXPORT SchedulerStateMachine {
     INSIDE_BEGIN_FRAME,
     INSIDE_DEADLINE,
   };
-  static perfetto::protos::pbzero::ChromeCompositorStateMachine::MajorState::
-      BeginImplFrameState
+  static perfetto::protos::pbzero::ChromeCompositorStateMachineV2::
+      MajorStateV2::BeginImplFrameState
       BeginImplFrameStateToProtozeroEnum(BeginImplFrameState state);
 
-  // These values are persisted to logs. Entries should not be renumbered and
-  // numeric values should never be reused.
-  // TODO(weiliangc): The histogram is used to understanding what type of
-  // deadline mode do we encounter in real world and is set to expire after
-  // 2022. The Enum can be changed after the histogram is removed.
   // The scheduler uses a deadline to wait for main thread updates before
   // submitting a compositor frame. BeginImplFrameDeadlineMode specifies when
   // the deadline should run.
   enum class BeginImplFrameDeadlineMode {
-    NONE = 0,  // No deadline should be scheduled e.g. for synchronous
-               // compositor.
+    NONE = 0,       // No deadline should be scheduled e.g. for synchronous
+                    // compositor.
     IMMEDIATE = 1,  // Deadline should be scheduled to run immediately.
-    REGULAR = 2,    // Deadline should be scheduled to run at the deadline
-                    // provided by in the BeginFrameArgs.
-    LATE = 3,       // Deadline should be scheduled run when the next frame is
-                    // expected to arrive.
-    BLOCKED = 4,    // Deadline should be blocked indefinitely until the next
-                    // frame arrives.
+    WAIT_FOR_SCROLL = 2,  // Deadline should be delayed as we are expecting
+                          // input for a scroll.
+    REGULAR = 3,          // Deadline should be scheduled to run at the deadline
+                          // provided by in the BeginFrameArgs.
+    LATE = 4,     // Deadline should be scheduled run when the next frame is
+                  // expected to arrive.
+    BLOCKED = 5,  // Deadline should be blocked indefinitely until the next
+                  // frame arrives.
     kMaxValue = BLOCKED,
   };
   // TODO(nuskos): Update Scheduler::ScheduleBeginImplFrameDeadline event to
   // used typed macros so we can remove this ToString function.
   static const char* BeginImplFrameDeadlineModeToString(
       BeginImplFrameDeadlineMode mode);
-  static perfetto::protos::pbzero::ChromeCompositorSchedulerState::
+  static perfetto::protos::pbzero::ChromeCompositorSchedulerStateV2::
       BeginImplFrameDeadlineMode
       BeginImplFrameDeadlineModeToProtozeroEnum(
           BeginImplFrameDeadlineMode mode);
@@ -99,8 +96,8 @@ class CC_EXPORT SchedulerStateMachine {
     READY_TO_COMMIT,  // A previously issued BeginMainFrame has been processed,
                       // and is ready to commit.
   };
-  static perfetto::protos::pbzero::ChromeCompositorStateMachine::MajorState::
-      BeginMainFrameState
+  static perfetto::protos::pbzero::ChromeCompositorStateMachineV2::
+      MajorStateV2::BeginMainFrameState
       BeginMainFrameStateToProtozeroEnum(BeginMainFrameState state);
 
   // When a redraw is forced, it goes through a complete commit -> activation ->
@@ -111,8 +108,8 @@ class CC_EXPORT SchedulerStateMachine {
     WAITING_FOR_ACTIVATION,
     WAITING_FOR_DRAW,
   };
-  static perfetto::protos::pbzero::ChromeCompositorStateMachine::MajorState::
-      ForcedRedrawOnTimeoutState
+  static perfetto::protos::pbzero::ChromeCompositorStateMachineV2::
+      MajorStateV2::ForcedRedrawOnTimeoutState
       ForcedRedrawOnTimeoutStateToProtozeroEnum(
           ForcedRedrawOnTimeoutState state);
 
@@ -142,17 +139,18 @@ class CC_EXPORT SchedulerStateMachine {
     DRAW_IF_POSSIBLE,
     DRAW_FORCED,
     DRAW_ABORT,
+    UPDATE_DISPLAY_TREE,
     BEGIN_LAYER_TREE_FRAME_SINK_CREATION,
     PREPARE_TILES,
     INVALIDATE_LAYER_TREE_FRAME_SINK,
     NOTIFY_BEGIN_MAIN_FRAME_NOT_EXPECTED_UNTIL,
     NOTIFY_BEGIN_MAIN_FRAME_NOT_EXPECTED_SOON,
   };
-  static perfetto::protos::pbzero::ChromeCompositorSchedulerAction
+  static perfetto::protos::pbzero::ChromeCompositorSchedulerActionV2
   ActionToProtozeroEnum(Action action);
 
   void AsProtozeroInto(
-      perfetto::protos::pbzero::ChromeCompositorStateMachine* state) const;
+      perfetto::protos::pbzero::ChromeCompositorStateMachineV2* state) const;
 
   Action NextAction() const;
   void WillSendBeginMainFrame();
@@ -163,6 +161,7 @@ class CC_EXPORT SchedulerStateMachine {
   void DidPostCommit();
   void WillActivate();
   void WillDraw();
+  void WillUpdateDisplayTree();
   void WillBeginLayerTreeFrameSinkCreation();
   void WillPrepareTiles();
   void WillInvalidateLayerTreeFrameSink();
@@ -213,6 +212,13 @@ class CC_EXPORT SchedulerStateMachine {
   void SetVisible(bool visible);
   bool visible() const { return visible_; }
 
+  // Indicates that warming up is requested to create a new LayerTreeFrameSink
+  // even if the LayerTreeHost is invisible. This is an experimental function
+  // and only used if `kWarmUpCompositor` is enabled. Currently, this will be
+  // requested only from prerendered pages. Please see crbug.com/40240492 for
+  // more details.
+  void SetShouldWarmUp();
+
   void SetBeginFrameSourcePaused(bool paused);
   bool begin_frame_source_paused() const { return begin_frame_source_paused_; }
 
@@ -224,6 +230,11 @@ class CC_EXPORT SchedulerStateMachine {
   void SetNeedsRedraw();
   bool needs_redraw() const { return needs_redraw_; }
 
+  // Indicates that the display tree needs an update, implying that the active
+  // tree has changed in some meaningful way since the last update.
+  void SetNeedsUpdateDisplayTree();
+  bool needs_update_display_tree() const { return needs_update_display_tree_; }
+
   bool did_invalidate_layer_tree_frame_sink() const {
     return did_invalidate_layer_tree_frame_sink_;
   }
@@ -363,11 +374,21 @@ class CC_EXPORT SchedulerStateMachine {
 
   bool resourceless_draw() const { return resourceless_draw_; }
 
+  void set_is_scrolling(bool is_scrolling) { is_scrolling_ = is_scrolling; }
+  void set_waiting_for_scroll_event(bool waiting_for_scroll_event) {
+    waiting_for_scroll_event_ = waiting_for_scroll_event;
+  }
+
  protected:
   bool BeginFrameRequiredForAction() const;
   bool BeginFrameNeededForVideo() const;
   bool ProactiveBeginFrameWanted() const;
 
+  // Indicates if we should post a deadline for drawing, and if we should delay
+  // sending BeginMainFrame. This is true when we are expecting a scroll event
+  // to arrive, are prioritizing smoothness, and have begun frame production.
+  bool ShouldWaitForScrollEvent() const;
+
   // Indicates if we should post the deadline to draw immediately. This is true
   // when we aren't expecting a commit or activation, or we're prioritizing
   // active tree draw (see ImplLatencyTakesPriority()).
@@ -386,6 +407,7 @@ class CC_EXPORT SchedulerStateMachine {
 
   bool ShouldBeginLayerTreeFrameSinkCreation() const;
   bool ShouldDraw() const;
+  bool ShouldUpdateDisplayTree() const;
   bool ShouldActivateSyncTree() const;
   bool ShouldSendBeginMainFrame() const;
   bool ShouldCommit() const;
@@ -434,6 +456,7 @@ class CC_EXPORT SchedulerStateMachine {
   // These are used to ensure that an action only happens once per frame,
   // deadline, etc.
   bool did_draw_ = false;
+  bool did_update_display_tree_ = false;
   bool did_send_begin_main_frame_for_current_frame_ = true;
 
   // Initialized to true to prevent begin main frame before begin frames have
@@ -454,7 +477,9 @@ class CC_EXPORT SchedulerStateMachine {
   bool needs_begin_main_frame_ = false;
   bool needs_one_begin_impl_frame_ = false;
   bool needs_post_commit_ = false;
+  bool needs_update_display_tree_ = false;
   bool visible_ = false;
+  bool should_warm_up_ = false;
   bool begin_frame_source_paused_ = false;
   bool resourceless_draw_ = false;
   bool can_draw_ = false;
@@ -504,6 +529,15 @@ class CC_EXPORT SchedulerStateMachine {
   bool draw_aborted_for_paused_begin_frame_ = false;
 
   unsigned consecutive_cant_draw_count_ = 0u;
+
+  // When true we will prioritize BeginImplFrameDeadlineMode::SCROLL if
+  // `SchedulerSettings.scroll_deadline_mode_enabled_` is enabled.
+  bool is_scrolling_ = false;
+  // Only true when `is_scrolling_` is also true. While true there was no
+  // available scroll events at the start of OnBeginImplFrame and we were
+  // expecting some. Once `is_scrolling_` is false, we are no longer expecting
+  // scroll events to arrive.
+  bool waiting_for_scroll_event_ = false;
 };
 
 }  // namespace cc
