@@ -8,8 +8,6 @@
 
 #include "base/memory/ptr_util.h"
 #include "cc/slim/delayed_scheduler.h"
-#include "cc/slim/features.h"
-#include "cc/slim/frame_sink_cc_wrapper.h"
 #include "cc/slim/frame_sink_impl.h"
 
 namespace cc::slim {
@@ -25,13 +23,6 @@ std::unique_ptr<FrameSink> FrameSink::Create(
     // Parameters below only used when wrapping cc.
     gpu::GpuMemoryBufferManager* gpu_memory_buffer_manager,
     base::PlatformThreadId io_thread_id) {
-  if (!features::IsSlimCompositorEnabled()) {
-    return base::WrapUnique<FrameSink>(new FrameSinkCcWrapper(
-        std::move(task_runner),
-        std::move(compositor_frame_sink_associated_remote),
-        std::move(client_receiver), std::move(context_provider),
-        gpu_memory_buffer_manager, io_thread_id));
-  }
   return base::WrapUnique<FrameSink>(
       new FrameSinkImpl(std::move(task_runner),
                         std::move(compositor_frame_sink_associated_remote),
