@@ -21,9 +21,12 @@
 #include "cc/base/tiling_data.h"
 #include "cc/cc_export.h"
 #include "cc/paint/paint_worklet_input.h"
+#include "cc/paint/scroll_offset_map.h"
 #include "cc/raster/raster_source.h"
 #include "cc/tiles/tile.h"
+#include "cc/tiles/tile_index.h"
 #include "cc/tiles/tile_priority.h"
+#include "cc/tiles/tiling_coverage_iterator.h"
 #include "cc/trees/occlusion.h"
 #include "ui/gfx/geometry/axis_transform2d.h"
 #include "ui/gfx/geometry/rect.h"
@@ -36,6 +39,8 @@ class TracedValue;
 
 namespace cc {
 
+class DrawImage;
+class GlobalStateThatImpactsTilePriority;
 class PictureLayerTiling;
 class PrioritizedTile;
 
@@ -53,41 +58,23 @@ class CC_EXPORT PictureLayerTilingClient {
   virtual bool HasValidTilePriorities() const = 0;
   virtual bool RequiresHighResToDraw() const = 0;
   virtual const PaintWorkletRecordMap& GetPaintWorkletRecords() const = 0;
-  virtual bool IsDirectlyCompositedImage() const = 0;
-  virtual bool ScrollInteractionInProgress() const = 0;
-  virtual bool CurrentScrollCheckerboardsDueToNoRecording() const = 0;
+  virtual std::vector<const DrawImage*> GetDiscardableImagesInRect(
+      const gfx::Rect& rect) const = 0;
+  virtual ScrollOffsetMap GetRasterInducingScrollOffsets() const = 0;
+  virtual const GlobalStateThatImpactsTilePriority& global_tile_state()
+      const = 0;
 
  protected:
   virtual ~PictureLayerTilingClient() {}
 };
 
-struct TileMapKey {
-  TileMapKey(int x, int y) : index_x(x), index_y(y) {}
-  explicit TileMapKey(const std::pair<int, int>& index)
-      : index_x(index.first), index_y(index.second) {}
-
-  bool operator==(const TileMapKey& other) const {
-    return index_x == other.index_x && index_y == other.index_y;
-  }
-  bool operator<(const TileMapKey& other) const {
-    return std::tie(index_x, index_y) < std::tie(other.index_x, other.index_y);
-  }
-
-  int index_x;
-  int index_y;
-};
-
-struct TileMapKeyHash {
-  size_t operator()(const TileMapKey& key) const {
-    uint16_t value1 = static_cast<uint16_t>(key.index_x);
-    uint16_t value2 = static_cast<uint16_t>(key.index_y);
-    uint32_t value1_32 = value1;
-    return (value1_32 << 16) | value2;
-  }
-};
+class PictureLayerTilingCoverageIterator;
 
 class CC_EXPORT PictureLayerTiling {
  public:
+  using Tile = cc::Tile;
+  using CoverageIterator = PictureLayerTilingCoverageIterator;
+
   static const int kBorderTexels = 1;
 
   // Note on raster_transform: In general raster_transform could be arbitrary,
@@ -150,14 +137,18 @@ class CC_EXPORT PictureLayerTiling {
     return can_require_tiles_for_activation_;
   }
 
+  gfx::Size raster_size() const { return raster_source()->size(); }
   const scoped_refptr<RasterSource>& raster_source() const {
     return raster_source_;
   }
   const PaintWorkletRecordMap& GetPaintWorkletRecords() const {
     return client_->GetPaintWorkletRecords();
   }
-  gfx::Size tiling_size() const { return tiling_data_.tiling_size(); }
-  gfx::Rect live_tiles_rect() const { return live_tiles_rect_; }
+  ScrollOffsetMap GetRasterInducingScrollOffsets() const {
+    return client_->GetRasterInducingScrollOffsets();
+  }
+  const gfx::Rect& tiling_rect() const { return tiling_data_.tiling_rect(); }
+  const gfx::Rect& live_tiles_rect() const { return live_tiles_rect_; }
   gfx::Size tile_size() const { return tiling_data_.max_texture_size(); }
   // PictureLayerTilingSet uses the scale component of the raster transform
   // as the key for indexing and sorting. In theory we can have multiple
@@ -172,8 +163,10 @@ class CC_EXPORT PictureLayerTiling {
   }
   const TilingData* tiling_data() const { return &tiling_data_; }
 
-  Tile* TileAt(int i, int j) const {
-    TileMap::const_iterator iter = tiles_.find(TileMapKey(i, j));
+  Tile* TileAt(int i, int j) const { return TileAt(TileIndex(i, j)); }
+
+  Tile* TileAt(const TileIndex& index) const {
+    TileMap::const_iterator iter = tiles_.find(index);
     return iter == tiles_.end() ? nullptr : iter->second.get();
   }
 
@@ -199,8 +192,9 @@ class CC_EXPORT PictureLayerTiling {
 
   // For testing functionality.
   void CreateAllTilesForTesting() {
-    SetLiveTilesRect(gfx::Rect(tiling_data_.tiling_size()));
+    CreateAllTilesForTesting(tiling_data_.tiling_rect());
   }
+  void CreateAllTilesForTesting(const gfx::Rect& rect_to_raster);
   const TilingData& TilingDataForTesting() const { return tiling_data_; }
   std::vector<Tile*> AllTilesForTesting() const {
     std::vector<Tile*> all_tiles;
@@ -210,9 +204,8 @@ class CC_EXPORT PictureLayerTiling {
   }
 
   void UpdateAllRequiredStateForTesting() {
-    for (const auto& key_tile_pair : tiles_) {
-      Tile* tile = key_tile_pair.second.get();
-      UpdateRequiredStatesOnTile(tile);
+    for (const auto& [index, tile] : tiles_) {
+      UpdateRequiredStatesOnTile(tile.get());
     }
   }
   std::map<const Tile*, PrioritizedTile>
@@ -229,17 +222,13 @@ class CC_EXPORT PictureLayerTiling {
   const gfx::Rect& GetCurrentVisibleRectForTesting() const {
     return current_visible_rect_;
   }
-  void SetTilePriorityRectsForTesting(
-      const gfx::Rect& visible_rect_in_content_space,
-      const gfx::Rect& skewport,
-      const gfx::Rect& soon_border_rect,
-      const gfx::Rect& eventually_rect) {
-    SetTilePriorityRects(1.f, visible_rect_in_content_space, skewport,
-                         soon_border_rect, eventually_rect, Occlusion());
-  }
+  void SetTilePriorityRectsForTesting(const gfx::Rect& visible_rect,
+                                      const gfx::Rect& skewport_rect,
+                                      const gfx::Rect& soon_border_rect,
+                                      const gfx::Rect& eventually_rect,
+                                      bool evicts_tiles = false);
 
-  using TileMap =
-      std::unordered_map<TileMapKey, std::unique_ptr<Tile>, TileMapKeyHash>;
+  using TileMap = std::unordered_map<TileIndex, std::unique_ptr<Tile>>;
 
   // Iterates over the tiles of a PictureLayerTiling. Order of iteration is not
   // defined.
@@ -257,66 +246,11 @@ class CC_EXPORT PictureLayerTiling {
     PictureLayerTiling::TileMap::iterator iter_;
   };
 
-  // Iterate over all tiles to fill content_rect.  Even if tiles are invalid
-  // (i.e. no valid resource) this tiling should still iterate over them.
-  // The union of all geometry_rect calls for each element iterated over should
-  // exactly equal content_rect and no two geometry_rects should intersect.
-  class CC_EXPORT CoverageIterator {
-   public:
-    CoverageIterator();
-    // This requests an iterator that produces a coverage of the
-    // |coverage_rect|, which is specified at |coverage_scale|.
-    CoverageIterator(const PictureLayerTiling* tiling,
-                     float coverage_scale,
-                     const gfx::Rect& coverage_rect);
-    ~CoverageIterator();
-
-    // Visible rect (no borders), always in the space of |coverage_rect|,
-    // regardless of the contents scale of the tiling.
-    gfx::Rect geometry_rect() const;
-    // Texture rect (in texels) for geometry_rect
-    gfx::RectF texture_rect() const;
-
-    Tile* operator->() const { return current_tile_; }
-    Tile* operator*() const { return current_tile_; }
-
-    CoverageIterator& operator++();
-    operator bool() const { return tile_j_ <= bottom_; }
-
-    int i() const { return tile_i_; }
-    int j() const { return tile_j_; }
-
-   private:
-    gfx::Rect ComputeGeometryRect() const;
-
-    // RAW_PTR_EXCLUSION: Performance reasons: based on analysis of sampling
-    // profiler data and tab_search:top100:2020.
-    RAW_PTR_EXCLUSION const PictureLayerTiling* tiling_ = nullptr;
-
-    gfx::Size coverage_rect_max_bounds_;
-    gfx::Rect coverage_rect_;
-    gfx::AxisTransform2d coverage_to_content_;
-
-    // RAW_PTR_EXCLUSION: Performance reasons: based on analysis of sampling
-    // profiler data and tab_search:top100:2020.
-    RAW_PTR_EXCLUSION Tile* current_tile_ = nullptr;
-
-    gfx::Rect current_geometry_rect_;
-    int tile_i_ = 0;
-    int tile_j_ = 0;
-    int left_ = 0;
-    int top_ = 0;
-    int right_ = -1;
-    int bottom_ = -1;
-
-    friend class PictureLayerTiling;
-  };
-
   void Reset();
 
   void ComputeTilePriorityRects(
       const gfx::Rect& visible_rect_in_layer_space,
-      const gfx::Rect& skewport_in_layer_space,
+      const gfx::Rect& skewport_rect_in_layer_space,
       const gfx::Rect& soon_border_rect_in_layer_space,
       const gfx::Rect& eventually_rect_in_layer_space,
       float ideal_contents_scale,
@@ -329,8 +263,11 @@ class CC_EXPORT PictureLayerTiling {
 
   void UpdateRequiredStatesOnTile(Tile* tile) const;
 
+  // Creates a coverage iterator to iterate over a minimal set of tiles from
+  // this tiling to cover `rect`, which has been pre-scaled by `scale`.
+  CoverageIterator Cover(const gfx::Rect& rect, float scale) const;
+
  protected:
-  friend class CoverageIterator;
   friend class PrioritizedTile;
   friend class TileIterator;
   friend class TilingSetRasterQueueAll;
@@ -386,8 +323,11 @@ class CC_EXPORT PictureLayerTiling {
       if (!active_twin || !TilingMatchesTileIndices(active_twin))
         return true;
 
-      if (active_twin->raster_source()->GetSize() != raster_source()->GetSize())
+      if (active_twin->raster_source()->size() != raster_source()->size() ||
+          active_twin->raster_source()->recorded_bounds() !=
+              raster_source()->recorded_bounds()) {
         return true;
+      }
 
       if (active_twin->current_visible_rect_ != current_visible_rect_)
         return true;
@@ -420,20 +360,16 @@ class CC_EXPORT PictureLayerTiling {
   }
 
   void SetLiveTilesRect(const gfx::Rect& live_tiles_rect);
-  void VerifyLiveTilesRect() const;
+  void VerifyTiles() const;
   Tile* CreateTile(const Tile::CreateInfo& info);
   // Removes the tile at i, j and returns it. Returns nullptr if the tile did
   // not exist.
   std::unique_ptr<Tile> TakeTileAt(int i, int j);
   bool TilingMatchesTileIndices(const PictureLayerTiling* twin) const;
 
-  // Save the required data for computing tile priorities later.
-  void SetTilePriorityRects(float content_to_screen_scale,
-                            const gfx::Rect& visible_rect_in_content_space,
-                            const gfx::Rect& skewport,
-                            const gfx::Rect& soon_border_rect,
-                            const gfx::Rect& eventually_rect,
-                            const Occlusion& occlusion_in_layer_space);
+  void SetPriorityRect(const gfx::Rect& rect_in_layer_space,
+                       PriorityRectType rect_type,
+                       bool evicts_tiles = false);
 
   bool IsTileOccludedOnCurrentTree(const Tile* tile) const;
   Tile::CreateInfo CreateInfoForTile(int i, int j) const;
@@ -509,7 +445,8 @@ class CC_EXPORT PictureLayerTiling {
   gfx::Rect EnclosingLayerRectFromContentsRect(
       const gfx::Rect& contents_rect) const;
 
-  void SetTilingSize(const gfx::Size& tiling_size);
+  gfx::Rect ComputeTilingRect() const;
+  void SetTilingRect(const gfx::Rect& tiling_rect);
 
   // Given properties.
   const gfx::AxisTransform2d raster_transform_;
@@ -522,28 +459,43 @@ class CC_EXPORT PictureLayerTiling {
   bool may_contain_low_resolution_tiles_ = false;
 
   // Internal data.
-  TilingData tiling_data_ = TilingData(gfx::Size(), gfx::Size(), kBorderTexels);
+  TilingData tiling_data_{gfx::Size(), gfx::Rect(), kBorderTexels};
   TileMap tiles_;  // It is not legal to have a NULL tile in the tiles_ map.
   gfx::Rect live_tiles_rect_;
 
   bool can_require_tiles_for_activation_ = false;
 
+  gfx::Rect tiling_rect_in_layer_space_;
+
   // Iteration rects in content space.
   gfx::Rect current_visible_rect_;
   gfx::Rect current_skewport_rect_;
   gfx::Rect current_soon_border_rect_;
   gfx::Rect current_eventually_rect_;
+
+  // Iteration rects in layer space.
+  gfx::Rect current_visible_rect_in_layer_space_;
+  gfx::Rect current_skewport_rect_in_layer_space_;
+  gfx::Rect current_soon_border_rect_in_layer_space_;
+  gfx::Rect current_eventually_rect_in_layer_space_;
+
   // Other properties used for tile iteration and prioritization.
   float current_content_to_screen_scale_ = 0.f;
   Occlusion current_occlusion_in_layer_space_;
   float max_skewport_extent_in_screen_space_ = 0.f;
 
-  bool has_visible_rect_tiles_ = false;
-  bool has_skewport_rect_tiles_ = false;
-  bool has_soon_border_rect_tiles_ = false;
-  bool has_eventually_rect_tiles_ = false;
-  bool all_tiles_done_ = true;
-  bool can_use_lcd_text_;
+  bool has_visible_rect_tiles_ : 1 = false;
+  bool has_skewport_rect_tiles_ : 1 = false;
+  bool has_soon_border_rect_tiles_ : 1 = false;
+  bool has_eventually_rect_tiles_ : 1 = false;
+  bool all_tiles_done_ : 1 = true;
+  bool can_use_lcd_text_ : 1;
+};
+
+class PictureLayerTilingCoverageIterator
+    : public TilingCoverageIterator<PictureLayerTiling> {
+ public:
+  using TilingCoverageIterator<PictureLayerTiling>::TilingCoverageIterator;
 };
 
 }  // namespace cc
