@@ -16,6 +16,7 @@
 #include "base/memory/raw_ptr.h"
 #include "base/memory/raw_ref.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/not_fatal_until.h"
 #include "base/numerics/ostream_operators.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/stringprintf.h"
@@ -72,13 +73,10 @@ class SoftwareImageDecodeTaskImpl : public TileTask {
       SoftwareImageDecodeCache* cache,
       const SoftwareImageDecodeCache::CacheKey& image_key,
       const PaintImage& paint_image,
-      SoftwareImageDecodeCache::DecodeTaskType task_type,
+      ImageDecodeCache::TaskType task_type,
       const ImageDecodeCache::TracingInfo& tracing_info)
       : TileTask(TileTask::SupportsConcurrentExecution::kYes,
-                 (base::FeatureList::IsEnabled(
-                      features::kNormalPriorityImageDecoding)
-                      ? TileTask::SupportsBackgroundThreadPriority::kNo
-                      : TileTask::SupportsBackgroundThreadPriority::kYes)),
+                 TileTask::SupportsBackgroundThreadPriority::kNo),
         cache_(cache),
         image_key_(image_key),
         paint_image_(paint_image),
@@ -101,7 +99,7 @@ class SoftwareImageDecodeTaskImpl : public TileTask {
     devtools_instrumentation::ScopedImageDecodeTask image_decode_task(
         paint_image_.GetSwSkImage().get(),
         devtools_instrumentation::ScopedImageDecodeTask::DecodeType::kSoftware,
-        ImageDecodeCache::ToScopedTaskType(tracing_info_.task_type),
+        ImageDecodeCache::ToScopedTaskType(task_type_),
         ImageDecodeCache::ToScopedImageType(image_type));
     SoftwareImageDecodeCache::TaskProcessingResult result =
         cache_->DecodeImageInTask(image_key_, paint_image_, task_type_);
@@ -130,7 +128,7 @@ class SoftwareImageDecodeTaskImpl : public TileTask {
   raw_ptr<SoftwareImageDecodeCache, AcrossTasksDanglingUntriaged> cache_;
   SoftwareImageDecodeCache::CacheKey image_key_;
   PaintImage paint_image_;
-  SoftwareImageDecodeCache::DecodeTaskType task_type_;
+  ImageDecodeCache::TaskType task_type_;
   const ImageDecodeCache::TracingInfo tracing_info_;
 };
 
@@ -188,11 +186,10 @@ ImageDecodeCache::TaskResult SoftwareImageDecodeCache::GetTaskForImageAndRef(
     ClientId client_id,
     const DrawImage& image,
     const TracingInfo& tracing_info) {
-  DCHECK_EQ(tracing_info.task_type, TaskType::kInRaster);
   DCHECK_EQ(client_id, ImageDecodeCache::kDefaultClientId)
       << "SoftwareImageDecodeCache cannot be shared between multiple clients.";
   return GetTaskForImageAndRefInternal(image, tracing_info,
-                                       DecodeTaskType::USE_IN_RASTER_TASKS);
+                                       TaskType::kInRaster);
 }
 
 ImageDecodeCache::TaskResult
@@ -201,16 +198,15 @@ SoftwareImageDecodeCache::GetOutOfRasterDecodeTaskForImageAndRef(
     const DrawImage& image) {
   DCHECK_EQ(client_id, ImageDecodeCache::kDefaultClientId)
       << "SoftwareImageDecodeCache cannot be shared between multiple clients.";
-  return GetTaskForImageAndRefInternal(
-      image, TracingInfo(0, TilePriority::NOW, TaskType::kOutOfRaster),
-      DecodeTaskType::USE_OUT_OF_RASTER_TASKS);
+  return GetTaskForImageAndRefInternal(image, TracingInfo(0, TilePriority::NOW),
+                                       TaskType::kOutOfRaster);
 }
 
 ImageDecodeCache::TaskResult
 SoftwareImageDecodeCache::GetTaskForImageAndRefInternal(
     const DrawImage& image,
     const TracingInfo& tracing_info,
-    DecodeTaskType task_type) {
+    TaskType task_type) {
   CacheKey key = CacheKey::FromDrawImage(
       image, GetColorTypeForPaintImage(image.target_color_params(),
                                        image.paint_image()));
@@ -242,8 +238,9 @@ SoftwareImageDecodeCache::GetTaskForImageAndRefInternal(
       return TaskResult(/*need_unref=*/false, /*is_at_raster_decode=*/true,
                         /*can_do_hardware_accelerated_decode=*/false);
     cache_entry = AddCacheEntry(key);
-    if (task_type == DecodeTaskType::USE_OUT_OF_RASTER_TASKS)
+    if (task_type == TaskType::kOutOfRaster) {
       cache_entry->mark_out_of_raster();
+    }
   } else {
     cache_entry = decoded_it->second.get();
   }
@@ -270,10 +267,9 @@ SoftwareImageDecodeCache::GetTaskForImageAndRefInternal(
     return TaskResult(/*need_unref=*/true, /*is_at_raster_decode=*/false,
                       /*can_do_hardware_accelerated_decode=*/false);
 
-  scoped_refptr<TileTask>& task =
-      task_type == DecodeTaskType::USE_IN_RASTER_TASKS
-          ? cache_entry->in_raster_task
-          : cache_entry->out_of_raster_task;
+  scoped_refptr<TileTask>& task = task_type == TaskType::kInRaster
+                                      ? cache_entry->in_raster_task
+                                      : cache_entry->out_of_raster_task;
   if (!task) {
     // Ref image once for the decode task.
     ++cache_entry->ref_count;
@@ -319,7 +315,7 @@ void SoftwareImageDecodeCache::UnrefImage(const DrawImage& image) {
 
 void SoftwareImageDecodeCache::UnrefImage(const CacheKey& key) {
   auto decoded_image_it = decoded_images_.Peek(key);
-  DCHECK(decoded_image_it != decoded_images_.end());
+  CHECK(decoded_image_it != decoded_images_.end(), base::NotFatalUntil::M130);
   auto* entry = decoded_image_it->second.get();
   DCHECK_GT(entry->ref_count, 0);
   if (--entry->ref_count == 0) {
@@ -335,13 +331,13 @@ void SoftwareImageDecodeCache::UnrefImage(const CacheKey& key) {
 SoftwareImageDecodeCache::TaskProcessingResult
 SoftwareImageDecodeCache::DecodeImageInTask(const CacheKey& key,
                                             const PaintImage& paint_image,
-                                            DecodeTaskType task_type) {
+                                            TaskType task_type) {
   TRACE_EVENT1("cc,benchmark", "SoftwareImageDecodeCache::DecodeImageInTask",
                "key", key.ToString());
   base::AutoLock lock(lock_);
 
   auto image_it = decoded_images_.Peek(key);
-  DCHECK(image_it != decoded_images_.end());
+  CHECK(image_it != decoded_images_.end(), base::NotFatalUntil::M130);
   auto* cache_entry = image_it->second.get();
   // These two checks must be true because we're running this from a task, which
   // means that we've budgeted this entry when we got the task and the ref count
@@ -505,7 +501,8 @@ SoftwareImageDecodeCache::FindCachedCandidate(const CacheKey& key) {
   auto image_keys_it = frame_key_to_image_keys_.find(key.frame_key());
   // We know that we must have at least our own |entry| in this list, so it
   // won't be empty.
-  DCHECK(image_keys_it != frame_key_to_image_keys_.end());
+  CHECK(image_keys_it != frame_key_to_image_keys_.end(),
+        base::NotFatalUntil::M130);
 
   auto& available_keys = image_keys_it->second;
   std::sort(available_keys.begin(), available_keys.end(),
@@ -527,7 +524,7 @@ SoftwareImageDecodeCache::FindCachedCandidate(const CacheKey& key) {
       continue;
     }
     auto image_it = decoded_images_.Peek(available_key);
-    DCHECK(image_it != decoded_images_.end());
+    CHECK(image_it != decoded_images_.end(), base::NotFatalUntil::M130);
     auto* available_entry = image_it->second.get();
     if (available_entry->is_locked || available_entry->Lock()) {
       return available_key;
@@ -638,7 +635,7 @@ void SoftwareImageDecodeCache::ReduceCacheUsageUntilWithinLimit(size_t limit) {
     const CacheKey& key = it->first;
     auto vector_it = frame_key_to_image_keys_.find(key.frame_key());
     auto item_it = base::ranges::find(vector_it->second, key);
-    DCHECK(item_it != vector_it->second.end());
+    CHECK(item_it != vector_it->second.end(), base::NotFatalUntil::M130);
     vector_it->second.erase(item_it);
     if (vector_it->second.empty())
       frame_key_to_image_keys_.erase(vector_it);
@@ -661,17 +658,16 @@ size_t SoftwareImageDecodeCache::GetMaximumMemoryLimitBytes() const {
   return locked_images_budget_.total_limit_bytes();
 }
 
-void SoftwareImageDecodeCache::OnImageDecodeTaskCompleted(
-    const CacheKey& key,
-    DecodeTaskType task_type) {
+void SoftwareImageDecodeCache::OnImageDecodeTaskCompleted(const CacheKey& key,
+                                                          TaskType task_type) {
   base::AutoLock hold(lock_);
 
   auto image_it = decoded_images_.Peek(key);
-  DCHECK(image_it != decoded_images_.end());
+  CHECK(image_it != decoded_images_.end(), base::NotFatalUntil::M130);
   CacheEntry* cache_entry = image_it->second.get();
   UMA_HISTOGRAM_BOOLEAN("Compositing.DecodeLCPCandidateImage.Software",
                         key.may_be_lcp_candidate());
-  auto& task = task_type == DecodeTaskType::USE_IN_RASTER_TASKS
+  auto& task = task_type == TaskType::kInRaster
                    ? cache_entry->in_raster_task
                    : cache_entry->out_of_raster_task;
   task = nullptr;
@@ -742,8 +738,8 @@ SkColorType SoftwareImageDecodeCache::GetColorTypeForPaintImage(
     const TargetColorParams& target_color_params,
     const PaintImage& paint_image) {
   const gfx::ColorSpace& target_color_space = target_color_params.color_space;
-  // TODO(crbug.com/1076568): Once we have access to the display's buffer format
-  // via gfx::DisplayColorSpaces, we should also do this for HBD images.
+  // TODO(crbug.com/40128725): Once we have access to the display's buffer
+  // format via gfx::DisplayColorSpaces, we should also do this for HBD images.
   // Do not decode an image to F16 unless the PaintImage reports that its type
   // is F16. Otherwise, image decode will fail.
   // https://crbug.com/1488786
