@@ -162,19 +162,16 @@ class LayerTreeHostReadbackPixelTest
     ASSERT_EQ(result->destination(),
               viz::CopyOutputResult::Destination::kNativeTextures);
 
-    gpu::Mailbox mailbox =
-        result->GetTextureResult()->mailbox_holders[0].mailbox;
-    gpu::SyncToken sync_token =
-        result->GetTextureResult()->mailbox_holders[0].sync_token;
+    gpu::Mailbox mailbox = result->GetTextureResult()->mailbox;
     gfx::ColorSpace color_space = result->GetTextureResult()->color_space;
-    EXPECT_EQ(result->GetTextureResult()->color_space, output_color_space_);
+    EXPECT_EQ(color_space, output_color_space_);
 
     viz::CopyOutputResult::ReleaseCallbacks release_callbacks =
         result->TakeTextureOwnership();
     EXPECT_EQ(1u, release_callbacks.size());
 
-    SkBitmap bitmap =
-        CopyMailboxToBitmap(result->size(), mailbox, sync_token, color_space);
+    SkBitmap bitmap = CopyMailboxToBitmap(result->size(), mailbox,
+                                          gpu::SyncToken(), color_space);
     std::move(release_callbacks[0]).Run(gpu::SyncToken(), false);
 
     ReadbackResultAsBitmap(std::make_unique<viz::CopyOutputSkBitmapResult>(
@@ -479,7 +476,7 @@ TEST_P(LayerTreeHostReadbackPixelTest, MultipleReadbacksOnLayer) {
       base::FilePath(FILE_PATH_LITERAL("green.png")));
 }
 
-// TODO(crbug.com/971257): Enable these tests for Skia Vulkan using texture
+// TODO(crbug.com/41463412): Enable these tests for Skia Vulkan using texture
 // readback.
 ReadbackTestConfig const kTestConfigs[] = {
     ReadbackTestConfig{viz::RendererType::kSoftware, TestReadBackType::kBitmap},
