@@ -20,10 +20,12 @@
 #include "base/trace_event/trace_event.h"
 #include "base/trace_event/traced_value.h"
 #include "base/trace_event/typed_macros.h"
+#include "base/types/optional_ref.h"
 #include "cc/base/devtools_instrumentation.h"
 #include "cc/base/features.h"
 #include "cc/benchmarks/benchmark_instrumentation.h"
 #include "cc/input/browser_controls_offset_manager.h"
+#include "cc/input/browser_controls_offset_tags_info.h"
 #include "cc/metrics/compositor_timing_history.h"
 #include "cc/paint/paint_worklet_layer_painter.h"
 #include "cc/trees/compositor_commit_data.h"
@@ -34,6 +36,7 @@
 #include "cc/trees/proxy_main.h"
 #include "cc/trees/render_frame_metadata_observer.h"
 #include "cc/trees/task_runner_provider.h"
+#include "cc/trees/trace_utils.h"
 #include "components/viz/common/frame_sinks/delay_based_time_source.h"
 #include "components/viz/common/frame_timing_details.h"
 #include "components/viz/common/gpu/raster_context_provider.h"
@@ -113,7 +116,6 @@ ProxyImpl::ProxyImpl(
   DCHECK(IsMainThreadBlocked());
 
   host_impl_ = layer_tree_host->CreateLayerTreeHostImpl(this);
-  send_compositor_frame_ack_ = settings->send_compositor_frame_ack;
 
   SchedulerSettings scheduler_settings(settings->ToSchedulerSettings());
   scheduler_settings.main_frame_before_commit_enabled =
@@ -121,7 +123,6 @@ ProxyImpl::ProxyImpl(
 
   std::unique_ptr<CompositorTimingHistory> compositor_timing_history(
       new CompositorTimingHistory(
-          scheduler_settings.using_synchronous_renderer_compositor,
           CompositorTimingHistory::RENDERER_UMA,
           rendering_stats_instrumentation));
   scheduler_ = std::make_unique<Scheduler>(
@@ -173,10 +174,11 @@ void ProxyImpl::InitializePaintWorkletLayerPainterOnImpl(
 void ProxyImpl::UpdateBrowserControlsStateOnImpl(
     BrowserControlsState constraints,
     BrowserControlsState current,
-    bool animate) {
+    bool animate,
+    base::optional_ref<const BrowserControlsOffsetTagsInfo> offset_tags_info) {
   DCHECK(IsImplThread());
   host_impl_->browser_controls_manager()->UpdateBrowserControlsState(
-      constraints, current, animate);
+      constraints, current, animate, offset_tags_info);
 }
 
 void ProxyImpl::InitializeLayerTreeFrameSinkOnImpl(
@@ -273,7 +275,16 @@ void ProxyImpl::SetVisibleOnImpl(bool visible) {
   scheduler_->SetVisible(visible);
 }
 
+void ProxyImpl::SetShouldWarmUpOnImpl() {
+  TRACE_EVENT0("cc", "ProxyImpl::SetShouldWarmUpOnImpl");
+  DCHECK(IsImplThread());
+  scheduler_->SetShouldWarmUp();
+}
+
 void ProxyImpl::ReleaseLayerTreeFrameSinkOnImpl(CompletionEvent* completion) {
+  TRACE_EVENT0("cc", "ProxyImpl::ReleaseLayerTreeFrameSinkOnImpl");
+  base::ScopedUmaHistogramTimer histogram_timer(
+      "Navigation.ProxyImpl.ReleaseLayerTreeFrameSinkOnImpl");
   DCHECK(IsImplThread());
 
   // Unlike DidLoseLayerTreeFrameSinkOnImplThread, we don't need to call
@@ -322,7 +333,15 @@ bool ProxyImpl::IsInSynchronousComposite() const {
 
 void ProxyImpl::FrameSinksToThrottleUpdated(
     const base::flat_set<viz::FrameSinkId>& ids) {
-  NOTREACHED();
+  NOTREACHED_IN_MIGRATION();
+}
+
+void ProxyImpl::SetHasActiveThreadedScroll(bool is_scrolling) {
+  scheduler_->SetIsScrolling(is_scrolling);
+}
+
+void ProxyImpl::SetWaitingForScrollEvent(bool waiting_for_scroll_event) {
+  scheduler_->SetWaitingForScrollEvent(waiting_for_scroll_event);
 }
 
 void ProxyImpl::NotifyReadyToCommitOnImpl(
@@ -334,12 +353,14 @@ void ProxyImpl::NotifyReadyToCommitOnImpl(
     bool scroll_and_viewport_changes_synced,
     CommitTimestamps* commit_timestamps,
     bool commit_timeout) {
-  {
-    TRACE_EVENT_WITH_FLOW0(
-        "viz,benchmark", "MainFrame.NotifyReadyToCommitOnImpl",
-        TRACE_ID_LOCAL(commit_state->trace_id),
-        TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT);
-  }
+  TRACE_EVENT("cc,benchmark", "ProxyImpl::ReadyToCommit",
+              [&](perfetto::EventContext ctx) {
+                EmitMainFramePipelineStep(
+                    ctx, commit_state->trace_id,
+                    perfetto::protos::pbzero::MainFramePipeline::Step::
+                        READY_TO_COMMIT_ON_IMPL);
+              });
+
   DCHECK(!data_for_commit_.get());
   DCHECK(IsImplThread());
   DCHECK(base::FeatureList::IsEnabled(features::kNonBlockingCommit) ||
@@ -389,6 +410,10 @@ void ProxyImpl::NotifyReadyToCommitOnImpl(
 
   // If scroll offsets were not synchronized by this commit we need another main
   // frame to sync them.
+  // Note that scroll_and_viewport_changes_synced will be false for the frame
+  // that contains the first contentful paint, since paint holding makes us skip
+  // ApplyCompositorChanges. This means we are guaranteed to run another main
+  // frame after FCP, which is good because there may be hover effects to apply.
   if (!scroll_and_viewport_changes_synced)
     scheduler_->SetNeedsBeginMainFrame();
 }
@@ -417,11 +442,6 @@ void ProxyImpl::DidReceiveCompositorFrameAckOnImplThread() {
                "ProxyImpl::DidReceiveCompositorFrameAckOnImplThread");
   DCHECK(IsImplThread());
   scheduler_->DidReceiveCompositorFrameAck();
-  if (send_compositor_frame_ack_) {
-    MainThreadTaskRunner()->PostTask(
-        FROM_HERE, base::BindOnce(&ProxyMain::DidReceiveCompositorFrameAck,
-                                  proxy_main_frame_sink_bound_weak_ptr_));
-  }
 }
 
 void ProxyImpl::OnCanDrawStateChanged(bool can_draw) {
@@ -431,14 +451,14 @@ void ProxyImpl::OnCanDrawStateChanged(bool can_draw) {
 }
 
 void ProxyImpl::NotifyReadyToActivate() {
-  if (host_impl_->sync_tree() &&
-      !scheduler_->pending_tree_is_ready_for_activation()) {
-    TRACE_EVENT_WITH_FLOW0(
-        "viz,benchmark", "MainFrame.NotifyReadyToActivate",
-        TRACE_ID_LOCAL(host_impl_->sync_tree()->trace_id()),
-        TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT);
-  }
   DCHECK(IsImplThread());
+  TRACE_EVENT_INSTANT("cc,benchmark", "ProxyImpl::ReadyToActivate",
+                      [&](perfetto::EventContext ctx) {
+                        EmitMainFramePipelineStep(
+                            ctx, host_impl_->active_tree()->trace_id(),
+                            perfetto::protos::pbzero::MainFramePipeline::Step::
+                                READY_TO_ACTIVATE);
+                      });
   scheduler_->NotifyReadyToActivate();
 }
 
@@ -465,6 +485,12 @@ void ProxyImpl::SetNeedsOneBeginImplFrameOnImplThread() {
   scheduler_->SetNeedsOneBeginImplFrame();
 }
 
+void ProxyImpl::SetNeedsUpdateDisplayTreeOnImplThread() {
+  TRACE_EVENT0("cc", "ProxyImpl::SetNeedsUpdateDisplayTreeOnImplThread");
+  DCHECK(IsImplThread());
+  scheduler_->SetNeedsUpdateDisplayTree();
+}
+
 void ProxyImpl::SetNeedsPrepareTilesOnImplThread() {
   DCHECK(IsImplThread());
   scheduler_->SetNeedsPrepareTiles();
@@ -495,10 +521,8 @@ void ProxyImpl::RenewTreePriority() {
   bool precise_scrolling_in_progress =
       host_impl_->GetActivelyScrollingType() == ActivelyScrollingType::kPrecise;
 
-  bool avoid_entering_smoothness =
-      host_impl_->CurrentScrollCheckerboardsDueToNoRecording() ||
-      (precise_scrolling_in_progress &&
-       host_impl_->IsCurrentScrollMainRepainted());
+  bool avoid_entering_smoothness = precise_scrolling_in_progress &&
+                                   host_impl_->IsCurrentScrollMainRepainted();
 
   bool non_scroll_interaction_in_progress =
       host_impl_->IsPinchGestureActive() ||
@@ -567,11 +591,6 @@ void ProxyImpl::DidActivateSyncTree() {
   }
 }
 
-void ProxyImpl::WillPrepareTiles() {
-  DCHECK(IsImplThread());
-  scheduler_->WillPrepareTiles();
-}
-
 void ProxyImpl::DidPrepareTiles() {
   DCHECK(IsImplThread());
   scheduler_->DidPrepareTiles();
@@ -591,14 +610,24 @@ void ProxyImpl::OnDrawForLayerTreeFrameSink(bool resourceless_software_draw,
                                           skip_draw);
 }
 
-void ProxyImpl::NeedsImplSideInvalidation(bool needs_first_draw_on_activation) {
+void ProxyImpl::SetNeedsImplSideInvalidation(
+    bool needs_first_draw_on_activation) {
   DCHECK(IsImplThread());
   scheduler_->SetNeedsImplSideInvalidation(needs_first_draw_on_activation);
 }
 
-void ProxyImpl::NotifyImageDecodeRequestFinished() {
+void ProxyImpl::NotifyImageDecodeRequestFinished(int request_id,
+                                                 bool decode_succeeded) {
   DCHECK(IsImplThread());
-  SetNeedsCommitOnImplThread();
+  if (base::FeatureList::IsEnabled(
+          features::kSendExplicitDecodeRequestsImmediately)) {
+    MainThreadTaskRunner()->PostTask(
+        FROM_HERE,
+        base::BindOnce(&ProxyMain::NotifyImageDecodeRequestFinished,
+                       proxy_main_weak_ptr_, request_id, decode_succeeded));
+  } else {
+    SetNeedsCommitOnImplThread();
+  }
 }
 
 void ProxyImpl::NotifyTransitionRequestFinished(uint32_t sequence_id) {
@@ -617,11 +646,11 @@ void ProxyImpl::DidPresentCompositorFrameOnImplThread(
       details);
 
   MainThreadTaskRunner()->PostTask(
-      FROM_HERE, base::BindOnce(&ProxyMain::DidPresentCompositorFrame,
-                                proxy_main_weak_ptr_, frame_token,
-                                std::move(activated.main_callbacks),
-                                std::move(activated.main_successful_callbacks),
-                                details.presentation_feedback));
+      FROM_HERE,
+      base::BindOnce(&ProxyMain::DidPresentCompositorFrame,
+                     proxy_main_weak_ptr_, frame_token,
+                     std::move(activated.main_callbacks),
+                     std::move(activated.main_successful_callbacks), details));
   if (scheduler_)
     scheduler_->DidPresentCompositorFrame(frame_token, details);
 }
@@ -703,36 +732,32 @@ void ProxyImpl::ScheduledActionSendBeginMainFrame(
                        : nullptr);
   begin_main_frame_state->completed_image_decode_requests =
       host_impl_->TakeCompletedImageDecodeRequests();
+  DCHECK(!base::FeatureList::IsEnabled(
+             features::kSendExplicitDecodeRequestsImmediately) ||
+         begin_main_frame_state->completed_image_decode_requests.empty());
   begin_main_frame_state->mutator_events = host_impl_->TakeMutatorEvents();
   begin_main_frame_state->active_sequence_trackers =
       host_impl_->FrameSequenceTrackerActiveTypes();
   begin_main_frame_state->evicted_ui_resources =
       host_impl_->EvictedUIResourcesExist();
-  begin_main_frame_state->trace_id =
-      (0x1llu << 51) |  // Signature bit chosen at random to avoid collisions
-      (args.frame_id.source_id << 32) |
-      (args.frame_id.sequence_number & 0xffffffff);
   host_impl_->WillSendBeginMainFrame();
   {
-    TRACE_EVENT(
-        "viz,benchmark,graphics.pipeline", "Graphics.Pipeline",
-        perfetto::TerminatingFlow::Global(args.trace_id),
-        [&](perfetto::EventContext ctx) {
-          auto* event = ctx.event<perfetto::protos::pbzero::ChromeTrackEvent>();
-          auto* data = event->set_chrome_graphics_pipeline();
-          data->set_step(perfetto::protos::pbzero::ChromeGraphicsPipeline::
-                             StepName::STEP_SEND_BEGIN_MAIN_FRAME);
+    TRACE_EVENT_INSTANT(
+        "cc,benchmark", "SendBeginMainFrame", [&](perfetto::EventContext ctx) {
+          auto* pipeline = EmitMainFramePipelineStep(
+              ctx, begin_main_frame_state->trace_id,
+              perfetto::protos::pbzero::MainFramePipeline::Step::
+                  SEND_BEGIN_MAIN_FRAME);
+
+          auto* begin_frame_id = pipeline->set_begin_frame_id();
+          begin_frame_id->set_sequence_number(args.frame_id.sequence_number);
+          begin_frame_id->set_source_id(args.frame_id.source_id);
         });
-    TRACE_EVENT_WITH_FLOW0("viz,benchmark",
-                           "MainFrame.SendBeginMainFrameOnImpl",
-                           TRACE_ID_LOCAL(begin_main_frame_state->trace_id),
-                           TRACE_EVENT_FLAG_FLOW_OUT);
   }
   MainThreadTaskRunner()->PostTask(
       FROM_HERE,
       base::BindOnce(&ProxyMain::BeginMainFrame, proxy_main_weak_ptr_,
                      std::move(begin_main_frame_state)));
-  host_impl_->DidSendBeginMainFrame(args);
   devtools_instrumentation::DidRequestMainThreadFrame(layer_tree_host_id_);
 }
 
@@ -754,13 +779,53 @@ DrawResult ProxyImpl::ScheduledActionDrawForced() {
   return DrawInternal(forced_draw);
 }
 
-void ProxyImpl::ScheduledActionCommit() {
-  {
-    TRACE_EVENT_WITH_FLOW0(
-        "viz,benchmark", "MainFrame.BeginCommit",
-        TRACE_ID_LOCAL(data_for_commit_->commit_state->trace_id),
-        TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT);
+void ProxyImpl::ScheduledActionUpdateDisplayTree() {
+  TRACE_EVENT0("cc", "ProxyImpl::ScheduledActionUpdateDisplayTree");
+  DCHECK(IsImplThread());
+  DCHECK(host_impl_.get());
+
+  TRACE_EVENT("cc,benchmark", "MainFrame.UpdateDisplayTree",
+              [&](perfetto::EventContext ctx) {
+                EmitMainFramePipelineStep(
+                    ctx, host_impl_->active_tree()->trace_id(),
+                    perfetto::protos::pbzero::MainFramePipeline::Step::
+                        UPDATE_DISPLAY_TREE);
+              });
+
+  // TODO(rockot): Maybe this flag should be renamed to reflect that we hold it
+  // during display tree updates too.
+  base::AutoReset<bool> mark_inside(&inside_draw_, true);
+
+  LayerTreeHostImpl::FrameData frame;
+  frame.begin_frame_ack = scheduler_->CurrentBeginFrameAckForActiveTree();
+  frame.origin_begin_main_frame_args =
+      scheduler_->last_activate_origin_frame_args();
+  host_impl_->UpdateDisplayTree(frame);
+
+  // Tell the main thread that the frame was drawn.
+  if (next_frame_is_newly_committed_frame_) {
+    next_frame_is_newly_committed_frame_ = false;
+    MainThreadTaskRunner()->PostTask(
+        FROM_HERE,
+        base::BindOnce(&ProxyMain::DidCommitAndDrawFrame, proxy_main_weak_ptr_,
+                       host_impl_->active_tree()->source_frame_number()));
   }
+
+  // The tile visibility/priority of the pending tree needs to be updated so
+  // that it doesn't get activated before the raster is complete.
+  if (host_impl_->pending_tree()) {
+    host_impl_->pending_tree()->UpdateDrawProperties(
+        /*update_tiles=*/true, /*update_image_animation_controller=*/true);
+  }
+}
+
+void ProxyImpl::ScheduledActionCommit() {
+  TRACE_EVENT(
+      "cc,benchmark", "ProxyImpl::Commit", [&](perfetto::EventContext ctx) {
+        EmitMainFramePipelineStep(
+            ctx, data_for_commit_->commit_state->trace_id,
+            perfetto::protos::pbzero::MainFramePipeline::Step::COMMIT_ON_IMPL);
+      });
   DCHECK(IsImplThread());
   DCHECK(base::FeatureList::IsEnabled(features::kNonBlockingCommit) ||
          IsMainThreadBlocked());
@@ -791,9 +856,6 @@ void ProxyImpl::ScheduledActionCommit() {
 
 void ProxyImpl::ScheduledActionPostCommit() {
   DCHECK(IsImplThread());
-  TRACE_EVENT_WITH_FLOW0("viz,benchmark", "MainFrame.CommitComplete",
-                         TRACE_ID_LOCAL(host_impl_->sync_tree()->trace_id()),
-                         TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT);
 
   // This is run as a separate step from commit because it can be time-consuming
   // and ought not delay sending the next BeginMainFrame.
@@ -804,10 +866,6 @@ void ProxyImpl::ScheduledActionActivateSyncTree() {
   if (host_impl_->sync_tree() &&
       host_impl_->sync_tree()->source_frame_number() !=
           host_impl_->active_tree()->source_frame_number()) {
-    TRACE_EVENT_WITH_FLOW0(
-        "viz,benchmark", "MainFrame.Activate",
-        TRACE_ID_LOCAL(host_impl_->sync_tree()->trace_id()),
-        TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT);
     next_frame_is_newly_committed_frame_ = true;
   }
   DCHECK(IsImplThread());
@@ -856,13 +914,29 @@ void ProxyImpl::ScheduledActionBeginMainFrameNotExpectedUntil(
                                 proxy_main_weak_ptr_, time));
 }
 
+void ProxyImpl::OnBeginImplFrameDeadline() {
+  DCHECK(IsImplThread());
+  host_impl_->OnBeginImplFrameDeadline();
+}
+
 DrawResult ProxyImpl::DrawInternal(bool forced_draw) {
   DCHECK(IsImplThread());
   DCHECK(host_impl_.get());
 
-  TRACE_EVENT_WITH_FLOW0("viz,benchmark", "MainFrame.Draw",
-                         TRACE_ID_LOCAL(host_impl_->active_tree()->trace_id()),
-                         TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT);
+  TRACE_EVENT(
+      "cc,benchmark", "MainFrame.Draw", [&](perfetto::EventContext ctx) {
+        auto* pipeline = EmitMainFramePipelineStep(
+            ctx, host_impl_->active_tree()->trace_id(),
+            perfetto::protos::pbzero::MainFramePipeline::Step::DRAW);
+        if (next_frame_is_newly_committed_frame_) {
+          viz::BeginFrameId frame_id =
+              host_impl_->CurrentBeginFrameArgs().frame_id;
+          auto* first_draw_frame_id =
+              pipeline->set_last_begin_frame_id_during_first_draw();
+          first_draw_frame_id->set_source_id(frame_id.source_id);
+          first_draw_frame_id->set_sequence_number(frame_id.sequence_number);
+        }
+      });
 
   base::AutoReset<bool> mark_inside(&inside_draw_, true);
 
@@ -892,13 +966,11 @@ DrawResult ProxyImpl::DrawInternal(bool forced_draw) {
   }
 
   if (draw_frame) {
-    if (std::optional<LayerTreeHostImpl::SubmitInfo> submit_info =
+    if (std::optional<SubmitInfo> submit_info =
             host_impl_->DrawLayers(&frame)) {
       DCHECK_NE(frame.frame_token, 0u);
       // Drawing implies we submitted a frame to the LayerTreeFrameSink.
-      scheduler_->DidSubmitCompositorFrame(
-          frame.frame_token, submit_info->time,
-          std::move(submit_info->events_metrics), frame.has_missing_content);
+      scheduler_->DidSubmitCompositorFrame(submit_info.value());
     }
     result = DrawResult::kSuccess;
   } else {
@@ -922,8 +994,10 @@ DrawResult ProxyImpl::DrawInternal(bool forced_draw) {
   // The tile visibility/priority of the pending tree needs to be updated so
   // that it doesn't get activated before the raster is complete. But this needs
   // to happen after the draw, off of the critical path to draw.
-  if (host_impl_->pending_tree())
-    host_impl_->pending_tree()->UpdateDrawProperties();
+  if (host_impl_->pending_tree()) {
+    host_impl_->pending_tree()->UpdateDrawProperties(
+        /*update_tiles=*/true, /*update_image_animation_controller=*/true);
+  }
 
   DCHECK_NE(DrawResult::kInvalidResult, result);
   return result;
@@ -941,6 +1015,11 @@ base::SingleThreadTaskRunner* ProxyImpl::MainThreadTaskRunner() {
   return task_runner_provider_->MainThreadTaskRunner();
 }
 
+void ProxyImpl::QueueImageDecodeOnImpl(int request_id,
+                                       std::unique_ptr<PaintImage> image) {
+  host_impl_->QueueImageDecode(request_id, *image);
+}
+
 void ProxyImpl::SetSourceURL(ukm::SourceId source_id, const GURL& url) {
   DCHECK(IsImplThread());
   host_impl_->SetActiveURL(url, source_id);
