@@ -19,6 +19,7 @@
 #include "base/containers/contains.h"
 #include "base/functional/bind.h"
 #include "base/json/json_writer.h"
+#include "base/notreached.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
@@ -52,20 +53,17 @@
 #include "extensions/common/manifest_handlers/background_info.h"
 #include "services/accessibility/public/mojom/assistive_technology_type.mojom.h"
 #include "ui/accessibility/accessibility_features.h"
-#include "ui/aura/client/cursor_client.h"
 #include "ui/aura/window_tree_host.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/ui_base_features.h"
 #include "ui/display/display.h"
 #include "ui/display/screen.h"
-#include "ui/events/base_event_utils.h"
 #include "ui/events/keycodes/dom/dom_codes_array.h"
 #include "ui/events/keycodes/keyboard_code_conversion.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/events/ozone/layout/keyboard_layout_engine.h"
 #include "ui/events/ozone/layout/keyboard_layout_engine_manager.h"
 #include "ui/strings/grit/ui_strings.h"
-#include "ui/wm/core/coordinate_conversion.h"
 
 namespace {
 
@@ -80,7 +78,7 @@ ash::AccessibilityToastType ConvertToastType(
     case accessibility_private::ToastType::kDictationMicMuted:
       return ash::AccessibilityToastType::kDictationMicMuted;
     case accessibility_private::ToastType::kNone:
-      NOTREACHED_NORETURN();
+      NOTREACHED();
   }
 }
 
@@ -104,11 +102,103 @@ ash::DictationBubbleHintType ConvertDictationHintType(
     case accessibility_private::DictationBubbleHintType::kCopy:
       return ash::DictationBubbleHintType::kCopy;
     case accessibility_private::DictationBubbleHintType::kNone:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
       return ash::DictationBubbleHintType::kTrySaying;
   }
 }
 
+ash::AccessibilityScrollDirection ConvertScrollDirection(
+    accessibility_private::ScrollDirection direction) {
+  switch (direction) {
+    case accessibility_private::ScrollDirection::kUp:
+      return ash::AccessibilityScrollDirection::kUp;
+    case accessibility_private::ScrollDirection::kDown:
+      return ash::AccessibilityScrollDirection::kDown;
+    case accessibility_private::ScrollDirection::kLeft:
+      return ash::AccessibilityScrollDirection::kLeft;
+    case accessibility_private::ScrollDirection::kRight:
+      return ash::AccessibilityScrollDirection::kRight;
+    case accessibility_private::ScrollDirection::kNone:
+      NOTREACHED_NORETURN();
+  }
+}
+
+std::string ConvertFacialGestureType(
+    accessibility_private::FacialGesture gesture_type) {
+  switch (gesture_type) {
+    case accessibility_private::FacialGesture::kBrowInnerUp:
+      return "browInnerUp";
+    case accessibility_private::FacialGesture::kBrowsDown:
+      return "browsDown";
+    case accessibility_private::FacialGesture::kEyeSquintLeft:
+      return "eyeSquintLeft";
+    case accessibility_private::FacialGesture::kEyeSquintRight:
+      return "eyeSquintRight";
+    case accessibility_private::FacialGesture::kEyesBlink:
+      return "eyesBlink";
+    case accessibility_private::FacialGesture::kEyesLookDown:
+      return "eyesLookDown";
+    case accessibility_private::FacialGesture::kEyesLookLeft:
+      return "eyesLookLeft";
+    case accessibility_private::FacialGesture::kEyesLookRight:
+      return "eyesLookRight";
+    case accessibility_private::FacialGesture::kEyesLookUp:
+      return "eyesLookUp";
+    case accessibility_private::FacialGesture::kJawLeft:
+      return "jawLeft";
+    case accessibility_private::FacialGesture::kJawOpen:
+      return "jawOpen";
+    case accessibility_private::FacialGesture::kJawRight:
+      return "jawRight";
+    case accessibility_private::FacialGesture::kMouthFunnel:
+      return "mouthFunnel";
+    case accessibility_private::FacialGesture::kMouthLeft:
+      return "mouthLeft";
+    case accessibility_private::FacialGesture::kMouthPucker:
+      return "mouthPucker";
+    case accessibility_private::FacialGesture::kMouthRight:
+      return "mouthRight";
+    case accessibility_private::FacialGesture::kMouthSmile:
+      return "mouthSmile";
+    case accessibility_private::FacialGesture::kMouthUpperUp:
+      return "mouthUpperUp";
+    case accessibility_private::FacialGesture::kNone:
+      NOTREACHED_NORETURN();
+  }
+}
+
+void DispatchAccessibilityFocusChangedEvent(
+    extensions::events::HistogramValue histogram_value,
+    extensions::api::accessibility_private::ScreenRect bounds) {
+  std::unique_ptr<extensions::Event> event;
+
+  if (histogram_value ==
+      extensions::events::ACCESSIBILITY_PRIVATE_ON_CHROMEVOX_FOCUS_CHANGED) {
+    auto event_args =
+        extensions::api::accessibility_private::OnChromeVoxFocusChanged::Create(
+            bounds);
+    event = std::make_unique<extensions::Event>(
+        histogram_value,
+        extensions::api::accessibility_private::OnChromeVoxFocusChanged::
+            kEventName,
+        std::move(event_args));
+  } else if (histogram_value ==
+             extensions::events::
+                 ACCESSIBILITY_PRIVATE_ON_SELECT_TO_SPEAK_FOCUS_CHANGED) {
+    auto event_args = extensions::api::accessibility_private::
+        OnSelectToSpeakFocusChanged::Create(bounds);
+    event = std::make_unique<extensions::Event>(
+        histogram_value,
+        extensions::api::accessibility_private::OnSelectToSpeakFocusChanged::
+            kEventName,
+        std::move(event_args));
+  }
+
+  extensions::EventRouter::Get(AccessibilityManager::Get()->profile())
+      ->DispatchEventWithLazyListener(
+          extension_misc::kAccessibilityCommonExtensionId, std::move(event));
+}
+
 }  // namespace
 
 ExtensionFunction::ResponseAction
@@ -335,6 +425,30 @@ void AccessibilityPrivateInstallFaceGazeAssetsFunction::OnInstallFinished(
   Respond(WithArguments(assets->ToValue()));
 }
 
+ExtensionFunction::ResponseAction
+AccessibilityPrivateSendGestureInfoToSettingsFunction::Run() {
+  std::optional<accessibility_private::SendGestureInfoToSettings::Params>
+      params(accessibility_private::SendGestureInfoToSettings::Params::Create(
+          args()));
+  EXTENSION_FUNCTION_VALIDATE(params);
+
+  std::vector<ash::FaceGazeGestureInfo> converted_gesture_info_array;
+
+  for (const accessibility_private::GestureInfo& gesture_info :
+       params->gesture_info) {
+    auto converted_gesture_info = ash::FaceGazeGestureInfo();
+    converted_gesture_info.confidence = gesture_info.confidence;
+    converted_gesture_info.gesture =
+        ConvertFacialGestureType(gesture_info.gesture);
+
+    converted_gesture_info_array.push_back(converted_gesture_info);
+  }
+
+  AccessibilityManager::Get()->SendGestureInfoToSettings(
+      std::move(converted_gesture_info_array));
+  return RespondNow(NoArguments());
+}
+
 ExtensionFunction::ResponseAction
 AccessibilityPrivateInstallPumpkinForDictationFunction::Run() {
   AccessibilityManager::Get()->InstallPumpkinForDictation(
@@ -363,10 +477,6 @@ AccessibilityPrivateIsFeatureEnabledFunction::Run() {
   accessibility_private::AccessibilityFeature params_feature = params->feature;
   bool enabled;
   switch (params_feature) {
-    case accessibility_private::AccessibilityFeature::kGoogleTtsLanguagePacks:
-      enabled = ::features::
-          IsExperimentalAccessibilityGoogleTtsLanguagePacksEnabled();
-      break;
     case accessibility_private::AccessibilityFeature::
         kGoogleTtsHighQualityVoices:
       enabled = ::features::
@@ -379,6 +489,9 @@ AccessibilityPrivateIsFeatureEnabledFunction::Run() {
     case accessibility_private::AccessibilityFeature::kFaceGaze:
       enabled = ::features::IsAccessibilityFaceGazeEnabled();
       break;
+    case accessibility_private::AccessibilityFeature::kFaceGazeGravityWells:
+      enabled = ::features::IsAccessibilityFaceGazeGravityWellsEnabled();
+      break;
     case accessibility_private::AccessibilityFeature::kNone:
       return RespondNow(Error("Unrecognized feature"));
   }
@@ -440,7 +553,7 @@ AccessibilityPrivatePerformAcceleratorActionFunction::Run() {
       accelerator_action = ash::AcceleratorAction::kFocusNextPane;
       break;
     case accessibility_private::AcceleratorAction::kNone:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
       return RespondNow(Error("Invalid accelerator action."));
   }
 
@@ -477,8 +590,8 @@ AccessibilityPrivateSendSyntheticKeyEventFunction::Run() {
   ui::KeyEvent synthetic_key_event(
       key_data->type ==
               accessibility_private::SyntheticKeyboardEventType::kKeyup
-          ? ui::ET_KEY_RELEASED
-          : ui::ET_KEY_PRESSED,
+          ? ui::EventType::kKeyReleased
+          : ui::EventType::kKeyPressed,
       keyboard_code, ui::UsLayoutKeyboardCodeToDomCode(keyboard_code),
       modifiers);
 
@@ -489,10 +602,12 @@ AccessibilityPrivateSendSyntheticKeyEventFunction::Run() {
   bool dictation_enabled = AccessibilityManager::Get()->IsDictationEnabled();
   bool from_accessibility_common =
       extension_id() == extension_misc::kAccessibilityCommonExtensionId;
-  if (dictation_enabled && from_accessibility_common &&
+  bool facegaze_enabled = AccessibilityManager::Get()->IsFaceGazeEnabled();
+  if ((dictation_enabled || facegaze_enabled) && from_accessibility_common &&
       params->use_rewriters.has_value() && params->use_rewriters.value()) {
-    // TODO(b/259397131): Remove the `useRewriters` property and remove this
-    // if statement.
+    // TODO(b/259397131): Remove the `useRewriters` property.
+    // Call SendEventToSink so that the event can be processed by event
+    // rewriters, like Game Controls.
     host->SendEventToSink(&synthetic_key_event);
   } else {
     host->DeliverEventToSink(&synthetic_key_event);
@@ -508,32 +623,32 @@ AccessibilityPrivateSendSyntheticMouseEventFunction::Run() {
   EXTENSION_FUNCTION_VALIDATE(params);
   accessibility_private::SyntheticMouseEvent* mouse_data = &params->mouse_event;
 
-  ui::EventType type = ui::ET_UNKNOWN;
+  ui::EventType type = ui::EventType::kUnknown;
   switch (mouse_data->type) {
     case accessibility_private::SyntheticMouseEventType::kPress:
-      type = ui::ET_MOUSE_PRESSED;
+      type = ui::EventType::kMousePressed;
       break;
     case accessibility_private::SyntheticMouseEventType::kRelease:
-      type = ui::ET_MOUSE_RELEASED;
+      type = ui::EventType::kMouseReleased;
       break;
     case accessibility_private::SyntheticMouseEventType::kDrag:
-      type = ui::ET_MOUSE_DRAGGED;
+      type = ui::EventType::kMouseDragged;
       break;
     case accessibility_private::SyntheticMouseEventType::kMove:
-      type = ui::ET_MOUSE_MOVED;
+      type = ui::EventType::kMouseMoved;
       break;
     case accessibility_private::SyntheticMouseEventType::kEnter:
-      type = ui::ET_MOUSE_ENTERED;
+      type = ui::EventType::kMouseEntered;
       break;
     case accessibility_private::SyntheticMouseEventType::kExit:
-      type = ui::ET_MOUSE_EXITED;
+      type = ui::EventType::kMouseExited;
       break;
     default:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
   }
 
   int flags = 0;
-  if (type != ui::ET_MOUSE_MOVED) {
+  if (type != ui::EventType::kMouseMoved) {
     switch (mouse_data->mouse_button) {
       case accessibility_private::SyntheticMouseEventButton::kLeft:
         flags |= ui::EF_LEFT_MOUSE_BUTTON;
@@ -562,46 +677,26 @@ AccessibilityPrivateSendSyntheticMouseEventFunction::Run() {
     flags |= ui::EF_TOUCH_ACCESSIBILITY;
   }
 
+  if (mouse_data->is_double_click) {
+    flags |= ui::EF_IS_DOUBLE_CLICK;
+  }
+
   // Locations are assumed to be in screen coordinates.
   gfx::Point location_in_screen(mouse_data->x, mouse_data->y);
-  const display::Display display =
-      display::Screen::GetScreen()->GetDisplayNearestPoint(location_in_screen);
-  auto* host = ash::GetWindowTreeHostForDisplay(display.id());
-  if (!host) {
-    return RespondNow(NoArguments());
-  }
+  AccessibilityManager::Get()->SendSyntheticMouseEvent(
+      type, flags, changed_button_flags, location_in_screen);
 
-  aura::Window* root_window = host->window();
-  if (!root_window) {
-    return RespondNow(NoArguments());
-  }
-
-  aura::client::CursorClient* cursor_client =
-      aura::client::GetCursorClient(root_window);
-
-  bool is_mouse_events_enabled = cursor_client->IsMouseEventsEnabled();
-  if (!is_mouse_events_enabled) {
-    cursor_client->EnableMouseEvents();
-  }
-
-  ::wm::ConvertPointFromScreen(root_window, &location_in_screen);
-
-  ui::MouseEvent synthetic_mouse_event(
-      type, location_in_screen, location_in_screen, ui::EventTimeForNow(),
-      flags, changed_button_flags);
-
-  // Transforming the coordinate to the root will apply the screen scale factor
-  // to the event's location and also the screen rotation degree.
-  synthetic_mouse_event.UpdateForRootTransform(
-      host->GetRootTransform(),
-      host->GetRootTransformForLocalEventCoordinates());
-  // This skips rewriters.
-  host->DeliverEventToSink(&synthetic_mouse_event);
-
-  if (!is_mouse_events_enabled) {
-    cursor_client->DisableMouseEvents();
-  }
+  return RespondNow(NoArguments());
+}
 
+ExtensionFunction::ResponseAction
+AccessibilityPrivateScrollAtPointFunction::Run() {
+  std::optional<accessibility_private::ScrollAtPoint::Params> params =
+      accessibility_private::ScrollAtPoint::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
+  gfx::Point point(params->target.x, params->target.y);
+  ash::AccessibilityController::Get()->ScrollAtPoint(
+      point, ConvertScrollDirection(params->direction));
   return RespondNow(NoArguments());
 }
 
@@ -681,7 +776,7 @@ AccessibilityPrivateSetFocusRingsFunction::Run() {
         focus_ring->type = ash::FocusRingType::GLOW;
         break;
       default:
-        NOTREACHED();
+        NOTREACHED_IN_MIGRATION();
     }
 
     if (focus_ring_info.stacking_order !=
@@ -698,7 +793,7 @@ AccessibilityPrivateSetFocusRingsFunction::Run() {
               ash::FocusRingStackingOrder::BELOW_ACCESSIBILITY_BUBBLES;
           break;
         default:
-          NOTREACHED();
+          NOTREACHED_IN_MIGRATION();
       }
     }
 
@@ -746,34 +841,63 @@ AccessibilityPrivateSetHighlightsFunction::Run() {
   return RespondNow(NoArguments());
 }
 
+// TODO(b/347953090): Investigate a generic SetAccessibilityFocusFunction to
+// share code for extensibility.
+ExtensionFunction::ResponseAction
+AccessibilityPrivateSetChromeVoxFocusFunction::Run() {
+  std::optional<accessibility_private::SetChromeVoxFocus::Params> params(
+      accessibility_private::SetChromeVoxFocus::Params::Create(args()));
+  EXTENSION_FUNCTION_VALIDATE(params);
+
+  if (!features::IsAccessibilityMagnifierFollowsChromeVoxEnabled()) {
+    return RespondNow(NoArguments());
+  }
+
+  if (!ash::AccessibilityController::Get()->fullscreen_magnifier().enabled() &&
+      !ash::AccessibilityController::Get()->docked_magnifier().enabled()) {
+    return RespondNow(NoArguments());
+  }
+
+  // Ship this event to AccessibilityCommon for docked or fullscreen
+  // magnifier.
+  extensions::api::accessibility_private::ScreenRect bounds;
+  bounds.left = params->bounds.left;
+  bounds.top = params->bounds.top;
+  bounds.width = params->bounds.width;
+  bounds.height = params->bounds.height;
+  DispatchAccessibilityFocusChangedEvent(
+      extensions::events::ACCESSIBILITY_PRIVATE_ON_CHROMEVOX_FOCUS_CHANGED,
+      std::move(bounds));
+
+  return RespondNow(NoArguments());
+}
+
 ExtensionFunction::ResponseAction
 AccessibilityPrivateSetSelectToSpeakFocusFunction::Run() {
   std::optional<accessibility_private::SetSelectToSpeakFocus::Params> params(
       accessibility_private::SetSelectToSpeakFocus::Params::Create(args()));
   EXTENSION_FUNCTION_VALIDATE(params);
 
-  if (features::IsAccessibilityMagnifierFollowsStsEnabled() &&
-      ash::AccessibilityController::Get()->fullscreen_magnifier().enabled()) {
-    // Ship this event to AccessibilityCommon for fullscreen magnifier.
-    auto bounds =
-        std::make_unique<extensions::api::accessibility_private::ScreenRect>();
-    bounds->left = params->bounds.left;
-    bounds->top = params->bounds.top;
-    bounds->width = params->bounds.width;
-    bounds->height = params->bounds.height;
-    auto event_args = extensions::api::accessibility_private::
-        OnSelectToSpeakFocusChanged::Create(params->bounds);
-    auto event = std::make_unique<extensions::Event>(
-        extensions::events::
-            ACCESSIBILITY_PRIVATE_ON_SELECT_TO_SPEAK_FOCUS_CHANGED,
-        extensions::api::accessibility_private::OnSelectToSpeakFocusChanged::
-            kEventName,
-        std::move(event_args));
-    extensions::EventRouter::Get(AccessibilityManager::Get()->profile())
-        ->DispatchEventWithLazyListener(
-            extension_misc::kAccessibilityCommonExtensionId, std::move(event));
+  if (!features::IsAccessibilityMagnifierFollowsStsEnabled()) {
+    return RespondNow(NoArguments());
   }
 
+  if (!ash::AccessibilityController::Get()->fullscreen_magnifier().enabled() &&
+      !ash::AccessibilityController::Get()->docked_magnifier().enabled()) {
+    return RespondNow(NoArguments());
+  }
+
+  // Ship this event to AccessibilityCommon for docked or fullscreen
+  // magnifier.
+  extensions::api::accessibility_private::ScreenRect bounds;
+  bounds.left = params->bounds.left;
+  bounds.top = params->bounds.top;
+  bounds.width = params->bounds.width;
+  bounds.height = params->bounds.height;
+  DispatchAccessibilityFocusChangedEvent(
+      extensions::events::ACCESSIBILITY_PRIVATE_ON_CHROMEVOX_FOCUS_CHANGED,
+      std::move(bounds));
+
   return RespondNow(NoArguments());
 }
 
@@ -932,11 +1056,12 @@ AccessibilityPrivateShowConfirmationDialogFunction::Run() {
 
   std::u16string title = base::UTF8ToUTF16(params->title);
   std::u16string description = base::UTF8ToUTF16(params->description);
+  std::u16string confirm = l10n_util::GetStringUTF16(IDS_APP_CONTINUE);
   std::u16string cancel_name =
       params->cancel_name ? base::UTF8ToUTF16(params->cancel_name.value())
                           : l10n_util::GetStringUTF16(IDS_APP_CANCEL);
   ash::AccessibilityController::Get()->ShowConfirmationDialog(
-      title, description, cancel_name,
+      title, description, confirm, cancel_name,
       base::BindOnce(
           &AccessibilityPrivateShowConfirmationDialogFunction::OnDialogResult,
           this, /* confirmed */ true),
@@ -972,7 +1097,7 @@ AccessibilityPrivateToggleDictationFunction::Run() {
              extension_misc::kAccessibilityCommonExtensionId) {
     source = ash::DictationToggleSource::kAccessibilityCommon;
   } else {
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
   }
 
   ash::AccessibilityController::Get()->ToggleDictationFromSource(source);
@@ -1004,7 +1129,7 @@ AccessibilityPrivateUpdateDictationBubbleFunction::Run() {
       icon = ash::DictationBubbleIconType::kMacroFail;
       break;
     case accessibility_private::DictationBubbleIconType::kNone:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
       break;
   }
 
