@@ -3,6 +3,7 @@
 // found in the LICENSE file.
 
 #include "build/build_config.h"
+#include "chrome/browser/interstitials/security_interstitial_page_test_utils.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/test/base/in_process_browser_test.h"
@@ -13,6 +14,7 @@
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
+#include "content/public/test/scoped_accessibility_mode_override.h"
 #include "content/public/test/test_navigation_observer.h"
 #include "net/test/embedded_test_server/embedded_test_server.h"
 #include "net/test/embedded_test_server/request_handler_util.h"
@@ -37,17 +39,6 @@ class InterstitialAccessibilityBrowserTest : public InProcessBrowserTest {
  protected:
   net::EmbeddedTestServer https_server_mismatched_;
 
-  bool IsShowingInterstitial(content::WebContents* tab) {
-    security_interstitials::SecurityInterstitialTabHelper* helper =
-        security_interstitials::SecurityInterstitialTabHelper::FromWebContents(
-            tab);
-    if (!helper) {
-      return false;
-    }
-    return helper->GetBlockingPageForCurrentlyCommittedNavigationForTesting() !=
-           nullptr;
-  }
-
   void ProceedThroughInterstitial(content::WebContents* web_contents) {
     content::TestNavigationObserver nav_observer(web_contents, 1);
     std::string javascript = "window.certificateErrorPageController.proceed();";
@@ -59,8 +50,7 @@ class InterstitialAccessibilityBrowserTest : public InProcessBrowserTest {
 
 // TODO(crbug.com/1453221): flakily times out on ChromeOS MSAN and Lacros ASAN
 // builders. Deflake and re-enable.
-#if (defined(MEMORY_SANITIZER) && BUILDFLAG(IS_CHROMEOS)) || \
-    (defined(ADDRESS_SANITIZER) && BUILDFLAG(IS_CHROMEOS_LACROS))
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_CHROMEOS_LACROS)
 #define MAYBE_TestSSLInterstitialAccessibility \
   DISABLED_TestSSLInterstitialAccessibility
 #else
@@ -72,7 +62,8 @@ IN_PROC_BROWSER_TEST_F(InterstitialAccessibilityBrowserTest,
 
   content::WebContents* web_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
-  content::EnableAccessibilityForWebContents(web_contents);
+  content::ScopedAccessibilityModeOverride scoped_accessibility_mode(
+      web_contents, ui::kAXModeComplete);
 
   ASSERT_TRUE(https_server_mismatched_.Start());
 
@@ -82,7 +73,8 @@ IN_PROC_BROWSER_TEST_F(InterstitialAccessibilityBrowserTest,
 
   // Ensure that we got an interstitial page.
   ASSERT_FALSE(web_contents->IsCrashed());
-  EXPECT_TRUE(IsShowingInterstitial(web_contents));
+  EXPECT_TRUE(
+      chrome_browser_interstitials::IsShowingInterstitial(web_contents));
 
   // Now check from the perspective of accessibility - we should be focused
   // on a page with title "Privacy error". Keep waiting on accessibility
