@@ -10,6 +10,8 @@
 #include "base/command_line.h"
 #include "base/containers/contains.h"
 #include "base/functional/bind.h"
+#include "base/not_fatal_until.h"
+#include "base/strings/strcat.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
@@ -32,6 +34,7 @@
 #include "chrome/browser/ui/tab_contents/tab_contents_iterator.h"
 #include "chrome/browser/web_applications/web_app.h"
 #include "chrome/browser/web_applications/web_app_provider.h"
+#include "chrome/browser/web_applications/web_app_registrar.h"
 #include "chrome/browser/web_applications/web_app_tab_helper.h"
 #include "chrome/browser/web_applications/web_app_utils.h"
 #include "chrome/common/chrome_switches.h"
@@ -56,6 +59,12 @@
 #include "ui/gfx/switches.h"
 #include "ui/views/controls/webview/webview.h"
 
+#if BUILDFLAG(IS_CHROMEOS)
+#include "chromeos/constants/chromeos_features.h"
+#include "chromeos/constants/pref_names.h"
+#include "components/prefs/pref_service.h"
+#endif
+
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "ash/constants/ash_switches.h"
 #endif
@@ -69,6 +78,37 @@ const char ChromeDevToolsManagerDelegate::kTypePage[] = "page";
 
 namespace {
 
+std::optional<std::string> GetIsolatedWebAppNameAndVersion(
+    content::WebContents* web_contents) {
+  const webapps::AppId* app_id =
+      web_app::WebAppTabHelper::GetAppId(web_contents);
+  if (!app_id) {
+    return std::nullopt;
+  }
+  const web_app::WebAppProvider* provider =
+      web_app::WebAppProvider::GetForWebContents(web_contents);
+  if (!provider) {
+    return std::nullopt;
+  }
+  // In this case we will not modify any data and reading stale data is
+  // fine, since the app will already be installed and open in the case
+  // it needs to be checked in DevTools.
+  const web_app::WebAppRegistrar& registrar = provider->registrar_unsafe();
+  const web_app::WebApp* web_app = registrar.GetAppById(*app_id);
+
+  if (web_app && registrar.IsIsolated(*app_id)) {
+    // Version is a key part of IWA so should be displayed in inspect tool
+    return base::StrCat({registrar.GetAppShortName(*app_id), " (",
+                         web_app->isolation_data()->version.GetString(), ")"});
+  }
+
+  return std::nullopt;
+}
+
+bool IsIsolatedWebApp(content::WebContents* web_contents) {
+  return GetIsolatedWebAppNameAndVersion(web_contents).has_value();
+}
+
 bool GetExtensionInfo(content::WebContents* wc,
                       std::string* name,
                       std::string* type) {
@@ -111,6 +151,27 @@ bool GetExtensionInfo(content::WebContents* wc,
   return true;
 }
 
+policy::DeveloperToolsPolicyHandler::Availability GetDevToolsAvailability(
+    Profile* profile) {
+  using Availability = policy::DeveloperToolsPolicyHandler::Availability;
+  Availability availability =
+      policy::DeveloperToolsPolicyHandler::GetEffectiveAvailability(profile);
+#if BUILDFLAG(IS_CHROMEOS)
+  // On ChromeOS disable dev tools for captive portal signin windows to prevent
+  // them from being used for general navigation.
+  if (chromeos::features::IsCaptivePortalPopupWindowEnabled() &&
+      availability != Availability::kDisallowed) {
+    const PrefService::Preference* const captive_portal_pref =
+        profile->GetPrefs()->FindPreference(
+            chromeos::prefs::kCaptivePortalSignin);
+    if (captive_portal_pref && captive_portal_pref->GetValue()->GetBool()) {
+      availability = Availability::kDisallowed;
+    }
+  }
+#endif
+  return availability;
+}
+
 ChromeDevToolsManagerDelegate* g_instance;
 
 }  // namespace
@@ -167,6 +228,28 @@ void ChromeDevToolsManagerDelegate::Inspect(
                                      DevToolsOpenedByAction::kInspectLink);
 }
 
+void ChromeDevToolsManagerDelegate::Activate(
+    content::DevToolsAgentHost* agent_host) {
+  auto* web_contents = agent_host->GetWebContents();
+  if (!web_contents) {
+    return;
+  }
+
+  // Brings the tab to foreground. We need to do this in case the devtools
+  // window is undocked and this is being called from another tab that is in
+  // the foreground.
+  web_contents->GetDelegate()->ActivateContents(web_contents);
+
+  // Brings a undocked devtools window to the foreground.
+  DevToolsWindow* devtools_window =
+      DevToolsWindow::GetInstanceForInspectedWebContents(
+          agent_host->GetWebContents());
+  if (!devtools_window) {
+    return;
+  }
+  devtools_window->ActivateWindow();
+}
+
 void ChromeDevToolsManagerDelegate::HandleCommand(
     content::DevToolsAgentHostClientChannel* channel,
     base::span<const uint8_t> message,
@@ -176,7 +259,7 @@ void ChromeDevToolsManagerDelegate::HandleCommand(
     std::move(callback).Run(message);
     // This should not happen, but happens. NOTREACHED tries to get
     // a repro in some test.
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return;
   }
   it->second->HandleCommand(message, std::move(callback));
@@ -184,6 +267,10 @@ void ChromeDevToolsManagerDelegate::HandleCommand(
 
 std::string ChromeDevToolsManagerDelegate::GetTargetType(
     content::WebContents* web_contents) {
+  if (IsIsolatedWebApp(web_contents)) {
+    return ChromeDevToolsManagerDelegate::kTypeApp;
+  }
+
   if (base::Contains(AllTabContentses(), web_contents))
     return DevToolsAgentHost::kTypePage;
 
@@ -202,10 +289,16 @@ std::string ChromeDevToolsManagerDelegate::GetTargetType(
 
 std::string ChromeDevToolsManagerDelegate::GetTargetTitle(
     content::WebContents* web_contents) {
+  if (auto iwa_name_version = GetIsolatedWebAppNameAndVersion(web_contents)) {
+    return *iwa_name_version;
+  }
+
   std::string extension_name;
   std::string extension_type;
-  if (!GetExtensionInfo(web_contents, &extension_name, &extension_type))
+  if (!GetExtensionInfo(web_contents, &extension_name, &extension_type)) {
     return std::string();
+  }
+
   return extension_name;
 }
 
@@ -269,8 +362,14 @@ bool ChromeDevToolsManagerDelegate::AllowInspection(
     Profile* profile,
     const extensions::Extension* extension) {
   using Availability = policy::DeveloperToolsPolicyHandler::Availability;
-  Availability availability =
-      policy::DeveloperToolsPolicyHandler::GetEffectiveAvailability(profile);
+  Availability availability;
+  if (extension) {
+    availability =
+        policy::DeveloperToolsPolicyHandler::GetEffectiveAvailability(profile);
+  } else {
+    // Perform additional checks for browser windows (extension == null).
+    availability = GetDevToolsAvailability(profile);
+  }
   switch (availability) {
     case Availability::kDisallowed:
       return false;
@@ -291,7 +390,7 @@ bool ChromeDevToolsManagerDelegate::AllowInspection(
       }
       return true;
     default:
-      NOTREACHED() << "Unknown developer tools policy";
+      NOTREACHED_IN_MIGRATION() << "Unknown developer tools policy";
       return true;
   }
 }
@@ -308,10 +407,19 @@ bool ChromeDevToolsManagerDelegate::AllowInspection(
       return false;
     case Availability::kAllowed:
       return true;
-    case Availability::kDisallowedForForceInstalledExtensions:
-      return !web_app || !web_app->IsKioskInstalledApp();
+    case Availability::kDisallowedForForceInstalledExtensions: {
+      if (!web_app) {
+        return true;
+      }
+      // DevTools should be blocked for Kiosk apps and policy-installed IWAs.
+      if (web_app->IsKioskInstalledApp() ||
+          web_app->IsIwaPolicyInstalledApp()) {
+        return false;
+      }
+      return true;
+    }
     default:
-      NOTREACHED() << "Unknown developer tools policy";
+      NOTREACHED_IN_MIGRATION() << "Unknown developer tools policy";
       return true;
   }
 }
@@ -396,7 +504,7 @@ void ChromeDevToolsManagerDelegate::UpdateDeviceDiscovery() {
     auto it1 = remote_locations.begin();
     auto it2 = remote_locations_.begin();
     while (it1 != remote_locations.end()) {
-      DCHECK(it2 != remote_locations_.end());
+      CHECK(it2 != remote_locations_.end(), base::NotFatalUntil::M130);
       if (!(*it1).Equals(*it2))
         equals = false;
       ++it1;
