@@ -9,7 +9,9 @@
 
 #include "base/containers/flat_map.h"
 #include "base/containers/span.h"
+#include "base/memory/raw_ptr.h"
 #include "build/chromeos_buildflags.h"
+#include "chrome/browser/devtools/protocol/extensions_handler.h"
 #include "chrome/browser/devtools/protocol/protocol.h"
 #include "chrome/browser/devtools/protocol/storage_handler.h"
 #include "content/public/browser/devtools_manager_delegate.h"
@@ -23,6 +25,7 @@ class EmulationHandler;
 class BrowserHandler;
 class CastHandler;
 class PageHandler;
+class PWAHandler;
 class SecurityHandler;
 class StorageHandler;
 class SystemInfoHandler;
@@ -62,10 +65,12 @@ class ChromeDevToolsSession : public protocol::FrontendChannel {
 
   protocol::UberDispatcher dispatcher_;
   std::unique_ptr<AutofillHandler> autofill_handler_;
+  std::unique_ptr<ExtensionsHandler> extensions_handler_;
   std::unique_ptr<BrowserHandler> browser_handler_;
   std::unique_ptr<CastHandler> cast_handler_;
   std::unique_ptr<EmulationHandler> emulation_handler_;
   std::unique_ptr<PageHandler> page_handler_;
+  std::unique_ptr<PWAHandler> pwa_handler_;
   std::unique_ptr<SecurityHandler> security_handler_;
   std::unique_ptr<StorageHandler> storage_handler_;
   std::unique_ptr<SystemInfoHandler> system_info_handler_;
@@ -73,7 +78,7 @@ class ChromeDevToolsSession : public protocol::FrontendChannel {
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   std::unique_ptr<WindowManagerHandler> window_manager_handler_;
 #endif
-  content::DevToolsAgentHostClientChannel* client_channel_;
+  raw_ptr<content::DevToolsAgentHostClientChannel> client_channel_;
 };
 
 #endif  // CHROME_BROWSER_DEVTOOLS_CHROME_DEVTOOLS_SESSION_H_
