@@ -3,6 +3,7 @@
 // found in the LICENSE file.
 
 #include <string>
+#include <string_view>
 
 #include "base/base64.h"
 #include "base/check_deref.h"
@@ -13,6 +14,8 @@
 #include "base/path_service.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
+#include "base/strings/utf_string_conversions.h"
+#include "base/test/bind.h"
 #include "base/test/test_switches.h"
 #include "base/test/values_test_util.h"
 #include "base/threading/thread_restrictions.h"
@@ -26,8 +29,7 @@
 #include "chrome/browser/data_saver/data_saver.h"
 #include "chrome/browser/devtools/devtools_window.h"
 #include "chrome/browser/devtools/protocol/devtools_protocol_test_support.h"
-#include "chrome/browser/dips/dips_service.h"
-#include "chrome/browser/dips/dips_storage.h"
+#include "chrome/browser/dips/dips_test_utils.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/extension_util.h"
 #include "chrome/browser/extensions/unpacked_installer.h"
@@ -68,6 +70,7 @@
 #include "net/ssl/ssl_connection_status_flags.h"
 #include "net/ssl/ssl_server_config.h"
 #include "printing/buildflags/buildflags.h"
+#include "services/network/public/cpp/network_switches.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "third_party/blink/public/common/features.h"
 #include "third_party/boringssl/src/include/openssl/ssl.h"
@@ -147,6 +150,44 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest,
   // Should not crash by this point.
 }
 
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest,
+                       CreateBrowserContextAcceptsProxyServer) {
+  AttachToBrowserTarget();
+  embedded_test_server()->RegisterRequestHandler(base::BindLambdaForTesting(
+      [&](const net::test_server::HttpRequest& request)
+          -> std::unique_ptr<net::test_server::HttpResponse> {
+        std::unique_ptr<net::test_server::BasicHttpResponse> http_response(
+            std::make_unique<net::test_server::BasicHttpResponse>());
+        http_response->set_code(net::HTTP_OK);
+        http_response->set_content_type("text/html");
+        http_response->set_content("<title>Hello from proxy server!</title>");
+        return std::move(http_response);
+      }));
+  ASSERT_TRUE(embedded_test_server()->Start());
+
+  base::Value::Dict params;
+  params.Set("proxyServer",
+             embedded_test_server()->host_port_pair().ToString());
+  const base::Value::Dict* result =
+      SendCommandSync("Target.createBrowserContext", std::move(params));
+  std::string context_id = *result->FindString("browserContextId");
+
+  content::WebContentsAddedObserver observer;
+
+  params = base::Value::Dict();
+  params.Set("url", "http://this-page-does-not-exist.com/site.html");
+  params.Set("browserContextId", context_id);
+  result = SendCommandSync("Target.createTarget", std::move(params));
+
+  content::WebContents* wc = observer.GetWebContents();
+  ASSERT_TRUE(content::WaitForLoadStop(wc));
+
+  EXPECT_EQ(GURL("http://this-page-does-not-exist.com/site.html"),
+            wc->GetURL());
+
+  EXPECT_EQ("Hello from proxy server!", base::UTF16ToUTF8(wc->GetTitle()));
+}
+
 IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, CreateInDefaultContextById) {
   AttachToBrowserTarget();
   const base::Value::Dict* result = SendCommandSync("Target.getTargets");
@@ -386,10 +427,11 @@ IN_PROC_BROWSER_TEST_F(
   GURL url("invalid.scheme:for-sure");
   ui_test_utils::AllBrowserTabAddedWaiter tab_added_waiter;
 
-  content::WebContents* web_contents =
-      browser()->OpenURL(content::OpenURLParams(
-          url, content::Referrer(), WindowOpenDisposition::NEW_FOREGROUND_TAB,
-          ui::PAGE_TRANSITION_TYPED, false));
+  content::WebContents* web_contents = browser()->OpenURL(
+      content::OpenURLParams(url, content::Referrer(),
+                             WindowOpenDisposition::NEW_FOREGROUND_TAB,
+                             ui::PAGE_TRANSITION_TYPED, false),
+      /*navigation_handle_callback=*/{});
   tab_added_waiter.Wait();
   ASSERT_TRUE(WaitForLoadStop(web_contents));
 
@@ -484,13 +526,20 @@ class DevToolsProtocolTest_BounceTrackingMitigations
     : public DevToolsProtocolTest {
  protected:
   void SetUp() override {
-    scoped_feature_list_.InitAndEnableFeatureWithParameters(
-        features::kDIPS,
-        {{"delete", "true"}, {"triggering_action", "stateful_bounce"}});
+    scoped_feature_list_.InitWithFeaturesAndParameters(
+        /*enabled_features=*/{{features::kDIPS,
+                               {{"delete", "true"},
+                                {"triggering_action", "stateful_bounce"}}}},
+        /*disabled_features=*/{});
 
     DevToolsProtocolTest::SetUp();
   }
 
+  void SetUpOnMainThread() override {
+    host_resolver()->AddRule("*", "127.0.0.1");
+    DevToolsProtocolTest::SetUpOnMainThread();
+  }
+
   void SetBlockThirdPartyCookies(bool value) {
     browser()->profile()->GetPrefs()->SetInteger(
         prefs::kCookieControlsMode,
@@ -514,15 +563,11 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest_BounceTrackingMitigations,
   const GURL bouncer(
       embedded_test_server()->GetURL("example.test", "/title1.html"));
 
-  // Get DIPS Service
-  DIPSService* dips_service = DIPSService::Get(browser()->profile());
-  dips_service->WaitForInitCompleteForTesting();
-
   // Record a stateful bounce for `bouncer`.
-  dips_service->storage()
-      ->AsyncCall(&DIPSStorage::RecordBounce)
-      .WithArgs(bouncer, base::Time::Now(), true);
-  dips_service->storage()->FlushPostedTasksForTesting();
+  ASSERT_TRUE(SimulateDipsBounce(
+      web_contents(), embedded_test_server()->GetURL("a.test", "/empty.html"),
+      bouncer, embedded_test_server()->GetURL("b.test", "/empty.html"),
+      embedded_test_server()->GetURL("c.test", "/empty.html")));
 
   SendCommandSync("Storage.runBounceTrackingMitigations");
 
@@ -811,7 +856,7 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, VisibleSecurityStateSecureState) {
     ASSERT_TRUE(base::Base64Decode(cert.GetString(), &decoded));
     der_certs.push_back(decoded);
   }
-  std::vector<base::StringPiece> cert_string_piece;
+  std::vector<std::string_view> cert_string_piece;
   for (const auto& str : der_certs) {
     cert_string_piece.push_back(str);
   }
@@ -871,7 +916,6 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest,
 }
 
 IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, UntrustedClient) {
-  std::unique_ptr<base::Value::Dict> params(new base::Value::Dict());
   SetIsTrusted(false);
   Attach();
   EXPECT_FALSE(SendCommandSync("HeapProfiler.enable"));  // Implemented in V8
@@ -879,8 +923,6 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, UntrustedClient) {
   EXPECT_FALSE(SendCommandSync(
       "Memory.prepareForLeakDetection"));        // Implemented in content
   EXPECT_FALSE(SendCommandSync("Cast.enable"));  // Implemented in content
-  EXPECT_FALSE(SendCommandSync("Storage.getCookies"));
-  EXPECT_FALSE(SendCommandSync("Network.getAllCookies"));
   EXPECT_TRUE(SendCommandSync("Accessibility.enable"));
 }
 
@@ -952,9 +994,9 @@ class ExtensionProtocolTest : public DevToolsProtocolTest {
   }
 
  private:
-  extensions::ExtensionService* extension_service_;
-  extensions::ExtensionRegistry* extension_registry_;
-  content::WebContents* background_web_contents_;
+  raw_ptr<extensions::ExtensionService, DanglingUntriaged> extension_service_;
+  raw_ptr<extensions::ExtensionRegistry, DanglingUntriaged> extension_registry_;
+  raw_ptr<content::WebContents, DanglingUntriaged> background_web_contents_;
 #if BUILDFLAG(IS_WIN)
   // This is needed to stop ExtensionProtocolTestsfrom creating a
   // shortcut in the Windows start menu. The override needs to last until the
@@ -1200,8 +1242,9 @@ IN_PROC_BROWSER_TEST_F(PrerenderDataSaverProtocolTest,
   prerender_helper()->AddPrerenderAsync(prerendering_url);
   observer.WaitForTrigger(prerendering_url);
 
-  int host_id = prerender_helper()->GetHostForUrl(prerendering_url);
-  EXPECT_EQ(host_id, content::RenderFrameHost::kNoFrameTreeNodeId);
+  content::FrameTreeNodeId host_id =
+      prerender_helper()->GetHostForUrl(prerendering_url);
+  EXPECT_TRUE(host_id.is_null());
 
   SendCommandAsync("Preload.enable");
   const base::Value::Dict result =
@@ -1255,4 +1298,55 @@ IN_PROC_BROWSER_TEST_F(PrivacySandboxAttestationsOverrideTest,
           net::SchemefulSite(GURL(attestation_url))));
 }
 
+class DevToolsProtocolTest_RelatedWebsiteSets : public DevToolsProtocolTest {
+ protected:
+  const char* kPrimarySite = "https://a.test";
+  const char* kAssociatedSite = "https://b.test";
+  const char* kServiceSite = "https://c.test";
+  const char* kPrimaryCcTLD = "https://a.cctld";
+
+  void SetUpCommandLine(base::CommandLine* command_line) override {
+    DevToolsProtocolTest::SetUpCommandLine(command_line);
+    command_line->AppendSwitchASCII(
+        network::switches::kUseRelatedWebsiteSet,
+        base::StringPrintf(R"({"primary": "%s",)"
+                           R"("associatedSites": ["%s"],)"
+                           R"("serviceSites": ["%s"],)"
+                           R"("ccTLDs": {"%s": ["%s"]}})",
+                           kPrimarySite, kAssociatedSite, kServiceSite,
+                           kPrimarySite, kPrimaryCcTLD));
+  }
+};
+
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest_RelatedWebsiteSets,
+                       GetRelatedWebsiteSets) {
+  ASSERT_TRUE(embedded_test_server()->Start());
+  const GURL url(embedded_test_server()->GetURL("/empty.html"));
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
+  Attach();
+
+  SendCommandSync("Storage.getRelatedWebsiteSets");
+
+  if (result()) {
+    const base::Value::List* set_list = result()->FindList("sets");
+    ASSERT_TRUE(set_list);
+
+    base::Value::List expected =
+        base::Value::List()  //
+            .Append(base::Value::Dict()
+                        .Set("associatedSites",
+                             base::Value::List().Append(kAssociatedSite))
+                        .Set("primarySites", base::Value::List()
+                                                 .Append(kPrimaryCcTLD)
+                                                 .Append(kPrimarySite))
+                        .Set("serviceSites",
+                             base::Value::List().Append(kServiceSite)));
+
+    EXPECT_EQ(*set_list, expected);
+  } else if (error()) {
+    EXPECT_EQ(*error()->FindString("message"),
+              "Failed fetching RelatedWebsiteSets");
+  }
+}
+
 }  // namespace
