@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include <stdint.h>
 
 #include <memory>
@@ -12,6 +17,8 @@
 #include "base/test/gmock_callback_support.h"
 #include "base/test/gmock_move_support.h"
 #include "chrome/browser/extensions/extension_apitest.h"
+#include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/browser.h"
 #include "content/public/test/browser_test.h"
 #include "device/bluetooth/test/mock_bluetooth_adapter.h"
 #include "device/bluetooth/test/mock_bluetooth_device.h"
@@ -1166,10 +1173,10 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GattConnection) {
   EXPECT_CALL(*mock_adapter_, GetDevice(kTestLeDeviceAddress1))
       .WillRepeatedly(Return(device1_.get()));
   static_assert(
-      BluetoothDevice::NUM_CONNECT_ERROR_CODES == 14,
+      BluetoothDevice::NUM_CONNECT_ERROR_CODES == 21,
       "Update required if the number of BluetoothDevice enums changes.");
   EXPECT_CALL(*device0_, CreateGattConnection(_, _))
-      .Times(9)
+      .Times(13)
       .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
                                    BluetoothDevice::ERROR_FAILED))
       .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
@@ -1184,6 +1191,14 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GattConnection) {
                                    BluetoothDevice::ERROR_AUTH_TIMEOUT))
       .WillOnce(RunOnceCallback<0>(
           /*connection=*/nullptr, BluetoothDevice::ERROR_UNSUPPORTED_DEVICE))
+      .WillOnce(RunOnceCallback<0>(
+          /*connection=*/nullptr, BluetoothDevice::ERROR_NO_MEMORY))
+      .WillOnce(RunOnceCallback<0>(
+          /*connection=*/nullptr, BluetoothDevice::ERROR_JNI_ENVIRONMENT))
+      .WillOnce(RunOnceCallback<0>(
+          /*connection=*/nullptr, BluetoothDevice::ERROR_JNI_THREAD_ATTACH))
+      .WillOnce(RunOnceCallback<0>(
+          /*connection=*/nullptr, BluetoothDevice::ERROR_WAKELOCK))
       .WillOnce(RunOnceCallback<0>(
           CreateGattConnection(mock_adapter_, kTestLeDeviceAddress0,
                                /*expect_disconnect=*/true),
