@@ -1,4 +1,4 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,32 +10,36 @@
 #include <string>
 #include <vector>
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/check.h"
+#include "base/check_op.h"
 #include "base/containers/span.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/hash/sha1.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/path_service.h"
 #include "base/run_loop.h"
+#include "base/scoped_observation.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/bind.h"
+#include "base/test/test_future.h"
 #include "base/values.h"
-#include "chrome/browser/browser_process.h"
-#include "chrome/browser/ash/certificate_provider/certificate_provider.h"
-#include "chrome/browser/ash/certificate_provider/certificate_provider_service.h"
-#include "chrome/browser/ash/certificate_provider/certificate_provider_service_factory.h"
-#include "chrome/browser/ash/certificate_provider/test_certificate_provider_extension.h"
-#include "chrome/browser/chromeos/ui/request_pin_view.h"
+#include "chrome/browser/certificate_provider/certificate_provider.h"
+#include "chrome/browser/certificate_provider/certificate_provider_service.h"
+#include "chrome/browser/certificate_provider/certificate_provider_service_factory.h"
+#include "chrome/browser/certificate_provider/test_certificate_provider_extension.h"
 #include "chrome/browser/extensions/api/certificate_provider/certificate_provider_api.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/browser/extensions/extension_service.h"
-#include "chrome/browser/net/system_network_context_manager.h"
+#include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
+#include "chrome/browser/ui/views/notifications/request_pin_view_chromeos.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/policy/core/browser/browser_policy_connector.h"
@@ -45,19 +49,24 @@
 #include "components/policy/policy_constants.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/web_contents.h"
+#include "content/public/common/isolated_world_ids.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/test_navigation_observer.h"
 #include "content/public/test/test_utils.h"
 #include "crypto/rsa_private_key.h"
+#include "extensions/browser/api/test/test_api_observer.h"
+#include "extensions/browser/api/test/test_api_observer_registry.h"
 #include "extensions/browser/disable_reason.h"
+#include "extensions/browser/extension_host.h"
+#include "extensions/browser/extension_host_test_helper.h"
 #include "extensions/browser/extension_registry.h"
 #include "extensions/browser/extension_system.h"
 #include "extensions/browser/process_manager.h"
 #include "extensions/browser/test_extension_registry_observer.h"
 #include "extensions/common/extension.h"
+#include "extensions/common/mojom/view_type.mojom.h"
 #include "extensions/test/extension_test_message_listener.h"
-#include "extensions/test/test_background_page_first_load_observer.h"
 #include "net/cert/x509_certificate.h"
 #include "net/http/http_status_code.h"
 #include "net/ssl/client_cert_identity.h"
@@ -80,19 +89,15 @@
 #include "ui/views/widget/widget.h"
 #include "ui/views/widget/widget_observer.h"
 
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+#include "chrome/browser/lacros/cert/cert_db_initializer_factory.h"
+#endif
+
 using testing::Return;
 using testing::_;
 
 namespace {
 
-void StoreDigest(std::vector<uint8_t>* digest,
-                 base::OnceClosure callback,
-                 base::Value value) {
-  ASSERT_TRUE(value.is_blob()) << "Unexpected value in StoreDigest";
-  digest->assign(value.GetBlob().begin(), value.GetBlob().end());
-  std::move(callback).Run();
-}
-
 bool RsaSignRawData(uint16_t openssl_signature_algorithm,
                     const std::vector<uint8_t>& input,
                     crypto::RSAPrivateKey* key,
@@ -157,29 +162,45 @@ std::string JsUint8Array(const std::vector<uint8_t>& bytes) {
 }
 
 std::string GetPageTextContent(content::WebContents* web_contents) {
-  std::string text_content;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      web_contents->GetMainFrame(),
-      "domAutomationController.send(document.body.textContent);",
-      &text_content));
-  return text_content;
+  return content::EvalJs(web_contents->GetPrimaryMainFrame(),
+                         "document.body.textContent;")
+      .ExtractString();
+  ;
 }
 
 std::string GetCertFingerprint1(const net::X509Certificate& cert) {
-  unsigned char hash[base::kSHA1Length];
-  base::SHA1HashBytes(CRYPTO_BUFFER_data(cert.cert_buffer()),
-                      CRYPTO_BUFFER_len(cert.cert_buffer()), hash);
-  return base::ToLowerASCII(base::HexEncode(hash, base::kSHA1Length));
+  return base::ToLowerASCII(base::HexEncode(base::SHA1Hash(cert.cert_span())));
 }
 
+// Generates a gtest failure whenever extension JS reports failure.
+class JsFailureObserver : public extensions::TestApiObserver {
+ public:
+  JsFailureObserver() {
+    test_api_observation_.Observe(
+        extensions::TestApiObserverRegistry::GetInstance());
+  }
+  ~JsFailureObserver() override = default;
+
+  void OnTestFailed(content::BrowserContext* browser_context,
+                    const std::string& message) override {
+    ADD_FAILURE() << "Received failure notification from the JS side: "
+                  << message;
+  }
+
+ private:
+  base::ScopedObservation<extensions::TestApiObserverRegistry,
+                          extensions::TestApiObserver>
+      test_api_observation_{this};
+};
+
 class CertificateProviderApiTest : public extensions::ExtensionApiTest {
  public:
   CertificateProviderApiTest() {}
 
   void SetUpInProcessBrowserTestFixture() override {
-    ON_CALL(provider_, IsInitializationComplete(_)).WillByDefault(Return(true));
-    ON_CALL(provider_, IsFirstPolicyLoadComplete(_))
-        .WillByDefault(Return(true));
+    provider_.SetDefaultReturns(
+        /*is_initialization_complete_return=*/true,
+        /*is_first_policy_load_complete_return=*/true);
     policy::BrowserPolicyConnector::SetPolicyProviderForTesting(&provider_);
 
     extensions::ExtensionApiTest::SetUpInProcessBrowserTestFixture();
@@ -187,17 +208,22 @@ class CertificateProviderApiTest : public extensions::ExtensionApiTest {
 
   void SetUpOnMainThread() override {
     extensions::ExtensionApiTest::SetUpOnMainThread();
+
+    // Observe all assertion failures in the JS code, even those that happen
+    // when there's no active `ResultCatcher`.
+    js_failure_observer_ = std::make_unique<JsFailureObserver>();
+
     // Set up the AutoSelectCertificateForUrls policy to avoid the client
     // certificate selection dialog.
     const std::string autoselect_pattern = R"({"pattern": "*", "filter": {}})";
 
-    base::Value autoselect_policy(base::Value::Type::LIST);
+    base::Value::List autoselect_policy;
     autoselect_policy.Append(autoselect_pattern);
 
     policy_map_.Set(policy::key::kAutoSelectCertificateForUrls,
                     policy::POLICY_LEVEL_MANDATORY, policy::POLICY_SCOPE_USER,
-                    policy::POLICY_SOURCE_CLOUD, std::move(autoselect_policy),
-                    nullptr);
+                    policy::POLICY_SOURCE_CLOUD,
+                    base::Value(std::move(autoselect_policy)), nullptr);
     provider_.UpdateChromePolicy(policy_map_);
 
     content::RunAllPendingInMessageLoop();
@@ -244,17 +270,17 @@ class CertificateProviderApiTest : public extensions::ExtensionApiTest {
 
   std::vector<scoped_refptr<net::X509Certificate>>
   GetAllProvidedCertificates() {
-    base::RunLoop run_loop;
     std::unique_ptr<chromeos::CertificateProvider> cert_provider =
         cert_provider_service_->CreateCertificateProvider();
+
+    base::test::TestFuture<net::ClientCertIdentityList> get_certificates_future;
+    cert_provider->GetCertificates(get_certificates_future.GetCallback());
+
     std::vector<scoped_refptr<net::X509Certificate>> all_provided_certificates;
-    auto callback = base::BindLambdaForTesting(
-        [&](net::ClientCertIdentityList cert_identity_list) {
-          for (const auto& cert_identity : cert_identity_list)
-            all_provided_certificates.push_back(cert_identity->certificate());
-        });
-    cert_provider->GetCertificates(callback.Then(run_loop.QuitClosure()));
-    run_loop.Run();
+    for (const auto& cert_identity : get_certificates_future.Get()) {
+      all_provided_certificates.push_back(cert_identity->certificate());
+    }
+
     return all_provided_certificates;
   }
 
@@ -264,7 +290,8 @@ class CertificateProviderApiTest : public extensions::ExtensionApiTest {
 
  protected:
   testing::NiceMock<policy::MockConfigurationPolicyProvider> provider_;
-  chromeos::CertificateProviderService* cert_provider_service_ = nullptr;
+  raw_ptr<chromeos::CertificateProviderService, AcrossTasksDanglingUntriaged>
+      cert_provider_service_ = nullptr;
   policy::PolicyMap policy_map_;
 
  private:
@@ -285,6 +312,7 @@ class CertificateProviderApiTest : public extensions::ExtensionApiTest {
     return response;
   }
 
+  std::unique_ptr<JsFailureObserver> js_failure_observer_;
   std::unique_ptr<net::EmbeddedTestServer> https_server_;
 };
 
@@ -293,13 +321,22 @@ class CertificateProviderApiTest : public extensions::ExtensionApiTest {
 class CertificateProviderApiMockedExtensionTest
     : public CertificateProviderApiTest {
  public:
+  void SetUpInProcessBrowserTestFixture() override {
+#if BUILDFLAG(IS_CHROMEOS_LACROS)  // Needed for ClientCertStoreLacros
+    CertDbInitializerFactory::GetInstance()
+        ->SetCreateWithBrowserContextForTesting(
+            /*should_create=*/true);
+#endif
+    CertificateProviderApiTest::SetUpInProcessBrowserTestFixture();
+  }
+
   void SetUpOnMainThread() override {
     CertificateProviderApiTest::SetUpOnMainThread();
 
     extension_path_ = test_data_dir_.AppendASCII("certificate_provider");
     extension_ = LoadExtension(extension_path_);
-    ui_test_utils::NavigateToURL(browser(),
-                                 extension_->GetResourceURL("basic.html"));
+    ASSERT_TRUE(ui_test_utils::NavigateToURL(
+        browser(), extension_->GetResourceURL("basic.html")));
 
     extension_contents_ = browser()->tab_strip_model()->GetActiveWebContents();
 
@@ -310,21 +347,18 @@ class CertificateProviderApiMockedExtensionTest
   }
 
   content::RenderFrameHost* GetExtensionMainFrame() const {
-    return extension_contents_->GetMainFrame();
+    return extension_contents_->GetPrimaryMainFrame();
   }
 
   void ExecuteJavascript(const std::string& function) const {
-    ASSERT_TRUE(content::ExecuteScript(GetExtensionMainFrame(), function));
+    ASSERT_TRUE(content::ExecJs(GetExtensionMainFrame(), function));
   }
 
-  // Calls |function| in the extension. |function| needs to return a bool. If
-  // that happens at the end of a callback, this will wait for the callback to
-  // complete.
+  // Calls |function| in the extension. |function| needs to return a bool or a
+  // Promise<bool>. If it returns a Promise<bool>, this waits for the promise to
+  // resolve.
   void ExecuteJavascriptAndWaitForCallback(const std::string& function) const {
-    bool success = false;
-    ASSERT_TRUE(content::ExecuteScriptAndExtractBool(GetExtensionMainFrame(),
-                                                     function, &success));
-    ASSERT_TRUE(success);
+    ASSERT_EQ(true, content::EvalJs(GetExtensionMainFrame(), function));
   }
 
   const extensions::Extension* extension() const { return extension_; }
@@ -341,8 +375,8 @@ class CertificateProviderApiMockedExtensionTest
   // chrome/test/data/extensions/api_test/certificate_provider
   scoped_refptr<net::X509Certificate> GetCertificate() const {
     std::string raw_certificate = GetCertificateData();
-    return net::X509Certificate::CreateFromBytes(raw_certificate.data(),
-                                                 raw_certificate.size());
+    return net::X509Certificate::CreateFromBytes(
+        base::as_bytes(base::make_span(raw_certificate)));
   }
 
   // Tests the api by navigating to a webpage that requests to perform a
@@ -358,39 +392,38 @@ class CertificateProviderApiMockedExtensionTest
         nullptr /* no WebContents */);
     navigation_observer.StartWatchingNewWebContents();
     ExtensionTestMessageListener sign_digest_listener(
-        "signature request received", /*will_reply=*/false);
+        "signature request received");
 
     // Navigate to a page which triggers a sign request. Navigation is blocked
     // by completion of this request, so we don't wait for navigation to finish.
     ui_test_utils::NavigateToURLWithDisposition(
         browser(), GetHttpsClientCertUrl(),
         WindowOpenDisposition::NEW_FOREGROUND_TAB,
-        ui_test_utils::BROWSER_TEST_NONE);
+        ui_test_utils::BROWSER_TEST_NO_WAIT);
 
     content::WebContents* const https_contents =
         browser()->tab_strip_model()->GetActiveWebContents();
 
     // Wait for the extension to receive the sign request.
     ASSERT_TRUE(sign_digest_listener.WaitUntilSatisfied());
+    EXPECT_GT(cert_provider_service_->pin_dialog_manager()
+                  ->StoredSignRequestsForTesting(),
+              0);
 
     // Check that the certificate is available.
     scoped_refptr<net::X509Certificate> certificate = GetCertificate();
     CheckCertificateProvidedByExtension(*certificate, *extension());
 
     // Fetch the data from the sign request.
-    const std::string request_algorithm =
-        ExecuteScriptAndGetValue(GetExtensionMainFrame(),
-                                 "signatureRequestAlgorithm;")
-            .GetString();
-    EXPECT_EQ(expected_request_signature_algorithm, request_algorithm);
-    std::vector<uint8_t> request_data;
-    {
-      base::RunLoop run_loop;
-      GetExtensionMainFrame()->ExecuteJavaScriptForTests(
-          base::ASCIIToUTF16("signatureRequestData;"),
-          base::BindOnce(&StoreDigest, &request_data, run_loop.QuitClosure()));
-      run_loop.Run();
-    }
+    EXPECT_EQ(
+        expected_request_signature_algorithm,
+        content::EvalJs(GetExtensionMainFrame(), "signatureRequestAlgorithm;"));
+
+    base::test::TestFuture<base::Value> exec_js_future;
+    GetExtensionMainFrame()->ExecuteJavaScriptForTests(
+        u"signatureRequestData;", exec_js_future.GetCallback(),
+        content::ISOLATED_WORLD_ID_GLOBAL);
+    std::vector<uint8_t> request_data(exec_js_future.Get().GetBlob());
 
     // Load the private key.
     std::string key_pk8 = GetKeyPk8();
@@ -415,6 +448,11 @@ class CertificateProviderApiMockedExtensionTest
     // Wait for the https navigation to finish.
     navigation_observer.Wait();
 
+    // Make sure that sign request is removed from pin dialog manager.
+    EXPECT_EQ(cert_provider_service_->pin_dialog_manager()
+                  ->StoredSignRequestsForTesting(),
+              0);
+
     // Check whether the server acknowledged that a client certificate was
     // presented.
     const std::string client_cert_fingerprint =
@@ -423,6 +461,26 @@ class CertificateProviderApiMockedExtensionTest
               "got client cert with fingerprint: " + client_cert_fingerprint);
   }
 
+  void SetInterstitialBypass() {
+    // Navigate to the test server in a new tab (to not clobber the test
+    // fixture setup.
+    ui_test_utils::NavigateToURLWithDisposition(
+        browser(), GetHttpsClientCertUrl(),
+        WindowOpenDisposition::NEW_FOREGROUND_TAB,
+        ui_test_utils::BROWSER_TEST_WAIT_FOR_LOAD_STOP);
+    auto* tab = browser()->tab_strip_model()->GetActiveWebContents();
+
+    // Proceed through the interstitial to set an SSL bypass for this host.
+    content::TestNavigationObserver nav_observer(tab,
+                                                 /*number_of_navigations=*/1);
+    ASSERT_TRUE(content::ExecJs(
+        tab, "window.certificateErrorPageController.proceed();"));
+    nav_observer.Wait();
+
+    // Close the new tab to go back to the state set up by SetUpOnMainThread().
+    tab->Close();
+  }
+
  private:
   std::string GetCertificateData() const {
     const base::FilePath certificate_path =
@@ -434,8 +492,10 @@ class CertificateProviderApiMockedExtensionTest
     return certificate_data;
   }
 
-  content::WebContents* extension_contents_ = nullptr;
-  const extensions::Extension* extension_ = nullptr;
+  raw_ptr<content::WebContents, AcrossTasksDanglingUntriaged>
+      extension_contents_ = nullptr;
+  raw_ptr<const extensions::Extension, AcrossTasksDanglingUntriaged>
+      extension_ = nullptr;
   base::FilePath extension_path_;
 };
 
@@ -449,7 +509,7 @@ class CertificateProviderRequestPinTest : public CertificateProviderApiTest {
   void SetUpOnMainThread() override {
     CertificateProviderApiTest::SetUpOnMainThread();
     command_request_listener_ = std::make_unique<ExtensionTestMessageListener>(
-        "GetCommand", /*will_reply=*/true);
+        "GetCommand", ReplyBehavior::kWillReply);
     LoadRequestPinExtension();
   }
 
@@ -470,11 +530,11 @@ class CertificateProviderRequestPinTest : public CertificateProviderApiTest {
   }
 
   void NavigateTo(const std::string& test_page_file_name) {
-    ui_test_utils::NavigateToURL(
-        browser(), extension_->GetResourceURL(test_page_file_name));
+    ASSERT_TRUE(ui_test_utils::NavigateToURL(
+        browser(), extension_->GetResourceURL(test_page_file_name)));
   }
 
-  chromeos::RequestPinView* GetActivePinDialogView() {
+  RequestPinView* GetActivePinDialogView() {
     return cert_provider_service_->pin_dialog_manager()
         ->default_dialog_host_for_testing()
         ->active_view_for_testing();
@@ -499,7 +559,7 @@ class CertificateProviderRequestPinTest : public CertificateProviderApiTest {
   // is expected to send "Success" message after the validation and request to
   // stopPinRequest is done.
   void EnterCorrectPinAndWaitForMessage() {
-    ExtensionTestMessageListener listener("Success", false);
+    ExtensionTestMessageListener listener("Success");
     EnterCode(kCorrectPin);
     ASSERT_TRUE(listener.WaitUntilSatisfied());
   }
@@ -509,14 +569,13 @@ class CertificateProviderRequestPinTest : public CertificateProviderApiTest {
   // extension code is expected to send "Invalid PIN" message after the
   // validation and the new requestPin (with the error) is done.
   void EnterWrongPinAndWaitForMessage() {
-    ExtensionTestMessageListener listener("Invalid PIN", false);
+    ExtensionTestMessageListener listener("Invalid PIN");
     EnterCode(kWrongPin);
     ASSERT_TRUE(listener.WaitUntilSatisfied());
 
     // Check that we have an error message displayed.
-    EXPECT_EQ(
-        gfx::kGoogleRed600,
-        GetActivePinDialogView()->error_label_for_testing()->GetEnabledColor());
+    EXPECT_TRUE(
+        GetActivePinDialogView()->IsTextStyleOfErrorLabelCorrectForTesting());
   }
 
   bool SendCommand(const std::string& command) {
@@ -529,8 +588,7 @@ class CertificateProviderRequestPinTest : public CertificateProviderApiTest {
 
   bool SendCommandAndWaitForMessage(const std::string& command,
                                     const std::string& expected_message) {
-    ExtensionTestMessageListener listener(expected_message,
-                                          /*will_reply=*/false);
+    ExtensionTestMessageListener listener(expected_message);
     if (!SendCommand(command))
       return false;
     return listener.WaitUntilSatisfied();
@@ -543,7 +601,8 @@ class CertificateProviderRequestPinTest : public CertificateProviderApiTest {
     extension_ = LoadExtension(extension_path);
   }
 
-  const extensions::Extension* extension_ = nullptr;
+  raw_ptr<const extensions::Extension, AcrossTasksDanglingUntriaged>
+      extension_ = nullptr;
   std::unique_ptr<ExtensionTestMessageListener> command_request_listener_;
 };
 
@@ -586,10 +645,8 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
                                                /*is_raw_data=*/false);
 
   // Replying to the signature request a second time must fail.
-  bool success = true;
-  ASSERT_TRUE(content::ExecuteScriptAndExtractBool(
-      GetExtensionMainFrame(), "replyWithSignatureSecondTime();", &success));
-  ASSERT_FALSE(success);
+  ASSERT_EQ(false, content::EvalJs(GetExtensionMainFrame(),
+                                   "replyWithSignatureSecondTime();"));
 }
 
 // Tests an extension that provides certificates both proactively with
@@ -684,59 +741,6 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
   EXPECT_TRUE(GetAllProvidedCertificates().empty());
 }
 
-// Tests the RSA MD5/SHA-1 signature algorithm. Note that TLS 1.1 is used in
-// order to make this algorithm employed.
-// TODO(cthomp): The SSLVersionMin policy will be removed in M-91, making these
-// algorithms unsupported.
-IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest, RsaMd5Sha1) {
-  // This test requires the SSLVersionMin policy is set to allow TLS 1.0.
-  base::Value ssl_policy("tls1");  // TLS 1.0
-  policy_map_.Set(policy::key::kSSLVersionMin, policy::POLICY_LEVEL_MANDATORY,
-                  policy::POLICY_SCOPE_MACHINE, policy::POLICY_SOURCE_CLOUD,
-                  std::move(ssl_policy), nullptr);
-  EXPECT_NO_FATAL_FAILURE(provider_.UpdateChromePolicy(policy_map_));
-
-  // Wait for the updated SSL configuration to be sent to the network service,
-  // to avoid a race.
-  g_browser_process->system_network_context_manager()
-      ->FlushSSLConfigManagerForTesting();
-
-  ASSERT_TRUE(StartHttpsServer(net::SSL_PROTOCOL_VERSION_TLS1_1));
-  ExecuteJavascript("supportedAlgorithms = ['RSASSA_PKCS1_v1_5_MD5_SHA1'];");
-  ExecuteJavascript("registerForSignatureRequests();");
-  ExecuteJavascriptAndWaitForCallback("setCertificates();");
-  TestNavigationToCertificateRequestingWebPage("RSASSA_PKCS1_v1_5_MD5_SHA1",
-                                               SSL_SIGN_RSA_PKCS1_MD5_SHA1,
-                                               /*is_raw_data=*/true);
-}
-
-// Tests the RSA MD5/SHA-1 signature algorithm using the legacy version of the
-// API. Note that TLS 1.1 is used in order to make this algorithm employed.
-// TODO(cthomp): The SSLVersionMin policy will be removed in M-91, making these
-// algorithms unsupported.
-IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
-                       LegacyRsaMd5Sha1) {
-  // This test requires the SSLVersionMin policy is set to allow TLS 1.0.
-  base::Value ssl_policy("tls1");  // TLS 1.0
-  policy_map_.Set(policy::key::kSSLVersionMin, policy::POLICY_LEVEL_MANDATORY,
-                  policy::POLICY_SCOPE_MACHINE, policy::POLICY_SOURCE_CLOUD,
-                  std::move(ssl_policy), nullptr);
-  EXPECT_NO_FATAL_FAILURE(provider_.UpdateChromePolicy(policy_map_));
-
-  // Wait for the updated SSL configuration to be sent to the network service,
-  // to avoid a race.
-  g_browser_process->system_network_context_manager()
-      ->FlushSSLConfigManagerForTesting();
-
-  ASSERT_TRUE(StartHttpsServer(net::SSL_PROTOCOL_VERSION_TLS1_1));
-  ExecuteJavascript("supportedLegacyHashes = ['MD5_SHA1'];");
-  ExecuteJavascript("registerAsLegacyCertificateProvider();");
-  ExecuteJavascript("registerForLegacySignatureRequests();");
-  TestNavigationToCertificateRequestingWebPage("MD5_SHA1",
-                                               SSL_SIGN_RSA_PKCS1_MD5_SHA1,
-                                               /*is_raw_data=*/false);
-}
-
 // Tests the RSA SHA-1 signature algorithm.
 IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest, RsaSha1) {
   ASSERT_TRUE(StartHttpsServer(net::SSL_PROTOCOL_VERSION_TLS1_2));
@@ -846,36 +850,6 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
                                                /*is_raw_data=*/true);
 }
 
-// Tests that the RSA MD5/SHA-1 signature algorithm is used in case of TLS 1.1,
-// even when there are other algorithms specified (which are stronger but aren't
-// supported on TLS 1.1).
-// TODO(cthomp): The SSLVersionMin policy will be removed in M-91, making these
-// algorithms unsupported.
-IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
-                       RsaMd5Sha1AndOthers) {
-  // This test requires the SSLVersionMin policy is set to allow TLS 1.0.
-  base::Value ssl_policy("tls1");  // TLS 1.0
-  policy_map_.Set(policy::key::kSSLVersionMin, policy::POLICY_LEVEL_MANDATORY,
-                  policy::POLICY_SCOPE_MACHINE, policy::POLICY_SOURCE_CLOUD,
-                  std::move(ssl_policy), nullptr);
-  EXPECT_NO_FATAL_FAILURE(provider_.UpdateChromePolicy(policy_map_));
-
-  // Wait for the updated SSL configuration to be sent to the network service,
-  // to avoid a race.
-  g_browser_process->system_network_context_manager()
-      ->FlushSSLConfigManagerForTesting();
-
-  ASSERT_TRUE(StartHttpsServer(net::SSL_PROTOCOL_VERSION_TLS1_1));
-  ExecuteJavascript(
-      "supportedAlgorithms = ['RSASSA_PKCS1_v1_5_SHA512', "
-      "'RSASSA_PKCS1_v1_5_SHA1', 'RSASSA_PKCS1_v1_5_MD5_SHA1'];");
-  ExecuteJavascript("registerForSignatureRequests();");
-  ExecuteJavascriptAndWaitForCallback("setCertificates();");
-  TestNavigationToCertificateRequestingWebPage("RSASSA_PKCS1_v1_5_MD5_SHA1",
-                                               SSL_SIGN_RSA_PKCS1_MD5_SHA1,
-                                               /*is_raw_data=*/true);
-}
-
 // Tests the RSA-PSS SHA-256 signature algorithm.
 IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
                        RsaPssSha256) {
@@ -914,32 +888,36 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderApiMockedExtensionTest,
 
 // Test that the certificateProvider events are delivered correctly in the
 // scenario when the event listener is in a lazy background page that gets idle.
-IN_PROC_BROWSER_TEST_F(CertificateProviderApiTest, LazyBackgroundPage) {
+// Disabled due to flakiness - https://crbug.com/1279724
+IN_PROC_BROWSER_TEST_F(CertificateProviderApiTest,
+                       DISABLED_LazyBackgroundPage) {
   ASSERT_TRUE(StartHttpsServer(net::SSL_PROTOCOL_VERSION_TLS1_2));
   // Make extension background pages idle immediately.
   extensions::ProcessManager::SetEventPageIdleTimeForTesting(1);
   extensions::ProcessManager::SetEventPageSuspendingTimeForTesting(1);
 
   // Load the test extension.
-  TestCertificateProviderExtension test_certificate_provider_extension(
+  ash::TestCertificateProviderExtension test_certificate_provider_extension(
       profile());
-  extensions::TestBackgroundPageFirstLoadObserver
-      test_background_page_first_load_observer(
-          profile(), TestCertificateProviderExtension::extension_id());
+  extensions::ExtensionHostTestHelper host_helper(
+      profile(), ash::TestCertificateProviderExtension::extension_id());
+  host_helper.RestrictToType(
+      extensions::mojom::ViewType::kExtensionBackgroundPage);
   const extensions::Extension* const extension =
       LoadExtension(base::PathService::CheckedGet(chrome::DIR_TEST_DATA)
                         .AppendASCII("extensions")
                         .AppendASCII("test_certificate_provider")
                         .AppendASCII("extension"));
   ASSERT_TRUE(extension);
-  EXPECT_EQ(extension->id(), TestCertificateProviderExtension::extension_id());
-  test_background_page_first_load_observer.Wait();
+  EXPECT_EQ(extension->id(),
+            ash::TestCertificateProviderExtension::extension_id());
+  host_helper.WaitForHostCompletedFirstLoad();
 
   // Navigate to the page that requests the client authentication. Use the
   // incognito profile in order to force re-authentication in the later request
   // made by the test.
-  const std::string client_cert_fingerprint =
-      GetCertFingerprint1(*TestCertificateProviderExtension::GetCertificate());
+  const std::string client_cert_fingerprint = GetCertFingerprint1(
+      *ash::TestCertificateProviderExtension::GetCertificate());
   Browser* const incognito_browser = CreateIncognitoBrowser(profile());
   ASSERT_TRUE(incognito_browser);
   ui_test_utils::NavigateToURLWithDisposition(
@@ -951,7 +929,7 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderApiTest, LazyBackgroundPage) {
                 incognito_browser->tab_strip_model()->GetActiveWebContents()),
             "got client cert with fingerprint: " + client_cert_fingerprint);
   CheckCertificateProvidedByExtension(
-      *TestCertificateProviderExtension::GetCertificate(), *extension);
+      *ash::TestCertificateProviderExtension::GetCertificate(), *extension);
 
   // Let the extension's background page become idle.
   WaitForExtensionIdle(extension->id());
@@ -989,18 +967,18 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderRequestPinTest, ShowPinDialogClose) {
   for (int i = 0;
        i < extensions::api::certificate_provider::kMaxClosedDialogsPerMinute;
        i++) {
-    ExtensionTestMessageListener listener("User closed the dialog", false);
+    ExtensionTestMessageListener listener("User closed the dialog");
     GetActivePinDialogWindow()->Close();
     ASSERT_TRUE(listener.WaitUntilSatisfied());
   }
 
-  ExtensionTestMessageListener close_listener("User closed the dialog", true);
+  ExtensionTestMessageListener close_listener("User closed the dialog",
+                                              ReplyBehavior::kWillReply);
   GetActivePinDialogWindow()->Close();
   ASSERT_TRUE(close_listener.WaitUntilSatisfied());
   close_listener.Reply("GetLastError");
   ExtensionTestMessageListener last_error_listener(
-      "This request exceeds the MAX_PIN_DIALOGS_CLOSED_PER_MINUTE quota.",
-      false);
+      "This request exceeds the MAX_PIN_DIALOGS_CLOSED_PER_MINUTE quota.");
   ASSERT_TRUE(last_error_listener.WaitUntilSatisfied());
   EXPECT_FALSE(GetActivePinDialogView());
 }
@@ -1035,7 +1013,7 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderRequestPinTest,
   EXPECT_FALSE(GetActivePinDialogView()->textfield_for_testing()->GetEnabled());
 
   // Close the dialog.
-  ExtensionTestMessageListener listener("No attempt left", false);
+  ExtensionTestMessageListener listener("No attempt left");
   GetActivePinDialogWindow()->Close();
   ASSERT_TRUE(listener.WaitUntilSatisfied());
   EXPECT_FALSE(GetActivePinDialogView());
@@ -1049,7 +1027,7 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderRequestPinTest,
 
   EXPECT_TRUE(SendCommandAndWaitForMessage("Request", "request1:begun"));
   ExtensionTestMessageListener listener(
-      base::StringPrintf("request1:success:%s", kWrongPin), false);
+      base::StringPrintf("request1:success:%s", kWrongPin));
   EnterCode(kWrongPin);
   EXPECT_TRUE(listener.WaitUntilSatisfied());
 
@@ -1164,7 +1142,7 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderRequestPinTest, ZeroAttemptsAtStart) {
   // The textfield has to be disabled, as there are no attempts left.
   EXPECT_FALSE(GetActivePinDialogView()->textfield_for_testing()->GetEnabled());
 
-  ExtensionTestMessageListener listener("request1:empty", false);
+  ExtensionTestMessageListener listener("request1:empty");
   GetActivePinDialogWindow()->Close();
   EXPECT_TRUE(listener.WaitUntilSatisfied());
 }
@@ -1230,7 +1208,7 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderRequestPinTest, StartAfterStop) {
 
   EXPECT_TRUE(SendCommandAndWaitForMessage("Request", "request2:begun"));
   ExtensionTestMessageListener listener(
-      base::StringPrintf("request2:success:%s", kCorrectPin), false);
+      base::StringPrintf("request2:success:%s", kCorrectPin));
   EnterCode(kCorrectPin);
   EXPECT_TRUE(listener.WaitUntilSatisfied());
   EXPECT_FALSE(GetActivePinDialogView()->textfield_for_testing()->GetEnabled());
@@ -1250,7 +1228,7 @@ IN_PROC_BROWSER_TEST_F(CertificateProviderRequestPinTest,
         "Request", base::StringPrintf("request%d:begun", i + 1)));
 
     ExtensionTestMessageListener listener(
-        base::StringPrintf("request%d:empty", i + 1), false);
+        base::StringPrintf("request%d:empty", i + 1));
     ASSERT_TRUE(GetActivePinDialogView());
     GetActivePinDialogView()->GetWidget()->CloseWithReason(
         views::Widget::ClosedReason::kCloseButtonClicked);
