@@ -1,9 +1,10 @@
 // Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
-
 #include "extensions/browser/api/declarative_net_request/action_tracker.h"
 
+#include <string_view>
+
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/test/simple_test_clock.h"
@@ -79,7 +80,7 @@ class ActionTrackerTest : public DNRTestBase {
 
   // Returns renderer-initiated request params for the given |url|.
   WebRequestInfoInitParams GetRequestParamsForURL(
-      base::StringPiece url,
+      std::string_view url,
       WebRequestResourceType web_request_type,
       int tab_id) {
     const int kRendererId = 1;
