@@ -99,8 +99,9 @@ DesktopCaptureChooseDesktopMediaFunction::Run() {
     target_render_frame_host = render_frame_host();
   }
 
-  if (!target_render_frame_host)
+  if (!target_render_frame_host) {
     return RespondNow(Error(kTargetTabRequiredFromServiceWorker));
+  }
 
   const bool exclude_system_audio =
       params->options &&
