@@ -2,72 +2,72 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_refptr.h"
+#include "base/notreached.h"
 #include "base/path_service.h"
 #include "base/strings/stringprintf.h"
+#include "base/test/values_test_util.h"
 #include "build/build_config.h"
 #include "chrome/browser/devtools/devtools_window.h"
 #include "chrome/browser/devtools/devtools_window_testing.h"
 #include "chrome/browser/extensions/api/developer_private/developer_private_api.h"
 #include "chrome/browser/extensions/extension_apitest.h"
-#include "chrome/browser/extensions/extension_function_test_utils.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
+#include "chrome/browser/extensions/manifest_v2_experiment_manager.h"
+#include "chrome/browser/extensions/mv2_experiment_stage.h"
+#include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/common/chrome_paths.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/test/browser_test.h"
+#include "content/public/test/browser_test_utils.h"
 #include "content/public/test/service_worker_test_helpers.h"
+#include "extensions/browser/api_test_utils.h"
 #include "extensions/browser/app_window/app_window.h"
 #include "extensions/browser/app_window/app_window_registry.h"
 #include "extensions/browser/browsertest_util.h"
 #include "extensions/browser/extension_host_test_helper.h"
 #include "extensions/browser/offscreen_document_host.h"
 #include "extensions/browser/process_manager.h"
+#include "extensions/common/extension_features.h"
 #include "extensions/common/manifest_handlers/background_info.h"
 #include "extensions/common/mojom/view_type.mojom.h"
 #include "extensions/test/result_catcher.h"
 #include "extensions/test/test_extension_dir.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "ui/views/widget/any_widget_observer.h"
+#include "ui/views/widget/widget.h"
+#include "ui/views/widget/widget_delegate.h"
+#include "ui/views/window/dialog_delegate.h"
 
 namespace extensions {
 
 class DeveloperPrivateApiTest : public ExtensionApiTest {
  protected:
-  std::unique_ptr<api::developer_private::ExtensionInfo> GetExtensionInfo(
+  std::optional<api::developer_private::ExtensionInfo> GetExtensionInfo(
       const Extension& extension) {
     auto get_info_function =
         base::MakeRefCounted<api::DeveloperPrivateGetExtensionInfoFunction>();
-    std::unique_ptr<base::Value> result =
-        extension_function_test_utils::RunFunctionAndReturnSingleResult(
+    std::optional<base::Value> result =
+        api_test_utils::RunFunctionAndReturnSingleResult(
             get_info_function.get(),
-            content::JsReplace(R"([$1])", extension.id()), browser());
+            content::JsReplace(R"([$1])", extension.id()), profile());
     if (!result) {
       ADD_FAILURE() << "No result back when getting extension info";
-      return nullptr;
+      return std::nullopt;
     }
-    std::unique_ptr<api::developer_private::ExtensionInfo> info =
+    std::optional<api::developer_private::ExtensionInfo> info =
         api::developer_private::ExtensionInfo::FromValue(*result);
-    if (!info)
+    if (!info) {
       ADD_FAILURE() << "Problem creating ExtensionInfo from result data";
+    }
     return info;
   }
 };
 
-IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, Basics) {
-  // Load up some extensions so that we can query their info and adjust their
-  // setings in the API test.
-  base::FilePath base_dir = test_data_dir_.AppendASCII("developer");
-  EXPECT_TRUE(LoadExtension(base_dir.AppendASCII("hosted_app")));
-  EXPECT_TRUE(InstallExtension(base_dir.AppendASCII("packaged_app"), 1,
-                               mojom::ManifestLocation::kInternal));
-  LoadExtension(base_dir.AppendASCII("simple_extension"));
-
-  ASSERT_TRUE(RunExtensionTest("developer/test",
-                               {.launch_as_platform_app = true},
-                               {.load_as_component = true}));
-}
-
 // Tests opening the developer tools for an app window.
 IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectAppWindowView) {
   base::FilePath dir;
@@ -87,7 +87,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectAppWindowView) {
   ASSERT_EQ(2u, info->views.size());
   const api::developer_private::ExtensionView* window_view = nullptr;
   for (const auto& view : info->views) {
-    if (view.type == api::developer_private::VIEW_TYPE_APP_WINDOW) {
+    if (view.type == api::developer_private::ViewType::kAppWindow) {
       window_view = &view;
       break;
     }
@@ -97,15 +97,15 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectAppWindowView) {
   // Inspect the app window.
   auto function =
       base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-  extension_function_test_utils::RunFunction(
+  api_test_utils::RunFunction(
       function.get(),
       base::StringPrintf("[{\"renderViewId\": %d, \"renderProcessId\": %d}]",
                          window_view->render_view_id,
                          window_view->render_process_id),
-      browser(), api_test_utils::NONE);
+      profile());
 
   // Verify that dev tools opened.
-  std::list<AppWindow*> app_windows =
+  std::list<raw_ptr<AppWindow, CtnExperimental>> app_windows =
       AppWindowRegistry::Get(profile())->GetAppWindowsForApp(app->id());
   ASSERT_EQ(1u, app_windows.size());
   EXPECT_TRUE(DevToolsWindow::GetInstanceForInspectedWebContents(
@@ -131,28 +131,40 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectEmbeddedOptionsPage) {
   // The embedded options page should show up.
   ASSERT_EQ(1u, info->views.size());
   const api::developer_private::ExtensionView& view = info->views[0];
-  ASSERT_EQ(api::developer_private::VIEW_TYPE_EXTENSION_GUEST, view.type);
+  ASSERT_EQ(api::developer_private::ViewType::kExtensionGuest, view.type);
 
   // Inspect the embedded options page.
   auto function =
       base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-  extension_function_test_utils::RunFunction(
+  api_test_utils::RunFunction(
       function.get(),
       base::StringPrintf("[{\"renderViewId\": %d, \"renderProcessId\": %d}]",
                          view.render_view_id, view.render_process_id),
-      browser(), api_test_utils::NONE);
+      profile());
 
   // Verify that dev tools opened.
-  content::RenderFrameHost* rfh = content::RenderFrameHost::FromID(
-      view.render_process_id, view.render_view_id);
-  ASSERT_TRUE(rfh);
-  content::WebContents* wc = content::WebContents::FromRenderFrameHost(rfh);
+  content::RenderFrameHost* render_frame_host =
+      content::RenderFrameHost::FromID(view.render_process_id,
+                                       view.render_view_id);
+  ASSERT_TRUE(render_frame_host);
+  content::WebContents* wc =
+      content::WebContents::FromRenderFrameHost(render_frame_host);
   ASSERT_TRUE(wc);
   EXPECT_TRUE(DevToolsWindow::GetInstanceForInspectedWebContents(wc));
 }
 
+// TODO(crbug.com/40273479): Test is flaky on MSan builders.
+// TODO(crbug.com/40282331): Disabled on ASAN due to leak caused by renderer gin
+// objects which are intended to be leaked.
+#if defined(ADDRESS_SANITIZER) || defined(MEMORY_SANITIZER)
+#define MAYBE_InspectInactiveServiceWorkerBackground \
+  DISABLED_InspectInactiveServiceWorkerBackground
+#else
+#define MAYBE_InspectInactiveServiceWorkerBackground \
+  InspectInactiveServiceWorkerBackground
+#endif
 IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
-                       InspectInactiveServiceWorkerBackground) {
+                       MAYBE_InspectInactiveServiceWorkerBackground) {
   ResultCatcher result_catcher;
   // Load an extension that is service worker-based.
   const Extension* extension =
@@ -175,9 +187,8 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   // There should be a worker based background for the extension.
   ASSERT_EQ(1u, info->views.size());
   const api::developer_private::ExtensionView& view = info->views[0];
-  EXPECT_EQ(
-      api::developer_private::VIEW_TYPE_EXTENSION_SERVICE_WORKER_BACKGROUND,
-      view.type);
+  EXPECT_EQ(api::developer_private::ViewType::kExtensionServiceWorkerBackground,
+            view.type);
   // The service worker should be inactive (indicated by -1 for
   // the process id).
   EXPECT_EQ(-1, view.render_process_id);
@@ -186,15 +197,15 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   DevToolsWindowCreationObserver devtools_window_created_observer;
   auto dev_tools_function =
       base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-  extension_function_test_utils::RunFunction(dev_tools_function.get(),
-                                             base::StringPrintf(
-                                                 R"([{"renderViewId": -1,
+  api_test_utils::RunFunction(dev_tools_function.get(),
+                              base::StringPrintf(
+                                  R"([{"renderViewId": -1,
                                                       "renderProcessId": -1,
                                                       "isServiceWorker": true,
                                                       "extensionId": "%s"
                                                    }])",
-                                                 extension->id().c_str()),
-                                             browser(), api_test_utils::NONE);
+                                  extension->id().c_str()),
+                              profile());
   devtools_window_created_observer.WaitForLoad();
 
   // Verify that dev tool window opened.
@@ -232,15 +243,14 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   // There should be a worker based background for the extension.
   ASSERT_EQ(1u, info->views.size());
   const api::developer_private::ExtensionView& view = info->views[0];
-  EXPECT_EQ(
-      api::developer_private::VIEW_TYPE_EXTENSION_SERVICE_WORKER_BACKGROUND,
-      view.type);
+  EXPECT_EQ(api::developer_private::ViewType::kExtensionServiceWorkerBackground,
+            view.type);
   EXPECT_NE(-1, view.render_process_id);
 
   // Inspect the service worker page.
   auto dev_tools_function =
       base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-  extension_function_test_utils::RunFunction(
+  api_test_utils::RunFunction(
       dev_tools_function.get(),
       base::StringPrintf(
           R"([{"renderViewId": -1,
@@ -249,7 +259,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
                "extensionId": "%s"
             }])",
           info->views[0].render_process_id, extension->id().c_str()),
-      browser(), api_test_utils::NONE);
+      profile());
 
   // Find the service worker background host.
   content::DevToolsAgentHost::List targets =
@@ -270,7 +280,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   EXPECT_TRUE(DevToolsWindow::FindDevToolsWindow(service_worker_host.get()));
 }
 
-// TODO(crbug.com/1395713): The test is flaky on MSAN and Linux. Re-enable it.
+// TODO(crbug.com/40882269): The test is flaky on MSAN and Linux. Re-enable it.
 #if defined(MEMORY_SANITIZER) || BUILDFLAG(IS_LINUX)
 #define MAYBE_InspectSplitModeServiceWorkerBackgrounds \
   DISABLED_InspectSplitModeServiceWorkerBackgrounds
@@ -310,7 +320,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   {
     const api::developer_private::ExtensionView& view = info->views[0];
     EXPECT_EQ(
-        api::developer_private::VIEW_TYPE_EXTENSION_SERVICE_WORKER_BACKGROUND,
+        api::developer_private::ViewType::kExtensionServiceWorkerBackground,
         view.type);
     EXPECT_NE(-1, view.render_process_id);
     main_render_process_id = view.render_process_id;
@@ -320,8 +330,8 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   // Now open up an incognito browser window page and check the inspectable
   // views again. Waiting for the result catcher will wait for the incognito
   // service worker to have become active.
-  Browser* inconito_browser = CreateIncognitoBrowser(browser()->profile());
-  ASSERT_TRUE(inconito_browser);
+  Browser* incognito_browser = CreateIncognitoBrowser(browser()->profile());
+  ASSERT_TRUE(incognito_browser);
   ASSERT_TRUE(result_catcher.GetNextResult());
   info = GetExtensionInfo(*extension);
   // The views should now have 2 entries, one for the main worker which will be
@@ -331,7 +341,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
   int incognito_render_process_id = -1;
   for (auto& view : info->views) {
     EXPECT_EQ(
-        api::developer_private::VIEW_TYPE_EXTENSION_SERVICE_WORKER_BACKGROUND,
+        api::developer_private::ViewType::kExtensionServiceWorkerBackground,
         view.type);
     EXPECT_NE(-1, view.render_process_id);
     if (view.incognito) {
@@ -355,11 +365,11 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
     DevToolsWindowCreationObserver devtools_window_created_observer;
     auto dev_tools_function =
         base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-    extension_function_test_utils::RunFunction(
+    api_test_utils::RunFunction(
         dev_tools_function.get(),
         content::JsReplace(kOpenDevToolsParams, main_render_process_id,
                            extension->id().c_str(), /*incognito:*/ false),
-        browser(), api_test_utils::NONE);
+        profile());
     devtools_window_created_observer.WaitForLoad();
     main_devtools_window = devtools_window_created_observer.devtools_window();
   }
@@ -368,11 +378,11 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest,
     DevToolsWindowCreationObserver devtools_window_created_observer;
     auto dev_tools_function =
         base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-    extension_function_test_utils::RunFunction(
+    api_test_utils::RunFunction(
         dev_tools_function.get(),
         content::JsReplace(kOpenDevToolsParams, incognito_render_process_id,
                            extension->id().c_str(), /*incognito:*/ true),
-        browser(), api_test_utils::NONE);
+        profile());
     devtools_window_created_observer.WaitForLoad();
     incognito_devtools_window =
         devtools_window_created_observer.devtools_window();
@@ -423,7 +433,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectOffscreenDocument) {
   // metadata.
   ASSERT_EQ(1u, info->views.size());
   const api::developer_private::ExtensionView& view = info->views[0];
-  EXPECT_EQ(api::developer_private::VIEW_TYPE_OFFSCREEN_DOCUMENT, view.type);
+  EXPECT_EQ(api::developer_private::ViewType::kOffscreenDocument, view.type);
   content::WebContents* offscreen_contents =
       offscreen_document->host_contents();
   EXPECT_EQ(offscreen_url.spec(), view.url);
@@ -441,7 +451,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectOffscreenDocument) {
   // Call the API function to inspect the offscreen document.
   auto dev_tools_function =
       base::MakeRefCounted<api::DeveloperPrivateOpenDevToolsFunction>();
-  extension_function_test_utils::RunFunction(
+  api_test_utils::RunFunction(
       dev_tools_function.get(),
       content::JsReplace(
           R"([{"renderViewId": $1,
@@ -449,7 +459,7 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectOffscreenDocument) {
                "extensionId": $3
             }])",
           view.render_view_id, view.render_process_id, extension->id()),
-      browser(), api_test_utils::NONE);
+      profile());
 
   // Validate that the devtools window is now shown.
   DevToolsWindow* dev_tools_window =
@@ -460,4 +470,170 @@ IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, InspectOffscreenDocument) {
   DevToolsWindowTesting::CloseDevToolsWindowSync(dev_tools_window);
 }
 
+IN_PROC_BROWSER_TEST_F(DeveloperPrivateApiTest, UninstallMultipleExtensions) {
+  // Load first extension.
+  static constexpr char kManifest_0[] =
+      R"({
+           "name": "Multiple extensions uninstall test 0",
+           "manifest_version": 3,
+           "version": "0.1"
+         })";
+  TestExtensionDir test_dir_0;
+  test_dir_0.WriteManifest(kManifest_0);
+  const Extension* extension_0 = LoadExtension(test_dir_0.UnpackedPath());
+  ASSERT_TRUE(extension_0);
+  std::string extension_0_id = extension_0->id();
+
+  // Load second extension.
+  static constexpr char kManifest_1[] =
+      R"({
+           "name": "Multiple extensions uninstall test 1",
+           "manifest_version": 3,
+           "version": "0.1"
+         })";
+  TestExtensionDir test_dir_1;
+  test_dir_1.WriteManifest(kManifest_1);
+  const Extension* extension_1 = LoadExtension(test_dir_1.UnpackedPath());
+  ASSERT_TRUE(extension_1);
+  std::string extension_1_id = extension_1->id();
+
+  auto function = base::MakeRefCounted<
+      api::DeveloperPrivateRemoveMultipleExtensionsFunction>();
+  std::unique_ptr<ExtensionFunctionDispatcher> dispatcher(
+      new ExtensionFunctionDispatcher(profile()));
+  function->SetDispatcher(dispatcher->AsWeakPtr());
+
+  std::string args =
+      base::StrCat({"[[\"", extension_0_id, "\", \"", extension_1_id, "\"]]"});
+  function->SetArgs(base::test::ParseJsonList(args));
+
+  // Create a waiter to wait for the uninstall dialog to show up.
+  views::NamedWidgetShownWaiter waiter(views::test::AnyWidgetTestPasskey{},
+                                       "ExtensionMultipleUninstallDialog");
+  api_test_utils::SendResponseHelper response_helper(function.get());
+
+  function->RunWithValidation().Execute();
+
+  auto* widget = waiter.WaitIfNeededAndGet();
+  widget->widget_delegate()->AsDialogDelegate()->AcceptDialog();
+  response_helper.WaitForResponse();
+
+  // Verify the extensions are uninstalled.
+  EXPECT_FALSE(extension_registry()->GetExtensionById(
+      extension_0_id, ExtensionRegistry::EVERYTHING));
+  EXPECT_FALSE(extension_registry()->GetExtensionById(
+      extension_1_id, ExtensionRegistry::EVERYTHING));
+}
+
+class DeveloperPrivateApiWithMV2DeprecationApiTest
+    : public DeveloperPrivateApiTest,
+      public testing::WithParamInterface<MV2ExperimentStage> {
+ public:
+  DeveloperPrivateApiWithMV2DeprecationApiTest() {
+    experiment_stage_ = GetParam();
+
+    std::vector<base::test::FeatureRef> enabled_features;
+    std::vector<base::test::FeatureRef> disabled_features;
+    switch (experiment_stage_) {
+      case MV2ExperimentStage::kNone:
+        NOTREACHED();
+      case MV2ExperimentStage::kWarning:
+        enabled_features.push_back(
+            extensions_features::kExtensionManifestV2DeprecationWarning);
+        disabled_features.push_back(
+            extensions_features::kExtensionManifestV2Disabled);
+        break;
+      case MV2ExperimentStage::kDisableWithReEnable:
+        enabled_features.push_back(
+            extensions_features::kExtensionManifestV2Disabled);
+        disabled_features.push_back(
+            extensions_features::kExtensionManifestV2DeprecationWarning);
+        break;
+    }
+
+    feature_list_.InitWithFeatures(enabled_features, disabled_features);
+  }
+
+  MV2ExperimentStage experiment_stage() const { return experiment_stage_; }
+
+ private:
+  base::test::ScopedFeatureList feature_list_;
+  MV2ExperimentStage experiment_stage_;
+};
+
+INSTANTIATE_TEST_SUITE_P(
+    ,
+    DeveloperPrivateApiWithMV2DeprecationApiTest,
+    testing::Values(MV2ExperimentStage::kWarning,
+                    MV2ExperimentStage::kDisableWithReEnable),
+    [](const testing::TestParamInfo<MV2ExperimentStage>& info) {
+      switch (info.param) {
+        case MV2ExperimentStage::kNone:
+          NOTREACHED();
+        case MV2ExperimentStage::kWarning:
+          return "WarningExperiment";
+        case MV2ExperimentStage::kDisableWithReEnable:
+          return "DisableExperiment";
+      }
+    });
+
+// Tests that an extension's MV2 deprecation notice is marked as deprecated when
+// the function is called and by accepting the dialog, if necessary.
+// Note: we don't test cancelling the dialog since that's done extensively in
+// unit tests.
+IN_PROC_BROWSER_TEST_P(DeveloperPrivateApiWithMV2DeprecationApiTest,
+                       DismissMv2DeprecationNotice) {
+  // Load MV2 extension.
+  static constexpr char kManifest[] =
+      R"({
+           "name": "MV2 extension",
+           "manifest_version": 2,
+           "version": "0.1"
+         })";
+  TestExtensionDir test_dir;
+  test_dir.WriteManifest(kManifest);
+  const Extension* extension = LoadExtension(test_dir.UnpackedPath());
+  ASSERT_TRUE(extension);
+
+  // Verify extension is affected by the MV2 deprecation and its notice hasn't
+  // been marked as acknowledged.
+  ManifestV2ExperimentManager* experiment_manager =
+      ManifestV2ExperimentManager::Get(profile());
+  EXPECT_TRUE(experiment_manager->IsExtensionAffected(*extension));
+  EXPECT_FALSE(experiment_manager->DidUserAcknowledgeNotice(extension->id()));
+
+  // Create the dismiss notice function.
+  auto dismiss_notice_function = base::MakeRefCounted<
+      api::DeveloperPrivateDismissMv2DeprecationNoticeForExtensionFunction>();
+  std::string args = base::StringPrintf(R"(["%s"])", extension->id().c_str());
+
+  if (experiment_stage() == MV2ExperimentStage::kDisableWithReEnable) {
+    // The function will trigger a dialog for this stage. Add a waiter for the
+    // dialog.
+    views::NamedWidgetShownWaiter waiter(views::test::AnyWidgetTestPasskey{},
+                                         "Mv2DeprecationKeepDialog");
+    api_test_utils::SendResponseHelper response_helper(
+        dismiss_notice_function.get());
+
+    // Add a dispatcher to wait for the response since the function won't return
+    // till the dialog is accepted/canceled.
+    std::unique_ptr<ExtensionFunctionDispatcher> dispatcher(
+        new ExtensionFunctionDispatcher(profile()));
+    dismiss_notice_function->SetDispatcher(dispatcher->AsWeakPtr());
+    dismiss_notice_function->SetArgs(base::test::ParseJsonList(args));
+    dismiss_notice_function->RunWithValidation().Execute();
+
+    // Wait for the dialog and accept it.
+    auto* widget = waiter.WaitIfNeededAndGet();
+    widget->widget_delegate()->AsDialogDelegate()->AcceptDialog();
+    response_helper.WaitForResponse();
+  } else {
+    api_test_utils::RunFunction(dismiss_notice_function.get(), args, profile());
+  }
+
+  // Extension's notice should be marked as acknowledged.
+  EXPECT_TRUE(experiment_manager->IsExtensionAffected(*extension));
+  EXPECT_TRUE(experiment_manager->DidUserAcknowledgeNotice(extension->id()));
+}
+
 }  // namespace extensions
