@@ -22,10 +22,11 @@
 #include "chrome/browser/ash/drive/drivefs_test_support.h"
 #include "chrome/browser/ash/file_manager/volume_manager.h"
 #include "chrome/browser/ash/login/users/fake_chrome_user_manager.h"
-#include "chrome/browser/ash/profiles/profile_helper.h"
 #include "chrome/browser/extensions/api/file_system/consent_provider_impl.h"
 #include "chrome/browser/extensions/component_loader.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/common/chrome_paths.h"
+#include "chromeos/ash/components/browser_context_helper/browser_context_types.h"
 #include "chromeos/components/kiosk/kiosk_test_utils.h"
 #include "components/file_access/scoped_file_access.h"
 #include "components/file_access/test/mock_scoped_file_access_delegate.h"
@@ -37,6 +38,7 @@
 #include "extensions/browser/event_router.h"
 #include "extensions/common/api/file_system.h"
 #include "storage/browser/file_system/external_mount_points.h"
+#include "ui/base/mojom/dialog_button.mojom.h"
 #include "ui/base/ui_base_types.h"
 
 // TODO(michaelpg): Port these tests to app_shell: crbug.com/505926.
@@ -62,7 +64,7 @@ const char kTestingExtensionId[] = "pkplfbidichfdicaijlchgnapepdginl";
 // simulates clicking of the specified dialog button.
 class ScopedSkipRequestFileSystemDialog {
  public:
-  explicit ScopedSkipRequestFileSystemDialog(ui::DialogButton button) {
+  explicit ScopedSkipRequestFileSystemDialog(ui::mojom::DialogButton button) {
     file_system_api::ConsentProviderDelegate::SetAutoDialogButtonForTest(
         button);
   }
@@ -74,7 +76,7 @@ class ScopedSkipRequestFileSystemDialog {
 
   ~ScopedSkipRequestFileSystemDialog() {
     file_system_api::ConsentProviderDelegate::SetAutoDialogButtonForTest(
-        ui::DIALOG_BUTTON_NONE);
+        ui::mojom::DialogButton::kNone);
   }
 };
 
@@ -157,9 +159,8 @@ class FileSystemApiTestForDrive : public PlatformAppBrowserTest {
   drive::DriveIntegrationService* CreateDriveIntegrationService(
       Profile* profile) {
     // Ignore signin and lock screen apps profile.
-    if (profile->GetPath() == ash::ProfileHelper::GetSigninProfileDir() ||
-        profile->GetPath() ==
-            ash::ProfileHelper::GetLockScreenAppProfilePath()) {
+    if (ash::IsSigninBrowserContext(profile) ||
+        ash::IsLockScreenAppBrowserContext(profile)) {
       return nullptr;
     }
 
@@ -282,9 +283,8 @@ class FileSystemApiTestForRequestFileSystem : public PlatformAppBrowserTest {
   drive::DriveIntegrationService* CreateDriveIntegrationService(
       Profile* profile) {
     // Ignore signin and lock screen apps profile.
-    if (profile->GetPath() == ash::ProfileHelper::GetSigninProfileDir() ||
-        profile->GetPath() ==
-            ash::ProfileHelper::GetLockScreenAppProfilePath()) {
+    if (ash::IsSigninBrowserContext(profile) ||
+        ash::IsLockScreenAppBrowserContext(profile)) {
       return nullptr;
     }
 
@@ -473,7 +473,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForDrive,
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, Background) {
   EnterKioskSession();
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_OK);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kOk);
   ASSERT_TRUE(
       RunExtensionTest("api_test/file_system/request_file_system_background",
                        {.launch_as_platform_app = true}))
@@ -482,7 +483,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, Background) {
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, ReadOnly) {
   EnterKioskSession();
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_OK);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kOk);
   ASSERT_TRUE(
       RunExtensionTest("api_test/file_system/request_file_system_read_only",
                        {.launch_as_platform_app = true}))
@@ -491,7 +493,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, ReadOnly) {
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, Writable) {
   EnterKioskSession();
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_OK);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kOk);
   ASSERT_TRUE(
       RunExtensionTest("api_test/file_system/request_file_system_writable",
                        {.launch_as_platform_app = true}))
@@ -500,7 +503,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, Writable) {
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, UserReject) {
   EnterKioskSession();
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_CANCEL);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kCancel);
   ASSERT_TRUE(
       RunExtensionTest("api_test/file_system/request_file_system_user_reject",
                        {.launch_as_platform_app = true}))
@@ -508,7 +512,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, UserReject) {
 }
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, NotKioskSession) {
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_OK);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kOk);
   ASSERT_TRUE(RunExtensionTest(
       "api_test/file_system/request_file_system_not_kiosk_session",
       {.launch_as_platform_app = true}))
@@ -517,7 +522,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem, NotKioskSession) {
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem,
                        AllowlistedComponent) {
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_CANCEL);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kCancel);
   ASSERT_TRUE(RunExtensionTest(
       "api_test/file_system/request_file_system_allowed_component",
       {.launch_as_platform_app = true}, {.load_as_component = true}))
@@ -526,7 +532,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem,
 
 IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem,
                        NotAllowlistedComponent) {
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_OK);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kOk);
   ASSERT_TRUE(RunExtensionTest(
       "api_test/file_system/request_file_system_not_allowed_component",
       {.launch_as_platform_app = true}, {.load_as_component = true}))
@@ -658,7 +665,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem,
       .WillOnce(base::test::RunOnceCallback<1>(
           file_access::ScopedFileAccess(true, base::ScopedFD())));
 
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_CANCEL);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kCancel);
   const base::FilePath test_file = temp_dir_.GetPath()
                                        .Append(kReadOnlyMountPointName)
                                        .AppendASCII("open_existing.txt");
@@ -691,7 +699,8 @@ IN_PROC_BROWSER_TEST_F(FileSystemApiTestForRequestFileSystem,
       .WillOnce(base::test::RunOnceCallback<1>(
           file_access::ScopedFileAccess(false, base::ScopedFD())));
 
-  ScopedSkipRequestFileSystemDialog dialog_skipper(ui::DIALOG_BUTTON_CANCEL);
+  ScopedSkipRequestFileSystemDialog dialog_skipper(
+      ui::mojom::DialogButton::kCancel);
   const base::FilePath test_file = temp_dir_.GetPath()
                                        .Append(kReadOnlyMountPointName)
                                        .AppendASCII("open_existing.txt");
