@@ -67,9 +67,10 @@
 #include "testing/gtest/include/gtest/gtest.h"
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-#include "chrome/browser/ash/login/users/scoped_test_user_manager.h"
+#include "chrome/browser/ash/login/users/chrome_user_manager_impl.h"
 #include "chrome/browser/ash/settings/scoped_cros_settings_test_helper.h"
 #include "chromeos/ash/components/dbus/concierge/concierge_client.h"
+#include "components/user_manager/scoped_user_manager.h"
 #endif
 
 namespace extensions {
@@ -270,7 +271,8 @@ class ExtensionGCMAppHandlerTest : public testing::Test {
 
     // This is needed to create extension service under CrOS.
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-    test_user_manager_ = std::make_unique<ash::ScopedTestUserManager>();
+    test_user_manager_.Reset(
+        ash::ChromeUserManagerImpl::CreateChromeUserManager());
     ash::ConciergeClient::InitializeFake(/*fake_cicerone_client=*/nullptr);
 #endif
 
@@ -299,7 +301,7 @@ class ExtensionGCMAppHandlerTest : public testing::Test {
 
   void TearDown() override {
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-    test_user_manager_.reset();
+    test_user_manager_.Reset();
 #endif
 
     waiter_.PumpUILoop();
@@ -319,7 +321,7 @@ class ExtensionGCMAppHandlerTest : public testing::Test {
   scoped_refptr<const Extension> CreateExtension() {
     scoped_refptr<const Extension> extension =
         ExtensionBuilder(kTestExtensionName)
-            .AddPermission("gcm")
+            .AddAPIPermission("gcm")
             .SetPath(temp_dir_.GetPath())
             .SetID("ldnnhddmnhbkjipkidpdiheffobcpfmf")
             .Build();
@@ -423,7 +425,7 @@ class ExtensionGCMAppHandlerTest : public testing::Test {
   // This is needed to create extension service under CrOS.
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   ash::ScopedCrosSettingsTestHelper cros_settings_test_helper_;
-  std::unique_ptr<ash::ScopedTestUserManager> test_user_manager_;
+  user_manager::ScopedUserManager test_user_manager_;
 #endif
 
   Waiter waiter_;
