@@ -145,7 +145,7 @@ void HistoryEventRouter::OnURLVisited(history::HistoryService* history_service,
                 api::history::OnVisited::kEventName, std::move(args));
 }
 
-void HistoryEventRouter::OnURLsDeleted(
+void HistoryEventRouter::OnHistoryDeletions(
     history::HistoryService* history_service,
     const history::DeletionInfo& deletion_info) {
   OnVisitRemoved::Removed removed;
@@ -376,7 +376,7 @@ ExtensionFunction::ResponseAction HistoryDeleteRangeFunction::Run() {
   history::WebHistoryService* web_history =
       WebHistoryServiceFactory::GetForProfile(GetProfile());
   hs->DeleteLocalAndRemoteHistoryBetween(
-      web_history, start_time, end_time,
+      web_history, start_time, end_time, history::kNoAppIdFilter,
       base::BindOnce(&HistoryDeleteRangeFunction::DeleteComplete,
                      base::Unretained(this)),
       &task_tracker_);
@@ -411,7 +411,7 @@ ExtensionFunction::ResponseAction HistoryDeleteAllFunction::Run() {
   hs->DeleteLocalAndRemoteHistoryBetween(
       web_history,
       /*begin_time*/ base::Time(),
-      /*end_time*/ base::Time::Max(),
+      /*end_time*/ base::Time::Max(), history::kNoAppIdFilter,
       base::BindOnce(&HistoryDeleteAllFunction::DeleteComplete,
                      base::Unretained(this)),
       &task_tracker_);
