@@ -171,24 +171,14 @@ class IdentityGetAuthTokenFunction : public ExtensionFunction,
 
   enum class InteractionType { kSignin, kConsent };
 
-  // Request the primary account info.
-  // |extension_gaia_id|: The GAIA ID that was set in the parameters for this
-  // instance, or empty if this was not in the parameters.
-  void GetAuthTokenForPrimaryAccount(const std::string& extension_gaia_id);
-
-  // Wrapper to FindExtendedAccountInfoByGaiaId() to avoid a synchronous call to
-  // IdentityManager in RunAsync().
-  void FetchExtensionAccountInfo(const std::string& gaia_id);
-
-  // Called when the AccountInfo that this instance should use is available.
-  void OnReceivedExtensionAccountInfo(const CoreAccountInfo& account_info);
+  // If `gaia_id` is empty or the account is not present in Chrome, this will
+  // use the primary account if it exists. Otherwise, interactive sign in flow
+  // might be started.
+  void GetAuthTokenForAccount(const std::string& gaia_id);
 
   // signin::IdentityManager::Observer implementation:
   void OnRefreshTokenUpdatedForAccount(
       const CoreAccountInfo& account_info) override;
-  void OnAccountsInCookieUpdated(
-      const signin::AccountsInCookieJarInfo& accounts_in_cookie_jar_info,
-      const GoogleServiceAuthError& error) override;
   void OnPrimaryAccountChanged(
       const signin::PrimaryAccountChangeEvent& event_details) override;
 
@@ -219,9 +209,8 @@ class IdentityGetAuthTokenFunction : public ExtensionFunction,
   void StartMintToken(IdentityMintRequestQueue::MintType type) override;
 
   // OAuth2MintTokenFlow::Delegate implementation:
-  void OnMintTokenSuccess(const std::string& access_token,
-                          const std::set<std::string>& granted_scopes,
-                          int time_to_live) override;
+  void OnMintTokenSuccess(
+      const OAuth2MintTokenFlow::MintTokenResult& result) override;
   void OnMintTokenFailure(const GoogleServiceAuthError& error) override;
   void OnRemoteConsentSuccess(
       const RemoteConsentResolutionData& resolution_data) override;
@@ -234,6 +223,14 @@ class IdentityGetAuthTokenFunction : public ExtensionFunction,
   virtual void StartDeviceAccessTokenRequest();
 #endif
 
+#if BUILDFLAG(ENABLE_DICE_SUPPORT)
+  // This dialog prompts the user to sign in with an account that is already
+  // present in the identity manager. This is different from the signin dialog
+  // shown when there are no accounts in the identity manager.
+  void MaybeShowChromeSigninDialog();
+  void OnChromeSigninDialogDestroyed();
+#endif
+
   // Methods for invoking UI. Overridable for testing.
   virtual void ShowExtensionLoginPrompt();
   virtual void ShowRemoteConsentDialog(
@@ -274,9 +271,6 @@ class IdentityGetAuthTokenFunction : public ExtensionFunction,
   // extension.
   std::string selected_gaia_id_;
 
-  // Shown in the extension login prompt.
-  std::string email_for_default_web_account_;
-
   ExtensionTokenKey token_key_{/*extension_id=*/"",
                                /*account_info=*/CoreAccountInfo(),
                                /*scopes=*/{}};
@@ -296,16 +290,7 @@ class IdentityGetAuthTokenFunction : public ExtensionFunction,
                           signin::IdentityManager::Observer>
       scoped_identity_manager_observation_{this};
 
-  // This class can be listening to account changes, but only for one type of
-  // events at a time.
-  enum class AccountListeningMode {
-    kNotListening,            // Not listening account changes
-    kListeningCookies,        // Listening cookie changes
-    kListeningTokens,         // Listening token changes
-    kListeningPrimaryAccount  // Listening primary account changes
-  };
-  AccountListeningMode account_listening_mode_ =
-      AccountListeningMode::kNotListening;
+  bool waiting_on_account_ = false;
 
   base::WeakPtrFactory<IdentityGetAuthTokenFunction> weak_ptr_factory_{this};
 };
