@@ -7,6 +7,7 @@
 #include <memory>
 #include <utility>
 
+#include "base/check_is_test.h"
 #include "base/functional/bind.h"
 #include "base/memory/raw_ptr.h"
 #include "base/strings/utf_string_conversions.h"
@@ -20,19 +21,22 @@
 #include "chrome/browser/extensions/extension_management.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/launch_util.h"
+#include "chrome/browser/extensions/manifest_v2_experiment_manager.h"
+#include "chrome/browser/extensions/mv2_experiment_stage.h"
 #include "chrome/browser/favicon/favicon_service_factory.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/ssl/security_state_tab_helper.h"
 #include "chrome/browser/ui/browser_dialogs.h"
 #include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/browser_tabstrip.h"
 #include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/extensions/extensions_dialogs.h"
 #include "chrome/browser/ui/scoped_tabbed_browser_displayer.h"
 #include "chrome/browser/ui/tab_helpers.h"
 #include "chrome/browser/ui/web_applications/web_app_dialog_utils.h"
 #include "chrome/browser/ui/webui/extensions/extension_icon_source.h"
 #include "chrome/browser/web_applications/extension_status_utils.h"
 #include "chrome/browser/web_applications/mojom/user_display_mode.mojom.h"
+#include "chrome/browser/web_applications/proto/web_app_install_state.pb.h"
 #include "chrome/browser/web_applications/web_app_command_scheduler.h"
 #include "chrome/browser/web_applications/web_app_helpers.h"
 #include "chrome/browser/web_applications/web_app_install_info.h"
@@ -46,30 +50,30 @@
 #include "chrome/common/extensions/extension_metrics.h"
 #include "chrome/common/extensions/manifest_handlers/app_launch_info.h"
 #include "components/favicon/core/favicon_service.h"
+#include "components/security_state/content/security_state_tab_helper.h"
 #include "components/services/app_service/public/cpp/app_launch_util.h"
-#include "components/supervised_user/core/common/buildflags.h"
 #include "components/webapps/browser/install_result_code.h"
 #include "components/webapps/browser/installable/installable_manager.h"
 #include "components/webapps/browser/installable/installable_metrics.h"
+#include "components/webapps/common/web_app_id.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/web_contents.h"
 #include "extensions/browser/api/management/management_api.h"
 #include "extensions/browser/api/management/management_api_constants.h"
 #include "extensions/browser/disable_reason.h"
+#include "extensions/browser/extension_dialog_auto_confirm.h"
 #include "extensions/browser/extension_prefs.h"
 #include "extensions/browser/extension_registry.h"
 #include "extensions/browser/extension_system.h"
+#include "extensions/browser/supervised_user_extensions_delegate.h"
 #include "extensions/common/api/management.h"
 #include "extensions/common/extension.h"
+#include "extensions/common/extension_id.h"
 #include "extensions/common/extension_urls.h"
 #include "extensions/common/mojom/context_type.mojom.h"
 #include "third_party/blink/public/mojom/manifest/display_mode.mojom.h"
 #include "third_party/blink/public/mojom/window_features/window_features.mojom.h"
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-#include "chrome/browser/ash/login/demo_mode/demo_session.h"
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
-
 namespace {
 using InstallOrLaunchWebAppCallback =
     extensions::ManagementAPIDelegate::InstallOrLaunchWebAppCallback;
@@ -221,9 +225,17 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
     }
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
-    auto web_app_info = std::make_unique<web_app::WebAppInstallInfo>();
+    // GenerateAppForLink API doesn't allow a manifest ID to be specified, so
+    // just use the launch_url for both manifest ID and start URL. This is a
+    // reasonable behavior for "DIY apps" generated for a specific URL but
+    // should be fixed if used for installing existing "Crafted Apps" (ie.
+    // apps with an existing manifest that should be used for updates).
+    GURL start_url = launch_url;
+    webapps::ManifestId manifest_id =
+        web_app::GenerateManifestIdFromStartUrlOnly(start_url);
+    auto web_app_info =
+        std::make_unique<web_app::WebAppInstallInfo>(manifest_id, start_url);
     web_app_info->title = base::UTF8ToUTF16(title);
-    web_app_info->start_url = launch_url;
     web_app_info->display_mode = web_app::DisplayMode::kBrowser;
     web_app_info->user_display_mode = web_app::mojom::UserDisplayMode::kBrowser;
 
@@ -235,16 +247,17 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
     auto* provider = web_app::WebAppProvider::GetForWebApps(
         Profile::FromBrowserContext(context));
 
-    provider->scheduler().InstallFromInfo(
+    provider->scheduler().InstallFromInfoWithParams(
         std::move(web_app_info),
         /*overwrite_existing_manifest_fields=*/false,
         webapps::WebappInstallSource::MANAGEMENT_API,
         base::BindOnce(OnGenerateAppForLinkCompleted,
-                       base::RetainedRef(function)));
+                       base::RetainedRef(function)),
+        web_app::WebAppInstallParams());
   }
 
   extensions::api::management::ExtensionInfo CreateExtensionInfoFromWebApp(
-      const std::string& app_id,
+      const extensions::ExtensionId& app_id,
       content::BrowserContext* context) override {
     auto* provider = web_app::WebAppProvider::GetForWebApps(
         Profile::FromBrowserContext(context));
@@ -254,7 +267,8 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
     extensions::api::management::ExtensionInfo info;
     info.id = app_id;
     info.name = registrar.GetAppShortName(app_id);
-    info.enabled = registrar.IsLocallyInstalled(app_id);
+    info.enabled = registrar.IsInstallState(
+        app_id, {web_app::proto::INSTALLED_WITH_OS_INTEGRATION});
     info.install_type =
         extensions::api::management::ExtensionInstallType::kOther;
     info.is_app = true;
@@ -291,6 +305,7 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
       case web_app::DisplayMode::kWindowControlsOverlay:
       case web_app::DisplayMode::kTabbed:
       case web_app::DisplayMode::kBorderless:
+      case web_app::DisplayMode::kPictureInPicture:
       case web_app::DisplayMode::kUndefined:
         info.launch_type = extensions::api::management::LaunchType::kNone;
         break;
@@ -306,7 +321,7 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
 void LaunchWebApp(const webapps::AppId& app_id, Profile* profile) {
   // Look at prefs to find the right launch container. If the user has not set a
   // preference, the default launch value will be returned.
-  // TODO(crbug.com/1003602): Make AppLaunchParams launch container Optional or
+  // TODO(crbug.com/40098656): Make AppLaunchParams launch container Optional or
   // add a "default" launch container enum value.
   auto* provider = web_app::WebAppProvider::GetForWebApps(profile);
   DCHECK(provider);
@@ -371,10 +386,9 @@ void OnWebAppInstallabilityChecked(
           base::BindOnce(&OnWebAppInstallCompleted, std::move(callback)));
       return;
   }
-  NOTREACHED();
+  NOTREACHED_IN_MIGRATION();
 }
 
-#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
 extensions::SupervisedUserExtensionsDelegate*
 GetSupervisedUserExtensionsDelegateFromContext(
     content::BrowserContext* context) {
@@ -386,8 +400,6 @@ GetSupervisedUserExtensionsDelegateFromContext(
   CHECK(supervised_user_extensions_delegate);
   return supervised_user_extensions_delegate;
 }
-#endif  // BUILDFLAG(ENABLE_SUPERVISED_USERS)
-
 }  // namespace
 
 ChromeManagementAPIDelegate::ChromeManagementAPIDelegate() = default;
@@ -400,13 +412,12 @@ bool ChromeManagementAPIDelegate::LaunchAppFunctionDelegate(
   // Look at prefs to find the right launch container.
   // If the user has not set a preference, the default launch value will be
   // returned.
-  // TODO(crbug.com/1003602): Make AppLaunchParams launch container Optional or
+  // TODO(crbug.com/40098656): Make AppLaunchParams launch container Optional or
   // add a "default" launch container enum value.
   apps::LaunchContainer launch_container =
       GetLaunchContainer(extensions::ExtensionPrefs::Get(context), extension);
   Profile* profile = Profile::FromBrowserContext(context);
-#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || \
-    BUILDFLAG(IS_FUCHSIA)
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX)
   if (extensions::IsExtensionUnsupportedDeprecatedApp(profile,
                                                       extension->id())) {
     return false;
@@ -424,11 +435,6 @@ bool ChromeManagementAPIDelegate::LaunchAppFunctionDelegate(
                             WindowOpenDisposition::NEW_FOREGROUND_TAB,
                             apps::LaunchSource::kFromManagementApi));
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  ash::DemoSession::RecordAppLaunchSourceIfInDemoMode(
-      ash::DemoSession::AppLaunchSource::kExtensionApi);
-#endif
-
   extensions::RecordAppLaunchType(extension_misc::APP_LAUNCH_EXTENSION_API,
                                   extension->GetType());
   return true;
@@ -534,7 +540,10 @@ void ChromeManagementAPIDelegate::InstallOrLaunchReplacementWebApp(
 
   // Launch the app if web_app_url happens to match start_url. If not, the app
   // could still be installed with different start_url.
-  if (provider->registrar_unsafe().IsLocallyInstalled(web_app_url)) {
+  webapps::AppId app_id = web_app::GenerateAppIdFromManifestId(web_app_url);
+  if (provider->registrar_unsafe().IsInstallState(
+          app_id, {web_app::proto::INSTALLED_WITHOUT_OS_INTEGRATION,
+                   web_app::proto::INSTALLED_WITH_OS_INTEGRATION})) {
     LaunchWebApp(
         web_app::GenerateAppId(/*manifest_id=*/std::nullopt, web_app_url),
         profile);
@@ -556,21 +565,18 @@ void ChromeManagementAPIDelegate::InstallOrLaunchReplacementWebApp(
 
 void ChromeManagementAPIDelegate::EnableExtension(
     content::BrowserContext* context,
-    const std::string& extension_id) const {
+    const extensions::ExtensionId& extension_id) const {
   const extensions::Extension* extension =
       extensions::ExtensionRegistry::Get(context)->GetExtensionById(
           extension_id, extensions::ExtensionRegistry::EVERYTHING);
+  // The extension must exist as this method is invoked on enabling an extension
+  // from the extensions management page (see `ManagementSetEnabledFunction`).
+  CHECK(extension);
 
-#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
-  // We add approval for the extension here under the assumption that prior
-  // to this point, the supervised child user has already been prompted
-  // for, and received parent permission to install the extension.
   extensions::SupervisedUserExtensionsDelegate* extensions_delegate =
       GetSupervisedUserExtensionsDelegateFromContext(context);
-
-  extensions_delegate->AddExtensionApproval(*extension);
+  extensions_delegate->MaybeRecordPermissionsIncreaseMetrics(*extension);
   extensions_delegate->RecordExtensionEnablementUmaMetrics(/*enabled=*/true);
-#endif  // BUILDFLAG(ENABLE_SUPERVISED_USERS)
 
   // If the extension was disabled for a permissions increase, the Management
   // API will have displayed a re-enable prompt to the user, so we know it's
@@ -583,13 +589,11 @@ void ChromeManagementAPIDelegate::EnableExtension(
 void ChromeManagementAPIDelegate::DisableExtension(
     content::BrowserContext* context,
     const extensions::Extension* source_extension,
-    const std::string& extension_id,
+    const extensions::ExtensionId& extension_id,
     extensions::disable_reason::DisableReason disable_reason) const {
-#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
   extensions::SupervisedUserExtensionsDelegate* extensions_delegate =
       GetSupervisedUserExtensionsDelegateFromContext(context);
   extensions_delegate->RecordExtensionEnablementUmaMetrics(/*enabled=*/false);
-#endif  // BUILDFLAG(ENABLE_SUPERVISED_USERS)
   extensions::ExtensionSystem::Get(context)
       ->extension_service()
       ->DisableExtensionWithSource(source_extension, extension_id,
@@ -598,7 +602,7 @@ void ChromeManagementAPIDelegate::DisableExtension(
 
 bool ChromeManagementAPIDelegate::UninstallExtension(
     content::BrowserContext* context,
-    const std::string& transient_extension_id,
+    const extensions::ExtensionId& transient_extension_id,
     extensions::UninstallReason reason,
     std::u16string* error) const {
   return extensions::ExtensionSystem::Get(context)
@@ -608,7 +612,7 @@ bool ChromeManagementAPIDelegate::UninstallExtension(
 
 void ChromeManagementAPIDelegate::SetLaunchType(
     content::BrowserContext* context,
-    const std::string& extension_id,
+    const extensions::ExtensionId& extension_id,
     extensions::LaunchType launch_type) const {
   extensions::SetLaunchType(context, extension_id, launch_type);
 }
@@ -616,7 +620,7 @@ void ChromeManagementAPIDelegate::SetLaunchType(
 GURL ChromeManagementAPIDelegate::GetIconURL(
     const extensions::Extension* extension,
     int icon_size,
-    ExtensionIconSet::MatchType match,
+    ExtensionIconSet::Match match,
     bool grayscale) const {
   return extensions::ExtensionIconSource::GetIconURL(extension, icon_size,
                                                      match, grayscale);
@@ -629,3 +633,39 @@ GURL ChromeManagementAPIDelegate::GetEffectiveUpdateURL(
       extensions::ExtensionManagementFactory::GetForBrowserContext(context);
   return extension_management->GetEffectiveUpdateURL(extension);
 }
+
+void ChromeManagementAPIDelegate::ShowMv2DeprecationReEnableDialog(
+    content::BrowserContext* context,
+    content::WebContents* web_contents,
+    const extensions::Extension& extension,
+    base::OnceCallback<void(bool)> done_callback) const {
+  // Extension should only be disabled due to MV2 deprecation in the "disable"
+  // experiment stage.
+  auto* mv2_experiment_manager =
+      extensions::ManifestV2ExperimentManager::Get(context);
+  CHECK_EQ(mv2_experiment_manager->GetCurrentExperimentStage(),
+           extensions::MV2ExperimentStage::kDisableWithReEnable);
+
+  // Tests can auto confirm the re-enable dialog.
+  auto confirm_value =
+      extensions::ScopedTestDialogAutoConfirm::GetAutoConfirmValue();
+  switch (confirm_value) {
+    case extensions::ScopedTestDialogAutoConfirm::NONE:
+      // Continue, auto confirm has not been set.
+      break;
+    case extensions::ScopedTestDialogAutoConfirm::CANCEL:
+      CHECK_IS_TEST();
+      std::move(done_callback).Run(/*enable_allowed=*/false);
+      return;
+    case extensions::ScopedTestDialogAutoConfirm::ACCEPT:
+    case extensions::ScopedTestDialogAutoConfirm::ACCEPT_AND_OPTION:
+      CHECK_IS_TEST();
+      std::move(done_callback).Run(/*enable_allowed=*/true);
+      return;
+  }
+
+  gfx::NativeWindow parent =
+      web_contents ? web_contents->GetTopLevelNativeWindow() : nullptr;
+  extensions::ShowMv2DeprecationReEnableDialog(
+      parent, extension.id(), extension.name(), std::move(done_callback));
+}
