@@ -6,6 +6,7 @@
 #define CHROME_BROWSER_EXTENSIONS_API_MESSAGING_CHROME_MESSAGING_DELEGATE_H_
 
 #include "extensions/browser/api/messaging/messaging_delegate.h"
+#include "extensions/common/extension_id.h"
 
 namespace extensions {
 
@@ -30,7 +31,7 @@ class ChromeMessagingDelegate : public MessagingDelegate {
       int tab_id) override;
   std::unique_ptr<MessagePort> CreateReceiverForTab(
       base::WeakPtr<MessagePort::ChannelDelegate> channel_delegate,
-      const std::string& extension_id,
+      const ExtensionId& extension_id,
       const PortId& receiver_port_id,
       content::WebContents* receiver_contents,
       int receiver_frame_id,
@@ -39,7 +40,7 @@ class ChromeMessagingDelegate : public MessagingDelegate {
       content::BrowserContext* browser_context,
       base::WeakPtr<MessagePort::ChannelDelegate> channel_delegate,
       content::RenderFrameHost* source,
-      const std::string& extension_id,
+      const ExtensionId& extension_id,
       const PortId& receiver_port_id,
       const std::string& native_app_name,
       bool allow_user_level,
