@@ -2,8 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "extensions/browser/api/offscreen/offscreen_api.h"
-
 #include "base/functional/callback_helpers.h"
 #include "base/ranges/algorithm.h"
 #include "base/run_loop.h"
@@ -11,22 +9,28 @@
 #include "build/build_config.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/browser/extensions/extension_util.h"
+#include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/extensions/extension_action_test_helper.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/version_info/channel.h"
 #include "content/public/test/browser_test.h"
+#include "content/public/test/browser_test_utils.h"
 #include "extensions/browser/api/offscreen/audio_lifetime_enforcer.h"
+#include "extensions/browser/api/offscreen/offscreen_api.h"
 #include "extensions/browser/api/offscreen/offscreen_document_manager.h"
 #include "extensions/browser/background_script_executor.h"
 #include "extensions/browser/extension_util.h"
 #include "extensions/browser/lazy_context_id.h"
 #include "extensions/browser/lazy_context_task_queue.h"
 #include "extensions/browser/offscreen_document_host.h"
+#include "extensions/browser/script_executor.h"
 #include "extensions/browser/test_extension_registry_observer.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/features/feature_channel.h"
 #include "extensions/common/switches.h"
 #include "extensions/test/extension_background_page_waiter.h"
+#include "extensions/test/extension_test_message_listener.h"
 #include "extensions/test/result_catcher.h"
 #include "extensions/test/test_extension_dir.h"
 #include "net/dns/mock_host_resolver.h"
@@ -112,9 +116,10 @@ void WakeUpServiceWorker(const Extension& extension, Profile& profile) {
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 class ContentBrowserClientMock : public ChromeContentBrowserClient {
  public:
-  MOCK_METHOD(bool,
-              IsGetAllScreensMediaAllowed,
-              (content::BrowserContext * context, const url::Origin& origin),
+  MOCK_METHOD(void,
+              CheckGetAllScreensMediaAllowed,
+              (content::RenderFrameHost * render_frame_host,
+               base::OnceCallback<void(bool)> callback),
               (override));
 };
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
@@ -236,9 +241,10 @@ IN_PROC_BROWSER_TEST_F(OffscreenApiTest, MAYBE_BasicDocumentManagement) {
 
 // Tests creating, querying, and closing offscreen documents in an incognito
 // split mode extension.
-// TODO(crbug.com/1484659): Disabled on ASAN due to leak caused by renderer gin
+// TODO(crbug.com/40282331): Disabled on ASAN due to leak caused by renderer gin
 // objects which are intended to be leaked.
-#if defined(ADDRESS_SANITIZER)
+// TODO(crbug.com/345326424): Flaky on Mac builds.
+#if defined(ADDRESS_SANITIZER) || BUILDFLAG(IS_MAC)
 #define MAYBE_IncognitoModeHandling_SplitMode \
   DISABLED_IncognitoModeHandling_SplitMode
 #else
@@ -317,9 +323,10 @@ IN_PROC_BROWSER_TEST_F(OffscreenApiTest,
 
 // Tests creating, querying, and closing offscreen documents in an incognito
 // spanning mode extension.
-// TODO(crbug.com/1484659): Disabled on ASAN due to leak caused by renderer gin
+// TODO(crbug.com/40282331): Disabled on ASAN due to leak caused by renderer gin
 // objects which are intended to be leaked.
-#if defined(ADDRESS_SANITIZER)
+// TODO(crbug.com/345326424): Flaky on Mac builds.
+#if defined(ADDRESS_SANITIZER) || BUILDFLAG(IS_MAC)
 #define MAYBE_IncognitoModeHandling_SpanningMode \
   DISABLED_IncognitoModeHandling_SpanningMode
 #else
@@ -547,8 +554,11 @@ IN_PROC_BROWSER_TEST_F(GetAllScreensMediaOffscreenApiTest,
   base::AddTagToTestResult("feature_id",
                            "screenplay-f3601ae4-bff7-495a-a51f-3c0997a46445");
   EXPECT_CALL(content_browser_client(),
-              IsGetAllScreensMediaAllowed(testing::_, testing::_))
-      .WillOnce(testing::Return(true));
+              CheckGetAllScreensMediaAllowed(testing::_, testing::_))
+      .WillOnce(testing::Invoke([](content::RenderFrameHost* render_frame_host,
+                                   base::OnceCallback<void(bool)> callback) {
+        std::move(callback).Run(true);
+      }));
   static constexpr char kManifest[] =
       R"({
            "name": "Offscreen Document Test",
@@ -560,6 +570,8 @@ IN_PROC_BROWSER_TEST_F(GetAllScreensMediaOffscreenApiTest,
   // An offscreen document that knows how to capture all screens.
   static constexpr char kOffscreenJs[] =
       R"(
+        'use strict';
+
         let streams;
 
         async function captureAllScreens() {
@@ -606,12 +618,22 @@ IN_PROC_BROWSER_TEST_F(GetAllScreensMediaOffscreenApiTest,
           } else {
             console.error('Unexpected message: ' + msg);
           }
-        }))";
+        })
+        R)";
   TestExtensionDir test_dir;
   test_dir.WriteManifest(kManifest);
   test_dir.WriteFile(FILE_PATH_LITERAL("background.js"), "// Blank.");
   test_dir.WriteFile(FILE_PATH_LITERAL("offscreen.html"),
-                     R"(<html><script src="offscreen.js"></script></html>)");
+                     R"(
+    <html>
+      <script src="offscreen.js"></script>
+      <meta http-equiv="Content-Security-Policy"
+        content="object-src 'none'; base-uri 'none';
+        script-src 'strict-dynamic'
+        'sha256-Y55VppSZfjQ4A035BPDo9OMXignyoxRXv+KKCZpnWiM=';
+        require-trusted-types-for 'script';trusted-types a;">
+    </html>
+    )");
   test_dir.WriteFile(FILE_PATH_LITERAL("offscreen.js"), kOffscreenJs);
 
   scoped_refptr<const Extension> extension =
@@ -646,12 +668,12 @@ IN_PROC_BROWSER_TEST_F(GetAllScreensMediaOffscreenApiTest,
         profile(), extension->id(), "chrome.runtime.sendMessage('stop');");
   }
 
-  // TODO(crbug.com/1443432): Add check if document gets shut down after the
+  // TODO(crbug.com/40267351): Add check if document gets shut down after the
   // screen capture with `getAllScreensMedia` is stopped.
 }
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
-// TODO(https://crbug.com/1453966): Failing on Windows.
+// TODO(crbug.com/40272130): Failing on Windows.
 #if BUILDFLAG(IS_WIN)
 #define MAYBE_TabCaptureStreams DISABLED_TabCaptureStreams
 #else
@@ -735,6 +757,68 @@ IN_PROC_BROWSER_TEST_F(OffscreenApiTest, LongLoadOffscreenDocument) {
   ASSERT_TRUE(RunExtensionTest(test_dir.UnpackedPath(), {}, {})) << message_;
 }
 
+// Tests user gestures are curried from service workers into offscreen
+// documents.
+IN_PROC_BROWSER_TEST_F(OffscreenApiTest,
+                       UserGesturesAreCurriedFromServiceWorkers) {
+  static constexpr char kManifest[] =
+      R"({
+           "name": "Offscreen Document Test",
+           "manifest_version": 3,
+           "version": "0.1",
+           "permissions": ["offscreen"],
+           "action": {},
+           "background": { "service_worker": "background.js" }
+         })";
+  static constexpr char kOffscreenHtml[] =
+      R"(<html><script src="offscreen.js"></script></html>)";
+  static constexpr char kOffscreenJs[] =
+      R"(chrome.runtime.onMessage.addListener((msg, sender, sendReply) => {
+           try {
+             const activeGesture = chrome.test.isProcessingUserGesture();
+             sendReply('active gesture: ' + activeGesture);
+           } catch (e) {
+             sendReply(`Error: ${e.toString()}`);
+           }
+         });)";
+  // The extension background script will:
+  // - Open a new offscreen document
+  // - Wait for an action click. This includes an active user action.
+  // - In the listener for the action click, dispatch a message to the
+  //   offscreen document. The active user gesture should be curried along.
+  static constexpr char kBackgroundJs[] =
+      R"((async () => {
+             await chrome.offscreen.createDocument(
+                       {
+                           url: 'offscreen.html',
+                           reasons: ['TESTING'],
+                           justification: 'testing'
+                       });
+             chrome.test.sendMessage('opened');
+         })();
+         chrome.action.onClicked.addListener(() => {
+           chrome.test.assertTrue(chrome.test.isProcessingUserGesture());
+           chrome.runtime.sendMessage('test message').then(response => {
+             chrome.test.assertEq('active gesture: true', response);
+             chrome.test.succeed();
+           });
+         });)";
+
+  TestExtensionDir test_dir;
+  test_dir.WriteManifest(kManifest);
+  test_dir.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundJs);
+  test_dir.WriteFile(FILE_PATH_LITERAL("offscreen.html"), kOffscreenHtml);
+  test_dir.WriteFile(FILE_PATH_LITERAL("offscreen.js"), kOffscreenJs);
+
+  ExtensionTestMessageListener test_listener("opened");
+  ResultCatcher result_catcher;
+  const Extension* extension = LoadExtension(test_dir.UnpackedPath());
+  ASSERT_TRUE(extension);
+  ASSERT_TRUE(test_listener.WaitUntilSatisfied());
+  ExtensionActionTestHelper::Create(browser())->Press(extension->id());
+  ASSERT_TRUE(result_catcher.GetNextResult()) << result_catcher.message();
+}
+
 // Tests that the `TESTING` reason is disallowed without the appropriate
 // commandline switch.
 IN_PROC_BROWSER_TEST_F(OffscreenApiTestWithoutCommandLineFlag,
