@@ -9,6 +9,7 @@
 #include <memory>
 #include <optional>
 #include <string>
+#include <string_view>
 #include <utility>
 #include <vector>
 
@@ -29,6 +30,7 @@
 #include "extensions/browser/event_router.h"
 #include "extensions/browser/extension_prefs.h"
 #include "extensions/browser/extension_prefs_factory.h"
+#include "extensions/common/extension_id.h"
 #include "ui/gfx/image/image.h"
 
 namespace extensions {
@@ -49,7 +51,7 @@ const char kOmniboxDefaultSuggestion[] = "omnibox_default_suggestion";
 
 std::optional<omnibox::SuggestResult> GetOmniboxDefaultSuggestion(
     Profile* profile,
-    const std::string& extension_id) {
+    const ExtensionId& extension_id) {
   ExtensionPrefs* prefs = ExtensionPrefs::Get(profile);
   if (!prefs) {
     return std::nullopt;
@@ -67,7 +69,7 @@ std::optional<omnibox::SuggestResult> GetOmniboxDefaultSuggestion(
 // false on failure.
 bool SetOmniboxDefaultSuggestion(
     Profile* profile,
-    const std::string& extension_id,
+    const ExtensionId& extension_id,
     const omnibox::DefaultSuggestResult& suggestion) {
   ExtensionPrefs* prefs = ExtensionPrefs::Get(profile);
   if (!prefs)
@@ -84,7 +86,7 @@ bool SetOmniboxDefaultSuggestion(
 }
 
 // Returns a string used as a template URL string of the extension.
-std::string GetTemplateURLStringForExtension(const std::string& extension_id) {
+std::string GetTemplateURLStringForExtension(const ExtensionId& extension_id) {
   // This URL is not actually used for navigation. It holds the extension's ID.
   return std::string(extensions::kExtensionScheme) + "://" +
       extension_id + "/?q={searchTerms}";
@@ -94,7 +96,8 @@ std::string GetTemplateURLStringForExtension(const std::string& extension_id) {
 
 // static
 void ExtensionOmniboxEventRouter::OnInputStarted(
-    Profile* profile, const std::string& extension_id) {
+    Profile* profile,
+    const ExtensionId& extension_id) {
   auto event = std::make_unique<Event>(events::OMNIBOX_ON_INPUT_STARTED,
                                        omnibox::OnInputStarted::kEventName,
                                        base::Value::List(), profile);
@@ -104,8 +107,10 @@ void ExtensionOmniboxEventRouter::OnInputStarted(
 
 // static
 bool ExtensionOmniboxEventRouter::OnInputChanged(
-    Profile* profile, const std::string& extension_id,
-    const std::string& input, int suggest_id) {
+    Profile* profile,
+    const ExtensionId& extension_id,
+    const std::string& input,
+    int suggest_id) {
   EventRouter* event_router = EventRouter::Get(profile);
   if (!event_router->ExtensionHasEventListener(
           extension_id, omnibox::OnInputChanged::kEventName))
@@ -125,7 +130,7 @@ bool ExtensionOmniboxEventRouter::OnInputChanged(
 // static
 void ExtensionOmniboxEventRouter::OnInputEntered(
     content::WebContents* web_contents,
-    const std::string& extension_id,
+    const ExtensionId& extension_id,
     const std::string& input,
     WindowOpenDisposition disposition) {
   Profile* profile =
@@ -150,6 +155,7 @@ void ExtensionOmniboxEventRouter::OnInputEntered(
   auto event = std::make_unique<Event>(events::OMNIBOX_ON_INPUT_ENTERED,
                                        omnibox::OnInputEntered::kEventName,
                                        std::move(args), profile);
+  event->user_gesture = EventRouter::USER_GESTURE_ENABLED;
   EventRouter::Get(profile)
       ->DispatchEventToExtension(extension_id, std::move(event));
 
@@ -158,7 +164,8 @@ void ExtensionOmniboxEventRouter::OnInputEntered(
 
 // static
 void ExtensionOmniboxEventRouter::OnInputCancelled(
-    Profile* profile, const std::string& extension_id) {
+    Profile* profile,
+    const ExtensionId& extension_id) {
   auto event = std::make_unique<Event>(events::OMNIBOX_ON_INPUT_CANCELLED,
                                        omnibox::OnInputCancelled::kEventName,
                                        base::Value::List(), profile);
@@ -168,7 +175,7 @@ void ExtensionOmniboxEventRouter::OnInputCancelled(
 
 void ExtensionOmniboxEventRouter::OnDeleteSuggestion(
     Profile* profile,
-    const std::string& extension_id,
+    const ExtensionId& extension_id,
     const std::string& suggestion_text) {
   base::Value::List args;
   args.Append(suggestion_text);
@@ -248,14 +255,14 @@ void OmniboxAPI::OnExtensionUnloaded(content::BrowserContext* browser_context,
   }
 }
 
-gfx::Image OmniboxAPI::GetOmniboxIcon(const std::string& extension_id) {
+gfx::Image OmniboxAPI::GetOmniboxIcon(const ExtensionId& extension_id) {
   return omnibox_icon_manager_.GetIcon(extension_id);
 }
 
 void OmniboxAPI::OnTemplateURLsLoaded() {
   // Register keywords for pending extensions.
   template_url_subscription_ = {};
-  for (const auto* i : pending_extensions_) {
+  for (const Extension* i : pending_extensions_) {
     url_service_->RegisterOmniboxKeyword(
         i->id(), i->short_name(), OmniboxInfo::GetKeyword(i),
         GetTemplateURLStringForExtension(i->id()),
@@ -279,7 +286,7 @@ ExtensionFunction::ResponseAction OmniboxSendSuggestionsFunction::Run() {
   EXTENSION_FUNCTION_VALIDATE(params_);
 
   if (is_from_service_worker() && !params_->suggest_results.empty()) {
-    std::vector<base::StringPiece> inputs;
+    std::vector<std::string_view> inputs;
     inputs.reserve(params_->suggest_results.size());
     for (const auto& suggestion : params_->suggest_results)
       inputs.push_back(suggestion.description);
