@@ -34,6 +34,7 @@ namespace extensions {
 extern const char kPasswordCheckDataKey[];
 
 class PasswordCheckProgress;
+class PasswordsPrivateEventRouter;
 
 // This class handles the part of the passwordsPrivate extension API that deals
 // with the bulk password check feature.
@@ -47,7 +48,8 @@ class PasswordCheckDelegate
 
   PasswordCheckDelegate(Profile* profile,
                         password_manager::SavedPasswordsPresenter* presenter,
-                        IdGenerator* id_generator);
+                        IdGenerator* id_generator,
+                        PasswordsPrivateEventRouter* event_router = nullptr);
   PasswordCheckDelegate(const PasswordCheckDelegate&) = delete;
   PasswordCheckDelegate& operator=(const PasswordCheckDelegate&) = delete;
   ~PasswordCheckDelegate() override;
@@ -183,6 +185,8 @@ class PasswordCheckDelegate
   // with the underlying `CredentialUIEntry` they are based on.
   raw_ptr<IdGenerator> id_generator_;
 
+  raw_ptr<PasswordsPrivateEventRouter> event_router_;
+
   // This indicate what was the reason to start the password check.
   password_manager::LeakDetectionInitiator password_check_initiator_ =
       password_manager::LeakDetectionInitiator::kClientUseCaseUnspecified;
