@@ -93,10 +93,26 @@ class TestPasswordsPrivateDelegate : public PasswordsPrivateDelegate {
       override;
   void RestartAuthTimer() override;
   void SwitchBiometricAuthBeforeFillingState(
-      content::WebContents* web_contents) override;
+      content::WebContents* web_contents,
+      AuthenticationCallback callback) override;
   void ShowAddShortcutDialog(content::WebContents* web_contents) override;
   void ShowExportedFileInShell(content::WebContents* web_contents,
                                std::string file_path) override;
+  void ChangePasswordManagerPin(
+      content::WebContents* web_contents,
+      base::OnceCallback<void(bool)> success_callback) override;
+  void IsPasswordManagerPinAvailable(
+      content::WebContents* web_contents,
+      base::OnceCallback<void(bool)> pin_available_callback) override;
+  void DisconnectCloudAuthenticator(
+      content::WebContents* web_contents,
+      base::OnceCallback<void(bool)> success_callback) override;
+  bool IsConnectedToCloudAuthenticator(
+      content::WebContents* web_contents) override;
+  void DeleteAllPasswordManagerData(
+      content::WebContents* web_contents,
+      base::OnceCallback<void(bool)> success_callback) override;
+
   base::WeakPtr<PasswordsPrivateDelegate> AsWeakPtr() override;
 
   void SetProfile(Profile* profile);
@@ -138,6 +154,18 @@ class TestPasswordsPrivateDelegate : public PasswordsPrivateDelegate {
     return exported_file_shown_in_shell_;
   }
 
+  bool get_change_password_manager_pin_called() const {
+    return change_password_manager_pin_called_;
+  }
+
+  bool get_disconnect_cloud_authenticator_called() const {
+    return disconnect_cloud_authenticator_called_;
+  }
+
+  bool get_delete_all_password_manager_data_called() const {
+    return delete_all_password_manager_data_called_;
+  }
+
  protected:
   ~TestPasswordsPrivateDelegate() override;
 
@@ -195,9 +223,18 @@ class TestPasswordsPrivateDelegate : public PasswordsPrivateDelegate {
   // Used to track whether shortcut creation dialog was shown.
   bool add_shortcut_dialog_shown_ = false;
 
-  // used to track whether the exported file was shown in shell.
+  // Used to track whether the exported file was shown in shell.
   bool exported_file_shown_in_shell_ = false;
 
+  // Used for checking whether `ChangePasswordManagerPin` is called.
+  bool change_password_manager_pin_called_ = false;
+
+  // Used to track whether `DisconnectCloudAuthenticator` was called.
+  bool disconnect_cloud_authenticator_called_ = false;
+
+  // Used to track whether `DeleteAllPasswordManagerData` was called.
+  bool delete_all_password_manager_data_called_ = false;
+
   base::WeakPtrFactory<TestPasswordsPrivateDelegate> weak_ptr_factory_{this};
 };
 }  // namespace extensions
