@@ -37,14 +37,14 @@ class SessionsGetRecentlyClosedFunction : public ExtensionFunction {
                              SESSIONS_GETRECENTLYCLOSED)
 
  private:
-  api::tabs::Tab CreateTabModel(const sessions::TabRestoreService::Tab& tab,
+  api::tabs::Tab CreateTabModel(const sessions::tab_restore::Tab& tab,
                                 bool active);
   api::windows::Window CreateWindowModel(
-      const sessions::TabRestoreService::Window& window);
+      const sessions::tab_restore::Window& window);
   api::tab_groups::TabGroup CreateGroupModel(
-      const sessions::TabRestoreService::Group& group);
+      const sessions::tab_restore::Group& group);
   api::sessions::Session CreateSessionModel(
-      const sessions::TabRestoreService::Entry& entry);
+      const sessions::tab_restore::Entry& entry);
 };
 
 class SessionsGetDevicesFunction : public ExtensionFunction {
