@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include <stddef.h>
 
 #include <memory>
@@ -22,6 +27,7 @@
 #include "chrome/browser/extensions/api/tabs/tabs_api.h"
 #include "chrome/browser/extensions/api/tabs/tabs_constants.h"
 #include "chrome/browser/extensions/extension_apitest.h"
+#include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sync/session_sync_service_factory.h"
 #include "chrome/browser/ui/tabs/tab_enums.h"
 #include "chrome/common/chrome_paths.h"
@@ -33,13 +39,12 @@
 #include "components/sync/base/client_tag_hash.h"
 #include "components/sync/engine/data_type_activation_response.h"
 #include "components/sync/model/data_type_activation_request.h"
-#include "components/sync/model/model_type_controller_delegate.h"
+#include "components/sync/model/data_type_controller_delegate.h"
 #include "components/sync/model/sync_data.h"
 #include "components/sync/protocol/entity_specifics.pb.h"
-#include "components/sync/protocol/model_type_state.pb.h"
 #include "components/sync/protocol/session_specifics.pb.h"
 #include "components/sync/protocol/sync_enums.pb.h"
-#include "components/sync/test/mock_model_type_worker.h"
+#include "components/sync/test/mock_data_type_worker.h"
 #include "components/sync_sessions/session_store.h"
 #include "components/sync_sessions/session_sync_service.h"
 #include "content/public/test/browser_test.h"
@@ -61,7 +66,8 @@ const char kTestCacheGuid[] = "TestCacheGuid";
 // (used to construct arbitrary tab info) to use in all tests.
 const char* const kSessionTags[] = {"tag0", "tag1", "tag2", "tag3", "tag4"};
 const SessionID::id_type kTabIDs[] = {5, 10, 13, 17};
-
+const int kActiveTabIndex = 2;
+const int kActiveTabId = kTabIDs[kActiveTabIndex];
 void BuildSessionSpecifics(const std::string& tag,
                            sync_pb::SessionSpecifics* meta) {
   meta->set_session_tag(tag);
@@ -76,7 +82,7 @@ void BuildWindowSpecifics(int window_id,
   sync_pb::SessionHeader* header = meta->mutable_header();
   sync_pb::SessionWindow* window = header->add_window();
   window->set_window_id(window_id);
-  window->set_selected_tab_index(0);
+  window->set_selected_tab_index(kActiveTabIndex);
   window->set_browser_type(sync_pb::SyncEnums_BrowserType_TYPE_TABBED);
   for (int tab : tab_list) {
     window->add_tab(tab);
@@ -129,11 +135,7 @@ testing::AssertionResult CheckSessionModels(const base::Value::List& devices,
       EXPECT_FALSE(tab.contains("id"));  // sessions API does not give tab IDs
       EXPECT_EQ(static_cast<int>(j), api_test_utils::GetInteger(tab, "index"));
       EXPECT_EQ(0, api_test_utils::GetInteger(tab, "windowId"));
-      // Test setup code always sets tab 0 to selected (which means active in
-      // extension terminology).
-      EXPECT_EQ(j == 0, api_test_utils::GetBoolean(tab, "active"));
-      // While selected/highlighted are different to active, and should always
-      // be false.
+      // Selected/highlighted tabs should always be false.
       EXPECT_FALSE(api_test_utils::GetBoolean(tab, "selected"));
       EXPECT_FALSE(api_test_utils::GetBoolean(tab, "highlighted"));
       EXPECT_FALSE(api_test_utils::GetBoolean(tab, "incognito"));
@@ -189,7 +191,7 @@ void ExtensionSessionsTest::SetUpOnMainThread() {
 
 void ExtensionSessionsTest::CreateTestExtension() {
   extension_ = ExtensionBuilder("Test")
-                   .AddPermissions({"sessions", "tabs"})
+                   .AddAPIPermissions({"sessions", "tabs"})
                    .SetLocation(mojom::ManifestLocation::kInternal)
                    .Build();
 }
@@ -207,13 +209,9 @@ void ExtensionSessionsTest::CreateSessionModels() {
       sync_start_future;
   service->GetControllerDelegate()->OnSyncStarting(
       request, sync_start_future.GetCallback());
-  syncer::MockModelTypeWorker worker(
-      sync_pb::ModelTypeState(), sync_start_future.Get()->type_processor.get());
-
-  // ClientTagBasedModelTypeProcessor requires connecting before other
-  // interactions with the worker happen.
-  sync_start_future.Get()->type_processor->ConnectSync(
-      worker.MakeForwardingCommitQueue());
+  std::unique_ptr<syncer::MockDataTypeWorker> worker =
+      syncer::MockDataTypeWorker::CreateWorkerAndConnectSync(
+          sync_start_future.Take());
 
   const base::Time time_now = base::Time::Now();
   syncer::SyncDataList initial_data;
@@ -232,7 +230,7 @@ void ExtensionSessionsTest::CreateSessionModels() {
 
     // We need to provide a recent timestamp to prevent garbage collection of
     // sessions (anything older than 14 days), so we cannot use
-    // MockModelTypeWorker's convenience functions, which internally use very
+    // MockDataTypeWorker's convenience functions, which internally use very
     // old timestamps.
     syncer::EntityData header_entity_data;
     header_entity_data.client_tag_hash =
@@ -248,18 +246,18 @@ void ExtensionSessionsTest::CreateSessionModels() {
     header_update.response_version = 1;
     syncer::UpdateResponseDataList updates;
     updates.push_back(std::move(header_update));
-    worker.UpdateFromServer(std::move(updates));
+    worker->UpdateFromServer(std::move(updates));
 
     for (const auto& tab : tabs) {
       sync_pb::EntitySpecifics tab_entity;
       *tab_entity.mutable_session() = tab;
-      worker.UpdateFromServer(TagHashFromSpecifics(tab_entity.session()),
-                              tab_entity);
+      worker->UpdateFromServer(TagHashFromSpecifics(tab_entity.session()),
+                               tab_entity);
     }
   }
 
   // Let the processor receive and honor all updates, which requires running
-  // the runloop because there is a ModelTypeProcessorProxy in between, posting
+  // the runloop because there is a DataTypeProcessorProxy in between, posting
   // tasks.
   base::RunLoop().RunUntilIdle();
 }
@@ -414,4 +412,69 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, DISABLED_SessionsApis) {
       << message_;
 }
 
+// Verify that the correct tab is active based on its tab ID (instead of relying
+// on tab index, since the tabs list might be filtered or sorted).
+IN_PROC_BROWSER_TEST_F(ExtensionSessionsTest, CheckActiveTabStatus) {
+  CreateSessionModels();
+
+  const base::Value::List result =
+      utils::ToList(utils::RunFunctionAndReturnSingleResult(
+          CreateFunction<SessionsGetDevicesFunction>(true).get(), "[]",
+          browser()->profile()));
+
+  ASSERT_FALSE(result.empty()) << "No devices found.";
+
+  const base::Value::Dict* device_dict = result[0].GetIfDict();
+  ASSERT_TRUE(device_dict) << "Failed to retrieve device information.";
+
+  const base::Value::List* sessions = device_dict->FindList("sessions");
+  ASSERT_TRUE(sessions) << "No sessions found in the device.";
+  ASSERT_FALSE(sessions->empty()) << "No session data found in the device.";
+
+  const base::Value::Dict* session_dict = (*sessions)[0].GetIfDict();
+  ASSERT_TRUE(session_dict) << "Failed to retrieve session information.";
+
+  const base::Value::Dict* window_dict = session_dict->FindDict("window");
+  ASSERT_TRUE(window_dict) << "Window information is missing from the session.";
+
+  const base::Value::List* tabs = window_dict->FindList("tabs");
+  ASSERT_TRUE(tabs) << "No tabs found in the session window.";
+  ASSERT_FALSE(tabs->empty()) << "Tabs list is empty.";
+
+  const base::Value::Dict* active_tab = nullptr;
+  for (const auto& tab_value : *tabs) {
+    const base::Value::Dict* tab_dict = tab_value.GetIfDict();
+    ASSERT_TRUE(tab_dict) << "Failed to retrieve tab information.";
+
+    // Extract the sessionId as a string
+    const std::string* session_id = tab_dict->FindString("sessionId");
+    ASSERT_TRUE(session_id) << "Session ID is missing.";
+
+    // Find the position of the '.' in the sessionId to separate the prefix and
+    // tabId
+    size_t dot_pos = session_id->find('.');
+    ASSERT_TRUE(dot_pos != std::string::npos) << "Invalid sessionId format.";
+
+    // Extract the tabId part from the sessionId (after the '.')
+    std::string tab_id_str = session_id->substr(dot_pos + 1);
+    // Convert string tabId to int using base::StringToInt
+    int tab_id;
+    ASSERT_TRUE(base::StringToInt(tab_id_str, &tab_id))
+        << "Failed to convert tabId to int.";
+
+    // Compare the extracted tabId with the activeTabId
+    if (tab_id == kActiveTabId) {
+      active_tab = tab_dict;
+      break;
+    }
+  }
+
+  ASSERT_TRUE(active_tab) << "Failed to retrieve active tab information.";
+
+  std::optional<bool> tab_active_status = active_tab->FindBool("active");
+  ASSERT_TRUE(tab_active_status) << "Active state for the tab is missing.";
+
+  EXPECT_TRUE(*tab_active_status) << "The selected tab should be active.";
+}
+
 }  // namespace extensions
