@@ -8,6 +8,8 @@
 #include <memory>
 #include <string>
 
+#include "content/public/browser/frame_tree_node_id.h"
+#include "extensions/common/extension_id.h"
 #include "third_party/blink/public/mojom/loader/transferrable_url_loader.mojom.h"
 
 namespace extensions {
@@ -22,10 +24,10 @@ class StreamsPrivateAPI {
   // document is embedded within another document. The |frame_tree_node_id|
   // parameter is used for the top level plugins case. (PDF, etc).
   static void SendExecuteMimeTypeHandlerEvent(
-      const std::string& extension_id,
+      const ExtensionId& extension_id,
       const std::string& stream_id,
       bool embedded,
-      int frame_tree_node_id,
+      content::FrameTreeNodeId frame_tree_node_id,
       blink::mojom::TransferrableURLLoaderPtr transferrable_loader,
       const GURL& original_url,
       const std::string& internal_id);
