@@ -26,49 +26,47 @@
 
 namespace windows_util {
 
-bool GetBrowserFromWindowID(ExtensionFunction* function,
-                            int window_id,
-                            extensions::WindowController::TypeFilter filter,
-                            Browser** browser,
-                            std::string* error) {
-  DCHECK(browser);
+bool GetControllerFromWindowID(ExtensionFunction* function,
+                               int window_id,
+                               extensions::WindowController::TypeFilter filter,
+                               extensions::WindowController** out_controller,
+                               std::string* error) {
+  DCHECK(out_controller);
   DCHECK(error);
 
-  *browser = nullptr;
+  *out_controller = nullptr;
   if (window_id == extension_misc::kCurrentWindowId) {
     // If there is a window controller associated with this extension, use that.
-    extensions::WindowController* window_controller =
-        function->dispatcher()->GetExtensionWindowController();
-    if (!window_controller) {
-      // Otherwise get the focused or most recently added window.
-      window_controller =
-          extensions::WindowControllerList::GetInstance()
-              ->CurrentWindowForFunctionWithFilter(function, filter);
+    if (extensions::WindowController* window_controller =
+            function->dispatcher()->GetExtensionWindowController()) {
+      *out_controller = window_controller;
+      return true;
     }
 
-    if (window_controller)
-      *browser = window_controller->GetBrowser();
-
-    if (!(*browser)) {
-      *error = extensions::tabs_constants::kNoCurrentWindowError;
-      return false;
+    // Otherwise get the focused or most recently added window.
+    if (extensions::WindowController* window_controller =
+            extensions::WindowControllerList::GetInstance()
+                ->CurrentWindowForFunctionWithFilter(function, filter)) {
+      *out_controller = window_controller;
+      return true;
     }
+
+    *error = extensions::tabs_constants::kNoCurrentWindowError;
+    return false;
   } else {
-    extensions::WindowController* window_controller =
-        extensions::WindowControllerList::GetInstance()
-            ->FindWindowForFunctionByIdWithFilter(function, window_id, filter);
-    if (window_controller)
-      *browser = window_controller->GetBrowser();
-
-    if (!(*browser)) {
-      *error = extensions::ErrorUtils::FormatErrorMessage(
-          extensions::tabs_constants::kWindowNotFoundError,
-          base::NumberToString(window_id));
-      return false;
+    if (extensions::WindowController* window_controller =
+            extensions::WindowControllerList::GetInstance()
+                ->FindWindowForFunctionByIdWithFilter(function, window_id,
+                                                      filter)) {
+      *out_controller = window_controller;
+      return true;
     }
+
+    *error = extensions::ErrorUtils::FormatErrorMessage(
+        extensions::tabs_constants::kWindowNotFoundError,
+        base::NumberToString(window_id));
+    return false;
   }
-  DCHECK(*browser);
-  return true;
 }
 
 bool CanOperateOnWindow(const ExtensionFunction* function,
@@ -77,7 +75,7 @@ bool CanOperateOnWindow(const ExtensionFunction* function,
   if (filter && !controller->MatchesFilter(filter))
     return false;
 
-  // TODO(https://crbug.com/807313): Remove this.
+  // TODO(crbug.com/41367902): Remove this.
   bool allow_dev_tools_windows = !!filter;
   if (function->extension() &&
       !controller->IsVisibleToTabsAPIForExtension(function->extension(),
