@@ -8,29 +8,25 @@
 #include <string>
 #include <vector>
 
-#include "chrome/browser/extensions/window_controller_list.h"
+#include "chrome/browser/extensions/window_controller.h"
 
 class ExtensionFunction;
 class Profile;
 class GURL;
 
-namespace extensions {
-class WindowController;
-}
-
 namespace windows_util {
 
-// Populates |browser| for given |window_id|. If the window is not found,
-// returns false and sets |error|.
-bool GetBrowserFromWindowID(ExtensionFunction* function,
-                            int window_id,
-                            extensions::WindowController::TypeFilter filter,
-                            Browser** browser,
-                            std::string* error);
+// Populates `*controller` for given `window_id`. If the window is not found,
+// returns false and sets `error`.
+bool GetControllerFromWindowID(ExtensionFunction* function,
+                               int window_id,
+                               extensions::WindowController::TypeFilter filter,
+                               extensions::WindowController** controller,
+                               std::string* error);
 
-// Returns true if |function| (and the profile and extension that it was
-// invoked from) can operate on the window wrapped by |window_controller|.
-// If |all_window_types| is set this function will return true for any
+// Returns true if `function` (and the profile and extension that it was
+// invoked from) can operate on the window wrapped by `window_controller`.
+// If `all_window_types` is set this function will return true for any
 // kind of window (including app and devtools), otherwise it will
 // return true only for normal browser windows as well as windows
 // created by the extension.
@@ -38,6 +34,11 @@ bool CanOperateOnWindow(const ExtensionFunction* function,
                         const extensions::WindowController* controller,
                         extensions::WindowController::TypeFilter filter);
 
+// Returns true if `function` was called from a logical child window of the
+// window wrapped by `controller`.
+bool CalledFromChildWindow(ExtensionFunction* function,
+                           const extensions::WindowController* controller);
+
 // Enum return value for `ShouldOpenIncognitoWindow`, indicating whether to use
 // incognito or the presence of an error.
 enum IncognitoResult { kRegular, kIncognito, kError };
