@@ -26,6 +26,12 @@
 #include "extensions/browser/view_type_utils.h"
 #include "extensions/common/mojom/view_type.mojom.h"
 #include "net/base/net_errors.h"
+#include "pdf/buildflags.h"
+
+#if BUILDFLAG(ENABLE_PDF)
+#include "components/pdf/common/pdf_util.h"
+#include "pdf/pdf_features.h"
+#endif  // BUILDFLAG(ENABLE_PDF)
 
 namespace GetFrame = extensions::api::web_navigation::GetFrame;
 namespace GetAllFrames = extensions::api::web_navigation::GetAllFrames;
@@ -573,6 +579,19 @@ ExtensionFunction::ResponseAction WebNavigationGetAllFramesFunction::Run() {
                   kSkipChildren;
             }
 
+#if BUILDFLAG(ENABLE_PDF)
+            if (chrome_pdf::features::IsOopifPdfEnabled()) {
+              // Don't expose any child frames of the PDF extension frame, such
+              // as the PDF content frame.
+              content::RenderFrameHost* parent = render_frame_host->GetParent();
+              if (parent &&
+                  IsPdfExtensionOrigin(parent->GetLastCommittedOrigin())) {
+                return content::RenderFrameHost::FrameIterationAction::
+                    kSkipChildren;
+              }
+            }
+#endif  // BUILDFLAG(ENABLE_PDF)
+
             auto* navigation_state =
                 FrameNavigationState::GetForCurrentDocument(render_frame_host);
 
