@@ -5,6 +5,7 @@
 #include "chrome/browser/media/webrtc/capture_policy_utils.h"
 
 #include "base/containers/contains.h"
+#include "base/test/test_future.h"
 #include "base/values.h"
 #include "chrome/browser/content_settings/host_content_settings_map_factory.h"
 #include "chrome/common/pref_names.h"
@@ -17,10 +18,27 @@
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "chrome/browser/ash/crosapi/crosapi_manager.h"
+#include "chrome/browser/ash/crosapi/idle_service_ash.h"
+#include "chrome/browser/ash/crosapi/test_crosapi_dependency_registry.h"
+#include "chrome/browser/ash/login/users/fake_chrome_user_manager.h"
+#include "chrome/test/base/testing_browser_process.h"
+#include "chrome/test/base/testing_profile_manager.h"
+#include "chromeos/ash/components/login/login_state/login_state.h"
+#include "components/account_id/account_id.h"
+#include "components/user_manager/scoped_user_manager.h"
+#include "components/user_manager/user_type.h"
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
 namespace {
 constexpr char kTestSite1[] = "https://foo.test.org";
 constexpr char kTestSite1Pattern[] = "foo.test.org";
 constexpr char kTestSite1NonMatchingPattern[] = "foo.org";
+
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+constexpr char kAccountId[] = "test_1@example.com";
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 }  // namespace
 
 class CapturePolicyUtilsTest : public testing::Test {
@@ -261,7 +279,7 @@ TEST_F(CapturePolicyUtilsTest, FilterMediaListRestrictedSameOrigin) {
   EXPECT_EQ(expected_media_types, actual_media_types);
 }
 
-#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_LINUX)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
 
 class MultiCaptureTest
     : public testing::Test,
@@ -271,11 +289,22 @@ class MultiCaptureTest
   void SetUp() override {
     testing::Test::SetUp();
 
-    TestingProfile::Builder builder;
-#if BUILDFLAG(IS_CHROMEOS_LACROS)
-    builder.SetIsMainProfile(IsMainProfile());
-#endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
-    profile_ = builder.Build();
+    fake_user_manager_.Reset(std::make_unique<ash::FakeChromeUserManager>());
+    CHECK(profile_manager_.SetUp());
+    profile_ = profile_manager_.CreateTestingProfile(kAccountId);
+
+    AccountId account_id = AccountId::FromUserEmail(kAccountId);
+    fake_user_manager_->AddUserWithAffiliationAndTypeAndProfile(
+        account_id, /*is_affiliated=*/true, user_manager::UserType::kRegular,
+        profile_);
+    fake_user_manager_->LoginUser(account_id);
+
+    // Settings required to create startup data.
+    crosapi::IdleServiceAsh::DisableForTesting();
+    if (!ash::LoginState::IsInitialized()) {
+      ash::LoginState::Initialize();
+    }
+    cros_api_manager_ = crosapi::CreateCrosapiManagerWithTestRegistry();
 
     HostContentSettingsMap* content_settings =
         HostContentSettingsMapFactory::GetForProfile(profile());
@@ -286,9 +315,12 @@ class MultiCaptureTest
     }
   }
 
-  void TearDown() override { profile_.reset(); }
+  void TearDown() override {
+    profile_ = nullptr;
+    // ash::LoginState::Shutdown();
+  }
 
-  Profile* profile() { return profile_.get(); }
+  Profile* profile() { return profile_; }
   bool IsMainProfile() const { return std::get<0>(GetParam()); }
   std::vector<std::string> AllowedOrigins() const {
     return std::get<1>(GetParam());
@@ -314,19 +346,28 @@ class MultiCaptureTest
   }
 
  private:
-  std::unique_ptr<TestingProfile> profile_;
+  raw_ptr<TestingProfile> profile_;
   content::BrowserTaskEnvironment task_environment_;
+  std::unique_ptr<crosapi::CrosapiManager> cros_api_manager_;
+  user_manager::TypedScopedUserManager<ash::FakeChromeUserManager>
+      fake_user_manager_;
+  TestingProfileManager profile_manager_{TestingBrowserProcess::GetGlobal()};
 };
 
 TEST_P(MultiCaptureTest, IsMultiCaptureAllowedBasedOnPolicy) {
-  EXPECT_EQ(ExpectedIsMultiCaptureAllowed(),
-            capture_policy::IsGetAllScreensMediaAllowed(profile(),
-                                                        GURL(CurrentOrigin())));
+  base::test::TestFuture<bool> future;
+  capture_policy::CheckGetAllScreensMediaAllowed(
+      profile(), GURL(CurrentOrigin()), future.GetCallback());
+  ASSERT_TRUE(future.Wait());
+  EXPECT_EQ(ExpectedIsMultiCaptureAllowed(), future.Get<bool>());
 }
 
 TEST_P(MultiCaptureTest, IsMultiCaptureAllowedForAnyUrl) {
-  EXPECT_EQ(ExpectedIsMultiCaptureAllowedForAnyUrl(),
-            capture_policy::IsGetAllScreensMediaAllowedForAnySite(profile()));
+  base::test::TestFuture<bool> future;
+  capture_policy::CheckGetAllScreensMediaAllowedForAnyOrigin(
+      profile(), future.GetCallback());
+  ASSERT_TRUE(future.Wait());
+  EXPECT_EQ(ExpectedIsMultiCaptureAllowedForAnyUrl(), future.Get<bool>());
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -343,4 +384,4 @@ INSTANTIATE_TEST_SUITE_P(
         ::testing::ValuesIn({std::string("https://www.google.com"),
                              std::string("https://www.notallowed.com")})));
 
-#endif  // BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_LINUX)
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
