@@ -58,7 +58,7 @@ class DesktopCaptureAccessHandlerTest : public ChromeRenderViewHostTestHarness {
   }
 
   void ProcessGenerateStreamRequest(
-      const std::string& requested_video_device_id,
+      const std::vector<std::string>& requested_video_device_ids,
       const GURL& origin,
       const extensions::Extension* extension,
       blink::mojom::MediaStreamRequestResult* request_result,
@@ -74,11 +74,12 @@ class DesktopCaptureAccessHandlerTest : public ChromeRenderViewHostTestHarness {
         web_contents()->GetPrimaryMainFrame()->GetRoutingID(),
         /*page_request_id=*/0, url::Origin::Create(origin),
         /*user_gesture=*/false, blink::MEDIA_GENERATE_STREAM,
-        /*requested_audio_device_id=*/std::string(), requested_video_device_id,
+        /*requested_audio_device_ids=*/{}, requested_video_device_ids,
         blink::mojom::MediaStreamType::NO_SERVICE,
         blink::mojom::MediaStreamType::GUM_DESKTOP_VIDEO_CAPTURE,
         /*disable_local_echo=*/false,
-        /*request_pan_tilt_zoom_permission=*/false);
+        /*request_pan_tilt_zoom_permission=*/false,
+        /*captured_surface_control_active=*/false);
     base::RunLoop wait_loop;
     content::MediaResponseCallback callback = base::BindOnce(
         [](base::RunLoop* wait_loop, bool expect_result,
@@ -126,10 +127,12 @@ class DesktopCaptureAccessHandlerTest : public ChromeRenderViewHostTestHarness {
                       : blink::mojom::MediaStreamType::NO_SERVICE;
     content::MediaStreamRequest request(
         0, 0, 0, url::Origin::Create(GURL(kOrigin)), false, request_type,
-        std::string(), std::string(), audio_type,
+        /*requested_audio_device_ids=*/{},
+        /*requested_video_device_ids=*/{}, audio_type,
         blink::mojom::MediaStreamType::GUM_DESKTOP_VIDEO_CAPTURE,
         /*disable_local_echo=*/false,
-        /*request_pan_tilt_zoom_permission=*/false);
+        /*request_pan_tilt_zoom_permission=*/false,
+        /*captured_surface_control_active=*/false);
 
     base::RunLoop wait_loop;
     content::MediaResponseCallback callback = base::BindOnce(
@@ -235,9 +238,10 @@ TEST_F(DesktopCaptureAccessHandlerTest,
   content::MediaStreamRequest request(
       render_process_id, render_frame_id, page_request_id,
       url::Origin::Create(GURL(kOrigin)), false, blink::MEDIA_DEVICE_UPDATE,
-      std::string(), std::string(), blink::mojom::MediaStreamType::NO_SERVICE,
-      stream_type,
-      /*disable_local_echo=*/false, /*request_pan_tilt_zoom_permission=*/false);
+      /*requested_audio_device_ids=*/{}, /*requested_video_device_ids=*/{},
+      blink::mojom::MediaStreamType::NO_SERVICE, stream_type,
+      /*disable_local_echo=*/false, /*request_pan_tilt_zoom_permission=*/false,
+      /*captured_surface_control_active=*/false);
   content::MediaResponseCallback callback;
   access_handler_->HandleRequest(web_contents(), request, std::move(callback),
                                  nullptr /* extension */);
@@ -267,10 +271,12 @@ TEST_F(DesktopCaptureAccessHandlerTest, ChangeSourceWebContentsDestroyed) {
   picker_factory_->SetTestFlags(test_flags, std::size(test_flags));
   content::MediaStreamRequest request(
       0, 0, 0, url::Origin::Create(GURL(kOrigin)), false,
-      blink::MEDIA_DEVICE_UPDATE, std::string(), std::string(),
+      blink::MEDIA_DEVICE_UPDATE, /*requested_audio_device_ids=*/{},
+      /*requested_video_device_ids=*/{},
       blink::mojom::MediaStreamType::NO_SERVICE,
       blink::mojom::MediaStreamType::GUM_DESKTOP_VIDEO_CAPTURE,
-      /*disable_local_echo=*/false, /*request_pan_tilt_zoom_permission=*/false);
+      /*disable_local_echo=*/false, /*request_pan_tilt_zoom_permission=*/false,
+      /*captured_surface_control_active=*/false);
   content::MediaResponseCallback callback;
   access_handler_->HandleRequest(web_contents(), request, std::move(callback),
                                  nullptr /* extension */);
@@ -308,11 +314,13 @@ TEST_F(DesktopCaptureAccessHandlerTest, ChangeSourceMultipleRequests) {
   for (base::RunLoop& loop : wait_loop) {
     content::MediaStreamRequest request(
         0, 0, 0, url::Origin::Create(GURL(kOrigin)), false,
-        blink::MEDIA_DEVICE_UPDATE, std::string(), std::string(),
+        blink::MEDIA_DEVICE_UPDATE, /*requested_audio_device_ids=*/{},
+        /*requested_video_device_ids=*/{},
         blink::mojom::MediaStreamType::NO_SERVICE,
         blink::mojom::MediaStreamType::GUM_DESKTOP_VIDEO_CAPTURE,
         /*disable_local_echo=*/false,
-        /*request_pan_tilt_zoom_permission=*/false);
+        /*request_pan_tilt_zoom_permission=*/false,
+        /*captured_surface_control_active=*/false);
     content::MediaResponseCallback callback = base::BindOnce(
         [](base::RunLoop* wait_loop,
            blink::mojom::MediaStreamRequestResult* request_result,
@@ -364,7 +372,7 @@ TEST_F(DesktopCaptureAccessHandlerTest, GenerateStreamSuccess) {
                                   content::DesktopMediaID::kFakeId),
           content::DesktopStreamRegistryType::kRegistryStreamTypeDesktop);
 
-  ProcessGenerateStreamRequest(id, origin, /*extension=*/nullptr, &result,
+  ProcessGenerateStreamRequest({id}, origin, /*extension=*/nullptr, &result,
                                &devices);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::OK, result);
@@ -390,9 +398,9 @@ TEST_F(DesktopCaptureAccessHandlerTest, ScreenCaptureAccessSuccess) {
   blink::mojom::MediaStreamRequestResult result;
   blink::mojom::StreamDevices devices;
 
-  ProcessGenerateStreamRequest(/*requested_video_device_id=*/std::string(),
-                               GURL(kOrigin), extensionBuilder.Build().get(),
-                               &result, &devices);
+  ProcessGenerateStreamRequest(/*requested_video_device_ids=*/{}, GURL(kOrigin),
+                               extensionBuilder.Build().get(), &result,
+                               &devices);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::OK, result);
   EXPECT_TRUE(devices.video_device.has_value());
@@ -425,9 +433,9 @@ TEST_F(DesktopCaptureAccessHandlerTest, ScreenCaptureAccessDlpRestricted) {
       blink::mojom::MediaStreamRequestResult::NOT_SUPPORTED;
   blink::mojom::StreamDevices devices;
 
-  ProcessGenerateStreamRequest(/*requested_video_device_id=*/std::string(),
-                               GURL(kOrigin), extensionBuilder.Build().get(),
-                               &result, &devices);
+  ProcessGenerateStreamRequest(/*requested_video_device_ids=*/{}, GURL(kOrigin),
+                               extensionBuilder.Build().get(), &result,
+                               &devices);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::PERMISSION_DENIED, result);
   EXPECT_FALSE(devices.video_device.has_value());
@@ -459,9 +467,9 @@ TEST_F(DesktopCaptureAccessHandlerTest, ScreenCaptureAccessDlpNotRestricted) {
       blink::mojom::MediaStreamRequestResult::NOT_SUPPORTED;
   blink::mojom::StreamDevices devices;
 
-  ProcessGenerateStreamRequest(/*requested_video_device_id=*/std::string(),
-                               GURL(kOrigin), extensionBuilder.Build().get(),
-                               &result, &devices);
+  ProcessGenerateStreamRequest(/*requested_video_device_id=*/{}, GURL(kOrigin),
+                               extensionBuilder.Build().get(), &result,
+                               &devices);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::OK, result);
   EXPECT_TRUE(devices.video_device.has_value());
@@ -496,9 +504,9 @@ TEST_F(DesktopCaptureAccessHandlerTest,
       blink::mojom::MediaStreamRequestResult::NOT_SUPPORTED;
   blink::mojom::StreamDevices devices;
 
-  ProcessGenerateStreamRequest(/*requested_video_device_id=*/std::string(),
-                               GURL(kOrigin), extensionBuilder.Build().get(),
-                               &result, &devices, /*expect_result=*/false);
+  ProcessGenerateStreamRequest(/*requested_video_device_id=*/{}, GURL(kOrigin),
+                               extensionBuilder.Build().get(), &result,
+                               &devices, /*expect_result=*/false);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::NOT_SUPPORTED, result);
   EXPECT_FALSE(devices.video_device.has_value());
@@ -529,7 +537,7 @@ TEST_F(DesktopCaptureAccessHandlerTest, GenerateStreamDlpRestricted) {
       blink::mojom::MediaStreamRequestResult::NOT_SUPPORTED;
   blink::mojom::StreamDevices devices;
 
-  ProcessGenerateStreamRequest(id, GURL(kOrigin), /*extension=*/nullptr,
+  ProcessGenerateStreamRequest({id}, GURL(kOrigin), /*extension=*/nullptr,
                                &result, &devices);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::PERMISSION_DENIED, result);
@@ -561,7 +569,7 @@ TEST_F(DesktopCaptureAccessHandlerTest, GenerateStreamDlpNotRestricted) {
       blink::mojom::MediaStreamRequestResult::NOT_SUPPORTED;
   blink::mojom::StreamDevices devices;
 
-  ProcessGenerateStreamRequest(id, GURL(kOrigin), /*extension=*/nullptr,
+  ProcessGenerateStreamRequest({id}, GURL(kOrigin), /*extension=*/nullptr,
                                &result, &devices);
 
   EXPECT_EQ(blink::mojom::MediaStreamRequestResult::OK, result);
