@@ -10,6 +10,7 @@
 #include "base/metrics/field_trial.h"
 #include "base/metrics/field_trial_params.h"
 #include "base/run_loop.h"
+#include "base/test/scoped_feature_list.h"
 #include "chrome/browser/content_settings/host_content_settings_map_factory.h"
 #include "chrome/browser/media/webrtc/media_capture_devices_dispatcher.h"
 #include "chrome/browser/media/webrtc/media_stream_capture_indicator.h"
@@ -26,6 +27,7 @@
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/content_settings/browser/page_specific_content_settings.h"
 #include "components/content_settings/core/browser/host_content_settings_map.h"
+#include "components/content_settings/core/common/features.h"
 #include "components/permissions/contexts/camera_pan_tilt_zoom_permission_context.h"
 #include "components/permissions/permission_context_base.h"
 #include "components/permissions/permission_manager.h"
@@ -53,6 +55,11 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
         example_video_id_("fake_video_dev"),
         media_stream_result_(
             blink::mojom::MediaStreamRequestResult::NUM_MEDIA_REQUEST_RESULTS) {
+    // `kLeftHandSideActivityIndicators` should be disabled as it changes the UI
+    // of the camera/mic activity indicator. The new UI will be tested
+    // separately.
+    scoped_feature_list_.InitWithFeatures(
+        {}, {content_settings::features::kLeftHandSideActivityIndicators});
   }
 
   void OnMediaStreamResponse(
@@ -178,9 +185,11 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
         GetWebContents()->GetPrimaryMainFrame()->GetRoutingID();
     return content::MediaStreamRequest(
         render_process_id, render_frame_id, 0,
-        url::Origin::Create(example_url()), false, request_type, audio_id,
-        video_id, audio_type, video_type,
-        /*disable_local_echo=*/false, request_pan_tilt_zoom_permission);
+        url::Origin::Create(example_url()), false, request_type,
+        /*requested_audio_device_ids=*/{audio_id},
+        /*requested_video_device_ids=*/{video_id}, audio_type, video_type,
+        /*disable_local_echo=*/false, request_pan_tilt_zoom_permission,
+        /*captured_surface_control_active=*/false);
   }
 
   content::MediaStreamRequest CreateRequest(
@@ -189,14 +198,14 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
       bool request_pan_tilt_zoom_permission) {
     return CreateRequestWithType(audio_id, video_id,
                                  request_pan_tilt_zoom_permission,
-                                 blink::MEDIA_DEVICE_ACCESS);
+                                 blink::MEDIA_GENERATE_STREAM);
   }
 
   void InitWithUrl(const GURL& url) {
     DCHECK(example_url_.is_empty());
     example_url_ = url;
     ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), example_url_));
-    EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Empty());
+    EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().empty());
   }
 
   virtual media::VideoCaptureControlSupport GetControlSupport() const {
@@ -276,6 +285,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
 
   std::unique_ptr<PermissionBubbleMediaAccessHandler>
       permission_bubble_media_access_handler_;
+  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 class MediaStreamDevicesControllerPtzTest
@@ -956,7 +966,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
 
   VerifyResultState(blink::mojom::MediaStreamRequestResult::PERMISSION_DENIED,
                     false, false);
-  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Empty());
+  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().empty());
 }
 
 IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
@@ -987,7 +997,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
 
   VerifyResultState(blink::mojom::MediaStreamRequestResult::PERMISSION_DENIED,
                     false, false);
-  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Empty());
+  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().empty());
 }
 
 IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
