@@ -11,6 +11,7 @@
 #include <cmath>
 #include <deque>
 #include <optional>
+#include <unordered_map>
 
 #include "base/apple/bridging.h"
 #include "base/apple/foundation_util.h"
@@ -23,10 +24,14 @@
 #include "base/logging.h"
 #include "base/memory/raw_ptr.h"
 #include "base/metrics/field_trial_params.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/strings/sys_string_conversions.h"
 #include "base/task/bind_post_task.h"
 #include "base/timer/timer.h"
+#include "chrome/browser/media/webrtc/delegated_source_list_capturer.h"
+#include "chrome/browser/media/webrtc/desktop_capturer_wrapper.h"
 #include "chrome/browser/media/webrtc/desktop_media_list_base.h"
+#include "media/base/media_switches.h"
 #include "media/capture/video_capture_types.h"
 #include "third_party/webrtc/modules/desktop_capture/mac/desktop_frame_utils.h"
 
@@ -181,7 +186,7 @@ const base::FeatureParam<CaptureMode>::Option capture_mode_options[] = {
     {CaptureMode::kSCScreenshotManager, "sc_screenshot_manager"},
 };
 const base::FeatureParam<CaptureMode> kThumbnailCapturerMacCaptureMode{
-    &kThumbnailCapturerMac, "capture_mode", CaptureMode::kSCStream,
+    &kThumbnailCapturerMac, "capture_mode", CaptureMode::kSCScreenshotManager,
     &capture_mode_options};
 
 CaptureMode GetCaptureModeFeatureParam() {
@@ -191,6 +196,22 @@ CaptureMode GetCaptureModeFeatureParam() {
   return CaptureMode::kSCStream;
 }
 
+content::DesktopMediaID::Type ConvertToDesktopMediaIDType(
+    DesktopMediaList::Type type) {
+  switch (type) {
+    case DesktopMediaList::Type::kScreen:
+      return content::DesktopMediaID::Type::TYPE_SCREEN;
+    case DesktopMediaList::Type::kWindow:
+      return content::DesktopMediaID::Type::TYPE_WINDOW;
+    case DesktopMediaList::Type::kWebContents:
+    case DesktopMediaList::Type::kCurrentTab:
+      return content::DesktopMediaID::Type::TYPE_WEB_CONTENTS;
+    case DesktopMediaList::Type::kNone:
+      break;
+  }
+  NOTREACHED_NORETURN();
+}
+
 // The sort mode controls the order of the source list that is returned from
 // GetSourceList().
 enum class SortMode {
@@ -365,6 +386,8 @@ void ScreenshotManagerCapturer::SelectSources(
     gfx::Size thumbnail_size) {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
 
+  thumbnail_size_ = thumbnail_size;
+
   // The iteration is in reverse order so that the sources
   // first in the list are captured first. This way we make sure that the first
   // thumbnails in the view are captured first.
@@ -420,10 +443,9 @@ void ScreenshotManagerCapturer::CaptureSource(
         return;
       }
 
-      NSArray<SCWindow*>* exclude_windows = nil;
       SCContentFilter* filter =
           [[SCContentFilter alloc] initWithDisplay:selected_display
-                                  excludingWindows:exclude_windows];
+                                  excludingWindows:@[]];
       SCScreenshotCaptureSource(filter, [selected_display frame],
                                 [selected_display displayID]);
       break;
@@ -444,7 +466,7 @@ void ScreenshotManagerCapturer::CaptureSource(
     case DesktopMediaList::Type::kNone:
     case DesktopMediaList::Type::kWebContents:
     case DesktopMediaList::Type::kCurrentTab:
-      NOTREACHED_NORETURN();
+      NOTREACHED();
   }
 }
 
@@ -507,7 +529,7 @@ class API_AVAILABLE(macos(13.2)) ThumbnailCapturerMac
     : public ThumbnailCapturer {
  public:
   explicit ThumbnailCapturerMac(DesktopMediaList::Type type);
-  ~ThumbnailCapturerMac() override {}
+  ~ThumbnailCapturerMac() override;
 
   void Start(Consumer* callback) override;
 
@@ -564,6 +586,8 @@ class API_AVAILABLE(macos(13.2)) ThumbnailCapturerMac
   int max_frame_rate_;
   const int minimum_window_size_;
   raw_ptr<Consumer> consumer_;
+  int shareable_content_callbacks_ = 0;
+  int shareable_content_errors_ = 0;
 
   // A cache of the shareable windows and shareable displays. sharable_windows_
   // is used to produce the source list. shareable_displays_ is used to
@@ -593,6 +617,18 @@ ThumbnailCapturerMac::ThumbnailCapturerMac(DesktopMediaList::Type type)
         type_ == DesktopMediaList::Type::kScreen);
 }
 
+ThumbnailCapturerMac::~ThumbnailCapturerMac() {
+  if (shareable_content_callbacks_ > 0) {
+    // Round upwards so that a single callback with error will show up in the
+    // histogram as greater than 0%.
+    int error_percentage = static_cast<int>(std::ceil(
+        (100.0 * shareable_content_errors_) / shareable_content_callbacks_));
+    base::UmaHistogramPercentage(
+        "Media.ThumbnailCapturerMac.ShareableContentErrorPercentage",
+        error_percentage);
+  }
+}
+
 void ThumbnailCapturerMac::Start(Consumer* consumer) {
   consumer_ = consumer;
   task_runner_ = base::SingleThreadTaskRunner::GetCurrentDefault();
@@ -636,7 +672,7 @@ bool ThumbnailCapturerMac::GetSourceList(SourceList* sources) {
     case DesktopMediaList::Type::kNone:
     case DesktopMediaList::Type::kWebContents:
     case DesktopMediaList::Type::kCurrentTab:
-      NOTREACHED_NORETURN();
+      NOTREACHED();
   }
 
   return true;
@@ -693,7 +729,7 @@ void ThumbnailCapturerMac::UpdateWindowsList() {
                           weak_factory_.GetWeakPtr()));
 
   auto handler = ^(SCShareableContent* content, NSError* error) {
-    content_callback.Run(content);
+    content_callback.Run(error ? nil : content);
   };
 
   // Exclude desktop windows (e.g., background image and deskktop icons) and
@@ -708,14 +744,17 @@ void ThumbnailCapturerMac::OnRecurrentShareableContent(
     SCShareableContent* content) {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
 
+  ++shareable_content_callbacks_;
   if (!content) {
+    DVLOG(2) << "Could not get shareable content.";
+    ++shareable_content_errors_;
     return;
   }
 
   shareable_displays_ = [content displays];
   UpdateShareableWindows([content windows]);
 
-  // TODO(https://crbug.com/1471931): Only call update if the list is changed:
+  // TODO(crbug.com/40278456): Only call update if the list is changed:
   // windows opened/closed, order of the list, and title.
   consumer_->OnSourceListUpdated();
 }
@@ -952,16 +991,29 @@ void ThumbnailCapturerMac::OnCapturedFrame(
                                       source_id);
 }
 
+bool ShouldUseSCContentSharingPicker() {
+  if (@available(macOS 15.0, *)) {
+    if (base::FeatureList::IsEnabled(media::kUseSCContentSharingPicker)) {
+      return true;
+    }
+  }
+  return false;
+}
+
 }  // namespace
 
 bool ShouldUseThumbnailCapturerMac(DesktopMediaList::Type type) {
-  if (@available(macOS 14.0, *)) {
+  // There was a bug in ScreenCaptureKit that was fixed in 14.4,
+  // see b/40076027.
+  if (@available(macOS 14.4, *)) {
     switch (type) {
       case DesktopMediaList::Type::kWindow:
-        return base::FeatureList::IsEnabled(
-            kScreenCaptureKitStreamPickerSonoma);
+        return ShouldUseSCContentSharingPicker() ||
+               base::FeatureList::IsEnabled(
+                   kScreenCaptureKitStreamPickerSonoma);
       case DesktopMediaList::Type::kScreen:
-        return base::FeatureList::IsEnabled(kScreenCaptureKitPickerScreen);
+        return ShouldUseSCContentSharingPicker() ||
+               base::FeatureList::IsEnabled(kScreenCaptureKitPickerScreen);
       case DesktopMediaList::Type::kNone:
       case DesktopMediaList::Type::kCurrentTab:
       case DesktopMediaList::Type::kWebContents:
@@ -979,8 +1031,13 @@ bool ShouldUseThumbnailCapturerMac(DesktopMediaList::Type type) {
 std::unique_ptr<ThumbnailCapturer> CreateThumbnailCapturerMac(
     DesktopMediaList::Type type) {
   CHECK(ShouldUseThumbnailCapturerMac(type));
+  if (ShouldUseSCContentSharingPicker()) {
+    return std::make_unique<DesktopCapturerWrapper>(
+        std::make_unique<DelegatedSourceListCapturer>(
+            ConvertToDesktopMediaIDType(type)));
+  }
   if (@available(macOS 13.2, *)) {
     return std::make_unique<ThumbnailCapturerMac>(type);
   }
-  NOTREACHED_NORETURN();
+  NOTREACHED();
 }
