@@ -14,8 +14,9 @@
 #include "testing/perf/perf_result_reporter.h"
 #include "testing/perf/perf_test.h"
 
-static std::string Statistic(const std::string& statistic,
-                             const std::string& bucket) {
+namespace {
+
+std::string Statistic(const std::string& statistic, const std::string& bucket) {
   // A ssrc stats key will be on the form stats.<bucket>-<key>.values.
   // This will give a json "path" which will dig into the time series for the
   // specified statistic. Buckets can be for instance ssrc_1212344, bweforvideo,
@@ -25,9 +26,9 @@ static std::string Statistic(const std::string& statistic,
                             statistic.c_str());
 }
 
-static void MaybePrintResultsForAudioReceive(const std::string& ssrc,
-                                             const base::Value::Dict& pc_dict,
-                                             const std::string& modifier) {
+void MaybePrintResultsForAudioReceive(const std::string& ssrc,
+                                      const base::Value::Dict& pc_dict,
+                                      const std::string& modifier) {
   const std::string* value =
       pc_dict.FindStringByDottedPath(Statistic("audioOutputLevel", ssrc));
   if (!value) {
@@ -60,9 +61,9 @@ static void MaybePrintResultsForAudioReceive(const std::string& ssrc,
                          *value, "%", false);
 }
 
-static void MaybePrintResultsForAudioSend(const std::string& ssrc,
-                                          const base::Value::Dict& pc_dict,
-                                          const std::string& modifier) {
+void MaybePrintResultsForAudioSend(const std::string& ssrc,
+                                   const base::Value::Dict& pc_dict,
+                                   const std::string& modifier) {
   const std::string* value =
       pc_dict.FindStringByDottedPath(Statistic("audioInputLevel", ssrc));
   if (!value) {
@@ -84,9 +85,9 @@ static void MaybePrintResultsForAudioSend(const std::string& ssrc,
                          *value, "packets", false);
 }
 
-static void MaybePrintResultsForVideoSend(const std::string& ssrc,
-                                          const base::Value::Dict& pc_dict,
-                                          const std::string& modifier) {
+void MaybePrintResultsForVideoSend(const std::string& ssrc,
+                                   const base::Value::Dict& pc_dict,
+                                   const std::string& modifier) {
   const std::string* value =
       pc_dict.FindStringByDottedPath(Statistic("googFrameRateSent", ssrc));
   if (!value) {
@@ -138,9 +139,9 @@ static void MaybePrintResultsForVideoSend(const std::string& ssrc,
                          "goog_encode_usage_percent", *value, "%", false);
 }
 
-static void MaybePrintResultsForVideoReceive(const std::string& ssrc,
-                                             const base::Value::Dict& pc_dict,
-                                             const std::string& modifier) {
+void MaybePrintResultsForVideoReceive(const std::string& ssrc,
+                                      const base::Value::Dict& pc_dict,
+                                      const std::string& modifier) {
   const std::string* value =
       pc_dict.FindStringByDottedPath(Statistic("googFrameRateReceived", ssrc));
   if (!value) {
@@ -198,7 +199,7 @@ static void MaybePrintResultsForVideoReceive(const std::string& ssrc,
                          "ms", false);
 }
 
-static std::string ExtractSsrcIdentifier(const std::string& key) {
+std::string ExtractSsrcIdentifier(const std::string& key) {
   // Example key: ssrc_1234-someStatName. Grab the part before the dash.
   size_t key_start_pos = 0;
   size_t key_end_pos = key.find("-");
@@ -208,7 +209,7 @@ static std::string ExtractSsrcIdentifier(const std::string& key) {
 
 // Returns the set of unique ssrc identifiers in the call (e.g. ssrc_1234,
 // ssrc_12356, etc). |stats_dict| is the .stats dict from one peer connection.
-static std::set<std::string> FindAllSsrcIdentifiers(
+std::set<std::string> FindAllSsrcIdentifiers(
     const base::Value::Dict& stats_dict) {
   std::set<std::string> result;
   for (auto kv : stats_dict) {
@@ -218,6 +219,8 @@ static std::set<std::string> FindAllSsrcIdentifiers(
   return result;
 }
 
+}  // namespace
+
 namespace test {
 
 void PrintBweForVideoMetrics(const base::Value::Dict& pc_dict,
