@@ -1,4 +1,4 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,13 +6,13 @@
 
 #include <limits>
 
-#include "base/bind.h"
-#include "base/callback_helpers.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/memory/ptr_util.h"
-#include "base/task/post_task.h"
+#include "base/task/sequenced_task_runner.h"
 #include "base/task/thread_pool.h"
 #include "build/build_config.h"
-#if !defined(TOOLKIT_QT)
+#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "chrome/browser/browser_process.h"
 #endif
 #include "chrome/browser/profiles/profile.h"
@@ -23,15 +23,17 @@
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/network_service_instance.h"
+#include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 
 namespace webrtc_event_logging {
 
 namespace {
 
-using BrowserContext = content::BrowserContext;
-using BrowserThread = content::BrowserThread;
-using RenderProcessHost = content::RenderProcessHost;
+using content::BrowserContext;
+using content::BrowserThread;
+using content::RenderFrameHost;
+using content::RenderProcessHost;
 
 using BrowserContextId = WebRtcEventLogManager::BrowserContextId;
 
@@ -63,7 +65,8 @@ class PeerConnectionTrackerProxyImpl
       WebRtcEventLogPeerConnectionKey key,
       int output_period_ms) {
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
-    RenderProcessHost* host = RenderProcessHost::FromID(key.render_process_id);
+    auto* host =
+        RenderFrameHost::FromID(key.render_process_id, key.render_frame_id);
     if (!host) {
       return;  // The host has been asynchronously removed; not a problem.
     }
@@ -73,7 +76,8 @@ class PeerConnectionTrackerProxyImpl
   static void DisableWebRtcEventLoggingInternal(
       WebRtcEventLogPeerConnectionKey key) {
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
-    RenderProcessHost* host = RenderProcessHost::FromID(key.render_process_id);
+    auto* host =
+        RenderFrameHost::FromID(key.render_process_id, key.render_frame_id);
     if (!host) {
       return;  // The host has been asynchronously removed; not a problem.
     }
@@ -86,7 +90,7 @@ class PeerConnectionTrackerProxyImpl
 // 1. Certain platforms (mobile) are blocked from remote-bound logging.
 // 2. There is a Finch-controlled kill-switch for the feature.
 bool IsRemoteLoggingFeatureEnabled() {
-#if defined(OS_ANDROID) || defined(TOOLKIT_QT)
+#if BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_QTWEBENGINE)
   bool enabled = false;
 #else
   bool enabled = base::FeatureList::IsEnabled(features::kWebRtcRemoteEventLog);
@@ -110,8 +114,8 @@ inline void MaybeReply(const base::Location& location,
                        base::OnceCallback<void(Args...)> reply,
                        Args... args) {
   if (reply) {
-    base::PostTask(location, {BrowserThread::UI},
-                   base::BindOnce(std::move(reply), args...));
+    content::GetUIThreadTaskRunner({})->PostTask(
+        location, base::BindOnce(std::move(reply), args...));
   }
 }
 
@@ -229,160 +233,53 @@ void WebRtcEventLogManager::DisableForBrowserContext(
           std::move(reply)));
 }
 
-void WebRtcEventLogManager::PeerConnectionAdded(
-    int render_process_id,
+void WebRtcEventLogManager::OnPeerConnectionAdded(
+    content::GlobalRenderFrameHostId frame_id,
     int lid,
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-
-  RenderProcessHost* rph = RenderProcessHost::FromID(render_process_id);
-  if (!rph) {
-    // RPH died before processing of this notification.
-    MaybeReply(FROM_HERE, std::move(reply), false);
-    return;
-  }
-
-  auto it = observed_render_process_hosts_.find(rph);
-  if (it == observed_render_process_hosts_.end()) {
-    // This is the first PeerConnection which we see that's associated
-    // with this RPH.
-    rph->AddObserver(this);
-    observed_render_process_hosts_.insert(rph);
-  }
-
-  const auto browser_context_id = GetBrowserContextId(rph->GetBrowserContext());
-  DCHECK_NE(browser_context_id, kNullBrowserContextId);
-
-  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
-  // will not be dereferenced after destruction.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(
-          &WebRtcEventLogManager::PeerConnectionAddedInternal,
-          base::Unretained(this),
-          PeerConnectionKey(render_process_id, lid, browser_context_id),
-          std::move(reply)));
+    base::ProcessId pid,
+    const std::string& url,
+    const std::string& rtc_configuration) {
+  OnPeerConnectionAdded(frame_id, lid, base::NullCallback());
 }
 
-void WebRtcEventLogManager::PeerConnectionRemoved(
-    int render_process_id,
+void WebRtcEventLogManager::OnPeerConnectionRemoved(
+    content::GlobalRenderFrameHostId frame_id,
+    int lid) {
+  OnPeerConnectionRemoved(frame_id, lid, base::NullCallback());
+}
+
+void WebRtcEventLogManager::OnPeerConnectionUpdated(
+    content::GlobalRenderFrameHostId frame_id,
     int lid,
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-
-  const auto browser_context_id = GetBrowserContextId(render_process_id);
-  if (browser_context_id == kNullBrowserContextId) {
-    // RPH died before processing of this notification. This is handled by
-    // RenderProcessExited() / RenderProcessHostDestroyed.
-    MaybeReply(FROM_HERE, std::move(reply), false);
-    return;
+    const std::string& type,
+    const std::string& value) {
+  // TODO(crbug.com/40562061): Get rid of magic value.
+  if (type == "stop") {
+    OnPeerConnectionStopped(frame_id, lid, base::NullCallback());
   }
-
-  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
-  // will not be dereferenced after destruction.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(
-          &WebRtcEventLogManager::PeerConnectionRemovedInternal,
-          base::Unretained(this),
-          PeerConnectionKey(render_process_id, lid, browser_context_id),
-          std::move(reply)));
 }
 
-void WebRtcEventLogManager::PeerConnectionStopped(
-    int render_process_id,
+void WebRtcEventLogManager::OnPeerConnectionSessionIdSet(
+    content::GlobalRenderFrameHostId frame_id,
     int lid,
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  return PeerConnectionRemoved(render_process_id, lid, std::move(reply));
-}
-
-void WebRtcEventLogManager::PeerConnectionSessionIdSet(
-    int render_process_id,
-    int lid,
-    const std::string& session_id,
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-
-  const auto browser_context_id = GetBrowserContextId(render_process_id);
-  if (browser_context_id == kNullBrowserContextId) {
-    // RPH died before processing of this notification. This is handled by
-    // RenderProcessExited() / RenderProcessHostDestroyed.
-    MaybeReply(FROM_HERE, std::move(reply), false);
-    return;
-  }
-
-  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
-  // will not be dereferenced after destruction.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(
-          &WebRtcEventLogManager::PeerConnectionSessionIdSetInternal,
-          base::Unretained(this),
-          PeerConnectionKey(render_process_id, lid, browser_context_id),
-          session_id, std::move(reply)));
-}
-
-void WebRtcEventLogManager::EnableLocalLogging(
-    const base::FilePath& base_path,
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  EnableLocalLogging(base_path, kDefaultMaxLocalLogFileSizeBytes,
-                     std::move(reply));
-}
-
-void WebRtcEventLogManager::EnableLocalLogging(
-    const base::FilePath& base_path,
-    size_t max_file_size_bytes,
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  DCHECK(!base_path.empty());
-  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
-  // will not be dereferenced after destruction.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(&WebRtcEventLogManager::EnableLocalLoggingInternal,
-                     base::Unretained(this), base_path, max_file_size_bytes,
-                     std::move(reply)));
-}
-
-void WebRtcEventLogManager::DisableLocalLogging(
-    base::OnceCallback<void(bool)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
-  // will not be dereferenced after destruction.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(&WebRtcEventLogManager::DisableLocalLoggingInternal,
-                     base::Unretained(this), std::move(reply)));
+    const std::string& session_id) {
+  OnPeerConnectionSessionIdSet(frame_id, lid, session_id, base::NullCallback());
 }
 
 void WebRtcEventLogManager::OnWebRtcEventLogWrite(
-    int render_process_id,
+    content::GlobalRenderFrameHostId frame_id,
     int lid,
-    const std::string& message,
-    base::OnceCallback<void(std::pair<bool, bool>)> reply) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+    const std::string& message) {
+  OnWebRtcEventLogWrite(frame_id, lid, message, base::NullCallback());
+}
 
-  const BrowserContext* browser_context = GetBrowserContext(render_process_id);
-  if (!browser_context) {
-    // RPH died before processing of this notification.
-    MaybeReply(FROM_HERE, std::move(reply), std::make_pair(false, false));
-    return;
-  }
+void WebRtcEventLogManager::EnableLocalLogging(
+    const base::FilePath& base_path) {
+  EnableLocalLogging(base_path, base::NullCallback());
+}
 
-  const auto browser_context_id = GetBrowserContextId(browser_context);
-  DCHECK_NE(browser_context_id, kNullBrowserContextId);
-
-  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
-  // will not be dereferenced after destruction.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(
-          &WebRtcEventLogManager::OnWebRtcEventLogWriteInternal,
-          base::Unretained(this),
-          PeerConnectionKey(render_process_id, lid, browser_context_id),
-          message, std::move(reply)));
+void WebRtcEventLogManager::DisableLocalLogging() {
+  DisableLocalLogging(base::NullCallback());
 }
 
 void WebRtcEventLogManager::StartRemoteLogging(
@@ -534,7 +431,7 @@ std::unique_ptr<LogFileWriter::Factory>
 WebRtcEventLogManager::CreateRemoteLogFileWriterFactory() {
   if (remote_log_file_writer_factory_for_testing_) {
     return std::move(remote_log_file_writer_factory_for_testing_);
-#if !defined(OS_ANDROID) && !defined(TOOLKIT_QT)
+#if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_QTWEBENGINE)
   } else if (base::FeatureList::IsEnabled(
                  features::kWebRtcRemoteEventLogGzipped)) {
     return std::make_unique<GzippedLogFileWriterFactory>(
@@ -579,13 +476,175 @@ void WebRtcEventLogManager::RenderProcessHostExitedDestroyed(
                      base::Unretained(this), host->GetID()));
 }
 
+void WebRtcEventLogManager::OnPeerConnectionAdded(
+    content::GlobalRenderFrameHostId frame_id,
+    int lid,
+    base::OnceCallback<void(bool)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  // TODO(crbug.com/40169214): Should this look at RFH shutdown instead of RPH?
+  RenderProcessHost* rph = RenderProcessHost::FromID(frame_id.child_id);
+  if (!rph) {
+    // RPH died before processing of this notification.
+    MaybeReply(FROM_HERE, std::move(reply), false);
+    return;
+  }
+
+  auto it = observed_render_process_hosts_.find(rph);
+  if (it == observed_render_process_hosts_.end()) {
+    // This is the first PeerConnection which we see that's associated
+    // with this RPH.
+    rph->AddObserver(this);
+    observed_render_process_hosts_.insert(rph);
+  }
+
+  const auto browser_context_id = GetBrowserContextId(rph->GetBrowserContext());
+  DCHECK_NE(browser_context_id, kNullBrowserContextId);
+
+  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
+  // will not be dereferenced after destruction.
+  task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(
+          &WebRtcEventLogManager::OnPeerConnectionAddedInternal,
+          base::Unretained(this),
+          PeerConnectionKey(frame_id.child_id, lid, browser_context_id,
+                            frame_id.frame_routing_id),
+          std::move(reply)));
+}
+
+void WebRtcEventLogManager::OnPeerConnectionRemoved(
+    content::GlobalRenderFrameHostId frame_id,
+    int lid,
+    base::OnceCallback<void(bool)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  const auto browser_context_id = GetBrowserContextId(frame_id.child_id);
+  if (browser_context_id == kNullBrowserContextId) {
+    // RPH died before processing of this notification. This is handled by
+    // RenderProcessExited() / RenderProcessHostDestroyed.
+    MaybeReply(FROM_HERE, std::move(reply), false);
+    return;
+  }
+
+  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
+  // will not be dereferenced after destruction.
+  task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(
+          &WebRtcEventLogManager::OnPeerConnectionRemovedInternal,
+          base::Unretained(this),
+          PeerConnectionKey(frame_id.child_id, lid, browser_context_id,
+                            frame_id.frame_routing_id),
+          std::move(reply)));
+}
+
+void WebRtcEventLogManager::OnPeerConnectionStopped(
+    content::GlobalRenderFrameHostId frame_id,
+    int lid,
+    base::OnceCallback<void(bool)> reply) {
+  // From the logger's perspective, we treat stopping a peer connection the
+  // same as we do its removal. Should a stopped peer connection be later
+  // removed, the removal callback will assume the value |false|.
+  OnPeerConnectionRemoved(frame_id, lid, std::move(reply));
+}
+
+void WebRtcEventLogManager::OnPeerConnectionSessionIdSet(
+    content::GlobalRenderFrameHostId frame_id,
+    int lid,
+    const std::string& session_id,
+    base::OnceCallback<void(bool)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  const auto browser_context_id = GetBrowserContextId(frame_id.child_id);
+  if (browser_context_id == kNullBrowserContextId) {
+    // RPH died before processing of this notification. This is handled by
+    // RenderProcessExited() / RenderProcessHostDestroyed.
+    MaybeReply(FROM_HERE, std::move(reply), false);
+    return;
+  }
+
+  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
+  // will not be dereferenced after destruction.
+  task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(
+          &WebRtcEventLogManager::OnPeerConnectionSessionIdSetInternal,
+          base::Unretained(this),
+          PeerConnectionKey(frame_id.child_id, lid, browser_context_id,
+                            frame_id.frame_routing_id),
+          session_id, std::move(reply)));
+}
+
+void WebRtcEventLogManager::OnWebRtcEventLogWrite(
+    content::GlobalRenderFrameHostId frame_id,
+    int lid,
+    const std::string& message,
+    base::OnceCallback<void(std::pair<bool, bool>)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  const BrowserContext* browser_context = GetBrowserContext(frame_id.child_id);
+  if (!browser_context) {
+    // RPH died before processing of this notification.
+    MaybeReply(FROM_HERE, std::move(reply), std::make_pair(false, false));
+    return;
+  }
+
+  const auto browser_context_id = GetBrowserContextId(browser_context);
+  DCHECK_NE(browser_context_id, kNullBrowserContextId);
+
+  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
+  // will not be dereferenced after destruction.
+  task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(
+          &WebRtcEventLogManager::OnWebRtcEventLogWriteInternal,
+          base::Unretained(this),
+          PeerConnectionKey(frame_id.child_id, lid, browser_context_id,
+                            frame_id.frame_routing_id),
+          message, std::move(reply)));
+}
+
+void WebRtcEventLogManager::EnableLocalLogging(
+    const base::FilePath& base_path,
+    base::OnceCallback<void(bool)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  EnableLocalLogging(base_path, kDefaultMaxLocalLogFileSizeBytes,
+                     std::move(reply));
+}
+
+void WebRtcEventLogManager::EnableLocalLogging(
+    const base::FilePath& base_path,
+    size_t max_file_size_bytes,
+    base::OnceCallback<void(bool)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  DCHECK(!base_path.empty());
+  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
+  // will not be dereferenced after destruction.
+  task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(&WebRtcEventLogManager::EnableLocalLoggingInternal,
+                     base::Unretained(this), base_path, max_file_size_bytes,
+                     std::move(reply)));
+}
+
+void WebRtcEventLogManager::DisableLocalLogging(
+    base::OnceCallback<void(bool)> reply) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  // |this| is destroyed by ~BrowserProcessImpl(), so base::Unretained(this)
+  // will not be dereferenced after destruction.
+  task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(&WebRtcEventLogManager::DisableLocalLoggingInternal,
+                     base::Unretained(this), std::move(reply)));
+}
+
 void WebRtcEventLogManager::OnLocalLogStarted(PeerConnectionKey peer_connection,
                                               const base::FilePath& file_path) {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
 
-  constexpr int kLogOutputPeriodMsForLocalLogging = 0;  // No batching.
   OnLoggingTargetStarted(LoggingTarget::kLocalLogging, peer_connection,
-                         kLogOutputPeriodMsForLocalLogging);
+                         /*output_period_ms=*/5000);
 
   if (local_logs_observer_) {
     local_logs_observer_->OnLocalLogStarted(peer_connection, file_path);
@@ -799,40 +858,53 @@ void WebRtcEventLogManager::
   MaybeReply(FROM_HERE, std::move(reply));
 }
 
-void WebRtcEventLogManager::PeerConnectionAddedInternal(
+void WebRtcEventLogManager::OnPeerConnectionAddedInternal(
     PeerConnectionKey key,
     base::OnceCallback<void(bool)> reply) {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
 
-  const bool local_result = local_logs_manager_.PeerConnectionAdded(key);
-  const bool remote_result = remote_logs_manager_.PeerConnectionAdded(key);
+  const bool local_result = local_logs_manager_.OnPeerConnectionAdded(key);
+  const bool remote_result = remote_logs_manager_.OnPeerConnectionAdded(key);
   DCHECK_EQ(local_result, remote_result);
 
   MaybeReply(FROM_HERE, std::move(reply), local_result);
 }
 
-void WebRtcEventLogManager::PeerConnectionRemovedInternal(
+void WebRtcEventLogManager::OnPeerConnectionRemovedInternal(
     PeerConnectionKey key,
     base::OnceCallback<void(bool)> reply) {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
 
-  const bool local_result = local_logs_manager_.PeerConnectionRemoved(key);
-  const bool remote_result = remote_logs_manager_.PeerConnectionRemoved(key);
+  const bool local_result = local_logs_manager_.OnPeerConnectionRemoved(key);
+  const bool remote_result = remote_logs_manager_.OnPeerConnectionRemoved(key);
   DCHECK_EQ(local_result, remote_result);
 
   MaybeReply(FROM_HERE, std::move(reply), local_result);
 }
 
-void WebRtcEventLogManager::PeerConnectionSessionIdSetInternal(
+void WebRtcEventLogManager::OnPeerConnectionSessionIdSetInternal(
     PeerConnectionKey key,
     const std::string& session_id,
     base::OnceCallback<void(bool)> reply) {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
   const bool result =
-      remote_logs_manager_.PeerConnectionSessionIdSet(key, session_id);
+      remote_logs_manager_.OnPeerConnectionSessionIdSet(key, session_id);
   MaybeReply(FROM_HERE, std::move(reply), result);
 }
 
+void WebRtcEventLogManager::OnWebRtcEventLogWriteInternal(
+    PeerConnectionKey key,
+    const std::string& message,
+    base::OnceCallback<void(std::pair<bool, bool>)> reply) {
+  DCHECK(task_runner_->RunsTasksInCurrentSequence());
+
+  const bool local_result = local_logs_manager_.EventLogWrite(key, message);
+  const bool remote_result = remote_logs_manager_.EventLogWrite(key, message);
+
+  MaybeReply(FROM_HERE, std::move(reply),
+             std::make_pair(local_result, remote_result));
+}
+
 void WebRtcEventLogManager::EnableLocalLoggingInternal(
     const base::FilePath& base_path,
     size_t max_file_size_bytes,
@@ -854,19 +926,6 @@ void WebRtcEventLogManager::DisableLocalLoggingInternal(
   MaybeReply(FROM_HERE, std::move(reply), result);
 }
 
-void WebRtcEventLogManager::OnWebRtcEventLogWriteInternal(
-    PeerConnectionKey key,
-    const std::string& message,
-    base::OnceCallback<void(std::pair<bool, bool>)> reply) {
-  DCHECK(task_runner_->RunsTasksInCurrentSequence());
-
-  const bool local_result = local_logs_manager_.EventLogWrite(key, message);
-  const bool remote_result = remote_logs_manager_.EventLogWrite(key, message);
-
-  MaybeReply(FROM_HERE, std::move(reply),
-             std::make_pair(local_result, remote_result));
-}
-
 void WebRtcEventLogManager::StartRemoteLoggingInternal(
     int render_process_id,
     BrowserContextId browser_context_id,
