@@ -2,9 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/media/webrtc/webrtc_event_log_manager_common.h"
 
 #include <limits>
+#include <string_view>
 
 #include "base/files/file_util.h"
 #include "base/logging.h"
@@ -12,7 +18,6 @@
 #include "base/metrics/histogram_functions.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/task/sequenced_task_runner.h"
@@ -237,7 +242,7 @@ bool BaseLogFileWriter::Init() {
   DCHECK(task_runner_->RunsTasksInCurrentSequence());
   DCHECK_EQ(state(), State::PRE_INIT);
 
-  // TODO(crbug.com/775415): Use a temporary filename which will indicate
+  // TODO(crbug.com/40545136): Use a temporary filename which will indicate
   // incompletion, and rename to something that is eligible for upload only
   // on an orderly and successful Close().
 
@@ -456,7 +461,7 @@ bool GzippedLogFileWriter::Write(const std::string& input) {
     }
   }
 
-  NOTREACHED_NORETURN();
+  NOTREACHED();
 }
 
 bool GzippedLogFileWriter::Finalize() {
@@ -588,7 +593,7 @@ LogCompressor::Result GzipLogCompressor::Compress(const std::string& input,
       return result;
   }
 
-  NOTREACHED_NORETURN();
+  NOTREACHED();
 }
 
 bool GzipLogCompressor::CreateFooter(std::string* output) {
@@ -741,7 +746,7 @@ bool GzipLogCompressor::Deflate(int flush, std::string* output) {
 // Given a string with a textual representation of a web-app ID, return the
 // ID in integer form. If the textual representation does not name a valid
 // web-app ID, return kInvalidWebRtcEventLogWebAppId.
-size_t ExtractWebAppId(base::StringPiece str) {
+size_t ExtractWebAppId(std::string_view str) {
   DCHECK_EQ(str.length(), kWebAppIdLength);
 
   // Avoid leading '+', etc.
@@ -957,7 +962,7 @@ bool IsValidRemoteBoundLogFilename(const std::string& filename) {
 
   // Expect web-app-ID.
   const size_t web_app_id =
-      ExtractWebAppId(base::StringPiece(&filename[index], kWebAppIdLength));
+      ExtractWebAppId(std::string_view(&filename[index], kWebAppIdLength));
   if (web_app_id == kInvalidWebRtcEventLogWebAppId) {
     return false;
   }
@@ -981,7 +986,7 @@ bool IsValidRemoteBoundLogFilePath(const base::FilePath& path) {
 }
 
 base::FilePath GetWebRtcEventLogHistoryFilePath(const base::FilePath& path) {
-  // TODO(crbug.com/775415): Check for validity (after fixing unit tests).
+  // TODO(crbug.com/40545136): Check for validity (after fixing unit tests).
   return path.RemoveExtension().AddExtension(kWebRtcEventLogHistoryExtension);
 }
 
@@ -1012,7 +1017,7 @@ size_t ExtractRemoteBoundWebRtcEventLogWebAppIdFromPath(
   // The +1 is for the underscore between the prefix and the web-app ID.
   // Length verified by above call to IsValidRemoteBoundLogFilename().
   DCHECK_GE(filename.length(), kPrefixLength + 1 + kWebAppIdLength);
-  base::StringPiece id_str(&filename[kPrefixLength + 1], kWebAppIdLength);
+  std::string_view id_str(&filename[kPrefixLength + 1], kWebAppIdLength);
 
   return ExtractWebAppId(id_str);
 }
@@ -1028,7 +1033,7 @@ bool DoesProfileDefaultToLoggingEnabled(const Profile* const profile) {
     return false;
   }
   const user_manager::UserType user_type = user->GetType();
-  if (user_type != user_manager::USER_TYPE_REGULAR) {
+  if (user_type != user_manager::UserType::kRegular) {
     return false;
   }
   if (ash::ProfileHelper::IsEphemeralUserProfile(profile)) {
