@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,18 +6,19 @@
 
 #include <memory>
 #include <numeric>
+#include <optional>
 #include <vector>
 
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
+#include "base/notreached.h"
 #include "base/rand_util.h"
 #include "base/test/task_environment.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/media/webrtc/webrtc_event_log_manager_unittest_helpers.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "third_party/zlib/google/compression_utils.h"
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
@@ -25,8 +26,8 @@
 #include "chrome/test/base/testing_profile.h"
 #include "components/account_id/account_id.h"
 #include "components/user_manager/scoped_user_manager.h"
+#include "components/user_manager/user_names.h"
 #include "content/public/test/browser_task_environment.h"
-#include "testing/gmock/include/gmock/gmock.h"
 #endif
 
 namespace webrtc_event_logging {
@@ -164,7 +165,7 @@ TEST_F(GzipLogCompressorTest, MultipleCallsToCompress) {
 TEST_F(GzipLogCompressorTest, UnlimitedBudgetSanity) {
   Init(std::make_unique<PerfectGzipEstimator::Factory>());
 
-  auto compressor = compressor_factory_->Create(absl::optional<size_t>());
+  auto compressor = compressor_factory_->Create(std::optional<size_t>());
   ASSERT_TRUE(compressor);
 
   std::string header;
@@ -306,7 +307,7 @@ class LogFileWriterTest
                 .AddExtension(log_file_writer_factory_->Extension());
   }
 
-  std::unique_ptr<LogFileWriter> CreateWriter(absl::optional<size_t> max_size) {
+  std::unique_ptr<LogFileWriter> CreateWriter(std::optional<size_t> max_size) {
     return log_file_writer_factory_->Create(path_, max_size);
   }
 
@@ -329,12 +330,14 @@ class LogFileWriterTest
         EXPECT_EQ(uncompressed, expected_contents);
         break;
       }
-      default: { NOTREACHED(); }
+      default: {
+        NOTREACHED_IN_MIGRATION();
+      }
     }
   }
 
   base::test::TaskEnvironment task_environment_;
-  absl::optional<WebRtcEventLogCompression> compression_;  // Set in Init().
+  std::optional<WebRtcEventLogCompression> compression_;  // Set in Init().
   base::ScopedTempDir temp_dir_;
   base::FilePath path_;
   std::unique_ptr<LogFileWriter::Factory> log_file_writer_factory_;
@@ -394,7 +397,7 @@ TEST_P(LogFileWriterTest, CallToWriteWithEmptyStringSucceeds) {
 TEST_P(LogFileWriterTest, UnlimitedBudgetSanity) {
   Init(GetParam());
 
-  auto writer = CreateWriter(absl::optional<size_t>());
+  auto writer = CreateWriter(std::optional<size_t>());
   ASSERT_TRUE(writer);
 
   const std::string log = "log";
@@ -683,38 +686,28 @@ TEST_P(DoesProfileDefaultToLoggingEnabledForUserTypeParametrizedTest,
   TestingProfile::Builder profile_builder;
   profile_builder.OverridePolicyConnectorIsManagedForTesting(true);
   std::unique_ptr<TestingProfile> testing_profile = profile_builder.Build();
-  std::unique_ptr<testing::NiceMock<ash::FakeChromeUserManager>>
-      fake_user_manager_ =
-          std::make_unique<testing::NiceMock<ash::FakeChromeUserManager>>();
+  auto fake_user_manager_ = std::make_unique<ash::FakeChromeUserManager>();
   // We use a standard Gaia account by default:
   AccountId account_id = AccountId::FromUserEmailGaiaId("name", "id");
 
   switch (test_case.user_type) {
-    case user_manager::USER_TYPE_REGULAR:
+    case user_manager::UserType::kRegular:
       fake_user_manager_->AddUserWithAffiliationAndTypeAndProfile(
           account_id, false, test_case.user_type, testing_profile.get());
       break;
-    case user_manager::USER_TYPE_GUEST:
-      account_id = fake_user_manager_->GetGuestAccountId();
+    case user_manager::UserType::kGuest:
+      account_id = user_manager::GuestAccountId();
       fake_user_manager_->AddGuestUser();
       break;
-    case user_manager::USER_TYPE_PUBLIC_ACCOUNT:
+    case user_manager::UserType::kPublicAccount:
       fake_user_manager_->AddPublicAccountUser(account_id);
       break;
-    case user_manager::USER_TYPE_KIOSK_APP:
+    case user_manager::UserType::kKioskApp:
       fake_user_manager_->AddKioskAppUser(account_id);
       break;
-    case user_manager::USER_TYPE_CHILD:
+    case user_manager::UserType::kChild:
       fake_user_manager_->AddChildUser(account_id);
       break;
-    case user_manager::USER_TYPE_ARC_KIOSK_APP:
-      fake_user_manager_->AddArcKioskAppUser(account_id);
-      break;
-    case user_manager::USER_TYPE_ACTIVE_DIRECTORY:
-      account_id = AccountId::AdFromObjGuid("guid");
-      fake_user_manager_->AddUserWithAffiliationAndTypeAndProfile(
-          account_id, false, test_case.user_type, testing_profile.get());
-      break;
     default:
       FAIL() << "Invalid test setup. Unexpected user type.";
   }
@@ -733,13 +726,12 @@ INSTANTIATE_TEST_SUITE_P(
     DoesProfileDefaultToLoggingEnabledForUserTypeParametrizedTest,
     testing::ValuesIn(
         std::vector<DoesProfileDefaultToLoggingEnabledForUserTypeTestCase>{
-            {user_manager::USER_TYPE_REGULAR, true},
-            {user_manager::USER_TYPE_GUEST, false},
-            {user_manager::USER_TYPE_PUBLIC_ACCOUNT, false},
-            {user_manager::USER_TYPE_KIOSK_APP, false},
-            {user_manager::USER_TYPE_CHILD, false},
-            {user_manager::USER_TYPE_ARC_KIOSK_APP, false},
-            {user_manager::USER_TYPE_ACTIVE_DIRECTORY, false}}));
+            {user_manager::UserType::kRegular, true},
+            {user_manager::UserType::kGuest, false},
+            {user_manager::UserType::kPublicAccount, false},
+            {user_manager::UserType::kKioskApp, false},
+            {user_manager::UserType::kChild, false},
+        }));
 
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
