@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -49,27 +49,22 @@ IN_PROC_BROWSER_TEST_P(WebRtcPanTiltZoomPermissionBrowserTest,
   ASSERT_TRUE(embedded_test_server()->Start());
   content::WebContents* tab = OpenTestPageInNewTab(kMainHtmlPage);
 
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("runGetUserMedia(%s);", GetParam().constraints),
-      &result));
-  EXPECT_EQ(result, "runGetUserMedia-success");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            base::StringPrintf("runGetUserMedia(%s);",
+                                               GetParam().constraints)),
+            "runGetUserMedia-success");
 
-  std::string microphone;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getMicrophonePermission();", &microphone));
-  EXPECT_EQ(microphone, GetParam().expected_microphone);
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getMicrophonePermission();"),
+      GetParam().expected_microphone);
 
-  std::string camera;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getCameraPermission();", &camera));
-  EXPECT_EQ(camera, GetParam().expected_camera);
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getCameraPermission();"),
+      GetParam().expected_camera);
 
-  std::string pan_tilt_zoom;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, GetParam().expected_pan_tilt_zoom);
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            GetParam().expected_pan_tilt_zoom);
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -233,39 +228,35 @@ IN_PROC_BROWSER_TEST_P(WebRtcPanTiltZoomTrackBrowserTest,
   ASSERT_TRUE(embedded_test_server()->Start());
   content::WebContents* tab = OpenTestPageInNewTab(kMainHtmlPage);
 
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("runGetUserMedia(%s);", GetParam().constraints),
-      &result));
-  EXPECT_EQ(result, "runGetUserMedia-success");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            base::StringPrintf("runGetUserMedia(%s);",
+                                               GetParam().constraints)),
+            "runGetUserMedia-success");
 
-  std::string pan_tilt_zoom;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, "granted");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            "granted");
 
-  double pan;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractDouble(
-      tab->GetMainFrame(), "getTrackSetting('pan');", &pan));
-  EXPECT_EQ(pan, GetParam().expected_pan);
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getTrackSetting('pan');")
+          .ExtractDouble(),
+      GetParam().expected_pan);
 
-  double tilt;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractDouble(
-      tab->GetMainFrame(), "getTrackSetting('tilt');", &tilt));
-  EXPECT_EQ(tilt, GetParam().expected_tilt);
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getTrackSetting('tilt');")
+          .ExtractDouble(),
+      GetParam().expected_tilt);
 
-  double zoom;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractDouble(
-      tab->GetMainFrame(), "getTrackSetting('zoom');", &zoom));
-  EXPECT_EQ(zoom, GetParam().expected_zoom);
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getTrackSetting('zoom');")
+          .ExtractDouble(),
+      GetParam().expected_zoom);
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("checkConstraints(%s);",
-                         GetParam().expected_constraints),
-      &result));
-  EXPECT_EQ(result, "checkConstraints-success");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(),
+                      base::StringPrintf("checkConstraints(%s);",
+                                         GetParam().expected_constraints)),
+      "checkConstraints-success");
 }
 
 // Default PTZ value is 100, min is 100, max is 400 as defined in fake video
@@ -338,34 +329,32 @@ IN_PROC_BROWSER_TEST_P(WebRtcPanTiltZoomConstraintsBrowserTest,
   ASSERT_TRUE(embedded_test_server()->Start());
   content::WebContents* tab = OpenTestPageInNewTab(kMainHtmlPage);
 
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("runGetUserMedia({ video: { width: 640, %s: 101 } });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "runGetUserMedia-success");
+  EXPECT_EQ(content::EvalJs(
+                tab->GetPrimaryMainFrame(),
+                base::StringPrintf(
+                    "runGetUserMedia({ video: { width: 640, %s: 101 } });",
+                    Constraint())),
+            "runGetUserMedia-success");
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("checkConstraints({ width: 640, %s: 101 });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "checkConstraints-success");
+  EXPECT_EQ(content::EvalJs(
+                tab->GetPrimaryMainFrame(),
+                base::StringPrintf("checkConstraints({ width: 640, %s: 101 });",
+                                   Constraint())),
+            "checkConstraints-success");
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("applyConstraints({ advanced: [{ %s: 102 }] });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "applyConstraints-success");
+  EXPECT_EQ(
+      content::EvalJs(
+          tab->GetPrimaryMainFrame(),
+          base::StringPrintf("applyConstraints({ advanced: [{ %s: 102 }] });",
+                             Constraint())),
+      "applyConstraints-success");
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("checkConstraints({ advanced: [{ %s: 102 }] });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "checkConstraints-success");
+  EXPECT_EQ(
+      content::EvalJs(
+          tab->GetPrimaryMainFrame(),
+          base::StringPrintf("checkConstraints({ advanced: [{ %s: 102 }] });",
+                             Constraint())),
+      "checkConstraints-success");
 }
 
 IN_PROC_BROWSER_TEST_P(WebRtcPanTiltZoomConstraintsBrowserTest,
@@ -373,31 +362,28 @@ IN_PROC_BROWSER_TEST_P(WebRtcPanTiltZoomConstraintsBrowserTest,
   ASSERT_TRUE(embedded_test_server()->Start());
   content::WebContents* tab = OpenTestPageInNewTab(kMainHtmlPage);
 
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("runGetUserMedia({ video: { width: 640, %s: 101 } });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "runGetUserMedia-success");
+  EXPECT_EQ(content::EvalJs(
+                tab->GetPrimaryMainFrame(),
+                base::StringPrintf(
+                    "runGetUserMedia({ video: { width: 640, %s: 101 } });",
+                    Constraint())),
+            "runGetUserMedia-success");
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("checkConstraints({ width: 640, %s: 101 });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "checkConstraints-success");
+  EXPECT_EQ(content::EvalJs(
+                tab->GetPrimaryMainFrame(),
+                base::StringPrintf("checkConstraints({ width: 640, %s: 101 });",
+                                   Constraint())),
+            "checkConstraints-success");
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      base::StringPrintf("runGetUserMedia({ video: { %s: true } });",
-                         Constraint()),
-      &result));
-  EXPECT_EQ(result, "runGetUserMedia-success");
+  EXPECT_EQ(content::EvalJs(
+                tab->GetPrimaryMainFrame(),
+                base::StringPrintf("runGetUserMedia({ video: { %s: true } });",
+                                   Constraint())),
+            "runGetUserMedia-success");
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "checkConstraints({});", &result));
-  EXPECT_EQ(result, "checkConstraints-success");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "checkConstraints({});"),
+      "checkConstraints-success");
 }
 
 INSTANTIATE_TEST_SUITE_P(ConstraintsFromGetUserMedia,
@@ -427,7 +413,7 @@ class WebRtcPanTiltZoomPermissionRequestBrowserTest
     blink::MediaStreamDevice fake_video_device(
         blink::mojom::MediaStreamType::DEVICE_VIDEO_CAPTURE, "fake_video_dev",
         "Fake Video Device", control_support, media::MEDIA_VIDEO_FACING_NONE,
-        absl::nullopt);
+        std::nullopt);
     video_devices.push_back(fake_video_device);
     MediaCaptureDevicesDispatcher::GetInstance()->SetTestVideoCaptureDevices(
         video_devices);
@@ -443,20 +429,17 @@ IN_PROC_BROWSER_TEST_P(WebRtcPanTiltZoomPermissionRequestBrowserTest,
   ASSERT_TRUE(embedded_test_server()->Start());
   content::WebContents* tab = OpenTestPageInNewTab(kMainHtmlPage);
 
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "runRequestPanTiltZoom();", &result));
-  EXPECT_EQ(result, "runRequestPanTiltZoom-success");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "runRequestPanTiltZoom();"),
+      "runRequestPanTiltZoom-success");
 
-  std::string camera;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getCameraPermission();", &camera));
-  EXPECT_EQ(camera, "granted");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getCameraPermission();"),
+      "granted");
 
-  std::string pan_tilt_zoom;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, IsPanTiltZoomSupported() ? "granted" : "prompt");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            IsPanTiltZoomSupported() ? "granted" : "prompt");
 }
 
 INSTANTIATE_TEST_SUITE_P(RequestPanTiltZoomPermission,
@@ -477,7 +460,7 @@ class WebRtcPanTiltZoomCameraDevicesBrowserTest : public WebRtcTestBase {
     blink::MediaStreamDevice fake_video_device(
         blink::mojom::MediaStreamType::DEVICE_VIDEO_CAPTURE, "fake_video_dev",
         "Fake Video Device", {pan_supported, tilt_supported, zoom_supported},
-        media::MEDIA_VIDEO_FACING_NONE, absl::nullopt);
+        media::MEDIA_VIDEO_FACING_NONE, std::nullopt);
     video_devices.push_back(fake_video_device);
     MediaCaptureDevicesDispatcher::GetInstance()->SetTestVideoCaptureDevices(
         video_devices);
@@ -497,31 +480,28 @@ IN_PROC_BROWSER_TEST_F(WebRtcPanTiltZoomCameraDevicesBrowserTest,
   // permission.
   SetVideoCaptureDevice(/*pan_supported=*/false, /*tilt_supported=*/false,
                         /*zoom_supported=*/false);
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "runRequestPanTiltZoom();", &result));
-  EXPECT_EQ(result, "runRequestPanTiltZoom-success");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "runRequestPanTiltZoom();"),
+      "runRequestPanTiltZoom-success");
 
   // Camera permission should be granted.
-  std::string camera;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getCameraPermission();", &camera));
-  EXPECT_EQ(camera, "granted");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getCameraPermission();"),
+      "granted");
 
   // Camera PTZ permission should not be granted.
-  std::string pan_tilt_zoom;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, "prompt");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            "prompt");
 
   // Simulate camera device with PTZ support.
   SetVideoCaptureDevice(/*pan_supported=*/true, /*tilt_supported=*/true,
                         /*zoom_supported=*/true);
 
   // Camera PTZ permission should still not be granted.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, "prompt");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            "prompt");
 }
 
 IN_PROC_BROWSER_TEST_F(WebRtcPanTiltZoomCameraDevicesBrowserTest,
@@ -532,31 +512,28 @@ IN_PROC_BROWSER_TEST_F(WebRtcPanTiltZoomCameraDevicesBrowserTest,
   // Simulate camera device with PTZ support and request PTZ camera permission.
   SetVideoCaptureDevice(/*pan_supported=*/true, /*tilt_supported=*/true,
                         /*zoom_supported=*/true);
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "runRequestPanTiltZoom();", &result));
-  EXPECT_EQ(result, "runRequestPanTiltZoom-success");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "runRequestPanTiltZoom();"),
+      "runRequestPanTiltZoom-success");
 
   // Camera permission should be granted.
-  std::string camera;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getCameraPermission();", &camera));
-  EXPECT_EQ(camera, "granted");
+  EXPECT_EQ(
+      content::EvalJs(tab->GetPrimaryMainFrame(), "getCameraPermission();"),
+      "granted");
 
   // Camera PTZ permission should be granted.
-  std::string pan_tilt_zoom;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, "granted");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            "granted");
 
   // Simulate camera device with no PTZ support.
   SetVideoCaptureDevice(/*pan_supported=*/false, /*tilt_supported=*/false,
                         /*zoom_supported=*/false);
 
   // Camera PTZ permission should still be granted.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "getPanTiltZoomPermission();", &pan_tilt_zoom));
-  EXPECT_EQ(pan_tilt_zoom, "granted");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "getPanTiltZoomPermission();"),
+            "granted");
 }
 
 class WebRtcPanTiltZoomFakeCameraDevicesBrowserTest : public WebRtcTestBase {
@@ -576,12 +553,11 @@ IN_PROC_BROWSER_TEST_F(WebRtcPanTiltZoomFakeCameraDevicesBrowserTest,
   content::WebContents* tab = OpenTestPageInNewTab(kMainHtmlPage);
 
   // Access PTZ camera.
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(),
-      "runGetUserMedia({ video: { pan: true, tilt: true, zoom: true } });",
-      &result));
-  EXPECT_EQ(result, "runGetUserMedia-success");
+  EXPECT_EQ(
+      content::EvalJs(
+          tab->GetPrimaryMainFrame(),
+          "runGetUserMedia({ video: { pan: true, tilt: true, zoom: true } });"),
+      "runGetUserMedia-success");
 
   // Hide page.
   tab->WasHidden();
@@ -590,22 +566,19 @@ IN_PROC_BROWSER_TEST_F(WebRtcPanTiltZoomFakeCameraDevicesBrowserTest,
             content::TitleWatcher(tab, expected_title).WaitAndGetTitle());
 
   // Pan can't be set when page is hidden.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "applyConstraints({ advanced: [{ pan: 102 }] });",
-      &result));
-  EXPECT_EQ(result, "applyConstraints-failure-SecurityError");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "applyConstraints({ advanced: [{ pan: 102 }] });"),
+            "applyConstraints-failure-SecurityError");
 
   // Tilt can't be set when page is hidden.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "applyConstraints({ advanced: [{ tilt: 102 }] });",
-      &result));
-  EXPECT_EQ(result, "applyConstraints-failure-SecurityError");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "applyConstraints({ advanced: [{ tilt: 102 }] });"),
+            "applyConstraints-failure-SecurityError");
 
   // Zoom can't be set when page is hidden.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "applyConstraints({ advanced: [{ zoom: 102 }] });",
-      &result));
-  EXPECT_EQ(result, "applyConstraints-failure-SecurityError");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "applyConstraints({ advanced: [{ zoom: 102 }] });"),
+            "applyConstraints-failure-SecurityError");
 
   // Show page.
   tab->WasShown();
@@ -614,20 +587,17 @@ IN_PROC_BROWSER_TEST_F(WebRtcPanTiltZoomFakeCameraDevicesBrowserTest,
             content::TitleWatcher(tab, expected_title).WaitAndGetTitle());
 
   // Pan can be set when page is shown again.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "applyConstraints({ advanced: [{ pan: 102 }] });",
-      &result));
-  EXPECT_EQ(result, "applyConstraints-success");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "applyConstraints({ advanced: [{ pan: 102 }] });"),
+            "applyConstraints-success");
 
   // Tilt can be set when page is shown again.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "applyConstraints({ advanced: [{ tilt: 102 }] });",
-      &result));
-  EXPECT_EQ(result, "applyConstraints-success");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "applyConstraints({ advanced: [{ tilt: 102 }] });"),
+            "applyConstraints-success");
 
   // Zoom can be set when page is shown again.
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      tab->GetMainFrame(), "applyConstraints({ advanced: [{ zoom: 102 }] });",
-      &result));
-  EXPECT_EQ(result, "applyConstraints-success");
+  EXPECT_EQ(content::EvalJs(tab->GetPrimaryMainFrame(),
+                            "applyConstraints({ advanced: [{ zoom: 102 }] });"),
+            "applyConstraints-success");
 }
