@@ -51,7 +51,7 @@ TEST(ChromeNetworkDelegateStaticTest, IsAccessAllowed) {
   base::FilePath temp_dir;
   ASSERT_TRUE(base::PathService::Get(base::DIR_TEMP, &temp_dir));
   // Chrome OS allows the following directories.
-  EXPECT_TRUE(IsAccessAllowed("/home/chronos/user/Downloads", ""));
+  EXPECT_TRUE(IsAccessAllowed("/home/chronos/user/MyFiles/Downloads", ""));
   EXPECT_TRUE(IsAccessAllowed("/home/chronos/user/MyFiles", ""));
   EXPECT_TRUE(IsAccessAllowed("/home/chronos/user/MyFiles/file.pdf", ""));
   EXPECT_TRUE(IsAccessAllowed("/home/chronos/user/WebRTC Logs", ""));
@@ -68,6 +68,7 @@ TEST(ChromeNetworkDelegateStaticTest, IsAccessAllowed) {
   // Files under the directories are allowed.
   EXPECT_TRUE(IsAccessAllowed("/var/log/foo.txt", ""));
   // Make sure similar paths are not allowed.
+  EXPECT_FALSE(IsAccessAllowed("/home/chronos/user/Downloads", ""));
   EXPECT_FALSE(IsAccessAllowed("/home/chronos/user/log.txt", ""));
   EXPECT_FALSE(IsAccessAllowed("/home/chronos/user", ""));
   EXPECT_FALSE(IsAccessAllowed("/home/chronos", ""));
@@ -77,11 +78,14 @@ TEST(ChromeNetworkDelegateStaticTest, IsAccessAllowed) {
   EXPECT_FALSE(IsAccessAllowed("/home/chronos/user/lacros", ""));
   EXPECT_FALSE(
       IsAccessAllowed("/home/chronos/user/lacros/subdir/lacros.log", ""));
+
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   // If profile path is given, the following additional paths are allowed.
-  EXPECT_TRUE(IsAccessAllowed("/profile/Downloads", "/profile"));
+  EXPECT_TRUE(IsAccessAllowed("/profile/MyFiles/Downloads", "/profile"));
   EXPECT_TRUE(IsAccessAllowed("/profile/MyFiles", "/profile"));
   EXPECT_TRUE(IsAccessAllowed("/profile/MyFiles/file.pdf", "/profile"));
+  EXPECT_FALSE(IsAccessAllowed("/profile/Downloads", "/profile"));
+
   // $HOME/Downloads is allowed for linux-chromeos, but not on devices.
   base::FilePath downloads_dir;
   base::PathService::Get(chrome::DIR_DEFAULT_DOWNLOADS, &downloads_dir);
