@@ -51,6 +51,7 @@
 #include "net/test/embedded_test_server/embedded_test_server.h"
 #include "net/test/url_request/url_request_failed_job.h"
 #include "services/network/public/cpp/features.h"
+#include "services/network/public/cpp/network_context_getter.h"
 #include "services/network/public/mojom/clear_data_filter.mojom.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -78,8 +79,7 @@ namespace {
 class DelayingDnsProbeService : public DnsProbeService {
  public:
   DelayingDnsProbeService(
-      const DnsProbeServiceFactory::NetworkContextGetter&
-          network_context_getter,
+      const network::NetworkContextGetter& network_context_getter,
       const DnsProbeServiceFactory::DnsConfigChangeManagerGetter&
           dns_config_change_manager_getter)
       : dns_probe_service_impl_(DnsProbeServiceFactory::CreateForTesting(
@@ -90,8 +90,7 @@ class DelayingDnsProbeService : public DnsProbeService {
   ~DelayingDnsProbeService() override { EXPECT_TRUE(delayed_probes_.empty()); }
 
   static std::unique_ptr<KeyedService> Create(
-      const DnsProbeServiceFactory::NetworkContextGetter&
-          network_context_getter,
+      const network::NetworkContextGetter& network_context_getter,
       const DnsProbeServiceFactory::DnsConfigChangeManagerGetter&
           dns_config_change_manager_getter,
       content::BrowserContext* context) {
@@ -278,7 +277,7 @@ void DnsProbeBrowserTest::SetFakeHostResolverResults(
 void DnsProbeBrowserTest::NavigateToDnsError() {
   ASSERT_TRUE(NavigateToURL(
       active_browser_,
-      URLRequestFailedJob::GetMockHttpUrl(net::ERR_NAME_NOT_RESOLVED)));
+      URLRequestFailedJob::GetMockHttpsUrl(net::ERR_NAME_NOT_RESOLVED)));
 }
 
 void DnsProbeBrowserTest::NavigateToOtherError() {
@@ -412,7 +411,7 @@ class DnsProbeCurrentSecureConfigFailingProbesTest
     // Mark as not enterprise managed to prevent the secure DNS mode from
     // being downgraded to off.
     base::win::ScopedDomainStateForTesting scoped_domain(false);
-    // TODO(crbug.com/1339062): What is the correct function to use here?
+    // TODO(crbug.com/40229843): What is the correct function to use here?
     EXPECT_FALSE(base::win::IsEnrolledToDomain());
 #endif
 
@@ -433,11 +432,16 @@ class DnsProbeCurrentSecureConfigFailingProbesTest
     content::FlushNetworkServiceInstanceForTesting();
 
     // Update prefs to enable Secure DNS in secure mode.
-    PrefService* local_state = g_browser_process->local_state();
-    local_state->SetString(prefs::kDnsOverHttpsMode,
-                           SecureDnsConfig::kModeSecure);
-    local_state->SetString(prefs::kDnsOverHttpsTemplates,
-                           "https://bar.test/dns-query{?dns}");
+    PrefService* pref_service = g_browser_process->local_state();
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+    // On Chrome OS, the local_state is shared between all users so the user-set
+    // pref is stored in the profile's pref service.
+    pref_service = browser()->profile()->GetPrefs();
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+    pref_service->SetString(prefs::kDnsOverHttpsMode,
+                            SecureDnsConfig::kModeSecure);
+    pref_service->SetString(prefs::kDnsOverHttpsTemplates,
+                            "https://bar.test/dns-query{?dns}");
 
     SetFakeHostResolverResults(
         {{net::ERR_NAME_NOT_RESOLVED,
