@@ -21,7 +21,6 @@
 #include "base/strings/stringprintf.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/test/bind.h"
-#include "base/test/scoped_feature_list.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/uuid.h"
 #include "build/build_config.h"
@@ -40,7 +39,7 @@
 #include "chrome/common/webui_url_constants.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "components/content_settings/core/browser/cookie_settings.h"
-#include "components/content_settings/core/common/features.h"
+#include "components/content_settings/core/common/content_settings.h"
 #include "components/content_settings/core/common/pref_names.h"
 #include "components/embedder_support/switches.h"
 #include "components/embedder_support/user_agent_utils.h"
@@ -51,7 +50,6 @@
 #include "components/policy/core/common/policy_map.h"
 #include "components/policy/policy_constants.h"
 #include "components/prefs/pref_service.h"
-#include "components/privacy_sandbox/tracking_protection_prefs.h"
 #include "components/proxy_config/proxy_config_dictionary.h"
 #include "components/proxy_config/proxy_config_pref_names.h"
 #include "content/public/browser/browser_context.h"
@@ -92,6 +90,7 @@
 #include "net/url_request/referrer_policy.h"
 #include "services/network/public/cpp/features.h"
 #include "services/network/public/cpp/network_connection_tracker.h"
+#include "services/network/public/cpp/resource_request.h"
 #include "services/network/public/cpp/simple_url_loader.h"
 #include "services/network/public/mojom/cookie_manager.mojom.h"
 #include "services/network/public/mojom/host_resolver.mojom.h"
@@ -219,6 +218,11 @@ class NetworkContextConfigurationBrowserTest
 
   NetworkContextConfigurationBrowserTest()
       : https_server_(net::EmbeddedTestServer::TYPE_HTTPS) {
+    // TODO(crbug.com/334954143) Fix the tests when turning on the reduce
+    // accept-language feature.
+    scoped_feature_list_.InitWithFeatures(
+        {}, {network::features::kReduceAcceptLanguage});
+
     // Have to get a port before setting up the command line, but can only set
     // up the connection listener after there's a main thread, so can't start
     // the test server here.
@@ -226,11 +230,6 @@ class NetworkContextConfigurationBrowserTest
     EXPECT_TRUE(https_server()->InitializeAndListen());
   }
 
-  void SetUp() override {
-    InitializeFeatureList();
-    InProcessBrowserTest::SetUp();
-  }
-
   // Returns a cacheable response (10 hours) that is some random text.
   static std::unique_ptr<net::test_server::HttpResponse> HandleCacheRandom(
       const net::test_server::HttpRequest& request) {
@@ -332,7 +331,7 @@ class NetworkContextConfigurationBrowserTest
     switch (network_context_type) {
       case NetworkContextType::kSystem:
       case NetworkContextType::kSafeBrowsing:
-        NOTREACHED() << "Network context has no storage partition";
+        NOTREACHED_IN_MIGRATION() << "Network context has no storage partition";
         return nullptr;
       case NetworkContextType::kProfile:
         return browser()->profile()->GetDefaultStoragePartition();
@@ -356,7 +355,7 @@ class NetworkContextConfigurationBrowserTest
         return incognito_->profile()->GetStoragePartition(kIncognitoConfig);
       }
     }
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return nullptr;
   }
 
@@ -383,7 +382,7 @@ class NetworkContextConfigurationBrowserTest
             ->GetURLLoaderFactoryForBrowserProcess()
             .get();
     }
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return nullptr;
   }
 
@@ -408,7 +407,7 @@ class NetworkContextConfigurationBrowserTest
         return GetStoragePartitionForContextType(network_context_type)
             ->GetNetworkContext();
     }
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return nullptr;
   }
 
@@ -425,7 +424,7 @@ class NetworkContextConfigurationBrowserTest
       case NetworkContextType::kOnDiskAppWithIncognitoProfile:
         return StorageType::kMemory;
     }
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return StorageType::kNone;
   }
 
@@ -441,7 +440,7 @@ class NetworkContextConfigurationBrowserTest
       case NetworkContextType::kOnDiskApp:
         return StorageType::kDisk;
     }
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return StorageType::kNone;
   }
 
@@ -706,10 +705,6 @@ class NetworkContextConfigurationBrowserTest
     provider_.UpdateChromePolicy(policy_map);
   }
 
-  base::test::ScopedFeatureList* feature_list() { return &feature_list_; }
-
-  virtual void InitializeFeatureList() {}
-
  private:
   void SimulateNetworkServiceCrashIfNecessary() {
     if (GetParam().network_service_state != NetworkServiceState::kRestarted ||
@@ -738,12 +733,12 @@ class NetworkContextConfigurationBrowserTest
   std::unique_ptr<net::test_server::ControllableHttpResponse>
       controllable_http_response_;
 
-  base::test::ScopedFeatureList feature_list_;
   testing::NiceMock<policy::MockConfigurationPolicyProvider> provider_;
   // Used in tests that need a live request during browser shutdown.
   std::unique_ptr<network::SimpleURLLoader> live_during_shutdown_simple_loader_;
   std::unique_ptr<content::SimpleURLLoaderTestHelper>
       live_during_shutdown_simple_loader_helper_;
+  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest,
@@ -1044,7 +1039,7 @@ IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest, PRE_DiskCache) {
 
 // Check if the URL loaded in PRE_DiskCache is still in the cache, across a
 // browser restart.
-// TODO(https://crbug.com/1464367): Fix test flake and re-enable
+// TODO(crbug.com/40922934): Fix test flake and re-enable
 #if BUILDFLAG(IS_MAC)
 #define MAYBE_DiskCache DISABLED_DiskCache
 #else
@@ -1497,14 +1492,6 @@ IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest,
       }));
 }
 
-class NetworkContextConfigurationBrowserPre3pcdTest
-    : public NetworkContextConfigurationBrowserTest {
-  void InitializeFeatureList() override {
-    feature_list()->InitAndDisableFeature(
-        content_settings::features::kTrackingProtection3pcd);
-  }
-};
-
 // Disabled due to flakiness. See https://crbug.com/1126755.
 #if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
 #define MAYBE_PRE_ThirdPartyCookiesBlocked DISABLED_PRE_ThirdPartyCookiesBlocked
@@ -1513,7 +1500,7 @@ class NetworkContextConfigurationBrowserPre3pcdTest
 #define MAYBE_PRE_ThirdPartyCookiesBlocked PRE_ThirdPartyCookiesBlocked
 #define MAYBE_ThirdPartyCookiesBlocked ThirdPartyCookiesBlocked
 #endif
-IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserPre3pcdTest,
+IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest,
                        MAYBE_PRE_ThirdPartyCookiesBlocked) {
   if (IsRestartStateWithInProcessNetworkService())
     return;
@@ -1535,7 +1522,7 @@ IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserPre3pcdTest,
 }
 
 // Disabled due to flakiness. See https://crbug.com/1126755.
-IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserPre3pcdTest,
+IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest,
                        MAYBE_ThirdPartyCookiesBlocked) {
   if (IsRestartStateWithInProcessNetworkService())
     return;
@@ -1556,10 +1543,9 @@ IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserPre3pcdTest,
 
   EXPECT_TRUE(GetCookies(https_server()->base_url()).empty());
 
-  // Set pref to false, third party cookies should be allowed now.
-  GetPrefService()->SetInteger(
-      prefs::kCookieControlsMode,
-      static_cast<int>(content_settings::CookieControlsMode::kOff));
+  // Add exception, third party cookies should be allowed now.
+  CookieSettingsFactory::GetForProfile(browser()->profile())
+      ->SetCookieSetting(https_server()->base_url(), CONTENT_SETTING_ALLOW);
   // Set a third-party cookie. It should actually get set this time.
   SetCookie(CookieType::kThirdParty, CookiePersistenceType::kSession,
             https_server());
@@ -1567,26 +1553,6 @@ IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserPre3pcdTest,
   EXPECT_FALSE(GetCookies(https_server()->base_url()).empty());
 }
 
-IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest,
-                       ThirdPartyCookiesBlocked) {
-  if (IsRestartStateWithInProcessNetworkService()) {
-    return;
-  }
-  // The system and SafeBrowsing network contexts don't support the third party
-  // cookie blocking options, since they have no notion of third parties.
-  bool system =
-      GetParam().network_context_type == NetworkContextType::kSystem ||
-      GetParam().network_context_type == NetworkContextType::kSafeBrowsing;
-  if (system) {
-    return;
-  }
-
-  GetPrefService()->SetBoolean(prefs::kTrackingProtection3pcdEnabled, true);
-  SetCookie(CookieType::kThirdParty, CookiePersistenceType::kSession,
-            https_server());
-  EXPECT_TRUE(GetCookies(https_server()->base_url()).empty());
-}
-
 IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationBrowserTest,
                        PRE_CookieSettings) {
   if (IsRestartStateWithInProcessNetworkService())
@@ -2257,8 +2223,6 @@ IN_PROC_BROWSER_TEST_P(NetworkContextConfigurationReportingAndNelBrowserTest,
       ::testing::Values(TEST_CASES(NetworkContextType::kIncognitoProfile)))
 
 INSTANTIATE_TEST_CASES_FOR_TEST_FIXTURE(NetworkContextConfigurationBrowserTest);
-INSTANTIATE_TEST_CASES_FOR_TEST_FIXTURE(
-    NetworkContextConfigurationBrowserPre3pcdTest);
 INSTANTIATE_TEST_CASES_FOR_TEST_FIXTURE(
     NetworkContextConfigurationFixedPortBrowserTest);
 INSTANTIATE_TEST_CASES_FOR_TEST_FIXTURE(
