@@ -26,6 +26,7 @@
 #include "components/prefs/pref_service.h"
 #include "content/public/browser/web_contents_observer.h"
 #include "content/public/test/browser_test.h"
+#include "content/public/test/browser_test_utils.h"
 #include "content/public/test/download_test_observer.h"
 #include "content/public/test/test_navigation_observer.h"
 #include "net/base/filename_util.h"
@@ -135,7 +136,7 @@ class NetworkRequestMetricsBrowserTest
         return base::StringPrintf("<script src='%s'></script>",
                                   subresource_path.c_str());
       case RequestType::kMainFrame:
-        NOTREACHED();
+        NOTREACHED_IN_MIGRATION();
     }
     return std::string();
   }
