@@ -15,20 +15,24 @@
 #include "base/memory/raw_ptr.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/scoped_observation.h"
+#include "base/threading/sequence_bound.h"
 #include "base/timer/timer.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/net/proxy_config_monitor.h"
+#include "chrome/browser/net/server_certificate_database.h"
 #include "chrome/common/buildflags.h"
 #include "components/content_settings/core/browser/content_settings_observer.h"
 #include "components/content_settings/core/browser/cookie_settings.h"
 #include "components/keyed_service/core/keyed_service.h"
 #include "components/prefs/pref_change_registrar.h"
 #include "components/prefs/pref_member.h"
-#include "components/privacy_sandbox/privacy_sandbox_settings.h"
+#include "content/public/browser/storage_partition.h"
 #include "net/net_buildflags.h"
 #include "services/cert_verifier/public/mojom/cert_verifier_service_factory.mojom-forward.h"
-#include "services/network/public/mojom/network_context.mojom.h"
+#include "services/network/public/mojom/cert_verifier_service_updater.mojom.h"
+#include "services/network/public/mojom/cookie_manager.mojom-forward.h"
+#include "services/network/public/mojom/network_context.mojom-forward.h"
 
 class PrefRegistrySimple;
 class Profile;
@@ -56,8 +60,7 @@ class PrefRegistrySyncable;
 class ProfileNetworkContextService
     : public KeyedService,
       public content_settings::Observer,
-      public content_settings::CookieSettings::Observer,
-      public privacy_sandbox::PrivacySandboxSettings::Observer {
+      public content_settings::CookieSettings::Observer {
  public:
   explicit ProfileNetworkContextService(Profile* profile);
 
@@ -77,9 +80,25 @@ class ProfileNetworkContextService
       cert_verifier::mojom::CertVerifierCreationParams*
           cert_verifier_creation_params);
 
-#if BUILDFLAG(IS_CHROMEOS)
+  // Update all of the profile_'s CertVerifierServices with certificates from
+  // enterprise policies.
   void UpdateAdditionalCertificates();
-#endif
+
+  struct CertificatePoliciesForView {
+    CertificatePoliciesForView();
+    ~CertificatePoliciesForView();
+    CertificatePoliciesForView(CertificatePoliciesForView&&);
+    CertificatePoliciesForView& operator=(CertificatePoliciesForView&& other);
+
+    cert_verifier::mojom::AdditionalCertificatesPtr certificate_policies;
+
+    bool is_include_system_trust_store_managed;
+
+    std::vector<std::vector<uint8_t>> full_distrusted_certs;
+  };
+
+  // Get enterprise certificate policies for viewing by end users.
+  CertificatePoliciesForView GetCertificatePolicyForView();
 
   static void RegisterProfilePrefs(user_prefs::PrefRegistrySyncable* registry);
   static void RegisterLocalStatePrefs(PrefRegistrySimple* registry);
@@ -96,6 +115,11 @@ class ProfileNetworkContextService
       const net::HostPortPair& host,
       const scoped_refptr<net::X509Certificate>& certificate);
 
+  // Flushes a cached client certificate preference if |certificate| matches
+  // the cached certificate.
+  void FlushMatchingCachedClientCert(
+      const scoped_refptr<net::X509Certificate>& certificate);
+
   // Flushes all pending proxy configuration changes.
   void FlushProxyConfigMonitorForTesting();
 
@@ -146,23 +170,21 @@ class ProfileNetworkContextService
 
   void ScheduleUpdateCTPolicy();
 
-#if BUILDFLAG(CHROME_CERTIFICATE_POLICIES_SUPPORTED)
-  // Get the current certificate policies from preferences.
-  cert_verifier::mojom::AdditionalCertificatesPtr GetCertificatePolicy();
-
-  // Update the certificate policy for all of the profile_'s
-  // CertVerifierServices.
-  void UpdateCertificatePolicy();
-
   void ScheduleUpdateCertificatePolicy();
-#endif
+
+  // Get the current certificate policies from preferences.
+  cert_verifier::mojom::AdditionalCertificatesPtr GetCertificatePolicy(
+      const base::FilePath& storage_partition_path);
 
   bool ShouldSplitAuthCacheByNetworkIsolationKey() const;
   void UpdateSplitAuthCacheByNetworkIsolationKey();
 
   void UpdateCorsNonWildcardRequestHeadersSupport();
 
-  void OnTruncatedCookieBlockingChanged();
+#if BUILDFLAG(ENABLE_REPORTING)
+  base::flat_map<std::string, GURL> GetEnterpriseReportingEndpoints() const;
+  void UpdateEnterpriseReportingEndpoints();
+#endif
 
   // Creates parameters for the NetworkContext. Use |in_memory| instead of
   // |profile_->IsOffTheRecord()| because sometimes normal profiles want off the
@@ -178,12 +200,6 @@ class ProfileNetworkContextService
   base::FilePath GetPartitionPath(
       const base::FilePath& relative_partition_path);
 
-  // Populates |network_context_params| with initial additional server and
-  // authority certificates for |relative_partition_path|.
-  void PopulateInitialAdditionalCerts(
-      const base::FilePath& relative_partition_path,
-      cert_verifier::mojom::CertVerifierCreationParams* creation_params);
-
   // content_settings::Observer:
   void OnContentSettingChanged(const ContentSettingsPattern& primary_pattern,
                                const ContentSettingsPattern& secondary_pattern,
@@ -195,9 +211,6 @@ class ProfileNetworkContextService
   void OnMitigationsEnabledFor3pcdChanged(bool enable) override;
   void OnTrackingProtectionEnabledFor3pcdChanged(bool enable) override;
 
-  // PrivacySandboxSettings::Observer:
-  void OnFirstPartySetsEnabledChanged(bool enabled) override;
-
   const raw_ptr<Profile> profile_;
 
   ProxyConfigMonitor proxy_config_monitor_;
@@ -211,19 +224,17 @@ class ProfileNetworkContextService
   base::ScopedObservation<content_settings::CookieSettings,
                           content_settings::CookieSettings::Observer>
       cookie_settings_observation_{this};
-  base::ScopedObservation<privacy_sandbox::PrivacySandboxSettings,
-                          privacy_sandbox::PrivacySandboxSettings::Observer>
-      privacy_sandbox_settings_observer_{this};
 
   // Used to post schedule CT and Certificate policy updates
   base::OneShotTimer ct_policy_update_timer_;
-#if BUILDFLAG(CHROME_CERTIFICATE_POLICIES_SUPPORTED)
   base::OneShotTimer cert_policy_update_timer_;
-#endif
 
   // Used for testing.
   base::RepeatingCallback<std::unique_ptr<net::ClientCertStore>()>
       client_cert_store_factory_;
+#if BUILDFLAG(CHROME_ROOT_STORE_CERT_MANAGEMENT_UI)
+  base::SequenceBound<net::ServerCertificateDatabase> server_cert_database_;
+#endif
 };
 
 #endif  // CHROME_BROWSER_NET_PROFILE_NETWORK_CONTEXT_SERVICE_H_
