@@ -13,7 +13,6 @@
 #include "base/feature_list.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/ranges/algorithm.h"
-#include "base/strings/string_piece.h"
 #include "build/build_config.h"
 #include "chrome/browser/net/dns_probe_runner.h"
 #include "chrome/common/chrome_features.h"
@@ -56,8 +55,8 @@ void RegisterProbesSettingBackupPref(PrefRegistrySimple* registry) {
 }
 
 void MigrateProbesSettingToOrFromBackup(PrefService* prefs) {
-// TODO(crbug.com/1177778): remove this code around M97 to make sure the vast
-// majority of the clients are migrated.
+  // TODO(crbug.com/40748688): remove this code around M97 to make sure the vast
+  // majority of the clients are migrated.
   if (!prefs->HasPrefPath(kAlternateErrorPagesBackup)) {
     // If the user never changed the value of the preference and still uses
     // the hardcoded default value, we'll consider it to be the user value for
@@ -90,10 +89,11 @@ net::DohProviderEntry::List ProvidersForCountry(
 net::DohProviderEntry::List SelectEnabledProviders(
     const net::DohProviderEntry::List& providers) {
   net::DohProviderEntry::List enabled_providers;
-  base::ranges::copy_if(providers, std::back_inserter(enabled_providers),
-                        [](const net::DohProviderEntry* entry) {
-                          return base::FeatureList::IsEnabled(entry->feature);
-                        });
+  base::ranges::copy_if(
+      providers, std::back_inserter(enabled_providers),
+      [](const net::DohProviderEntry* entry) {
+        return base::FeatureList::IsEnabled(entry->feature.get());
+      });
   return enabled_providers;
 }
 
@@ -107,7 +107,7 @@ void UpdateProbeHistogram(bool success) {
 
 std::unique_ptr<DnsProbeRunner> MakeProbeRunner(
     net::DnsOverHttpsConfig doh_config,
-    const DnsProbeRunner::NetworkContextGetter& network_context_getter) {
+    const network::NetworkContextGetter& network_context_getter) {
   net::DnsConfigOverrides overrides;
   overrides.search = std::vector<std::string>();
   overrides.attempts = 1;
