@@ -23,7 +23,7 @@ ChromePrefModelAssociatorClient::ChromePrefModelAssociatorClient() = default;
 ChromePrefModelAssociatorClient::~ChromePrefModelAssociatorClient() = default;
 
 base::Value ChromePrefModelAssociatorClient::MaybeMergePreferenceValues(
-    const std::string& pref_name,
+    std::string_view pref_name,
     const base::Value& local_value,
     const base::Value& server_value) const {
   if (pref_name == prefs::kNetworkEasterEggHighScore) {
