@@ -32,7 +32,6 @@
 #include "chrome/grit/generated_resources.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "chrome/test/base/ui_test_utils.h"
-#include "content/public/browser/browser_accessibility_state.h"
 #include "content/public/browser/plugin_service.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
@@ -40,8 +39,10 @@
 #include "content/public/common/webplugininfo.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
+#include "content/public/test/scoped_accessibility_mode_override.h"
 #include "content/public/test/test_utils.h"
 #include "ipc/ipc_message_macros.h"
+#include "ui/accessibility/ax_mode.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/ui_base_switches.h"
 #include "url/gurl.h"
@@ -146,7 +147,7 @@ class PrintPreviewDialogControllerBrowserTest : public InProcessBrowserTest {
 
 // Test to verify that when a initiator navigates, we can create a new preview
 // dialog for the new tab contents.
-// TODO(crbug.com/1403898): Test is flaky on Mac
+// TODO(crbug.com/40251696): Test is flaky on Mac
 #if BUILDFLAG(IS_MAC)
 #define MAYBE_NavigateFromInitiatorTab DISABLED_NavigateFromInitiatorTab
 #else
@@ -187,7 +188,7 @@ IN_PROC_BROWSER_TEST_F(PrintPreviewDialogControllerBrowserTest,
 
 // Test to verify that after reloading the initiator, it creates a new print
 // preview dialog.
-// TODO(crbug.com/1403898): Test is flaky on Mac
+// TODO(crbug.com/40251696): Test is flaky on Mac
 #if BUILDFLAG(IS_MAC)
 #define MAYBE_ReloadInitiatorTab DISABLED_ReloadInitiatorTab
 #else
@@ -231,7 +232,7 @@ IN_PROC_BROWSER_TEST_F(PrintPreviewDialogControllerBrowserTest,
 
 // Test to verify that after print preview works even when the PDF plugin is
 // disabled for webpages.
-// TODO(crbug.com/1401532): Flaky on Mac12 Test.
+// TODO(crbug.com/40884297): Flaky on Mac12 Test.
 #if BUILDFLAG(IS_MAC)
 #define MAYBE_PdfPluginDisabled DISABLED_PdfPluginDisabled
 #else
@@ -305,7 +306,7 @@ GetTrackedTags() {
 
 }  // namespace
 
-// TODO(crbug.com/1385142): Flaky on macos12 builds.
+// TODO(crbug.com/40879071): Flaky on macos12 builds.
 #if BUILDFLAG(IS_MAC)
 #define MAYBE_TaskManagementTest DISABLED_TaskManagementTest
 #else
@@ -358,12 +359,17 @@ IN_PROC_BROWSER_TEST_F(PrintPreviewDialogControllerBrowserTest,
 
 IN_PROC_BROWSER_TEST_F(PrintPreviewDialogControllerBrowserTest,
                        PrintPreviewPdfAccessibility) {
-  content::BrowserAccessibilityState::GetInstance()->EnableAccessibility();
-  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(),
-                                           GURL("data:text/html,HelloWorld")));
+  content::ScopedAccessibilityModeOverride mode_override(ui::kAXModeComplete);
+  // Put a DIV after the text we're going to search for. The last node in the
+  // tree will not have a newline appended, but all the others will. Avoid
+  // making assumptions about whether it's the last node or not. There may be
+  // nodes for headers and footers following the document contents.
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(
+      browser(), GURL("data:text/html,HelloWorld<div>next</div>")));
   PrintPreview();
   WebContents* preview_dialog = GetPrintPreviewDialog();
-  WaitForAccessibilityTreeToContainNodeWithName(preview_dialog, "HelloWorld");
+  WaitForAccessibilityTreeToContainNodeWithName(preview_dialog,
+                                                "HelloWorld\r\n");
 
   PrintPreviewDone();
 }
