@@ -10,7 +10,7 @@
 #include "base/check_op.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/printing/prefs_util.h"
+#include "chrome/browser/printing/oop_features.h"
 #include "chrome/browser/printing/print_backend_service_manager.h"
 #include "chrome/browser/printing/print_job_worker_oop.h"
 #include "components/device_event_log/device_event_log.h"
@@ -28,7 +28,7 @@ PrinterQueryOop::~PrinterQueryOop() = default;
 std::unique_ptr<PrintJobWorker> PrinterQueryOop::TransferContextToNewWorker(
     PrintJob* print_job) {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-  // TODO(crbug.com/1414968)  Do extra setup on the worker as needed for
+  // TODO(crbug.com/40256381)  Do extra setup on the worker as needed for
   // supporting OOP system print dialogs.
   return CreatePrintJobWorkerOop(print_job);
 }
@@ -72,7 +72,7 @@ void PrinterQueryOop::OnDidUseDefaultSettings(
     PRINTER_LOG(ERROR) << "Error trying to use default settings via service: "
                        << result;
 
-    // TODO(crbug.com/809738)  Fill in support for handling of access-denied
+    // TODO(crbug.com/40561724)  Fill in support for handling of access-denied
     // result code.  Blocked on crbug.com/1243873 for Windows.
   } else {
     VLOG(1) << "Use default settings from service complete";
@@ -100,6 +100,12 @@ void PrinterQueryOop::OnDidAskUserForSettings(
         PrintBackendServiceManager::GetInstance()
             .RegisterPrintDocumentClientReusingClientRemote(
                 *query_with_ui_client_id_);
+    if (!print_document_client_id_.has_value()) {
+      // A failure after getting settings, override result to failure.
+      result = mojom::ResultCode::kFailed;
+      PRINTER_LOG(ERROR) << "Error after getting settings from user via "
+                            "service due to service unavailable";
+    }
   } else {
     result = print_settings->get_result_code();
     DCHECK_NE(result, mojom::ResultCode::kSuccess);
@@ -108,7 +114,7 @@ void PrinterQueryOop::OnDidAskUserForSettings(
                          << result;
     }
 
-    // TODO(crbug.com/809738)  Fill in support for handling of access-denied
+    // TODO(crbug.com/40561724)  Fill in support for handling of access-denied
     // result code.  Blocked on crbug.com/1243873 for Windows.
   }
 
@@ -127,6 +133,12 @@ void PrinterQueryOop::OnDidAskUserForSettings(
         PrintBackendServiceManager::GetInstance()
             .RegisterPrintDocumentClientReusingClientRemote(
                 *query_with_ui_client_id_);
+    if (!print_document_client_id_.has_value()) {
+      // A failure after getting settings, override result to failure.
+      result = mojom::ResultCode::kFailed;
+      PRINTER_LOG(ERROR) << "Error after getting settings from user via "
+                            "service due to service unavailable";
+    }
   }
   std::move(callback).Run(std::move(new_settings), result);
 }
@@ -170,7 +182,8 @@ void PrinterQueryOop::GetSettingsWithUI(uint32_t document_page_count,
   //   - macOS:  It is impossible to invoke a system dialog UI from a service
   //       utility and have that dialog be application modal for a window that
   //       was launched by the browser process.
-  //   - Linux:  TODO(crbug.com/809738)  Determine if Linux Wayland can be made
+  //   - Linux:  TODO(crbug.com/40561724)  Determine if Linux Wayland can be
+  //   made
   //       to have a system dialog be modal against an application window in the
   //       browser process.
   //   - Other platforms don't have a system print UI or do not use OOP
@@ -261,7 +274,7 @@ void PrinterQueryOop::OnDidUpdatePrintSettings(
       context_id_.reset();
     }
 
-    // TODO(crbug.com/809738)  Fill in support for handling of access-denied
+    // TODO(crbug.com/40561724)  Fill in support for handling of access-denied
     // result code.
   } else {
     VLOG(1) << "Update print settings via service complete for " << device_name;
@@ -276,6 +289,12 @@ void PrinterQueryOop::OnDidUpdatePrintSettings(
           PrintBackendServiceManager::GetInstance()
               .RegisterPrintDocumentClientReusingClientRemote(
                   *query_with_ui_client_id_);
+      if (!print_document_client_id_.has_value()) {
+        // A failure after getting settings, override result to failure.
+        result = mojom::ResultCode::kFailed;
+        PRINTER_LOG(ERROR) << "Error after updating print settings via service "
+                              "due to service unavailable";
+      }
     }
   }
   InvokeSettingsCallback(std::move(callback), result);
