@@ -4,6 +4,8 @@
 
 #include "chrome/browser/push_messaging/push_messaging_features.h"
 
+#include "chrome/browser/push_messaging/push_messaging_constants.h"
+
 namespace features {
 
 BASE_FEATURE(kPushMessagingDisallowSenderIDs,
@@ -14,6 +16,10 @@ BASE_FEATURE(kPushSubscriptionWithExpirationTime,
              "PushSubscriptionWithExpirationTime",
              base::FEATURE_DISABLED_BY_DEFAULT);
 
+BASE_FEATURE(kPushMessagingGcmEndpointEnvironment,
+             "PushMessagingGcmEndpointEnvironment",
+             base::FEATURE_ENABLED_BY_DEFAULT);
+
 #if BUILDFLAG(IS_ANDROID)
 BASE_FEATURE(kRevokeNotificationsPermissionIfDisabledOnAppLevel,
              "RevokeNotificationsPermissionIfDisabledOnAppLevel",
