@@ -132,6 +132,7 @@ void PushMessagingNotificationManager::EnforceUserVisibleOnlyRequirements(
                                             requested_user_visible_only)) {
     std::move(message_handled_callback)
         .Run(/* did_show_generic_notification= */ false);
+    LogSilentPushEvent(SilentPushEvent::kNotificationEnforcementSkipped);
     return;
   }
 
@@ -158,7 +159,7 @@ void PushMessagingNotificationManager::DidCountVisibleNotifications(
   // user-visible action done in response to a push message - but make sure that
   // sending two messages in rapid succession which show then hide a
   // notification doesn't count.
-  // TODO(crbug.com/891339): Scheduling a notification should count as a
+  // TODO(crbug.com/40596304): Scheduling a notification should count as a
   // user-visible action, if it is not immediately cancelled or the |origin|
   // schedules too many notifications too far in the future.
   bool notification_shown = notification_count > 0;
@@ -254,11 +255,13 @@ void PushMessagingNotificationManager::ProcessSilentPush(
     EnforceRequirementsCallback message_handled_callback,
     bool silent_push_allowed) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  LogSilentPushEvent(SilentPushEvent::kSilentRequest);
 
   // If the origin was allowed to issue a silent push, just return.
   if (silent_push_allowed) {
     std::move(message_handled_callback)
         .Run(/* did_show_generic_notification= */ false);
+    LogSilentPushEvent(SilentPushEvent::kAllowedWithoutNotification);
     return;
   }
 
@@ -296,6 +299,7 @@ void PushMessagingNotificationManager::DidWriteNotificationData(
 
   std::move(message_handled_callback)
       .Run(/* did_show_generic_notification= */ true);
+  LogSilentPushEvent(SilentPushEvent::kAllowedWithGenericNotification);
 }
 
 bool PushMessagingNotificationManager::ShouldSkipUserVisibleOnlyRequirements(
@@ -313,6 +317,11 @@ bool PushMessagingNotificationManager::ShouldSkipUserVisibleOnlyRequirements(
   return false;
 }
 
+void PushMessagingNotificationManager::LogSilentPushEvent(
+    SilentPushEvent event) {
+  UMA_HISTOGRAM_ENUMERATION("PushMessaging.SilentNotification", event);
+}
+
 #if BUILDFLAG(ENABLE_EXTENSIONS)
 bool PushMessagingNotificationManager::
     ShouldSkipExtensionUserVisibleOnlyRequirements(
