@@ -10,8 +10,15 @@
 
 class GURL;
 
+namespace version_info {
+enum class Channel;
+}  // namespace version_info
+
 namespace push_messaging {
 
+// Returns the appropriate GCM endpoint for the given Chrome |channel|.
+std::string GetGcmEndpointForChannel(version_info::Channel channel);
+
 // Returns the URL used to send push messages to the subscription identified
 // by |subscription_id|.
 GURL CreateEndpoint(const std::string& subscription_id);
