@@ -4,11 +4,13 @@
 
 import './manager.js';
 
+export {CrIconElement} from 'chrome://resources/cr_elements/cr_icon/cr_icon.js';
 export {CrToastManagerElement} from 'chrome://resources/cr_elements/cr_toast/cr_toast_manager.js';
 export {loadTimeData} from 'chrome://resources/js/load_time_data.js';
 export {BrowserProxy} from './browser_proxy.js';
+export {DownloadsDangerousDownloadInterstitialElement} from './dangerous_download_interstitial.js';
 export {MojomData} from './data.js';
-export {DangerType, PageCallbackRouter, PageHandlerInterface, PageRemote, SafeBrowsingState, State} from './downloads.mojom-webui.js';
+export {DangerType, PageCallbackRouter, PageHandlerInterface, PageRemote, SafeBrowsingState, State, TailoredWarningType} from './downloads.mojom-webui.js';
 export {IconLoader, IconLoaderImpl} from './icon_loader.js';
 export {DownloadsItemElement} from './item.js';
 export {DownloadsManagerElement} from './manager.js';
