@@ -1,11 +1,11 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import 'chrome://resources/cr_elements/cr_dialog/cr_dialog.m.js';
+import 'chrome://resources/cr_elements/cr_dialog/cr_dialog.js';
 
-import {CrDialogElement} from 'chrome://resources/cr_elements/cr_dialog/cr_dialog.m.js';
-import {EventTracker} from 'chrome://resources/js/event_tracker.m.js';
+import type {CrDialogElement} from 'chrome://resources/cr_elements/cr_dialog/cr_dialog.js';
+import {EventTracker} from 'chrome://resources/js/event_tracker.js';
 import {Debouncer, PolymerElement, timeOut} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {navigation, Page} from './navigation_helper.js';
@@ -74,7 +74,10 @@ export class ExtensionsOptionsDialogElement extends PolymerElement {
    * into account the window width/height.
    */
   private updateDialogSize_() {
-    const headerHeight = this.$.body.offsetTop;
+    let headerHeight = this.$.body.offsetTop;
+    if (this.$.body.assignedSlot && this.$.body.assignedSlot.parentElement) {
+      headerHeight = this.$.body.assignedSlot.parentElement.offsetTop;
+    }
     const maxHeight =
         Math.min(0.9 * window.innerHeight, OptionsDialogMaxHeight);
     const effectiveHeight =
