@@ -10,8 +10,8 @@ import 'chrome://resources/cr_elements/cr_shared_vars.css.js';
 import './pack_dialog_alert.js';
 import './strings.m.js';
 
-import {CrDialogElement} from 'chrome://resources/cr_elements/cr_dialog/cr_dialog.js';
-import {CrInputElement} from 'chrome://resources/cr_elements/cr_input/cr_input.js';
+import type {CrDialogElement} from 'chrome://resources/cr_elements/cr_dialog/cr_dialog.js';
+import type {CrInputElement} from 'chrome://resources/cr_elements/cr_input/cr_input.js';
 import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {getTemplate} from './pack_dialog.html.js';
@@ -93,11 +93,11 @@ export class ExtensionsPackDialogElement extends PolymerElement {
     });
   }
 
-  private onCancelTap_() {
+  private onCancelClick_() {
     this.$.dialog.cancel();
   }
 
-  private onConfirmTap_() {
+  private onConfirmClick_() {
     this.delegate.packExtension(this.packDirectory_, this.keyFile_, 0)
         .then(response => this.onPackResponse_(response));
   }
