@@ -3,14 +3,13 @@
 // found in the LICENSE file.
 
 import './iframe.js';
-import './realbox/realbox.js';
 import './logo.js';
+import './strings.m.js';
+import 'chrome://resources/cr_components/searchbox/searchbox.js';
 import 'chrome://resources/cr_elements/cr_button/cr_button.js';
-import 'chrome://resources/cr_elements/cr_shared_style.css.js';
 
 import {ColorChangeUpdater} from 'chrome://resources/cr_components/color_change_listener/colors_css_updater.js';
-import type {HelpBubbleMixinInterface} from 'chrome://resources/cr_components/help_bubble/help_bubble_mixin.js';
-import {HelpBubbleMixin} from 'chrome://resources/cr_components/help_bubble/help_bubble_mixin.js';
+import {HelpBubbleMixinLit} from 'chrome://resources/cr_components/help_bubble/help_bubble_mixin_lit.js';
 import type {CrToastElement} from 'chrome://resources/cr_elements/cr_toast/cr_toast.js';
 import type {ClickInfo} from 'chrome://resources/js/browser_command.mojom-webui.js';
 import {Command} from 'chrome://resources/js/browser_command.mojom-webui.js';
@@ -18,15 +17,16 @@ import {BrowserCommandProxy} from 'chrome://resources/js/browser_command/browser
 import {hexColorToSkColor, skColorToRgba} from 'chrome://resources/js/color_utils.js';
 import {EventTracker} from 'chrome://resources/js/event_tracker.js';
 import {FocusOutlineManager} from 'chrome://resources/js/focus_outline_manager.js';
+import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
 import {getTrustedScriptURL} from 'chrome://resources/js/static_types.js';
+import {CrLitElement} from 'chrome://resources/lit/v3_0/lit.rollup.js';
+import type {PropertyValues} from 'chrome://resources/lit/v3_0/lit.rollup.js';
 import type {SkColor} from 'chrome://resources/mojo/skia/public/mojom/skcolor.mojom-webui.js';
-import type {DomIf} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
-import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
-import {getTemplate} from './app.html.js';
+import {getCss} from './app.css.js';
+import {getHtml} from './app.html.js';
 import {BackgroundManager} from './background_manager.js';
 import {CustomizeDialogPage} from './customize_dialog_types.js';
-import {loadTimeData} from './i18n_setup.js';
 import type {IframeElement} from './iframe.js';
 import type {LogoElement} from './logo.js';
 import {recordDuration, recordLoadDuration} from './metrics_utils.js';
@@ -64,7 +64,9 @@ export enum NtpElement {
   MODULE = 7,
   CUSTOMIZE = 8,  // Obsolete
   CUSTOMIZE_BUTTON = 9,
-  CUSTOMIZE_DIALOG = 10,
+  CUSTOMIZE_DIALOG = 10,  // Obsolete
+  WALLPAPER_SEARCH_BUTTON = 11,
+  MAX_VALUE = WALLPAPER_SEARCH_BUTTON,
 }
 
 /**
@@ -76,6 +78,19 @@ export enum NtpCustomizeChromeEntryPoint {
   CUSTOMIZE_BUTTON = 0,
   MODULE = 1,
   URL = 2,
+  WALLPAPER_SEARCH_BUTTON = 3,
+  MAX_VALUE = WALLPAPER_SEARCH_BUTTON,
+}
+
+/**
+ * Defines the conditions that hide the wallpaper search button on the New Tab
+ * Page.
+ */
+enum NtpWallpaperSearchButtonHideCondition {
+  NONE = 0,
+  BACKGROUND_IMAGE_SET = 1,
+  THEME_SET = 2,
+  MAX_VALUE = THEME_SET,
 }
 
 const CUSTOMIZE_URL_PARAM: string = 'customize';
@@ -84,15 +99,19 @@ const OGB_IFRAME_ORIGIN = 'chrome-untrusted://new-tab-page';
 export const CUSTOMIZE_CHROME_BUTTON_ELEMENT_ID =
     'NewTabPageUI::kCustomizeChromeButtonElementId';
 
+// 900px ~= 561px (max value for --ntp-search-box-width) * 1.5 + some margin.
+const realboxCanShowSecondarySideMediaQueryList =
+    window.matchMedia('(min-width: 900px)');
+
 function recordClick(element: NtpElement) {
   chrome.metricsPrivate.recordEnumerationValue(
-      'NewTabPage.Click', element, Object.keys(NtpElement).length);
+      'NewTabPage.Click', element, NtpElement.MAX_VALUE + 1);
 }
 
 function recordCustomizeChromeOpen(element: NtpCustomizeChromeEntryPoint) {
   chrome.metricsPrivate.recordEnumerationValue(
       'NewTabPage.CustomizeChromeOpened', element,
-      Object.keys(NtpCustomizeChromeEntryPoint).length);
+      NtpCustomizeChromeEntryPoint.MAX_VALUE + 1);
 }
 
 // Adds a <script> tag that holds the lazy loaded code.
@@ -103,12 +122,10 @@ function ensureLazyLoaded() {
   document.body.appendChild(script);
 }
 
-const AppElementBase = HelpBubbleMixin(PolymerElement) as
-    {new (): PolymerElement & HelpBubbleMixinInterface};
+const AppElementBase = HelpBubbleMixinLit(CrLitElement);
 
 export interface AppElement {
   $: {
-    customizeDialogIf: DomIf,
     oneGoogleBarClipPath: HTMLElement,
     logo: LogoElement,
   };
@@ -119,178 +136,101 @@ export class AppElement extends AppElementBase {
     return 'ntp-app';
   }
 
-  static get template() {
-    return getTemplate();
+  static override get styles() {
+    return getCss();
   }
 
-  static get properties() {
+  override render() {
+    return getHtml.bind(this)();
+  }
+
+  static override get properties() {
     return {
-      oneGoogleBarIframeOrigin_: {
-        type: String,
-        value: OGB_IFRAME_ORIGIN,
-      },
+      oneGoogleBarIframeOrigin_: {type: String},
+      oneGoogleBarIframePath_: {type: String},
+      oneGoogleBarLoaded_: {type: Boolean},
+      theme_: {type: Object},
+      showCustomize_: {type: Boolean},
+      showCustomizeChromeText_: {type: Boolean},
 
-      oneGoogleBarIframePath_: {
-        type: String,
-        value: () => {
-          const params = new URLSearchParams();
-          params.set(
-              'paramsencoded',
-              btoa(window.location.search.replace(/^[?]/, '&')));
-          return `${OGB_IFRAME_ORIGIN}/one-google-bar?${params}`;
-        },
-      },
-
-      theme_: {
-        observer: 'onThemeChange_',
-        type: Object,
-      },
-
-      showCustomize_: {
+      showWallpaperSearch_: {
         type: Boolean,
-        value: () =>
-            WindowProxy.getInstance().url.searchParams.has(CUSTOMIZE_URL_PARAM),
+        reflect: true,
       },
 
-      showCustomizeDialog_: {
-        type: Boolean,
-        computed:
-            'computeShowCustomizeDialog_(customizeChromeEnabled_, showCustomize_)',
-      },
-
-      selectedCustomizeDialogPage_: {
-        type: String,
-        value: () =>
-            WindowProxy.getInstance().url.searchParams.get(CUSTOMIZE_URL_PARAM),
-      },
-
-      showVoiceSearchOverlay_: Boolean,
+      selectedCustomizeDialogPage_: {type: String},
+      showVoiceSearchOverlay_: {type: Boolean},
 
       showBackgroundImage_: {
-        computed: 'computeShowBackgroundImage_(theme_)',
-        observer: 'onShowBackgroundImageChange_',
-        reflectToAttribute: true,
+        reflect: true,
         type: Boolean,
       },
 
-      backgroundImageAttribution1_: {
+      backgroundImageAttribution1_: {type: String},
+      backgroundImageAttribution2_: {type: String},
+      backgroundImageAttributionUrl_: {type: String},
+      backgroundColor_: {type: Object},
+
+      // Used in cr-searchbox component via host-context.
+      colorSourceIsBaseline: {type: Boolean},
+      logoColor_: {type: String},
+      singleColoredLogo_: {type: Boolean},
+
+      /**
+       * Whether the secondary side can be shown based on the feature state and
+       * the width available to the dropdown for the ntp searchbox.
+       */
+      realboxCanShowSecondarySide: {
+        type: Boolean,
+        reflect: true,
+      },
+
+      /**
+       * Whether the searchbox secondary side was at any point available to
+       * be shown.
+       */
+      realboxHadSecondarySide: {
+        type: Boolean,
+        reflect: true,
+        notify: true,
+      },
+
+      realboxIsTall_: {
+        type: Boolean,
+        reflect: true,
+      },
+
+      realboxShown_: {type: Boolean},
+
+      /* Searchbox width behavior. */
+      searchboxWidthBehavior_: {
         type: String,
-        computed: `computeBackgroundImageAttribution1_(theme_)`,
+        reflect: true,
       },
 
-      backgroundImageAttribution2_: {
-        type: String,
-        computed: `computeBackgroundImageAttribution2_(theme_)`,
-      },
-
-      backgroundImageAttributionUrl_: {
-        type: String,
-        computed: `computeBackgroundImageAttributionUrl_(theme_)`,
-      },
-
-      backgroundColor_: {
-        computed: 'computeBackgroundColor_(showBackgroundImage_, theme_)',
-        type: Object,
-      },
-
-      // Used in ntp-realbox component via host-context.
-      colorSourceIsBaseline: {
-        type: Boolean,
-        computed: 'computeColorSourceIsBaseline(theme_)',
-      },
-
-      customizeChromeEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('customizeChromeEnabled'),
-      },
-
-      logoColor_: {
-        type: String,
-        computed: 'computeLogoColor_(theme_)',
-      },
-
-      singleColoredLogo_: {
-        computed: 'computeSingleColoredLogo_(theme_)',
-        type: Boolean,
-      },
-
-      realboxLensSearchEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('realboxLensSearch'),
-      },
-
-      realboxShown_: {
-        type: Boolean,
-        computed: 'computeRealboxShown_(theme_, showLensUploadDialog_)',
-      },
-
-      logoEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('logoEnabled'),
-      },
-
-      oneGoogleBarEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('oneGoogleBarEnabled'),
-      },
-
-      shortcutsEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('shortcutsEnabled'),
-      },
-
-      singleRowShortcutsEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('singleRowShortcutsEnabled'),
-      },
-
-      modulesFreShown: {
-        type: Boolean,
-        reflectToAttribute: true,
-      },
-
-      middleSlotPromoEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('middleSlotPromoEnabled'),
-      },
-
-      modulesEnabled_: {
-        type: Boolean,
-        value: () => loadTimeData.getBoolean('modulesEnabled'),
-      },
+      logoEnabled_: {type: Boolean},
+      oneGoogleBarEnabled_: {type: Boolean},
+      shortcutsEnabled_: {type: Boolean},
+      singleRowShortcutsEnabled_: {type: Boolean},
+      middleSlotPromoEnabled_: {type: Boolean},
+      modulesEnabled_: {type: Boolean},
 
       modulesRedesignedEnabled_: {
         type: Boolean,
-        value: () => loadTimeData.getBoolean('modulesRedesignedEnabled'),
-        reflectToAttribute: true,
-      },
-
-      mostVisitedReflowOnOverflowEnabled_: {
-        type: Boolean,
-        value: () =>
-            loadTimeData.getBoolean('mostVisitedReflowOnOverflowEnabled'),
-        reflectToAttribute: true,
+        reflect: true,
       },
 
       wideModulesEnabled_: {
         type: Boolean,
-        value: () => loadTimeData.getBoolean('wideModulesEnabled'),
-        reflectToAttribute: true,
+        reflect: true,
       },
 
-      middleSlotPromoLoaded_: {
-        type: Boolean,
-        value: false,
-      },
-
-      modulesLoaded_: {
-        type: Boolean,
-        value: false,
-      },
+      middleSlotPromoLoaded_: {type: Boolean},
+      modulesLoaded_: {type: Boolean},
 
       modulesShownToUser: {
         type: Boolean,
-        reflectToAttribute: true,
+        reflect: true,
       },
 
       /**
@@ -298,71 +238,87 @@ export class AppElement extends AppElementBase {
        * are loaded. If modules are disabled, the promo is shown as soon as it
        * is loaded.
        */
-      promoAndModulesLoaded_: {
-        type: Boolean,
-        computed: `computePromoAndModulesLoaded_(middleSlotPromoLoaded_,
-            modulesLoaded_)`,
-        observer: 'onPromoAndModulesLoadedChange_',
-      },
+      promoAndModulesLoaded_: {type: Boolean},
 
-      showLensUploadDialog_: Boolean,
+      showLensUploadDialog_: {type: Boolean},
 
       /**
        * If true, renders additional elements that were not deemed crucial to
        * to show up immediately on load.
        */
-      lazyRender_: Boolean,
+      lazyRender_: {type: Boolean},
 
-      scrolledToTop_: {
+      scrolledToTop_: {type: Boolean},
+
+      wallpaperSearchButtonAnimationEnabled_: {
         type: Boolean,
-        value: document.documentElement.scrollTop <= 0,
+        reflect: true,
+      },
+
+      showWallpaperSearchButton_: {
+        type: Boolean,
+        reflect: true,
       },
     };
   }
 
-  static get observers() {
-    return [
-      'updateOneGoogleBarAppearance_(oneGoogleBarLoaded_, theme_)',
-    ];
-  }
-
-  private oneGoogleBarIframePath_: string;
-  private oneGoogleBarLoaded_: boolean;
-  private theme_: Theme;
-  private showCustomize_: boolean;
-  private showCustomizeDialog_: boolean;
+  protected oneGoogleBarIframeOrigin_: string = OGB_IFRAME_ORIGIN;
+  protected oneGoogleBarIframePath_: string;
+  protected oneGoogleBarLoaded_: boolean;
+  protected theme_?: Theme;
+  protected showCustomize_: boolean;
+  protected showCustomizeChromeText_: boolean;
+  protected showWallpaperSearch_: boolean = false;
   private selectedCustomizeDialogPage_: string|null;
-  private showVoiceSearchOverlay_: boolean;
-  private showBackgroundImage_: boolean;
-  private backgroundImageAttribution1_: string;
-  private backgroundImageAttribution2_: string;
-  private backgroundImageAttributionUrl_: string;
-  private backgroundColor_: SkColor;
-  private customizeChromeEnabled_: boolean;
-  private logoColor_: string;
-  private singleColoredLogo_: boolean;
-  private realboxLensSearchEnabled_: boolean;
-  private realboxShown_: boolean;
-  private showLensUploadDialog_: boolean = false;
-  private logoEnabled_: boolean;
-  private oneGoogleBarEnabled_: boolean;
-  private shortcutsEnabled_: boolean;
+  protected showVoiceSearchOverlay_: boolean = false;
+  protected showBackgroundImage_: boolean;
+  protected backgroundImageAttribution1_: string;
+  protected backgroundImageAttribution2_: string;
+  protected backgroundImageAttributionUrl_: string;
+  protected backgroundColor_: SkColor|null;
+  protected colorSourceIsBaseline: boolean;
+  protected logoColor_: SkColor|null = null;
+  protected singleColoredLogo_: boolean;
+  realboxCanShowSecondarySide: boolean;
+  realboxHadSecondarySide: boolean;
+  protected realboxIsTall_ = loadTimeData.getBoolean('realboxIsTall');
+  protected realboxShown_: boolean;
+  protected searchboxWidthBehavior_: string =
+      loadTimeData.getString('searchboxWidthBehavior');
+  protected showLensUploadDialog_: boolean = false;
+  protected logoEnabled_: boolean = loadTimeData.getBoolean('logoEnabled');
+  protected oneGoogleBarEnabled_: boolean =
+      loadTimeData.getBoolean('oneGoogleBarEnabled');
+  protected shortcutsEnabled_: boolean =
+      loadTimeData.getBoolean('shortcutsEnabled');
+  protected singleRowShortcutsEnabled_: boolean =
+      loadTimeData.getBoolean('singleRowShortcutsEnabled');
   private modulesFreShown: boolean;
-  private middleSlotPromoEnabled_: boolean;
-  private modulesEnabled_: boolean;
-  private modulesRedesignedEnabled_: boolean;
-  private middleSlotPromoLoaded_: boolean;
-  private modulesLoaded_: boolean;
-  private modulesShownToUser: boolean;
-  private promoAndModulesLoaded_: boolean;
-  private lazyRender_: boolean;
-  private scrolledToTop_: boolean;
+  protected middleSlotPromoEnabled_: boolean =
+      loadTimeData.getBoolean('middleSlotPromoEnabled');
+  protected modulesEnabled_: boolean =
+      loadTimeData.getBoolean('modulesEnabled');
+  protected modulesRedesignedEnabled_: boolean =
+      loadTimeData.getBoolean('modulesRedesignedEnabled');
+  protected wideModulesEnabled_ = loadTimeData.getBoolean('wideModulesEnabled');
+  private middleSlotPromoLoaded_: boolean = false;
+  private modulesLoaded_: boolean = false;
+  protected modulesShownToUser: boolean;
+  protected promoAndModulesLoaded_: boolean = false;
+  protected lazyRender_: boolean;
+  protected scrolledToTop_: boolean = document.documentElement.scrollTop <= 0;
+  private wallpaperSearchButtonAnimationEnabled_: boolean =
+      loadTimeData.getBoolean('wallpaperSearchButtonAnimationEnabled');
+  protected wallpaperSearchButtonEnabled_: boolean =
+      loadTimeData.getBoolean('wallpaperSearchButtonEnabled');
+  protected showWallpaperSearchButton_: boolean;
 
   private callbackRouter_: PageCallbackRouter;
   private pageHandler_: PageHandlerRemote;
   private backgroundManager_: BackgroundManager;
   private setThemeListenerId_: number|null = null;
   private setCustomizeChromeSidePanelVisibilityListener_: number|null = null;
+  private setWallpaperSearchButtonVisibilityListener_: number|null = null;
   private eventTracker_: EventTracker = new EventTracker();
   private shouldPrintPerformance_: boolean;
   private backgroundImageLoadStartEpoch_: number;
@@ -378,6 +334,22 @@ export class AppElement extends AppElementBase {
     this.shouldPrintPerformance_ =
         new URLSearchParams(location.search).has('print_perf');
 
+    this.oneGoogleBarIframePath_ = (() => {
+      const params = new URLSearchParams();
+      params.set(
+          'paramsencoded', btoa(window.location.search.replace(/^[?]/, '&')));
+      return `${OGB_IFRAME_ORIGIN}/one-google-bar?${params}`;
+    })();
+
+    this.showCustomize_ =
+        WindowProxy.getInstance().url.searchParams.has(CUSTOMIZE_URL_PARAM);
+
+    this.selectedCustomizeDialogPage_ =
+        WindowProxy.getInstance().url.searchParams.get(CUSTOMIZE_URL_PARAM);
+
+    this.realboxCanShowSecondarySide =
+        realboxCanShowSecondarySideMediaQueryList.matches;
+
     /**
      * Initialized with the start of the performance timeline in case the
      * background image load is not triggered by JS.
@@ -408,6 +380,8 @@ export class AppElement extends AppElementBase {
 
   override connectedCallback() {
     super.connectedCallback();
+    realboxCanShowSecondarySideMediaQueryList.addEventListener(
+        'change', this.onRealboxCanShowSecondarySideChanged_.bind(this));
     this.setThemeListenerId_ =
         this.callbackRouter_.setTheme.addListener((theme: Theme) => {
           if (!this.theme_) {
@@ -420,6 +394,9 @@ export class AppElement extends AppElementBase {
         this.callbackRouter_.setCustomizeChromeSidePanelVisibility.addListener(
             (visible: boolean) => {
               this.showCustomize_ = visible;
+              if (!visible) {
+                this.showWallpaperSearch_ = false;
+              }
             });
     this.showWebstoreToastListenerId_ =
         this.callbackRouter_.showWebstoreToast.addListener(() => {
@@ -431,6 +408,18 @@ export class AppElement extends AppElementBase {
             }
           }
         });
+    this.setWallpaperSearchButtonVisibilityListener_ =
+        this.callbackRouter_.setWallpaperSearchButtonVisibility.addListener(
+            (visible: boolean) => {
+              // We only show the button if wallpaper search is enabled when the
+              // NTP loads. This prevents the button from showing if Customize
+              // Chrome doesn't have the wallpaper search element yet.
+              if (!visible) {
+                this.wallpaperSearchButtonEnabled_ = visible;
+                this.showWallpaperSearchButton_ =
+                    this.computeShowWallpaperSearchButton_();
+              }
+            });
 
     // Open Customize Chrome if there are Customize Chrome URL params.
     if (this.showCustomize_) {
@@ -478,15 +467,18 @@ export class AppElement extends AppElementBase {
 
   override disconnectedCallback() {
     super.disconnectedCallback();
+    realboxCanShowSecondarySideMediaQueryList.removeEventListener(
+        'change', this.onRealboxCanShowSecondarySideChanged_.bind(this));
     this.callbackRouter_.removeListener(this.setThemeListenerId_!);
     this.callbackRouter_.removeListener(
         this.setCustomizeChromeSidePanelVisibilityListener_!);
     this.callbackRouter_.removeListener(this.showWebstoreToastListenerId_!);
+    this.callbackRouter_.removeListener(
+        this.setWallpaperSearchButtonVisibilityListener_!);
     this.eventTracker_.removeAll();
   }
 
-  override ready() {
-    super.ready();
+  override firstUpdated() {
     this.pageHandler_.onAppRendered(WindowProxy.getInstance().now());
     // Let the browser breathe and then render remaining elements.
     WindowProxy.getInstance().waitForLazyRender().then(() => {
@@ -497,6 +489,74 @@ export class AppElement extends AppElementBase {
     performance.measure('app-creation', 'app-creation-start');
   }
 
+  override willUpdate(changedProperties: PropertyValues<this>) {
+    super.willUpdate(changedProperties);
+
+    const changedPrivateProperties =
+        changedProperties as Map<PropertyKey, unknown>;
+
+    if (changedPrivateProperties.has('theme_')) {
+      this.showBackgroundImage_ = this.computeShowBackgroundImage_();
+      this.backgroundImageAttribution1_ =
+          this.computeBackgroundImageAttribution1_();
+      this.backgroundImageAttribution2_ =
+          this.computeBackgroundImageAttribution2_();
+      this.backgroundImageAttributionUrl_ =
+          this.computeBackgroundImageAttributionUrl_();
+      this.colorSourceIsBaseline = this.computeColorSourceIsBaseline();
+      this.logoColor_ = this.computeLogoColor_();
+      this.singleColoredLogo_ = this.computeSingleColoredLogo_();
+    }
+
+    // theme_, showBackgroundImage_
+    this.backgroundColor_ = this.computeBackgroundColor_();
+
+    // theme_, showLensUploadDialog_
+    this.realboxShown_ = this.computeRealboxShown_();
+
+    // middleSlotPromoLoaded_, modulesLoaded_
+    this.promoAndModulesLoaded_ = this.computePromoAndModulesLoaded_();
+
+    // wallpaperSearchButtonEnabled_, showBackgroundImage_, backgroundColor_
+    this.showWallpaperSearchButton_ = this.computeShowWallpaperSearchButton_();
+
+    // showWallpaperSearchButton_, showBackgroundImage_
+    this.showCustomizeChromeText_ = this.computeShowCustomizeChromeText_();
+  }
+
+  override updated(changedProperties: PropertyValues<this>) {
+    super.updated(changedProperties);
+
+    const changedPrivateProperties =
+        changedProperties as Map<PropertyKey, unknown>;
+
+    if (changedPrivateProperties.has('lazyRender_') && this.lazyRender_) {
+      this.onLazyRendered_();
+    }
+
+    if (changedPrivateProperties.has('theme_')) {
+      this.onThemeChange_();
+    }
+
+    if (changedPrivateProperties.has('logoColor_')) {
+      this.style.setProperty(
+          '--ntp-logo-color', this.rgbaOrInherit_(this.logoColor_));
+    }
+
+    if (changedPrivateProperties.has('showBackgroundImage_')) {
+      this.onShowBackgroundImageChange_();
+    }
+
+    if (changedPrivateProperties.has('promoAndModulesLoaded_')) {
+      this.onPromoAndModulesLoadedChange_();
+    }
+
+    if (changedPrivateProperties.has('oneGoogleBarLoaded_') ||
+        changedPrivateProperties.has('theme_')) {
+      this.updateOneGoogleBarAppearance_();
+    }
+  }
+
   // Called to update the OGB of relevant NTP state changes.
   private updateOneGoogleBarAppearance_() {
     if (this.oneGoogleBarLoaded_) {
@@ -510,8 +570,11 @@ export class AppElement extends AppElementBase {
     }
   }
 
-  private computeShowCustomizeDialog_(): boolean {
-    return !this.customizeChromeEnabled_ && this.showCustomize_;
+  private computeShowCustomizeChromeText_(): boolean {
+    if (this.showWallpaperSearchButton_) {
+      return false;
+    }
+    return !this.showBackgroundImage_;
   }
 
   private computeBackgroundImageAttribution1_(): string {
@@ -528,10 +591,9 @@ export class AppElement extends AppElementBase {
         '';
   }
 
-  private computeRealboxShown_(theme: Theme, showLensUploadDialog: boolean):
-      boolean {
+  private computeRealboxShown_(): boolean {
     // Do not show the realbox if the upload dialog is showing.
-    return theme && !showLensUploadDialog;
+    return !!this.theme_ && !this.showLensUploadDialog_;
   }
 
   private computePromoAndModulesLoaded_(): boolean {
@@ -540,6 +602,10 @@ export class AppElement extends AppElementBase {
         (!loadTimeData.getBoolean('modulesEnabled') || this.modulesLoaded_);
   }
 
+  private onRealboxCanShowSecondarySideChanged_(e: MediaQueryListEvent) {
+    this.realboxCanShowSecondarySide = e.matches;
+  }
+
   private async onLazyRendered_() {
     // Integration tests use this attribute to determine when lazy load has
     // completed.
@@ -547,44 +613,71 @@ export class AppElement extends AppElementBase {
     this.registerHelpBubble(
         CUSTOMIZE_CHROME_BUTTON_ELEMENT_ID, '#customizeButton', {fixed: true});
     this.pageHandler_.maybeShowFeaturePromo(IphFeature.kCustomizeChrome);
+    if (this.showWallpaperSearchButton_) {
+      this.pageHandler_.incrementWallpaperSearchButtonShownCount();
+    }
   }
 
-  private onOpenVoiceSearch_() {
+  protected onOpenVoiceSearch_() {
     this.showVoiceSearchOverlay_ = true;
     recordVoiceAction(VoiceAction.ACTIVATE_SEARCH_BOX);
   }
 
-  private onOpenLensSearch_() {
+  protected onOpenLensSearch_() {
     this.showLensUploadDialog_ = true;
   }
 
-  private onCloseLensSearch_() {
+  protected onCloseLensSearch_() {
     this.showLensUploadDialog_ = false;
   }
 
-  private onCustomizeClick_() {
-    // Let customize dialog or side panel decide what page or section to show.
+  protected onCustomizeClick_() {
+    // Let side panel decide what page or section to show.
     this.selectedCustomizeDialogPage_ = null;
-    if (this.customizeChromeEnabled_) {
-      this.setCustomizeChromeSidePanelVisible_(!this.showCustomize_);
-      if (!this.showCustomize_) {
-        this.pageHandler_.incrementCustomizeChromeButtonOpenCount();
-        recordCustomizeChromeOpen(
-            NtpCustomizeChromeEntryPoint.CUSTOMIZE_BUTTON);
-      }
-    } else {
-      this.showCustomize_ = true;
+    this.setCustomizeChromeSidePanelVisible_(!this.showCustomize_);
+    if (!this.showCustomize_) {
+      this.pageHandler_.incrementCustomizeChromeButtonOpenCount();
       recordCustomizeChromeOpen(NtpCustomizeChromeEntryPoint.CUSTOMIZE_BUTTON);
     }
   }
 
-  private onCustomizeDialogClose_() {
-    this.showCustomize_ = false;
-    // Let customize dialog decide what page to show on next open.
-    this.selectedCustomizeDialogPage_ = null;
+  protected computeShowWallpaperSearchButton_() {
+    if (!this.wallpaperSearchButtonEnabled_) {
+      return false;
+    }
+
+    switch (loadTimeData.getInteger('wallpaperSearchButtonHideCondition')) {
+      case NtpWallpaperSearchButtonHideCondition.NONE:
+        return true;
+      case NtpWallpaperSearchButtonHideCondition.BACKGROUND_IMAGE_SET:
+        return !this.showBackgroundImage_;
+      case NtpWallpaperSearchButtonHideCondition.THEME_SET:
+        return this.colorSourceIsBaseline && !this.showBackgroundImage_;
+    }
+    return false;
   }
 
-  private onVoiceSearchOverlayClose_() {
+  protected onWallpaperSearchClick_() {
+    // Close the side panel if Wallpaper Search is open.
+    if (this.showCustomize_ && this.showWallpaperSearch_) {
+      this.selectedCustomizeDialogPage_ = null;
+      this.setCustomizeChromeSidePanelVisible_(!this.showCustomize_);
+      return;
+    }
+
+    // Open Wallpaper Search if the side panel is closed. Otherwise, navigate
+    // the side panel to Wallpaper Search.
+    this.selectedCustomizeDialogPage_ = CustomizeDialogPage.WALLPAPER_SEARCH;
+    this.showWallpaperSearch_ = true;
+    this.setCustomizeChromeSidePanelVisible_(this.showWallpaperSearch_);
+    if (!this.showCustomize_) {
+      this.pageHandler_.incrementCustomizeChromeButtonOpenCount();
+      recordCustomizeChromeOpen(
+          NtpCustomizeChromeEntryPoint.WALLPAPER_SEARCH_BUTTON);
+    }
+  }
+
+  protected onVoiceSearchOverlayClose_() {
     this.showVoiceSearchOverlay_ = false;
   }
 
@@ -618,21 +711,39 @@ export class AppElement extends AppElementBase {
   private onThemeChange_() {
     if (this.theme_) {
       this.backgroundManager_.setBackgroundColor(this.theme_.backgroundColor);
+      this.style.setProperty(
+          '--color-new-tab-page-attribution-foreground',
+          this.rgbaOrInherit_(this.theme_.textColor));
+      this.style.setProperty(
+          '--color-new-tab-page-most-visited-foreground',
+          this.rgbaOrInherit_(this.theme_.textColor));
     }
     this.updateBackgroundImagePath_();
   }
 
 
   private onThemeLoaded_(theme: Theme) {
-    chrome.metricsPrivate.recordEnumerationValue(
-        'NewTabPage.BackgroundImageSource',
-        (theme.backgroundImage ? theme.backgroundImage.imageSource :
-                                 NtpBackgroundImageSource.kNoImage),
-        NtpBackgroundImageSource.MAX_VALUE);
-
     chrome.metricsPrivate.recordSparseValueWithPersistentHash(
         'NewTabPage.Collections.IdOnLoad',
         theme.backgroundImageCollectionId ?? '');
+
+    if (!theme.backgroundImage || !theme.backgroundImage.imageSource) {
+      chrome.metricsPrivate.recordEnumerationValue(
+          'NewTabPage.BackgroundImageSource', NtpBackgroundImageSource.kNoImage,
+          NtpBackgroundImageSource.MAX_VALUE + 1);
+      return;
+    } else {
+      chrome.metricsPrivate.recordEnumerationValue(
+          'NewTabPage.BackgroundImageSource', theme.backgroundImage.imageSource,
+          NtpBackgroundImageSource.MAX_VALUE + 1);
+    }
+
+    if (theme.backgroundImage.imageSource ===
+            NtpBackgroundImageSource.kWallpaperSearch ||
+        theme.backgroundImage.imageSource ===
+            NtpBackgroundImageSource.kWallpaperSearchInspiration) {
+      this.wallpaperSearchButtonAnimationEnabled_ = false;
+    }
   }
 
   private onPromoAndModulesLoadedChange_() {
@@ -660,24 +771,28 @@ export class AppElement extends AppElementBase {
   }
 
   private computeBackgroundColor_(): SkColor|null {
-    if (this.showBackgroundImage_) {
+    if (this.showBackgroundImage_ || !this.theme_) {
       return null;
     }
-    return this.theme_ && this.theme_.backgroundColor;
+
+    return this.theme_.backgroundColor;
   }
 
   private computeColorSourceIsBaseline(): boolean {
-    return this.theme_.isBaseline;
+    return !!this.theme_ && this.theme_.isBaseline;
   }
 
   private computeLogoColor_(): SkColor|null {
-    return this.theme_ &&
-        (this.theme_.logoColor ||
-         (this.theme_.isDark ? hexColorToSkColor('#ffffff') : null));
+    if (!this.theme_) {
+      return null;
+    }
+
+    return this.theme_.logoColor ||
+        (this.theme_.isDark ? hexColorToSkColor('#ffffff') : null);
   }
 
   private computeSingleColoredLogo_(): boolean {
-    return this.theme_ && (!!this.theme_.logoColor || this.theme_.isDark);
+    return !!this.theme_ && (!!this.theme_.logoColor || this.theme_.isDark);
   }
 
   /**
@@ -775,15 +890,15 @@ export class AppElement extends AppElementBase {
     }
   }
 
-  private onMiddleSlotPromoLoaded_() {
+  protected onMiddleSlotPromoLoaded_() {
     this.middleSlotPromoLoaded_ = true;
   }
 
-  private onModulesLoaded_() {
+  protected onModulesLoaded_() {
     this.modulesLoaded_ = true;
   }
 
-  private onCustomizeModule_() {
+  protected onCustomizeModule_() {
     this.showCustomize_ = true;
     this.selectedCustomizeDialogPage_ = CustomizeDialogPage.MODULES;
     recordCustomizeChromeOpen(NtpCustomizeChromeEntryPoint.MODULE);
@@ -791,9 +906,6 @@ export class AppElement extends AppElementBase {
   }
 
   private setCustomizeChromeSidePanelVisible_(visible: boolean) {
-    if (!this.customizeChromeEnabled_) {
-      return;
-    }
     let section: CustomizeChromeSection = CustomizeChromeSection.kUnspecified;
     switch (this.selectedCustomizeDialogPage_) {
       case CustomizeDialogPage.BACKGROUNDS:
@@ -806,6 +918,9 @@ export class AppElement extends AppElementBase {
       case CustomizeDialogPage.MODULES:
         section = CustomizeChromeSection.kModules;
         break;
+      case CustomizeDialogPage.WALLPAPER_SEARCH:
+        section = CustomizeChromeSection.kWallpaperSearch;
+        break;
     }
     this.pageHandler_.setCustomizeChromeSidePanelVisible(visible, section);
   }
@@ -848,7 +963,7 @@ export class AppElement extends AppElementBase {
     });
   }
 
-  private onWebstoreToastButtonClick_() {
+  protected onWebstoreToastButtonClick_() {
     window.location.assign(
         `https://chrome.google.com/webstore/category/collection/chrome_color_themes?hl=${
             window.navigator.language}`);
@@ -864,7 +979,7 @@ export class AppElement extends AppElementBase {
         case $$(this, 'ntp-logo'):
           recordClick(NtpElement.LOGO);
           return;
-        case $$(this, 'ntp-realbox'):
+        case $$(this, 'cr-searchbox'):
           recordClick(NtpElement.REALBOX);
           return;
         case $$(this, 'cr-most-visited'):
@@ -879,13 +994,30 @@ export class AppElement extends AppElementBase {
         case $$(this, '#customizeButton'):
           recordClick(NtpElement.CUSTOMIZE_BUTTON);
           return;
-        case $$(this, 'ntp-customize-dialog'):
-          recordClick(NtpElement.CUSTOMIZE_DIALOG);
+        case $$(this, '#wallpaperSearchButton'):
+          recordClick(NtpElement.WALLPAPER_SEARCH_BUTTON);
           return;
       }
     }
     recordClick(NtpElement.OTHER);
   }
+
+  protected isThemeDark_(): boolean {
+    return !!this.theme_ && this.theme_.isDark;
+  }
+
+  protected showThemeAttribution_(): boolean {
+    return !!this.theme_?.backgroundImage?.attributionUrl;
+  }
+
+  protected onRealboxHadSecondarySideChanged_(
+      e: CustomEvent<{value: boolean}>) {
+    this.realboxHadSecondarySide = e.detail.value;
+  }
+
+  protected onModulesShownToUserChanged_(e: CustomEvent<{value: boolean}>) {
+    this.modulesShownToUser = e.detail.value;
+  }
 }
 
 declare global {
