@@ -5,7 +5,7 @@
 import 'chrome://resources/cr_elements/cr_link_row/cr_link_row.js';
 import 'chrome://resources/cr_elements/cr_button/cr_button.js';
 import 'chrome://resources/cr_elements/md_select.css.js';
-import 'chrome://resources/polymer/v3_0/iron-icon/iron-icon.js';
+import 'chrome://resources/cr_elements/cr_icon/cr_icon.js';
 import 'chrome://resources/polymer/v3_0/paper-spinner/paper-spinner-lite.js';
 import './site_favicon.js';
 import './dialogs/password_preview_item.js';
@@ -214,7 +214,7 @@ export class PasswordsImporterElement extends PasswordsImporterElementBase {
 
   private closeDialog_() {
     this.dialogState_ = DialogState.NO_DIALOG;
-    // TODO(crbug/1432962): Make sure that focus behaves correctly when the
+    // TODO(crbug.com/40264206): Make sure that focus behaves correctly when the
     // dialog is closed.
   }
 
@@ -482,7 +482,7 @@ export class PasswordsImporterElement extends PasswordsImporterElementBase {
 
   private getFailedEntryErrorMessage_(
       status: chrome.passwordsPrivate.ImportEntryStatus): string {
-    // TODO(crbug/1434221): Use constants for length limits.
+    // TODO(crbug.com/40264637): Use constants for length limits.
     switch (status) {
       case chrome.passwordsPrivate.ImportEntryStatus.MISSING_PASSWORD:
         return this.i18n('importPasswordsMissingPassword');
