@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,12 +14,12 @@
  * |children| is an array of the |Bookmark|s that are below this in a table of
  * contents tree
  */
-export type Bookmark = {
-  title: string,
-  children: Bookmark[],
-  page?: number,
-  x?: number,
-  y?: number,
-  zoom?: number,
-  uri?: string,
-};
+export interface Bookmark {
+  title: string;
+  children: Bookmark[];
+  page?: number;
+  x?: number;
+  y?: number;
+  zoom?: number;
+  uri?: string;
+}
