@@ -2,51 +2,77 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import './elements/viewer-error-dialog.js';
+import './elements/viewer_error_dialog.js';
 // <if expr="enable_ink">
-import './elements/viewer-ink-host.js';
+import './elements/viewer_ink_host.js';
 // </if>
-import './elements/viewer-password-dialog.js';
-import './elements/viewer-pdf-sidenav.js';
-import './elements/viewer-properties-dialog.js';
-import './elements/viewer-toolbar.js';
-import './elements/shared-vars.css.js';
-import './pdf_viewer_shared_style.css.js';
-import 'chrome://resources/cr_elements/cr_hidden_style.css.js';
-import 'chrome://resources/cr_elements/cr_shared_vars.css.js';
+import './elements/viewer_password_dialog.js';
+// <if expr="enable_pdf_ink2">
+import './elements/viewer_side_panel.js';
+// </if>
+import './elements/viewer_pdf_sidenav.js';
+import './elements/viewer_properties_dialog.js';
+import './elements/viewer_toolbar.js';
 
 import {assert, assertNotReached} from 'chrome://resources/js/assert.js';
 import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
+import {isMac} from 'chrome://resources/js/platform.js';
 import {listenOnce} from 'chrome://resources/js/util.js';
+import type {PropertyValues} from 'chrome://resources/lit/v3_0/lit.rollup.js';
 
+// <if expr="enable_ink or enable_pdf_ink2">
+import {BeforeUnloadProxyImpl} from './before_unload_proxy.js';
+// </if>
 import type {Bookmark} from './bookmark_type.js';
 import type {BrowserApi} from './browser_api.js';
 import type {Attachment, DocumentMetadata, ExtendedKeyEvent, Point} from './constants.js';
-import {FittingType, SaveRequestType} from './constants.js';
+import {FittingType, FormFieldFocusType, SaveRequestType} from './constants.js';
 import type {MessageData} from './controller.js';
 import {PluginController} from './controller.js';
+// <if expr="enable_pdf_ink2">
+import {PluginControllerEventType} from './controller.js';
+// </if>
 // <if expr="enable_ink">
 import type {ContentController} from './controller.js';
 // </if>
-import type {ChangePageAndXyDetail, ChangePageDetail, NavigateDetail} from './elements/viewer-bookmark.js';
-import {ChangePageOrigin} from './elements/viewer-bookmark.js';
-import type {ViewerErrorDialogElement} from './elements/viewer-error-dialog.js';
-import type {ViewerPasswordDialogElement} from './elements/viewer-password-dialog.js';
-import type {ViewerPdfSidenavElement} from './elements/viewer-pdf-sidenav.js';
-import type {ViewerToolbarElement} from './elements/viewer-toolbar.js';
+import type {ChangePageAndXyDetail, ChangePageDetail, NavigateDetail} from './elements/viewer_bookmark.js';
+import {ChangePageOrigin} from './elements/viewer_bookmark.js';
+import type {ViewerErrorDialogElement} from './elements/viewer_error_dialog.js';
+import type {ViewerPasswordDialogElement} from './elements/viewer_password_dialog.js';
+// <if expr="enable_ink">
+import type {ViewerPdfSidenavElement} from './elements/viewer_pdf_sidenav.js';
+//</if>
+// <if expr="enable_pdf_ink2">
+import type {Ink2ThumbnailData} from './elements/viewer_thumbnail_bar.js';
+//</if>
+import type {ViewerToolbarElement} from './elements/viewer_toolbar.js';
 // <if expr="enable_ink">
 import {InkController, InkControllerEventType} from './ink_controller.js';
 //</if>
 import {LocalStorageProxyImpl} from './local_storage_proxy.js';
-import {record, UserAction} from './metrics.js';
+import {record, recordEnumeration, UserAction} from './metrics.js';
 import {NavigatorDelegateImpl, PdfNavigator, WindowOpenDisposition} from './navigator.js';
 import {deserializeKeyEvent, LoadState} from './pdf_scripting_api.js';
-import {getTemplate} from './pdf_viewer.html.js';
+import {getCss} from './pdf_viewer.css.js';
+import {getHtml} from './pdf_viewer.html.js';
 import type {KeyEventData} from './pdf_viewer_base.js';
 import {PdfViewerBaseElement} from './pdf_viewer_base.js';
-import type {DestinationMessageData, DocumentDimensionsMessageData} from './pdf_viewer_utils.js';
+import {PdfViewerPrivateProxyImpl} from './pdf_viewer_private_proxy.js';
+import type {DocumentDimensionsMessageData} from './pdf_viewer_utils.js';
 import {hasCtrlModifier, hasCtrlModifierOnly, shouldIgnoreKeyEvents} from './pdf_viewer_utils.js';
 
+/**
+ * Keep in sync with the values for enum PDFPostMessageDataType in
+ * tools/metrics/histograms/metadata/pdf/enums.xml.
+ * These values are persisted to logs. Entries should not be renumbered, removed
+ * or reused.
+ */
+enum PostMessageDataType {
+  GET_SELECTED_TEXT = 0,
+  PRINT = 1,
+  SELECT_ALL = 2,
+}
+
 interface EmailMessageData {
   type: string;
   to: string;
@@ -73,9 +99,9 @@ interface ZoomBounds {
  */
 export function getFilenameFromURL(url: string): string {
   // Ignore the query and fragment.
-  const mainUrl = url.split(/#|\?/)[0];
+  const mainUrl = url.split(/#|\?/)[0] || '';
   const components = mainUrl.split(/\/|\\/);
-  const filename = components[components.length - 1];
+  const filename = components[components.length - 1] || '';
   try {
     return decodeURIComponent(filename);
   } catch (e) {
@@ -94,11 +120,10 @@ function eventToPromise(event: string, target: HTMLElement): Promise<void> {
 const LOCAL_STORAGE_SIDENAV_COLLAPSED_KEY: string = 'sidenavCollapsed';
 
 /**
- * The background color used for the regular viewer. Its decimal value in string
- * format should match `kPdfViewerBackgroundColor` in
- * components/pdf/browser/plugin_response_writer.cc.
+ * The background color used for the regular viewer.
  */
 const BACKGROUND_COLOR: number = 0xff525659;
+const CR23_BACKGROUND_COLOR: number = 0xff282828;
 
 export interface PdfViewerElement {
   $: {
@@ -114,176 +139,131 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     return 'pdf-viewer';
   }
 
-  static get template() {
-    return getTemplate();
+  static override get styles() {
+    return getCss();
+  }
+
+  override render() {
+    return getHtml.bind(this)();
   }
 
   static override get properties() {
     return {
-      annotationAvailable_: {
-        type: Boolean,
-        computed: 'computeAnnotationAvailable_(' +
-            'hadPassword_, clockwiseRotations_, canSerializeDocument_,' +
-            'twoUpViewEnabled_)',
-      },
+      // from PdfViewerBaseElement
+      pdfCr23Enabled: {type: Boolean},
+      showErrorDialog: {type: Boolean},
+      strings: {type: Object},
 
-      annotationMode_: {
-        type: Boolean,
-        value: false,
-      },
-
-      attachments_: {
-        type: Array,
-        value: () => [],
-      },
-
-      bookmarks_: {
-        type: Array,
-        value: () => [],
-      },
-
-      canSerializeDocument_: {
-        type: Boolean,
-        value: false,
-      },
-
-      clockwiseRotations_: {
-        type: Number,
-        value: 0,
-      },
+      annotationMode_: {type: Boolean},
+      attachments_: {type: Array},
+      bookmarks_: {type: Array},
+      canSerializeDocument_: {type: Boolean},
+      clockwiseRotations_: {type: Number},
 
       /** The number of pages in the PDF document. */
-      docLength_: Number,
+      docLength_: {type: Number},
+      documentHasFocus_: {type: Boolean},
 
-      documentHasFocus_: {
-        type: Boolean,
-        value: false,
-      },
+      documentMetadata_: {type: Object},
 
-      documentMetadata_: {
-        type: Object,
-        value: () => {},
-      },
+      fileName_: {type: String},
+      hadPassword_: {type: Boolean},
+      hasEdits_: {type: Boolean},
+      hasEnteredAnnotationMode_: {type: Boolean},
 
-      fileName_: String,
-
-      hadPassword_: {
-        type: Boolean,
-        value: false,
-      },
-
-      hasEdits_: {
-        type: Boolean,
-        value: false,
-      },
-
-      hasEnteredAnnotationMode_: {
-        type: Boolean,
-        value: false,
-      },
-
-      isFormFieldFocused_: {
-        type: Boolean,
-        value: false,
-      },
-
-      /** The current loading progress of the PDF document (0 - 100). */
-      loadProgress_: Number,
-
-      /** The number of the page being viewed (1-based). */
-      pageNo_: Number,
-
-      pdfAnnotationsEnabled_: {
-        type: Boolean,
-        value: false,
-      },
-
-      // <if expr="enable_screen_ai_service">
-      pdfOcrEnabled_: {
-        type: Boolean,
-        value: false,
-      },
+      // <if expr="enable_pdf_ink2">
+      hasInk2Edits_: {type: Boolean},
       // </if>
 
-      printingEnabled_: {
-        type: Boolean,
-        value: false,
-      },
+      formFieldFocus_: {type: String},
 
-      showPasswordDialog_: {
-        type: Boolean,
-        value: false,
-      },
+      /** The current loading progress of the PDF document (0 - 100). */
+      loadProgress_: {type: Number},
 
-      showPropertiesDialog_: {
-        type: Boolean,
-        value: false,
-      },
+      /** The number of the page being viewed (1-based). */
+      pageNo_: {type: Number},
+      pdfAnnotationsEnabled_: {type: Boolean},
 
-      sidenavCollapsed_: {
-        type: Boolean,
-        value: false,
-      },
+      // <if expr="enable_pdf_ink2">
+      pdfInk2Enabled_: {type: Boolean},
+      // </if>
 
-      title_: String,
-
-      twoUpViewEnabled_: {
-        type: Boolean,
-        value: false,
-      },
-
-      viewportZoom_: {
-        type: Number,
-        value: 1,
-      },
-
-      zoomBounds_: {
-        type: Object,
-        value: () => ({min: 0, max: 0}),
-      },
+      printingEnabled_: {type: Boolean},
+      showPasswordDialog_: {type: Boolean},
+      showPropertiesDialog_: {type: Boolean},
+      sidenavCollapsed_: {type: Boolean},
+      title_: {type: String},
+      twoUpViewEnabled_: {type: Boolean},
+      viewportZoom_: {type: Number},
+      zoomBounds_: {type: Object},
     };
   }
 
   beepCount: number = 0;
-  private annotationAvailable_: boolean;
-  private annotationMode_: boolean;
-  private attachments_: Attachment[];
-  private bookmarks_: Bookmark[];
-  private canSerializeDocument_: boolean;
-  private clockwiseRotations_: number;
-  private docLength_: number;
-  private documentHasFocus_: boolean;
-  private documentMetadata_: DocumentMetadata;
-  private embedded_: boolean;
-  private fileName_: string;
-  private hadPassword_: boolean;
-  private hasEdits_: boolean;
-  private hasEnteredAnnotationMode_: boolean;
-  private isFormFieldFocused_: boolean;
-  private loadProgress_: number;
+  protected annotationMode_: boolean = false;
+  protected attachments_: Attachment[] = [];
+  protected bookmarks_: Bookmark[] = [];
+  private canSerializeDocument_: boolean = false;
+  protected clockwiseRotations_: number = 0;
+  protected docLength_: number = 0;
+  protected documentHasFocus_: boolean = false;
+  protected documentMetadata_: DocumentMetadata = {
+    author: '',
+    canSerializeDocument: false,
+    creationDate: '',
+    creator: '',
+    fileSize: '',
+    keywords: '',
+    linearized: false,
+    modDate: '',
+    pageSize: '',
+    producer: '',
+    subject: '',
+    title: '',
+    version: '',
+  };
+  protected embedded_: boolean = false;
+  protected fileName_: string = '';
+  private hadPassword_: boolean = false;
+  protected hasEdits_: boolean = false;
+  protected hasEnteredAnnotationMode_: boolean = false;
+  // <if expr="enable_pdf_ink2">
+  protected hasInk2Edits_: boolean = false;
+  private hasSavedEdits_: boolean = false;
+  // </if>
+  protected formFieldFocus_: FormFieldFocusType = FormFieldFocusType.NONE;
+  protected loadProgress_: number = 0;
   private navigator_: PdfNavigator|null = null;
-  private pageNo_: number;
-  private pdfAnnotationsEnabled_: boolean;
-  // <if expr="enable_screen_ai_service">
-  private pdfOcrEnabled_: boolean;
+  protected pageNo_: number = 0;
+  protected pdfAnnotationsEnabled_: boolean = false;
+  // <if expr="enable_pdf_ink2">
+  protected pdfInk2Enabled_: boolean = false;
   // </if>
   private pluginController_: PluginController|null = null;
-  private printingEnabled_: boolean;
-  private showPasswordDialog_: boolean;
-  private showPropertiesDialog_: boolean;
-  private sidenavCollapsed_: boolean;
+  protected printingEnabled_: boolean = false;
+  // <if expr="enable_pdf_ink2">
+  private restoreAnnotationMode_: boolean = false;
+  // </if>
+  // <if expr="enable_ink or enable_pdf_ink2">
+  private showBeforeUnloadDialog_: boolean = false;
+  // </if>
+  protected showPasswordDialog_: boolean = false;
+  protected showPropertiesDialog_: boolean = false;
+  protected sidenavCollapsed_: boolean;
 
+  // <if expr="enable_ink">
   /**
    * The state to which to restore `sidenavCollapsed_` after exiting annotation
    * mode.
    */
   private sidenavRestoreState_: boolean = false;
+  // </if>
 
-  private title_: string;
-  private toolbarEnabled_: boolean = false;
-  private twoUpViewEnabled_: boolean;
-  private viewportZoom_: number;
-  private zoomBounds_: ZoomBounds;
+  protected title_: string = '';
+  protected toolbarEnabled_: boolean = false;
+  protected twoUpViewEnabled_: boolean = false;
+  protected viewportZoom_: number = 1;
+  protected zoomBounds_: ZoomBounds = {min: 0, max: 0};
 
   // <if expr="enable_ink">
   private inkController_: InkController|null = null;
@@ -299,8 +279,16 @@ export class PdfViewerElement extends PdfViewerBaseElement {
         10));
   }
 
+  override updated(changedProperties: PropertyValues<this>) {
+    super.updated(changedProperties);
+
+    if (changedProperties.has('showErrorDialog') && this.showErrorDialog) {
+      this.onErrorDialog_();
+    }
+  }
+
   getBackgroundColor(): number {
-    return BACKGROUND_COLOR;
+    return this.pdfCr23Enabled ? CR23_BACKGROUND_COLOR : BACKGROUND_COLOR;
   }
 
   setPluginSrc(plugin: HTMLEmbedElement) {
@@ -319,11 +307,7 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     this.tracker.add(
         this.inkController_.getEventTarget(),
         InkControllerEventType.HAS_UNSAVED_CHANGES, () => {
-          // TODO(crbug.com/1445746): Write an equivalent API call for
-          // chrome.pdfViewerPrivate.
-          if (!this.pdfOopifEnabled) {
-            chrome.mimeHandlerPrivate.setShowBeforeUnloadDialog(true);
-          }
+          this.setShowBeforeUnloadDialog_(true);
         });
     // </if>
 
@@ -352,6 +336,12 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     }
 
     this.embedded_ = this.browserApi!.getStreamInfo().embedded;
+
+    if (this.pdfOopifEnabled && !this.embedded_) {
+      // Give the full page PDF viewer focus so it can handle keyboard events
+      // immediately.
+      window.focus();
+    }
   }
 
   handleKeyEvent(e: KeyboardEvent) {
@@ -360,7 +350,8 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     }
 
     // Let the viewport handle directional key events.
-    if (this.viewport.handleDirectionalKeyEvent(e, this.isFormFieldFocused_)) {
+    if (this.viewport.handleDirectionalKeyEvent(
+            e, this.formFieldFocus_ !== FormFieldFocusType.NONE)) {
       return;
     }
 
@@ -411,15 +402,29 @@ export class PdfViewerElement extends PdfViewerBaseElement {
    */
   private handleToolbarKeyEvent_(e: KeyboardEvent) {
     // TODO(thestig): Should this use hasCtrlModifier() or stay as is?
-    if (e.key === '\\' && e.ctrlKey) {
-      this.$.toolbar.fitToggle();
+    if (isMac ? !e.metaKey || e.ctrlKey : !e.ctrlKey || e.metaKey) {
+      return;
     }
+
     // TODO: Add handling for additional relevant hotkeys for the new unified
     // toolbar.
+    switch (e.key) {
+      case '\\':
+        this.$.toolbar.fitToggle();
+        return;
+      // <if expr="enable_pdf_ink2">
+      case 'z':
+        this.$.toolbar.undo();
+        return;
+      case 'y':
+        this.$.toolbar.redo();
+        return;
+      // </if>
+    }
   }
 
   // <if expr="enable_ink">
-  private onResetView_() {
+  protected onResetView_() {
     if (this.twoUpViewEnabled_) {
       assert(this.currentController);
       this.currentController.setTwoUpView(false);
@@ -461,10 +466,26 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     return this.sidenavCollapsed_ ? Promise.resolve() :
                                     this.waitForSidenavTransition_();
   }
+  // </if>
 
+  // <if expr="enable_ink or enable_pdf_ink2">
   /** Handles the annotation mode being toggled on or off. */
-  private async onAnnotationModeToggled_(e: CustomEvent<boolean>) {
+  protected async onAnnotationModeToggled_(e: CustomEvent<boolean>) {
     const annotationMode = e.detail;
+    // <if expr="enable_pdf_ink2">
+    if (this.pdfInk2Enabled_) {
+      if (!this.restoreAnnotationMode_) {
+        record(
+            annotationMode ? UserAction.ENTER_ANNOTATION_MODE :
+                             UserAction.EXIT_ANNOTATION_MODE);
+      }
+      this.pluginController_!.setAnnotationMode(annotationMode);
+      this.annotationMode_ = annotationMode;
+      return;
+    }
+    // </if> enable_pdf_ink2
+
+    // <if expr="enable_ink">
     if (annotationMode) {
       // Enter annotation mode.
       assert(this.pluginController_!.isActive);
@@ -516,8 +537,11 @@ export class PdfViewerElement extends PdfViewerBaseElement {
       this.currentController = this.pluginController_!;
       await this.pluginController_!.load(result.fileName, result.dataToSave);
     }
+    // </if> enable_ink
   }
+  // </if> enable_ink or enable_pdf_ink2
 
+  // <if expr="enable_ink">
   /** Exits annotation mode if active. */
   private async exitAnnotationMode_(): Promise<void> {
     if (!this.$.toolbar.annotationMode) {
@@ -528,14 +552,23 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     await this.restoreSidenav_();
     await this.loaded;
   }
-  // </if>
+  // </if> enable_ink
 
-  private onDisplayAnnotationsChanged_(e: CustomEvent<boolean>) {
+  protected onDisplayAnnotationsChanged_(e: CustomEvent<boolean>) {
     assert(this.currentController);
     this.currentController.setDisplayAnnotations(e.detail);
   }
 
   private async enterPresentationMode_(): Promise<void> {
+    // <if expr="enable_pdf_ink2">
+    // Exit annotation mode if it was enabled.
+    if (this.pdfInk2Enabled_ && this.annotationMode_) {
+      this.restoreAnnotationMode_ = true;
+      this.$.toolbar.toggleAnnotation();
+      assert(!this.annotationMode_);
+    }
+    // </if>
+
     const scroller = this.$.scroller;
 
     this.viewport.saveZoomState();
@@ -569,9 +602,19 @@ export class PdfViewerElement extends PdfViewerBaseElement {
 
     // Set zoom back to original zoom before presentation mode.
     this.viewport.restoreZoomState();
+
+    // <if expr="enable_pdf_ink2">
+    // Enter annotation mode again if it was enabled before entering
+    // Presentation mode.
+    if (this.restoreAnnotationMode_) {
+      this.$.toolbar.toggleAnnotation();
+      assert(this.annotationMode_);
+      this.restoreAnnotationMode_ = false;
+    }
+    // </if>
   }
 
-  private async onPresentClick_() {
+  protected async onPresentClick_() {
     await this.enterPresentationMode_();
 
     // When fullscreen changes, it means that the user exited Presentation
@@ -582,12 +625,12 @@ export class PdfViewerElement extends PdfViewerBaseElement {
   }
 
 
-  private onPropertiesClick_() {
+  protected onPropertiesClick_() {
     assert(!this.showPropertiesDialog_);
     this.showPropertiesDialog_ = true;
   }
 
-  private onPropertiesDialogClose_() {
+  protected onPropertiesDialogClose_() {
     assert(this.showPropertiesDialog_);
     this.showPropertiesDialog_ = false;
   }
@@ -596,7 +639,7 @@ export class PdfViewerElement extends PdfViewerBaseElement {
    * Changes two up view mode for the controller. Controller will trigger
    * layout update later, which will update the viewport accordingly.
    */
-  private onTwoUpViewChanged_(e: CustomEvent<boolean>) {
+  protected onTwoUpViewChanged_(e: CustomEvent<boolean>) {
     const twoUpViewEnabled = e.detail;
     assert(this.currentController);
     this.currentController.setTwoUpView(twoUpViewEnabled);
@@ -626,6 +669,11 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     return this.bookmarks_;
   }
 
+  /** @return The title. Used for testing. */
+  get pdfTitle(): string {
+    return this.title_;
+  }
+
   override setLoadState(loadState: LoadState) {
     super.setLoadState(loadState);
     if (loadState === LoadState.FAILED) {
@@ -640,7 +688,7 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     super.updateProgress(progress);
   }
 
-  private onErrorDialog_() {
+  protected onErrorDialog_() {
     // The error screen can only reload from a normal tab.
     if (!chrome.tabs || this.browserApi!.getStreamInfo().tabId === -1) {
       return;
@@ -663,7 +711,7 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     }
   }
 
-  private onPasswordDialogClose_() {
+  protected onPasswordDialogClose_() {
     this.showPasswordDialog_ = false;
   }
 
@@ -672,7 +720,7 @@ export class PdfViewerElement extends PdfViewerBaseElement {
    * when an event is entered into the password dialog.
    * @param event A password-submitted event.
    */
-  private onPasswordSubmitted_(event: CustomEvent<{password: string}>) {
+  protected onPasswordSubmitted_(event: CustomEvent<{password: string}>) {
     this.pluginController_!.getPasswordComplete(event.detail.password);
   }
 
@@ -691,14 +739,15 @@ export class PdfViewerElement extends PdfViewerBaseElement {
 
     this.pdfAnnotationsEnabled_ =
         loadTimeData.getBoolean('pdfAnnotationsEnabled');
-    // <if expr="enable_screen_ai_service">
-    this.pdfOcrEnabled_ = loadTimeData.getBoolean('pdfOcrEnabled');
+    // <if expr="enable_pdf_ink2">
+    this.pdfInk2Enabled_ = loadTimeData.getBoolean('pdfInk2Enabled');
     // </if>
     this.printingEnabled_ = loadTimeData.getBoolean('printingEnabled');
     const presetZoomFactors = this.viewport.presetZoomFactors;
-    this.zoomBounds_.min = Math.round(presetZoomFactors[0] * 100);
+    assert(presetZoomFactors.length > 0);
+    this.zoomBounds_.min = Math.round(presetZoomFactors[0]! * 100);
     this.zoomBounds_.max =
-        Math.round(presetZoomFactors[presetZoomFactors.length - 1] * 100);
+        Math.round(presetZoomFactors[presetZoomFactors.length - 1]! * 100);
   }
 
   override handleScriptingMessage(message: MessageEvent<any>) {
@@ -710,20 +759,28 @@ export class PdfViewerElement extends PdfViewerBaseElement {
       return true;
     }
 
+    let messageType;
     switch (message.data.type.toString()) {
       case 'getSelectedText':
+        messageType = PostMessageDataType.GET_SELECTED_TEXT;
         this.pluginController_!.getSelectedText().then(
             this.handleSelectedTextReply.bind(this));
         break;
       case 'print':
+        messageType = PostMessageDataType.PRINT;
         this.pluginController_!.print();
         break;
       case 'selectAll':
+        messageType = PostMessageDataType.SELECT_ALL;
         this.pluginController_!.selectAll();
         break;
       default:
         return false;
     }
+
+    recordEnumeration(
+        'PDF.PostMessageDataType', messageType,
+        Object.keys(PostMessageDataType).length);
     return true;
   }
 
@@ -746,6 +803,10 @@ export class PdfViewerElement extends PdfViewerBaseElement {
         this.setDocumentDimensions(
             data as unknown as DocumentDimensionsMessageData);
         return;
+      case 'documentFocusChanged':
+        const hasFocusData = data as unknown as {hasFocus: boolean};
+        this.documentHasFocus_ = hasFocusData.hasFocus;
+        return;
       case 'email':
         const emailData = data as unknown as EmailMessageData;
         const href = 'mailto:' + emailData.to + '?cc=' + emailData.cc +
@@ -753,6 +814,31 @@ export class PdfViewerElement extends PdfViewerBaseElement {
             '&body=' + emailData.body;
         this.handleNavigate_(href, WindowOpenDisposition.CURRENT_TAB);
         return;
+      case 'executedEditCommand':
+        const editCommandData = data as unknown as {editCommand: string};
+        const editCommand = editCommandData.editCommand;
+        switch (editCommand) {
+          case 'Cut':
+            record(UserAction.CUT);
+            return;
+          case 'Copy':
+            record(UserAction.COPY);
+            return;
+          case 'Paste':
+            record(UserAction.PASTE);
+            return;
+        }
+        assertNotReached(
+            'Unknown executedEditCommand data received: ' + editCommand);
+      // <if expr="enable_pdf_ink2">
+      case 'finishInkStroke':
+        this.handleFinishInkStroke_();
+        return;
+      // </if>
+      case 'formFocusChange':
+        const focusedData = data as unknown as {focused: FormFieldFocusType};
+        this.formFieldFocus_ = focusedData.focused;
+        return;
       case 'getPassword':
         this.handlePasswordRequest_();
         return;
@@ -760,45 +846,14 @@ export class PdfViewerElement extends PdfViewerBaseElement {
         const progressData = data as unknown as {progress: number};
         this.updateProgress(progressData.progress);
         return;
-      case 'navigate':
-        const navigateData = data as unknown as NavigateMessageData;
-        this.handleNavigate_(navigateData.url, navigateData.disposition);
-        return;
-      case 'navigateToDestination':
-        const destinationData = data as unknown as DestinationMessageData;
-        this.viewport.handleNavigateToDestination(
-            destinationData.page, destinationData.x, destinationData.y,
-            destinationData.zoom);
-        return;
       case 'metadata':
         const metadataData =
             data as unknown as {metadataData: DocumentMetadata};
         this.setDocumentMetadata_(metadataData.metadataData);
         return;
-      case 'setIsEditing':
-        // Editing mode can only be entered once, and cannot be exited.
-        this.hasEdits_ = true;
-        return;
-      case 'setIsSelecting':
-        const selectingData = data as unknown as {isSelecting: boolean};
-        this.viewportScroller!.setEnableScrolling(selectingData.isSelecting);
-        return;
-      case 'setSmoothScrolling':
-        this.viewport.setSmoothScrolling(
-            (data as unknown as {smoothScrolling: boolean}).smoothScrolling);
-        return;
-      case 'formFocusChange':
-        const focusedData = data as unknown as {focused: boolean};
-        this.isFormFieldFocused_ = focusedData.focused;
-        return;
-      case 'touchSelectionOccurred':
-        this.sendScriptingMessage({
-          type: 'touchSelectionOccurred',
-        });
-        return;
-      case 'documentFocusChanged':
-        const hasFocusData = data as unknown as {hasFocus: boolean};
-        this.documentHasFocus_ = hasFocusData.hasFocus;
+      case 'navigate':
+        const navigateData = data as unknown as NavigateMessageData;
+        this.handleNavigate_(navigateData.url, navigateData.disposition);
         return;
       case 'sendKeyEvent':
         const keyEventData = data as unknown as KeyEventData;
@@ -807,6 +862,27 @@ export class PdfViewerElement extends PdfViewerBaseElement {
         keyEvent.fromPlugin = true;
         this.handleKeyEvent(keyEvent);
         return;
+      case 'setIsEditing':
+        // Editing mode can only be entered once, and cannot be exited.
+        this.hasEdits_ = true;
+        return;
+      case 'startedFindInPage':
+        record(UserAction.FIND_IN_PAGE);
+        return;
+      case 'touchSelectionOccurred':
+        this.sendScriptingMessage({
+          type: 'touchSelectionOccurred',
+        });
+        return;
+        // <if expr="enable_pdf_ink2">
+      case 'updateInk2Thumbnail':
+        const thumbnailData = data as unknown as Ink2ThumbnailData;
+        this.pluginController_!.getEventTarget().dispatchEvent(
+            new CustomEvent<Ink2ThumbnailData>(
+                PluginControllerEventType.UPDATE_INK_THUMBNAIL,
+                {detail: thumbnailData}));
+        return;
+        // </if>
     }
     assertNotReached('Unknown message type received: ' + data.type);
   }
@@ -860,6 +936,16 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     this.navigator_!.navigate(url, disposition);
   }
 
+  // <if expr="enable_pdf_ink2">
+  /** Handles a new ink stroke in annotation mode. */
+  private handleFinishInkStroke_() {
+    this.hasInk2Edits_ = true;
+    this.pluginController_!.getEventTarget().dispatchEvent(
+        new CustomEvent(PluginControllerEventType.FINISH_INK_STROKE));
+    this.setShowBeforeUnloadDialog_(true);
+  }
+  // </if>
+
   /** Sets the document attachment data. */
   private setAttachments_(attachments: Attachment[]) {
     this.attachments_ = attachments;
@@ -874,7 +960,18 @@ export class PdfViewerElement extends PdfViewerBaseElement {
   private setDocumentMetadata_(metadata: DocumentMetadata) {
     this.documentMetadata_ = metadata;
     this.title_ = this.documentMetadata_.title || this.fileName_;
-    document.title = this.title_;
+
+    // Tab title is updated only when document.title is called in a
+    // top-level document (`main_frame` of `WebContents`). For OOPIF PDF viewer,
+    // the current document is the child of a top-level document, hence using a
+    // private API to set the tab title.
+    // NOTE: Title should only be set for full-page PDFs.
+    if (this.pdfOopifEnabled && !this.embedded_) {
+      PdfViewerPrivateProxyImpl.getInstance().setPdfDocumentTitle(this.title_);
+    } else {
+      document.title = this.title_;
+    }
+
     this.canSerializeDocument_ = this.documentMetadata_.canSerializeDocument;
   }
 
@@ -885,8 +982,9 @@ export class PdfViewerElement extends PdfViewerBaseElement {
    * downloading.
    * @param e The event which contains the index of attachment to be downloaded.
    */
-  private async onSaveAttachment_(e: CustomEvent<number>) {
+  protected async onSaveAttachment_(e: CustomEvent<number>) {
     const index = e.detail;
+    assert(this.attachments_[index] !== undefined);
     const size = this.attachments_[index].size;
     assert(size !== -1);
 
@@ -927,13 +1025,11 @@ export class PdfViewerElement extends PdfViewerBaseElement {
           }
           entry!.createWriter((writer: FileWriter) => {
             writer.write(blob);
+            // <if expr="enable_ink">
             // Unblock closing the window now that the user has saved
             // successfully.
-            // TODO(crbug.com/1445746): Write an equivalent API call for
-            // chrome.pdfViewerPrivate.
-            if (!this.pdfOopifEnabled) {
-              chrome.mimeHandlerPrivate.setShowBeforeUnloadDialog(false);
-            }
+            this.setShowBeforeUnloadDialog_(false);
+            // </if>
           });
         });
   }
@@ -960,11 +1056,11 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     this.save_(saveMode);
   }
 
-  private onToolbarSave_(e: CustomEvent<SaveRequestType>) {
+  protected onToolbarSave_(e: CustomEvent<SaveRequestType>) {
     this.save_(e.detail);
   }
 
-  private onChangePage_(e: CustomEvent<ChangePageDetail>) {
+  protected onChangePage_(e: CustomEvent<ChangePageDetail>) {
     this.viewport.goToPage(e.detail.page);
     if (e.detail.origin === ChangePageOrigin.BOOKMARK) {
       record(UserAction.FOLLOW_BOOKMARK);
@@ -975,19 +1071,19 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     }
   }
 
-  private onChangePageAndXy_(e: CustomEvent<ChangePageAndXyDetail>) {
+  protected onChangePageAndXy_(e: CustomEvent<ChangePageAndXyDetail>) {
     const point = this.viewport.convertPageToScreen(e.detail.page, e.detail);
     this.goToPageAndXy_(e.detail.origin, e.detail.page, point);
   }
 
-  private onNavigate_(e: CustomEvent<NavigateDetail>) {
+  protected onNavigate_(e: CustomEvent<NavigateDetail>) {
     const disposition = e.detail.newtab ?
         WindowOpenDisposition.NEW_BACKGROUND_TAB :
         WindowOpenDisposition.CURRENT_TAB;
     this.navigator_!.navigate(e.detail.uri, disposition);
   }
 
-  private onSidenavToggleClick_() {
+  protected onSidenavToggleClick_() {
     this.sidenavCollapsed_ = !this.sidenavCollapsed_;
 
     // Workaround for crbug.com/1119944, so that the PDF plugin resizes only
@@ -1005,6 +1101,17 @@ export class PdfViewerElement extends PdfViewerBaseElement {
         (this.sidenavCollapsed_ ? 1 : 0).toString());
   }
 
+  // <if expr="enable_pdf_ink2">
+  protected onStrokesUpdated_(e: CustomEvent<number>) {
+    this.hasInk2Edits_ = e.detail > 0;
+
+    // If the user already saved, always show the beforeunload dialog if the
+    // strokes have updated. If the user hasn't saved, only show the
+    // beforeunload dialog if there's edits.
+    this.setShowBeforeUnloadDialog_(this.hasSavedEdits_ || this.hasInk2Edits_);
+  }
+  // </if>
+
   /**
    * Saves the current PDF document to disk.
    */
@@ -1019,20 +1126,29 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     // Always send requests of type ORIGINAL to the plugin controller, not the
     // ink controller. The ink controller always saves the edited document.
     // TODO(dstockwell): Report an error to user if this fails.
-    let result: {fileName: string, dataToSave: ArrayBuffer}|null = null;
     assert(this.currentController);
-    if (requestType !== SaveRequestType.ORIGINAL || !this.annotationMode_) {
-      result = await this.currentController.save(requestType);
-    } else {
-      // <if expr="enable_ink">
-      // Request type original in annotation mode --> need to exit annotation
-      // mode before saving. See https://crbug.com/919364.
+
+    // <if expr="enable_ink">
+    // For Ink, request type original in annotation mode --> need to exit
+    // annotation mode before saving. See https://crbug.com/919364.
+    let shouldExitAnnotationMode =
+        this.annotationMode_ && requestType === SaveRequestType.ORIGINAL;
+
+    // Ink2 overrides Ink, and Ink2 does not need to exit annotation mode.
+    // Only exit annotation mode if Ink2 is disabled.
+    // <if expr="enable_pdf_ink2">
+    shouldExitAnnotationMode =
+        shouldExitAnnotationMode && !this.pdfInk2Enabled_;
+    // </if> enable_pdf_ink2
+
+    if (shouldExitAnnotationMode) {
       await this.exitAnnotationMode_();
       assert(!this.annotationMode_);
-      result = await this.currentController.save(SaveRequestType.ORIGINAL);
-      // </if>
     }
-    if (result == null) {
+    // </if> enable_ink
+
+    const result = await this.currentController.save(requestType);
+    if (result === null) {
       // The content controller handled the save internally.
       return;
     }
@@ -1062,16 +1178,25 @@ export class PdfViewerElement extends PdfViewerBaseElement {
           }
           entry!.createWriter((writer: FileWriter) => {
             writer.write(blob);
+            // <if expr="enable_ink or enable_pdf_ink2">
             // Unblock closing the window now that the user has saved
             // successfully.
-            // TODO(crbug.com/1445746): Write an equivalent API call for
-            // chrome.pdfViewerPrivate.
-            if (!this.pdfOopifEnabled) {
-              chrome.mimeHandlerPrivate.setShowBeforeUnloadDialog(false);
-            }
+            this.setShowBeforeUnloadDialog_(false);
+            // </if>
+            // <if expr="enable_pdf_ink2">
+            this.hasSavedEdits_ =
+                this.hasSavedEdits_ || requestType === SaveRequestType.EDITED;
+            // </if>
           });
         });
 
+    // <if expr="enable_pdf_ink2">
+    // Ink2 doesn't need to exit annotation mode after save.
+    if (this.pdfInk2Enabled_) {
+      return;
+    }
+    // </if>
+
     // <if expr="enable_ink">
     // Saving in Annotation mode is destructive: crbug.com/919364
     this.exitAnnotationMode_();
@@ -1098,7 +1223,7 @@ export class PdfViewerElement extends PdfViewerBaseElement {
     }
   }
 
-  private async onPrint_() {
+  protected async onPrint_() {
     record(UserAction.PRINT);
     // <if expr="enable_ink">
     await this.exitAnnotationMode_();
@@ -1112,15 +1237,56 @@ export class PdfViewerElement extends PdfViewerBaseElement {
    * conditions.
    * @return Whether annotations are available.
    */
-  private computeAnnotationAvailable_(): boolean {
+  protected annotationAvailable_(): boolean {
     return this.canSerializeDocument_ && !this.hadPassword_;
   }
 
   /** @return Whether the PDF contents are rotated. */
-  private isRotated_(): boolean {
+  protected isRotated_(): boolean {
     return this.clockwiseRotations_ !== 0;
   }
 
+  // <if expr="enable_pdf_ink2">
+  /**
+   * @returns Whether the PDF viewer has Ink2 enabled and is in annotation mode.
+   */
+  protected inInk2AnnotationMode_() {
+    return this.pdfInk2Enabled_ && this.annotationMode_;
+  }
+  // </if>
+
+  // <if expr="enable_ink or enable_pdf_ink2">
+  /**
+   * Handles the `BeforeUnloadEvent` event.
+   * @param event The `BeforeUnloadEvent` object representing the event.
+   */
+  override onBeforeUnload(event: BeforeUnloadEvent) {
+    super.onBeforeUnload(event);
+    // When a user tries to leave PDF with unsaved changes, show the 'Leave
+    // site' dialog. OOPIF PDF only, since MimeHandler handles the beforeunload
+    // event instead.
+    if (this.pdfOopifEnabled && this.showBeforeUnloadDialog_) {
+      BeforeUnloadProxyImpl.getInstance().preventDefault(event);
+    }
+  }
+
+  /**
+   * Sets whether to show the beforeunload dialog when navigating away from pdf.
+   * @param showDialog A boolean indicating whether to show the beforeunload
+   * dialog.
+   */
+  private setShowBeforeUnloadDialog_(showDialog: boolean) {
+    if (this.showBeforeUnloadDialog_ === showDialog) {
+      return;
+    }
+
+    this.showBeforeUnloadDialog_ = showDialog;
+    if (!this.pdfOopifEnabled) {
+      chrome.mimeHandlerPrivate.setShowBeforeUnloadDialog(showDialog);
+    }
+  }
+  // </if>
+
   // <if expr="enable_ink">
   getCurrentControllerForTesting(): ContentController|null {
     return this.currentController;
