@@ -6,18 +6,25 @@ import {assert} from 'chrome://resources/js/assert.js';
 import {EventTracker} from 'chrome://resources/js/event_tracker.js';
 import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
 
-import {CapabilitiesResponse, NativeLayer, NativeLayerImpl} from '../native_layer.js';
+import type {CapabilitiesResponse, NativeLayer} from '../native_layer.js';
+import {NativeLayerImpl} from '../native_layer.js';
 // <if expr="is_chromeos">
-import {NativeLayerCros, NativeLayerCrosImpl, PrinterSetupResponse} from '../native_layer_cros.js';
+import type {NativeLayerCros, PrinterSetupResponse} from '../native_layer_cros.js';
+import {NativeLayerCrosImpl} from '../native_layer_cros.js';
 
 // </if>
-import {Cdd, MediaSizeOption} from './cdd.js';
-import {createDestinationKey, createRecentDestinationKey, Destination, DestinationOrigin, GooglePromotedDestinationId, isPdfPrinter, PDF_DESTINATION_KEY, PrinterType, RecentDestination} from './destination.js';
+import type {Cdd, MediaSizeOption} from './cdd.js';
+import type {RecentDestination} from './destination.js';
+import {createDestinationKey, createRecentDestinationKey, Destination, DestinationOrigin, GooglePromotedDestinationId, isPdfPrinter, PDF_DESTINATION_KEY, PrinterType} from './destination.js';
+
 // <if expr="is_chromeos">
 import {DestinationProvisionalType} from './destination.js';
 // </if>
+
 import {DestinationMatch} from './destination_match.js';
-import {ExtensionDestinationInfo, LocalDestinationInfo, parseDestination} from './local_parsers.js';
+import type {ExtensionDestinationInfo, LocalDestinationInfo} from './local_parsers.js';
+import {parseDestination} from './local_parsers.js';
+
 // <if expr="is_chromeos">
 import {parseExtensionDestination} from './local_parsers.js';
 import {getStatusReasonFromPrinterStatus, PrinterStatusReason} from './printer_status_cros.js';
@@ -25,7 +32,6 @@ import {getStatusReasonFromPrinterStatus, PrinterStatusReason} from './printer_s
 
 /**
  * Printer search statuses used by the destination store.
- * @enum {string}
  */
 enum DestinationStorePrinterSearchStatus {
   START = 'start',
@@ -258,12 +264,10 @@ export class DestinationStore extends EventTarget {
             this.onPrintersAdded_(type, printers));
 
     // <if expr="is_chromeos">
-    if (loadTimeData.getBoolean('isLocalPrinterObservingEnabled')) {
-      addListenerCallback(
-          'local-printers-updated',
-          (printers: LocalDestinationInfo[]) =>
-              this.onLocalPrintersUpdated_(printers));
-    }
+    addListenerCallback(
+        'local-printers-updated',
+        (printers: LocalDestinationInfo[]) =>
+            this.onLocalPrintersUpdated_(printers));
     // </if>
   }
 
@@ -980,10 +984,6 @@ export class DestinationStore extends EventTarget {
 
   // <if expr="is_chromeos">
   private observeLocalPrinters_() {
-    if (!loadTimeData.getBoolean('isLocalPrinterObservingEnabled')) {
-      return;
-    }
-
     this.nativeLayerCros_.observeLocalPrinters().then(
         (printers: LocalDestinationInfo[]) =>
             this.onLocalPrintersUpdated_(printers));
