@@ -2,10 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+import 'chrome://resources/cr_elements/cr_collapse/cr_collapse.js';
 import 'chrome://resources/cr_elements/cr_hidden_style.css.js';
 import 'chrome://resources/cr_elements/cr_shared_style.css.js';
 import 'chrome://resources/cr_elements/cr_shared_vars.css.js';
-import 'chrome://resources/polymer/v3_0/iron-collapse/iron-collapse.js';
 import './advanced_options_settings.js';
 import './button_strip.js';
 import './color_settings.js';
@@ -37,11 +37,13 @@ import {WebUiListenerMixin} from 'chrome://resources/cr_elements/web_ui_listener
 import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {DarkModeMixin} from '../dark_mode_mixin.js';
-import {Destination} from '../data/destination.js';
-import {Error, State} from '../data/state.js';
+import type {Destination} from '../data/destination.js';
+import type {Settings} from '../data/model.js';
+import type {Error} from '../data/state.js';
+import {State} from '../data/state.js';
 import {MetricsContext, PrintSettingsUiBucket} from '../metrics.js';
 
-import {DestinationState, PrintPreviewDestinationSettingsElement} from './destination_settings.js';
+import type {DestinationState, PrintPreviewDestinationSettingsElement} from './destination_settings.js';
 import {SettingsMixin} from './settings_mixin.js';
 import {getTemplate} from './sidebar.html.js';
 
@@ -210,7 +212,7 @@ export class PrintPreviewSidebarElement extends PrintPreviewSidebarElementBase {
   private computeShouldShowMoreSettings_(): boolean {
     // Destination settings is always available. See if the total number of
     // available sections exceeds the maximum number to show.
-    return [
+    const keys: Array<keyof Settings> = [
       'pages',
       'copies',
       'layout',
@@ -224,7 +226,8 @@ export class PrintPreviewSidebarElement extends PrintPreviewSidebarElementBase {
       'duplex',
       'otherOptions',
       'vendorItems',
-    ].reduce((count, setting) => {
+    ];
+    return keys.reduce((count, setting) => {
       return this.getSetting(setting).available ? count + 1 : count;
     }, 1) > MAX_SECTIONS_TO_SHOW;
   }
