@@ -4,18 +4,19 @@
 
 import 'chrome://resources/cr_elements/cr_shared_style.css.js';
 import 'chrome://resources/cr_elements/cr_button/cr_button.js';
+import 'chrome://resources/cr_elements/icons.html.js';
+import 'chrome://resources/cr_elements/cr_icon/cr_icon.js';
 import './strings.m.js';
 import './shared_style.css.js';
 import './privacy_sandbox_dialog_learn_more.js';
 
-import {CrScrollableMixin} from 'chrome://resources/cr_elements/cr_scrollable_mixin.js';
 import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {PrivacySandboxDialogMixin} from './privacy_sandbox_dialog_mixin.js';
 import {getTemplate} from './privacy_sandbox_dialog_notice_step.html.js';
 
 const PrivacySandboxDialogNoticeStepElementBase =
-    CrScrollableMixin(PrivacySandboxDialogMixin(PolymerElement));
+    PrivacySandboxDialogMixin(PolymerElement);
 
 export class PrivacySandboxDialogNoticeStepElement extends
     PrivacySandboxDialogNoticeStepElementBase {
