@@ -8,15 +8,18 @@ import 'chrome://resources/cr_elements/cr_input/cr_input.js';
 import 'chrome://resources/cr_elements/cr_shared_style.css.js';
 import 'chrome://resources/polymer/v3_0/iron-list/iron-list.js';
 
-import {CrDialogElement} from 'chrome://resources/cr_elements/cr_dialog/cr_dialog.js';
-import {CrInputElement} from 'chrome://resources/cr_elements/cr_input/cr_input.js';
+import type {CrDialogElement} from 'chrome://resources/cr_elements/cr_dialog/cr_dialog.js';
+import type {CrInputElement} from 'chrome://resources/cr_elements/cr_input/cr_input.js';
 import {assertNotReached} from 'chrome://resources/js/assert.js';
 import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
-import {DomRepeatEvent, PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import type {DomRepeatEvent} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {getTemplate} from './power_bookmarks_edit_dialog.html.js';
 import {getFolderDescendants} from './power_bookmarks_service.js';
 
+export const TEMP_FOLDER_ID_PREFIX = 'tmp_new_folder_';
+
 export interface PowerBookmarksEditDialogElement {
   $: {
     dialog: CrDialogElement,
@@ -184,12 +187,12 @@ export class PowerBookmarksEditDialogElement extends PolymerElement {
 
   private validateUrl_(): boolean {
     const urlInput = this.$.urlInput;
-    const originalValue = urlInput.inputElement.value;
 
     if (urlInput.validate()) {
       return true;
     }
 
+    const originalValue = urlInput.inputElement.value;
     urlInput.inputElement.value = 'http://' + originalValue;
 
     if (urlInput.validate()) {
@@ -229,7 +232,7 @@ export class PowerBookmarksEditDialogElement extends PolymerElement {
     const parentId =
         parent ? parent.id : loadTimeData.getString('otherBookmarksId');
     const newFolder: chrome.bookmarks.BookmarkTreeNode = {
-      id: 'tmp_new_folder_' + this.newFolders_.length,
+      id: TEMP_FOLDER_ID_PREFIX + this.newFolders_.length,
       title: loadTimeData.getString('newFolderTitle'),
       children: [],
       parentId: parentId,
