@@ -27,31 +27,38 @@ import '//resources/cr_elements/cr_toolbar/cr_toolbar_selection_overlay.js';
 import '//resources/cr_elements/icons.html.js';
 import '//resources/polymer/v3_0/iron-list/iron-list.js';
 
-import {ShoppingServiceApiProxy, ShoppingServiceApiProxyImpl} from '//bookmarks-side-panel.top-chrome/shared/commerce/shopping_service_api_proxy.js';
-import {BookmarkProductInfo} from '//bookmarks-side-panel.top-chrome/shared/shopping_list.mojom-webui.js';
-import {SpEmptyStateElement} from '//bookmarks-side-panel.top-chrome/shared/sp_empty_state.js';
+import type {SpEmptyStateElement} from '//bookmarks-side-panel.top-chrome/shared/sp_empty_state.js';
 import {ColorChangeUpdater} from '//resources/cr_components/color_change_listener/colors_css_updater.js';
+import type {BrowserProxy as ShoppingServiceApiProxy} from '//resources/cr_components/commerce/browser_proxy.js';
+import {BrowserProxyImpl as ShoppingServiceApiProxyImpl} from '//resources/cr_components/commerce/browser_proxy.js';
+import type {BookmarkProductInfo} from '//resources/cr_components/commerce/shopping_service.mojom-webui.js';
 import {getInstance as getAnnouncerInstance} from '//resources/cr_elements/cr_a11y_announcer/cr_a11y_announcer.js';
-import {CrActionMenuElement} from '//resources/cr_elements/cr_action_menu/cr_action_menu.js';
-import {CrDialogElement} from '//resources/cr_elements/cr_dialog/cr_dialog.js';
-import {CrLazyRenderElement} from '//resources/cr_elements/cr_lazy_render/cr_lazy_render.js';
-import {CrToastElement} from '//resources/cr_elements/cr_toast/cr_toast.js';
-import {CrToolbarSearchFieldElement} from '//resources/cr_elements/cr_toolbar/cr_toolbar_search_field.js';
+import type {CrActionMenuElement} from '//resources/cr_elements/cr_action_menu/cr_action_menu.js';
+import type {CrDialogElement} from '//resources/cr_elements/cr_dialog/cr_dialog.js';
+import type {CrLazyRenderElement} from '//resources/cr_elements/cr_lazy_render/cr_lazy_render.js';
+import type {CrToastElement} from '//resources/cr_elements/cr_toast/cr_toast.js';
+import type {CrToolbarSearchFieldElement} from '//resources/cr_elements/cr_toolbar/cr_toolbar_search_field.js';
 import {FocusOutlineManager} from '//resources/js/focus_outline_manager.js';
 import {loadTimeData} from '//resources/js/load_time_data.js';
 import {PluralStringProxyImpl} from '//resources/js/plural_string_proxy.js';
 import {listenOnce} from '//resources/js/util.js';
-import {IronListElement} from '//resources/polymer/v3_0/iron-list/iron-list.js';
-import {afterNextRender, DomRepeatEvent, PolymerElement} from '//resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import type {CrLitElement} from '//resources/lit/v3_0/lit.rollup.js';
+import type {IronListElement} from '//resources/polymer/v3_0/iron-list/iron-list.js';
+import type {DomRepeatEvent} from '//resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import {afterNextRender, PolymerElement} from '//resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {ActionSource, SortOrder, ViewType} from './bookmarks.mojom-webui.js';
-import {BookmarksApiProxy, BookmarksApiProxyImpl} from './bookmarks_api_proxy.js';
-import {PowerBookmarksContextMenuElement} from './power_bookmarks_context_menu.js';
+import type {BookmarksApiProxy} from './bookmarks_api_proxy.js';
+import {BookmarksApiProxyImpl} from './bookmarks_api_proxy.js';
+import type {PowerBookmarksContextMenuElement} from './power_bookmarks_context_menu.js';
 import {PowerBookmarksDragManager} from './power_bookmarks_drag_manager.js';
-import {PowerBookmarksEditDialogElement} from './power_bookmarks_edit_dialog.js';
-import {PowerBookmarksLabelsElement} from './power_bookmarks_labels.js';
+import type {PowerBookmarksEditDialogElement} from './power_bookmarks_edit_dialog.js';
+import {TEMP_FOLDER_ID_PREFIX} from './power_bookmarks_edit_dialog.js';
+import type {PowerBookmarksLabelsElement} from './power_bookmarks_labels.js';
 import {getTemplate} from './power_bookmarks_list.html.js';
-import {editingDisabledByPolicy, Label, PowerBookmarksService} from './power_bookmarks_service.js';
+import type {Label} from './power_bookmarks_service.js';
+import {editingDisabledByPolicy, PowerBookmarksService} from './power_bookmarks_service.js';
+import {getFolderLabel} from './power_bookmarks_utils.js';
 
 const ADD_FOLDER_ACTION_UMA = 'Bookmarks.FolderAddedFromSidePanel';
 const ADD_URL_ACTION_UMA = 'Bookmarks.AddedFromSidePanel';
@@ -254,7 +261,7 @@ export class PowerBookmarksListElement extends PolymerElement {
       ShoppingServiceApiProxyImpl.getInstance();
   private shoppingListenerIds_: number[] = [];
   private displayLists_: chrome.bookmarks.BookmarkTreeNode[][];
-  private trackedProductInfos_ = new Map<string, BookmarkProductInfo>();
+  private trackedProductInfos_: {[key: string]: BookmarkProductInfo} = {};
   private availableProductInfos_ = new Map<string, BookmarkProductInfo>();
   private bookmarksService_: PowerBookmarksService =
       new PowerBookmarksService(this);
@@ -264,7 +271,7 @@ export class PowerBookmarksListElement extends PolymerElement {
   private compact_: boolean;
   private activeFolderPath_: chrome.bookmarks.BookmarkTreeNode[];
   private labels_: Label[];
-  private imageUrls_ = new Map<string, string>();
+  private imageUrls_: {[key: string]: string} = {};
   private activeSortIndex_: number;
   private sortTypes_: SortOption[];
   private searchQuery_: string|undefined;
@@ -283,6 +290,10 @@ export class PowerBookmarksListElement extends PolymerElement {
   private hasShownBookmarks_: boolean;
   private sectionVisibility_: SectionVisibility = {};
   private shoppingCollectionFolderId_: string;
+  private recordCountMetricsOnNextUpdate_: boolean = false;
+  private updatedElementIds_: string[] = [];
+  private bookmarksTreeViewEnabled_: boolean =
+      loadTimeData.getBoolean('bookmarksTreeViewEnabled');
 
   constructor() {
     super();
@@ -318,11 +329,9 @@ export class PowerBookmarksListElement extends PolymerElement {
                 this.onBookmarkPriceUntracked_(product)),
     );
 
-    if (document.documentElement.hasAttribute('chrome-refresh-2023')) {
-      this.shownBookmarksResizeObserver_ =
-          new ResizeObserver(this.onShownBookmarksResize_.bind(this));
-      this.shownBookmarksResizeObserver_.observe(this.$.bookmarks);
-    }
+    this.shownBookmarksResizeObserver_ =
+        new ResizeObserver(this.onShownBookmarksResize_.bind(this));
+    this.shownBookmarksResizeObserver_.observe(this.$.bookmarks);
 
     this.updateListScrollOffset_();
 
@@ -335,10 +344,8 @@ export class PowerBookmarksListElement extends PolymerElement {
     this.shoppingListenerIds_.forEach(
         id => this.shoppingServiceApi_.getCallbackRouter().removeListener(id));
 
-    if (this.shownBookmarksResizeObserver_) {
-      this.shownBookmarksResizeObserver_.disconnect();
-      this.shownBookmarksResizeObserver_ = undefined;
-    }
+    this.shownBookmarksResizeObserver_!.disconnect();
+    this.shownBookmarksResizeObserver_ = undefined;
 
     this.bookmarksDragManager_.stopObserving();
   }
@@ -358,9 +365,9 @@ export class PowerBookmarksListElement extends PolymerElement {
 
   onBookmarkChanged(id: string, changedInfo: chrome.bookmarks.ChangeInfo) {
     const bookmark = this.bookmarksService_.findBookmarkWithId(id)!;
-    if (this.hasSomeActiveFilter_ &&
-        (this.bookmarkShouldShow_(bookmark) ||
-         this.bookmarkIsShowing_(bookmark))) {
+     this.updatedElementIds_ = [bookmark.id];
+    if (this.bookmarkShouldShow_(bookmark) ||
+        this.bookmarkIsShowing_(bookmark)) {
       this.updateDisplayLists_();
     }
     Object.keys(changedInfo).forEach(key => {
@@ -401,6 +408,7 @@ export class PowerBookmarksListElement extends PolymerElement {
         }
       }
     }
+    this.updatedElementIds_ = [bookmark.id];
     this.updateShoppingData_();
     this.notifyPathIfVisible_(parent.id, 'children');
   }
@@ -426,6 +434,7 @@ export class PowerBookmarksListElement extends PolymerElement {
         this.$.bookmarks.scrollTop = scrollTop;
       });
     }
+    this.updatedElementIds_ = [newParent.id, oldParent.id];
     // If the new parent folder is visible, notify to ensure its displayed
     // child count is updated.
     this.notifyPathIfVisible_(newParent.id, 'children');
@@ -446,7 +455,7 @@ export class PowerBookmarksListElement extends PolymerElement {
     if (this.shoppingCollectionFolderId_ === bookmark.id) {
       this.shoppingCollectionFolderId_ = '';
     }
-
+    this.updatedElementIds_ = [bookmark.parentId!];
     this.set(`trackedProductInfos_.${bookmark.id}`, null);
     this.availableProductInfos_.delete(bookmark.id);
 
@@ -455,8 +464,16 @@ export class PowerBookmarksListElement extends PolymerElement {
     this.notifyPathIfVisible_(bookmark.parentId!, 'children');
   }
 
-  isPriceTracked(bookmark: chrome.bookmarks.BookmarkTreeNode): boolean {
-    return !!this.get(`trackedProductInfos_.${bookmark.id}`);
+  getTrackedProductInfos(): {[key: string]: BookmarkProductInfo} {
+    return this.trackedProductInfos_;
+  }
+
+  getAvailableProductInfos(): Map<string, BookmarkProductInfo> {
+    return this.availableProductInfos_;
+  }
+
+  getSelectedBookmarks(): {[key: string]: boolean} {
+    return this.selectedBookmarks_;
   }
 
   getProductImageUrl(bookmark: chrome.bookmarks.BookmarkTreeNode): string {
@@ -491,10 +508,6 @@ export class PowerBookmarksListElement extends PolymerElement {
     }, {once: true});
   }
 
-  getBookmarkDescriptionForTests(bookmark: chrome.bookmarks.BookmarkTreeNode) {
-    return this.getBookmarkDescription_(bookmark);
-  }
-
   clickBookmarkRowForTests(bookmark: chrome.bookmarks.BookmarkTreeNode) {
     const event = new CustomEvent('row-clicked', {
       bubbles: true,
@@ -534,17 +547,12 @@ export class PowerBookmarksListElement extends PolymerElement {
 
   private focusBookmark_(id: string) {
     const bookmarkElement =
-        this.shadowRoot!.querySelector(`#bookmark-${id}`) as HTMLElement;
+        this.shadowRoot!.querySelector<HTMLElement>(`#bookmark-${id}`);
     if (bookmarkElement) {
       bookmarkElement.focus();
     }
   }
 
-  private isPriceTrackingEligible_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      boolean {
-    return !!this.availableProductInfos_.get(bookmark.id);
-  }
-
   private onBookmarkPriceTracked_(product: BookmarkProductInfo) {
     this.set(`trackedProductInfos_.${product.bookmarkId.toString()}`, product);
   }
@@ -555,7 +563,8 @@ export class PowerBookmarksListElement extends PolymerElement {
 
   private bookmarkIsShowing_(bookmark: chrome.bookmarks.BookmarkTreeNode):
       boolean {
-    return this.displayLists_.some(list => list.includes(bookmark));
+    return this.displayLists_.some(
+        list => list.some(item => item.id === bookmark.id));
   }
 
   private removeNodeFromDisplayLists_(nodeId: string) {
@@ -602,56 +611,13 @@ export class PowerBookmarksListElement extends PolymerElement {
     const parentFolder = this.bookmarksService_.findBookmarkWithId(
         activeFolder ? activeFolder.parentId : undefined);
     return loadTimeData.getStringF(
-        'backButtonLabel', this.getFolderLabel_(parentFolder));
+        'backButtonLabel', getFolderLabel(parentFolder));
   }
 
   private getBookmarksListRole_(): string {
     return this.editing_ ? 'listbox' : 'list';
   }
 
-  private getBookmarkDescription_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      string|undefined {
-    if (this.compact_) {
-      if (bookmark.url) {
-        return undefined;
-      }
-      const count = bookmark.children ? bookmark.children.length : 0;
-      return loadTimeData.getStringF('bookmarkFolderChildCount', count);
-    } else {
-      let urlString;
-      if (bookmark.url) {
-        const url = new URL(bookmark.url);
-        // Show chrome:// if it's a chrome internal url
-        if (url.protocol === 'chrome:') {
-          urlString = 'chrome://' + url.hostname;
-        }
-        urlString = url.hostname;
-      }
-      if (urlString && this.searchQuery_ && bookmark.parentId) {
-        const parentFolder =
-            this.bookmarksService_.findBookmarkWithId(bookmark.parentId);
-        const folderLabel = this.getFolderLabel_(parentFolder);
-        return loadTimeData.getStringF(
-            'urlFolderDescription', urlString, folderLabel);
-      }
-      return urlString;
-    }
-  }
-
-  private getBookmarkDescriptionMeta_(bookmark:
-                                          chrome.bookmarks.BookmarkTreeNode) {
-    // If there is a price available for the product and it isn't being
-    // tracked, return the current price which will be added to the description
-    // meta section.
-    const productInfo = this.availableProductInfos_.get(bookmark.id);
-    if (productInfo && productInfo.info.currentPrice &&
-        !this.isPriceTracked(bookmark)) {
-      return productInfo.info.currentPrice;
-    }
-
-    return '';
-  }
-
   private getViewButtonIcon_() {
     return this.compact_ ? 'bookmarks:compact-view' : 'bookmarks:visual-view';
   }
@@ -661,110 +627,23 @@ export class PowerBookmarksListElement extends PolymerElement {
                            loadTimeData.getString('visualView');
   }
 
-  private getBookmarkMenuA11yLabel_(url: string, title: string): string {
-    if (url) {
-      return loadTimeData.getStringF('bookmarkMenuLabel', title);
-    } else {
-      return loadTimeData.getStringF('folderMenuLabel', title);
-    }
-  }
-
-  private getBookmarkA11yLabel_(id: string, url: string, title: string):
-      string {
-    if (this.editing_) {
-      if (this.get(`selectedBookmarks_.${id}`)) {
-        if (url) {
-          return loadTimeData.getStringF('deselectBookmarkLabel', title);
-        }
-        return loadTimeData.getStringF('deselectFolderLabel', title);
-      } else {
-        if (url) {
-          return loadTimeData.getStringF('selectBookmarkLabel', title);
-        }
-        return loadTimeData.getStringF('selectFolderLabel', title);
-      }
-    }
-    if (url) {
-      return loadTimeData.getStringF('openBookmarkLabel', title);
-    }
-    return loadTimeData.getStringF('openFolderLabel', title);
-  }
-
-  private getBookmarkA11yDescription_(
-      bookmark: chrome.bookmarks.BookmarkTreeNode): string {
-    let description = '';
-    if (this.isPriceTracked(bookmark)) {
-      description += loadTimeData.getStringF(
-          'a11yDescriptionPriceTracking', this.getCurrentPrice_(bookmark));
-      const previousPrice = this.getPreviousPrice_(bookmark);
-      if (previousPrice) {
-        description += loadTimeData.getStringF(
-            'a11yDescriptionPriceChange', previousPrice);
-      }
-    }
-    return description;
-  }
-
   private updateShoppingCollectionFolderId_(): void {
     this.shoppingServiceApi_.getShoppingCollectionBookmarkFolderId().then(res => {
       this.shoppingCollectionFolderId_ = res.collectionId.toString();
     });
   }
 
-  private isShoppingCollection_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      boolean {
-    return bookmark.id === this.shoppingCollectionFolderId_;
-  }
-
-  private getBookmarkImageUrls_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      string[] {
-    const imageUrls: string[] = [];
-    if (bookmark.url) {
-      const imageUrl = this.get(`imageUrls_.${bookmark.id.toString()}`);
-      if (imageUrl) {
-        imageUrls.push(imageUrl);
-      }
-    } else if (
-        this.canEdit_(bookmark) && bookmark.children &&
-        !this.isShoppingCollection_(bookmark)) {
-      bookmark.children.forEach((child) => {
-        const childImageUrl: string =
-            this.get(`imageUrls_.${child.id.toString()}`);
-        if (childImageUrl) {
-          imageUrls.push(childImageUrl);
-        }
-      });
-    }
-    return imageUrls;
-  }
-
-  private getBookmarkForceHover_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      boolean {
-    return bookmark === this.contextMenuBookmark_;
-  }
-
   private getActiveFolderLabel_(): string {
-    return this.getFolderLabel_(this.getActiveFolder_());
-  }
-
-  private getFolderLabel_(folder: chrome.bookmarks.BookmarkTreeNode|
-                          undefined): string {
-    if (folder && folder.id !== loadTimeData.getString('otherBookmarksId') &&
-        folder.id !== loadTimeData.getString('mobileBookmarksId')) {
-      return folder!.title;
-    } else {
+    if (this.bookmarksTreeViewEnabled_ && this.compact_) {
       return loadTimeData.getString('allBookmarks');
     }
+    return getFolderLabel(this.getActiveFolder_());
   }
 
   private getSortLabel_(): string {
     return this.sortTypes_[this.activeSortIndex_]!.label;
   }
 
-  private renamingItem_(id: string) {
-    return id === this.renamingId_;
-  }
-
   private updateShoppingData_() {
     this.availableProductInfos_.clear();
     this.shoppingServiceApi_.getAllShoppingBookmarkProductInfo().then(res => {
@@ -790,7 +669,13 @@ export class PowerBookmarksListElement extends PolymerElement {
    * Update the lists of bookmarks and folders displayed to the user.
    */
   private updateDisplayLists_() {
-    const activeFolder = this.getActiveFolder_();
+    let activeFolder;
+    if (this.bookmarksTreeViewEnabled_ && this.compact_) {
+      activeFolder = this.bookmarksService_.findBookmarkWithId(
+          loadTimeData.getString('otherBookmarksId'));
+    } else {
+      activeFolder = this.getActiveFolder_();
+    }
     const primaryList = this.bookmarksService_.filterBookmarks(
         activeFolder, this.activeSortIndex_, this.searchQuery_, this.labels_);
     this.displayLists_ = [primaryList];
@@ -803,6 +688,22 @@ export class PowerBookmarksListElement extends PolymerElement {
     this.displayLists_.forEach(
         list => this.bookmarksService_.refreshDataForBookmarks(list));
     this.updateListScrollOffset_();
+
+    if (this.recordCountMetricsOnNextUpdate_) {
+      this.recordBookmarkCountMetrics_();
+      this.recordCountMetricsOnNextUpdate_ = false;
+    }
+
+    // After the lists are updated and all children updates are complete,
+    // notify iron-list to resize.
+    afterNextRender(this, () => {
+      const children = [...this.shadowRoot!.querySelectorAll<CrLitElement>(
+          'power-bookmark-row')];
+      if (children.length > 0) {
+        Promise.all(children.map(el => el.updateComplete))
+            .then(() => this.notifyBookmarksListResize_());
+      }
+    });
   }
 
   private updateListScrollOffset_() {
@@ -841,6 +742,15 @@ export class PowerBookmarksListElement extends PolymerElement {
     chrome.metricsPrivate.recordEnumerationValue(
         'PowerBookmarks.SidePanel.Search.CTR', SearchAction.SHOWN,
         SearchAction.COUNT);
+    this.recordCountMetricsOnNextUpdate_ = true;
+  }
+
+  private recordBookmarkCountMetrics_() {
+    const count =
+        this.displayLists_.reduce((prev, curr) => prev + curr.length, 0);
+    const metricName = `PowerBookmarks.SidePanel${
+        this.hasSomeActiveFilter_ ? '.SearchOrFilter' : ''}.BookmarksShown`;
+    chrome.metricsPrivate.recordMediumCount(metricName, count);
   }
 
   private canAddCurrentUrl_(): boolean {
@@ -848,11 +758,6 @@ export class PowerBookmarksListElement extends PolymerElement {
         this.currentUrl_, this.getActiveFolder_());
   }
 
-  private canEdit_(bookmark: chrome.bookmarks.BookmarkTreeNode): boolean {
-    return bookmark.id !== loadTimeData.getString('bookmarksBarId') &&
-        bookmark.id !== loadTimeData.getString('managedBookmarksFolderId');
-  }
-
   private getSortMenuItemLabel_(sortType: SortOption): string {
     return loadTimeData.getStringF('sortByType', sortType.label);
   }
@@ -866,11 +771,19 @@ export class PowerBookmarksListElement extends PolymerElement {
         sortType.sortOrder;
   }
 
-  private bookmarkIsSelected_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      boolean {
-    return this.get(`selectedBookmarks_.${bookmark.id.toString()}`);
+  private onRowToggled_(event: CustomEvent<{
+    bookmark: chrome.bookmarks.BookmarkTreeNode,
+    expanded: boolean,
+    event: MouseEvent,
+  }>) {
+    const bookmark = event.detail.bookmark;
+    if (event.detail.expanded) {
+      this.activeFolderPath_ = this.bookmarksService_.findPathToId(bookmark.id);
+    } else if (bookmark === this.getActiveFolder_()) {
+      this.pop('activeFolderPath_');
+    }
+    this.notifyBookmarksListResize_();
   }
-
   /**
    * Invoked when the user clicks a power bookmarks row. This will either
    * display children in the case of a folder row, or open the URL in the case
@@ -883,6 +796,7 @@ export class PowerBookmarksListElement extends PolymerElement {
     event.stopPropagation();
     if (!this.editing_) {
       if (event.detail.bookmark.children) {
+        this.recordCountMetricsOnNextUpdate_ = true;
         this.push('activeFolderPath_', event.detail.bookmark);
         // Cancel search when changing active folder.
         this.$.searchField.setValue('');
@@ -919,7 +833,8 @@ export class PowerBookmarksListElement extends PolymerElement {
           {bookmark: chrome.bookmarks.BookmarkTreeNode, checked: boolean}>) {
     event.preventDefault();
     event.stopPropagation();
-    const isSelected = this.bookmarkIsSelected_(event.detail.bookmark);
+    const isSelected =
+        !!this.bookmarksService_?.bookmarkIsSelected(event.detail.bookmark);
     if (event.detail.checked && !isSelected) {
       this.set(
           `selectedBookmarks_.${event.detail.bookmark.id.toString()}`, true);
@@ -947,6 +862,13 @@ export class PowerBookmarksListElement extends PolymerElement {
       if (folder.id === parentId) {
         parentId = newFolder.id;
       }
+      // Removing folders added in edit menu while editing a bookmark as they
+      // are made with TEMP_FOLDER_ID_PREFIX bookmark-id and are again created
+      // with correct id with createFolder method above
+      const parentFolder =
+          this.bookmarksService_.findBookmarkWithId(folder.parentId)!;
+      parentFolder.children = parentFolder.children?.filter(
+          child => !child.id.startsWith(TEMP_FOLDER_ID_PREFIX));
     }
     this.bookmarksApi_.editBookmarks(
         event.detail.bookmarks.map(bookmark => bookmark.id), event.detail.name,
@@ -1020,10 +942,19 @@ export class PowerBookmarksListElement extends PolymerElement {
    * Moves the displayed folders up one level when the back button is clicked.
    */
   private onBackClicked_() {
+    this.recordCountMetricsOnNextUpdate_ = true;
     this.pop('activeFolderPath_');
   }
 
+  private shouldHideBackButton_(): boolean {
+    if (this.compact_ && this.bookmarksTreeViewEnabled_) {
+      return true;
+    }
+    return !this.activeFolderPath_.length;
+  }
+
   private onSearchChanged_(e: CustomEvent<string>) {
+    this.recordCountMetricsOnNextUpdate_ = true;
     this.searchQuery_ = e.detail.toLocaleLowerCase();
   }
 
@@ -1045,9 +976,10 @@ export class PowerBookmarksListElement extends PolymerElement {
     if (!event.detail.bookmark) {
       return;
     }
-    const priceTracked = this.isPriceTracked(event.detail.bookmark);
+    const priceTracked =
+        !!this.bookmarksService_.getPriceTrackedInfo(event.detail.bookmark);
     const priceTrackingEligible =
-        this.isPriceTrackingEligible_(event.detail.bookmark);
+        !!this.bookmarksService_.getAvailableProductInfo(event.detail.bookmark);
     const bookmark = event.detail.bookmark;
     if (event.detail.event.button === 0) {
       this.$.contextMenu.showAt(
@@ -1206,6 +1138,11 @@ export class PowerBookmarksListElement extends PolymerElement {
     event.preventDefault();
     event.stopPropagation();
     this.compact_ = !this.compact_;
+    if(this.bookmarksTreeViewEnabled_ && this.compact_){
+      // While changing visual view to tree view, displayList_ should get back
+      // to allBookmarks list.
+      this.updateDisplayLists_();
+    }
     this.notifyBookmarksListResize_();
     const viewType = this.compact_ ? ViewType.kCompact : ViewType.kExpanded;
     this.bookmarksApi_.setViewType(viewType);
@@ -1295,38 +1232,6 @@ export class PowerBookmarksListElement extends PolymerElement {
     };
   }
 
-  /**
-   * Whether the given price-tracked bookmark should display as if discounted.
-   */
-  private showDiscountedPrice_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      boolean {
-    const bookmarkProductInfo = this.get(`trackedProductInfos_.${bookmark.id}`);
-    if (bookmarkProductInfo) {
-      return bookmarkProductInfo.info.previousPrice.length > 0;
-    }
-    return false;
-  }
-
-  private getCurrentPrice_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      string {
-    const bookmarkProductInfo = this.get(`trackedProductInfos_.${bookmark.id}`);
-    if (bookmarkProductInfo) {
-      return bookmarkProductInfo.info.currentPrice;
-    } else {
-      return '';
-    }
-  }
-
-  private getPreviousPrice_(bookmark: chrome.bookmarks.BookmarkTreeNode):
-      string {
-    const bookmarkProductInfo = this.get(`trackedProductInfos_.${bookmark.id}`);
-    if (bookmarkProductInfo) {
-      return bookmarkProductInfo.info.previousPrice;
-    } else {
-      return '';
-    }
-  }
-
   private onShownBookmarksResize_() {
     // The iron-lists of `displayLists_` are in a dynamically sized card.
     // Any time the size changes, let iron-list know so that iron-list can
