@@ -4,12 +4,14 @@
 
 // This file contains business logic for power bookmarks side panel content.
 
+import type {BookmarkProductInfo} from '//resources/cr_components/commerce/shopping_service.mojom-webui.js';
 import {PageImageServiceBrowserProxy} from '//resources/cr_components/page_image_service/browser_proxy.js';
 import {ClientId as PageImageServiceClientId} from '//resources/cr_components/page_image_service/page_image_service.mojom-webui.js';
 import {loadTimeData} from '//resources/js/load_time_data.js';
-import {Url} from '//resources/mojo/url/mojom/url.mojom-webui.js';
+import type {Url} from '//resources/mojo/url/mojom/url.mojom-webui.js';
 
-import {BookmarksApiProxy, BookmarksApiProxyImpl} from './bookmarks_api_proxy.js';
+import type {BookmarksApiProxy} from './bookmarks_api_proxy.js';
+import {BookmarksApiProxyImpl} from './bookmarks_api_proxy.js';
 
 // This corresponds to the max number of concurrent ImageService requests
 // before further requests get dropped. Further requests up to 600 should be
@@ -36,7 +38,9 @@ interface PowerBookmarksDelegate {
       oldParent: chrome.bookmarks.BookmarkTreeNode,
       newParent: chrome.bookmarks.BookmarkTreeNode): void;
   onBookmarkRemoved(bookmark: chrome.bookmarks.BookmarkTreeNode): void;
-  isPriceTracked(bookmark: chrome.bookmarks.BookmarkTreeNode): boolean;
+  getTrackedProductInfos(): {[key: string]: BookmarkProductInfo};
+  getAvailableProductInfos(): Map<string, BookmarkProductInfo>;
+  getSelectedBookmarks(): {[key: string]: boolean};
   getProductImageUrl(bookmark: chrome.bookmarks.BookmarkTreeNode): string;
 }
 
@@ -314,7 +318,7 @@ export class PowerBookmarksService {
   findBookmarkWithId(id: string|undefined): chrome.bookmarks.BookmarkTreeNode
       |undefined {
     if (id) {
-      const path = this.findPathToId_(id);
+      const path = this.findPathToId(id);
       if (path) {
         return path[path.length - 1];
       }
@@ -354,6 +358,27 @@ export class PowerBookmarksService {
     this.maxImageServiceRequests_ = max;
   }
 
+  getPriceTrackedInfo(bookmark: chrome.bookmarks.BookmarkTreeNode):
+      BookmarkProductInfo|undefined {
+    const trackedProductInfos = this.delegate_.getTrackedProductInfos();
+    const priceTrackValue = Object.entries(trackedProductInfos)
+                                .find(([key, _val]) => key === bookmark.id)
+                                ?.[1];
+    return priceTrackValue;
+  }
+
+  getAvailableProductInfo(bookmark: chrome.bookmarks.BookmarkTreeNode):
+      BookmarkProductInfo|undefined {
+    const availableProductInfos = this.delegate_.getAvailableProductInfos();
+    return availableProductInfos.get(bookmark.id);
+  }
+
+  bookmarkIsSelected(bookmark: chrome.bookmarks.BookmarkTreeNode): boolean {
+    const selectedBookmarks = this.delegate_.getSelectedBookmarks();
+    return Object.entries(selectedBookmarks)
+               .find(([key, _val]) => key === bookmark.id)?.[1] ?? false;
+  }
+
 
   private applySearchQueryAndLabels_(
       labels: Label[], searchQuery: string|undefined,
@@ -377,8 +402,8 @@ export class PowerBookmarksService {
   private nodeMatchesContentFilters_(
       bookmark: chrome.bookmarks.BookmarkTreeNode, labels: Label[]): boolean {
     // Price tracking label
-    if (labels[0] && labels[0]!.active &&
-        !this.delegate_.isPriceTracked(bookmark)) {
+    const isPriceTracked = !!this.getPriceTrackedInfo(bookmark);
+    if (labels[0] && labels[0]!.active && !isPriceTracked) {
       return false;
     }
     return true;
@@ -392,7 +417,17 @@ export class PowerBookmarksService {
   private onChanged_(id: string, changedInfo: chrome.bookmarks.ChangeInfo) {
     const bookmark = this.findBookmarkWithId(id)!;
     Object.assign(bookmark, changedInfo);
-    this.findBookmarkImageUrls_(bookmark, false, true);
+    // Deep copy is necessary to ensure that the original bookmark object is
+    // not directly mutated. This helps LitElement's change detection recognize
+    // the changes since the reference to the object will change.
+    const deepCopyBookmark = structuredClone(bookmark);
+    const parent = this.findBookmarkWithId(bookmark.parentId);
+    if (parent) {
+      const index =
+          parent.children!.findIndex(child => child.id === bookmark.id);
+      parent.children![index] = deepCopyBookmark;
+    }
+    this.findBookmarkImageUrls_(deepCopyBookmark, false, true);
     this.delegate_.onBookmarkChanged(id, changedInfo);
   }
 
@@ -426,7 +461,7 @@ export class PowerBookmarksService {
   }
 
   private onRemoved_(id: string) {
-    const oldPath = this.findPathToId_(id);
+    const oldPath = this.findPathToId(id);
     const removedNode = oldPath.pop()!;
     const oldParent = oldPath[oldPath.length - 1]!;
     oldParent.children!.splice(oldParent.children!.indexOf(removedNode), 1);
@@ -437,7 +472,7 @@ export class PowerBookmarksService {
    * Finds the node within all bookmarks and returns the path to the node in
    * the tree.
    */
-  private findPathToId_(id: string): chrome.bookmarks.BookmarkTreeNode[] {
+  findPathToId(id: string): chrome.bookmarks.BookmarkTreeNode[] {
     const path: chrome.bookmarks.BookmarkTreeNode[] = [];
 
     function findPathByIdInternal(
@@ -511,6 +546,11 @@ export class PowerBookmarksService {
 
     // Fetch the representative image for this page, if possible.
     this.activeImageServiceRequestCount_++;
+    // TODO(b/303613231): Update this code to distinguish account bookmarks
+    // (which can get images from PageImageService) from local bookmarks (which
+    // can't), once the account bookmark store exists. The "is account bookmark"
+    // bit will likely need to be plumbed here. (For reference:
+    // crrev.com/c/5346717 made the equivalent change for Android.)
     const {result} =
         await PageImageServiceBrowserProxy.getInstance()
             .handler.getPageImageUrl(
