@@ -3,16 +3,17 @@
 // found in the LICENSE file.
 
 import 'chrome://resources/cr_elements/cr_auto_img/cr_auto_img.js';
-import 'chrome://resources/polymer/v3_0/iron-pages/iron-pages.js';
-import 'chrome://resources/polymer/v3_0/paper-ripple/paper-ripple.js';
+import 'chrome://resources/cr_elements/cr_ripple/cr_ripple.js';
 
 import {assert} from 'chrome://resources/js/assert.js';
 import {skColorToRgba} from 'chrome://resources/js/color_utils.js';
-import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import {CrLitElement} from 'chrome://resources/lit/v3_0/lit.rollup.js';
+import type {PropertyValues} from 'chrome://resources/lit/v3_0/lit.rollup.js';
 
-import {CustomizeChromePageCallbackRouter, CustomizeChromePageHandlerInterface, Theme} from './customize_chrome.mojom-webui.js';
+import type {CustomizeChromePageCallbackRouter, CustomizeChromePageHandlerInterface, Theme} from './customize_chrome.mojom-webui.js';
 import {CustomizeChromeApiProxy} from './customize_chrome_api_proxy.js';
-import {getTemplate} from './theme_snapshot.html.js';
+import {getCss} from './theme_snapshot.css.js';
+import {getHtml} from './theme_snapshot.html.js';
 
 export enum CustomizeThemeType {
   CLASSIC_CHROME = 'classicChrome',
@@ -21,33 +22,28 @@ export enum CustomizeThemeType {
 }
 
 /** Element used to display a snapshot of the NTP. */
-export class ThemeSnapshotElement extends PolymerElement {
+export class ThemeSnapshotElement extends CrLitElement {
   static get is() {
     return 'customize-chrome-theme-snapshot';
   }
 
-  static get template() {
-    return getTemplate();
+  static override get styles() {
+    return getCss();
   }
 
-  static get properties() {
+  override render() {
+    return getHtml.bind(this)();
+  }
+
+  static override get properties() {
     return {
-      chromeRefresh2023Enabled_: {
-        type: Boolean,
-      },
-
-      theme_: Object,
-
-      themeType_: {
-        type: String,
-        computed: 'computeThemeType_(theme_)',
-      },
+      theme_: {type: Object},
+      themeType_: {type: String},
     };
   }
 
-  private chromeRefresh2023Enabled_: boolean;
-  private theme_: Theme|undefined = undefined;
-  private themeType_: CustomizeThemeType|null = null;
+  protected theme_: Theme|null = null;
+  protected themeType_: CustomizeThemeType|null = null;
 
   private callbackRouter_: CustomizeChromePageCallbackRouter;
   private pageHandler_: CustomizeChromePageHandlerInterface;
@@ -61,16 +57,13 @@ export class ThemeSnapshotElement extends PolymerElement {
 
   override connectedCallback() {
     super.connectedCallback();
-    this.chromeRefresh2023Enabled_ =
-        document.documentElement.hasAttribute('chrome-refresh-2023');
     this.setThemeListenerId_ =
         this.callbackRouter_.setTheme.addListener((theme: Theme) => {
           this.theme_ = theme;
           if (this.theme_) {
-            this.updateStyles({
-              '--customize-chrome-color-background-color':
-                  skColorToRgba(this.theme_.backgroundColor),
-            });
+            this.style.setProperty(
+                '--customize-chrome-color-background-color',
+                skColorToRgba(this.theme_.backgroundColor));
           }
         });
     this.pageHandler_.updateTheme();
@@ -83,22 +76,36 @@ export class ThemeSnapshotElement extends PolymerElement {
     this.callbackRouter_.removeListener(this.setThemeListenerId_);
   }
 
+  override willUpdate(changedProperties: PropertyValues<this>) {
+    super.willUpdate(changedProperties);
+
+    const changedPrivateProperties =
+        changedProperties as Map<PropertyKey, unknown>;
+
+    if (changedPrivateProperties.has('theme_')) {
+      this.themeType_ = this.computeThemeType_();
+    }
+  }
+
   private computeThemeType_(): CustomizeThemeType|null {
     if (this.theme_) {
       if (!this.theme_.backgroundImage) {
         return CustomizeThemeType.CLASSIC_CHROME;
-      } else if (this.theme_.backgroundImage.isUploadedImage) {
+      }
+
+      if (this.theme_.backgroundImage.isUploadedImage) {
         return CustomizeThemeType.UPLOADED_IMAGE;
-      } else if (this.theme_.backgroundImage.snapshotUrl?.url) {
+      }
+
+      if (this.theme_.backgroundImage.snapshotUrl?.url) {
         return CustomizeThemeType.CUSTOM_THEME;
       }
     }
     return null;
   }
 
-  private onThemeSnapshotClick_() {
-    if (!this.chromeRefresh2023Enabled_ ||
-        (this.theme_ && this.theme_.backgroundManagedByPolicy)) {
+  protected onThemeSnapshotClick_() {
+    if (this.theme_ && this.theme_.backgroundManagedByPolicy) {
       return;
     }
     this.dispatchEvent(new Event('edit-theme-click'));
