@@ -2,7 +2,8 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import {WallpaperSearchClientCallbackRouter, WallpaperSearchHandlerFactory, WallpaperSearchHandlerInterface, WallpaperSearchHandlerRemote} from '../wallpaper_search.mojom-webui.js';
+import type {WallpaperSearchHandlerInterface} from '../wallpaper_search.mojom-webui.js';
+import {WallpaperSearchClientCallbackRouter, WallpaperSearchHandlerFactory, WallpaperSearchHandlerRemote} from '../wallpaper_search.mojom-webui.js';
 
 let instance: WallpaperSearchProxy|null = null;
 
