@@ -2,17 +2,22 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import {CrLitElement} from 'chrome://resources/lit/v3_0/lit.rollup.js';
 
-import {getTemplate} from './wallpaper_search_tile.html.js';
+import {getCss} from './wallpaper_search_tile.css.js';
+import {getHtml} from './wallpaper_search_tile.html.js';
 
-export class WallpaperSearchTileElement extends PolymerElement {
+export class WallpaperSearchTileElement extends CrLitElement {
   static get is() {
     return 'customize-chrome-wallpaper-search-tile';
   }
 
-  static get template() {
-    return getTemplate();
+  static override get styles() {
+    return getCss();
+  }
+
+  override render() {
+    return getHtml.bind(this)();
   }
 }
 
