@@ -4,21 +4,19 @@
 
 import 'chrome://resources/cr_elements/cr_url_list_item/cr_url_list_item.js';
 import 'chrome://resources/cr_elements/cr_icon_button/cr_icon_button.js';
-import 'chrome://resources/cr_elements/cr_icons.css.js';
-import 'chrome://resources/cr_elements/icons.html.js';
-import 'chrome://resources/cr_elements/mwb_element_shared_style.css.js';
-import 'chrome://resources/cr_elements/mwb_shared_vars.css.js';
-import 'chrome://resources/cr_elements/cr_shared_vars.css.js';
+import 'chrome://resources/cr_elements/icons_lit.html.js';
 import './icons.html.js';
 
-import {CrUrlListItemElement} from 'chrome://resources/cr_elements/cr_url_list_item/cr_url_list_item.js';
-import {MouseHoverableMixin} from 'chrome://resources/cr_elements/mouse_hoverable_mixin.js';
+import type {CrUrlListItemElement} from 'chrome://resources/cr_elements/cr_url_list_item/cr_url_list_item.js';
+import {MouseHoverableMixinLit} from 'chrome://resources/cr_elements/mouse_hoverable_mixin_lit.js';
 import {assertNotReached} from 'chrome://resources/js/assert.js';
-import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import {CrLitElement} from 'chrome://resources/lit/v3_0/lit.rollup.js';
 
-import {ReadLaterEntry} from './reading_list.mojom-webui.js';
-import {ReadingListApiProxy, ReadingListApiProxyImpl} from './reading_list_api_proxy.js';
-import {getTemplate} from './reading_list_item.html.js';
+import type {ReadLaterEntry} from './reading_list.mojom-webui.js';
+import type {ReadingListApiProxy} from './reading_list_api_proxy.js';
+import {ReadingListApiProxyImpl} from './reading_list_api_proxy.js';
+import {getCss} from './reading_list_item.css.js';
+import {getHtml} from './reading_list_item.html.js';
 
 export const MARKED_AS_READ_UI_EVENT = 'reading-list-marked-as-read';
 
@@ -33,45 +31,48 @@ export interface ReadingListItemElement {
   };
 }
 
-const ReadingListItemElementBase = MouseHoverableMixin(PolymerElement);
+const ReadingListItemElementBase = MouseHoverableMixinLit(CrLitElement);
 
 export class ReadingListItemElement extends ReadingListItemElementBase {
   static get is() {
     return 'reading-list-item';
   }
 
-  static get template() {
-    return getTemplate();
+  static override get styles() {
+    return getCss();
   }
 
-  static get properties() {
+  override render() {
+    return getHtml.bind(this)();
+  }
+
+  static override get properties() {
     return {
-      data: Object,
-      buttonRipples: Boolean,
-      title: {
-        computed: 'computeTitle_(data.title)',
-        reflectToAttribute: true,
-      },
+      data: {type: Object},
+      buttonRipples: {type: Boolean},
     };
   }
 
-  data: ReadLaterEntry;
-  buttonRipples: boolean;
+  data: ReadLaterEntry = {
+    title: '',
+    url: {url: ''},
+    displayUrl: '',
+    updateTime: 0n,
+    read: false,
+    displayTimeSinceUpdate: '',
+  };
+
+  buttonRipples: boolean = false;
   private apiProxy_: ReadingListApiProxy =
       ReadingListApiProxyImpl.getInstance();
 
-  override ready() {
-    super.ready();
+  override firstUpdated() {
     this.addEventListener('click', this.onClick_);
     this.addEventListener('auxclick', this.onAuxClick_.bind(this));
     this.addEventListener('contextmenu', this.onContextMenu_.bind(this));
     this.addEventListener('keydown', this.onKeyDown_.bind(this));
   }
 
-  private computeTitle_(): string {
-    return this.data.title;
-  }
-
   override focus() {
     this.$.crUrlListItem.focus();
   }
@@ -123,20 +124,18 @@ export class ReadingListItemElement extends ReadingListItemElementBase {
       case 'Enter':
         this.onClick_(e);
         break;
-      case 'ArrowRight':
-        if (focusedIndex >= focusableElements.length - 1) {
-          focusableElements[0].focus();
-        } else {
-          focusableElements[focusedIndex + 1].focus();
-        }
+      case 'ArrowRight': {
+        const index =
+            focusedIndex >= focusableElements.length - 1 ? 0 : focusedIndex + 1;
+        focusableElements[index]!.focus();
         break;
-      case 'ArrowLeft':
-        if (focusedIndex <= 0) {
-          focusableElements[focusableElements.length - 1].focus();
-        } else {
-          focusableElements[focusedIndex - 1].focus();
-        }
+      }
+      case 'ArrowLeft': {
+        const index =
+            focusedIndex <= 0 ? focusableElements.length - 1 : focusedIndex - 1;
+        focusableElements[index]!.focus();
         break;
+      }
       default:
         assertNotReached();
     }
@@ -144,7 +143,7 @@ export class ReadingListItemElement extends ReadingListItemElementBase {
     e.stopPropagation();
   }
 
-  private onUpdateStatusClick_(e: Event) {
+  protected onUpdateStatusClick_(e: Event) {
     e.stopPropagation();
     this.apiProxy_.updateReadStatus(this.data.url, !this.data.read);
     if (!this.data.read) {
@@ -153,7 +152,7 @@ export class ReadingListItemElement extends ReadingListItemElementBase {
     }
   }
 
-  private onItemDeleteClick_(e: Event) {
+  protected onItemDeleteClick_(e: Event) {
     e.stopPropagation();
     this.apiProxy_.removeEntry(this.data.url);
   }
@@ -161,7 +160,7 @@ export class ReadingListItemElement extends ReadingListItemElementBase {
   /**
    * @return The appropriate icon for the current state
    */
-  private getUpdateStatusButtonIcon_(
+  protected getUpdateStatusButtonIcon_(
       markAsUnreadIcon: string, markAsReadIcon: string): string {
     return this.data.read ? markAsUnreadIcon : markAsReadIcon;
   }
@@ -169,7 +168,7 @@ export class ReadingListItemElement extends ReadingListItemElementBase {
   /**
    * @return The appropriate tooltip for the current state
    */
-  private getUpdateStatusButtonTooltip_(
+  protected getUpdateStatusButtonTooltip_(
       markAsUnreadTooltip: string, markAsReadTooltip: string): string {
     return this.data.read ? markAsUnreadTooltip : markAsReadTooltip;
   }
