@@ -4,41 +4,17 @@
 
 import {sendWithPromise} from 'chrome://resources/js/cr.js';
 
-/**
- * @see chrome/browser/ui/webui/tab_search/tab_search_sync_handler.cc
- */
-export interface AccountInfo {
-  name: string;
-  email: string;
-  avatarImage?: string;
-}
-
-export interface SyncInfo {
-  syncing: boolean;
-  paused: boolean;
-  syncingHistory: boolean;
-}
-
 export interface TabSearchSyncBrowserProxy {
   /**
-   * Gets the current sync info.
+   * Returns whether the user is signed in
    */
-  getSyncInfo(): Promise<SyncInfo>;
-
-  /**
-   * Gets the current account info.
-   */
-  getAccountInfo(): Promise<AccountInfo>;
+  getSignInState(): Promise<boolean>;
 }
 
 export class TabSearchSyncBrowserProxyImpl implements
     TabSearchSyncBrowserProxy {
-  getSyncInfo() {
-    return sendWithPromise('GetSyncInfo');
-  }
-
-  getAccountInfo() {
-    return sendWithPromise('GetAccountInfo');
+  getSignInState() {
+    return sendWithPromise('GetSignInState');
   }
 
   static getInstance(): TabSearchSyncBrowserProxy {
