@@ -34,16 +34,15 @@
 #include "chrome/browser/signin/chrome_device_id_helper.h"
 #include "chrome/browser/signin/force_signin_verifier.h"
 #include "chrome/browser/signin/identity_manager_factory.h"
-#include "chrome/browser/signin/signin_features.h"
 #include "chrome/browser/signin/signin_util.h"
 #include "chrome/common/buildflags.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/pref_names.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/bookmarks/browser/url_and_title.h"
+#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "components/content_settings/core/browser/cookie_settings.h"
 #include "components/metrics/metrics_service.h"
-#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "components/policy/core/browser/browser_policy_connector.h"
 #endif
 #include "components/prefs/pref_service.h"
@@ -58,7 +57,6 @@
 #include "components/signin/public/identity_manager/identity_manager.h"
 #include "components/signin/public/identity_manager/primary_account_change_event.h"
 #include "components/signin/public/identity_manager/scope_set.h"
-#include "components/supervised_user/core/common/buildflags.h"
 #include "components/version_info/channel.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/storage_partition.h"
@@ -67,10 +65,6 @@
 #include "ui/base/models/tree_node_iterator.h"
 #include "url/gurl.h"
 
-#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
-#include "components/supervised_user/core/common/supervised_user_constants.h"
-#endif
-
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "chrome/browser/signin/wait_for_network_callback_helper_ash.h"
 #include "chromeos/ash/components/network/network_handler.h"
@@ -110,6 +104,7 @@
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 #include "chrome/browser/signin/bound_session_credentials/bound_session_cookie_refresh_service.h"
 #include "chrome/browser/signin/bound_session_credentials/bound_session_cookie_refresh_service_factory.h"
+#include "chrome/browser/signin/bound_session_credentials/bound_session_oauth_multilogin_delegate_impl.h"
 #include "chrome/browser/signin/bound_session_credentials/bound_session_request_throttled_handler_browser_impl.h"
 #include "chrome/browser/signin/bound_session_credentials/throttled_gaia_auth_fetcher.h"
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
@@ -117,7 +112,7 @@
 namespace {
 
 // List of sources for which sign out is always allowed.
-// TODO(crbug.com/1161966): core product logic should not rely on metric
+// TODO(crbug.com/40162614): core product logic should not rely on metric
 // sources/callsites.  Consider removing such logic, potentially as part of
 // introducing a cross-platform SigninManager.
 signin_metrics::ProfileSignout kAlwaysAllowedSignoutSources[] = {
@@ -142,6 +137,14 @@ signin_metrics::ProfileSignout kAlwaysAllowedSignoutSources[] = {
     // to the web only before showing the sync confirmation dialog. The account
     // was signed in to the profile in order to show the sync confirmation.
     signin_metrics::ProfileSignout::kCancelSyncConfirmationOnWebOnlySignedIn,
+    // Allowed as the user wasn't signed in initially and data has not been
+    // synced yet.
+    signin_metrics::ProfileSignout::kCancelSyncConfirmationRemoveAccount,
+    // Data not synced yet.
+    // Used when moving the primary account (e.g. profile switch).
+    signin_metrics::ProfileSignout::kMovePrimaryAccount,
+    // Allowed as the profile is being deleted anyway.
+    signin_metrics::ProfileSignout::kSignoutDuringProfileDeletion,
 };
 
 // Returns the histogram suffix name per group of `signin_metrics::AccessPoint`.
@@ -264,6 +267,10 @@ network::mojom::CookieManager* ChromeSigninClient::GetCookieManager() {
       ->GetCookieManagerForBrowserProcess();
 }
 
+network::mojom::NetworkContext* ChromeSigninClient::GetNetworkContext() {
+  return profile_->GetDefaultStoragePartition()->GetNetworkContext();
+}
+
 bool ChromeSigninClient::AreSigninCookiesAllowed() {
   return ProfileAllowsSigninCookies(profile_);
 }
@@ -322,7 +329,7 @@ void ChromeSigninClient::PreSignOut(
   // `signin_metrics::ProfileSignout::kRevokeSyncFromSettings` when the user
   // turns off sync from the settings, we should also keep the window open at
   // this point.
-  // TODO(https://crbug.com/1478102): Check for managed accounts to be modified
+  // TODO(crbug.com/40280466): Check for managed accounts to be modified
   // when aligning Managed vs Consumer accounts.
   bool user_declines_sync_after_consenting_to_management =
       (signout_source_metric == signin_metrics::ProfileSignout::kAbortSignin ||
@@ -330,7 +337,7 @@ void ChromeSigninClient::PreSignOut(
            signin_metrics::ProfileSignout::kRevokeSyncFromSettings ||
        signout_source_metric == signin_metrics::ProfileSignout::
                                     kCancelSyncConfirmationOnWebOnlySignedIn) &&
-      chrome::enterprise_util::UserAcceptedAccountManagement(profile_);
+      enterprise_util::UserAcceptedAccountManagement(profile_);
   // These sign out won't remove the policy cache, keep the window opened.
   bool keep_window_opened =
       signout_source_metric ==
@@ -379,7 +386,7 @@ std::unique_ptr<GaiaAuthFetcher> ChromeSigninClient::CreateGaiaAuthFetcher(
   if (BoundSessionCookieRefreshService* bound_session_cookie_refresh_service =
           BoundSessionCookieRefreshServiceFactory::GetForProfile(profile_);
       bound_session_cookie_refresh_service) {
-    CHECK(switches::IsBoundSessionCredentialsEnabled());
+    CHECK(switches::IsBoundSessionCredentialsEnabled(profile_->GetPrefs()));
     return std::make_unique<ThrottledGaiaAuthFetcher>(
         consumer, source, GetURLLoaderFactory(),
         bound_session_cookie_refresh_service->GetBoundSessionThrottlerParams(),
@@ -399,10 +406,8 @@ version_info::Channel ChromeSigninClient::GetClientChannel() {
 #endif
 }
 
-void ChromeSigninClient::OnPrimaryAccountChangedWithEventSource(
-    signin::PrimaryAccountChangeEvent event_details,
-    absl::variant<signin_metrics::AccessPoint, signin_metrics::ProfileSignout>
-        event_source) {
+void ChromeSigninClient::OnPrimaryAccountChanged(
+    signin::PrimaryAccountChangeEvent event_details) {
   for (signin::ConsentLevel consent_level :
        {signin::ConsentLevel::kSignin, signin::ConsentLevel::kSync}) {
     switch (event_details.GetEventTypeFor(consent_level)) {
@@ -410,10 +415,9 @@ void ChromeSigninClient::OnPrimaryAccountChangedWithEventSource(
       case signin::PrimaryAccountChangeEvent::Type::kCleared:
         break;
       case signin::PrimaryAccountChangeEvent::Type::kSet:
-        CHECK(
-            absl::holds_alternative<signin_metrics::AccessPoint>(event_source));
+        CHECK(event_details.GetSetPrimaryAccountAccessPoint().has_value());
         signin_metrics::AccessPoint access_point =
-            absl::get<signin_metrics::AccessPoint>(event_source);
+            event_details.GetSetPrimaryAccountAccessPoint().value();
 
         // Only record metrics when setting the primary account.
         std::optional<size_t> all_bookmarks_count = GetAllBookmarksCount();
@@ -436,10 +440,23 @@ void ChromeSigninClient::OnPrimaryAccountChangedWithEventSource(
   }
 }
 
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+std::unique_ptr<signin::BoundSessionOAuthMultiLoginDelegate>
+ChromeSigninClient::CreateBoundSessionOAuthMultiloginDelegate() const {
+  if (BoundSessionCookieRefreshService* bound_session_cookie_refresh_service =
+          BoundSessionCookieRefreshServiceFactory::GetForProfile(profile_);
+      bound_session_cookie_refresh_service) {
+    return std::make_unique<BoundSessionOAuthMultiLoginDelegateImpl>(
+        bound_session_cookie_refresh_service->GetWeakPtr());
+  }
+  return nullptr;
+}
+#endif
+
 SigninClient::SignoutDecision ChromeSigninClient::GetSignoutDecision(
     bool has_sync_account,
     const std::optional<signin_metrics::ProfileSignout> signout_source) const {
-  // TODO(crbug.com/1366360): Revisit |kAlwaysAllowedSignoutSources| in general
+  // TODO(crbug.com/40239707): Revisit |kAlwaysAllowedSignoutSources| in general
   // and for Lacros main profile.
   for (const auto& always_allowed_source : kAlwaysAllowedSignoutSources) {
     if (!signout_source.has_value()) {
@@ -471,9 +488,9 @@ SigninClient::SignoutDecision ChromeSigninClient::GetSignoutDecision(
   }
 #endif
 
-#if !BUILDFLAG(IS_QTWEBENGINE)
+#if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_QTWEBENGINE)
   // Check if managed user.
-  if (chrome::enterprise_util::UserAcceptedAccountManagement(profile_)) {
+  if (enterprise_util::UserAcceptedAccountManagement(profile_)) {
     if (base::FeatureList::IsEnabled(kDisallowManagedProfileSignout)) {
       // Allow revoke sync but disallow signout regardless of consent level of
       // the primary account.
