@@ -4,32 +4,38 @@
 
 #include "chrome/browser/signin/dice_response_handler.h"
 
+#include <string_view>
+
 #include "base/feature_list.h"
 #include "base/functional/bind.h"
 #include "base/functional/callback.h"
 #include "base/location.h"
 #include "base/logging.h"
+#include "base/memory/raw_ref.h"
 #include "base/memory/singleton.h"
-#include "base/metrics/histogram_macros.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/strings/stringprintf.h"
 #include "base/task/sequenced_task_runner.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/time/time.h"
+#include "base/types/expected.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/profiles/profile_keyed_service_factory.h"
 #include "chrome/browser/signin/about_signin_internals_factory.h"
 #include "chrome/browser/signin/account_reconcilor_factory.h"
 #include "chrome/browser/signin/chrome_signin_client_factory.h"
 #include "chrome/browser/signin/identity_manager_factory.h"
-#include "chrome/browser/signin/signin_features.h"
 #include "components/signin/core/browser/about_signin_internals.h"
 #include "components/signin/core/browser/signin_header_helper.h"
 #include "components/signin/public/base/consent_level.h"
+#include "components/signin/public/base/session_binding_utils.h"
 #include "components/signin/public/base/signin_buildflags.h"
 #include "components/signin/public/base/signin_client.h"
 #include "components/signin/public/base/signin_metrics.h"
+#include "components/signin/public/base/signin_switches.h"
 #include "components/signin/public/identity_manager/accounts_mutator.h"
 #include "components/signin/public/identity_manager/identity_manager.h"
+#include "components/signin/public/identity_manager/identity_utils.h"
 #include "google_apis/gaia/gaia_auth_fetcher.h"
 #include "google_apis/gaia/gaia_auth_util.h"
 #include "google_apis/gaia/google_service_auth_error.h"
@@ -37,12 +43,15 @@
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 #include <optional>
 
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/signin/bound_session_credentials/registration_token_helper.h"  // nogncheck
 #include "chrome/browser/signin/bound_session_credentials/unexportable_key_service_factory.h"  // nogncheck
+#include "components/embedder_support/user_agent_utils.h"
 #include "components/signin/public/base/signin_switches.h"
 #include "components/unexportable_keys/unexportable_key_id.h"       // nogncheck
 #include "components/unexportable_keys/unexportable_key_service.h"  // nogncheck
 #include "google_apis/gaia/gaia_urls.h"
+#include "third_party/blink/public/common/user_agent/user_agent_metadata.h"
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 
 const int kDiceTokenFetchTimeoutSeconds = 10;
@@ -55,6 +64,8 @@ namespace {
 // The UMA histograms that logs events related to Dice responses.
 const char kDiceResponseHeaderHistogram[] = "Signin.DiceResponseHeader";
 const char kDiceTokenFetchResultHistogram[] = "Signin.DiceTokenFetchResult";
+const char kDiceTokenBindingOutcomeHistogram[] =
+    "Signin.DiceTokenBindingOutcome";
 
 // Used for UMA. Do not reorder, append new values at the end.
 enum DiceResponseHeader {
@@ -88,24 +99,20 @@ enum DiceTokenFetchResult {
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 std::unique_ptr<RegistrationTokenHelper> BuildRegistrationTokenHelper(
     unexportable_keys::UnexportableKeyService& unexportable_key_service,
-    base::StringPiece client_id,
-    base::StringPiece auth_code,
-    const GURL& registration_url,
-    base::OnceCallback<void(std::optional<RegistrationTokenHelper::Result>)>
-        callback) {
-  return RegistrationTokenHelper::CreateForTokenBinding(
-      unexportable_key_service, client_id, auth_code, registration_url,
-      std::move(callback));
+    RegistrationTokenHelper::KeyInitParam key_init_param) {
+  return std::make_unique<RegistrationTokenHelper>(unexportable_key_service,
+                                                   std::move(key_init_param));
 }
 
 DiceResponseHandler::RegistrationTokenHelperFactory
 CreateRegistrationTokenHelperFactory(
+    const PrefService* profile_prefs,
     unexportable_keys::UnexportableKeyService* unexportable_key_service) {
   if (!unexportable_key_service) {
     return {};
   }
 
-  if (!switches::IsChromeRefreshTokenBindingEnabled()) {
+  if (!switches::IsChromeRefreshTokenBindingEnabled(profile_prefs)) {
     return {};
   }
 
@@ -113,6 +120,27 @@ CreateRegistrationTokenHelperFactory(
   return base::BindRepeating(&BuildRegistrationTokenHelper,
                              std::ref(*unexportable_key_service));
 }
+
+// If any of existing accounts is already bound, returns its binding key.
+// Otherwise, returns an empty key indicating that a new key needs to be
+// generated.
+std::vector<uint8_t> GetWrappedBindingKeyToReuse(
+    const signin::IdentityManager& identity_manager) {
+  std::vector<CoreAccountInfo> accounts =
+      identity_manager.GetAccountsWithRefreshTokens();
+  for (const auto& account : accounts) {
+    std::vector<uint8_t> account_binding_key =
+        identity_manager.GetWrappedBindingKeyOfRefreshTokenForAccount(
+            account.account_id);
+    if (!account_binding_key.empty()) {
+      // All bound tokens are supposed to use the same key, so return the first
+      // non-empty key. Having two different keys should be considered a bug.
+      return account_binding_key;
+    }
+  }
+
+  return {};
+}
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 
 class DiceResponseHandlerFactory : public ProfileKeyedServiceFactory {
@@ -151,6 +179,7 @@ class DiceResponseHandlerFactory : public ProfileKeyedServiceFactory {
         registration_token_helper_factory;
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
     registration_token_helper_factory = CreateRegistrationTokenHelperFactory(
+        profile->GetPrefs(),
         UnexportableKeyServiceFactory::GetForProfile(profile));
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
     return new DiceResponseHandler(
@@ -162,17 +191,14 @@ class DiceResponseHandlerFactory : public ProfileKeyedServiceFactory {
   }
 };
 
-// Histogram macros expand to a lot of code, so it is better to wrap them in
-// functions.
-
 void RecordDiceResponseHeader(DiceResponseHeader header) {
-  UMA_HISTOGRAM_ENUMERATION(kDiceResponseHeaderHistogram, header,
-                            kDiceResponseHeaderCount);
+  base::UmaHistogramEnumeration(kDiceResponseHeaderHistogram, header,
+                                kDiceResponseHeaderCount);
 }
 
 void RecordDiceFetchTokenResult(DiceTokenFetchResult result) {
-  UMA_HISTOGRAM_ENUMERATION(kDiceTokenFetchResultHistogram, result,
-                            kDiceTokenFetchResultCount);
+  base::UmaHistogramEnumeration(kDiceTokenFetchResultHistogram, result,
+                                kDiceTokenFetchResultCount);
 }
 
 }  // namespace
@@ -188,7 +214,10 @@ DiceResponseHandler::DiceTokenFetcher::DiceTokenFetcher(
     SigninClient* signin_client,
     AccountReconcilor* account_reconcilor,
     std::unique_ptr<ProcessDiceHeaderDelegate> delegate,
-    const RegistrationTokenHelperFactory& registration_token_helper_factory,
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+    base::expected<raw_ref<RegistrationTokenHelper>, TokenBindingOutcome>
+        registration_token_helper_or_error,
+#endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
     DiceResponseHandler* dice_response_handler)
     : gaia_id_(gaia_id),
       email_(email),
@@ -204,11 +233,12 @@ DiceResponseHandler::DiceTokenFetcher::DiceTokenFetcher(
   account_reconcilor_lock_ =
       std::make_unique<AccountReconcilor::Lock>(account_reconcilor);
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
-  if (!registration_token_helper_factory.is_null()) {
-    CHECK(switches::IsChromeRefreshTokenBindingEnabled());
-    StartBindingKeyGeneration(registration_token_helper_factory);
+  if (registration_token_helper_or_error.has_value()) {
+    StartBindingKeyGeneration(registration_token_helper_or_error->get());
     // Wait until the binding key is generated before fetching a token.
     return;
+  } else {
+    token_binding_outcome_ = registration_token_helper_or_error.error();
   }
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
   StartTokenFetch();
@@ -231,12 +261,18 @@ void DiceResponseHandler::DiceTokenFetcher::OnClientOAuthSuccess(
   gaia_auth_fetcher_.reset();
   timeout_closure_.Cancel();
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
-  if (!switches::IsChromeRefreshTokenBindingEnabled() ||
-      !result.is_bound_to_key) {
-    // Pass an empty binding key if conditions don't apply. This key won't be
-    // needed for anything else, so we can just clear it in place.
-    wrapped_binding_key_.clear();
+  if (!wrapped_binding_key_.empty()) {
+    CHECK(switches::IsChromeRefreshTokenBindingEnabled(
+        signin_client_->GetPrefs()));
+    if (!result.is_bound_to_key) {
+      wrapped_binding_key_.clear();
+      token_binding_outcome_ = TokenBindingOutcome::kNotBoundServerRejectedKey;
+    } else {
+      token_binding_outcome_ = TokenBindingOutcome::kBound;
+    }
   }
+  base::UmaHistogramEnumeration(kDiceTokenBindingOutcomeHistogram,
+                                token_binding_outcome_);
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
   dice_response_handler_->OnTokenExchangeSuccess(
       this, result.refresh_token, result.is_under_advanced_protection
@@ -265,7 +301,10 @@ void DiceResponseHandler::DiceTokenFetcher::StartTokenFetch() {
   // `binding_registration_token_` is empty if the binding key was not
   // generated.
   gaia_auth_fetcher_->StartAuthCodeForOAuth2TokenExchange(
-      authorization_code_, binding_registration_token_);
+      authorization_code_,
+      embedder_support::GetUserAgentMetadata(g_browser_process->local_state())
+          .SerializeBrandFullVersionList(),
+      binding_registration_token_);
 #else
   gaia_auth_fetcher_->StartAuthCodeForOAuth2TokenExchange(authorization_code_);
 #endif
@@ -276,26 +315,29 @@ void DiceResponseHandler::DiceTokenFetcher::StartTokenFetch() {
 
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 void DiceResponseHandler::DiceTokenFetcher::StartBindingKeyGeneration(
-    const RegistrationTokenHelperFactory& registration_token_helper_factory) {
-  CHECK(switches::IsChromeRefreshTokenBindingEnabled());
-  // `base::Unretained()` is safe because `this` owns
-  // `registration_token_helper_`.
-  registration_token_helper_ = registration_token_helper_factory.Run(
+    RegistrationTokenHelper& registration_token_helper) {
+  CHECK(
+      switches::IsChromeRefreshTokenBindingEnabled(signin_client_->GetPrefs()));
+  // `base::Unretained()` is safe because `DiceResponseHandler` guarantees that
+  // `registration_token_helper` outlives `this`.
+  registration_token_helper.GenerateForTokenBinding(
       GaiaUrls::GetInstance()->oauth2_chrome_client_id(), authorization_code_,
-      GaiaUrls::GetInstance()->oauth2_token_url(),
+      GURL("https://accounts.google.com/accountmanager"),
       base::BindOnce(&DiceTokenFetcher::OnRegistrationTokenGenerated,
                      base::Unretained(this)));
-  registration_token_helper_->Start();
 }
 
 void DiceResponseHandler::DiceTokenFetcher::OnRegistrationTokenGenerated(
     std::optional<RegistrationTokenHelper::Result> result) {
-  CHECK(switches::IsChromeRefreshTokenBindingEnabled());
+  CHECK(
+      switches::IsChromeRefreshTokenBindingEnabled(signin_client_->GetPrefs()));
   if (result.has_value()) {
     binding_registration_token_ = std::move(result->registration_token);
     wrapped_binding_key_ = std::move(result->wrapped_binding_key);
+  } else {
+    token_binding_outcome_ =
+        TokenBindingOutcome::kNotBoundRegistrationTokenGenerationFailed;
   }
-  registration_token_helper_.reset();
   StartTokenFetch();
 }
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
@@ -339,7 +381,9 @@ void DiceResponseHandler::ProcessDiceHeader(
           dice_params.signin_info->account_info;
       ProcessDiceSigninHeader(
           info.gaia_id, info.email, dice_params.signin_info->authorization_code,
-          dice_params.signin_info->no_authorization_code, std::move(delegate));
+          dice_params.signin_info->no_authorization_code,
+          dice_params.signin_info->supported_algorithms_for_token_binding,
+          std::move(delegate));
       return;
     }
     case signin::DiceAction::ENABLE_SYNC: {
@@ -353,10 +397,10 @@ void DiceResponseHandler::ProcessDiceHeader(
       ProcessDiceSignoutHeader(dice_params.signout_info->account_infos);
       return;
     case signin::DiceAction::NONE:
-      NOTREACHED() << "Invalid Dice response parameters.";
+      NOTREACHED_IN_MIGRATION() << "Invalid Dice response parameters.";
       return;
   }
-  NOTREACHED();
+  NOTREACHED_IN_MIGRATION();
 }
 
 size_t DiceResponseHandler::GetPendingDiceTokenFetchersCountForTesting() const {
@@ -375,7 +419,8 @@ void DiceResponseHandler::SetTaskRunner(
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 void DiceResponseHandler::SetRegistrationTokenHelperFactoryForTesting(
     RegistrationTokenHelperFactory factory) {
-  CHECK(switches::IsChromeRefreshTokenBindingEnabled());
+  CHECK(
+      switches::IsChromeRefreshTokenBindingEnabled(signin_client_->GetPrefs()));
   registration_token_helper_factory_ = std::move(factory);
 }
 #endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
@@ -385,6 +430,7 @@ void DiceResponseHandler::ProcessDiceSigninHeader(
     const std::string& email,
     const std::string& authorization_code,
     bool no_authorization_code,
+    const std::string& supported_algorithms_for_token_binding,
     std::unique_ptr<ProcessDiceHeaderDelegate> delegate) {
   if (no_authorization_code) {
     lock_ = std::make_unique<AccountReconcilor::Lock>(account_reconcilor_);
@@ -392,8 +438,9 @@ void DiceResponseHandler::ProcessDiceSigninHeader(
         "Missing authorization code due to OAuth outage in Dice.");
     if (!timer_) {
       timer_ = std::make_unique<base::OneShotTimer>();
-      if (task_runner_)
+      if (task_runner_) {
         timer_->SetTaskRunner(task_runner_);
+      }
     }
     // If there is already another lock, the timer will be reset and
     // we'll wait another full timeout.
@@ -420,9 +467,31 @@ void DiceResponseHandler::ProcessDiceSigninHeader(
     }
   }
 
+  if (base::FeatureList::IsEnabled(
+          ::switches::kPreconnectAccountCapabilitiesPostSignin)) {
+    // The user is signing in, which means that account fetching will shortly be
+    // triggered.
+    //
+    // Notify identity manager. This will trigger pre-connecting the network
+    // socket to the AccountCapabilities endpoint, in parallel with the LST and
+    // access token requests (instead of waiting for these to complete).
+    identity_manager_->PrepareForAddingNewAccount();
+  }
+
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+  base::expected<raw_ref<RegistrationTokenHelper>, TokenBindingOutcome>
+      registration_token_helper_or_error =
+          MaybeGetBindingRegistrationTokenHelper(
+              supported_algorithms_for_token_binding);
+#endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+
   token_fetchers_.push_back(std::make_unique<DiceTokenFetcher>(
       gaia_id, email, authorization_code, signin_client_, account_reconcilor_,
-      std::move(delegate), registration_token_helper_factory_, this));
+      std::move(delegate),
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+      registration_token_helper_or_error,
+#endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+      this));
 }
 
 void DiceResponseHandler::ProcessEnableSyncHeader(
@@ -450,23 +519,28 @@ void DiceResponseHandler::ProcessDiceSignoutHeader(
     const std::vector<signin::DiceResponseParams::AccountInfo>& account_infos) {
   VLOG(1) << "Start processing Dice signout response";
 
-  // If there is a restriction on removing the primary account. Do not remove
-  // the account regardless of the consent level. Else, the sync account can
-  // only be invalidated.
-  signin::ConsentLevel level =
-      signin_client_->IsClearPrimaryAccountAllowed(
-          identity_manager_->HasPrimaryAccount(signin::ConsentLevel::kSync))
-          ? signin::ConsentLevel::kSync
-          : signin::ConsentLevel::kSignin;
+  // In some cases, the primary account can only be invalidated:
+  // - There is a sync primary account
+  // - Browser explicit sign in is enabled, setting/clearing the primary account
+  //   requires explicit user action through chrome UI.
+  // - If there is a policy restriction on removing the primary account.
+  bool invalidate_only_primary_account =
+      identity_manager_->HasPrimaryAccount(signin::ConsentLevel::kSync) ||
+      !signin::IsImplicitBrowserSigninOrExplicitDisabled(
+          identity_manager_, signin_client_->GetPrefs()) ||
+      !signin_client_->IsClearPrimaryAccountAllowed(
+          /*has_sync_account=*/false);
 
-  CoreAccountId primary_account = identity_manager_->GetPrimaryAccountId(level);
+  CoreAccountId primary_account =
+      identity_manager_->GetPrimaryAccountId(signin::ConsentLevel::kSignin);
   bool primary_account_signed_out = false;
   auto* accounts_mutator = identity_manager_->GetAccountsMutator();
   for (const auto& account_info : account_infos) {
     CoreAccountId signed_out_account =
         identity_manager_->PickAccountIdForAccount(account_info.gaia_id,
                                                    account_info.email);
-    if (signed_out_account == primary_account) {
+    if (invalidate_only_primary_account &&
+        signed_out_account == primary_account) {
       primary_account_signed_out = true;
       RecordDiceResponseHeader(kSignoutPrimary);
 
@@ -492,18 +566,29 @@ void DiceResponseHandler::ProcessDiceSignoutHeader(
     }
   }
 
-  if (!primary_account_signed_out)
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+  if (token_fetchers_.empty()) {
+    registration_token_helper_.reset();
+  }
+#endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+
+  if (!primary_account_signed_out) {
     RecordDiceResponseHeader(kSignoutSecondary);
+  }
 }
 
 void DiceResponseHandler::DeleteTokenFetcher(DiceTokenFetcher* token_fetcher) {
-  for (auto it = token_fetchers_.begin(); it != token_fetchers_.end(); ++it) {
-    if (it->get() == token_fetcher) {
-      token_fetchers_.erase(it);
-      return;
-    }
+  size_t delete_count =
+      std::erase_if(token_fetchers_, [token_fetcher](const auto& current) {
+        return current.get() == token_fetcher;
+      });
+  CHECK_EQ(delete_count, 1U);
+
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+  if (token_fetchers_.empty()) {
+    registration_token_helper_.reset();
   }
-  NOTREACHED();
+#endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 }
 
 void DiceResponseHandler::OnTokenExchangeSuccess(
@@ -523,11 +608,10 @@ void DiceResponseHandler::OnTokenExchangeSuccess(
       identity_manager_->PickAccountIdForAccount(gaia_id, email);
   bool is_new_account =
       !identity_manager_->HasAccountWithRefreshToken(account_id);
-  // If this is a reauth, do not update the access point.
+
   identity_manager_->GetAccountsMutator()->AddOrUpdateAccount(
       gaia_id, email, refresh_token, is_under_advanced_protection,
-      is_new_account ? token_fetcher->delegate()->GetAccessPoint()
-                     : signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN,
+      token_fetcher->delegate()->GetAccessPoint(),
       signin_metrics::SourceForRefreshTokenOperation::
           kDiceResponseHandler_Signin
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
@@ -561,6 +645,45 @@ void DiceResponseHandler::OnTokenExchangeFailure(
   DeleteTokenFetcher(token_fetcher);
 }
 
+#if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+base::expected<raw_ref<RegistrationTokenHelper>,
+               DiceResponseHandler::TokenBindingOutcome>
+DiceResponseHandler::MaybeGetBindingRegistrationTokenHelper(
+    std::string_view supported_algorithms) {
+  if (registration_token_helper_factory_.is_null()) {
+    return base::unexpected(TokenBindingOutcome::kNotBoundNotSupported);
+  }
+
+  if (supported_algorithms.empty()) {
+    return base::unexpected(TokenBindingOutcome::kNotBoundNotEligible);
+  }
+
+  CHECK(
+      switches::IsChromeRefreshTokenBindingEnabled(signin_client_->GetPrefs()));
+
+  // If `registration_token_helper_` doesn't exist, create it.
+  if (!registration_token_helper_) {
+    std::vector<uint8_t> wrapped_binding_key_to_reuse =
+        GetWrappedBindingKeyToReuse(*identity_manager_);
+    if (!wrapped_binding_key_to_reuse.empty()) {
+      // Ignore the value of `supported_algorithms` in favor of an existing
+      // binding key.
+      registration_token_helper_ = registration_token_helper_factory_.Run(
+          std::move(wrapped_binding_key_to_reuse));
+    } else {
+      registration_token_helper_ = registration_token_helper_factory_.Run(
+          signin::ParseSignatureAlgorithmList(supported_algorithms));
+    }
+  }
+
+  // If `registration_token_helper_` was reused, its supported algorithm
+  // list may mismatch `supported_algorithms`. We ignore this because it's more
+  // important to reuse the same key.
+  CHECK(registration_token_helper_);
+  return raw_ref<RegistrationTokenHelper>(*registration_token_helper_);
+}
+#endif  // BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
+
 // static
 void DiceResponseHandler::EnsureFactoryBuilt() {
   DiceResponseHandlerFactory::GetInstance();
