@@ -22,6 +22,7 @@
 #include "components/policy/core/browser/signin/profile_separation_policies.h"
 #include "components/search_engines/template_url_data.h"
 #include "components/signin/public/base/signin_metrics.h"
+#include "components/signin/public/base/signin_prefs.h"
 #include "components/signin/public/identity_manager/identity_manager.h"
 #include "google_apis/gaia/core_account_id.h"
 #include "third_party/skia/include/core/SkColor.h"
@@ -59,15 +60,29 @@ enum class ShouldShowChromeSigninBubbleWithReason {
 
   // The bubble should not be shown: multiple reasons listed below with order of
   // priority.
-  kShouldNotShowMaxShownCountReached = 1,
+  // Deprecated: kShouldNotShowMaxShownCountReached = 1,
   kShouldNotShowAlreadySignedIn = 2,
-  kShouldNotShowSecondaryAccount = 3,
+  // Deprecated: kShouldNotShowSecondaryAccount = 3,
   kShouldNotShowUnknownAccessPoint = 4,
   kShouldNotShowNotFromWebSignin = 5,
+  kShouldNotShowUserChoice = 6,
 
-  kMaxValue = kShouldNotShowNotFromWebSignin,
+  kMaxValue = kShouldNotShowUserChoice,
 };
 
+// Supervision state of the user who is shown the sign-in intercept bubble.
+// These values are logged to UMA. Entries should not be renumbered and
+// numeric values should never be reused.
+//
+// LINT.IfChange(SinginInterceptSupervisionState)
+enum class SinginInterceptSupervisionState {
+  kRegularUser = 0,
+  kSupervisedUser = 1,
+  kUnknownSupervision = 2,
+  kMaxValue = kUnknownSupervision,
+};
+// LINT.ThenChange(//tools/metrics/histograms/metadata/signin/enums.xml:SinginInterceptSupervisionState)
+
 // Called after web signed in, after a successful token exchange through Dice.
 // The DiceWebSigninInterceptor may offer the user to create a new profile or
 // switch to another existing profile.
@@ -133,11 +148,14 @@ class DiceWebSigninInterceptor : public KeyedService,
   // in |entry|.
   // In some cases the outcome cannot be fully computed synchronously, when this
   // happens, the signin interception is highly likely (but not guaranteed).
+  // `gaia_id` is optional as some usages may not have the information yet. It
+  // is currently only mandatory for the checks of the Chrome Signin bubble.
   std::optional<SigninInterceptionHeuristicOutcome> GetHeuristicOutcome(
       bool is_new_account,
       bool is_sync_signin,
       const std::string& email,
-      bool record_signin_metrics = false,
+      const std::string& gaia_id = std::string(),
+      bool update_state = false,
       const ProfileAttributesEntry** entry = nullptr) const;
 
   // Returns true if the interception is in progress (running the heuristic or
@@ -152,6 +170,10 @@ class DiceWebSigninInterceptor : public KeyedService,
         std::move(value);
   }
 
+  static base::TimeDelta GetTimeSinceLastChromeSigninDeclineForTesting(
+      const SigninPrefs& signin_prefs,
+      const std::string& gaia_id);
+
   // KeyedService:
   void Shutdown() override;
 
@@ -168,6 +190,8 @@ class DiceWebSigninInterceptor : public KeyedService,
                            ShouldShowEnterpriseBubbleWithoutUPA);
   FRIEND_TEST_ALL_PREFIXES(DiceWebSigninInterceptorTest,
                            ShouldShowMultiUserBubble);
+  FRIEND_TEST_ALL_PREFIXES(DiceWebSigninInterceptorTest,
+                           ShouldShowMultiUserBubbleNoPrimaryAccount);
   FRIEND_TEST_ALL_PREFIXES(DiceWebSigninInterceptorTest, PersistentHash);
   FRIEND_TEST_ALL_PREFIXES(DiceWebSigninInterceptorTest,
                            ShouldEnforceEnterpriseProfileSeparation);
@@ -221,6 +245,7 @@ class DiceWebSigninInterceptor : public KeyedService,
       const AccountInfo& intercepted_account_info) const;
   bool ShouldShowMultiUserBubble(
       const AccountInfo& intercepted_account_info) const;
+  bool ShouldShowChromeSigninBubble(const std::string& gaia_id);
 
   // Helper function to call `delegate_->ShowSigninInterceptionBubble()`.
   void ShowSigninInterceptionBubble(
@@ -257,6 +282,15 @@ class DiceWebSigninInterceptor : public KeyedService,
   void OnChromeSigninChoice(const AccountInfo& account_info,
                             SigninInterceptionResult result);
 
+  // Processes the intercept result:
+  // - counting dismissals, after 5 dismissals the result is transformed to a
+  // decline.
+  // - saving the accept/decline result in a pref.
+  // - returns the processed result.
+  SigninInterceptionResult ProcessChromeSigninUserChoice(
+      SigninInterceptionResult result,
+      const std::string& gaia_id);
+
   // A non `std::nullopt` `profile_presets` will be applied to the
   // `new_profile` when the function is called.
   void OnNewSignedInProfileCreated(
@@ -285,22 +319,22 @@ class DiceWebSigninInterceptor : public KeyedService,
   // Only a hash of the email is saved, as Chrome does not need to store the
   // actual email, but only need to compare emails. The hash has low entropy to
   // ensure it cannot be reversed.
-  void IncrementEmailToCountDictionaryPref(const char* pref_name,
-                                           const std::string& email);
+  // Returns the incremented value of the pref.
+  size_t IncrementEmailToCountDictionaryPref(const char* pref_name,
+                                             const std::string& email);
 
-  // Records the number of times the user previously declined the Chrome Signin
-  // bubble when accepting it. Also resets the value in the prefs.
-  void RecordAndResetChromeSigninNumberOfAttemptsBeforeAccept(
-      const std::string& email);
+  // Records the number of times the user previously dismissed the Chrome Signin
+  // bubble when accepting/declining it. Result is expected to be either
+  // `SigninInterceptionResult::kAccepted` or
+  // `SigninInterceptionResult::kDeclined`.
+  void RecordChromeSigninNumberOfDismissesForAccount(
+      const std::string& gaia_id,
+      SigninInterceptionResult result);
 
   // Checks if the user previously declined 2 times creating a new profile for
   // this account.
   bool HasUserDeclinedProfileCreation(const std::string& email) const;
 
-  // Returns the number of times the Chrome Signin Bubble was shown per `email`.
-  // The value is stored in a profile pref.
-  size_t GetChromeSigninBubbleShownCount(const std::string& email) const;
-
   // Fetches the value of the cloud user level value of the
   // ManagedAccountsSigninRestriction policy for 'account_info' and runs
   // `callback` with the result. This is a network call that has a 5 seconds
@@ -349,6 +383,8 @@ class DiceWebSigninInterceptor : public KeyedService,
         interception_type_;
     signin_metrics::AccessPoint access_point_ =
         signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN;
+    std::optional<ShouldShowChromeSigninBubbleWithReason>
+        should_show_chrome_signin_bubble_;
 
     // Timeout for waiting for full information to be available (see
     // `ProcessInterceptionOrWait()`).
