@@ -3,12 +3,15 @@
 // found in the LICENSE file.
 #include "chrome/browser/signin/signin_manager.h"
 
+#include <tuple>
+
 #include "base/memory/raw_ptr.h"
 #include "base/scoped_observation.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_feature_list.h"
 #include "build/buildflag.h"
 #include "build/chromeos_buildflags.h"
+#include "components/signin/public/base/consent_level.h"
 #include "components/signin/public/base/signin_pref_names.h"
 #include "components/signin/public/base/signin_switches.h"
 #include "components/signin/public/base/test_signin_client.h"
@@ -18,8 +21,6 @@
 #include "components/signin/public/identity_manager/identity_test_environment.h"
 #include "components/signin/public/identity_manager/identity_test_utils.h"
 #include "components/signin/public/identity_manager/primary_account_mutator.h"
-#include "components/supervised_user/core/common/buildflags.h"
-#include "components/supervised_user/core/common/features.h"
 #include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "content/public/test/browser_task_environment.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -65,10 +66,20 @@ class FakeIdentityManagerObserver : public IdentityManager::Observer {
       identity_manager_observation_{this};
 };
 
+struct SigninManagerTestParams {
+  using TupleType = std::tuple<bool, bool>;
+  bool explicit_browser_signin;
+  bool is_signout_allowed;
+  explicit SigninManagerTestParams(TupleType params)
+      : explicit_browser_signin(std::get<0>(params)),
+        is_signout_allowed(std::get<1>(params)) {}
+};
+
 }  // namespace
 
-class SigninManagerTest : public testing::Test,
-                          public ::testing::WithParamInterface<bool> {
+class SigninManagerTest
+    : public testing::Test,
+      public ::testing::WithParamInterface<SigninManagerTestParams> {
  public:
   SigninManagerTest()
       : client_(&prefs_),
@@ -76,7 +87,13 @@ class SigninManagerTest : public testing::Test,
                            /*pref_service=*/&prefs_,
                            &client_),
         observer_(identity_test_env_.identity_manager()) {
-    scoped_feature_list_.InitAndDisableFeature(switches::kUnoDesktop);
+    if (explicit_browser_signin()) {
+      scoped_feature_list_.InitAndEnableFeature(
+          switches::kExplicitBrowserSigninUIOnDesktop);
+    } else {
+      scoped_feature_list_.InitAndDisableFeature(
+          switches::kExplicitBrowserSigninUIOnDesktop);
+    }
     RecreateSigninManager();
   }
 
@@ -96,7 +113,42 @@ class SigninManagerTest : public testing::Test,
     }
   }
 
-  bool is_signout_allowed() const { return GetParam(); }
+  bool is_signout_allowed() const { return GetParam().is_signout_allowed; }
+
+  bool explicit_browser_signin() {
+    // The `SigninManager` will not automatically set the primary account.
+    // The account will be updated only if it was implicitly sign in.
+    // In practice, this can only happen if the user was signed in prior to the
+    // feature being enabled.
+    return GetParam().explicit_browser_signin;
+  }
+
+  void Signin(const std::string& email,
+              signin_metrics::AccessPoint access_point,
+              ConsentLevel consent_level) {
+    identity_test_env()->MakeAccountAvailable(
+        identity_test_env()
+            ->CreateAccountAvailabilityOptionsBuilder()
+            .WithAccessPoint(access_point)
+            .AsPrimary(consent_level)
+            .Build(email));
+    CHECK(identity_manager()->HasPrimaryAccount(consent_level));
+  }
+
+  void SigninImplicitlyWithAccount(
+      const std::string& email,
+      ConsentLevel consent_level = ConsentLevel::kSignin) {
+    Signin(email, signin_metrics::AccessPoint::ACCESS_POINT_WEB_SIGNIN,
+           consent_level);
+  }
+
+  void SigninExplicitlyWithAccount(const std::string& email) {
+    CHECK(base::FeatureList::IsEnabled(
+        switches::kExplicitBrowserSigninUIOnDesktop));
+    Signin(email,
+           signin_metrics::AccessPoint::ACCESS_POINT_AVATAR_BUBBLE_SIGN_IN,
+           ConsentLevel::kSignin);
+  }
 
   void ExpectUnconsentedPrimaryAccountSetEvent(
       const CoreAccountInfo& expected_primary_account) {
@@ -160,16 +212,15 @@ class SigninManagerTest : public testing::Test,
       const std::string& email,
       signin_metrics::AccessPoint access_point =
           signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN) {
-    AccountInfo account = identity_test_env_.MakeAccountAvailable(email);
-    EXPECT_FALSE(account.IsEmpty());
-    account.access_point = access_point;
-    identity_test_env_.UpdateAccountInfoForAccount(account);
+    AccountAvailabilityOptionsBuilder builder =
+        identity_test_env()
+            ->CreateAccountAvailabilityOptionsBuilder()
+            .WithAccessPoint(access_point);
+
+    AccountInfo account =
+        identity_test_env_.MakeAccountAvailable(builder.Build(email));
     signin::CookieParamsForTest cookie_params = {account.email, account.gaia};
     identity_test_env_.SetCookieAccounts({cookie_params});
-    EXPECT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
-    EXPECT_EQ(account,
-              identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
-    EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSync));
     return account;
   }
 
@@ -202,21 +253,33 @@ TEST_P(
     UnconsentedPrimaryAccountUpdatedOnItsAccountRefreshTokenUpdateWithValidTokenWhenNoSyncConsent) {
   // Add an unconsented primary account, incl. proper cookies.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
-  ExpectUnconsentedPrimaryAccountSetEvent(account);
-  EXPECT_EQ(account,
-            identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
+  if (explicit_browser_signin()) {
+    EXPECT_EQ(0U, observer().events().size());
+    EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
+  } else {
+    ExpectUnconsentedPrimaryAccountSetEvent(account);
+    EXPECT_EQ(account,
+              identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
+  }
 }
 
 TEST_P(
     SigninManagerTest,
-    UnconsentedPrimaryAccountUpdatedOnItsAccountRefreshTokenUpdateWithInvalidTokenWhenNoSyncConsent) {
+    UnconsentedPrimaryAccountUpdatedOnItsAccountRefreshTokenUpdateWithPersistentErrorWhenNoSyncConsent) {
   // Prerequisite: add an unconsented primary account, incl. proper cookies.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    SigninImplicitlyWithAccount(account.email);
+  }
   ExpectUnconsentedPrimaryAccountSetEvent(account);
   InitializeSignoutDecision();
 
   // Invalid token.
-  SetInvalidRefreshTokenForAccount(identity_manager(), account.account_id);
+  identity_test_env()->UpdatePersistentErrorOfRefreshTokenForAccount(
+      identity_manager()->GetPrimaryAccountId(ConsentLevel::kSignin),
+      GoogleServiceAuthError::FromInvalidGaiaCredentialsReason(
+          GoogleServiceAuthError::InvalidGaiaCredentialsReason::
+              CREDENTIALS_REJECTED_BY_CLIENT));
 
   if (is_signout_allowed()) {
 #if BUILDFLAG(IS_CHROMEOS_LACROS)
@@ -228,9 +291,16 @@ TEST_P(
     EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
     // Update with a valid token.
     SetRefreshTokenForAccount(identity_manager(), account.account_id, "");
-    ExpectUnconsentedPrimaryAccountSetEvent(account);
-    EXPECT_EQ(identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin),
-              account);
+    if (explicit_browser_signin()) {
+      EXPECT_EQ(0U, observer().events().size());
+      EXPECT_FALSE(
+          identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
+    } else {
+      ExpectUnconsentedPrimaryAccountSetEvent(account);
+      EXPECT_EQ(
+          identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin),
+          account);
+    }
 #endif
   } else {
     EXPECT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
@@ -239,11 +309,46 @@ TEST_P(
   }
 }
 
+TEST_P(
+    SigninManagerTest,
+    UnconsentedPrimaryAccountUpdatedOnItsAccountRefreshTokenUpdateWithInvalidTokenWhenNoSyncConsent) {
+  // Setting an invalid refresh token is only possible when signing out is not
+  // allowed (e.g. enterprise/kids accounts) or when ExplicitBrowserSignin is
+  // enabled with an explicit signed in account.
+  if (is_signout_allowed() && !explicit_browser_signin()) {
+    GTEST_SKIP();
+  }
+
+  // Prerequisite: add an unconsented primary account, incl. proper cookies.
+  AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    // When attempting to set an invalid refresh token, the account must be
+    // explicitly signed in. Implicitly signed in accounts will be removed and
+    // create an undesired flow.
+    SigninExplicitlyWithAccount(account.email);
+  }
+  ExpectUnconsentedPrimaryAccountSetEvent(account);
+  InitializeSignoutDecision();
+
+  // Invalid token.
+  SetInvalidRefreshTokenForAccount(identity_manager(), account.account_id);
+
+  EXPECT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
+  EXPECT_EQ(identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin),
+            account);
+  EXPECT_TRUE(
+      identity_manager()->HasAccountWithRefreshTokenInPersistentErrorState(
+          account.account_id));
+}
+
 TEST_P(
     SigninManagerTest,
     UnconsentedPrimaryAccountRemovedOnItsAccountRefreshTokenRemovalWhenNoSyncConsent) {
   // Prerequisite: Add an unconsented primary account, incl. proper cookies.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    SigninImplicitlyWithAccount(account.email);
+  }
   ExpectUnconsentedPrimaryAccountSetEvent(account);
   InitializeSignoutDecision();
 
@@ -261,6 +366,10 @@ TEST_P(
 TEST_P(SigninManagerTest, UnconsentedPrimaryAccountChangedBlockedByHandle) {
   // Prerequisite: Add an unconsented primary account, incl. proper cookies.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    // Set the primary account and mark it to be implicitly signed in.
+    SigninImplicitlyWithAccount(account.email);
+  }
   ExpectUnconsentedPrimaryAccountSetEvent(account);
   InitializeSignoutDecision();
 
@@ -320,6 +429,9 @@ TEST_P(SigninManagerTest,
        UnconsentedPrimaryAccountTokenRevokedWithStaleCookies) {
   // Prerequisite: add an unconsented primary account, incl. proper cookies.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    SigninImplicitlyWithAccount(account.email);
+  }
   ExpectUnconsentedPrimaryAccountSetEvent(account);
   InitializeSignoutDecision();
 
@@ -350,7 +462,9 @@ TEST_P(SigninManagerTest,
   identity_test_env()->SetCookieAccounts(
       {{main_account.email, main_account.gaia},
        {secondary_account.email, secondary_account.gaia}});
-
+  if (explicit_browser_signin()) {
+    SigninImplicitlyWithAccount(main_account.email);
+  }
   EXPECT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
   EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSync));
   EXPECT_EQ(main_account,
@@ -377,6 +491,10 @@ TEST_P(SigninManagerTest,
        SameUnconsentedPrimaryAccountTokenWithRemovedCookies) {
   // Prerequisite: add an unconsented primary account, incl. proper cookies.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    // Set the primary account and mark it to be implicitly signed in.
+    SigninImplicitlyWithAccount(account.email);
+  }
   ExpectUnconsentedPrimaryAccountSetEvent(account);
   InitializeSignoutDecision();
 
@@ -385,19 +503,6 @@ TEST_P(SigninManagerTest,
   EXPECT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
   EXPECT_EQ(0U, observer().events().size());
 }
-
-TEST_F(SigninManagerTest, UnconsentedPrimaryAccountRemovedCookiesEmpty) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitAndDisableFeature(kPreventSignoutIfAccountValid);
-  // Prerequisite: add an unconsented primary account, incl. proper cookies.
-  AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
-  ExpectUnconsentedPrimaryAccountSetEvent(account);
-
-  // Set Gaia accounts in the cookie to empty.
-  identity_test_env()->SetCookieAccounts({});
-  EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
-  ExpectUnconsentedPrimaryAccountClearedEvent(account);
-}
 #endif  // !BUILDFLAG(IS_CHROMEOS_LACROS)
 
 TEST_P(SigninManagerTest, UnconsentedPrimaryAccountDuringLoad) {
@@ -409,6 +514,10 @@ TEST_P(SigninManagerTest, UnconsentedPrimaryAccountDuringLoad) {
   identity_test_env()->SetCookieAccounts(
       {{main_account.email, main_account.gaia},
        {secondary_account.email, secondary_account.gaia}});
+  if (explicit_browser_signin()) {
+    // Set the primary account and mark it to be implicitly signed in.
+    SigninImplicitlyWithAccount(main_account.email);
+  }
   ASSERT_EQ(main_account,
             identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
   ASSERT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSync));
@@ -463,14 +572,17 @@ TEST_P(SigninManagerTest,
   identity_test_env()->SetCookieAccounts(
       {{first_account.email, first_account.gaia},
        {second_account.email, second_account.gaia}});
+  if (explicit_browser_signin()) {
+    SigninImplicitlyWithAccount(first_account.email);
+  }
   ASSERT_EQ(first_account,
             identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
   ExpectUnconsentedPrimaryAccountSetEvent(first_account);
 
   // Set the sync primary account to the second account in cookies.
   // The unconsented primary account should be updated.
-  identity_test_env()->SetPrimaryAccount(second_account.email,
-                                         signin::ConsentLevel::kSync);
+  SigninImplicitlyWithAccount(second_account.email,
+                              signin::ConsentLevel::kSync);
   InitializeSignoutDecision();
 
   EXPECT_EQ(second_account,
@@ -530,23 +642,32 @@ TEST_P(SigninManagerTest,
 
 TEST_P(SigninManagerTest, UnconsentedPrimaryAccountUpdatedOnHandleDestroyed) {
   base::HistogramTester histogram_tester;
+  AccountAvailabilityOptionsBuilder builder =
+      identity_test_env()
+          ->CreateAccountAvailabilityOptionsBuilder()
+          .WithAccessPoint(signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN);
   AccountInfo first_account =
-      identity_test_env()->MakeAccountAvailable(kTestEmail);
+      identity_test_env()->MakeAccountAvailable(builder.Build(kTestEmail));
   AccountInfo second_account =
-      identity_test_env()->MakeAccountAvailable(kTestEmail2);
+      identity_test_env()->MakeAccountAvailable(builder.Build(kTestEmail2));
   identity_test_env()->SetCookieAccounts(
       {{first_account.email, first_account.gaia},
        {second_account.email, second_account.gaia}});
+  signin_metrics::AccessPoint access_point =
+      signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER;
+  if (explicit_browser_signin()) {
+    SigninImplicitlyWithAccount(first_account.email);
+    access_point = signin_metrics::AccessPoint::ACCESS_POINT_WEB_SIGNIN;
+  }
   ASSERT_EQ(first_account,
             identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
   ExpectUnconsentedPrimaryAccountSetEvent(first_account);
-  histogram_tester.ExpectUniqueSample(
-      "Signin.SignIn.Completed",
-      signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER, 1);
-  histogram_tester.ExpectUniqueSample(
-      "Signin.SigninManager.SigninAccessPoint",
-      signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER, 1);
-
+  if (!explicit_browser_signin()) {
+    histogram_tester.ExpectUniqueSample("Signin.SignIn.Completed", access_point,
+                                        1);
+    histogram_tester.ExpectUniqueSample(
+        "Signin.SigninManager.SigninAccessPoint", access_point, 1);
+  }
   std::unique_ptr<AccountSelectionInProgressHandle> handle =
       signin_manager_->CreateAccountSelectionInProgressHandle();
   ASSERT_TRUE(handle);
@@ -554,17 +675,18 @@ TEST_P(SigninManagerTest, UnconsentedPrimaryAccountUpdatedOnHandleDestroyed) {
   // Set the primary account to the second account in cookies. This simulates
   // that the user chose the second account as the to-be-synced account.
   // The unconsented primary account should be updated.
-  identity_test_env()->SetPrimaryAccount(second_account.email,
-                                         ConsentLevel::kSignin);
+  SigninImplicitlyWithAccount(second_account.email);
   InitializeSignoutDecision();
   EXPECT_EQ(second_account,
             identity_manager()->GetPrimaryAccountInfo(ConsentLevel::kSignin));
   EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSync));
-  // TODO(crbug.com/1261772): The change should be logged in some way.
-  histogram_tester.ExpectUniqueSample(
-      "Signin.SignIn.Completed",
-      signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER, 1);
-  histogram_tester.ExpectTotalCount("Signin.SignOut.Completed", 0);
+  if (!explicit_browser_signin()) {
+    // TODO(crbug.com/40202341): The change should be logged in some way.
+    histogram_tester.ExpectUniqueSample(
+        "Signin.SignIn.Completed",
+        signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER, 1);
+    histogram_tester.ExpectTotalCount("Signin.SignOut.Completed", 0);
+  }
   observer().Reset();
 
   // Release the handle. The unconsented primary account should be updated to be
@@ -592,11 +714,13 @@ TEST_P(SigninManagerTest, UnconsentedPrimaryAccountUpdatedOnHandleDestroyed) {
     EXPECT_EQ(0U, observer().events().size());
   }
 #endif
-  // TODO(crbug.com/1261772): The change should be logged in some way.
-  histogram_tester.ExpectUniqueSample(
-      "Signin.SignIn.Completed",
-      signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER, 1);
-  histogram_tester.ExpectTotalCount("Signin.SignOut.Completed", 0);
+  // TODO(crbug.com/40202341): The change should be logged in some way.
+  if (!explicit_browser_signin()) {
+    histogram_tester.ExpectUniqueSample(
+        "Signin.SignIn.Completed",
+        signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER, 1);
+    histogram_tester.ExpectTotalCount("Signin.SignOut.Completed", 0);
+  }
 }
 
 TEST_P(SigninManagerTest, ClearPrimaryAccountAndSignOut) {
@@ -622,6 +746,10 @@ TEST_P(SigninManagerTest,
        UnconsentedPrimaryAccountClearedWhenSigninDisallowed) {
   // Prerequisite: add an unconsented primary account.
   AccountInfo account = MakeAccountAvailableWithCookies(kTestEmail);
+  if (explicit_browser_signin()) {
+    // Set the primary account and mark it to be implicitly signed in.
+    SigninImplicitlyWithAccount(account.email);
+  }
   ExpectUnconsentedPrimaryAccountSetEvent(account);
 
   prefs_.SetBoolean(prefs::kSigninAllowed, false);
@@ -639,9 +767,10 @@ TEST_P(SigninManagerTest,
 }
 #endif  // !BUILDFLAG(IS_CHROMEOS_LACROS)
 
-INSTANTIATE_TEST_SUITE_P(SignoutAllowed, SigninManagerTest, ::testing::Bool());
-
-TEST_F(SigninManagerTest, SigninCompletedMetric) {
+TEST_P(SigninManagerTest, SigninCompletedMetric) {
+  if (explicit_browser_signin()) {
+    GTEST_SKIP();
+  }
   base::HistogramTester histogram_tester;
 
   signin_metrics::AccessPoint access_point =
@@ -649,18 +778,26 @@ TEST_F(SigninManagerTest, SigninCompletedMetric) {
   AccountInfo account =
       MakeAccountAvailableWithCookies(kTestEmail, access_point);
   ExpectUnconsentedPrimaryAccountSetEvent(account);
-#if BUILDFLAG(IS_CHROMEOS_LACROS)
-  // Lacros always records `ACCESS_POINT_DESKTOP_SIGNIN_MANAGER`.
-  access_point =
-      signin_metrics::AccessPoint::ACCESS_POINT_DESKTOP_SIGNIN_MANAGER;
-#endif
   histogram_tester.ExpectUniqueSample("Signin.SignIn.Completed", access_point,
                                       1);
   histogram_tester.ExpectUniqueSample("Signin.SigninManager.SigninAccessPoint",
                                       access_point, 1);
 }
 
-#if BUILDFLAG(ENABLE_DICE_SUPPORT) && BUILDFLAG(ENABLE_SUPERVISED_USERS)
+INSTANTIATE_TEST_SUITE_P(
+    ,
+    SigninManagerTest,
+    ::testing::ConvertGenerator<SigninManagerTestParams::TupleType>(
+        ::testing::Combine(::testing::Bool(), ::testing::Bool())),
+    [](const ::testing::TestParamInfo<SigninManagerTestParams>& info) {
+      std::string name = base::StrCat(
+          {info.param.explicit_browser_signin ? "Explicit" : "Implicit", "And",
+           info.param.is_signout_allowed ? "SignoutAllowed"
+                                         : "SignoutRestricted"});
+      return name;
+    });
+
+#if BUILDFLAG(ENABLE_DICE_SUPPORT)
 class SigninManagerSupervisedUserTest : public SigninManagerTest {
  public:
   SigninManagerSupervisedUserTest() = default;
@@ -682,11 +819,9 @@ class SigninManagerSupervisedUserTest : public SigninManagerTest {
   base::test::ScopedFeatureList feature_list_;
 };
 
-TEST_F(SigninManagerSupervisedUserTest, SignoutOnCookiesDeletedNotAllowed) {
+TEST_P(SigninManagerSupervisedUserTest, SignoutOnCookiesDeletedNotAllowed) {
   base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitWithFeatures(
-      {supervised_user::kClearingCookiesKeepsSupervisedUsersSignedIn},
-      {kPreventSignoutIfAccountValid});
+  scoped_feature_list.InitWithFeatures({}, {kPreventSignoutIfAccountValid});
   AddSupervisedAccount(ConsentLevel::kSignin);
   ASSERT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
   ASSERT_EQ(1U, observer().events().size());
@@ -698,21 +833,11 @@ TEST_F(SigninManagerSupervisedUserTest, SignoutOnCookiesDeletedNotAllowed) {
   EXPECT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
 }
 
-TEST_F(SigninManagerSupervisedUserTest, SignoutOnCookiesDeletedAllowed) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitWithFeatures(
-      {}, {supervised_user::kClearingCookiesKeepsSupervisedUsersSignedIn,
-           kPreventSignoutIfAccountValid});
-  AddSupervisedAccount(ConsentLevel::kSignin);
-  ASSERT_TRUE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
-  ASSERT_EQ(1U, observer().events().size());
-  observer().Reset();
-
-  // Remove the cookie, the account should be cleared.
-  identity_test_env()->SetCookieAccounts({});
-  EXPECT_EQ(1U, observer().events().size());
-  EXPECT_FALSE(identity_manager()->HasPrimaryAccount(ConsentLevel::kSignin));
-}
-#endif  // BUILDFLAG(ENABLE_DICE_SUPPORT) && BUILDFLAG(ENABLE_SUPERVISED_USERS)
+INSTANTIATE_TEST_SUITE_P(,
+                         SigninManagerSupervisedUserTest,
+                         ::testing::Values(SigninManagerTestParams(
+                             {/*explicit_browser_signin=*/false,
+                              /*is_signout_allowed=*/true})));
+#endif  // BUILDFLAG(ENABLE_DICE_SUPPORT)
 
 }  // namespace signin
