@@ -54,9 +54,7 @@ const char kSecondaryGaiaID[] = "secondary_gaia_id";
 
 using testing::_;
 
-class GetAllowedDomainTest : public ::testing::Test {};
-
-TEST_F(GetAllowedDomainTest, WithInvalidPattern) {
+TEST(GetAllowedDomainTest, WithInvalidPattern) {
   EXPECT_EQ(std::string(), GetAllowedDomain("email"));
   EXPECT_EQ(std::string(), GetAllowedDomain("email@a@b"));
   EXPECT_EQ(std::string(), GetAllowedDomain("email@a[b"));
@@ -69,7 +67,7 @@ TEST_F(GetAllowedDomainTest, WithInvalidPattern) {
   EXPECT_EQ(std::string(), GetAllowedDomain(""));
 }
 
-TEST_F(GetAllowedDomainTest, WithValidPattern) {
+TEST(GetAllowedDomainTest, WithValidPattern) {
   EXPECT_EQ("example.com", GetAllowedDomain("email@example.com"));
   EXPECT_EQ("example.com", GetAllowedDomain("email@example.com\\E"));
   EXPECT_EQ("example.com", GetAllowedDomain("email@example.com$"));
@@ -107,11 +105,12 @@ class MockSigninUiDelegate : public SigninUiDelegate {
                signin_metrics::AccessPoint access_point,
                signin_metrics::PromoAction promo_action,
                const CoreAccountId& account_id,
-               TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode),
+               TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode,
+               bool is_sync_promo),
               ());
 };
 #elif BUILDFLAG(ENABLE_DICE_SUPPORT)
-// TODO(https://crbug.com/1316608): move out testing of SigninUiDelegateImplDice
+// TODO(crbug.com/40834209): move out testing of SigninUiDelegateImplDice
 // in a separate file.
 class MockSigninUiDelegate : public SigninUiDelegateImplDice {
  public:
@@ -121,14 +120,15 @@ class MockSigninUiDelegate : public SigninUiDelegateImplDice {
                signin_metrics::AccessPoint access_point,
                signin_metrics::PromoAction promo_action,
                const CoreAccountId& account_id,
-               TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode),
+               TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode,
+               bool is_sync_promo),
               ());
 };
 #endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
 
 }  // namespace
 
-// TODO(https://crbug.com/1316608): merge SigninUiUtilTest with
+// TODO(crbug.com/40834209): merge SigninUiUtilTest with
 // MirrorSigninUiUtilTest.
 #if !BUILDFLAG(IS_CHROMEOS_LACROS)
 class SigninUiUtilTest : public BrowserWithTestWindowTest {
@@ -155,14 +155,19 @@ class SigninUiUtilTest : public BrowserWithTestWindowTest {
                                     is_default_promo_account);
   }
 
-  void ExpectTurnSyncOn(
-      signin_metrics::AccessPoint access_point,
-      signin_metrics::PromoAction promo_action,
-      const CoreAccountId& account_id,
-      TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode) {
-    EXPECT_CALL(mock_delegate_,
-                ShowTurnSyncOnUI(profile(), access_point, promo_action,
-                                 account_id, signin_aborted_mode));
+  void SignIn(const CoreAccountInfo& account_info) {
+    SignInFromSingleAccountPromo(profile(), account_info, access_point_);
+  }
+
+  void ExpectTurnSyncOn(signin_metrics::AccessPoint access_point,
+                        signin_metrics::PromoAction promo_action,
+                        const CoreAccountId& account_id,
+                        TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode,
+                        bool is_sync_promo) {
+    EXPECT_CALL(
+        mock_delegate_,
+        ShowTurnSyncOnUI(profile(), access_point, promo_action, account_id,
+                         signin_aborted_mode, is_sync_promo));
   }
 
   void ExpectNoSigninStartedHistograms(
@@ -272,7 +277,8 @@ TEST_F(SigninUiUtilTest, EnableSyncWithExistingAccount) {
             : signin_metrics::PromoAction::PROMO_ACTION_NOT_DEFAULT;
     ExpectTurnSyncOn(signin_metrics::AccessPoint::ACCESS_POINT_BOOKMARK_BUBBLE,
                      expected_promo_action, account_id,
-                     TurnSyncOnHelper::SigninAbortedMode::KEEP_ACCOUNT);
+                     TurnSyncOnHelper::SigninAbortedMode::KEEP_ACCOUNT,
+                     /*is_sync_promo=*/false);
     EnableSync(
         GetIdentityManager()->FindExtendedAccountInfoByAccountId(account_id),
         is_default_promo_account);
@@ -403,6 +409,87 @@ TEST_F(SigninUiUtilTest, EnableSyncForNewAccountWithOneTab) {
             active_contents->GetVisibleURL());
 }
 
+TEST_F(SigninUiUtilTest, SignInWithAlreadySignedInAccount) {
+  AddTab(browser(), GURL("http://example.com"));
+  CoreAccountId account_id =
+      GetIdentityManager()->GetAccountsMutator()->AddOrUpdateAccount(
+          kMainGaiaID, kMainEmail, "refresh_token", false,
+          signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN,
+          signin_metrics::SourceForRefreshTokenOperation::kUnknown);
+  GetIdentityManager()->GetPrimaryAccountMutator()->SetPrimaryAccount(
+      account_id, signin::ConsentLevel::kSignin,
+      signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN);
+
+  SignIn(GetIdentityManager()->FindExtendedAccountInfoByAccountId(account_id));
+
+  // Verify that the primary account is still set.
+  EXPECT_TRUE(
+      GetIdentityManager()->HasPrimaryAccount(signin::ConsentLevel::kSignin));
+
+  // Verify that the active tab does not open the DICE sign-in URL.
+  TabStripModel* tab_strip = browser()->tab_strip_model();
+  content::WebContents* active_contents = tab_strip->GetActiveWebContents();
+  ASSERT_TRUE(active_contents);
+  EXPECT_EQ(GURL("http://example.com"), active_contents->GetVisibleURL());
+  tab_strip->CloseWebContentsAt(
+      tab_strip->GetIndexOfWebContents(active_contents),
+      TabCloseTypes::CLOSE_USER_GESTURE);
+}
+
+TEST_F(SigninUiUtilTest, SignInWithAccountThatNeedsReauth) {
+  AddTab(browser(), GURL("http://example.com"));
+  CoreAccountId account_id =
+      GetIdentityManager()->GetAccountsMutator()->AddOrUpdateAccount(
+          kMainGaiaID, kMainEmail, "refresh_token", false,
+          signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN,
+          signin_metrics::SourceForRefreshTokenOperation::kUnknown);
+
+  // Add an account and then put its refresh token into an error state to
+  // require a reauth before signing in.
+  signin::UpdatePersistentErrorOfRefreshTokenForAccount(
+      GetIdentityManager(), account_id,
+      GoogleServiceAuthError(GoogleServiceAuthError::INVALID_GAIA_CREDENTIALS));
+
+  SignIn(GetIdentityManager()->FindExtendedAccountInfoByAccountId(account_id));
+
+  // Verify that the active tab has the correct DICE sign-in URL.
+  TabStripModel* tab_strip = browser()->tab_strip_model();
+  content::WebContents* active_contents = tab_strip->GetActiveWebContents();
+  ASSERT_TRUE(active_contents);
+  EXPECT_EQ(signin::GetAddAccountURLForDice(
+                kMainEmail, GURL(google_util::kGoogleHomepageURL)),
+            active_contents->GetVisibleURL());
+  tab_strip->CloseWebContentsAt(
+      tab_strip->GetIndexOfWebContents(active_contents),
+      TabCloseTypes::CLOSE_USER_GESTURE);
+}
+
+TEST_F(SigninUiUtilTest, SignInForNewAccountWithNoTab) {
+  SignIn(CoreAccountInfo());
+
+  // Verify that the active tab has the correct DICE sign-in URL.
+  content::WebContents* active_contents =
+      browser()->tab_strip_model()->GetActiveWebContents();
+  ASSERT_TRUE(active_contents);
+  EXPECT_EQ(signin::GetAddAccountURLForDice(
+                std::string(), GURL(google_util::kGoogleHomepageURL)),
+            active_contents->GetVisibleURL());
+}
+
+TEST_F(SigninUiUtilTest, SignInForNewAccountWithOneTab) {
+  AddTab(browser(), GURL("http://foo/1"));
+
+  SignIn(CoreAccountInfo());
+
+  // Verify that the active tab has the correct DICE sign-in URL.
+  content::WebContents* active_contents =
+      browser()->tab_strip_model()->GetActiveWebContents();
+  ASSERT_TRUE(active_contents);
+  EXPECT_EQ(signin::GetAddAccountURLForDice(
+                std::string(), GURL(google_util::kGoogleHomepageURL)),
+            active_contents->GetVisibleURL());
+}
+
 TEST_F(SigninUiUtilTest, GetOrderedAccountsForDisplay) {
   signin::IdentityManager* identity_manager_empty =
       IdentityManagerFactory::GetForProfile(profile());
@@ -547,10 +634,9 @@ TEST_F(SigninUiUtilTest, ShowReauthTab) {
   TabStripModel* tab_strip = browser()->tab_strip_model();
   content::WebContents* active_contents = tab_strip->GetActiveWebContents();
   ASSERT_TRUE(active_contents);
-  EXPECT_TRUE(
-      base::StartsWith(active_contents->GetVisibleURL().spec(),
-                       GaiaUrls::GetInstance()->add_account_url().spec(),
-                       base::CompareCase::INSENSITIVE_ASCII));
+  EXPECT_THAT(
+      active_contents->GetVisibleURL().spec(),
+      testing::StartsWith(GaiaUrls::GetInstance()->add_account_url().spec()));
 }
 
 TEST_F(SigninUiUtilTest,
@@ -603,6 +689,9 @@ TEST_F(
 }
 
 TEST_F(SigninUiUtilTest, ShowExtensionSigninPrompt) {
+  const GURL add_account_url = GaiaUrls::GetInstance()->add_account_url();
+  const GURL sync_url = GaiaUrls::GetInstance()->signin_chrome_sync_dice();
+
   Profile* profile = browser()->profile();
   TabStripModel* tab_strip = browser()->tab_strip_model();
   ShowExtensionSigninPrompt(profile, /*enable_sync=*/true,
@@ -615,10 +704,8 @@ TEST_F(SigninUiUtilTest, ShowExtensionSigninPrompt) {
 
   content::WebContents* tab = tab_strip->GetWebContentsAt(0);
   ASSERT_TRUE(tab);
-  EXPECT_TRUE(base::StartsWith(
-      tab->GetVisibleURL().spec(),
-      GaiaUrls::GetInstance()->signin_chrome_sync_dice().spec(),
-      base::CompareCase::INSENSITIVE_ASCII));
+  EXPECT_TRUE(base::StartsWith(tab->GetVisibleURL().spec(), sync_url.spec(),
+                               base::CompareCase::INSENSITIVE_ASCII));
 
   // Changing the parameter opens a new tab.
   ShowExtensionSigninPrompt(profile, /*enable_sync=*/false,
@@ -630,9 +717,14 @@ TEST_F(SigninUiUtilTest, ShowExtensionSigninPrompt) {
   EXPECT_EQ(2, tab_strip->count());
   tab = tab_strip->GetWebContentsAt(1);
   ASSERT_TRUE(tab);
+  // With explicit signin, `sync_url` is used even though Sync is not going to
+  // be enabled. This is because that web page displays additional text
+  // explaining to the user that they are signing in to Chrome.
   EXPECT_TRUE(
       base::StartsWith(tab->GetVisibleURL().spec(),
-                       GaiaUrls::GetInstance()->add_account_url().spec(),
+                       switches::IsExplicitBrowserSigninUIOnDesktopEnabled()
+                           ? sync_url.spec()
+                           : add_account_url.spec(),
                        base::CompareCase::INSENSITIVE_ASCII));
 }
 
@@ -680,10 +772,59 @@ TEST_F(SigninUiUtilTest, ShowSigninPromptFromPromoWithExistingAccount) {
 }
 #endif  // !BUILDFLAG(IS_CHROMEOS_LACROS)
 
+#if BUILDFLAG(ENABLE_DICE_SUPPORT)
+
+TEST_F(SigninUiUtilTest, GetSignInTabWithAccessPoint) {
+  signin::MakePrimaryAccountAvailable(GetIdentityManager(), "foo@example.com",
+                                      signin::ConsentLevel::kSignin);
+
+  Profile* profile = browser()->profile();
+  TabStripModel* tab_strip = browser()->tab_strip_model();
+  EXPECT_EQ(0, tab_strip->count());
+
+  // Add tabs.
+  ShowReauthForAccount(profile, "test1@gmail.com",
+                       signin_metrics::AccessPoint::ACCESS_POINT_SETTINGS);
+  ShowReauthForAccount(
+      profile, "test2@gmail.com",
+      signin_metrics::AccessPoint::ACCESS_POINT_CHROME_SIGNIN_INTERCEPT_BUBBLE);
+  ShowReauthForAccount(
+      profile, "test3@gmail.com",
+      signin_metrics::AccessPoint::ACCESS_POINT_PASSWORD_BUBBLE);
+  EXPECT_EQ(3, tab_strip->count());
+
+  // Look for existing tab.
+  content::WebContents* sign_in_tab = GetSignInTabWithAccessPoint(
+      browser(),
+      signin_metrics::AccessPoint::ACCESS_POINT_CHROME_SIGNIN_INTERCEPT_BUBBLE);
+  EXPECT_EQ(signin::GetAddAccountURLForDice(
+                "test2@gmail.com", GURL(google_util::kGoogleHomepageURL)),
+            sign_in_tab->GetVisibleURL());
+
+  // Look for non existing tab.
+  sign_in_tab = GetSignInTabWithAccessPoint(
+      browser(), signin_metrics::AccessPoint::ACCESS_POINT_FORCED_SIGNIN);
+  EXPECT_EQ(nullptr, sign_in_tab);
+
+  // Two tabs with the same access point, will return the first tab found.
+  ShowReauthForAccount(profile, "test4@gmail.com",
+                       signin_metrics::AccessPoint::ACCESS_POINT_SETTINGS);
+  EXPECT_EQ(4, tab_strip->count());
+
+  sign_in_tab = GetSignInTabWithAccessPoint(
+      browser(), signin_metrics::AccessPoint::ACCESS_POINT_SETTINGS);
+  EXPECT_EQ(signin::GetAddAccountURLForDice(
+                "test1@gmail.com", GURL(google_util::kGoogleHomepageURL)),
+            sign_in_tab->GetVisibleURL());
+}
+
+#endif  // BUILDFLAG(ENABLE_DICE_SUPPORT)
+
 #if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_WIN)
 class SigninUiUtilWithUnoDesktopTest : public SigninUiUtilTest {
  private:
-  base::test::ScopedFeatureList feature_list_{switches::kUnoDesktop};
+  base::test::ScopedFeatureList feature_list_{
+      switches::kExplicitBrowserSigninUIOnDesktop};
 };
 
 TEST_F(SigninUiUtilWithUnoDesktopTest, EnableSyncWithExistingWebOnlyAccount) {
@@ -708,7 +849,8 @@ TEST_F(SigninUiUtilWithUnoDesktopTest, EnableSyncWithExistingWebOnlyAccount) {
     ExpectTurnSyncOn(
         signin_metrics::AccessPoint::ACCESS_POINT_BOOKMARK_BUBBLE,
         expected_promo_action, account_id,
-        TurnSyncOnHelper::SigninAbortedMode::KEEP_ACCOUNT_ON_WEB_ONLY);
+        TurnSyncOnHelper::SigninAbortedMode::KEEP_ACCOUNT_ON_WEB_ONLY,
+        /*is_sync_promo=*/false);
     EnableSync(
         GetIdentityManager()->FindExtendedAccountInfoByAccountId(account_id),
         is_default_promo_account);
@@ -718,6 +860,70 @@ TEST_F(SigninUiUtilWithUnoDesktopTest, EnableSyncWithExistingWebOnlyAccount) {
                      "Signin_Signin_FromBookmarkBubble"));
   }
 }
+
+TEST_F(SigninUiUtilWithUnoDesktopTest, SignInWithExistingWebOnlyAccount) {
+  CoreAccountId account_id =
+      GetIdentityManager()->GetAccountsMutator()->AddOrUpdateAccount(
+          kMainGaiaID, kMainEmail, "refresh_token", false,
+          signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN,
+          signin_metrics::SourceForRefreshTokenOperation::kUnknown);
+
+  // Verify that the primary account is not set before.
+  EXPECT_FALSE(
+      GetIdentityManager()->HasPrimaryAccount(signin::ConsentLevel::kSignin));
+
+  SignIn(GetIdentityManager()->FindExtendedAccountInfoByAccountId(account_id));
+
+  // Verify that the primary account has been set.
+  EXPECT_TRUE(
+      GetIdentityManager()->HasPrimaryAccount(signin::ConsentLevel::kSignin));
+}
+
+TEST_F(SigninUiUtilWithUnoDesktopTest, ShowExtensionSigninPrompt) {
+  Profile* profile = browser()->profile();
+  TabStripModel* tab_strip = browser()->tab_strip_model();
+  ShowExtensionSigninPrompt(profile, /*enable_sync=*/false,
+                            /*email_hint=*/std::string());
+  EXPECT_EQ(1, tab_strip->count());
+  // Calling the function again reuses the tab.
+  ShowExtensionSigninPrompt(profile, /*enable_sync=*/false,
+                            /*email_hint=*/std::string());
+  EXPECT_EQ(1, tab_strip->count());
+
+  content::WebContents* tab = tab_strip->GetWebContentsAt(0);
+  ASSERT_TRUE(tab);
+  EXPECT_TRUE(base::StartsWith(
+      tab->GetVisibleURL().spec(),
+      GaiaUrls::GetInstance()->signin_chrome_sync_dice().spec(),
+      base::CompareCase::INSENSITIVE_ASCII));
+  EXPECT_NE(tab->GetVisibleURL().query().find("flow=promo"), std::string::npos);
+}
+
+TEST_F(SigninUiUtilWithUnoDesktopTest, ShowExtensionSigninPromptReauth) {
+  CoreAccountId account_id =
+      GetIdentityManager()->GetAccountsMutator()->AddOrUpdateAccount(
+          kMainGaiaID, kMainEmail, "refresh_token", false,
+          signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN,
+          signin_metrics::SourceForRefreshTokenOperation::kUnknown);
+  GetIdentityManager()->GetPrimaryAccountMutator()->SetPrimaryAccount(
+      account_id, signin::ConsentLevel::kSignin,
+      signin_metrics::AccessPoint::ACCESS_POINT_UNKNOWN);
+  signin::UpdatePersistentErrorOfRefreshTokenForAccount(
+      GetIdentityManager(), account_id,
+      GoogleServiceAuthError(GoogleServiceAuthError::INVALID_GAIA_CREDENTIALS));
+
+  Profile* profile = browser()->profile();
+  TabStripModel* tab_strip = browser()->tab_strip_model();
+  ShowExtensionSigninPrompt(profile, /*enable_sync=*/false, kMainEmail);
+  EXPECT_EQ(1, tab_strip->count());
+
+  content::WebContents* tab = tab_strip->GetWebContentsAt(0);
+  ASSERT_TRUE(tab);
+  EXPECT_TRUE(
+      base::StartsWith(tab->GetVisibleURL().spec(),
+                       GaiaUrls::GetInstance()->add_account_url().spec(),
+                       base::CompareCase::INSENSITIVE_ASCII));
+}
 #endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_WIN)
 
 #endif  // BUILDFLAG(ENABLE_DICE_SUPPORT) || BUILDFLAG(IS_CHROMEOS_LACROS)
@@ -755,9 +961,10 @@ class MirrorSigninUiUtilTest : public BrowserWithTestWindowTest {
       signin_metrics::PromoAction promo_action,
       const CoreAccountId& account_id,
       TurnSyncOnHelper::SigninAbortedMode signin_aborted_mode) {
-    EXPECT_CALL(mock_delegate_,
-                ShowTurnSyncOnUI(profile(), access_point, promo_action,
-                                 account_id, signin_aborted_mode));
+    EXPECT_CALL(
+        mock_delegate_,
+        ShowTurnSyncOnUI(profile(), access_point, promo_action, account_id,
+                         signin_aborted_mode, /*is_sync_promo=*/false));
   }
 
  protected:
