@@ -9,6 +9,7 @@
 #include <utility>
 
 #include "base/check_op.h"
+#include "base/containers/span.h"
 #include "base/files/file_util.h"
 #include "base/functional/bind.h"
 #include "base/lazy_instance.h"
@@ -163,7 +164,7 @@ void SpellcheckHunspellDictionary::RetryDownloadDictionary(
     content::BrowserContext* browser_context) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   if (dictionary_file_.file.IsValid()) {
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
     return;
   }
   browser_context_ = browser_context;
@@ -247,7 +248,7 @@ void SpellcheckHunspellDictionary::OnSimpleLoaderComplete(
 #if !BUILDFLAG(IS_ANDROID)
   // To prevent corrupted dictionary data from causing a renderer crash, scan
   // the dictionary data and verify it is sane before save it to a file.
-  if (!hunspell::BDict::Verify(data->data(), data->size())) {
+  if (!hunspell::BDict::Verify(base::as_byte_span(*data))) {
     // Let PostTaskAndReply caller send to InformListenersOfInitialization
     // through SaveDictionaryDataComplete().
     SaveDictionaryDataComplete(false);
@@ -361,24 +362,23 @@ SpellcheckHunspellDictionary::OpenDictionaryFile(base::TaskRunner* task_runner,
   dictionary.path = path;
 #endif  // BUILDFLAG(IS_WIN)
 
-  // Read the dictionary file and scan its data to check for corruption. The
-  // scoping closes the memory-mapped file before it is opened or deleted.
-  bool bdict_is_valid = false;
+  // Open the dictionary file and verify there is no corruption. If verification
+  // fails the file must be deleted.
 
-  {
-    base::MemoryMappedFile map;
-    bdict_is_valid =
-        base::PathExists(dictionary.path) &&
-        map.Initialize(dictionary.path) &&
-        hunspell::BDict::Verify(reinterpret_cast<const char*>(map.data()),
-                                map.length());
+  dictionary.file.Initialize(dictionary.path,
+                             base::File::FLAG_READ | base::File::FLAG_OPEN);
+  if (!dictionary.file.IsValid()) {
+    dictionary.file.Close();
+    base::DeleteFile(dictionary.path);
+    return dictionary;
   }
 
-  if (bdict_is_valid) {
-    dictionary.file.Initialize(dictionary.path,
-                               base::File::FLAG_READ | base::File::FLAG_OPEN);
-  } else {
+  std::vector<uint8_t> data;
+  data.resize(dictionary.file.GetLength());
+  if (!dictionary.file.ReadAndCheck(0, data) ||
+      !hunspell::BDict::Verify(data)) {
 #if !BUILDFLAG(IS_QTWEBENGINE)
+    dictionary.file.Close();
     base::DeleteFile(dictionary.path);
 #endif
   }
@@ -482,7 +482,7 @@ void SpellcheckHunspellDictionary::PlatformSupportsLanguageComplete(
       return;
     }
 #endif  // BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
   } else {
     // Either the platform spellchecker is unavailable / disabled, or it doesn't
     // support this language. In either case, we must use Hunspell for this
