@@ -1,23 +1,30 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/spellchecker/spellcheck_service.h"
 
-#include <algorithm>
 #include <iterator>
+#include <memory>
 #include <set>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/check_op.h"
 #include "base/containers/contains.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/no_destructor.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/string_split.h"
 #include "base/supports_user_data.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/values.h"
 #include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
+#include "chrome/browser/profiles/profile.h"
+#if !BUILDFLAG(IS_QTWEBENGINE)
+#include "chrome/browser/profiles/profiles_state.h"
+#endif
 #include "chrome/browser/spellchecker/spellcheck_factory.h"
 #include "chrome/browser/spellchecker/spellcheck_hunspell_dictionary.h"
 #include "components/language/core/browser/pref_names.h"
@@ -35,19 +42,20 @@
 #include "components/user_prefs/user_prefs.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/browser_thread.h"
-#include "content/public/browser/notification_service.h"
-#include "content/public/browser/notification_types.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/storage_partition.h"
 #include "mojo/public/cpp/bindings/remote.h"
 #include "ui/base/l10n/l10n_util.h"
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-#include "base/task/post_task.h"
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "ash/constants/ash_features.h"
+#endif
+
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 #include "base/task/task_traits.h"
 #include "base/task/thread_pool.h"
 #include "components/spellcheck/browser/windows_spell_checker.h"
-#endif  // defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#endif  // BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 
 using content::BrowserThread;
 
@@ -58,13 +66,23 @@ SpellcheckService::SpellCheckerBinder& GetSpellCheckerBinderOverride() {
   return *binder;
 }
 
+// Only record spelling-configuration metrics for profiles in which the user
+// can configure spelling.
+bool RecordSpellingConfigurationMetrics(content::BrowserContext* context) {
+#if !BUILDFLAG(IS_QTWEBENGINE)
+  return profiles::IsRegularUserProfile(Profile::FromBrowserContext(context));
+#else
+  return true;
+#endif
+}
+
 }  // namespace
 
 // TODO(rlp): I do not like globals, but keeping these for now during
 // transition.
 // An event used by browser tests to receive status events from this class and
 // its derived classes.
-base::WaitableEvent* g_status_event = NULL;
+base::WaitableEvent* g_status_event = nullptr;
 SpellcheckService::EventType g_status_type =
     SpellcheckService::BDICT_NOTINITIALIZED;
 
@@ -78,7 +96,7 @@ SpellcheckService::SpellcheckService(content::BrowserContext* context)
   dictionaries_pref.Init(spellcheck::prefs::kSpellCheckDictionaries, prefs);
   std::string first_of_dictionaries;
 
-#if (defined(OS_MAC) || defined(OS_ANDROID)) && !defined(TOOLKIT_QT)
+#if (BUILDFLAG(IS_MAC) || BUILDFLAG(IS_ANDROID)) && !BUILDFLAG(IS_QTWEBENGINE)
   // Ensure that the renderer always knows the platform spellchecking
   // language. This language is used for initialization of the text iterator.
   // If the iterator is not initialized, then the context menu does not show
@@ -104,38 +122,23 @@ SpellcheckService::SpellcheckService(content::BrowserContext* context)
   }
 
   single_dictionary_pref.SetValue("");
-#endif  // defined(OS_MAC) || defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_MAC) || BUILDFLAG(IS_ANDROID)
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   if (!spellcheck::UseBrowserSpellChecker()) {
     // A user may have disabled the Windows spellcheck feature after adding
     // non-Hunspell supported languages on the language settings page. Remove
     // preferences for non-Hunspell languages so that there is no attempt to
     // load a non-existent Hunspell dictionary, and so that Hunspell
     // spellchecking isn't broken because of the failed load.
-    ListPrefUpdate update(prefs, spellcheck::prefs::kSpellCheckDictionaries);
-    update->EraseListValueIf([](const base::Value& entry) {
+    ScopedListPrefUpdate update(prefs,
+                                spellcheck::prefs::kSpellCheckDictionaries);
+    update->EraseIf([](const base::Value& entry) {
       return spellcheck::GetCorrespondingSpellCheckLanguage(entry.GetString())
           .empty();
     });
   }
-#endif  // defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-
-  // Migrating kSpellCheckBlacklistedDictionaries preference to
-  // kSpellCheckBlocklistedDictionaries.
-  // TODO(crbug/1161062): Remove after M91.
-  StringListPrefMember old_blocked_dict_pref;
-  old_blocked_dict_pref.Init(
-      spellcheck::prefs::kSpellCheckBlacklistedDictionaries, prefs);
-  StringListPrefMember blocked_dict_pref;
-  blocked_dict_pref.Init(spellcheck::prefs::kSpellCheckBlocklistedDictionaries,
-                         prefs);
-
-  if (blocked_dict_pref.GetValue().empty() &&
-      !old_blocked_dict_pref.GetValue().empty()) {
-    blocked_dict_pref.SetValue(old_blocked_dict_pref.GetValue());
-    old_blocked_dict_pref.SetValue(std::vector<std::string>());
-  }
+#endif  // BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 
   pref_change_registrar_.Add(
       spellcheck::prefs::kSpellCheckDictionaries,
@@ -162,14 +165,12 @@ SpellcheckService::SpellcheckService(content::BrowserContext* context)
       base::BindRepeating(&SpellcheckService::InitForAllRenderers,
                           base::Unretained(this)));
 
-  custom_dictionary_.reset(new SpellcheckCustomDictionary(context_->GetPath()));
+  custom_dictionary_ =
+      std::make_unique<SpellcheckCustomDictionary>(context_->GetPath());
   custom_dictionary_->AddObserver(this);
   custom_dictionary_->Load();
 
-  registrar_.Add(this, content::NOTIFICATION_RENDERER_PROCESS_CREATED,
-                 content::NotificationService::AllSources());
-
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   if (spellcheck::UseBrowserSpellChecker() &&
       base::FeatureList::IsEnabled(
           spellcheck::kWinDelaySpellcheckServiceInit)) {
@@ -178,7 +179,7 @@ SpellcheckService::SpellcheckService(content::BrowserContext* context)
     // with a callback.
     return;
   }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
   InitializeDictionaries(base::DoNothing());
 }
@@ -192,7 +193,7 @@ base::WeakPtr<SpellcheckService> SpellcheckService::GetWeakPtr() {
   return weak_ptr_factory_.GetWeakPtr();
 }
 
-#if !defined(OS_MAC)
+#if !BUILDFLAG(IS_MAC)
 // static
 void SpellcheckService::GetDictionaries(
     content::BrowserContext* browser_context,
@@ -200,10 +201,10 @@ void SpellcheckService::GetDictionaries(
   PrefService* prefs = user_prefs::UserPrefs::Get(browser_context);
   std::set<std::string> spellcheck_dictionaries;
   for (const auto& value :
-       *prefs->GetList(spellcheck::prefs::kSpellCheckDictionaries)) {
-    std::string dictionary;
-    if (value.GetAsString(&dictionary))
-      spellcheck_dictionaries.insert(dictionary);
+       prefs->GetList(spellcheck::prefs::kSpellCheckDictionaries)) {
+    const std::string* dictionary = value.GetIfString();
+    if (dictionary)
+      spellcheck_dictionaries.insert(*dictionary);
   }
 
   dictionaries->clear();
@@ -212,7 +213,7 @@ void SpellcheckService::GetDictionaries(
                         ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
   for (const auto& accept_language : accept_languages) {
     Dictionary dictionary;
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
     if (spellcheck::UseBrowserSpellChecker()) {
       SpellcheckService* spellcheck =
           SpellcheckServiceFactory::GetForContext(browser_context);
@@ -227,7 +228,7 @@ void SpellcheckService::GetDictionaries(
 #else
     dictionary.language =
         spellcheck::GetCorrespondingSpellCheckLanguage(accept_language);
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
     if (dictionary.language.empty())
       continue;
@@ -237,7 +238,7 @@ void SpellcheckService::GetDictionaries(
     dictionaries->push_back(dictionary);
   }
 }
-#endif  // !OS_MAC
+#endif  // !BUILDFLAG(IS_MAC)
 
 // static
 bool SpellcheckService::SignalStatusEvent(
@@ -266,7 +267,7 @@ std::string SpellcheckService::GetSupportedAcceptLanguageCode(
         /* include_script_tag= */ false);
   }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   if (!spellcheck::UseBrowserSpellChecker())
     return supported_accept_language;
 
@@ -288,12 +289,12 @@ std::string SpellcheckService::GetSupportedAcceptLanguageCode(
   // First try exact match. Per BCP47, tags are in ASCII and should be treated
   // as case-insensitive (although there are conventions for the capitalization
   // of subtags).
-  auto iter =
-      std::find_if(accept_languages.begin(), accept_languages.end(),
-                   [supported_language_full_tag](const auto& accept_language) {
-                     return base::EqualsCaseInsensitiveASCII(
-                         supported_language_full_tag, accept_language);
-                   });
+  auto iter = base::ranges::find_if(
+      accept_languages,
+      [supported_language_full_tag](const auto& accept_language) {
+        return base::EqualsCaseInsensitiveASCII(supported_language_full_tag,
+                                                accept_language);
+      });
   if (iter != accept_languages.end())
     return *iter;
 
@@ -305,17 +306,17 @@ std::string SpellcheckService::GetSupportedAcceptLanguageCode(
   if (!base::Contains(supported_language_full_tag, "-"))
     return "";
 
-  iter =
-      std::find_if(accept_languages.begin(), accept_languages.end(),
-                   [supported_language_full_tag](const auto& accept_language) {
-                     return base::EqualsCaseInsensitiveASCII(
-                         SpellcheckService::GetLanguageAndScriptTag(
-                             supported_language_full_tag,
-                             /* include_script_tag= */ true),
-                         SpellcheckService::GetLanguageAndScriptTag(
-                             accept_language,
-                             /* include_script_tag= */ true));
-                   });
+  iter = base::ranges::find_if(
+      accept_languages,
+      [supported_language_full_tag](const auto& accept_language) {
+        return base::EqualsCaseInsensitiveASCII(
+            SpellcheckService::GetLanguageAndScriptTag(
+                supported_language_full_tag,
+                /* include_script_tag= */ true),
+            SpellcheckService::GetLanguageAndScriptTag(
+                accept_language,
+                /* include_script_tag= */ true));
+      });
 
   if (iter != accept_languages.end())
     return *iter;
@@ -327,19 +328,19 @@ std::string SpellcheckService::GetSupportedAcceptLanguageCode(
   return GetSupportedAcceptLanguageCodeGenericOnly(supported_language_full_tag,
                                                    accept_languages);
 
-#endif  // defined(OS_WIN)
-
+#else
   return supported_accept_language;
+#endif  // BUILDFLAG(IS_WIN)
 }
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // static
 void SpellcheckService::EnableFirstUserLanguageForSpellcheck(
     PrefService* prefs) {
   // Ensure that spellcheck is enabled for the first language in the
   // accept languages list.
-  base::Value user_dictionaries =
-      prefs->GetList(spellcheck::prefs::kSpellCheckDictionaries)->Clone();
+  base::Value::List user_dictionaries =
+      prefs->GetList(spellcheck::prefs::kSpellCheckDictionaries).Clone();
   std::vector<std::string> user_languages =
       base::SplitString(prefs->GetString(language::prefs::kAcceptLanguages),
                         ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
@@ -361,7 +362,7 @@ void SpellcheckService::EnableFirstUserLanguageForSpellcheck(
   }
 
   bool first_user_language_spellchecked = false;
-  for (const auto& dictionary_value : user_dictionaries.GetList()) {
+  for (const auto& dictionary_value : user_dictionaries) {
     first_user_language_spellchecked =
         base::Contains(dictionary_value.GetString(), first_user_language);
     if (first_user_language_spellchecked)
@@ -369,22 +370,30 @@ void SpellcheckService::EnableFirstUserLanguageForSpellcheck(
   }
 
   if (!first_user_language_spellchecked) {
-    user_dictionaries.Insert(user_dictionaries.GetList().begin(),
+    user_dictionaries.Insert(user_dictionaries.begin(),
                              base::Value(first_user_language));
-    prefs->Set(spellcheck::prefs::kSpellCheckDictionaries, user_dictionaries);
+    prefs->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                   std::move(user_dictionaries));
   }
 }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 void SpellcheckService::StartRecordingMetrics(bool spellcheck_enabled) {
   metrics_ = std::make_unique<SpellCheckHostMetrics>();
-  metrics_->RecordEnabledStats(spellcheck_enabled);
+  auto record_configuration_metrics =
+      RecordSpellingConfigurationMetrics(context_);
+  if (record_configuration_metrics) {
+    metrics_->RecordEnabledStats(spellcheck_enabled);
+  }
+
   OnUseSpellingServiceChanged();
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-  RecordChromeLocalesStats();
-  RecordSpellcheckLocalesStats();
-#endif  // defined(OS_WIN)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+  if (record_configuration_metrics) {
+    RecordChromeLocalesStats();
+    RecordSpellcheckLocalesStats();
+  }
+#endif  // BUILDFLAG(IS_WIN)
 }
 
 void SpellcheckService::InitForRenderer(content::RenderProcessHost* host) {
@@ -431,27 +440,27 @@ void SpellcheckService::LoadDictionaries() {
   PrefService* prefs = user_prefs::UserPrefs::Get(context_);
   DCHECK(prefs);
 
-  const base::ListValue* user_dictionaries =
+  const base::Value::List& user_dictionaries =
       prefs->GetList(spellcheck::prefs::kSpellCheckDictionaries);
-  const base::ListValue* forced_dictionaries =
+  const base::Value::List& forced_dictionaries =
       prefs->GetList(spellcheck::prefs::kSpellCheckForcedDictionaries);
 
   // Build a lookup of blocked dictionaries to skip loading them.
-  const base::ListValue* blocked_dictionaries =
+  const base::Value::List& blocked_dictionaries =
       prefs->GetList(spellcheck::prefs::kSpellCheckBlocklistedDictionaries);
   std::unordered_set<std::string> blocked_dictionaries_lookup;
-  for (const auto& blocked_dict : blocked_dictionaries->GetList()) {
+  for (const auto& blocked_dict : blocked_dictionaries) {
     blocked_dictionaries_lookup.insert(blocked_dict.GetString());
   }
 
   // Merge both lists of dictionaries. Use a set to avoid duplicates.
   std::set<std::string> dictionaries;
-  for (const auto& dictionary_value : user_dictionaries->GetList()) {
+  for (const auto& dictionary_value : user_dictionaries) {
     if (blocked_dictionaries_lookup.find(dictionary_value.GetString()) ==
         blocked_dictionaries_lookup.end())
       dictionaries.insert(dictionary_value.GetString());
   }
-  for (const auto& dictionary_value : forced_dictionaries->GetList()) {
+  for (const auto& dictionary_value : forced_dictionaries) {
     dictionaries.insert(dictionary_value.GetString());
   }
 
@@ -459,7 +468,7 @@ void SpellcheckService::LoadDictionaries() {
     // The spellcheck language passed to platform APIs may differ from the
     // accept language.
     std::string platform_spellcheck_language;
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
     if (spellcheck::UseBrowserSpellChecker()) {
       std::string windows_dictionary_name =
           GetSupportedWindowsDictionaryLanguage(dictionary);
@@ -469,7 +478,7 @@ void SpellcheckService::LoadDictionaries() {
                 dictionary, windows_dictionary_name);
       }
     }
-#endif  // defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#endif  // BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 
     hunspell_dictionaries_.push_back(
         std::make_unique<SpellcheckHunspellDictionary>(
@@ -478,8 +487,10 @@ void SpellcheckService::LoadDictionaries() {
     hunspell_dictionaries_.back()->Load();
   }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-  RecordSpellcheckLocalesStats();
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+  if (RecordSpellingConfigurationMetrics(context_)) {
+    RecordSpellcheckLocalesStats();
+  }
 
   if (base::FeatureList::IsEnabled(
           spellcheck::kWinDelaySpellcheckServiceInit) &&
@@ -496,7 +507,7 @@ void SpellcheckService::LoadDictionaries() {
     }
     return;
   }
-#endif  // defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#endif  // BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   dictionaries_loaded_ = true;
 }
 
@@ -509,7 +520,7 @@ bool SpellcheckService::IsSpellcheckEnabled() const {
   const PrefService* prefs = user_prefs::UserPrefs::Get(context_);
 
   bool enable_if_uninitialized = false;
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   if (spellcheck::UseBrowserSpellChecker() &&
       base::FeatureList::IsEnabled(
           spellcheck::kWinDelaySpellcheckServiceInit)) {
@@ -519,17 +530,19 @@ bool SpellcheckService::IsSpellcheckEnabled() const {
     if (!dictionaries_loaded())
       enable_if_uninitialized = true;
   }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
+
+#if BUILDFLAG(IS_QTWEBENGINE) && BUILDFLAG(IS_MAC) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+  enable_if_uninitialized = true;
+#endif
 
   return prefs->GetBoolean(spellcheck::prefs::kSpellCheckEnable) &&
          (!hunspell_dictionaries_.empty() || enable_if_uninitialized);
 }
 
-void SpellcheckService::Observe(int type,
-                                const content::NotificationSource& source,
-                                const content::NotificationDetails& details) {
-  DCHECK_EQ(content::NOTIFICATION_RENDERER_PROCESS_CREATED, type);
-  InitForRenderer(content::Source<content::RenderProcessHost>(source).ptr());
+void SpellcheckService::OnRenderProcessHostCreated(
+    content::RenderProcessHost* host) {
+  InitForRenderer(host);
 }
 
 void SpellcheckService::OnCustomDictionaryLoaded() {
@@ -570,7 +583,7 @@ void SpellcheckService::OnHunspellDictionaryDownloadSuccess(
 
 void SpellcheckService::OnHunspellDictionaryDownloadFailure(
     const std::string& language) {
-#ifdef TOOLKIT_QT
+#if BUILDFLAG(IS_QTWEBENGINE)
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
     context_->FailedToLoadDictionary(language);
 #endif
@@ -583,12 +596,12 @@ void SpellcheckService::InitializeDictionaries(base::OnceClosure done) {
     return;
   }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   dictionaries_loaded_callback_ = std::move(done);
   // Need to initialize the platform spellchecker in order to record platform
   // locale stats even if the platform spellcheck feature is disabled.
   InitializePlatformSpellchecker();
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
   PrefService* prefs = user_prefs::UserPrefs::Get(context_);
   DCHECK(prefs);
@@ -597,7 +610,7 @@ void SpellcheckService::InitializeDictionaries(base::OnceClosure done) {
   StartRecordingMetrics(
       prefs->GetBoolean(spellcheck::prefs::kSpellCheckEnable));
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
   if (spellcheck::UseBrowserSpellChecker() && platform_spell_checker()) {
     spellcheck_platform::RetrieveSpellcheckLanguages(
         platform_spell_checker(),
@@ -605,13 +618,13 @@ void SpellcheckService::InitializeDictionaries(base::OnceClosure done) {
                        GetWeakPtr()));
     return;
   }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
   // Using Hunspell.
   LoadDictionaries();
 }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 void SpellcheckService::InitWindowsDictionaryLanguages(
     const std::vector<std::string>& windows_spellcheck_languages) {
   windows_spellcheck_dictionary_map_.clear();
@@ -663,23 +676,24 @@ void SpellcheckService::InitWindowsDictionaryLanguages(
   DCHECK(prefs);
   // When following object goes out of scope, preference change observers will
   // be notified (even if there is no preference change).
-  ListPrefUpdate update(prefs, spellcheck::prefs::kSpellCheckDictionaries);
-  update->EraseListValueIf([this](const base::Value& entry) {
+  ScopedListPrefUpdate update(prefs,
+                              spellcheck::prefs::kSpellCheckDictionaries);
+  update->EraseIf([this](const base::Value& entry) {
     const std::string dictionary_name = entry.GetString();
     return (!UsesWindowsDictionary(dictionary_name) &&
             spellcheck::GetCorrespondingSpellCheckLanguage(dictionary_name)
                 .empty());
   });
 
-  // No need to call LoadDictionaries() as when the ListPrefUpdate
-  // object goes out of scope, the preference change handler will do this.
+  // No need to call LoadDictionaries() as when the ScopedListPrefUpdate object
+  // goes out of scope, the preference change handler will do this.
 }
 
 bool SpellcheckService::UsesWindowsDictionary(
     std::string accept_language) const {
   return !GetSupportedWindowsDictionaryLanguage(accept_language).empty();
 }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 // static
 void SpellcheckService::OverrideBinderForTesting(SpellCheckerBinder binder) {
@@ -716,22 +730,22 @@ std::string SpellcheckService::GetLanguageAndScriptTag(
   return language_and_script_tag;
 }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 // static
 std::string SpellcheckService::GetSupportedAcceptLanguageCodeGenericOnly(
     const std::string& supported_language_full_tag,
     const std::vector<std::string>& accept_languages) {
-  auto iter =
-      std::find_if(accept_languages.begin(), accept_languages.end(),
-                   [supported_language_full_tag](const auto& accept_language) {
-                     return base::EqualsCaseInsensitiveASCII(
-                         SpellcheckService::GetLanguageAndScriptTag(
-                             supported_language_full_tag,
-                             /* include_script_tag= */ false),
-                         SpellcheckService::GetLanguageAndScriptTag(
-                             accept_language,
-                             /* include_script_tag= */ false));
-                   });
+  auto iter = base::ranges::find_if(
+      accept_languages,
+      [supported_language_full_tag](const auto& accept_language) {
+        return base::EqualsCaseInsensitiveASCII(
+            SpellcheckService::GetLanguageAndScriptTag(
+                supported_language_full_tag,
+                /* include_script_tag= */ false),
+            SpellcheckService::GetLanguageAndScriptTag(
+                accept_language,
+                /* include_script_tag= */ false));
+      });
 
   if (iter != accept_languages.end()) {
     // Special case for Serbian--"sr" implies Cyrillic script. Don't mark it as
@@ -790,7 +804,7 @@ std::string SpellcheckService::GetTagToPassToWindowsSpellchecker(
       /* include_script_tag= */ true);
 }
 
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 // static
 void SpellcheckService::AttachStatusEvent(base::WaitableEvent* status_event) {
@@ -836,12 +850,12 @@ void SpellcheckService::OnSpellCheckDictionariesChanged() {
   // If there are no hunspell dictionaries to load, then immediately let the
   // renderers know the new state.
   if (hunspell_dictionaries_.empty()) {
-#if !defined(OS_MAC)
+#if !BUILDFLAG(IS_MAC)
     // Only update non-MacOS platform because basic spell check on Mac OS
     // is controlled by OS and doesn't depend on users' dictionaries pref
     user_prefs::UserPrefs::Get(context_)->SetBoolean(
         spellcheck::prefs::kSpellCheckEnable, false);
-#endif  // !defined(OS_MAC)
+#endif  // !BUILDFLAG(IS_MAC)
     InitForAllRenderers();
   }
 }
@@ -849,11 +863,14 @@ void SpellcheckService::OnSpellCheckDictionariesChanged() {
 void SpellcheckService::OnUseSpellingServiceChanged() {
   bool enabled = pref_change_registrar_.prefs()->GetBoolean(
       spellcheck::prefs::kSpellCheckUseSpellingService);
-  if (metrics_)
+  if (metrics_ && RecordSpellingConfigurationMetrics(context_)) {
     metrics_->RecordSpellingServiceStats(enabled);
+  }
 }
 
 void SpellcheckService::OnAcceptLanguagesChanged() {
+  // Accept-Languages and spell check are decoupled on CrOS.
+#if !BUILDFLAG(IS_CHROMEOS_ASH)
   std::vector<std::string> accept_languages = GetNormalizedAcceptLanguages();
 
   StringListPrefMember dictionaries_pref;
@@ -870,9 +887,12 @@ void SpellcheckService::OnAcceptLanguagesChanged() {
 
   dictionaries_pref.SetValue(filtered_dictionaries);
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-  RecordChromeLocalesStats();
-#endif  // defined(OS_WIN)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+  if (RecordSpellingConfigurationMetrics(context_)) {
+    RecordChromeLocalesStats();
+  }
+#endif  // BUILDFLAG(IS_WIN)
+#endif  // !BUILDFLAG(IS_CHROMEOS_ASH)
 }
 
 std::vector<std::string> SpellcheckService::GetNormalizedAcceptLanguages(
@@ -883,14 +903,14 @@ std::vector<std::string> SpellcheckService::GetNormalizedAcceptLanguages(
                         ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
 
   if (normalize_for_spellcheck) {
-    std::transform(
-        accept_languages.begin(), accept_languages.end(),
-        accept_languages.begin(), [&](const std::string& language) {
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+    base::ranges::transform(
+        accept_languages, accept_languages.begin(),
+        [&](const std::string& language) {
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
           if (spellcheck::UseBrowserSpellChecker() &&
               UsesWindowsDictionary(language))
             return language;
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
           return spellcheck::GetCorrespondingSpellCheckLanguage(language);
         });
   }
@@ -898,39 +918,36 @@ std::vector<std::string> SpellcheckService::GetNormalizedAcceptLanguages(
   return accept_languages;
 }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 void SpellcheckService::InitializePlatformSpellchecker() {
   // The Windows spell checker must be created before the dictionaries are
   // initialized. Note it is instantiated even if only Hunspell is being used
   // since metrics on the availability of Windows platform language packs are
   // being recorded. Thus method should only be called once, except in test
   // code.
-  if (!platform_spell_checker() &&
-      spellcheck::WindowsVersionSupportsSpellchecker()) {
+  if (!platform_spell_checker()) {
     platform_spell_checker_ = std::make_unique<WindowsSpellChecker>(
         base::ThreadPool::CreateCOMSTATaskRunner({base::MayBlock()}));
   }
 }
 
 void SpellcheckService::RecordSpellcheckLocalesStats() {
-  if (spellcheck::WindowsVersionSupportsSpellchecker() && metrics_ &&
-      platform_spell_checker() && !hunspell_dictionaries_.empty()) {
+  if (metrics_ && platform_spell_checker() && !hunspell_dictionaries_.empty()) {
     std::vector<std::string> hunspell_locales;
     for (auto& dict : hunspell_dictionaries_) {
       hunspell_locales.push_back(dict->GetLanguage());
     }
     spellcheck_platform::RecordSpellcheckLocalesStats(
-        platform_spell_checker(), std::move(hunspell_locales), metrics_.get());
+        platform_spell_checker(), std::move(hunspell_locales));
   }
 }
 
 void SpellcheckService::RecordChromeLocalesStats() {
   const auto& accept_languages =
       GetNormalizedAcceptLanguages(/* normalize_for_spellcheck */ false);
-  if (spellcheck::WindowsVersionSupportsSpellchecker() && metrics_ &&
-      platform_spell_checker() && !accept_languages.empty()) {
-    spellcheck_platform::RecordChromeLocalesStats(
-        platform_spell_checker(), std::move(accept_languages), metrics_.get());
+  if (metrics_ && platform_spell_checker() && !accept_languages.empty()) {
+    spellcheck_platform::RecordChromeLocalesStats(platform_spell_checker(),
+                                                  std::move(accept_languages));
   }
 }
 
@@ -963,4 +980,4 @@ void SpellcheckService::AddSpellcheckLanguagesForTesting(
         platform_spell_checker(), languages);
   }
 }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
