@@ -1,14 +1,15 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/spellchecker/spellcheck_service.h"
 
+#include <optional>
 #include <ostream>
 
 #include "base/command_line.h"
 #include "base/containers/contains.h"
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/supports_user_data.h"
@@ -86,6 +87,11 @@ static std::unique_ptr<KeyedService> BuildSpellcheckService(
 class SpellcheckServiceUnitTestBase : public testing::Test {
  public:
   SpellcheckServiceUnitTestBase() = default;
+
+  SpellcheckServiceUnitTestBase(const SpellcheckServiceUnitTestBase&) = delete;
+  SpellcheckServiceUnitTestBase& operator=(
+      const SpellcheckServiceUnitTestBase&) = delete;
+
   ~SpellcheckServiceUnitTestBase() override = default;
 
   content::BrowserContext* browser_context() { return &profile_; }
@@ -93,12 +99,6 @@ class SpellcheckServiceUnitTestBase : public testing::Test {
 
  protected:
   void SetUp() override {
-#if defined(OS_WIN)
-    // Tests were designed assuming Hunspell dictionary used and may fail when
-    // Windows spellcheck is enabled by default.
-    feature_list_.InitAndDisableFeature(spellcheck::kWinUseBrowserSpellChecker);
-#endif  // defined(OS_WIN)
-
     // Use SetTestingFactoryAndUse to force creation and initialization.
     SpellcheckServiceFactory::GetInstance()->SetTestingFactoryAndUse(
         &profile_, base::BindRepeating(&BuildSpellcheckService));
@@ -106,54 +106,66 @@ class SpellcheckServiceUnitTestBase : public testing::Test {
 
   content::BrowserTaskEnvironment task_environment_;
 
-#if defined(OS_WIN)
-  // feature_list_ needs to be destroyed after profile_.
-  base::test::ScopedFeatureList feature_list_;
-#endif  // defined(OS_WIN)
   TestingProfile profile_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(SpellcheckServiceUnitTestBase);
 };
 
 class SpellcheckServiceUnitTest : public SpellcheckServiceUnitTestBase,
                                   public testing::WithParamInterface<TestCase> {
+ private:
+#if BUILDFLAG(IS_WIN)
+  // Tests were designed assuming Hunspell dictionary used and may fail when
+  // Windows spellcheck is enabled by default.
+  spellcheck::ScopedDisableBrowserSpellCheckerForTesting
+      disable_browser_spell_checker_;
+#endif  // BUILDFLAG(IS_WIN)
 };
 
 INSTANTIATE_TEST_SUITE_P(
     TestCases,
     SpellcheckServiceUnitTest,
     testing::Values(
-        TestCase("en,aa", {"aa"}, {}, {}),
-        TestCase("en,en-JP,fr,aa", {"fr"}, {"fr"}, {"fr"}),
-        TestCase("en,en-JP,fr,zz,en-US", {"fr"}, {"fr", "en-US"}, {"fr"}),
-        TestCase("en,en-US,en-GB", {"en-GB"}, {"en-US", "en-GB"}, {"en-GB"}),
-        TestCase("en,en-US,en-AU", {"en-AU"}, {"en-US", "en-AU"}, {"en-AU"}),
-        TestCase("en,en-US,en-AU", {"en-US"}, {"en-US", "en-AU"}, {"en-US"}),
-        TestCase("en,en-US", {"en-US"}, {"en-US"}, {"en-US"}),
-        TestCase("en,en-US,fr", {"en-US"}, {"en-US", "fr"}, {"en-US"}),
+        TestCase("en-JP,aa", {"aa"}, {}, {}),
+        TestCase("en,aa", {"aa"}, {"en"}, {}),
+        TestCase("en,en-JP,fr,aa", {"fr"}, {"en", "fr"}, {"fr"}),
+        TestCase("en,en-JP,fr,zz,en-US", {"fr"}, {"en", "fr", "en-US"}, {"fr"}),
+        TestCase("en,en-US,en-GB",
+                 {"en-GB"},
+                 {"en", "en-US", "en-GB"},
+                 {"en-GB"}),
+        TestCase("en,en-US,en-AU",
+                 {"en-AU"},
+                 {"en", "en-US", "en-AU"},
+                 {"en-AU"}),
+        TestCase("en,en-US,en-AU",
+                 {"en-US"},
+                 {"en", "en-US", "en-AU"},
+                 {"en-US"}),
+        TestCase("en,en-US", {"en-US"}, {"en", "en-US"}, {"en-US"}),
+        TestCase("en,en-US,fr", {"en-US"}, {"en", "en-US", "fr"}, {"en-US"}),
         TestCase("en,fr,en-US,en-AU",
                  {"en-US", "fr"},
-                 {"fr", "en-US", "en-AU"},
+                 {"en", "fr", "en-US", "en-AU"},
                  {"fr", "en-US"}),
-        TestCase("en-US,en", {"en-US"}, {"en-US"}, {"en-US"}),
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+        TestCase("en-US,en", {"en-US"}, {"en-US", "en"}, {"en-US"}),
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
         // Scenario where user disabled the Windows spellcheck feature with some
         // non-Hunspell languages set in preferences.
         TestCase("fr,eu,en-US,ar",
                  {"fr", "eu", "en-US", "ar"},
                  {"fr", "en-US"},
                  {"fr", "en-US"}),
-#endif  // defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#endif  // BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
         TestCase("hu-HU,hr-HR", {"hr"}, {"hu", "hr"}, {"hr"})));
 
 TEST_P(SpellcheckServiceUnitTest, GetDictionaries) {
   prefs()->SetString(language::prefs::kAcceptLanguages,
                      GetParam().accept_languages);
-  base::ListValue spellcheck_dictionaries;
-  spellcheck_dictionaries.AppendStrings(GetParam().spellcheck_dictionaries);
-  prefs()->Set(spellcheck::prefs::kSpellCheckDictionaries,
-               spellcheck_dictionaries);
+  base::Value::List spellcheck_dictionaries;
+  for (const std::string& dictionary : GetParam().spellcheck_dictionaries) {
+    spellcheck_dictionaries.Append(dictionary);
+  }
+  prefs()->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                   std::move(spellcheck_dictionaries));
 
   std::vector<SpellcheckService::Dictionary> dictionaries;
   SpellcheckService::GetDictionaries(browser_context(), &dictionaries);
@@ -161,7 +173,7 @@ TEST_P(SpellcheckServiceUnitTest, GetDictionaries) {
   EXPECT_EQ(GetParam().expected_dictionaries, dictionaries);
 }
 
-#if defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
 class SpellcheckServiceHybridUnitTestBase
     : public SpellcheckServiceUnitTestBase {
  public:
@@ -176,9 +188,7 @@ class SpellcheckServiceHybridUnitTestBase
         &profile_, base::BindRepeating(&BuildSpellcheckService));
   }
 
-  virtual void InitFeatures() {
-    feature_list_.InitAndEnableFeature(spellcheck::kWinUseBrowserSpellChecker);
-  }
+  virtual void InitFeatures() {}
 
   virtual void InitializeSpellcheckService(
       const std::vector<std::string>& spellcheck_languages_for_testing) {
@@ -209,21 +219,22 @@ class SpellcheckServiceHybridUnitTestBase
   static const std::vector<std::string>
       windows_spellcheck_languages_for_testing_;
 
-  SpellcheckService* spellcheck_service_;
+  base::test::ScopedFeatureList feature_list_;
+
+  raw_ptr<SpellcheckService> spellcheck_service_;
 };
 
 void SpellcheckServiceHybridUnitTestBase::RunGetDictionariesTest(
     const std::string accept_languages,
     const std::vector<std::string> spellcheck_dictionaries,
     const std::vector<SpellcheckService::Dictionary> expected_dictionaries) {
-  if (!spellcheck::WindowsVersionSupportsSpellchecker())
-    return;
-
   prefs()->SetString(language::prefs::kAcceptLanguages, accept_languages);
-  base::ListValue spellcheck_dictionaries_list;
-  spellcheck_dictionaries_list.AppendStrings(spellcheck_dictionaries);
-  prefs()->Set(spellcheck::prefs::kSpellCheckDictionaries,
-               spellcheck_dictionaries_list);
+  base::Value::List spellcheck_dictionaries_list;
+  for (std::string dict : spellcheck_dictionaries) {
+    spellcheck_dictionaries_list.Append(dict);
+  }
+  prefs()->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                   std::move(spellcheck_dictionaries_list));
 
   // Simulate first-run scenario (method is normally called during browser
   // start-up). If the primary accept language has no dictionary support, it is
@@ -244,9 +255,6 @@ void SpellcheckServiceHybridUnitTestBase::RunDictionaryMappingTest(
     const std::string expected_tag_passed_to_spellcheck,
     const std::string expected_accept_language_generic,
     const std::string expected_tag_passed_to_spellcheck_generic) {
-  if (!spellcheck::WindowsVersionSupportsSpellchecker())
-    return;
-
   InitializeSpellcheckService({full_tag});
 
   std::string supported_dictionary;
@@ -325,9 +333,17 @@ const std::vector<std::string> SpellcheckServiceHybridUnitTestBase::
                              // dictionaries.
 };
 
-class SpellcheckServiceHybridUnitTest
+class GetDictionariesHybridUnitTestNoDelayInit
     : public SpellcheckServiceHybridUnitTestBase,
-      public testing::WithParamInterface<TestCase> {};
+      public testing::WithParamInterface<TestCase> {
+ protected:
+  void InitFeatures() override {
+    // Disable kWinDelaySpellcheckServiceInit, as the case where it's enabled
+    // is tested in SpellcheckServiceWindowsDictionaryMappingUnitTestDelayInit.
+    feature_list_.InitAndDisableFeature(
+        spellcheck::kWinDelaySpellcheckServiceInit);
+  }
+};
 
 static const TestCase kHybridGetDictionariesParams[] = {
     // Galician (gl) has only Windows support, no Hunspell dictionary. Croatian
@@ -347,7 +363,10 @@ static const TestCase kHybridGetDictionariesParams[] = {
     TestCase("ceb", {}, {}, {}),
     TestCase("ceb,gl,hr", {"gl", "hr"}, {"gl", "hr"}, {"gl", "hr"}),
     // Finnish has only "fi" in hard-coded list of accept languages.
-    TestCase("fi-FI,fi,en-US,en", {"en-US"}, {"fi", "en-US"}, {"fi", "en-US"}),
+    TestCase("fi-FI,fi,en-US,en",
+             {"en-US"},
+             {"fi", "en-US", "en"},
+             {"fi", "en-US"}),
     // First language is supported by Windows but private use dictionaries
     // are ignored.
     TestCase("ja,gl", {"gl"}, {"gl"}, {"gl"}),
@@ -379,10 +398,10 @@ static const TestCase kHybridGetDictionariesParams[] = {
 };
 
 INSTANTIATE_TEST_SUITE_P(TestCases,
-                         SpellcheckServiceHybridUnitTest,
+                         GetDictionariesHybridUnitTestNoDelayInit,
                          testing::ValuesIn(kHybridGetDictionariesParams));
 
-TEST_P(SpellcheckServiceHybridUnitTest, GetDictionaries) {
+TEST_P(GetDictionariesHybridUnitTestNoDelayInit, GetDictionaries) {
   RunGetDictionariesTest(GetParam().accept_languages,
                          GetParam().spellcheck_dictionaries,
                          GetParam().expected_dictionaries);
@@ -412,7 +431,15 @@ std::ostream& operator<<(std::ostream& out,
 
 class SpellcheckServiceWindowsDictionaryMappingUnitTest
     : public SpellcheckServiceHybridUnitTestBase,
-      public testing::WithParamInterface<DictionaryMappingTestCase> {};
+      public testing::WithParamInterface<DictionaryMappingTestCase> {
+ protected:
+  void InitFeatures() override {
+    // Disable kWinDelaySpellcheckServiceInit, as the case where it's enabled
+    // is tested in SpellcheckServiceWindowsDictionaryMappingUnitTestDelayInit.
+    feature_list_.InitAndDisableFeature(
+        spellcheck::kWinDelaySpellcheckServiceInit);
+  }
+};
 
 static const DictionaryMappingTestCase kHybridDictionaryMappingsParams[] = {
     DictionaryMappingTestCase({"en-CA", "en-CA", "en-CA", "en", "en"}),
@@ -462,10 +489,8 @@ class SpellcheckServiceHybridUnitTestDelayInitBase
  protected:
   void InitFeatures() override {
     // Don't initialize the SpellcheckService on browser launch.
-    feature_list_.InitWithFeatures(
-        /*enabled_features=*/{spellcheck::kWinUseBrowserSpellChecker,
-                              spellcheck::kWinDelaySpellcheckServiceInit},
-        /*disabled_features=*/{});
+    feature_list_.InitAndEnableFeature(
+        spellcheck::kWinDelaySpellcheckServiceInit);
   }
 
   void InitializeSpellcheckService(
@@ -539,4 +564,4 @@ TEST_P(SpellcheckServiceWindowsDictionaryMappingUnitTestDelayInit,
       GetParam().expected_accept_language_generic,
       GetParam().expected_tag_passed_to_spellcheck_generic);
 }
-#endif  // defined(OS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#endif  // BUILDFLAG(IS_WIN) && BUILDFLAG(USE_BROWSER_SPELLCHECKER)
