@@ -20,11 +20,11 @@
 #include "chrome/test/base/ui_test_utils.h"
 #include "content/public/browser/back_forward_cache.h"
 #include "content/public/browser/navigation_entry.h"
-#include "content/public/browser/notification_service.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/content_switches.h"
+#include "content/public/common/isolated_world_ids.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/prerender_test_util.h"
@@ -211,8 +211,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   // original bug to reproduce.
   {
     GURL url = embedded_test_server()->GetURL("a.com", "/title1.html");
-    ui_test_utils::UrlLoadObserver load_complete(
-        url, content::NotificationService::AllSources());
+    ui_test_utils::UrlLoadObserver load_complete(url);
     EXPECT_TRUE(
         content::ExecJs(browser()->tab_strip_model()->GetActiveWebContents(),
                         "window.open('" + url.spec() + "');"));
@@ -231,8 +230,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   // Navigate back in session history to ensure view-source mode is still
   // active.
   {
-    ui_test_utils::UrlLoadObserver load_complete(
-        url_viewsource, content::NotificationService::AllSources());
+    ui_test_utils::UrlLoadObserver load_complete(url_viewsource);
     chrome::GoBack(browser(), WindowOpenDisposition::CURRENT_TAB);
     load_complete.Wait();
   }
@@ -652,7 +650,8 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheEnabledTest,
         subframe_url.c_str());
     content::TestNavigationObserver navigation_observer(original_contents);
     original_contents->GetPrimaryMainFrame()->ExecuteJavaScriptForTests(
-        base::ASCIIToUTF16(create_frame_script), base::NullCallback());
+        base::ASCIIToUTF16(create_frame_script), base::NullCallback(),
+        content::ISOLATED_WORLD_ID_GLOBAL);
     navigation_observer.Wait();
   }
 
@@ -811,7 +810,8 @@ IN_PROC_BROWSER_TEST_F(ViewSourcePrerenderTest, ViewSourceForPrerender) {
   set_target(content::WebContents::FromRenderFrameHost(referrer_frame));
 
   prerender_test_helper().AddPrerender(prerender_url);
-  int host_id = prerender_test_helper().GetHostForUrl(prerender_url);
+  content::FrameTreeNodeId host_id =
+      prerender_test_helper().GetHostForUrl(prerender_url);
   content::RenderFrameHost* prerender_frame =
       prerender_test_helper().GetPrerenderedMainFrameHost(host_id);
   EXPECT_TRUE(prerender_frame);
