@@ -41,15 +41,6 @@ const int kAlternateUrlTimeout = 15;
 // alternate URL.
 const int kAlternateUrlPollInterval = 200;
 
-// Runs the callback provided to `DriveUploadHandler::Upload`.
-void OnUploadDone(scoped_refptr<DriveUploadHandler> drive_upload_handler,
-                  DriveUploadHandler::UploadCallback callback,
-                  OfficeTaskResult task_result,
-                  std::optional<GURL> hosted_url,
-                  int64_t upload_size) {
-  std::move(callback).Run(task_result, std::move(hosted_url), upload_size);
-}
-
 std::string GetTargetAppName(base::FilePath file_path) {
   const std::string extension = base::ToLowerASCII(file_path.FinalExtension());
   if (base::Contains(file_manager::file_tasks::WordGroupExtensions(),
@@ -69,22 +60,10 @@ std::string GetTargetAppName(base::FilePath file_path) {
 
 }  // namespace
 
-// static.
-void DriveUploadHandler::Upload(
+DriveUploadHandler::DriveUploadHandler(
     Profile* profile,
     const FileSystemURL& source_url,
     UploadCallback callback,
-    base::SafeRef<CloudOpenMetrics> cloud_open_metrics) {
-  scoped_refptr<DriveUploadHandler> drive_upload_handler =
-      new DriveUploadHandler(profile, source_url, cloud_open_metrics);
-  // Keep `drive_upload_handler` alive until `UploadDone` executes.
-  drive_upload_handler->Run(
-      base::BindOnce(&OnUploadDone, drive_upload_handler, std::move(callback)));
-}
-
-DriveUploadHandler::DriveUploadHandler(
-    Profile* profile,
-    const FileSystemURL source_url,
     base::SafeRef<CloudOpenMetrics> cloud_open_metrics)
     : profile_(profile),
       file_system_context_(
@@ -95,13 +74,13 @@ DriveUploadHandler::DriveUploadHandler(
       notification_manager_(
           base::MakeRefCounted<CloudUploadNotificationManager>(
               profile,
-              source_url.path().BaseName().value(),
               l10n_util::GetStringUTF8(IDS_OFFICE_CLOUD_PROVIDER_GOOGLE_DRIVE),
               GetTargetAppName(source_url.path()),
               // TODO(b/242685536) Update when support for multi-files is added.
               /*num_files=*/1,
               upload_type_)),
       source_url_(source_url),
+      callback_(std::move(callback)),
       cloud_open_metrics_(cloud_open_metrics) {
   observed_copy_task_id_ = -1;
   observed_delete_task_id_ = -1;
@@ -109,10 +88,8 @@ DriveUploadHandler::DriveUploadHandler(
 
 DriveUploadHandler::~DriveUploadHandler() = default;
 
-void DriveUploadHandler::Run(UploadCallback callback) {
-  DCHECK(callback);
-  DCHECK(!callback_);
-  callback_ = std::move(callback);
+void DriveUploadHandler::Run() {
+  DCHECK(callback_);
 
   if (!profile_) {
     LOG(ERROR) << "No profile";
@@ -264,9 +241,7 @@ void DriveUploadHandler::OnSuccessfulUpload(
   const OfficeTaskResult task_result = upload_type_ == UploadType::kCopy
                                            ? OfficeTaskResult::kCopied
                                            : OfficeTaskResult::kMoved;
-  if (callback_) {
-    std::move(callback_).Run(task_result, hosted_url, upload_size_);
-  }
+  std::move(callback_).Run(task_result, hosted_url, upload_size_);
 }
 
 void DriveUploadHandler::OnFailedUpload(OfficeFilesUploadResult result_metric,
@@ -281,10 +256,7 @@ void DriveUploadHandler::OnFailedUpload(OfficeFilesUploadResult result_metric,
     LOG(ERROR) << "Upload to Google Drive: " << error_message;
     notification_manager_->ShowUploadError(error_message);
   }
-  if (callback_) {
-    std::move(callback_).Run(OfficeTaskResult::kFailedToUpload, std::nullopt,
-                             0);
-  }
+  std::move(callback_).Run(OfficeTaskResult::kFailedToUpload, std::nullopt, 0);
 }
 
 void DriveUploadHandler::OnIOTaskStatus(
@@ -355,8 +327,9 @@ void DriveUploadHandler::OnCopyStatus(
       ShowIOTaskError(status);
       return;
     case file_manager::io_task::State::kNeedPassword:
-      NOTREACHED() << "Encrypted file should not need password to be copied or "
-                      "moved. Case should not be reached.";
+      NOTREACHED_IN_MIGRATION()
+          << "Encrypted file should not need password to be copied or "
+             "moved. Case should not be reached.";
       return;
   }
 }
@@ -365,8 +338,9 @@ void DriveUploadHandler::OnDeleteStatus(
     const ::file_manager::io_task::ProgressStatus& status) {
   switch (status.state) {
     case file_manager::io_task::State::kCancelled:
-      NOTREACHED() << "Deletion of source or destination file should not have "
-                      "been cancelled.";
+      NOTREACHED_IN_MIGRATION()
+          << "Deletion of source or destination file should not have "
+             "been cancelled.";
       ABSL_FALLTHROUGH_INTENDED;
     case file_manager::io_task::State::kError:
     case file_manager::io_task::State::kSuccess:
@@ -482,7 +456,7 @@ void DriveUploadHandler::OnSyncingStatusUpdate(
         OnEndCopy(OfficeFilesUploadResult::kSyncError);
         return;
       case drivefs::mojom::ItemEvent::State::kCancelledAndDeleted:
-        NOTREACHED();
+        NOTREACHED_IN_MIGRATION();
         return;
       case drivefs::mojom::ItemEvent::State::kCancelledAndTrashed:
         LOG(ERROR) << "Drive sync error: cancelled and trashed";
@@ -563,8 +537,20 @@ void DriveUploadHandler::OnGetDriveMetadata(
   // host.
   if (hosted_url.host() != "docs.google.com") {
     if (timed_out) {
-      LOG(ERROR) << "Unexpected alternate URL - Drive editing unavailable";
-      OnEndCopy(OfficeFilesUploadResult::kUnexpectedAlternateUrlHost);
+      if (hosted_url.host() == "drive.google.com" &&
+          !file_manager::file_tasks::IsOfficeFileMimeType(
+              metadata->content_mime_type)) {
+        // The drive.google.com will appear if an uploaded file has an Office
+        // extension but is not actually an Office file. For example, the user
+        // just renamed their .mp4 to a .doc.
+        LOG(ERROR) << "Non-Office file cannot be opened with Google Docs";
+        OnEndCopy(OfficeFilesUploadResult::kFileNotAnOfficeFile,
+                  base::unexpected(GetNotAValidDocumentErrorMessage()));
+      } else {
+        LOG(ERROR) << "Unexpected alternate URL - Drive editing unavailable: "
+                   << hosted_url.host();
+        OnEndCopy(OfficeFilesUploadResult::kUnexpectedAlternateUrlHost);
+      }
     } else {
       alternate_url_poll_timer_.Start(
           FROM_HERE, base::Milliseconds(kAlternateUrlPollInterval),
