@@ -11,6 +11,7 @@
 #include "base/values.h"
 #include "chrome/browser/ash/app_mode/kiosk_chrome_app_manager.h"
 #include "chrome/browser/ash/login/oobe_screen.h"
+#include "chrome/browser/ash/login/screens/error_screen.h"
 #include "chrome/browser/ash/login/screens/network_error.h"
 #include "chrome/browser/ui/webui/ash/login/error_screen_handler.h"
 #include "chrome/browser/ui/webui/ash/login/network_state_informer.h"
@@ -132,7 +133,7 @@ void AppLaunchSplashScreenHandler::ShowNetworkConfigureUI(
   error_screen_->SetUIState(NetworkError::UI_STATE_KIOSK_MODE);
   error_screen_->SetIsPersistentError(true);
   error_screen_->AllowGuestSignin(false);
-  error_screen_->AllowOfflineLogin(false);
+  error_screen_->DisallowOfflineLogin();
   switch (network_state) {
     case NetworkStateInformer::CAPTIVE_PORTAL: {
       error_screen_->SetErrorState(NetworkError::ERROR_STATE_PORTAL,
