@@ -15,11 +15,10 @@
 #include "chrome/browser/ash/login/enrollment/enrollment_screen_view.h"
 #include "chrome/browser/ash/policy/enrollment/enrollment_config.h"
 #include "chrome/browser/ui/webui/ash/login/base_screen_handler.h"
-#include "net/cookies/canonical_cookie.h"
+#include "chrome/browser/ui/webui/ash/login/online_login_utils.h"
 
 namespace ash {
 
-class CookieWaiter;
 class HelpAppLauncher;
 
 // Possible error states of the Active Directory screen. Must be in the same
@@ -79,23 +78,25 @@ class EnrollmentScreenHandler : public BaseScreenHandler,
   void DeclareLocalizedValues(
       ::login::LocalizedValuesBuilder* builder) override;
   void DeclareJSCallbacks() override;
-  void GetAdditionalParameters(base::Value::Dict* parameters) override;
 
-  void ContinueAuthenticationWhenCookiesAvailable(const std::string& user,
-                                                  int license_type);
   void OnCookieWaitTimeout();
 
  private:
   // Handlers for WebUI messages.
   void HandleToggleFakeEnrollmentAndCompleteLogin(const std::string& user,
+                                                  const std::string& gaia_id,
+                                                  const std::string& password,
+                                                  bool using_saml,
                                                   int license_type);
   void HandleClose(const std::string& reason);
-  void HandleCompleteLogin(const std::string& user, int license_type);
-  void OnGetCookiesForCompleteLogin(
-      const std::string& user,
-      int license_type,
-      const net::CookieAccessResultList& cookies,
-      const net::CookieAccessResultList& excluded_cookies);
+  void HandleCompleteLogin(const std::string& user,
+                           const std::string& gaia_id,
+                           const std::string& password,
+                           bool using_saml,
+                           int license_type);
+  void CompleteAuthWithCookies(login::OnlineSigninArtifacts,
+                               int license_type,
+                               login::GaiaCookiesData cookies);
   void HandleIdentifierEntered(const std::string& email);
   void HandleRetry();
   void HandleFrameLoadingCompleted();
@@ -129,8 +130,8 @@ class EnrollmentScreenHandler : public BaseScreenHandler,
   // Shows the screen with the given data dictionary.
   void DoShowWithData(base::Value::Dict screen_data);
 
-  // Screen data to be passed to web ui for attestation enrollment.
-  base::Value::Dict ScreenDataForAttestationEnrollment();
+  // Screen data to be passed to web ui for automatic enrollment.
+  base::Value::Dict ScreenDataForAutomaticEnrollment();
 
   // Screen data to be passed to web ui for gaia oauth-based enrollment.
   base::Value::Dict ScreenDataForOAuthEnrollment();
@@ -166,7 +167,7 @@ class EnrollmentScreenHandler : public BaseScreenHandler,
   // Help application used for help dialogs.
   scoped_refptr<HelpAppLauncher> help_app_;
 
-  std::unique_ptr<CookieWaiter> oauth_code_waiter_;
+  std::unique_ptr<GaiaCookieRetriever> gaia_cookie_retriever_;
 
   bool use_fake_login_for_testing_ = false;
 
