@@ -6,22 +6,27 @@
 
 #include <string>
 
+#include "base/trace_event/trace_event.h"
 #include "base/values.h"
 #include "chrome/browser/ash/login/screens/locale_switch_screen.h"
+#include "chrome/browser/ui/ash/login/login_display_host.h"
 #include "chrome/browser/ui/webui/ash/login/core_oobe_handler.h"
 #include "chrome/browser/ui/webui/ash/login/oobe_ui.h"
 
 namespace ash {
 
-LocaleSwitchScreenHandler::LocaleSwitchScreenHandler(
-    CoreOobeView* core_oobe_view)
-    : BaseScreenHandler(kScreenId), core_oobe_view_(core_oobe_view) {}
+LocaleSwitchScreenHandler::LocaleSwitchScreenHandler()
+    : BaseScreenHandler(kScreenId) {}
 
 LocaleSwitchScreenHandler::~LocaleSwitchScreenHandler() = default;
 
 void LocaleSwitchScreenHandler::UpdateStrings() {
-  base::Value::Dict localized_strings = GetOobeUI()->GetLocalizedStrings();
-  core_oobe_view_->ReloadContent(std::move(localized_strings));
+  TRACE_EVENT0("login", "LocaleSwitchScreenHandler::UpdateStrings");
+  GetOobeUI()->GetCoreOobe()->ReloadContent();
+}
+
+base::WeakPtr<LocaleSwitchView> LocaleSwitchScreenHandler::AsWeakPtr() {
+  return weak_ptr_factory_.GetWeakPtr();
 }
 
 void LocaleSwitchScreenHandler::DeclareLocalizedValues(
