@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/ui/webui/ash/office_fallback/office_fallback_ui.h"
 
 #include <utility>
@@ -36,11 +41,11 @@ OfficeFallbackUI::OfficeFallbackUI(content::WebUI* web_ui)
   static constexpr webui::LocalizedString kStrings[] = {
       {"officeFallbackCancel", IDS_OFFICE_FALLBACK_CANCEL},
       {"officeFallbackTryAgain", IDS_OFFICE_FALLBACK_TRY_AGAIN},
-      {"officeFallbackOpenWithOfflineEditor",
-       IDS_OFFICE_FALLBACK_OPEN_WITH_OFFLINE_EDITOR},
+      {"officeFallbackOk", IDS_OFFICE_FALLBACK_OK},
+      {"officeFallbackOpenInBasicEditor",
+       IDS_OFFICE_FALLBACK_OPEN_IN_BASIC_EDITOR},
   };
   source->AddLocalizedStrings(kStrings);
-  source->AddBoolean("isJellyEnabled", chromeos::features::IsJellyEnabled());
   webui::SetupWebUIDataSource(
       source,
       base::make_span(kOfficeFallbackResources, kOfficeFallbackResourcesSize),
@@ -71,6 +76,9 @@ void OfficeFallbackUI::CloseDialog(mojom::DialogChoice choice) {
     case mojom::DialogChoice::kCancel:
       args.Append(kDialogChoiceCancel);
       break;
+    case mojom::DialogChoice::kOk:
+      args.Append(kDialogChoiceOk);
+      break;
     case mojom::DialogChoice::kQuickOffice:
       args.Append(kDialogChoiceQuickOffice);
       break;
