@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/ui/webui/ash/settings/os_settings_ui.h"
 
 #include <utility>
@@ -13,6 +18,9 @@
 #include "ash/public/cpp/esim_manager.h"
 #include "ash/public/cpp/hotspot_config_service.h"
 #include "ash/public/cpp/network_config_service.h"
+#include "ash/public/mojom/hid_preserving_bluetooth_state_controller.mojom.h"
+#include "ash/system/bluetooth/hid_preserving_controller/hid_preserving_bluetooth_state_service.h"
+#include "ash/webui/common/mojom/accelerator_fetcher.mojom.h"
 #include "ash/webui/common/trusted_types_util.h"
 #include "ash/webui/personalization_app/search/search.mojom.h"
 #include "ash/webui/personalization_app/search/search_handler.h"
@@ -32,7 +40,9 @@
 #include "chrome/browser/nearby_sharing/nearby_sharing_service_factory.h"
 #include "chrome/browser/nearby_sharing/nearby_sharing_service_impl.h"
 #include "chrome/browser/ui/webui/ash/settings/pages/apps/app_notification_handler.h"
+#include "chrome/browser/ui/webui/ash/settings/pages/apps/app_parental_controls_handler.h"
 #include "chrome/browser/ui/webui/ash/settings/pages/privacy/app_permission_handler.h"
+#include "chrome/browser/ui/webui/ash/settings/pages/search/magic_boost_notice_page_handler_factory.h"
 #include "chrome/browser/ui/webui/ash/settings/pages/storage/device_storage_handler.h"
 #include "chrome/browser/ui/webui/ash/settings/pref_names.h"
 #include "chrome/browser/ui/webui/ash/settings/search/search_handler.h"
@@ -42,6 +52,7 @@
 #include "chrome/browser/ui/webui/ash/settings/services/settings_manager/os_settings_manager.h"
 #include "chrome/browser/ui/webui/ash/settings/services/settings_manager/os_settings_manager_factory.h"
 #include "chrome/browser/ui/webui/managed_ui_handler.h"
+#include "chrome/browser/ui/webui/sanitized_image_source.h"
 #include "chrome/browser/ui/webui/webui_util.h"
 #include "chrome/common/webui_url_constants.h"
 #include "chrome/grit/os_settings_resources.h"
@@ -49,12 +60,17 @@
 #include "chromeos/ash/services/auth_factor_config/in_process_instances.h"
 #include "chromeos/ash/services/cellular_setup/cellular_setup_impl.h"
 #include "chromeos/ash/services/cellular_setup/public/mojom/esim_manager.mojom.h"
+#include "chromeos/ash/services/ime/public/mojom/ime_service.mojom.h"
+#include "chromeos/ash/services/ime/public/mojom/input_method_user_data.mojom.h"
+#include "chromeos/components/in_session_auth/in_process_instances.h"
+#include "chromeos/components/in_session_auth/mojom/in_session_auth.mojom.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "components/prefs/pref_service.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/browser/web_ui_data_source.h"
 #include "mojo/public/cpp/bindings/pending_receiver.h"
 #include "mojo/public/cpp/bindings/self_owned_receiver.h"
+#include "ui/base/ime/ash/input_method_manager.h"
 #include "ui/gfx/native_widget_types.h"
 #include "ui/views/widget/widget.h"
 #include "ui/webui/color_change_listener/color_change_handler.h"
@@ -100,7 +116,8 @@ OSSettingsUI::OSSettingsUI(content::WebUI* web_ui)
   content::WebUIDataSource* html_source =
       content::WebUIDataSource::CreateAndAdd(profile,
                                              chrome::kChromeUIOSSettingsHost);
-
+  content::URLDataSource::Add(profile,
+                              std::make_unique<SanitizedImageSource>(profile));
   OsSettingsManager* manager = OsSettingsManagerFactory::GetForProfile(profile);
   manager->AddHandlers(web_ui);
   manager->AddLoadTimeData(html_source);
@@ -163,6 +180,12 @@ OSSettingsUI::~OSSettingsUI() {
   // background and the state remains stored in the manager, so we will reset
   // that knowledge.
   settingsHatsManager->SetSettingsUsedSearch(false);
+
+  // Resets the tracking of device IDs associated with notification clicks.
+  // This method is called when the Settings app is closed to prevent the
+  // recording of metrics if a user changes settings long after clicking a
+  // notification.
+  InputDeviceSettingsController::Get()->ResetNotificationDeviceTracking();
 }
 
 void OSSettingsUI::BindInterface(
@@ -216,6 +239,14 @@ void OSSettingsUI::BindInterface(
       ->BindInterface(std::move(receiver));
 }
 
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<
+        app_parental_controls::mojom::AppParentalControlsHandler> receiver) {
+  OsSettingsManagerFactory::GetForProfile(Profile::FromWebUI(web_ui()))
+      ->app_parental_controls_handler()
+      ->BindInterface(std::move(receiver));
+}
+
 void OSSettingsUI::BindInterface(
     mojo::PendingReceiver<app_permission::mojom::AppPermissionsHandler>
         receiver) {
@@ -310,6 +341,14 @@ void OSSettingsUI::BindInterface(
       ->BindInterface(std::move(receiver));
 }
 
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<::ash::common::mojom::AcceleratorFetcher> receiver) {
+  CHECK(::features::IsShortcutCustomizationEnabled());
+  OsSettingsManagerFactory::GetForProfile(Profile::FromWebUI(web_ui()))
+      ->accelerator_fetcher()
+      ->BindInterface(std::move(receiver));
+}
+
 void OSSettingsUI::BindInterface(
     mojo::PendingReceiver<::ash::common::mojom::ShortcutInputProvider>
         receiver) {
@@ -361,6 +400,11 @@ void OSSettingsUI::BindInterface(
       g_browser_process->local_state());
 }
 
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<chromeos::auth::mojom::InSessionAuth> receiver) {
+  chromeos::auth::BindToInSessionAuthService(std::move(receiver));
+}
+
 void OSSettingsUI::BindInterface(
     mojo::PendingReceiver<google_drive::mojom::PageHandlerFactory> receiver) {
   // The PageHandlerFactory is reused across same-origin navigations, so ensure
@@ -384,6 +428,34 @@ void OSSettingsUI::BindInterface(
   ash::GetPasspointService(std::move(receiver));
 }
 
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<ash::mojom::HidPreservingBluetoothStateController>
+        receiver) {
+  DCHECK(features::IsBluetoothDisconnectWarningEnabled());
+  GetHidPreservingBluetoothStateControllerService(std::move(receiver));
+}
+
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<ash::ime::mojom::InputMethodUserDataService>
+        receiver) {
+  input_method::InputMethodManager::Get()->BindInputMethodUserDataService(
+      std::move(receiver));
+}
+
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<date_time::mojom::PageHandlerFactory> receiver) {
+  date_time_handler_factory_ = std::make_unique<DateTimeHandlerFactory>(
+      web_ui(), Profile::FromWebUI(web_ui()), std::move(receiver));
+}
+
+void OSSettingsUI::BindInterface(
+    mojo::PendingReceiver<magic_boost_handler::mojom::PageHandlerFactory>
+        receiver) {
+  magic_boost_notice_page_handler_factory_ =
+      std::make_unique<MagicBoostNoticePageHandlerFactory>(
+          Profile::FromWebUI(web_ui()), std::move(receiver));
+}
+
 WEB_UI_CONTROLLER_TYPE_IMPL(OSSettingsUI)
 
 }  // namespace ash::settings
