@@ -11,9 +11,10 @@
 #include "base/functional/bind.h"
 #include "base/metrics/histogram_functions.h"
 #include "base/no_destructor.h"
-#include "chrome/browser/ui/webui/ash/bluetooth_shared_load_time_data_provider.h"
+#include "chrome/browser/ui/webui/ash/bluetooth/bluetooth_shared_load_time_data_provider.h"
 #include "chrome/browser/ui/webui/ash/settings/pages/bluetooth/bluetooth_handler.h"
 #include "chrome/browser/ui/webui/ash/settings/pages/bluetooth/fast_pair_saved_devices_handler.h"
+#include "chrome/browser/ui/webui/ash/settings/pages/bluetooth/fast_pair_software_scanning_handler.h"
 #include "chrome/browser/ui/webui/ash/settings/search/mojom/search.mojom.h"
 #include "chrome/browser/ui/webui/ash/settings/search/mojom/search_result_icon.mojom.h"
 #include "chrome/browser/ui/webui/ash/settings/search/search_tag_registry.h"
@@ -334,6 +335,8 @@ void BluetoothSection::AddLoadTimeData(content::WebUIDataSource* html_source) {
       "bluetoothRevampHatsSurveyFlag",
       base::FeatureList::IsEnabled(
           ::features::kHappinessTrackingSystemBluetoothRevamp));
+  html_source->AddBoolean("bluetoothDisconnectWarningFlag",
+                          features::IsBluetoothDisconnectWarningEnabled());
   bluetooth::AddLoadTimeData(html_source);
 }
 
@@ -344,6 +347,12 @@ void BluetoothSection::AddHandlers(content::WebUI* web_ui) {
       features::IsFastPairSavedDevicesEnabled()) {
     web_ui->AddMessageHandler(std::make_unique<FastPairSavedDevicesHandler>());
   }
+
+  if (features::IsFastPairSoftwareScanningSupportEnabled()) {
+    web_ui->AddMessageHandler(std::make_unique<FastPairSoftwareScanningHandler>(
+        std::make_unique<quick_pair::BatterySaverActiveProvider>(),
+        std::make_unique<quick_pair::HardwareOffloadingSupportedProvider>()));
+  }
 }
 
 int BluetoothSection::GetSectionNameMessageId() const {
@@ -459,8 +468,7 @@ void BluetoothSection::UpdateSearchTags() {
   updater.AddSearchTags(GetBluetoothSearchConcepts());
 
   if (features::IsFastPairEnabled() &&
-      base::FeatureList::IsEnabled(
-          ash::features::kAllowCrossDeviceFeatureSuite)) {
+      features::IsCrossDeviceFeatureSuiteAllowed()) {
     if (pref_service_->GetBoolean(ash::prefs::kFastPairEnabled)) {
       updater.AddSearchTags(GetFastPairOnSearchConcepts());
     } else {
