@@ -5,18 +5,23 @@
 #include <cstdint>
 #include <memory>
 
+#include "ash/accelerators/accelerator_lookup.h"
 #include "ash/ash_element_identifiers.h"
+#include "ash/public/cpp/accelerator_actions.h"
+#include "ash/shell.h"
 #include "ash/webui/settings/public/constants/routes.mojom.h"
 #include "base/check.h"
+#include "base/strings/stringprintf.h"
 #include "base/test/gtest_tags.h"
 #include "chrome/browser/ui/chrome_pages.h"
 #include "chrome/browser/ui/settings_window_manager_chromeos.h"
-#include "chrome/test/base/chromeos/crosier/interactive_ash_test.h"
+#include "chrome/test/base/ash/interactive/interactive_ash_test.h"
 #include "chrome/test/interaction/webcontents_interaction_test_util.h"
 #include "chromeos/ash/components/audio/cras_audio_handler.h"
 #include "ui/base/interaction/element_identifier.h"
 #include "ui/base/interaction/interaction_sequence.h"
 #include "ui/base/interaction/state_observer.h"
+#include "ui/events/event_constants.h"
 
 namespace ash {
 
@@ -48,12 +53,15 @@ constexpr char kOutputSliderSelector[] = "#outputVolumeSlider";
 constexpr char kInputDeviceDropdownSelector[] = "#audioInputDeviceDropdown";
 constexpr char kInputMuteSelector[] = "#audioInputGainMuteButton";
 constexpr char kInputSliderSelector[] = "#audioInputGainVolumeSlider";
+constexpr char kInputNoiseCancellationToggle[] =
+    "#audioInputNoiseCancellationToggle";
 
 // Devices' ID configured here:
 // chromeos/ash/components/dbus/audio/fake_cras_audio_client.cc.
 constexpr uint64_t fake_internal_speaker = 0x100000001;
 constexpr uint64_t fake_headphone = 0x200000001;
 constexpr uint64_t fake_internal_mic = 0x100000002;
+constexpr uint64_t fake_usb_mic = 0x200000002;
 
 // ActiveAudioNodeStateObserver tracks when primary input or output device
 // changes. Returns state change with primary active device ID depending for
@@ -134,6 +142,7 @@ class AudioSettingsInteractiveUiTest : public InteractiveAshTest {
 
     // Ensure the OS Settings system web app (SWA) is installed.
     InstallSystemApps();
+    SetupContextWidget();
 
     audio_handler_ = ash::CrasAudioHandler::Get();
   }
@@ -177,31 +186,32 @@ class AudioSettingsInteractiveUiTest : public InteractiveAshTest {
     DCHECK(audio_handler());
     const uint64_t primary_node = audio_handler()->GetPrimaryActiveInputNode();
 
-    return Steps(
-        If([primary_node, device_id]() { return primary_node != device_id; },
-           Steps(Log(std::format(
-                     "Waiting for primary input device to match node ID: {0}",
-                     device_id)),
-                 WaitForState(kActiveInputNodeState, device_id))));
+    return Steps(If(
+        [primary_node, device_id]() { return primary_node != device_id; },
+        Steps(Log(base::StringPrintf(
+                  "Waiting for primary input device to match node ID: %" PRIu64,
+                  device_id)),
+              WaitForState(kActiveInputNodeState, device_id))));
   }
 
   auto MaybeWaitForOutputDevice(const uint64_t device_id) {
     DCHECK(audio_handler());
     const uint64_t primary_node = audio_handler()->GetPrimaryActiveOutputNode();
 
-    return Steps(
-        If([primary_node, device_id]() { return primary_node != device_id; },
-           Steps(Log(std::format(
-                     "Waiting for primary output device to match node ID: {0}",
-                     device_id)),
-                 WaitForState(kActiveOutputNodeState, device_id))));
+    return Steps(If(
+        [primary_node, device_id]() { return primary_node != device_id; },
+        Steps(
+            Log(base::StringPrintf(
+                "Waiting for primary output device to match node ID: %" PRIu64,
+                device_id)),
+            WaitForState(kActiveOutputNodeState, device_id))));
   }
 
   // Wait for an element described by `selector` to exists. Valid selector
   // restrictions come from `CreateAudioPageDeepQueryForSelector`.
   auto WaitForAudioElementExists(const std::string& selector) {
     return Steps(
-        Log(std::format("Wait for {0}", selector)),
+        Log(base::StringPrintf("Wait for %s", selector.c_str())),
         WaitForElementExists(kOsSettingsElementId,
                              CreateAudioPageDeepQueryForSelector(selector)));
   }
@@ -214,9 +224,8 @@ class AudioSettingsInteractiveUiTest : public InteractiveAshTest {
       const AudioDevice* audio_device =
           audio_handler()->GetDeviceFromId(device_id);
       DCHECK(audio_device);
-      audio_handler()->SwitchToDevice(
-          *audio_device, /*notify=*/true,
-          CrasAudioHandler::DeviceActivateType::ACTIVATE_BY_USER);
+      audio_handler()->SwitchToDevice(*audio_device, /*notify=*/true,
+                                      DeviceActivateType::kActivateByUser);
       if (audio_device->is_input) {
         MaybeWaitForInputDevice(device_id);
       } else {
@@ -229,6 +238,14 @@ class AudioSettingsInteractiveUiTest : public InteractiveAshTest {
     return Steps(ExecuteJsAt(kOsSettingsElementId, query, "el => el.focus()"));
   }
 
+  ui::Accelerator GetAccelerator(uint32_t action) {
+    return Shell::Get()
+        ->accelerator_lookup()
+        ->GetAcceleratorsForAction(action)
+        .front()
+        .accelerator;
+  }
+
   CrasAudioHandler* audio_handler() const { return audio_handler_; }
 
  private:
@@ -240,7 +257,6 @@ class AudioSettingsInteractiveUiTest : public InteractiveAshTest {
 // devices exist.
 IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, RenderAudioPage) {
   base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
-  SetupContextWidget();
 
   RunTestSequence(
       Log("Setup active node changed state observers"),
@@ -277,11 +293,11 @@ IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, RenderAudioPage) {
       Log("Expected input controls exist"));
 }
 
-// Verify changing output device is reflected in UI.
-IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ChangeOutputDevice) {
+// Verify changing active input and output device is reflected in UI.
+IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ChangeActiveDevice) {
   DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kFakeHeadphoneActiveEvent);
+  DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kFakeUsbMicActiveEvent);
   base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
-  SetupContextWidget();
 
   StateChange fake_headphone_active;
   fake_headphone_active.type = StateChange::Type::kExistsAndConditionTrue;
@@ -293,17 +309,31 @@ IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ChangeOutputDevice) {
       "el => el.children[1].text.includes('Headphone') && "
       "el.children[1].selected";
 
+  StateChange fake_usb_mic_active;
+  fake_usb_mic_active.type = StateChange::Type::kExistsAndConditionTrue;
+  fake_usb_mic_active.event = kFakeUsbMicActiveEvent;
+  fake_usb_mic_active.where =
+      CreateAudioPageDeepQueryForSelector(kInputDeviceDropdownSelector);
+  // Fake usb mic is the second dropdown option.
+  fake_usb_mic_active.test_function =
+      "el => el.children[1].text.includes('Mic (USB)') && "
+      "el.children[1].selected";
+
   RunTestSequence(
-      // Set fake headphone as active output device.
+      // Set fake headphone and usb mic as active output and input device.
       DoSetActiveDevice(fake_headphone),
       Log("Expected headphone output device configured"),
+      DoSetActiveDevice(fake_usb_mic),
+      Log("Expected usb mic input device configured"),
 
       Log("Open audio settings page and ensure it exists"),
       LoadAudioSettingsPage(),
 
-      // Test that headphone is selected.
+      // Test that headphone and usb mic is selected.
       WaitForStateChange(kOsSettingsElementId, fake_headphone_active),
-      Log("Expected headphone is selected in the active output dropdown"));
+      Log("Expected headphone is selected in the active output dropdown"),
+      WaitForStateChange(kOsSettingsElementId, fake_usb_mic_active),
+      Log("Expected usb mic is selected in the active input dropdown"));
 }
 
 // Verify changing mute state in UI is reflected in cras.
@@ -311,7 +341,6 @@ IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ToggleMuteStatus) {
   DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kFakeInternalMicMutedEvent);
   DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kFakeInternalSpeakerMutedEvent);
   base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
-  SetupContextWidget();
 
   // Ensure input and output are not muted in cras.
   EXPECT_FALSE(audio_handler()->IsInputMuted());
@@ -368,7 +397,6 @@ IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ToggleMuteStatus) {
 IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ChangeOutputVolume) {
   DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kFakeInternalSpeakerExists);
   base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
-  SetupContextWidget();
 
   int initial_volume = audio_handler()->GetOutputVolumePercent();
 
@@ -410,6 +438,155 @@ IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ChangeOutputVolume) {
   EXPECT_GE(audio_handler()->GetOutputVolumePercent(), initial_volume);
 }
 
+// Verify changing input volume in UI is reflected in cras.
+IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest, ChangeInputVolume) {
+  DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kFakeInternalMicExists);
+  base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
+
+  int initial_volume = audio_handler()->GetInputGainPercent();
+
+  StateChange fake_internal_mic_exists;
+  fake_internal_mic_exists.type = StateChange::Type::kExists;
+  fake_internal_mic_exists.event = kFakeInternalMicExists;
+  fake_internal_mic_exists.where =
+      CreateAudioPageDeepQueryForSelector(kInputSliderSelector);
+
+  RunTestSequence(
+      // Set fake internal mic as active input device.
+      DoSetActiveDevice(fake_internal_mic),
+      Log("Expected internal mic input device configured"),
+
+      Log("Open audio settings page and ensure it exists"),
+      LoadAudioSettingsPage(),
+
+      Log("Move input volume slider towards left"),
+      FocusElement(CreateAudioPageDeepQueryForSelector(kInputSliderSelector)),
+      SendAccelerator(
+          kOsSettingsElementId,
+          ui::Accelerator{ui::KeyboardCode::VKEY_LEFT, ui::EF_NONE}),
+
+      WaitForStateChange(kOsSettingsElementId, fake_internal_mic_exists));
+
+  // Expect that input volume has decreased.
+  EXPECT_LE(audio_handler()->GetInputGainPercent(), initial_volume);
+  initial_volume = audio_handler()->GetInputGainPercent();
+
+  RunTestSequence(
+      Log("Move input volume slider towards right"),
+      SendAccelerator(
+          kOsSettingsElementId,
+          ui::Accelerator{ui::KeyboardCode::VKEY_RIGHT, ui::EF_NONE}),
+
+      WaitForStateChange(kOsSettingsElementId, fake_internal_mic_exists));
+
+  // Expect that input volume has increased.
+  EXPECT_GE(audio_handler()->GetInputGainPercent(), initial_volume);
+}
+
+// Verify toggling input noise cancellation in UI is reflected in cras.
+IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest,
+                       ToggleInputNoiseCancellation) {
+  DEFINE_LOCAL_CUSTOM_ELEMENT_EVENT_TYPE(kInputNoiseCancellationToggleEvent);
+  base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
+
+  // Set noise cancellation as supported.
+  audio_handler()->SetNoiseCancellationSupportedForTesting(true);
+
+  // Expect noise cancellation is default to off.
+  EXPECT_FALSE(audio_handler()->GetNoiseCancellationState());
+
+  StateChange input_noise_cancellation_checked;
+  input_noise_cancellation_checked.type =
+      StateChange::Type::kExistsAndConditionTrue;
+  input_noise_cancellation_checked.event = kInputNoiseCancellationToggleEvent;
+  input_noise_cancellation_checked.where =
+      CreateAudioPageDeepQueryForSelector(kInputNoiseCancellationToggle);
+  input_noise_cancellation_checked.test_function = "toggle => toggle.checked";
+
+  RunTestSequence(
+      // Set fake internal mic as active device.
+      DoSetActiveDevice(fake_internal_mic),
+      Log("Expected internal mic configured"),
+
+      Log("Open audio settings page and ensure it exists"),
+      LoadAudioSettingsPage(),
+
+      Log("Toggle input noise cancellation in UI"),
+      ClickElement(kOsSettingsElementId, CreateAudioPageDeepQueryForSelector(
+                                             kInputNoiseCancellationToggle)),
+
+      // Test input noise cancallation is checked in UI.
+      WaitForStateChange(kOsSettingsElementId,
+                         input_noise_cancellation_checked),
+      Log("Expected input noise cancallation is checked in UI"));
+
+  // Expect noise cancellation is on now.
+  EXPECT_TRUE(audio_handler()->GetNoiseCancellationState());
+
+  StateChange input_noise_cancellation_unchecked;
+  input_noise_cancellation_unchecked.type =
+      StateChange::Type::kExistsAndConditionTrue;
+  input_noise_cancellation_unchecked.event = kInputNoiseCancellationToggleEvent;
+  input_noise_cancellation_unchecked.where =
+      CreateAudioPageDeepQueryForSelector(kInputNoiseCancellationToggle);
+  input_noise_cancellation_unchecked.test_function =
+      "toggle => !toggle.checked";
+
+  RunTestSequence(
+      Log("Toggle input noise cancellation in UI"),
+      ClickElement(kOsSettingsElementId, CreateAudioPageDeepQueryForSelector(
+                                             kInputNoiseCancellationToggle)),
+
+      // Test input noise cancallation is unchecked in UI.
+      WaitForStateChange(kOsSettingsElementId,
+                         input_noise_cancellation_unchecked),
+      Log("Expected input noise cancallation is unchecked in UI"));
+
+  // Expect noise cancellation is off now.
+  EXPECT_FALSE(audio_handler()->GetNoiseCancellationState());
+}
+
+// Verify quick settings button to launch audio settings is disabled on lock
+// screen.
+IN_PROC_BROWSER_TEST_F(AudioSettingsInteractiveUiTest,
+                       LaunchAudioSettingDisabledOnLockScreen) {
+  base::AddFeatureIdTagToTestResult(kAudioSettingsFeatureIdTag);
+
+  RunTestSequence(
+      Log("Open quick settings bubble"),
+      PressButton(kUnifiedSystemTrayElementId),
+      WaitForShow(kQuickSettingsViewElementId),
+
+      Log("Click audio detailed view button"),
+      PressButton(kQuickSettingsAudioDetailedViewButtonElementId),
+      WaitForShow(kQuickSettingsAudioDetailedViewAudioSettingsButtonElementId),
+
+      CheckViewProperty(
+          kQuickSettingsAudioDetailedViewAudioSettingsButtonElementId,
+          &views::Button::GetEnabled, true),
+      Log("Expected audio settings button is enabled"),
+
+      Log("Open audio settings page and ensure it exists"),
+      LoadAudioSettingsPage(),
+
+      Log("Lock screen"),
+      SendAccelerator(kOsSettingsElementId,
+                      GetAccelerator(AcceleratorAction::kLockScreen)),
+
+      Log("Open quick settings bubble"),
+      PressButton(kUnifiedSystemTrayElementId),
+      WaitForShow(kQuickSettingsViewElementId),
+
+      Log("Click audio detailed view button"),
+      PressButton(kQuickSettingsAudioDetailedViewButtonElementId),
+      WaitForShow(kQuickSettingsAudioDetailedViewAudioSettingsButtonElementId),
+
+      CheckViewProperty(
+          kQuickSettingsAudioDetailedViewAudioSettingsButtonElementId,
+          &views::Button::GetEnabled, false),
+      Log("Expected audio settings button is disabled"));
+}
+
 }  // namespace
 
 }  // namespace ash
