@@ -8,14 +8,18 @@
 #include <vector>
 
 #include "ash/constants/ash_features.h"
+#include "ash/constants/url_constants.h"
 #include "ash/public/cpp/assistant/assistant_state.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/no_destructor.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/ash/assistant/assistant_util.h"
+#include "chrome/browser/ash/input_method/editor_mediator_factory.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/webui/ash/assistant_optin/assistant_optin_utils.h"
 #include "chrome/browser/ui/webui/ash/settings/pages/search/google_assistant_handler.h"
+#include "chrome/browser/ui/webui/ash/settings/search/search_concept.h"
 #include "chrome/browser/ui/webui/ash/settings/search/search_tag_registry.h"
 #include "chrome/browser/ui/webui/settings/search_engines_handler.h"
 #include "chrome/browser/ui/webui/webui_util.h"
@@ -23,6 +27,9 @@
 #include "chrome/grit/generated_resources.h"
 #include "chromeos/ash/services/assistant/public/cpp/assistant_prefs.h"
 #include "chromeos/ash/services/assistant/public/cpp/features.h"
+#include "chromeos/components/magic_boost/public/cpp/magic_boost_state.h"
+#include "chromeos/components/mahi/public/cpp/mahi_manager.h"
+#include "chromeos/components/quick_answers/public/cpp/quick_answers_state.h"
 #include "chromeos/constants/chromeos_features.h"
 #include "components/language/core/browser/pref_names.h"
 #include "components/prefs/pref_service.h"
@@ -46,13 +53,37 @@ using ::chromeos::settings::mojom::Subpage;
 
 namespace {
 
-bool ShouldShowQuickAnswersSettings() {
-  return QuickAnswersState::Get() && QuickAnswersState::Get()->is_eligible();
+// Whether Quick answers is supported for the current language.
+bool IsQuickAnswersSupported() {
+  return QuickAnswersState::IsEligibleAs(
+      QuickAnswersState::FeatureType::kQuickAnswers);
+}
+
+bool IsMagicBoostNoticeBannerVisible(Profile* profile) {
+  chromeos::MagicBoostState* magic_boost_state =
+      chromeos::MagicBoostState::Get();
+
+  bool hmr_needs_notice_banner = magic_boost_state->IsMagicBoostAvailable() &&
+                                 magic_boost_state->CanShowNoticeBannerForHMR();
+
+  bool hmw_needs_notice_banner = false;
+
+  if (chromeos::features::IsOrcaEnabled()) {
+    ash::input_method::EditorMediator* editor_mediator =
+        ash::input_method::EditorMediatorFactory::GetInstance()->GetForProfile(
+            profile);
+
+    hmw_needs_notice_banner = editor_mediator &&
+                              editor_mediator->IsAllowedForUse() &&
+                              editor_mediator->CanShowNoticeBanner();
+  }
+
+  return hmr_needs_notice_banner || hmw_needs_notice_banner;
 }
 
 const std::vector<SearchConcept>& GetSearchPageSearchConcepts(
     const char* section_path) {
-  if (ShouldShowQuickAnswersSettings()) {
+  if (IsQuickAnswersSupported()) {
     static const base::NoDestructor<std::vector<SearchConcept>> tags({
         {IDS_OS_SETTINGS_TAG_PREFERRED_SEARCH_ENGINE,
          mojom::kSearchSubpagePath,
@@ -192,6 +223,38 @@ const std::vector<SearchConcept>& GetAssistantVoiceMatchSearchConcepts() {
   return *tags;
 }
 
+const std::vector<SearchConcept>& GetMagicBoostSearchConcepts(
+    const char* section_path) {
+  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+      {IDS_OS_SETTINGS_TAG_MAGIC_BOOST,
+       section_path,
+       mojom::SearchResultIcon::kMagicBoost,
+       mojom::SearchResultDefaultRank::kMedium,
+       mojom::SearchResultType::kSetting,
+       {.setting = mojom::Setting::kMagicBoostOnOff}},
+  });
+  return *tags;
+}
+
+const std::vector<SearchConcept>& GetMagicBoostSubSearchConcepts(
+    const char* section_path) {
+  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+      {IDS_OS_SETTINGS_TAG_MAGIC_BOOST_HMR,
+       section_path,
+       mojom::SearchResultIcon::kHelpMeRead,
+       mojom::SearchResultDefaultRank::kMedium,
+       mojom::SearchResultType::kSetting,
+       {.setting = mojom::Setting::kMahiOnOff}},
+      {IDS_OS_SETTINGS_TAG_MAGIC_BOOST_HMW,
+       section_path,
+       mojom::SearchResultIcon::kHelpMeWrite,
+       mojom::SearchResultDefaultRank::kMedium,
+       mojom::SearchResultType::kSetting,
+       {.setting = mojom::Setting::kShowOrca}},
+  });
+  return *tags;
+}
+
 bool IsVoiceMatchAllowed() {
   return !assistant::features::IsVoiceMatchDisabled();
 }
@@ -284,6 +347,13 @@ SearchSection::SearchSection(Profile* profile,
     QuickAnswersState::Get()->AddObserver(this);
     UpdateQuickAnswersSearchTags();
   }
+
+  auto* magic_boost_state = chromeos::MagicBoostState::Get();
+  if (magic_boost_state && magic_boost_state->IsMagicBoostAvailable()) {
+    updater.AddSearchTags(GetMagicBoostSearchConcepts(GetSectionPath()));
+    magic_boost_state->AddObserver(this);
+    UpdateSubMagicBoostSearchTags();
+  }
 }
 
 SearchSection::~SearchSection() {
@@ -298,6 +368,18 @@ void SearchSection::AddLoadTimeData(content::WebUIDataSource* html_source) {
       ash::features::IsOsSettingsRevampWayfindingEnabled();
 
   webui::LocalizedString kLocalizedStrings[] = {
+      {"enableMagicBoost", IDS_OS_SETTINGS_ENABLE_MAGIC_BOOST},
+      {"enableMagicBoostDesc", IDS_OS_SETTINGS_ENABLE_MAGIC_BOOST_DESCRIPTION},
+      {"magicBoostReviewTermsBannerDescription",
+       IDS_OS_SETTINGS_MAGIC_BOOST_REVIEW_TERMS_BANNER_DESCRIPTION},
+      {"magicBoostReviewTermsButtonLabel",
+       IDS_OS_SETTINGS_MAGIC_BOOST_REVIEW_TERMS_BUTTON_LABEL},
+      {"enableMagicBoostDesc", IDS_OS_SETTINGS_ENABLE_MAGIC_BOOST_DESCRIPTION},
+      {"enableHelpMeRead", IDS_OS_SETTINGS_ENABLE_HELP_ME_READ},
+      {"enableHelpMeReadDesc", IDS_OS_SETTINGS_ENABLE_HELP_ME_READ_DESCRIPTION},
+      {"enableHelpMeWrite", IDS_OS_SETTINGS_ENABLE_HELP_ME_WRITE},
+      {"enableHelpMeWriteDesc",
+       IDS_OS_SETTINGS_ENABLE_HELP_ME_WRITE_DESCRIPTION},
       {"osSearchEngineLabel", kIsRevampEnabled
                                   ? IDS_OS_SETTINGS_REVAMP_SEARCH_ENGINE_LABEL
                                   : IDS_OS_SETTINGS_SEARCH_ENGINE_LABEL},
@@ -314,8 +396,18 @@ void SearchSection::AddLoadTimeData(content::WebUIDataSource* html_source) {
   };
   html_source->AddLocalizedStrings(kLocalizedStrings);
 
-  html_source->AddBoolean("shouldShowQuickAnswersSettings",
-                          ShouldShowQuickAnswersSettings());
+  html_source->AddString("helpMeReadWriteLearnMoreUrl",
+                         chrome::kHelpMeReadWriteLearnMoreURL);
+
+  html_source->AddBoolean("isQuickAnswersSupported", IsQuickAnswersSupported());
+
+  html_source->AddBoolean(
+      "isMagicBoostFeatureEnabled",
+      chromeos::MagicBoostState::Get()->IsMagicBoostAvailable());
+
+  html_source->AddBoolean("isMagicBoostNoticeBannerVisible",
+                          IsMagicBoostNoticeBannerVisible(profile()));
+
   const bool is_assistant_allowed = IsAssistantAllowed();
   html_source->AddBoolean("isAssistantAllowed", is_assistant_allowed);
   html_source->AddLocalizedString("osSearchPageTitle",
@@ -359,17 +451,40 @@ const char* SearchSection::GetSectionPath() const {
 
 bool SearchSection::LogMetric(mojom::Setting setting,
                               base::Value& value) const {
-  // Unimplemented.
-  return false;
+  switch (setting) {
+    case mojom::Setting::kMagicBoostOnOff:
+      base::UmaHistogramBoolean("ChromeOS.Settings.MagicBoost.Enabled",
+                                value.GetBool());
+      return true;
+
+    case mojom::Setting::kMahiOnOff:
+      base::UmaHistogramBoolean(
+          "ChromeOS.Settings.MagicBoost.HelpMeReadEnabled", value.GetBool());
+      return true;
+
+    case mojom::Setting::kShowOrca:
+      base::UmaHistogramBoolean(
+          "ChromeOS.Settings.MagicBoost.HelpMeWriteEnabled", value.GetBool());
+      return true;
+
+    default:
+      return false;
+  }
 }
 
 void SearchSection::RegisterHierarchy(HierarchyGenerator* generator) const {
   // Register Preferred search engine as top level settings if Quick answers is
   // not available.
-  if (!ShouldShowQuickAnswersSettings()) {
+  if (!IsQuickAnswersSupported()) {
     generator->RegisterTopLevelSetting(mojom::Setting::kPreferredSearchEngine);
   }
 
+  // TODO(b:337868408): Setting::kShowOrca is already registered in
+  // device/input_section.cc, therefore UMA emitted from search_secion fails to
+  // log it.
+  generator->RegisterTopLevelSetting(mojom::Setting::kMahiOnOff);
+  generator->RegisterTopLevelSetting(mojom::Setting::kMagicBoostOnOff);
+
   // Search.
   generator->RegisterTopLevelSubpage(
       IDS_SETTINGS_SEARCH_SUBPAGE_TITLE, mojom::Subpage::kSearch,
@@ -377,7 +492,7 @@ void SearchSection::RegisterHierarchy(HierarchyGenerator* generator) const {
       mojom::kSearchSubpagePath);
   // Register Preferred search engine under Search subpage if Quick answers is
   // available.
-  if (ShouldShowQuickAnswersSettings()) {
+  if (IsQuickAnswersSupported()) {
     static constexpr mojom::Setting kSearchSettingsWithPreferredSearchEngine[] =
         {
             mojom::Setting::kQuickAnswersOnOff,
@@ -443,6 +558,14 @@ void SearchSection::OnEligibilityChanged(bool eligible) {
   UpdateQuickAnswersSearchTags();
 }
 
+void SearchSection::OnMagicBoostEnabledUpdated(bool enabled) {
+  UpdateSubMagicBoostSearchTags();
+}
+
+void SearchSection::OnIsDeleting() {
+  magic_boost_state_observation_.Reset();
+}
+
 bool SearchSection::IsAssistantAllowed() const {
   // NOTE: This will be false when the flag is disabled.
   return ::assistant::IsAssistantAllowedForProfile(profile()) ==
@@ -486,16 +609,30 @@ void SearchSection::UpdateQuickAnswersSearchTags() {
   updater.RemoveSearchTags(GetQuickAnswersSearchConcepts());
   updater.RemoveSearchTags(GetQuickAnswersOnSearchConcepts());
 
-  if (!ShouldShowQuickAnswersSettings()) {
+  if (!IsQuickAnswersSupported()) {
     return;
   }
 
   updater.AddSearchTags(GetQuickAnswersSearchConcepts());
 
   if (chromeos::features::IsQuickAnswersV2SettingsSubToggleEnabled() &&
-      QuickAnswersState::Get()->settings_enabled()) {
+      QuickAnswersState::IsEnabledAs(
+          QuickAnswersState::FeatureType::kQuickAnswers)) {
     updater.AddSearchTags(GetQuickAnswersOnSearchConcepts());
   }
 }
 
+void SearchSection::UpdateSubMagicBoostSearchTags() {
+  auto* magic_boost_state = chromeos::MagicBoostState::Get();
+  DCHECK(magic_boost_state);
+
+  SearchTagRegistry::ScopedTagUpdater updater = registry()->StartUpdate();
+
+  updater.RemoveSearchTags(GetMagicBoostSubSearchConcepts(GetSectionPath()));
+
+  if (magic_boost_state->magic_boost_enabled().value_or(false)) {
+    updater.AddSearchTags(GetMagicBoostSubSearchConcepts(GetSectionPath()));
+  }
+}
+
 }  // namespace ash::settings
