@@ -1,20 +1,20 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/webui/conflicts/conflicts_data_fetcher.h"
 
 #include <string>
+#include <string_view>
 #include <utility>
 
 #include "base/strings/string_util.h"
-#include "base/values.h"
-#include "base/win/windows_version.h"
+#include "build/branding_buildflags.h"
 #include "chrome/browser/win/conflicts/module_database.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 #include "base/win/win_util.h"
 #include "chrome/browser/win/conflicts/incompatible_applications_updater.h"
 #include "chrome/browser/win/conflicts/module_blocklist_cache_updater.h"
@@ -42,7 +42,7 @@ std::string GetProcessTypesString(const ModuleInfoData& module_data) {
   return result;
 }
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 // Strings used twice.
 constexpr char kNotLoaded[] = "Not loaded";
@@ -65,10 +65,10 @@ constexpr char kAllowedSameDirectory[] =
     "Allowed - In executable directory (dev builds only)";
 #endif
 
-void AppendString(base::StringPiece input, std::string* output) {
+void AppendString(std::string_view input, std::string* output) {
   if (!output->empty())
     *output += ", ";
-  output->append(input.data(), input.size());
+  output->append(input);
 }
 
 // Returns a string describing the current module blocking status: loaded or
@@ -103,7 +103,7 @@ std::string GetBlockingDecisionString(
   // startup.
   switch (blocking_state.blocking_decision) {
     case BlockingDecision::kUnknown:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
       break;
     case BlockingDecision::kNotLoaded:
       return kNotLoaded;
@@ -174,7 +174,7 @@ std::string GetModuleWarningDecisionString(
       return "Incompatible";
     case WarningDecision::kAddedToBlocklist:
     case WarningDecision::kUnknown:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
       break;
   }
 
@@ -216,13 +216,11 @@ std::string GetModuleStatusString(
 
   return status;
 }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 enum ThirdPartyFeaturesStatus {
   // The third-party features are not available in non-Google Chrome builds.
   kNonGoogleChromeBuild,
-  // The third-party features are not available on Windows 7.
-  kNotAvailableWin7,
   // The ThirdPartyBlockingEnabled group policy is disabled.
   kPolicyDisabled,
   // Both the IncompatibleApplicationsWarning and the
@@ -241,9 +239,9 @@ enum ThirdPartyFeaturesStatus {
   kWarningAndBlockingInitialized,
 };
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 ThirdPartyFeaturesStatus GetThirdPartyFeaturesStatus(
-    base::Optional<ThirdPartyConflictsManager::State>
+    std::optional<ThirdPartyConflictsManager::State>
         third_party_conflicts_manager_state) {
   // The ThirdPartyConflictsManager instance exists if we have its state.
   if (third_party_conflicts_manager_state.has_value()) {
@@ -265,10 +263,6 @@ ThirdPartyFeaturesStatus GetThirdPartyFeaturesStatus(
     }
   }
 
-  // Figure out why the manager instance doesn't exist.
-  if (base::win::GetVersion() <= base::win::Version::WIN7)
-    return kNotAvailableWin7;
-
   if (!ModuleDatabase::IsThirdPartyBlockingPolicyEnabled())
     return kPolicyDisabled;
 
@@ -279,7 +273,7 @@ ThirdPartyFeaturesStatus GetThirdPartyFeaturesStatus(
 
   // The above 3 cases are the only possible reasons why the manager wouldn't
   // exist.
-  NOTREACHED();
+  NOTREACHED_IN_MIGRATION();
   return kFeatureDisabled;
 }
 #endif
@@ -294,14 +288,9 @@ std::string GetThirdPartyFeaturesStatusString(ThirdPartyFeaturesStatus status) {
     case ThirdPartyFeaturesStatus::kNonGoogleChromeBuild:
       return "The third-party features are not available in non-Google Chrome "
              "builds.";
-    case ThirdPartyFeaturesStatus::kNotAvailableWin7:
-      return "The third-party features are not available on Windows 7.";
     case ThirdPartyFeaturesStatus::kPolicyDisabled:
       return "The ThirdPartyBlockingEnabled group policy is disabled.";
     case ThirdPartyFeaturesStatus::kFeatureDisabled:
-      if (base::win::GetVersion() < base::win::Version::WIN10)
-        return "The ThirdPartyModulesBlocking feature is disabled.";
-
       return "Both the IncompatibleApplicationsWarning and "
              "ThirdPartyModulesBlocking features are disabled.";
     case ThirdPartyFeaturesStatus::kModuleListInvalid:
@@ -309,17 +298,12 @@ std::string GetThirdPartyFeaturesStatusString(ThirdPartyFeaturesStatus status) {
     case ThirdPartyFeaturesStatus::kNoModuleListAvailable:
       return "Disabled - There is no Module List version available.";
     case ThirdPartyFeaturesStatus::kWarningInitialized:
-      DCHECK_GE(base::win::GetVersion(), base::win::Version::WIN10);
       return "The IncompatibleApplicationsWarning feature is enabled, while "
              "the ThirdPartyModulesBlocking feature is disabled.";
     case ThirdPartyFeaturesStatus::kBlockingInitialized:
-      if (base::win::GetVersion() < base::win::Version::WIN10)
-        return "The ThirdPartyModulesBlocking feature is enabled.";
-
       return "The ThirdPartyModulesBlocking feature is enabled, while the "
              "IncompatibleApplicationsWarning feature is disabled.";
     case ThirdPartyFeaturesStatus::kWarningAndBlockingInitialized:
-      DCHECK_GE(base::win::GetVersion(), base::win::Version::WIN10);
       return "Both the IncompatibleApplicationsWarning and "
              "ThirdPartyModulesBlocking features are enabled";
   }
@@ -328,36 +312,35 @@ std::string GetThirdPartyFeaturesStatusString(ThirdPartyFeaturesStatus status) {
 void OnConflictsDataFetched(
     ConflictsDataFetcher::OnConflictsDataFetchedCallback
         on_conflicts_data_fetched_callback,
-    base::DictionaryValue results,
+    base::Value::Dict results,
     ThirdPartyFeaturesStatus third_party_features_status) {
   // Third-party conflicts status.
-  results.SetBoolean("thirdPartyFeatureEnabled",
-                     IsThirdPartyFeatureEnabled(third_party_features_status));
-  results.SetString(
-      "thirdPartyFeatureStatus",
-      GetThirdPartyFeaturesStatusString(third_party_features_status));
+  results.Set("thirdPartyFeatureEnabled",
+              IsThirdPartyFeatureEnabled(third_party_features_status));
+  results.Set("thirdPartyFeatureStatus",
+              GetThirdPartyFeaturesStatusString(third_party_features_status));
 
   std::move(on_conflicts_data_fetched_callback).Run(std::move(results));
 }
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 void OnModuleDataFetched(ConflictsDataFetcher::OnConflictsDataFetchedCallback
                              on_conflicts_data_fetched_callback,
-                         base::DictionaryValue results,
-                         base::Optional<ThirdPartyConflictsManager::State>
+                         base::Value::Dict results,
+                         std::optional<ThirdPartyConflictsManager::State>
                              third_party_conflicts_manager_state) {
   OnConflictsDataFetched(
       std::move(on_conflicts_data_fetched_callback), std::move(results),
       GetThirdPartyFeaturesStatus(third_party_conflicts_manager_state));
 }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 }  // namespace
 
 ConflictsDataFetcher::~ConflictsDataFetcher() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  if (module_list_)
+  if (module_list_.has_value())
     ModuleDatabase::GetInstance()->RemoveObserver(this);
 }
 
@@ -373,7 +356,7 @@ ConflictsDataFetcher::ConflictsDataFetcher(
     OnConflictsDataFetchedCallback on_conflicts_data_fetched_callback)
     : on_conflicts_data_fetched_callback_(
           std::move(on_conflicts_data_fetched_callback))
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
       ,
       weak_ptr_factory_(this)
 #endif
@@ -388,7 +371,7 @@ ConflictsDataFetcher::ConflictsDataFetcher(
 }
 
 void ConflictsDataFetcher::InitializeOnModuleDatabaseTaskRunner() {
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
   // If the ThirdPartyConflictsManager instance exists, wait until it is fully
   // initialized before retrieving the list of modules.
   auto* third_party_conflicts_manager =
@@ -404,7 +387,7 @@ void ConflictsDataFetcher::InitializeOnModuleDatabaseTaskRunner() {
   GetListOfModules();
 }
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 void ConflictsDataFetcher::OnManagerInitializationComplete(
     ThirdPartyConflictsManager::State state) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
@@ -413,17 +396,17 @@ void ConflictsDataFetcher::OnManagerInitializationComplete(
 
   GetListOfModules();
 }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 void ConflictsDataFetcher::GetListOfModules() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   // The request is handled asynchronously, filling up the |module_list_|,
   // and will callback via OnModuleDatabaseIdle() on completion.
-  module_list_ = std::make_unique<base::ListValue>();
+  module_list_ = base::Value::List();
 
   auto* module_database = ModuleDatabase::GetInstance();
-  module_database->IncreaseInspectionPriority();
+  module_database->StartInspection();
   module_database->AddObserver(this);
 }
 
@@ -432,10 +415,10 @@ void ConflictsDataFetcher::OnNewModuleFound(const ModuleInfoKey& module_key,
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(module_list_);
 
-  auto data = std::make_unique<base::DictionaryValue>();
+  base::Value::Dict data;
 
-  data->SetString("third_party_module_status", std::string());
-#if defined(GOOGLE_CHROME_BUILD)
+  data.Set("third_party_module_status", std::string());
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
   if (ModuleDatabase::GetInstance()->third_party_conflicts_manager()) {
     auto* incompatible_applications_updater =
         ModuleDatabase::GetInstance()
@@ -446,27 +429,27 @@ void ConflictsDataFetcher::OnNewModuleFound(const ModuleInfoKey& module_key,
             ->third_party_conflicts_manager()
             ->module_blocklist_cache_updater();
 
-    data->SetString(
+    data.Set(
         "third_party_module_status",
         GetModuleStatusString(module_key, incompatible_applications_updater,
                               module_blocklist_cache_updater));
   }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
   std::string type_string;
   if (module_data.module_properties & ModuleInfoData::kPropertyShellExtension)
     type_string = "Shell extension";
-  data->SetString("type_description", type_string);
+  data.Set("type_description", type_string);
 
   const auto& inspection_result = *module_data.inspection_result;
-  data->SetString("location", inspection_result.location);
-  data->SetString("name", inspection_result.basename);
-  data->SetString("product_name", inspection_result.product_name);
-  data->SetString("description", inspection_result.description);
-  data->SetString("version", inspection_result.version);
-  data->SetString("digital_signer", inspection_result.certificate_info.subject);
-  data->SetString("code_id", GenerateCodeId(module_key));
-  data->SetString("process_types", GetProcessTypesString(module_data));
+  data.Set("location", inspection_result.location);
+  data.Set("name", inspection_result.basename);
+  data.Set("product_name", inspection_result.product_name);
+  data.Set("description", inspection_result.description);
+  data.Set("version", inspection_result.version);
+  data.Set("digital_signer", inspection_result.certificate_info.subject);
+  data.Set("code_id", GenerateCodeId(module_key));
+  data.Set("process_types", GetProcessTypesString(module_data));
 
   module_list_->Append(std::move(data));
 }
@@ -477,11 +460,12 @@ void ConflictsDataFetcher::OnModuleDatabaseIdle() {
 
   ModuleDatabase::GetInstance()->RemoveObserver(this);
 
-  base::DictionaryValue results;
-  results.SetInteger("moduleCount", module_list_->GetSize());
-  results.Set("moduleList", std::move(module_list_));
+  base::Value::Dict results;
+  results.Set("moduleCount", static_cast<int>(module_list_->size()));
+  results.Set("moduleList", std::move(*module_list_));
+  module_list_ = std::nullopt;
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
   // The state of third-party features must be determined on the UI thread.
   content::GetUIThreadTaskRunner({})->PostTask(
       FROM_HERE,
