@@ -10,6 +10,8 @@
 #include "base/base64.h"
 #include "base/functional/bind.h"
 #include "base/json/json_string_value_serializer.h"
+#include "base/memory/weak_ptr.h"
+#include "base/not_fatal_until.h"
 #include "base/task/bind_post_task.h"
 #include "base/task/cancelable_task_tracker.h"
 #include "base/task/sequenced_task_runner.h"
@@ -22,7 +24,6 @@
 #include "components/performance_manager/public/graph/node_data_describer.h"
 #include "components/performance_manager/public/graph/node_data_describer_registry.h"
 #include "components/performance_manager/public/performance_manager.h"
-#include "components/performance_manager/public/web_contents_proxy.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/web_contents.h"
@@ -54,7 +55,7 @@ class DiscardsGraphDumpImpl::FaviconRequestHelper {
   FaviconRequestHelper& operator=(const FaviconRequestHelper&) = delete;
 
   void RequestFavicon(GURL page_url,
-                      performance_manager::WebContentsProxy contents_proxy,
+                      base::WeakPtr<content::WebContents> web_contents,
                       FaviconAvailableCallback on_favicon_available);
   void FaviconDataAvailable(FaviconAvailableCallback on_favicon_available,
                             const favicon_base::FaviconRawBitmapResult& result);
@@ -66,10 +67,9 @@ class DiscardsGraphDumpImpl::FaviconRequestHelper {
 
 void DiscardsGraphDumpImpl::FaviconRequestHelper::RequestFavicon(
     GURL page_url,
-    performance_manager::WebContentsProxy contents_proxy,
+    base::WeakPtr<content::WebContents> web_contents,
     FaviconAvailableCallback on_favicon_available) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  content::WebContents* web_contents = contents_proxy.Get();
   if (!web_contents)
     return;
 
@@ -108,7 +108,6 @@ DiscardsGraphDumpImpl::DiscardsGraphDumpImpl() {}
 
 DiscardsGraphDumpImpl::~DiscardsGraphDumpImpl() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  DCHECK(!graph_);
   DCHECK(!change_subscriber_);
 }
 
@@ -156,20 +155,21 @@ void DiscardsGraphDumpImpl::SubscribeToChanges(
   change_subscriber_.Bind(std::move(change_subscriber));
 
   // Give all existing nodes an ID.
+  performance_manager::Graph* graph = GetOwningGraph();
   for (const performance_manager::FrameNode* frame_node :
-       graph_->GetAllFrameNodes()) {
+       graph->GetAllFrameNodes()) {
     AddNode(frame_node);
   }
   for (const performance_manager::PageNode* page_node :
-       graph_->GetAllPageNodes()) {
+       graph->GetAllPageNodes()) {
     AddNode(page_node);
   }
   for (const performance_manager::ProcessNode* process_node :
-       graph_->GetAllProcessNodes()) {
+       graph->GetAllProcessNodes()) {
     AddNode(process_node);
   }
   for (const performance_manager::WorkerNode* worker_node :
-       graph_->GetAllWorkerNodes()) {
+       graph->GetAllWorkerNodes()) {
     AddNode(worker_node);
   }
 
@@ -188,10 +188,10 @@ void DiscardsGraphDumpImpl::SubscribeToChanges(
   SendNotificationToAllNodes(/* created = */ false);
 
   // Subscribe to subsequent notifications.
-  graph_->AddFrameNodeObserver(this);
-  graph_->AddPageNodeObserver(this);
-  graph_->AddProcessNodeObserver(this);
-  graph_->AddWorkerNodeObserver(this);
+  graph->AddFrameNodeObserver(this);
+  graph->AddPageNodeObserver(this);
+  graph->AddProcessNodeObserver(this);
+  graph->AddWorkerNodeObserver(this);
 }
 
 void DiscardsGraphDumpImpl::RequestNodeDescriptions(
@@ -199,7 +199,7 @@ void DiscardsGraphDumpImpl::RequestNodeDescriptions(
     RequestNodeDescriptionsCallback callback) {
   base::flat_map<int64_t, std::string> descriptions;
   performance_manager::NodeDataDescriberRegistry* describer_registry =
-      graph_->GetNodeDataDescriberRegistry();
+      GetOwningGraph()->GetNodeDataDescriberRegistry();
   for (int64_t node_id : node_ids) {
     auto it = nodes_by_id_.find(NodeId::FromUnsafeValue(node_id));
     // The requested node may have been removed by the time the request arrives,
@@ -214,23 +214,19 @@ void DiscardsGraphDumpImpl::RequestNodeDescriptions(
 }
 
 void DiscardsGraphDumpImpl::OnPassedToGraph(performance_manager::Graph* graph) {
-  DCHECK(!graph_);
-  graph_ = graph;
+  // Do nothing.
 }
 
 void DiscardsGraphDumpImpl::OnTakenFromGraph(
     performance_manager::Graph* graph) {
-  DCHECK_EQ(graph_, graph);
-
   if (change_subscriber_) {
-    graph_->RemoveFrameNodeObserver(this);
-    graph_->RemovePageNodeObserver(this);
-    graph_->RemoveProcessNodeObserver(this);
-    graph_->RemoveWorkerNodeObserver(this);
+    graph->RemoveFrameNodeObserver(this);
+    graph->RemovePageNodeObserver(this);
+    graph->RemoveProcessNodeObserver(this);
+    graph->RemoveWorkerNodeObserver(this);
   }
 
   change_subscriber_.reset();
-  graph_ = nullptr;
 }
 
 void DiscardsGraphDumpImpl::OnFrameNodeAdded(
@@ -327,6 +323,12 @@ void DiscardsGraphDumpImpl::OnFinalResponseURLDetermined(
   SendWorkerNotification(worker_node, false);
 }
 
+void DiscardsGraphDumpImpl::OnBeforeClientFrameAdded(
+    const performance_manager::WorkerNode* worker_node,
+    const performance_manager::FrameNode* client_frame_node) {
+  // Nothing to do.
+}
+
 void DiscardsGraphDumpImpl::OnClientFrameAdded(
     const performance_manager::WorkerNode* worker_node,
     const performance_manager::FrameNode* client_frame_node) {
@@ -339,6 +341,12 @@ void DiscardsGraphDumpImpl::OnBeforeClientFrameRemoved(
   SendWorkerNotification(worker_node, false);
 }
 
+void DiscardsGraphDumpImpl::OnBeforeClientWorkerAdded(
+    const performance_manager::WorkerNode* worker_node,
+    const performance_manager::WorkerNode* client_worker_node) {
+  // Nothing to do.
+}
+
 void DiscardsGraphDumpImpl::OnClientWorkerAdded(
     const performance_manager::WorkerNode* worker_node,
     const performance_manager::WorkerNode* client_worker_node) {
@@ -360,7 +368,7 @@ void DiscardsGraphDumpImpl::AddNode(const performance_manager::Node* node) {
 
 void DiscardsGraphDumpImpl::RemoveNode(const performance_manager::Node* node) {
   auto it = node_ids_.find(node);
-  DCHECK(it != node_ids_.end());
+  CHECK(it != node_ids_.end(), base::NotFatalUntil::M130);
   NodeId node_id = it->second;
   node_ids_.erase(it);
   size_t erased = nodes_by_id_.erase(node_id);
@@ -378,7 +386,7 @@ int64_t DiscardsGraphDumpImpl::GetNodeId(
     return 0;
 
   auto it = node_ids_.find(node);
-  DCHECK(it != node_ids_.end());
+  CHECK(it != node_ids_.end(), base::NotFatalUntil::M130);
   return it->second.GetUnsafeValue();
 }
 
@@ -406,7 +414,7 @@ void DiscardsGraphDumpImpl::StartPageFaviconRequest(
 
   EnsureFaviconRequestHelper()
       .AsyncCall(&FaviconRequestHelper::RequestFavicon)
-      .WithArgs(page_node->GetMainFrameUrl(), page_node->GetContentsProxy(),
+      .WithArgs(page_node->GetMainFrameUrl(), page_node->GetWebContents(),
                 GetFaviconAvailableCallback(GetNodeId(page_node)));
 }
 
@@ -419,17 +427,19 @@ void DiscardsGraphDumpImpl::StartFrameFaviconRequest(
   EnsureFaviconRequestHelper()
       .AsyncCall(&FaviconRequestHelper::RequestFavicon)
       .WithArgs(frame_node->GetURL(),
-                frame_node->GetPageNode()->GetContentsProxy(),
+                frame_node->GetPageNode()->GetWebContents(),
                 GetFaviconAvailableCallback(GetNodeId(frame_node)));
 }
 
 void DiscardsGraphDumpImpl::SendNotificationToAllNodes(bool created) {
+  performance_manager::Graph* graph = GetOwningGraph();
   for (const performance_manager::ProcessNode* process_node :
-       graph_->GetAllProcessNodes())
+       graph->GetAllProcessNodes()) {
     SendProcessNotification(process_node, created);
+  }
 
   for (const performance_manager::PageNode* page_node :
-       graph_->GetAllPageNodes()) {
+       graph->GetAllPageNodes()) {
     SendPageNotification(page_node, created);
     if (created)
       StartPageFaviconRequest(page_node);
@@ -448,7 +458,7 @@ void DiscardsGraphDumpImpl::SendNotificationToAllNodes(bool created) {
   }
 
   for (const performance_manager::WorkerNode* worker_node :
-       graph_->GetAllWorkerNodes()) {
+       graph->GetAllWorkerNodes()) {
     SendWorkerNotification(worker_node, created);
   }
 }
@@ -457,7 +467,7 @@ void DiscardsGraphDumpImpl::SendFrameNotification(
     const performance_manager::FrameNode* frame,
     bool created) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  // TODO(https://crbug.com/1028117): Add more frame properties.
+  // TODO(crbug.com/40109021): Add more frame properties.
   discards::mojom::FrameInfoPtr frame_info = discards::mojom::FrameInfo::New();
 
   frame_info->id = GetNodeId(frame);
@@ -473,7 +483,8 @@ void DiscardsGraphDumpImpl::SendFrameNotification(
 
   frame_info->url = frame->GetURL();
   frame_info->description_json =
-      ToJSON(graph_->GetNodeDataDescriberRegistry()->DescribeNodeData(frame));
+      ToJSON(GetOwningGraph()->GetNodeDataDescriberRegistry()->DescribeNodeData(
+          frame));
 
   if (created) {
     change_subscriber_->FrameCreated(std::move(frame_info));
@@ -486,15 +497,16 @@ void DiscardsGraphDumpImpl::SendPageNotification(
     const performance_manager::PageNode* page_node,
     bool created) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  // TODO(https://crbug.com/1028117): Add more page_node properties.
+  // TODO(crbug.com/40109021): Add more page_node properties.
   discards::mojom::PageInfoPtr page_info = discards::mojom::PageInfo::New();
 
   page_info->id = GetNodeId(page_node);
   page_info->main_frame_url = page_node->GetMainFrameUrl();
   page_info->opener_frame_id = GetNodeId(page_node->GetOpenerFrameNode());
   page_info->embedder_frame_id = GetNodeId(page_node->GetEmbedderFrameNode());
-  page_info->description_json = ToJSON(
-      graph_->GetNodeDataDescriberRegistry()->DescribeNodeData(page_node));
+  page_info->description_json =
+      ToJSON(GetOwningGraph()->GetNodeDataDescriberRegistry()->DescribeNodeData(
+          page_node));
 
   if (created)
     change_subscriber_->PageCreated(std::move(page_info));
@@ -505,7 +517,7 @@ void DiscardsGraphDumpImpl::SendPageNotification(
 void DiscardsGraphDumpImpl::SendProcessNotification(
     const performance_manager::ProcessNode* process,
     bool created) {
-  // TODO(https://crbug.com/1028117): Add more process properties.
+  // TODO(crbug.com/40109021): Add more process properties.
   discards::mojom::ProcessInfoPtr process_info =
       discards::mojom::ProcessInfo::New();
 
@@ -514,7 +526,8 @@ void DiscardsGraphDumpImpl::SendProcessNotification(
   process_info->private_footprint_kb = process->GetPrivateFootprintKb();
 
   process_info->description_json =
-      ToJSON(graph_->GetNodeDataDescriberRegistry()->DescribeNodeData(process));
+      ToJSON(GetOwningGraph()->GetNodeDataDescriberRegistry()->DescribeNodeData(
+          process));
 
   if (created)
     change_subscriber_->ProcessCreated(std::move(process_info));
@@ -525,7 +538,7 @@ void DiscardsGraphDumpImpl::SendProcessNotification(
 void DiscardsGraphDumpImpl::SendWorkerNotification(
     const performance_manager::WorkerNode* worker,
     bool created) {
-  // TODO(https://crbug.com/1028117): Add more process properties.
+  // TODO(crbug.com/40109021): Add more process properties.
   discards::mojom::WorkerInfoPtr worker_info =
       discards::mojom::WorkerInfo::New();
 
@@ -547,7 +560,8 @@ void DiscardsGraphDumpImpl::SendWorkerNotification(
   }
 
   worker_info->description_json =
-      ToJSON(graph_->GetNodeDataDescriberRegistry()->DescribeNodeData(worker));
+      ToJSON(GetOwningGraph()->GetNodeDataDescriberRegistry()->DescribeNodeData(
+          worker));
 
   if (created)
     change_subscriber_->WorkerCreated(std::move(worker_info));
@@ -570,16 +584,13 @@ void DiscardsGraphDumpImpl::SendFaviconNotification(
   discards::mojom::FavIconInfoPtr icon_info =
       discards::mojom::FavIconInfo::New();
   icon_info->node_id = serialization_id;
-
-  base::Base64Encode(
-      base::StringPiece(reinterpret_cast<const char*>(bitmap_data->front()),
-                        bitmap_data->size()),
-      &icon_info->icon_data);
+  icon_info->icon_data = base::Base64Encode(*bitmap_data);
 
   change_subscriber_->FavIconDataAvailable(std::move(icon_info));
 }
 
 // static
 void DiscardsGraphDumpImpl::OnConnectionError(DiscardsGraphDumpImpl* impl) {
-  std::unique_ptr<GraphOwned> owned_impl = impl->graph_->TakeFromGraph(impl);
+  std::unique_ptr<GraphOwned> owned_impl =
+      impl->GetOwningGraph()->TakeFromGraph(impl);
 }
