@@ -9,6 +9,7 @@
 
 #include "base/files/file_path.h"
 #include "base/memory/raw_ptr.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "chrome/browser/download/download_item_model.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/safe_browsing/download_protection/download_protection_service.h"
@@ -41,6 +42,7 @@ using ::testing::ReturnRef;
 using ::testing::ReturnRefOfCopy;
 
 const char kTestDangerousDownloadUrl[] = "http://evildownload.com";
+const char kTestDangerousDownloadReferrerUrl[] = "http://referrer.test";
 
 class TestDownloadsDOMHandler : public DownloadsDOMHandler {
  public:
@@ -84,10 +86,11 @@ class DownloadsDOMHandlerTest : public testing::Test {
 
  protected:
   testing::StrictMock<MockPage> page_;
+  content::BrowserTaskEnvironment task_environment_{
+      base::test::TaskEnvironment::TimeSource::MOCK_TIME};
 
  private:
   // NOTE: The initialization order of these members matters.
-  content::BrowserTaskEnvironment task_environment_;
   TestingProfile profile_;
   content::RenderViewHostTestEnabler rvh_test_enabler_;
   std::unique_ptr<content::WebContents> web_contents_;
@@ -195,6 +198,8 @@ class DownloadsDOMHandlerWithFakeSafeBrowsingTest
     EXPECT_CALL(dangerous_download_, IsDone()).WillRepeatedly(Return(false));
     EXPECT_CALL(dangerous_download_, GetURL())
         .WillRepeatedly(ReturnRef(download_url_));
+    EXPECT_CALL(dangerous_download_, GetReferrerUrl())
+        .WillRepeatedly(ReturnRef(referrer_url_));
     ON_CALL(dangerous_download_, GetTargetFilePath())
         .WillByDefault(
             ReturnRefOfCopy(base::FilePath(FILE_PATH_LITERAL("foo.pdf"))));
@@ -225,6 +230,7 @@ class DownloadsDOMHandlerWithFakeSafeBrowsingTest
   raw_ptr<TestingBrowserProcess> browser_process_;
   scoped_refptr<safe_browsing::SafeBrowsingService> sb_service_;
   GURL download_url_ = GURL(kTestDangerousDownloadUrl);
+  GURL referrer_url_ = GURL(kTestDangerousDownloadReferrerUrl);
   testing::NiceMock<download::MockDownloadItem> dangerous_download_;
 };
 
@@ -314,19 +320,7 @@ TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
                   .empty());
 }
 
-class DownloadsDOMHandlerTestImprovedDownloadPageWarnings
-    : public DownloadsDOMHandlerWithFakeSafeBrowsingTest {
- public:
-  DownloadsDOMHandlerTestImprovedDownloadPageWarnings() {
-    feature_list_.InitAndEnableFeature(
-        safe_browsing::kImprovedDownloadPageWarnings);
-  }
-
- private:
-  base::test::ScopedFeatureList feature_list_;
-};
-
-TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
        SaveSuspiciousRequiringGesture) {
   SetUpDangerousDownload();
 
@@ -354,7 +348,7 @@ TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
                 ->serialized_download_report());
 }
 
-TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
        SaveSuspiciousRequiringGesture_InsecureDownload) {
   SetUpInsecureDownload();
 
@@ -372,7 +366,7 @@ TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
                   .empty());
 }
 
-TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
        SaveSuspiciousRequiringGesture_NoRecentInteraction) {
   SetUpDangerousDownload();
 
@@ -383,8 +377,8 @@ TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
   handler.SaveSuspiciousRequiringGesture("1");
 }
 
-TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
-       SaveDangerousFromPromptRequiringGesture) {
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
+       SaveDangerousFromDialogRequiringGesture) {
   SetUpDangerousDownload();
 
   TestDownloadsDOMHandler handler(page_.BindAndGetRemote(), manager(),
@@ -393,7 +387,7 @@ TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
   SimulateMouseGestureOnWebUI();
 
   EXPECT_CALL(dangerous_download_, ValidateDangerousDownload());
-  handler.SaveDangerousFromPromptRequiringGesture("1");
+  handler.SaveDangerousFromDialogRequiringGesture("1");
 
   // Verify that dangerous download report is sent.
   safe_browsing::ClientSafeBrowsingReportRequest expected_report;
@@ -411,26 +405,26 @@ TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
                 ->serialized_download_report());
 }
 
-TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
-       SaveDangerousFromPromptRequiringGesture_NoRecentInteraction) {
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
+       SaveDangerousFromDialogRequiringGesture_NoRecentInteraction) {
   SetUpDangerousDownload();
 
   TestDownloadsDOMHandler handler(page_.BindAndGetRemote(), manager(),
                                   web_ui());
 
   EXPECT_CALL(dangerous_download_, ValidateDangerousDownload()).Times(0);
-  handler.SaveDangerousFromPromptRequiringGesture("1");
+  handler.SaveDangerousFromDialogRequiringGesture("1");
 }
 
-TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
-       RecordCancelBypassWarningPrompt) {
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTest,
+       RecordCancelBypassWarningDialog) {
   SetUpDangerousDownload();
 
   TestDownloadsDOMHandler handler(page_.BindAndGetRemote(), manager(),
                                   web_ui());
 
   EXPECT_CALL(dangerous_download_, ValidateDangerousDownload()).Times(0);
-  handler.RecordCancelBypassWarningPrompt("1");
+  handler.RecordCancelBypassWarningDialog("1");
 
   // Verify no cancel report is sent, since it's not a terminal action.
   EXPECT_TRUE(test_safe_browsing_factory_->test_safe_browsing_service()
@@ -438,13 +432,143 @@ TEST_F(DownloadsDOMHandlerTestImprovedDownloadPageWarnings,
                   .empty());
 }
 
+class DownloadsDOMHandlerTestDangerousDownloadInterstitial
+    : public DownloadsDOMHandlerWithFakeSafeBrowsingTest {
+ public:
+  DownloadsDOMHandlerTestDangerousDownloadInterstitial() = default;
+
+  void SetUp() override {
+    DownloadsDOMHandlerWithFakeSafeBrowsingTest::SetUp();
+    SetUpDangerousDownload();
+    handler_ = std::make_unique<TestDownloadsDOMHandler>(
+        page_.BindAndGetRemote(), manager(), web_ui());
+    handler_->RecordOpenBypassWarningInterstitial("1");
+    task_environment_.FastForwardBy(base::Milliseconds(200));
+  }
+
+  void TearDown() override {
+    DownloadsDOMHandlerWithFakeSafeBrowsingTest::TearDown();
+  }
+
+ protected:
+  base::HistogramTester histogram_tester_;
+  std::unique_ptr<TestDownloadsDOMHandler> handler_;
+
+ private:
+  base::test::ScopedFeatureList feature_list_{
+      safe_browsing::kDangerousDownloadInterstitial};
+};
+
+TEST_F(DownloadsDOMHandlerTestDangerousDownloadInterstitial,
+       RecordOpenBypassWarningInterstitial) {
+  EXPECT_CALL(dangerous_download_, ValidateDangerousDownload()).Times(0);
+
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.Action",
+      DangerousDownloadInterstitialAction::kOpenInterstitial, 1);
+
+  // Verify no report is sent, since it's not a terminal action.
+  EXPECT_TRUE(test_safe_browsing_factory_->test_safe_browsing_service()
+                  ->serialized_download_report()
+                  .empty());
+}
+
+TEST_F(DownloadsDOMHandlerTestDangerousDownloadInterstitial,
+       RecordOpenSurveyOnDangerousInterstitial) {
+  EXPECT_CALL(dangerous_download_, ValidateDangerousDownload()).Times(0);
+  handler_->RecordOpenSurveyOnDangerousInterstitial("1");
+
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.Action",
+      DangerousDownloadInterstitialAction::kOpenSurvey, 1);
+  histogram_tester_.ExpectTimeBucketCount(
+      "Download.DangerousDownloadInterstitial.InteractionTime.OpenSurvey",
+      base::Milliseconds(200), 1);
+}
+
+TEST_F(DownloadsDOMHandlerTestDangerousDownloadInterstitial,
+       RecordCancelBypassWarningInterstitial) {
+  EXPECT_CALL(dangerous_download_, ValidateDangerousDownload()).Times(0);
+  handler_->RecordCancelBypassWarningInterstitial("1");
+
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.Action",
+      DangerousDownloadInterstitialAction::kCancelInterstitial, 1);
+  histogram_tester_.ExpectTimeBucketCount(
+      "Download.DangerousDownloadInterstitial.InteractionTime."
+      "CancelInterstitial",
+      base::Milliseconds(200), 1);
+
+  // Verify no report is sent, since it's not a terminal action.
+  EXPECT_TRUE(test_safe_browsing_factory_->test_safe_browsing_service()
+                  ->serialized_download_report()
+                  .empty());
+}
+
+TEST_F(DownloadsDOMHandlerTestDangerousDownloadInterstitial,
+       SaveDangerousFromInterstitialNeedGesture) {
+  SimulateMouseGestureOnWebUI();
+
+  EXPECT_CALL(dangerous_download_, ValidateDangerousDownload());
+
+  handler_->RecordOpenSurveyOnDangerousInterstitial("1");
+  task_environment_.FastForwardBy(base::Milliseconds(100));
+  handler_->SaveDangerousFromInterstitialNeedGesture(
+      "1", downloads::mojom::DangerousDownloadInterstitialSurveyOptions::
+               kAcceptRisk);
+
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.Action",
+      DangerousDownloadInterstitialAction::kSaveDangerous, 1);
+  histogram_tester_.ExpectTimeBucketCount(
+      "Download.DangerousDownloadInterstitial.InteractionTime.CompleteSurvey",
+      base::Milliseconds(100), 1);
+  histogram_tester_.ExpectTimeBucketCount(
+      "Download.DangerousDownloadInterstitial.InteractionTime.SaveDangerous",
+      base::Milliseconds(300), 1);
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.SurveyResponse",
+      downloads::mojom::DangerousDownloadInterstitialSurveyOptions::kAcceptRisk,
+      1);
+
+  // Verify that dangerous download report is sent.
+  safe_browsing::ClientSafeBrowsingReportRequest expected_report;
+  std::string expected_serialized_report;
+  expected_report.set_url(GURL(kTestDangerousDownloadUrl).spec());
+  expected_report.set_type(safe_browsing::ClientSafeBrowsingReportRequest::
+                               DANGEROUS_DOWNLOAD_RECOVERY);
+  expected_report.set_did_proceed(true);
+  expected_report.set_download_verdict(
+      safe_browsing::ClientDownloadResponse::DANGEROUS);
+  expected_report.set_token("token");
+  expected_report.SerializeToString(&expected_serialized_report);
+  EXPECT_EQ(expected_serialized_report,
+            test_safe_browsing_factory_->test_safe_browsing_service()
+                ->serialized_download_report());
+}
+
+TEST_F(DownloadsDOMHandlerTestDangerousDownloadInterstitial,
+       SaveDangerousFromInterstitialNeedGesture_NoRecentInteraction) {
+  EXPECT_CALL(dangerous_download_, ValidateDangerousDownload()).Times(0);
+
+  handler_->RecordOpenSurveyOnDangerousInterstitial("1");
+  handler_->SaveDangerousFromInterstitialNeedGesture(
+      "1",
+      downloads::mojom::DangerousDownloadInterstitialSurveyOptions::kTrustSite);
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.Action",
+      DangerousDownloadInterstitialAction::kSaveDangerous, 0);
+  histogram_tester_.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.SurveyResponse",
+      downloads::mojom::DangerousDownloadInterstitialSurveyOptions::kTrustSite,
+      0);
+}
+
 class DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService
     : public DownloadsDOMHandlerWithFakeSafeBrowsingTest {
  public:
-  DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService() {
-    feature_list_.InitAndEnableFeature(
-        safe_browsing::kImprovedDownloadPageWarnings);
-  }
+  DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService() =
+      default;
 
   void ExpectTrustSafetySentimentServiceCall(
       DownloadItemWarningData::WarningSurface surface,
@@ -459,7 +583,6 @@ class DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService
   }
 
  private:
-  base::test::ScopedFeatureList feature_list_;
   raw_ptr<MockTrustSafetySentimentService> mock_sentiment_service_;
 };
 
@@ -496,7 +619,7 @@ TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService,
 }
 
 TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService,
-       SaveDangerousFromPrompt_CallsTrustSafetySentimentService) {
+       SaveDangerousFromDialog_CallsTrustSafetySentimentService) {
   profile()->GetPrefs()->SetBoolean(prefs::kSafeBrowsingSurveysEnabled, true);
   SetUpDangerousDownload();
   ExpectTrustSafetySentimentServiceCall(
@@ -509,5 +632,38 @@ TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService,
   SimulateMouseGestureOnWebUI();
 
   EXPECT_CALL(dangerous_download_, ValidateDangerousDownload());
-  handler.SaveDangerousFromPromptRequiringGesture("1");
+  handler.SaveDangerousFromDialogRequiringGesture("1");
+}
+
+TEST_F(DownloadsDOMHandlerWithFakeSafeBrowsingTestTrustSafetySentimentService,
+       SaveDangerousFromInterstitial_CallsTrustSafetySentimentService) {
+  base::test::ScopedFeatureList feature_list_;
+  feature_list_.InitAndEnableFeature(
+      safe_browsing::kDangerousDownloadInterstitial);
+
+  profile()->GetPrefs()->SetBoolean(prefs::kSafeBrowsingSurveysEnabled, true);
+  SetUpDangerousDownload();
+  ExpectTrustSafetySentimentServiceCall(
+      DownloadItemWarningData::WarningSurface::DOWNLOAD_PROMPT,
+      DownloadItemWarningData::WarningAction::PROCEED);
+
+  TestDownloadsDOMHandler handler(page_.BindAndGetRemote(), manager(),
+                                  web_ui());
+
+  SimulateMouseGestureOnWebUI();
+
+  EXPECT_CALL(dangerous_download_, ValidateDangerousDownload());
+
+  handler.RecordOpenBypassWarningInterstitial("1");
+  handler.RecordOpenSurveyOnDangerousInterstitial("1");
+  handler.SaveDangerousFromInterstitialNeedGesture(
+      "1", downloads::mojom::DangerousDownloadInterstitialSurveyOptions::
+               kCreatedFile);
+
+  base::HistogramTester histogram_tester;
+  histogram_tester.ExpectBucketCount(
+      "Download.DangerousDownloadInterstitial.SurveyResponse",
+      downloads::mojom::DangerousDownloadInterstitialSurveyOptions::
+          kCreatedFile,
+      0);
 }
