@@ -16,6 +16,7 @@
 #include "base/i18n/rtl.h"
 #include "base/i18n/unicodestring.h"
 #include "base/metrics/histogram_functions.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/escape.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
@@ -29,10 +30,13 @@
 #include "chrome/browser/enterprise/connectors/common.h"
 #include "chrome/browser/extensions/api/downloads/downloads_api.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/signin/identity_manager_factory.h"
 #include "chrome/browser/ui/webui/downloads/downloads.mojom.h"
 #include "components/download/public/common/download_danger_type.h"
 #include "components/download/public/common/download_item.h"
+#include "components/download/public/common/download_item_rename_handler.h"
 #include "components/safe_browsing/core/common/features.h"
+#include "components/signin/public/identity_manager/identity_manager.h"
 #include "components/url_formatter/elide_url.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/download_item_utils.h"
@@ -52,6 +56,7 @@
 using content::BrowserContext;
 using content::DownloadManager;
 using download::DownloadItem;
+using TailoredWarningType = DownloadUIModel::TailoredWarningType;
 
 using DownloadVector = DownloadManager::DownloadVector;
 
@@ -66,11 +71,10 @@ downloads::mojom::DangerType GetDangerType(
       return downloads::mojom::DangerType::kDangerousFile;
     case download::DOWNLOAD_DANGER_TYPE_DANGEROUS_URL:
       return downloads::mojom::DangerType::kDangerousUrl;
-    // Account compromise is represented in the UI the same as dangerous
-    // content.
-    case download::DOWNLOAD_DANGER_TYPE_DANGEROUS_ACCOUNT_COMPROMISE:
     case download::DOWNLOAD_DANGER_TYPE_DANGEROUS_CONTENT:
       return downloads::mojom::DangerType::kDangerousContent;
+    case download::DOWNLOAD_DANGER_TYPE_DANGEROUS_ACCOUNT_COMPROMISE:
+      return downloads::mojom::DangerType::kCookieTheft;
     case download::DOWNLOAD_DANGER_TYPE_UNCOMMON_CONTENT:
       return downloads::mojom::DangerType::kUncommonContent;
     case download::DOWNLOAD_DANGER_TYPE_DANGEROUS_HOST:
@@ -95,8 +99,8 @@ downloads::mojom::DangerType GetDangerType(
       return downloads::mojom::DangerType::kDeepScannedSafe;
     case download::DOWNLOAD_DANGER_TYPE_DEEP_SCANNED_OPENED_DANGEROUS:
       return downloads::mojom::DangerType::kDeepScannedOpenedDangerous;
-    case download::DOWNLOAD_DANGER_TYPE_BLOCKED_UNSUPPORTED_FILETYPE:
-      return downloads::mojom::DangerType::kBlockedUnsupportedFileType;
+    case download::DOWNLOAD_DANGER_TYPE_BLOCKED_SCAN_FAILED:
+      return downloads::mojom::DangerType::kBlockedScanFailed;
     case download::DOWNLOAD_DANGER_TYPE_PROMPT_FOR_SCANNING:
     case download::DOWNLOAD_DANGER_TYPE_PROMPT_FOR_LOCAL_PASSWORD_SCANNING:
     case download::DOWNLOAD_DANGER_TYPE_NOT_DANGEROUS:
@@ -108,6 +112,23 @@ downloads::mojom::DangerType GetDangerType(
   }
 }
 
+// Returns an enum value to be used as the |tailored_warning_type| value in
+// CreateDownloadData().
+downloads::mojom::TailoredWarningType GetTailoredWarningType(
+    TailoredWarningType tailored_warning_type) {
+  switch (tailored_warning_type) {
+    case TailoredWarningType::kSuspiciousArchive:
+      return downloads::mojom::TailoredWarningType::kSuspiciousArchive;
+    case TailoredWarningType::kCookieTheft:
+      return downloads::mojom::TailoredWarningType::kCookieTheft;
+    case TailoredWarningType::kCookieTheftWithAccountInfo:
+      return downloads::mojom::TailoredWarningType::kCookieTheftWithAccountInfo;
+    case TailoredWarningType::kNoTailoredWarning:
+      return downloads::mojom::TailoredWarningType::
+          kNoApplicableTailoredWarningType;
+  }
+}
+
 downloads::mojom::SafeBrowsingState GetSafeBrowsingState(Profile* profile) {
 #if BUILDFLAG(FULL_SAFE_BROWSING)
   safe_browsing::SafeBrowsingState state =
@@ -140,13 +161,29 @@ std::u16string GetFormattedDisplayUrl(const GURL& url) {
   // crbug.com/1070451). If it's really this long, the user won't be able to see
   // the whole thing anyway. We truncate the beginning so that the end of it is
   // shown, which contains the eTLD+1.
-  // Note: This may truncate the scheme part of the URL.
-  if (result.size() > url::kMaxURLChars) {
-    result = result.substr(result.size() - url::kMaxURLChars);
+  // Note:
+  // - This may truncate the scheme part of the URL.
+  // - Use a much smaller limit than url::kMaxURLChars (2M) since this is for
+  //   display only, and long URLs will affect page load speed and may cause
+  //   JavaScript errors (https://crbug.com/1522764).
+  const size_t kMaxDisplayURLChars = 16 * 1024;
+  if (result.size() > kMaxDisplayURLChars) {
+    result = result.substr(result.size() - kMaxDisplayURLChars);
   }
   return result;
 }
 
+void FillUrlFields(const GURL& url,
+                   std::optional<GURL>& data_url,
+                   std::u16string& display_url_out) {
+  // If URL is too long, don't make it clickable.
+  if (url.is_valid() && url.spec().length() <= url::kMaxURLChars) {
+    data_url = std::make_optional<GURL>(url);
+  }
+
+  display_url_out = GetFormattedDisplayUrl(url);
+}
+
 }  // namespace
 
 DownloadsListTracker::DownloadsListTracker(
@@ -207,6 +244,31 @@ void DownloadsListTracker::Stop() {
   sending_updates_ = false;
 }
 
+int DownloadsListTracker::NumDangerousItemsSent() const {
+  auto sent_items_end_it = sorted_items_.begin();
+  std::advance(sent_items_end_it, sent_to_page_);
+
+  return base::ranges::count_if(
+      sorted_items_.begin(), sent_items_end_it,
+      [](download::DownloadItem* item) { return item->IsDangerous(); });
+}
+
+download::DownloadItem* DownloadsListTracker::GetFirstActiveWarningItem() {
+  auto sent_items_end_it = sorted_items_.begin();
+  std::advance(sent_items_end_it, sent_to_page_);
+
+  auto iter = base::ranges::find_if(
+      sorted_items_.begin(), sent_items_end_it,
+      [](download::DownloadItem* item) {
+        return item->GetState() != download::DownloadItem::CANCELLED &&
+               item->IsDangerous();
+      });
+  if (iter != sent_items_end_it) {
+    return *iter;
+  }
+  return nullptr;
+}
+
 DownloadManager* DownloadsListTracker::GetMainNotifierManager() const {
   return main_notifier_.GetManager();
 }
@@ -307,20 +369,18 @@ downloads::mojom::DataPtr DownloadsListTracker::CreateDownloadData(
   file_name = base::i18n::GetDisplayStringInLTRDirectionality(file_name);
 
   file_value->file_name = base::UTF16ToUTF8(file_name);
-  // If URL is too long, don't make it clickable.
-  if (download_item->GetURL().is_valid() &&
-      download_item->GetURL().spec().length() <= url::kMaxURLChars) {
-    file_value->url = std::make_optional<GURL>(download_item->GetURL());
+  FillUrlFields(download_item->GetURL(), file_value->url,
+                file_value->display_url);
+  if (download_item->HasUserGesture()) {
+    FillUrlFields(download_item->GetReferrerUrl(), file_value->referrer_url,
+                  file_value->display_referrer_url);
   }
-  file_value->display_url = GetFormattedDisplayUrl(download_item->GetURL());
   file_value->total = download_item->GetTotalBytes();
   file_value->file_externally_removed =
       download_item->GetFileExternallyRemoved();
   file_value->resume = download_item->CanResume();
   file_value->otr = IsIncognito(*download_item);
 
-  downloads::mojom::DangerType danger_type =
-      GetDangerType(download_item->GetDangerType());
   std::u16string last_reason_text;
   // -2 is invalid, -1 means indeterminate, and 0-100 are in-progress.
   int percent = -2;
@@ -351,7 +411,7 @@ downloads::mojom::DataPtr DownloadsListTracker::CreateDownloadData(
         state = downloads::mojom::State::kInProgress;
       }
       progress_status_text = download_model.GetTabProgressStatusText();
-      percent = download_item->PercentComplete();
+      percent = GetPercentComplete(download_item);
       break;
     }
 
@@ -360,9 +420,9 @@ downloads::mojom::DataPtr DownloadsListTracker::CreateDownloadData(
       progress_status_text = download_model.GetTabProgressStatusText();
 
       if (download_item->CanResume())
-        percent = download_item->PercentComplete();
+        percent = GetPercentComplete(download_item);
 
-      // TODO(https://crbug.com/609255): GetHistoryPageStatusText() is using
+      // TODO(crbug.com/40467967): GetHistoryPageStatusText() is using
       // GetStatusText() as a temporary measure until the layout is fixed to
       // accommodate the longer string. Should update it to simply use
       // GetInterruptDescription().
@@ -385,19 +445,24 @@ downloads::mojom::DataPtr DownloadsListTracker::CreateDownloadData(
       break;
 
     case download::DownloadItem::MAX_DOWNLOAD_STATE:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
   }
 
   CHECK(state);
 
+  downloads::mojom::DangerType danger_type =
+      GetDangerType(download_item->GetDangerType());
+  downloads::mojom::TailoredWarningType tailored_warning_type =
+      GetTailoredWarningType(download_model.GetTailoredWarningType());
   file_value->danger_type = danger_type;
+  file_value->tailored_warning_type = tailored_warning_type;
   file_value->is_dangerous = download_item->IsDangerous();
   file_value->is_insecure = download_item->IsInsecure();
   file_value->is_reviewable =
       enterprise_connectors::ShouldPromptReviewForDownload(
           Profile::FromBrowserContext(
               content::DownloadItemUtils::GetBrowserContext(download_item)),
-          download_item->GetDangerType());
+          download_item);
 
   file_value->last_reason_text = base::UTF16ToUTF8(last_reason_text);
   file_value->percent = percent;
@@ -415,11 +480,6 @@ downloads::mojom::DataPtr DownloadsListTracker::CreateDownloadData(
   file_value->has_safe_browsing_verdict =
       WasSafeBrowsingVerdictObtained(download_item);
 
-  if (download_model.IsDangerous()) {
-    base::UmaHistogramBoolean(
-        "Download.DownloadsPageDangerousWarningWasShownBefore",
-        download_model.WasUIWarningShown());
-  }
   MaybeRecordDangerousDownloadWarningShown(download_model);
 
   if (download_item->IsDangerous()) {
@@ -434,6 +494,20 @@ downloads::mojom::DataPtr DownloadsListTracker::CreateDownloadData(
         DownloadItemWarningData::WarningAction::SHOWN);
   }
 
+  if (tailored_warning_type ==
+      downloads::mojom::TailoredWarningType::kCookieTheftWithAccountInfo) {
+    if (auto* identity_manager =
+            IdentityManagerFactory::GetForProfile(download_model.profile());
+        identity_manager) {
+      std::string email =
+          identity_manager->GetPrimaryAccountInfo(signin::ConsentLevel::kSignin)
+              .email;
+      if (!email.empty()) {
+        file_value->account_email = std::move(email);
+      }
+    }
+  }
+
   return file_value;
 }
 
@@ -544,3 +618,16 @@ void DownloadsListTracker::RemoveItem(const SortedSet::iterator& remove) {
   }
   sorted_items_.erase(remove);
 }
+
+int DownloadsListTracker::GetPercentComplete(
+    download::DownloadItem* download_item) const {
+  auto* renamer = download_item->GetRenameHandler();
+  if (renamer && renamer->ShowRenameProgress()) {
+    return static_cast<int>(((download_item->GetReceivedBytes() +
+                              download_item->GetUploadedBytes()) *
+                             0.5 * 100.0) /
+                            download_item->GetTotalBytes());
+  } else {
+    return download_item->PercentComplete();
+  }
+}
