@@ -6,8 +6,10 @@
 
 #include <memory>
 #include <string>
+#include <utility>
 
 #include "base/functional/bind.h"
+#include "base/memory/weak_ptr.h"
 #include "base/strings/strcat.h"
 #include "base/values.h"
 #include "build/build_config.h"
@@ -19,6 +21,7 @@
 #include "content/public/common/url_constants.h"
 #include "extensions/common/api/feedback_private.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/base/mojom/ui_base_types.mojom-shared.h"
 #include "ui/base/ui_base_types.h"
 #include "ui/views/widget/widget.h"
 #include "ui/web_dialogs/web_dialog_delegate.h"
@@ -27,7 +30,7 @@
 namespace {
 
 void ShowChildPage(Profile* profile,
-                   const FeedbackDialog* dialog,
+                   const base::WeakPtr<FeedbackDialog>& dialog,
                    const GURL& url,
                    const std::u16string& title,
                    const std::string& args = "",
@@ -35,13 +38,19 @@ void ShowChildPage(Profile* profile,
                    int dialog_height = 400,
                    bool can_resize = true,
                    bool can_minimize = true) {
+  CHECK(profile);
+  if (!dialog) {
+    return;
+  }
+
   const bool is_parent_modal = dialog->GetWidget()->IsModal();
 
   auto delegate = std::make_unique<ui::WebDialogDelegate>();
   delegate->set_dialog_args(args);
   delegate->set_dialog_content_url(url);
-  delegate->set_dialog_modal_type(is_parent_modal ? ui::MODAL_TYPE_SYSTEM
-                                                  : ui::MODAL_TYPE_NONE);
+  delegate->set_dialog_modal_type(is_parent_modal
+                                      ? ui::mojom::ModalType::kSystem
+                                      : ui::mojom::ModalType::kNone);
   delegate->set_dialog_size(gfx::Size(dialog_width, dialog_height));
   delegate->set_dialog_title(title);
   delegate->set_minimum_dialog_size(gfx::Size(400, 120));
@@ -61,8 +70,8 @@ GURL ChildPageURL(const std::string& child_page) {
 }
 }  // namespace
 
-FeedbackHandler::FeedbackHandler(const FeedbackDialog* dialog)
-    : dialog_(dialog) {}
+FeedbackHandler::FeedbackHandler(base::WeakPtr<FeedbackDialog> dialog)
+    : dialog_(std::move(dialog)) {}
 
 FeedbackHandler::~FeedbackHandler() = default;
 
@@ -75,10 +84,6 @@ void FeedbackHandler::RegisterMessages() {
       "showAssistantLogsInfo",
       base::BindRepeating(&FeedbackHandler::HandleShowAssistantLogsInfo,
                           base::Unretained(this)));
-  web_ui()->RegisterMessageCallback(
-      "showBluetoothLogsInfo",
-      base::BindRepeating(&FeedbackHandler::HandleShowBluetoothLogsInfo,
-                          base::Unretained(this)));
 #endif  // BUILDFLAG(IS_CHROMEOS)
 
   web_ui()->RegisterMessageCallback(
@@ -95,7 +100,9 @@ void FeedbackHandler::RegisterMessages() {
 }
 
 void FeedbackHandler::HandleShowDialog(const base::Value::List& args) {
-  dialog_->Show();
+  if (dialog_) {
+    dialog_->Show();
+  }
 }
 
 #if BUILDFLAG(IS_CHROMEOS)
@@ -107,14 +114,6 @@ void FeedbackHandler::HandleShowAssistantLogsInfo(
                 /*dialog_width=*/400, /*dialog_height=*/120,
                 /*can_resize=*/false, /*can_minimize=*/false);
 }
-void FeedbackHandler::HandleShowBluetoothLogsInfo(
-    const base::Value::List& args) {
-  ShowChildPage(Profile::FromWebUI(web_ui()), dialog_,
-                ChildPageURL("html/bluetooth_logs_info.html"), std::u16string(),
-                std::string(),
-                /*dialog_width=*/400, /*dialog_height=*/190,
-                /*can_resize=*/false, /*can_minimize=*/false);
-}
 #endif  // BUILDFLAG(IS_CHROMEOS)
 
 void FeedbackHandler::HandleShowAutofillMetadataInfo(
@@ -129,7 +128,7 @@ void FeedbackHandler::HandleShowAutofillMetadataInfo(
 
 void FeedbackHandler::HandleShowSystemInfo(const base::Value::List& args) {
   ShowChildPage(Profile::FromWebUI(web_ui()), dialog_,
-                ChildPageURL("html/sys_info.html"),
+                ChildPageURL("html/system_info.html"),
                 l10n_util::GetStringUTF16(IDS_FEEDBACK_SYSINFO_PAGE_TITLE));
 }
 
