@@ -32,17 +32,14 @@ class FlagsUIHandler : public content::WebUIMessageHandler {
             flags_ui::FlagAccess access);
 
   // Sends experimental features lists to the UI.
-  void SendExperimentalFeatures();
-
-  // Configures the handler to return either all features or deprecated
-  // features only.
-  void set_deprecated_features_only(bool deprecatedFeaturesOnly) {
-    deprecated_features_only_ = deprecatedFeaturesOnly;
-  }
+  void SendExperimentalFeatures(bool deprecated_features_only);
 
   // WebUIMessageHandler implementation.
   void RegisterMessages() override;
 
+  // Callback for the "requestDeprecatedFeatures" message.
+  void HandleRequestDeprecatedFeatures(const base::Value::List& args);
+
   // Callback for the "requestExperimentFeatures" message.
   void HandleRequestExperimentalFeatures(const base::Value::List& args);
 
@@ -70,7 +67,7 @@ class FlagsUIHandler : public content::WebUIMessageHandler {
   std::unique_ptr<flags_ui::FlagsStorage> flags_storage_;
   flags_ui::FlagAccess access_;
   std::string experimental_features_callback_id_;
-  bool deprecated_features_only_;
+  std::string deprecated_features_callback_id_;
 };
 
 #endif  // CHROME_BROWSER_UI_WEBUI_FLAGS_FLAGS_UI_HANDLER_H_
