@@ -50,9 +50,7 @@ class VersionUpdaterCrosTest : public ::testing::Test {
 
  protected:
   VersionUpdaterCrosTest()
-      : version_updater_(VersionUpdater::Create(nullptr)),
-        version_updater_cros_ptr_(
-            reinterpret_cast<VersionUpdaterCros*>(version_updater_.get())),
+      : version_updater_(std::make_unique<VersionUpdaterCros>(nullptr)),
         fake_update_engine_client_(nullptr),
         user_manager_enabler_(std::make_unique<FakeChromeUserManager>()) {}
 
@@ -97,8 +95,7 @@ class VersionUpdaterCrosTest : public ::testing::Test {
 
   content::BrowserTaskEnvironment task_environment_;
   std::unique_ptr<ash::NetworkHandlerTestHelper> network_handler_test_helper_;
-  std::unique_ptr<VersionUpdater> version_updater_;
-  raw_ptr<VersionUpdaterCros, DanglingUntriaged> version_updater_cros_ptr_;
+  std::unique_ptr<VersionUpdaterCros> version_updater_;
   raw_ptr<ash::FakeUpdateEngineClient, DanglingUntriaged>
       fake_update_engine_client_;  // Not owned.
 
@@ -201,7 +198,7 @@ TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_NoCallbackDuringInstallations) {
 
   // Expect the callback not to be called as it's an installation (not update).
   StrictMock<base::MockCallback<VersionUpdater::StatusCallback>> mock_callback;
-  version_updater_cros_ptr_->GetUpdateStatus(mock_callback.Get());
+  version_updater_->GetUpdateStatus(mock_callback.Get());
 }
 
 TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_CallbackDuringUpdates) {
@@ -212,7 +209,7 @@ TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_CallbackDuringUpdates) {
   // Expect the callbac kto be called as it's an update status change.
   StrictMock<base::MockCallback<VersionUpdater::StatusCallback>> mock_callback;
   EXPECT_CALL(mock_callback, Run(_, _, _, _, _, _, _)).Times(1);
-  version_updater_cros_ptr_->GetUpdateStatus(mock_callback.Get());
+  version_updater_->GetUpdateStatus(mock_callback.Get());
 }
 
 TEST_F(VersionUpdaterCrosTest,
@@ -228,7 +225,7 @@ TEST_F(VersionUpdaterCrosTest,
   StrictMock<base::MockCallback<VersionUpdater::StatusCallback>> mock_callback;
   EXPECT_CALL(mock_callback, Run(VersionUpdater::UPDATED, 0, _, _, _, _, _))
       .Times(1);
-  version_updater_cros_ptr_->GetUpdateStatus(mock_callback.Get());
+  version_updater_->GetUpdateStatus(mock_callback.Get());
 }
 
 TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_UpdatedButDeferred) {
@@ -244,7 +241,7 @@ TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_UpdatedButDeferred) {
   StrictMock<base::MockCallback<VersionUpdater::StatusCallback>> mock_callback;
   EXPECT_CALL(mock_callback, Run(VersionUpdater::DEFERRED, _, _, _, _, _, _))
       .Times(1);
-  version_updater_cros_ptr_->GetUpdateStatus(mock_callback.Get());
+  version_updater_->GetUpdateStatus(mock_callback.Get());
 }
 
 TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_UpdatedNeedReboot) {
@@ -259,7 +256,7 @@ TEST_F(VersionUpdaterCrosTest, GetUpdateStatus_UpdatedNeedReboot) {
   EXPECT_CALL(mock_callback,
               Run(VersionUpdater::NEARLY_UPDATED, _, _, _, _, _, _))
       .Times(1);
-  version_updater_cros_ptr_->GetUpdateStatus(mock_callback.Get());
+  version_updater_->GetUpdateStatus(mock_callback.Get());
 }
 
 TEST_F(VersionUpdaterCrosTest,
@@ -279,7 +276,7 @@ TEST_F(VersionUpdaterCrosTest,
               Run(VersionUpdater::UPDATE_TO_ROLLBACK_VERSION_DISALLOWED, _, _,
                   _, _, _, _))
       .Times(1);
-  version_updater_cros_ptr_->GetUpdateStatus(mock_callback.Get());
+  version_updater_->GetUpdateStatus(mock_callback.Get());
 }
 
 TEST_F(VersionUpdaterCrosTest, ToggleFeature) {
@@ -296,8 +293,7 @@ TEST_F(VersionUpdaterCrosTest, IsFeatureEnabled) {
   StrictMock<base::MockCallback<VersionUpdater::IsFeatureEnabledCallback>>
       mock_callback;
   EXPECT_CALL(mock_callback, Run(_)).Times(1);
-  version_updater_cros_ptr_->IsFeatureEnabled("feature-foo",
-                                              mock_callback.Get());
+  version_updater_->IsFeatureEnabled("feature-foo", mock_callback.Get());
 
   EXPECT_EQ(1, fake_update_engine_client_->is_feature_enabled_count());
 }
@@ -309,7 +305,7 @@ TEST_F(VersionUpdaterCrosTest, ApplyDeferredUpdate) {
   fake_update_engine_client_->NotifyObserversThatStatusChanged(status);
 
   EXPECT_EQ(0, fake_update_engine_client_->apply_deferred_update_count());
-  version_updater_cros_ptr_->ApplyDeferredUpdate();
+  version_updater_->ApplyDeferredUpdate();
   EXPECT_EQ(1, fake_update_engine_client_->apply_deferred_update_count());
 }
 
