@@ -25,12 +25,16 @@
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search_engines/template_url_service_factory.h"
 #include "chrome/browser/sync/sync_service_factory.h"
-#include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_actions.h"
 #include "chrome/browser/ui/browser_commands.h"
-#include "chrome/browser/ui/browser_finder.h"
+#include "chrome/browser/ui/browser_window/public/browser_window_features.h"
+#include "chrome/browser/ui/browser_window/public/browser_window_interface.h"
+#include "chrome/browser/ui/tabs/public/tab_interface.h"
 #include "chrome/browser/ui/tabs/tab_group.h"
 #include "chrome/browser/ui/tabs/tab_group_model.h"
+#include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/tabs/tab_strip_model_observer.h"
+#include "chrome/browser/ui/webui/top_chrome/top_chrome_web_ui_controller.h"
 #include "components/history/core/browser/history_service.h"
 #include "components/history/core/browser/history_types.h"
 #include "components/history/core/common/pref_names.h"
@@ -39,7 +43,7 @@
 #include "components/history_clusters/core/features.h"
 #include "components/history_clusters/core/history_cluster_type_utils.h"
 #include "components/history_clusters/core/history_clusters_prefs.h"
-#include "components/history_clusters/ui/query_clusters_state.h"
+#include "components/history_clusters/core/query_clusters_state.h"
 #include "components/keyed_service/core/service_access_type.h"
 #include "components/page_image_service/image_service.h"
 #include "components/prefs/pref_service.h"
@@ -48,13 +52,14 @@
 #include "components/strings/grit/components_strings.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/web_contents.h"
+#include "third_party/abseil-cpp/absl/types/variant.h"
+#include "ui/actions/actions.h"
 #include "ui/base/clipboard/scoped_clipboard_writer.h"
 #include "ui/base/l10n/time_format.h"
 #include "ui/base/models/simple_menu_model.h"
 #include "ui/base/mojom/window_open_disposition.mojom.h"
 #include "ui/base/window_open_disposition.h"
 #include "ui/base/window_open_disposition_utils.h"
-#include "ui/webui/mojo_bubble_web_ui_controller.h"
 #include "ui/webui/resources/cr_components/history_clusters/history_clusters.mojom.h"
 #include "url/gurl.h"
 
@@ -62,12 +67,28 @@ namespace history_clusters {
 
 namespace {
 
+void InvokeAction(actions::ActionId id, actions::ActionItem* scope) {
+  actions::ActionManager::Get().FindAction(id, scope)->InvokeAction();
+}
+
+// Returns the current browser window, regardless of whether this instance is
+// tab-scoped or window-scoped.
+BrowserWindowInterface* GetBrowserWindowInterface(
+    absl::variant<BrowserWindowInterface*, tabs::TabInterface*> interface) {
+  if (absl::holds_alternative<BrowserWindowInterface*>(interface)) {
+    return absl::get<BrowserWindowInterface*>(interface);
+  }
+  return absl::get<tabs::TabInterface*>(interface)->GetBrowserWindowInterface();
+}
+
 class HistoryClustersSidePanelContextMenu
     : public ui::SimpleMenuModel,
       public ui::SimpleMenuModel::Delegate {
  public:
-  HistoryClustersSidePanelContextMenu(Browser* browser, GURL url)
-      : ui::SimpleMenuModel(this), browser_(browser), url_(url) {
+  HistoryClustersSidePanelContextMenu(
+      absl::variant<BrowserWindowInterface*, tabs::TabInterface*> interface,
+      GURL url)
+      : ui::SimpleMenuModel(this), interface_(interface), url_(url) {
     AddItemWithStringId(IDC_CONTENT_CONTEXT_OPENLINKNEWTAB,
                         IDS_HISTORY_CLUSTERS_OPEN_IN_NEW_TAB);
     AddItemWithStringId(IDC_CONTENT_CONTEXT_OPENLINKNEWWINDOW,
@@ -79,8 +100,10 @@ class HistoryClustersSidePanelContextMenu
     AddItemWithStringId(IDC_CONTENT_CONTEXT_COPYLINKLOCATION,
                         IDS_HISTORY_CLUSTERS_COPY_LINK);
   }
-  HistoryClustersSidePanelContextMenu(Browser* browser, std::string query)
-      : ui::SimpleMenuModel(this), browser_(browser), query_(query) {
+  HistoryClustersSidePanelContextMenu(
+      absl::variant<BrowserWindowInterface*, tabs::TabInterface*> interface,
+      std::string query)
+      : ui::SimpleMenuModel(this), interface_(interface), query_(query) {
     AddItemWithStringId(IDC_CUT, IDS_HISTORY_CLUSTERS_CUT);
     AddItemWithStringId(IDC_COPY, IDS_HISTORY_CLUSTERS_COPY);
     AddItemWithStringId(IDC_PASTE, IDS_HISTORY_CLUSTERS_PASTE);
@@ -93,7 +116,9 @@ class HistoryClustersSidePanelContextMenu
         content::OpenURLParams params(url_, content::Referrer(),
                                       WindowOpenDisposition::NEW_BACKGROUND_TAB,
                                       ui::PAGE_TRANSITION_AUTO_BOOKMARK, false);
-        browser_->OpenURL(params);
+        GetBrowserWindowInterface(interface_)
+            ->OpenURL(params,
+                      /*navigation_handle_callback=*/{});
         break;
       }
 
@@ -101,7 +126,9 @@ class HistoryClustersSidePanelContextMenu
         content::OpenURLParams params(url_, content::Referrer(),
                                       WindowOpenDisposition::NEW_WINDOW,
                                       ui::PAGE_TRANSITION_AUTO_BOOKMARK, false);
-        browser_->OpenURL(params);
+        GetBrowserWindowInterface(interface_)
+            ->OpenURL(params,
+                      /*navigation_handle_callback=*/{});
         break;
       }
 
@@ -109,7 +136,9 @@ class HistoryClustersSidePanelContextMenu
         content::OpenURLParams params(url_, content::Referrer(),
                                       WindowOpenDisposition::OFF_THE_RECORD,
                                       ui::PAGE_TRANSITION_AUTO_BOOKMARK, false);
-        browser_->OpenURL(params);
+        GetBrowserWindowInterface(interface_)
+            ->OpenURL(params,
+                      /*navigation_handle_callback=*/{});
         break;
       }
       case IDC_CONTENT_CONTEXT_COPYLINKLOCATION: {
@@ -118,19 +147,31 @@ class HistoryClustersSidePanelContextMenu
         break;
       }
       case IDC_CUT:
+        InvokeAction(actions::kActionCut, GetBrowserWindowInterface(interface_)
+                                              ->GetActions()
+                                              ->root_action_item());
+        break;
       case IDC_COPY:
-      case IDC_PASTE: {
-        chrome::CutCopyPaste(browser_, command_id);
+        InvokeAction(actions::kActionCopy, GetBrowserWindowInterface(interface_)
+                                               ->GetActions()
+                                               ->root_action_item());
+        break;
+      case IDC_PASTE:
+        InvokeAction(actions::kActionPaste,
+                     GetBrowserWindowInterface(interface_)
+                         ->GetActions()
+                         ->root_action_item());
         break;
-      }
       default:
-        NOTREACHED();
+        NOTREACHED_IN_MIGRATION();
         break;
     }
   }
 
  private:
-  const raw_ptr<Browser> browser_;
+  // Exactly one of `browser_window_interface_` and `tab_interface_` will be
+  // non-nullptr.
+  absl::variant<BrowserWindowInterface*, tabs::TabInterface*> interface_;
   std::string query_;
   GURL url_;
 };
@@ -164,10 +205,28 @@ mojom::QueryResultPtr QueryClustersResultToMojom(
 HistoryClustersHandler::HistoryClustersHandler(
     mojo::PendingReceiver<mojom::PageHandler> pending_page_handler,
     Profile* profile,
-    content::WebContents* web_contents)
+    content::WebContents* web_contents,
+    BrowserWindowInterface* browser_window_interface)
     : profile_(profile),
       web_contents_(web_contents),
+      interface_(browser_window_interface),
       page_handler_(this, std::move(pending_page_handler)) {
+  CommonInit();
+}
+
+HistoryClustersHandler::HistoryClustersHandler(
+    mojo::PendingReceiver<mojom::PageHandler> pending_page_handler,
+    Profile* profile,
+    content::WebContents* web_contents,
+    tabs::TabInterface* tab_interface)
+    : profile_(profile),
+      web_contents_(web_contents),
+      interface_(tab_interface),
+      page_handler_(this, std::move(pending_page_handler)) {
+  CommonInit();
+}
+
+void HistoryClustersHandler::CommonInit() {
   DCHECK(profile_);
   DCHECK(web_contents_);
 
@@ -187,11 +246,14 @@ HistoryClustersHandler::HistoryClustersHandler(
 HistoryClustersHandler::~HistoryClustersHandler() = default;
 
 void HistoryClustersHandler::SetSidePanelUIEmbedder(
-    base::WeakPtr<ui::MojoBubbleWebUIController::Embedder>
-        side_panel_embedder) {
+    base::WeakPtr<TopChromeWebUIController::Embedder> side_panel_embedder) {
   history_clusters_side_panel_embedder_ = side_panel_embedder;
 }
 
+void HistoryClustersHandler::SetContextInterface(ContextInterface interface) {
+  interface_ = interface;
+}
+
 void HistoryClustersHandler::SetQuery(const std::string& query) {
   if (page_) {
     page_->OnQueryChangedByUser(query);
@@ -201,10 +263,6 @@ void HistoryClustersHandler::SetQuery(const std::string& query) {
 void HistoryClustersHandler::OpenHistoryCluster(
     const GURL& url,
     ui::mojom::ClickModifiersPtr click_modifiers) {
-  Browser* browser = chrome::FindLastActive();
-  if (!browser)
-    return;
-
   // In the Side Panel, the default is the current tab. From History WebUI, the
   // default is a new foreground tab.
   WindowOpenDisposition default_disposition =
@@ -219,7 +277,9 @@ void HistoryClustersHandler::OpenHistoryCluster(
   content::OpenURLParams params(url, content::Referrer(), open_location,
                                 ui::PAGE_TRANSITION_AUTO_BOOKMARK,
                                 /*is_renderer_initiated=*/false);
-  browser->OpenURL(params);
+  GetBrowserWindowInterface(interface_)
+      ->OpenURL(params,
+                /*navigation_handle_callback=*/{});
 }
 
 void HistoryClustersHandler::SetPage(
@@ -240,8 +300,10 @@ void HistoryClustersHandler::ToggleVisibility(
   std::move(callback).Run(visible);
 }
 
-void HistoryClustersHandler::StartQueryClusters(const std::string& query,
-                                                bool recluster) {
+void HistoryClustersHandler::StartQueryClusters(
+    const std::string& query,
+    std::optional<base::Time> begin_time,
+    bool recluster) {
   last_query_issued_ = query;
 
   if (!query.empty()) {
@@ -257,7 +319,8 @@ void HistoryClustersHandler::StartQueryClusters(const std::string& query,
   auto* history_clusters_service =
       HistoryClustersServiceFactory::GetForBrowserContext(profile_);
   query_clusters_state_ = std::make_unique<QueryClustersState>(
-      history_clusters_service->GetWeakPtr(), query, recluster);
+      history_clusters_service->GetWeakPtr(), history_service_, query,
+      begin_time.value_or(base::Time()), recluster);
   LoadMoreClusters(query);
 }
 
@@ -342,11 +405,6 @@ void HistoryClustersHandler::RemoveVisits(
 void HistoryClustersHandler::OpenVisitUrlsInTabGroup(
     std::vector<mojom::URLVisitPtr> visits,
     const std::optional<std::string>& tab_group_name) {
-  auto* browser = chrome::FindTabbedBrowser(profile_, false);
-  if (!browser) {
-    return;
-  }
-
   // Hard cap the number of opened visits in a tab group to 32. It's a
   // relatively high cap chosen fairly arbitrarily, because the user took an
   // affirmative action to open this many tabs. And hidden visits aren't opened.
@@ -355,14 +413,18 @@ void HistoryClustersHandler::OpenVisitUrlsInTabGroup(
     visits.resize(kMaxVisitsToOpenInTabGroup);
   }
 
-  auto* model = browser->tab_strip_model();
+  auto* model =
+      GetBrowserWindowInterface(interface_)->GetFeatures().tab_strip_model();
   std::vector<int> tab_indices;
   tab_indices.reserve(visits.size());
   for (const auto& visit_ptr : visits) {
-    auto* opened_web_contents = browser->OpenURL(
-        content::OpenURLParams(visit_ptr->normalized_url, content::Referrer(),
-                               WindowOpenDisposition::NEW_BACKGROUND_TAB,
-                               ui::PAGE_TRANSITION_AUTO_BOOKMARK, false));
+    auto* opened_web_contents =
+        GetBrowserWindowInterface(interface_)
+            ->OpenURL(content::OpenURLParams(
+                          visit_ptr->normalized_url, content::Referrer(),
+                          WindowOpenDisposition::NEW_BACKGROUND_TAB,
+                          ui::PAGE_TRANSITION_AUTO_BOOKMARK, false),
+                      /*navigation_handle_callback=*/{});
 
     // Only add those tabs to a new group that actually opened in this browser.
     const int tab_index = model->GetIndexOfWebContents(opened_web_contents);
@@ -471,21 +533,19 @@ void HistoryClustersHandler::RecordToggledVisibility(bool visible) {
 void HistoryClustersHandler::ShowContextMenuForSearchbox(
     const std::string& query,
     const gfx::Point& point) {
-  Browser* browser = chrome::FindLastActive();
   if (history_clusters_side_panel_embedder_) {
     history_clusters_side_panel_embedder_->ShowContextMenu(
-        point,
-        std::make_unique<HistoryClustersSidePanelContextMenu>(browser, query));
+        point, std::make_unique<HistoryClustersSidePanelContextMenu>(interface_,
+                                                                     query));
   }
 }
 
 void HistoryClustersHandler::ShowContextMenuForURL(const GURL& url,
                                                    const gfx::Point& point) {
-  Browser* browser = chrome::FindLastActive();
   if (history_clusters_side_panel_embedder_) {
     history_clusters_side_panel_embedder_->ShowContextMenu(
         point,
-        std::make_unique<HistoryClustersSidePanelContextMenu>(browser, url));
+        std::make_unique<HistoryClustersSidePanelContextMenu>(interface_, url));
   }
 }
 
