@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/ui/webui/net_internals/net_internals_ui.h"
 
 #include <memory>
@@ -10,6 +15,7 @@
 #include <utility>
 
 #include "base/base64.h"
+#include "base/containers/to_value_list.h"
 #include "base/containers/unique_ptr_adapters.h"
 #include "base/functional/bind.h"
 #include "base/functional/callback_helpers.h"
@@ -43,8 +49,9 @@
 #include "net/base/schemeful_site.h"
 #include "net/dns/public/host_resolver_results.h"
 #include "net/dns/public/resolve_error_info.h"
-#include "net/extras/shared_dictionary/shared_dictionary_isolation_key.h"
 #include "net/extras/shared_dictionary/shared_dictionary_usage_info.h"
+#include "net/shared_dictionary/shared_dictionary_isolation_key.h"
+#include "services/network/public/cpp/request_destination.h"
 #include "services/network/public/mojom/clear_data_filter.mojom.h"
 #include "services/network/public/mojom/content_security_policy.mojom.h"
 #include "services/network/public/mojom/host_resolver.mojom.h"
@@ -72,11 +79,8 @@ void IgnoreBoolCallback(bool result) {}
 // This function converts std::vector<net::IPEndPoint> to base::Value::List.
 base::Value::List IPEndpointsToBaseList(
     const std::vector<net::IPEndPoint>& resolved_addresses) {
-  base::Value::List resolved_addresses_list;
-  for (const net::IPEndPoint& resolved_address : resolved_addresses) {
-    resolved_addresses_list.Append(resolved_address.ToStringWithoutPort());
-  }
-  return resolved_addresses_list;
+  return base::ToValueList(resolved_addresses,
+                           &net::IPEndPoint::ToStringWithoutPort);
 }
 
 // This function converts std::optional<net::HostResolverEndpointResults> to
@@ -109,6 +113,17 @@ base::Value::List HostResolverEndpointResultsToBaseList(
   return endpoint_results_list;
 }
 
+base::Value::List GetMatchDestList(
+    const std::vector<::network::mojom::RequestDestination>& match_dest) {
+  base::Value::List result =
+      base::Value::List::with_capacity(match_dest.size());
+  for (const auto& item : match_dest) {
+    result.Append(network::RequestDestinationToString(
+        item, network::EmptyRequestDestinationOption::kUseTheEmptyString));
+  }
+  return result;
+}
+
 // This class implements network::mojom::ResolveHostClient.
 class NetInternalsResolveHostClient : public network::mojom::ResolveHostClient {
  public:
@@ -145,10 +160,10 @@ class NetInternalsResolveHostClient : public network::mojom::ResolveHostClient {
                              endpoint_results_with_metadata, this);
   }
   void OnTextResults(const std::vector<std::string>& text_results) override {
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
   }
   void OnHostnameResults(const std::vector<net::HostPortPair>& hosts) override {
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
   }
 
  private:
@@ -469,7 +484,7 @@ void NetInternalsMessageHandler::OnResolveHostDone(
       IPEndpointsToBaseList(resolved_addresses->endpoints());
   result.Set("resolved_addresses", std::move(resolved_addresses_list));
 
-  // TODO(crbug.com/1416410): Rename `endpoint_results_with_metadata` in the
+  // TODO(crbug.com/40256843): Rename `endpoint_results_with_metadata` in the
   // Mojo API to `alternative_endpoints`, to match the terminology used in the
   // specification.
   base::Value::List alternative_endpoints_list =
@@ -503,7 +518,10 @@ void NetInternalsMessageHandler::OnGetSharedDictionaryInfoDone(
   for (const auto& item : dictionaries) {
     base::Value::Dict dict;
     dict.Set("match", item->match);
+    dict.Set("match_dest", GetMatchDestList(item->match_dest));
+    dict.Set("id", item->id);
     dict.Set("dictionary_url", item->dictionary_url.spec());
+    dict.Set("last_fetch_time", base::TimeFormatHTTP(item->last_fetch_time));
     dict.Set("response_time", base::TimeFormatHTTP(item->response_time));
     dict.Set("expiration", base::NumberToString(item->expiration.InSeconds()));
     dict.Set("last_used_time", base::TimeFormatHTTP(item->last_used_time));
