@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/ui/webui/new_tab_page_third_party/new_tab_page_third_party_ui.h"
 
 #include <memory>
@@ -13,12 +18,12 @@
 #include "chrome/browser/themes/theme_service.h"
 #include "chrome/browser/ui/color/chrome_color_id.h"
 #include "chrome/browser/ui/webui/cr_components/most_visited/most_visited_handler.h"
-#include "chrome/browser/ui/webui/customize_themes/chrome_customize_themes_handler.h"
 #include "chrome/browser/ui/webui/favicon_source.h"
 #include "chrome/browser/ui/webui/new_tab_page_third_party/new_tab_page_third_party_handler.h"
 #include "chrome/browser/ui/webui/ntp/ntp_resource_cache.h"
 #include "chrome/browser/ui/webui/theme_source.h"
 #include "chrome/browser/ui/webui/webui_util.h"
+#include "chrome/browser/ui/webui/webui_util_desktop.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/common/webui_url_constants.h"
@@ -63,8 +68,8 @@ void CreateAndAddNewTabPageThirdPartyUiHtmlSource(Profile* profile,
   source->AddLocalizedStrings(kStrings);
 
   const ui::ThemeProvider* theme_provider =
-      webui::GetThemeProvider(web_contents);
-  // TODO(crbug.com/1299925): Always mock theme provider in tests so that
+      webui::GetThemeProviderDeprecated(web_contents);
+  // TODO(crbug.com/40823895): Always mock theme provider in tests so that
   // `theme_provider` is never nullptr.
   if (theme_provider) {
     const ui::ColorProvider& color_provider = web_contents->GetColorProvider();
@@ -74,9 +79,9 @@ void CreateAndAddNewTabPageThirdPartyUiHtmlSource(Profile* profile,
                       GetNewTabBackgroundTilingCSS(*theme_provider));
     source->AddString("colorBackground",
                       color_utils::SkColorToRgbaString(GetThemeColor(
-                          webui::GetNativeTheme(web_contents), color_provider,
-                          kColorNewTabPageBackground)));
-    // TODO(crbug.com/1056758): don't get theme id from profile.
+                          webui::GetNativeThemeDeprecated(web_contents),
+                          color_provider, kColorNewTabPageBackground)));
+    // TODO(crbug.com/40120448): don't get theme id from profile.
     source->AddString("themeId",
                       profile->GetPrefs()->GetString(prefs::kCurrentThemeID));
     source->AddString("hascustombackground",
@@ -97,17 +102,20 @@ void CreateAndAddNewTabPageThirdPartyUiHtmlSource(Profile* profile,
     source->AddString("isdark", "");
   }
 
-  source->AddBoolean(
-      "handleMostVisitedNavigationExplicitly",
-      base::FeatureList::IsEnabled(
-          ntp_features::kNtpHandleMostVisitedNavigationExplicitly));
-
-  source->AddBoolean(
-      "prerenderEnabled",
-      base::FeatureList::IsEnabled(features::kNewTabPageTriggerForPrerender2));
   source->AddInteger(
       "prerenderStartTimeThreshold",
       features::kNewTabPagePrerenderStartDelayOnMouseHoverByMiliSeconds.Get());
+  source->AddInteger(
+      "preconnectStartTimeThreshold",
+      features::kNewTabPagePreconnectStartDelayOnMouseHoverByMiliSeconds.Get());
+  source->AddBoolean(
+      "prerenderOnPressEnabled",
+      base::FeatureList::IsEnabled(features::kNewTabPageTriggerForPrerender2) &&
+          features::kPrerenderNewTabPageOnMousePressedTrigger.Get());
+  source->AddBoolean(
+      "prerenderOnHoverEnabled",
+      base::FeatureList::IsEnabled(features::kNewTabPageTriggerForPrerender2) &&
+          features::kPrerenderNewTabPageOnMouseHoverTrigger.Get());
 
   // Needed by <cr-most-visited> but not used in
   // chrome://new-tab-page-third-party/.
