@@ -29,7 +29,7 @@
 #include "chrome/browser/ui/cookie_controls/cookie_controls_service_factory.h"
 #include "chrome/browser/ui/layout_constants.h"
 #include "chrome/browser/ui/webui/ntp/cookie_controls_handler.h"
-#include "chrome/browser/ui/webui/webui_util.h"
+#include "chrome/browser/ui/webui/webui_util_desktop.h"
 #include "chrome/common/buildflags.h"
 #include "chrome/common/chrome_features.h"
 #include "chrome/common/pref_names.h"
@@ -64,10 +64,6 @@
 #include "ui/chromeos/devicetype_utils.h"
 #endif
 
-#if !BUILDFLAG(IS_CHROMEOS)
-#include "chrome/browser/ui/webui/ntp/app_launcher_handler.h"
-#endif
-
 using content::BrowserThread;
 
 namespace {
@@ -91,10 +87,8 @@ const char kLearnMoreGuestSessionUrl[] =
 std::string ReplaceTemplateExpressions(
     const scoped_refptr<base::RefCountedMemory>& bytes,
     const ui::TemplateReplacements& replacements) {
-  return ui::ReplaceTemplateExpressions(
-      base::StringPiece(reinterpret_cast<const char*>(bytes->front()),
-                        bytes->size()),
-      replacements);
+  return ui::ReplaceTemplateExpressions(base::as_string_view(*bytes),
+                                        replacements);
 }
 
 }  // namespace
@@ -144,10 +138,9 @@ NTPResourceCache::NTPResourceCache(Profile* profile)
 
   // Watch for pref changes that cause us to need to invalidate the HTML cache.
   profile_pref_change_registrar_.Init(profile_->GetPrefs());
-  profile_pref_change_registrar_.Add(prefs::kNtpShownPage, callback);
   profile_pref_change_registrar_.Add(prefs::kCookieControlsMode, callback);
 
-  // TODO(crbug/1056916): Remove the global accessor to NativeTheme.
+  // TODO(crbug.com/40677117): Remove the global accessor to NativeTheme.
   theme_observation_.Observe(ui::NativeTheme::GetInstanceForNativeUi());
 
   policy_change_registrar_ = std::make_unique<policy::PolicyChangeRegistrar>(
@@ -200,7 +193,7 @@ base::RefCountedMemory* NTPResourceCache::GetNewTabHTML(
       return new_tab_non_primary_otr_html_.get();
 
     case NORMAL:
-      NOTREACHED();
+      NOTREACHED_IN_MIGRATION();
       return nullptr;
   }
 }
@@ -236,7 +229,7 @@ void NTPResourceCache::Shutdown() {
 }
 
 void NTPResourceCache::OnNativeThemeUpdated(ui::NativeTheme* updated_theme) {
-  // TODO(crbug/1056916): Remove the global accessor to NativeTheme.
+  // TODO(crbug.com/40677117): Remove the global accessor to NativeTheme.
   DCHECK_EQ(updated_theme, ui::NativeTheme::GetInstanceForNativeUi());
   Invalidate();
 }
@@ -278,39 +271,18 @@ void NTPResourceCache::CreateNewTabIncognitoHTML(
   bool is_tracking_protection_3pcd_enabled =
       tracking_protection_settings->IsTrackingProtection3pcdEnabled();
 
-  bool use_revamped_ui =
-      base::FeatureList::IsEnabled(features::kIncognitoNtpRevamp);
-  if (use_revamped_ui) {
-    replacements["incognitoTabHeading"] =
-        l10n_util::GetStringUTF8(IDS_REVAMPED_INCOGNITO_NTP_TITLE);
-    replacements["incognitoDoesHeader"] =
-        l10n_util::GetStringUTF8(IDS_REVAMPED_INCOGNITO_NTP_DOES_HEADER);
-    replacements["incognitoDoesDescription"] =
-        l10n_util::GetStringUTF8(IDS_REVAMPED_INCOGNITO_NTP_DOES_DESCRIPTION);
-    replacements["incognitoDoesNotHeader"] =
-        l10n_util::GetStringUTF8(IDS_REVAMPED_INCOGNITO_NTP_DOES_NOT_HEADER);
-    replacements["incognitoDoesNotDescription"] = l10n_util::GetStringUTF8(
-        IDS_REVAMPED_INCOGNITO_NTP_DOES_NOT_DESCRIPTION);
-    replacements["learnMore"] =
-        l10n_util::GetStringUTF8(IDS_REVAMPED_INCOGNITO_NTP_LEARN_MORE);
-    replacements["cookieControlsTitle"] = l10n_util::GetStringUTF8(
-        IDS_REVAMPED_INCOGNITO_NTP_OTR_THIRD_PARTY_COOKIE);
-    replacements["cookieControlsDescription"] = l10n_util::GetStringUTF8(
-        IDS_REVAMPED_INCOGNITO_NTP_OTR_THIRD_PARTY_COOKIE_SUBLABEL);
-  } else {
-    replacements["incognitoTabHeading"] =
-        l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_TITLE);
-    replacements["incognitoTabWarning"] =
-        l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_VISIBLE);
-    replacements["incognitoTabFeatures"] =
-        l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_NOT_SAVED);
-    replacements["learnMore"] =
-        l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_LEARN_MORE_LINK);
-    replacements["cookieControlsTitle"] =
-        l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_THIRD_PARTY_COOKIE);
-    replacements["cookieControlsDescription"] =
-        l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_THIRD_PARTY_COOKIE_SUBLABEL);
-  }
+  replacements["incognitoTabHeading"] =
+      l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_TITLE);
+  replacements["incognitoTabWarning"] =
+      l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_VISIBLE);
+  replacements["incognitoTabFeatures"] =
+      l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_NOT_SAVED);
+  replacements["learnMore"] =
+      l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_LEARN_MORE_LINK);
+  replacements["cookieControlsTitle"] =
+      l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_THIRD_PARTY_COOKIE);
+  replacements["cookieControlsDescription"] =
+      l10n_util::GetStringUTF8(IDS_NEW_TAB_OTR_THIRD_PARTY_COOKIE_SUBLABEL);
 
   replacements["learnMoreLink"] = kLearnMoreIncognitoUrl;
   replacements["learnMoreA11yLabel"] = l10n_util::GetStringUTF8(
@@ -351,13 +323,10 @@ void NTPResourceCache::CreateNewTabIncognitoHTML(
   const std::string& app_locale = g_browser_process->GetApplicationLocale();
   webui::SetLoadTimeDataDefaults(app_locale, &replacements);
 
-  int incognito_tab_html_resource_id = use_revamped_ui
-                                           ? IDR_REVAMPED_INCOGNITO_TAB_HTML
-                                           : IDR_INCOGNITO_TAB_HTML;
   static const base::NoDestructor<scoped_refptr<base::RefCountedMemory>>
       incognito_tab_html(
           ui::ResourceBundle::GetSharedInstance().LoadDataResourceBytes(
-              incognito_tab_html_resource_id));
+              IDR_INCOGNITO_TAB_HTML));
   CHECK(*incognito_tab_html);
   new_tab_incognito_html_ = base::MakeRefCounted<base::RefCountedString>(
       ReplaceTemplateExpressions(*incognito_tab_html, replacements));
@@ -384,6 +353,10 @@ void NTPResourceCache::CreateNewTabGuestHTML() {
                           l10n_util::GetStringUTF16(IDS_LEARN_MORE));
     localized_strings.Set("enterpriseInfoHintLink",
                           chrome::kLearnMoreEnterpriseURL);
+    localized_strings.Set(
+        "enterpriseLearnMoreA11yLabel",
+        l10n_util::GetStringUTF16(
+            IDS_NEW_TAB_ENTERPRISE_GUEST_SESSION_LEARN_MORE_ACCESSIBILITY_TEXT));
     std::u16string enterprise_info;
     if (connector->IsCloudManaged()) {
       const std::string enterprise_domain_manager =
@@ -392,7 +365,7 @@ void NTPResourceCache::CreateNewTabGuestHTML() {
           IDS_ASH_ENTERPRISE_DEVICE_MANAGED_BY, ui::GetChromeOSDeviceName(),
           base::UTF8ToUTF16(enterprise_domain_manager));
     } else {
-      NOTREACHED() << "Unknown management type";
+      NOTREACHED_IN_MIGRATION() << "Unknown management type";
     }
     localized_strings.Set("enterpriseInfoMessage", enterprise_info);
   } else {
@@ -400,6 +373,7 @@ void NTPResourceCache::CreateNewTabGuestHTML() {
     localized_strings.Set("enterpriseInfoMessage", "");
     localized_strings.Set("enterpriseLearnMore", "");
     localized_strings.Set("enterpriseInfoHintLink", "");
+    localized_strings.Set("enterpriseLearnMoreA11yLabel", "");
   }
 #endif
 
@@ -432,7 +406,8 @@ void NTPResourceCache::CreateNewTabGuestHTML() {
 void NTPResourceCache::CreateNewTabIncognitoCSS(
     const content::WebContents::Getter& wc_getter) {
   auto* web_contents = wc_getter.Run();
-  const ui::NativeTheme* native_theme = webui::GetNativeTheme(web_contents);
+  const ui::NativeTheme* native_theme =
+      webui::GetNativeThemeDeprecated(web_contents);
   DCHECK(native_theme);
 
   const ui::ThemeProvider& tp = ThemeService::GetThemeProviderForProfile(
@@ -465,7 +440,8 @@ void NTPResourceCache::CreateNewTabIncognitoCSS(
 void NTPResourceCache::CreateNewTabCSS(
     const content::WebContents::Getter& wc_getter) {
   auto* web_contents = wc_getter.Run();
-  const ui::NativeTheme* native_theme = webui::GetNativeTheme(web_contents);
+  const ui::NativeTheme* native_theme =
+      webui::GetNativeThemeDeprecated(web_contents);
   DCHECK(native_theme);
 
   const ui::ThemeProvider& tp =
