@@ -17,20 +17,30 @@ NTPResourceCache* NTPResourceCacheFactory::GetForProfile(Profile* profile) {
 
 // static
 NTPResourceCacheFactory* NTPResourceCacheFactory::GetInstance() {
-  return base::Singleton<NTPResourceCacheFactory>::get();
+  static base::NoDestructor<NTPResourceCacheFactory> instance;
+  return instance.get();
 }
 
 NTPResourceCacheFactory::NTPResourceCacheFactory()
     : ProfileKeyedServiceFactory(
           "NTPResourceCache",
-          ProfileSelections::BuildRedirectedInIncognito()) {
+          ProfileSelections::Builder()
+              .WithRegular(ProfileSelection::kRedirectedToOriginal)
+              // TODO(crbug.com/40257657): Check if this service is needed in
+              // Guest mode.
+              .WithGuest(ProfileSelection::kRedirectedToOriginal)
+              // TODO(crbug.com/41488885): Check if this service is needed for
+              // Ash Internals.
+              .WithAshInternals(ProfileSelection::kRedirectedToOriginal)
+              .Build()) {
   DependsOn(IdentityManagerFactory::GetInstance());
   DependsOn(ThemeServiceFactory::GetInstance());
 }
 
-NTPResourceCacheFactory::~NTPResourceCacheFactory() {}
+NTPResourceCacheFactory::~NTPResourceCacheFactory() = default;
 
-KeyedService* NTPResourceCacheFactory::BuildServiceInstanceFor(
+std::unique_ptr<KeyedService>
+NTPResourceCacheFactory::BuildServiceInstanceForBrowserContext(
     content::BrowserContext* profile) const {
-  return new NTPResourceCache(static_cast<Profile*>(profile));
+  return std::make_unique<NTPResourceCache>(static_cast<Profile*>(profile));
 }
