@@ -28,7 +28,7 @@
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search/search.h"
 #include "chrome/browser/search_engines/template_url_service_factory.h"
-#include "chrome/browser/ui/webui/omnibox/omnibox.mojom-forward.h"
+#include "chrome/browser/ui/webui/omnibox/omnibox_internals.mojom-forward.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/history/core/browser/history_service.h"
 #include "components/history/core/browser/url_database.h"
@@ -39,49 +39,54 @@
 #include "components/omnibox/browser/autocomplete_match.h"
 #include "components/omnibox/browser/autocomplete_provider.h"
 #include "components/omnibox/browser/autocomplete_result.h"
-#include "components/omnibox/browser/autocomplete_scoring_model_service.h"
+#include "components/omnibox/browser/omnibox_feature_configs.h"
 #include "components/omnibox/browser/omnibox_field_trial.h"
+#include "components/optimization_guide/machine_learning_tflite_buildflags.h"
 #include "components/search_engines/template_url.h"
 #include "content/public/browser/web_ui.h"
 #include "third_party/metrics_proto/omnibox_event.pb.h"
 #include "third_party/metrics_proto/omnibox_focus_type.pb.h"
+#include "third_party/omnibox_proto/answer_data.pb.h"
+#include "third_party/omnibox_proto/answer_type.pb.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/gfx/image/image.h"
 
+#if BUILDFLAG(BUILD_WITH_TFLITE_LIB)
+#include "components/omnibox/browser/autocomplete_scoring_model_service.h"
+#endif
+
 using bookmarks::BookmarkModel;
 
 namespace {
 
-std::string SuggestionAnswerTypeToString(int answer_type) {
+std::string AnswerTypeToString(int answer_type) {
   switch (answer_type) {
-    case SuggestionAnswer::ANSWER_TYPE_INVALID:
+    case omnibox::ANSWER_TYPE_UNSPECIFIED:
       return "invalid";
-    case SuggestionAnswer::ANSWER_TYPE_DICTIONARY:
+    case omnibox::ANSWER_TYPE_DICTIONARY:
       return "dictionary";
-    case SuggestionAnswer::ANSWER_TYPE_FINANCE:
+    case omnibox::ANSWER_TYPE_FINANCE:
       return "finance";
-    case SuggestionAnswer::ANSWER_TYPE_KNOWLEDGE_GRAPH:
+    case omnibox::ANSWER_TYPE_GENERIC_ANSWER:
       return "knowledge graph";
-    case SuggestionAnswer::ANSWER_TYPE_LOCAL:
-      return "local";
-    case SuggestionAnswer::ANSWER_TYPE_SPORTS:
+    case omnibox::ANSWER_TYPE_SPORTS:
       return "sports";
-    case SuggestionAnswer::ANSWER_TYPE_SUNRISE:
+    case omnibox::ANSWER_TYPE_SUNRISE_SUNSET:
       return "sunrise";
-    case SuggestionAnswer::ANSWER_TYPE_TRANSLATION:
+    case omnibox::ANSWER_TYPE_TRANSLATION:
       return "translation";
-    case SuggestionAnswer::ANSWER_TYPE_WEATHER:
+    case omnibox::ANSWER_TYPE_WEATHER:
       return "weather";
-    case SuggestionAnswer::ANSWER_TYPE_WHEN_IS:
+    case omnibox::ANSWER_TYPE_WHEN_IS:
       return "when is";
-    case SuggestionAnswer::ANSWER_TYPE_CURRENCY:
+    case omnibox::ANSWER_TYPE_CURRENCY:
       return "currency";
-    case SuggestionAnswer::ANSWER_TYPE_LOCAL_TIME:
+    case omnibox::ANSWER_TYPE_LOCAL_TIME:
       return "local time";
-    case SuggestionAnswer::ANSWER_TYPE_PLAY_INSTALL:
+    case omnibox::ANSWER_TYPE_PLAY_INSTALL:
       return "play install";
     default:
-      return std::to_string(answer_type);
+      return base::NumberToString(answer_type);
   }
 }
 
@@ -131,15 +136,20 @@ struct TypeConverter<mojom::SignalsPtr, AutocompleteMatch::ScoringSignals> {
       const AutocompleteMatch::ScoringSignals signals) {
     // Keep consistent:
     // - omnibox_event.proto `ScoringSignals`
+    // - omnibox_scoring_signals.proto `OmniboxScoringSignals`
     // - autocomplete_scoring_model_handler.cc
     //   `AutocompleteScoringModelHandler::ExtractInputFromScoringSignals()`
     // - autocomplete_match.cc `AutocompleteMatch::MergeScoringSignals()`
+    // - autocomplete_controller.cc `RecordScoringSignalCoverageForProvider()`
+    // - omnibox_metrics_provider.cc `GetScoringSignalsForLogging()`
     // - omnibox.mojom `struct Signals`
     // - omnibox_page_handler.cc
     //   `TypeConverter<AutocompleteMatch::ScoringSignals, mojom::SignalsPtr>`
     // - omnibox_page_handler.cc `TypeConverter<mojom::SignalsPtr,
     //   AutocompleteMatch::ScoringSignals>`
     // - omnibox_util.ts `signalNames`
+    // - omnibox/histograms.xml
+    //   `Omnibox.URLScoringModelExecuted.ScoringSignalCoverage`
 
     mojom::SignalsPtr mojom_signals(mojom::Signals::New());
 
@@ -167,6 +177,13 @@ struct TypeConverter<mojom::SignalsPtr, AutocompleteMatch::ScoringSignals> {
     PROTO_TO_MOJOM_SIGNAL(length_of_url);
     PROTO_TO_MOJOM_SIGNAL(site_engagement);
     PROTO_TO_MOJOM_SIGNAL(allowed_to_be_default_match);
+    PROTO_TO_MOJOM_SIGNAL(search_suggest_relevance);
+    PROTO_TO_MOJOM_SIGNAL(is_search_suggest_entity);
+    PROTO_TO_MOJOM_SIGNAL(is_verbatim);
+    PROTO_TO_MOJOM_SIGNAL(is_navsuggest);
+    PROTO_TO_MOJOM_SIGNAL(is_search_suggest_tail);
+    PROTO_TO_MOJOM_SIGNAL(is_answer_suggest);
+    PROTO_TO_MOJOM_SIGNAL(is_calculator_suggest);
 
     return mojom_signals;
   }
@@ -178,15 +195,20 @@ struct TypeConverter<AutocompleteMatch::ScoringSignals, mojom::SignalsPtr> {
       const mojom::SignalsPtr& mojom_signals) {
     // Keep consistent:
     // - omnibox_event.proto `ScoringSignals`
+    // - omnibox_scoring_signals.proto `OmniboxScoringSignals`
     // - autocomplete_scoring_model_handler.cc
-    // `AutocompleteScoringModelHandler::ExtractInputFromScoringSignals()`
+    //   `AutocompleteScoringModelHandler::ExtractInputFromScoringSignals()`
     // - autocomplete_match.cc `AutocompleteMatch::MergeScoringSignals()`
+    // - autocomplete_controller.cc `RecordScoringSignalCoverageForProvider()`
+    // - omnibox_metrics_provider.cc `GetScoringSignalsForLogging()`
     // - omnibox.mojom `struct Signals`
     // - omnibox_page_handler.cc
-    // `TypeConverter<AutocompleteMatch::ScoringSignals, mojom::SignalsPtr>`
+    //   `TypeConverter<AutocompleteMatch::ScoringSignals, mojom::SignalsPtr>`
     // - omnibox_page_handler.cc `TypeConverter<mojom::SignalsPtr,
-    // AutocompleteMatch::ScoringSignals>`
+    //   AutocompleteMatch::ScoringSignals>`
     // - omnibox_util.ts `signalNames`
+    // - omnibox/histograms.xml
+    //   `Omnibox.URLScoringModelExecuted.ScoringSignalCoverage`
 
     AutocompleteMatch::ScoringSignals signals;
 
@@ -214,6 +236,13 @@ struct TypeConverter<AutocompleteMatch::ScoringSignals, mojom::SignalsPtr> {
     MOJOM_TO_PROTO_SIGNAL(length_of_url);
     MOJOM_TO_PROTO_SIGNAL(site_engagement);
     MOJOM_TO_PROTO_SIGNAL(allowed_to_be_default_match);
+    MOJOM_TO_PROTO_SIGNAL(search_suggest_relevance);
+    MOJOM_TO_PROTO_SIGNAL(is_search_suggest_entity);
+    MOJOM_TO_PROTO_SIGNAL(is_verbatim);
+    MOJOM_TO_PROTO_SIGNAL(is_navsuggest);
+    MOJOM_TO_PROTO_SIGNAL(is_search_suggest_tail);
+    MOJOM_TO_PROTO_SIGNAL(is_answer_suggest);
+    MOJOM_TO_PROTO_SIGNAL(is_calculator_suggest);
 
     return signals;
   }
@@ -264,12 +293,18 @@ struct TypeConverter<mojom::AutocompleteMatchPtr, AutocompleteMatch> {
         mojo::ConvertTo<std::vector<mojom::ACMatchClassificationPtr>>(
             input.description_class);
     result->swap_contents_and_description = input.swap_contents_and_description;
-    if (input.answer) {
+    if (omnibox_feature_configs::SuggestionAnswerMigration::Get().enabled &&
+        input.answer_template) {
+      omnibox::AnswerData answer_data = input.answer_template->answers(0);
+      result->answer = answer_data.headline().text() + " / " +
+                       answer_data.subhead().text() + " / " +
+                       AnswerTypeToString(input.answer_type);
+    } else if (input.answer) {
       result->answer =
           SuggestionAnswerImageLineToString(input.answer->first_line()) +
           " / " +
           SuggestionAnswerImageLineToString(input.answer->second_line()) +
-          " / " + SuggestionAnswerTypeToString(input.answer->type());
+          " / " + AnswerTypeToString(input.answer_type);
     }
     result->transition =
         ui::PageTransitionGetCoreTransitionString(input.transition);
@@ -370,37 +405,37 @@ void OmniboxPageHandler::OnResultChanged(AutocompleteController* controller,
     response->combined_results =
         mojo::ConvertTo<std::vector<mojom::AutocompleteMatchPtr>>(matches);
   }
+  std::vector<scoped_refptr<AutocompleteProvider>> providers = {};
+  for (const auto& provider : controller->providers())
+    if (controller->ShouldRunProvider(provider.get()))
+      providers.push_back(provider);
   response->results_by_provider =
       mojo::ConvertTo<std::vector<mojom::AutocompleteResultsForProviderPtr>>(
-          controller->providers());
+          providers);
 
   // Fill AutocompleteMatch::starred.
   BookmarkModel* bookmark_model =
       BookmarkModelFactory::GetForBrowserContext(profile_);
   if (bookmark_model) {
-    for (size_t i = 0; i < response->combined_results.size(); ++i) {
-      response->combined_results[i]->starred = bookmark_model->IsBookmarked(
-          GURL(response->combined_results[i]->destination_url));
+    for (const auto& match : response->combined_results) {
+      match->starred =
+          bookmark_model->IsBookmarked(GURL(match->destination_url));
     }
-    for (size_t i = 0; i < response->results_by_provider.size(); ++i) {
-      const mojom::AutocompleteResultsForProvider& result_by_provider =
-          *response->results_by_provider[i];
-      for (size_t j = 0; j < result_by_provider.results.size(); ++j) {
-        result_by_provider.results[j]->starred = bookmark_model->IsBookmarked(
-            GURL(result_by_provider.results[j]->destination_url));
+    for (const auto& results_by_provider : response->results_by_provider) {
+      for (const auto& match : results_by_provider->results) {
+        match->starred =
+            bookmark_model->IsBookmarked(GURL(match->destination_url));
       }
     }
   }
 
   // Obtain a vector of all image urls required.
   std::vector<std::string> image_urls;
-  for (size_t i = 0; i < response->combined_results.size(); ++i)
-    image_urls.push_back(response->combined_results[i]->image);
-  for (size_t i = 0; i < response->results_by_provider.size(); ++i) {
-    const mojom::AutocompleteResultsForProvider& result_by_provider =
-        *response->results_by_provider[i];
-    for (size_t j = 0; j < result_by_provider.results.size(); ++j)
-      image_urls.push_back(result_by_provider.results[j]->image);
+  for (const auto& match : response->combined_results)
+    image_urls.push_back(match->image);
+  for (const auto& results_by_provider : response->results_by_provider) {
+    for (const auto& match : results_by_provider->results)
+      image_urls.push_back(match->image);
   }
 
   auto type = GetAutocompleteControllerType(controller);
@@ -435,9 +470,7 @@ void OmniboxPageHandler::OnBitmapFetched(mojom::AutocompleteControllerType type,
                                          const std::string& image_url,
                                          const SkBitmap& bitmap) {
   auto data = gfx::Image::CreateFrom1xBitmap(bitmap).As1xPNGBytes();
-  std::string base_64;
-  base::Base64Encode(base::StringPiece(data->front_as<char>(), data->size()),
-                     &base_64);
+  std::string base_64 = base::Base64Encode(*data);
   const char kDataUrlPrefix[] = "data:image/png;base64,";
   std::string data_url = GURL(kDataUrlPrefix + base_64).spec();
   page_->HandleAnswerImageData(type, image_url, data_url);
@@ -500,6 +533,7 @@ void OmniboxPageHandler::StartOmniboxQuery(const std::string& input_string,
 }
 
 void OmniboxPageHandler::GetMlModelVersion(GetMlModelVersionCallback callback) {
+#if BUILDFLAG(BUILD_WITH_TFLITE_LIB)
   if (auto* service = GetMlService()) {
     auto version = service->GetModelVersion();
     if (version == -1) {
@@ -512,10 +546,14 @@ void OmniboxPageHandler::GetMlModelVersion(GetMlModelVersionCallback callback) {
   } else {
     std::move(callback).Run(-1);
   }
+#else
+  std::move(callback).Run(-1);
+#endif
 }
 
 void OmniboxPageHandler::StartMl(mojom::SignalsPtr mojom_signals,
                                  StartMlCallback callback) {
+#if BUILDFLAG(BUILD_WITH_TFLITE_LIB)
   if (auto* service = GetMlService()) {
     AutocompleteMatch::ScoringSignals signals =
         mojo::ConvertTo<AutocompleteMatch::ScoringSignals>(mojom_signals);
@@ -525,13 +563,23 @@ void OmniboxPageHandler::StartMl(mojom::SignalsPtr mojom_signals,
   } else {
     std::move(callback).Run(-1);
   }
+#else
+  std::move(callback).Run(-1);
+#endif
 }
 
 std::unique_ptr<AutocompleteController> OmniboxPageHandler::CreateController(
     bool ml_disabled) {
+  auto providers = AutocompleteClassifier::DefaultOmniboxProviders();
+  // `HistoryEmbeddingsProvider` only supports 1 query at a time. Running it for
+  // the traditional-scoring controller used in the ML before/after comparisons
+  // would break history embeddings for the other, more important controllers.
+  if (ml_disabled)
+    providers &= ~AutocompleteProvider::TYPE_HISTORY_EMBEDDINGS;
+
   auto controller = std::make_unique<AutocompleteController>(
-      std::make_unique<ChromeAutocompleteProviderClient>(profile_),
-      AutocompleteClassifier::DefaultOmniboxProviders(), false, ml_disabled);
+      std::make_unique<ChromeAutocompleteProviderClient>(profile_), providers,
+      false, ml_disabled);
   // We will observe our internal AutocompleteController directly, so there's
   // no reason to hook it up to the profile-keyed AutocompleteControllerEmitter.
   controller->AddObserver(this);
@@ -549,8 +597,12 @@ OmniboxPageHandler::GetAutocompleteControllerType(
 }
 
 AutocompleteScoringModelService* OmniboxPageHandler::GetMlService() {
+#if BUILDFLAG(BUILD_WITH_TFLITE_LIB)
   return OmniboxFieldTrial::IsMlUrlScoringEnabled()
              ? AutocompleteScoringModelServiceFactory::GetInstance()
                    ->GetForProfile(profile_)
              : nullptr;
+#else
+  return nullptr;
+#endif
 }
