@@ -6,12 +6,24 @@
 #define CHROME_BROWSER_UI_WEBUI_OMNIBOX_OMNIBOX_UI_H_
 
 #include "build/build_config.h"
-#include "chrome/browser/ui/webui/omnibox/omnibox.mojom-forward.h"
+#include "chrome/browser/ui/webui/omnibox/omnibox_internals.mojom-forward.h"
+#include "chrome/common/webui_url_constants.h"
+#include "content/public/browser/webui_config.h"
+#include "content/public/common/url_constants.h"
 #include "mojo/public/cpp/bindings/pending_receiver.h"
 #include "ui/webui/mojo_web_ui_controller.h"
 
 class OmniboxPageHandler;
 
+class OmniboxUI;
+
+class OmniboxUIConfig : public content::DefaultWebUIConfig<OmniboxUI> {
+ public:
+  OmniboxUIConfig()
+      : DefaultWebUIConfig(content::kChromeUIScheme,
+                           chrome::kChromeUIOmniboxHost) {}
+};
+
 // The UI for chrome://omnibox/
 class OmniboxUI : public ui::MojoWebUIController {
  public:
