@@ -4,25 +4,29 @@
 
 #include <stddef.h>
 
+#include <string_view>
+
 #include "base/test/mock_callback.h"
-#include "base/test/scoped_feature_list.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
+#include "chrome/browser/enterprise/browser_management/browser_management_service.h"
 #include "chrome/browser/enterprise/browser_management/management_service_factory.h"
 #include "chrome/browser/lifetime/application_lifetime.h"
 #include "chrome/browser/policy/chrome_browser_policy_connector.h"
 #include "chrome/browser/policy/profile_policy_connector.h"
 #include "chrome/browser/policy/profile_policy_connector_builder.h"
+#include "chrome/browser/policy/schema_registry_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/profiles/profile_impl.h"
+#include "chrome/browser/ui/webui/policy/policy_ui.h"
 #include "chrome/browser/ui/webui/policy/policy_ui_handler.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/test/base/chrome_test_utils.h"
+#include "chrome/test/base/platform_browser_test.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "components/policy/core/browser/browser_policy_connector.h"
 #include "components/policy/core/common/configuration_policy_provider.h"
-#include "components/policy/core/common/features.h"
 #include "components/policy/core/common/local_test_policy_provider.h"
 #include "components/policy/core/common/management/management_service.h"
 #include "components/policy/core/common/management/scoped_management_service_override_for_testing.h"
@@ -43,11 +47,8 @@
 #include "ui/base/l10n/l10n_util.h"
 #include "url/gurl.h"
 
-#if BUILDFLAG(IS_ANDROID)
-#include "chrome/test/base/android/android_browser_test.h"
-#else
+#if !BUILDFLAG(IS_ANDROID)
 #include "chrome/browser/ui/browser.h"
-#include "chrome/test/base/in_process_browser_test.h"
 #endif
 
 #if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS)
@@ -62,12 +63,7 @@ class PolicyTestPageVisibilityTest
     : public PlatformBrowserTest,
       public ::testing::WithParamInterface<std::tuple<bool, bool, bool>> {
  public:
-  PolicyTestPageVisibilityTest() {
-    // Enable or disable feature as needed
-    scoped_feature_list_.InitWithFeatureState(
-        policy::features::kEnablePolicyTestPage,
-        IsPolicyTestPageEnabledByFeature());
-  }
+  PolicyTestPageVisibilityTest() = default;
   PolicyTestPageVisibilityTest(const PolicyTestPageVisibilityTest&) = delete;
   PolicyTestPageVisibilityTest& operator=(const PolicyTestPageVisibilityTest&) =
       delete;
@@ -111,12 +107,11 @@ class PolicyTestPageVisibilityTest
 
   testing::NiceMock<policy::MockConfigurationPolicyProvider> provider_;
 
-  bool IsPolicyTestPageEnabledByFeature() { return std::get<0>(GetParam()); }
-  bool IsPolicyTestPageEnabledByPolicy() { return std::get<1>(GetParam()); }
+  bool IsPolicyTestPageEnabledByPolicy() { return std::get<0>(GetParam()); }
 
   // Returns true if this profile is not managed.
   bool IsPolicyTestPageEnabledByManagedProfile() {
-    return std::get<2>(GetParam());
+    return std::get<1>(GetParam());
   }
 
   int GetProfileManagement() {
@@ -128,8 +123,7 @@ class PolicyTestPageVisibilityTest
   }
 
   bool GetExpectedValue() {
-    return IsPolicyTestPageEnabledByFeature() &&
-           IsPolicyTestPageEnabledByPolicy() &&
+    return IsPolicyTestPageEnabledByPolicy() &&
            IsPolicyTestPageEnabledByManagedProfile();
   }
 
@@ -153,9 +147,6 @@ class PolicyTestPageVisibilityTest
       EXPECT_TRUE(content::ExecJs(web_contents(), kJavaScript));
     }
   }
-
- private:
-  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 // Verify that the chrome://policy/test page is visible only when both the flag
@@ -193,9 +184,6 @@ INSTANTIATE_TEST_SUITE_P(PolicyTestPageUITestInstance,
 class PolicyTestHandlerTest : public PlatformBrowserTest {
  public:
   PolicyTestHandlerTest() {
-    scoped_feature_list_.InitWithFeatureState(
-        policy::features::kEnablePolicyTestPage, true);
-
 #if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS)
     if (policy::utils::IsPolicyTestingEnabled(/*pref_service=*/nullptr,
                                               chrome::GetChannel())) {
@@ -222,7 +210,7 @@ class PolicyTestHandlerTest : public PlatformBrowserTest {
 
 #if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS)
   void SetUpRelaunchChromeOverrideForPRETests() {
-    base::StringPiece test_name =
+    std::string_view test_name =
         ::testing::UnitTest::GetInstance()->current_test_info()->name();
 
     if (base::StartsWith(test_name, "PRE_")) {
@@ -256,7 +244,6 @@ class PolicyTestHandlerTest : public PlatformBrowserTest {
   content::TestWebUI* web_ui() { return &web_ui_; }
 
  private:
-  base::test::ScopedFeatureList scoped_feature_list_;
   content::TestWebUI web_ui_;
 
 #if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS)
@@ -268,6 +255,57 @@ class PolicyTestHandlerTest : public PlatformBrowserTest {
 #endif
 };
 
+IN_PROC_BROWSER_TEST_F(PolicyTestHandlerTest,
+                       HandleSetLocalTestPoliciesNotSupported) {
+  // Ensure chrome://policy/test not supported.
+  policy::ScopedManagementServiceOverrideForTesting profile_management(
+      policy::ManagementServiceFactory::GetForProfile(GetProfile()),
+      policy::EnterpriseManagementAuthority::CLOUD);
+  std::unique_ptr<PolicyUIHandler> handler = SetUpHandler();
+  const std::string jsonString =
+      R"([
+      {"level": 0,"scope": 0,"source": 0, "namespace": "chrome",
+       "name": "AutofillAddressEnabled","value": false},
+      {"level": 1,"scope": 1,"source": 2, "namespace": "chrome",
+       "name": "CloudReportingEnabled","value": true}
+      ])";
+  const std::string revertAppliedPoliciesButtonDisabledJs =
+      R"(
+        document
+          .querySelector('#revert-applied-policies')
+          .disabled;
+      )";
+
+  base::Value::List list_args;
+
+  list_args.Append("setLocalTestPolicies");
+  list_args.Append(jsonString);
+  list_args.Append("{}");
+
+  // Open chrome://policy
+  ASSERT_TRUE(
+      content::NavigateToURL(web_contents(), GURL(chrome::kChromeUIPolicyURL)));
+  web_ui()->HandleReceivedMessage("setLocalTestPolicies", list_args);
+
+  base::RunLoop().RunUntilIdle();
+
+  const policy::PolicyNamespace chrome_namespace(policy::POLICY_DOMAIN_CHROME,
+                                                 std::string());
+  policy::PolicyService* policy_service =
+      GetProfile()->GetProfilePolicyConnector()->policy_service();
+
+  // Check policies not applied
+  const policy::PolicyMap* policy_map =
+      &policy_service->GetPolicies(chrome_namespace);
+  ASSERT_TRUE(policy_map);
+
+  {
+    const policy::PolicyMap::Entry* entry =
+        policy_map->Get(policy::key::kAutofillAddressEnabled);
+    ASSERT_FALSE(entry);
+  }
+}
+
 IN_PROC_BROWSER_TEST_F(PolicyTestHandlerTest,
                        HandleSetAndRevertLocalTestPolicies) {
   if (!policy::utils::IsPolicyTestingEnabled(/*pref_service=*/nullptr,
@@ -277,10 +315,10 @@ IN_PROC_BROWSER_TEST_F(PolicyTestHandlerTest,
   std::unique_ptr<PolicyUIHandler> handler = SetUpHandler();
   const std::string jsonString =
       R"([
-      {"level": 0,"scope": 0,"source": 0,
-      "name": "AutofillAddressEnabled","value": false},
-      {"level": 1,"scope": 1,"source": 2,
-      "name": "CloudReportingEnabled","value": true}
+      {"level": 0,"scope": 0,"source": 0, "namespace": "chrome",
+       "name": "AutofillAddressEnabled","value": false},
+      {"level": 1,"scope": 1,"source": 2, "namespace": "chrome",
+       "name": "CloudReportingEnabled","value": true}
       ])";
   const std::string revertAppliedPoliciesButtonDisabledJs =
       R"(
@@ -633,13 +671,27 @@ IN_PROC_BROWSER_TEST_F(PolicyTestHandlerTestDisabledByPolicy,
 #endif  // !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS)
 
 namespace {
+
+const char kExtensionId[] = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
+const char kExtensionSchemaJson[] = R"({
+  "type": "object",
+  "properties": {
+    "normal_boolean": {
+      "type": "boolean"
+    },
+    "normal_number": {
+      "type": "number"
+    },
+    "sensitive_number": {
+      "type": "number",
+      "sensitiveValue": true,
+    }
+  }
+})";
+
 class PolicyTestUITest : public PlatformBrowserTest {
  public:
-  PolicyTestUITest() {
-    // Enable kEnablePolicyTestPage feature.
-    scoped_feature_list_.InitWithFeatureState(
-        policy::features::kEnablePolicyTestPage, true);
-  }
+  PolicyTestUITest() = default;
   PolicyTestUITest(const PolicyTestUITest&) = delete;
   PolicyTestUITest& operator=(const PolicyTestUITest&) = delete;
 
@@ -647,6 +699,15 @@ class PolicyTestUITest : public PlatformBrowserTest {
 
   void SetUpOnMainThread() override {
     PlatformBrowserTest::SetUpOnMainThread();
+
+    // Add a fake "extension" to the SchemaRegistry.
+    auto* registry = GetProfile()->GetPolicySchemaRegistryService()->registry();
+    const auto schema = policy::Schema::Parse(kExtensionSchemaJson);
+    ASSERT_TRUE(schema.has_value()) << schema.error();
+    registry->RegisterComponent(
+        policy::PolicyNamespace(policy::POLICY_DOMAIN_EXTENSIONS, kExtensionId),
+        *schema);
+
     // Enable kPolicyTestPageEnabled policy.
     policy::PolicyMap policy_map;
     base::Value::List policy_list;
@@ -714,7 +775,7 @@ class PolicyTestUITest : public PlatformBrowserTest {
   }
 
  private:
-  base::test::ScopedFeatureList scoped_feature_list_;
+  policy::Schema extension_schema_;
   testing::NiceMock<policy::MockConfigurationPolicyProvider> provider_;
 };
 }  // namespace
@@ -807,6 +868,138 @@ IN_PROC_BROWSER_TEST_F(PolicyTestUITest, TestPresetAutofill) {
   EXPECT_EQ(content::EvalJs(web_contents(), getSourceValueJs), "sourceCloud");
 }
 
+IN_PROC_BROWSER_TEST_F(PolicyTestUITest, GetSchema) {
+  base::Value schema = PolicyUI::GetSchema(GetProfile());
+  ASSERT_NE(nullptr, schema.GetDict().FindDict(kExtensionId));
+  // The extension's schema should exclude "sensitive_number", because it's
+  // a sensitive policy.
+  ASSERT_EQ(base::Value::Dict()
+                .Set("normal_boolean", "boolean")
+                .Set("normal_number", "number"),
+            *schema.GetDict().FindDict(kExtensionId));
+}
+
+IN_PROC_BROWSER_TEST_F(PolicyTestUITest, TestExtensionPoliciesIncluded) {
+  if (!policy::utils::IsPolicyTestingEnabled(/*pref_service=*/nullptr,
+                                             chrome::GetChannel())) {
+    GTEST_SKIP() << "chrome://policy/test not allowed on this build.";
+  }
+
+  ASSERT_TRUE(content::NavigateToURL(web_contents(),
+                                     GURL(chrome::kChromeUIPolicyTestURL)));
+
+  base::Value schema = PolicyUI::GetSchema(GetProfile());
+  base::Value chrome_policy_names(base::Value::Type::LIST);
+  for (const auto [key, _] : *schema.GetDict().FindDict("chrome")) {
+    chrome_policy_names.GetList().Append(key);
+  }
+  base::Value extension_policy_names(base::Value::Type::LIST);
+  for (const auto [key, _] : *schema.GetDict().FindDict(kExtensionId)) {
+    extension_policy_names.GetList().Append(key);
+  }
+
+  // The namespace <select> should offer the Chrome policy domain, and the
+  // extension's namespace.
+  const std::string getNamespacesFromSelect =
+      R"(
+        Array.from(
+            document
+                .querySelector('policy-test-table')
+                .shadowRoot
+                .querySelector('policy-test-row')
+                .shadowRoot
+                .querySelector('.namespace')
+                .options)
+            .map(e => e.innerText)
+      )";
+  EXPECT_EQ(
+      base::Value(base::Value::List().Append("Chrome").Append(kExtensionId)),
+      content::EvalJs(web_contents(), getNamespacesFromSelect));
+
+  // Get list of available policies from the <datalist>. They should initially
+  // correspond to the Chrome policy domain.
+  const std::string getPolicyNamesFromDatalist =
+      R"(
+        Array.from(
+            document
+                .querySelector('policy-test-table')
+                .shadowRoot
+                .querySelector('policy-test-row')
+                .shadowRoot
+                .getElementById('policy-name-list')
+                .options)
+            .map(e => e.innerText)
+      )";
+  content::EvalJsResult policy_names =
+      content::EvalJs(web_contents(), getPolicyNamesFromDatalist);
+  EXPECT_EQ(chrome_policy_names, policy_names);
+
+  // Switch to the extension's namespace. This should change the <datalist>'s
+  // policy names.
+  std::string switchToExtensionNamespace =
+      R"(
+        const namespaceDropdown =
+            document
+                .querySelector('policy-test-table')
+                .shadowRoot
+                .querySelector('policy-test-row')
+                .shadowRoot
+                .querySelector('.namespace');
+        namespaceDropdown.value = 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa';
+        namespaceDropdown.dispatchEvent(new CustomEvent('change'));
+      )";
+  std::ignore = content::EvalJs(web_contents(), switchToExtensionNamespace);
+  content::EvalJsResult policy_names2 =
+      content::EvalJs(web_contents(), getPolicyNamesFromDatalist);
+  EXPECT_EQ(extension_policy_names, policy_names2);
+}
+
+IN_PROC_BROWSER_TEST_F(PolicyTestUITest, TestSchemaChangeReflected) {
+  if (!policy::utils::IsPolicyTestingEnabled(/*pref_service=*/nullptr,
+                                             chrome::GetChannel())) {
+    GTEST_SKIP() << "chrome://policy/test not allowed on this build.";
+  }
+
+  ASSERT_TRUE(content::NavigateToURL(web_contents(),
+                                     GURL(chrome::kChromeUIPolicyTestURL)));
+
+  base::Value schema = PolicyUI::GetSchema(GetProfile());
+  base::Value chrome_policy_names(base::Value::Type::LIST);
+  for (const auto [key, _] : *schema.GetDict().FindDict("chrome")) {
+    chrome_policy_names.GetList().Append(key);
+  }
+  base::Value extension_policy_names(base::Value::Type::LIST);
+  for (const auto [key, _] : *schema.GetDict().FindDict(kExtensionId)) {
+    extension_policy_names.GetList().Append(key);
+  }
+
+  // The namespace <select> should offer the Chrome policy domain, and the
+  // extension's namespace.
+  const std::string getNamespacesFromSelect =
+      R"(
+        Array.from(
+            document
+                .querySelector('policy-test-table')
+                .shadowRoot
+                .querySelector('policy-test-row')
+                .shadowRoot
+                .querySelector('.namespace')
+                .options)
+            .map(e => e.innerText)
+      )";
+  EXPECT_EQ(
+      base::Value(base::Value::List().Append("Chrome").Append(kExtensionId)),
+      content::EvalJs(web_contents(), getNamespacesFromSelect));
+
+  // Delete the extension from the schema. It should disappear from the
+  // namespace <select>.
+  auto* registry = GetProfile()->GetPolicySchemaRegistryService()->registry();
+  registry->UnregisterComponent(
+      policy::PolicyNamespace(policy::POLICY_DOMAIN_EXTENSIONS, kExtensionId));
+  EXPECT_EQ(base::Value(base::Value::List().Append("Chrome")),
+            content::EvalJs(web_contents(), getNamespacesFromSelect));
+}
+
 IN_PROC_BROWSER_TEST_F(PolicyTestUITest, TestPolicyNameChangesInputType) {
   if (!policy::utils::IsPolicyTestingEnabled(/*pref_service=*/nullptr,
                                              chrome::GetChannel())) {
