@@ -30,6 +30,7 @@
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/test/base/chrome_test_utils.h"
+#include "chrome/test/base/platform_browser_test.h"
 #include "components/policy/core/browser/browser_policy_connector.h"
 #include "components/policy/core/browser/webui/policy_status_provider.h"
 #include "components/policy/core/common/cloud/cloud_policy_refresh_scheduler.h"
@@ -61,12 +62,11 @@
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/install_verifier.h"
 #include "chrome/browser/extensions/test_extension_system.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/account_id/account_id.h"
 #include "extensions/common/extension_builder.h"
 #include "extensions/common/features/simple_feature.h"
-#else
-#include "chrome/browser/toolbar_manager_test_helper_android.h"
 #endif  // !BUILDFLAG(IS_ANDROID)
 
 using testing::_;
@@ -197,14 +197,7 @@ class PolicyUITest : public PlatformBrowserTest {
   testing::NiceMock<policy::MockConfigurationPolicyProvider> provider_;
 };
 
-PolicyUITest::PolicyUITest() {
-#if BUILDFLAG(IS_ANDROID)
-  // Skips recreating the Android activity when homepage settings are changed.
-  // This happens when the feature chrome::android::kStartSurfaceAndroid is
-  // enabled.
-  toolbar_manager::setSkipRecreateForTesting(true);
-#endif  // BUILDFLAG(IS_ANDROID)
-}
+PolicyUITest::PolicyUITest() = default;
 
 PolicyUITest::~PolicyUITest() = default;
 
@@ -297,19 +290,15 @@ class PolicyUIStatusTest : public MixinBasedInProcessBrowserTest {
   bool ReadStatusFor(const std::string& policy_legend,
                      base::flat_map<std::string, std::string>* policy_status);
   bool ReloadPolicies();
+  bool ReloadPolicies(content::WebContents* contents);
 
  protected:
   ash::DeviceStateMixin device_state_{
       &mixin_host_,
       ash::DeviceStateMixin::State::OOBE_COMPLETED_CLOUD_ENROLLED};
   ash::LoggedInUserMixin logged_in_user_mixin_{
-      &mixin_host_,
-      ash::LoggedInUserMixin::LogInType::kRegular,
-      embedded_test_server(),
-      this,
-      /*should_launch_browser=*/true,
-      AccountId::FromUserEmailGaiaId(policy::PolicyBuilder::kFakeUsername,
-                                     policy::PolicyBuilder::kFakeGaiaId)};
+      &mixin_host_, /*test_base=*/this, embedded_test_server(),
+      ash::LoggedInUserMixin::LogInType::kManaged};
 };
 
 bool PolicyUIStatusTest::ReadStatusFor(
@@ -366,6 +355,12 @@ bool PolicyUIStatusTest::ReadStatusFor(
 }
 
 bool PolicyUIStatusTest::ReloadPolicies() {
+  content::WebContents* contents =
+      chrome_test_utils::GetActiveWebContents(this);
+  return ReloadPolicies(contents);
+}
+
+bool PolicyUIStatusTest::ReloadPolicies(content::WebContents* contents) {
   const std::string javascript = R"JS(
     (function() {
       const reloadPoliciesBtn = document.getElementById('reload-policies');
@@ -385,11 +380,21 @@ bool PolicyUIStatusTest::ReloadPolicies() {
       }).then(waitForPoliciesToReload);
     })();
   )JS";
-  content::WebContents* contents =
-      chrome_test_utils::GetActiveWebContents(this);
   return content::ExecJs(contents, javascript);
 }
 
+#if !BUILDFLAG(IS_ANDROID)
+IN_PROC_BROWSER_TEST_F(PolicyUIStatusTest, CheckPolicyUiInGuestProfile) {
+  // Verifies that the page opens in guest session.
+  const Browser* policy_browser = OpenURLOffTheRecord(
+      browser()->profile(), GURL(chrome::kChromeUIPolicyURL));
+  ASSERT_TRUE(policy_browser);
+  content::WebContents* contents =
+      policy_browser->tab_strip_model()->GetActiveWebContents();
+  ASSERT_TRUE(ReloadPolicies(contents));
+}
+#endif  // !BUILDFLAG(IS_ANDROID)
+
 IN_PROC_BROWSER_TEST_F(PolicyUIStatusTest,
                        ShowsZeroSecondsSinceRefreshAfterReloadingPolicies) {
   // Verifies that the time since refresh of a policy set is set to 0 seconds
@@ -770,7 +775,7 @@ class ExtensionPolicyUITest : public PolicyUITest,
   }
 };
 
-// TODO(https://crbug.com/911661) Flaky time outs on Linux Chromium OS ASan
+// TODO(crbug.com/41429868) Flaky time outs on Linux Chromium OS ASan
 // LSan bot.
 #if defined(ADDRESS_SANITIZER)
 #define MAYBE_ExtensionLoadAndSendPolicy DISABLED_ExtensionLoadAndSendPolicy
