@@ -1,19 +1,18 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CHROME_BROWSER_UI_WEBUI_PRINT_PREVIEW_PRINT_PREVIEW_HANDLER_CHROMEOS_H_
 #define CHROME_BROWSER_UI_WEBUI_PRINT_PREVIEW_PRINT_PREVIEW_HANDLER_CHROMEOS_H_
 
-#include <memory>
 #include <string>
 
 #include "base/gtest_prod_util.h"
-#include "base/macros.h"
-#include "base/memory/ref_counted.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/weak_ptr.h"
+#include "base/values.h"
 #include "build/chromeos_buildflags.h"
-#include "chrome/browser/chromeos/printing/print_servers_manager.h"
+#include "chrome/browser/ash/printing/print_servers_manager.h"
 #include "chrome/common/buildflags.h"
 #include "chromeos/crosapi/mojom/local_printer.mojom.h"
 #include "components/prefs/pref_service.h"
@@ -24,8 +23,8 @@
 #include "printing/buildflags/buildflags.h"
 #include "printing/print_job_constants.h"
 
-namespace base {
-class DictionaryValue;
+namespace content {
+class WebContents;
 }
 
 namespace printing {
@@ -38,8 +37,10 @@ class PrinterHandler;
 class PrintPreviewHandler;
 
 // The handler for Javascript messages related to the print preview dialog.
-class PrintPreviewHandlerChromeOS : public content::WebUIMessageHandler,
-                                    public crosapi::mojom::PrintServerObserver {
+class PrintPreviewHandlerChromeOS
+    : public content::WebUIMessageHandler,
+      public crosapi::mojom::PrintServerObserver,
+      public crosapi::mojom::LocalPrintersObserver {
  public:
   PrintPreviewHandlerChromeOS();
   PrintPreviewHandlerChromeOS(const PrintPreviewHandlerChromeOS&) = delete;
@@ -61,7 +62,6 @@ class PrintPreviewHandlerChromeOS : public content::WebUIMessageHandler,
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   friend class TestPrintServersManager;
 #endif
-  class AccessTokenService;
 
   PrintPreviewHandler* GetPrintPreviewHandler();
 
@@ -69,20 +69,13 @@ class PrintPreviewHandlerChromeOS : public content::WebUIMessageHandler,
 
   // Grants an extension access to a provisional printer.  First element of
   // |args| is the provisional printer ID.
-  void HandleGrantExtensionPrinterAccess(const base::ListValue* args);
+  void HandleGrantExtensionPrinterAccess(const base::Value::List& args);
 
   // Performs printer setup. First element of |args| is the printer name.
-  void HandlePrinterSetup(const base::ListValue* args);
-
-  // Generates new token and sends back to UI.
-  void HandleGetAccessToken(const base::ListValue* args);
+  void HandlePrinterSetup(const base::Value::List& args);
 
   // Gets the EULA URL.
-  void HandleGetEulaUrl(const base::ListValue* args);
-
-  // Send OAuth2 access token.
-  void SendAccessToken(const std::string& callback_id,
-                       const std::string& access_token);
+  void HandleGetEulaUrl(const base::Value::List& args);
 
   // Send the EULA URL;
   void SendEulaUrl(const std::string& callback_id, const std::string& eula_url);
@@ -91,17 +84,20 @@ class PrintPreviewHandlerChromeOS : public content::WebUIMessageHandler,
   // printer capabilities.
   void SendPrinterSetup(const std::string& callback_id,
                         const std::string& printer_name,
-                        base::Value settings_info);
+                        base::Value::Dict settings_info);
 
   // Called when an extension reports information requested for a provisional
   // printer.
   // |callback_id|: The javascript callback to resolve or reject.
   // |printer_info|: The data reported by the extension.
   void OnGotExtensionPrinterInfo(const std::string& callback_id,
-                                 const base::DictionaryValue& printer_info);
+                                 const base::Value::Dict& printer_info);
 
   // Called to initiate a status request for a printer.
-  void HandleRequestPrinterStatusUpdate(const base::ListValue* args);
+  void HandleRequestPrinterStatusUpdate(const base::Value::List& args);
+  void HandleRequestPrinterStatusUpdateCompletion(
+      base::Value callback_id,
+      std::optional<base::Value::Dict> result);
 
   // crosapi::mojom::PrintServerObserver Implementation
   void OnPrintServersChanged(
@@ -110,22 +106,61 @@ class PrintPreviewHandlerChromeOS : public content::WebUIMessageHandler,
 
   // Loads printers corresponding to the print server(s).  First element of
   // |args| is the print server IDs.
-  void HandleChoosePrintServers(const base::ListValue* args);
+  void HandleChoosePrintServers(const base::Value::List& args);
 
   // Gets the list of print servers and fetching mode.
-  void HandleGetPrintServersConfig(const base::ListValue* args);
+  void HandleGetPrintServersConfig(const base::Value::List& args);
 
-  // Holds token service to get OAuth2 access tokens.
-  std::unique_ptr<AccessTokenService> token_service_;
+  // Records the `PrintPreview.PrintAttemptOutcome` histogram.
+  void HandleRecordPrintAttemptOutcome(const base::Value::List& args);
+
+  // Gets the WebContents that initiated print preview request using
+  // `PrintPreviewDialogController`.
+  content::WebContents* GetInitiator();
+
+  // Gets whether the UI should show the button to open printer settings. Button
+  // should be hidden if preview launched from the settings SWA.
+  void HandleGetShowManagePrinters(const base::Value::List& args);
+
+  void HandleObserveLocalPrinters(const base::Value::List& args);
+
+  // Callback for `HandleGetShowManagePrinters()`.
+  void OnHandleObserveLocalPrinters(
+      const std::string& callback_id,
+      std::vector<crosapi::mojom::LocalDestinationInfoPtr> printers);
+
+  // crosapi::mojom::LocalPrintersObserver Implementation:
+  void OnLocalPrintersUpdated(
+      std::vector<crosapi::mojom::LocalDestinationInfoPtr> printers) override;
+
+  void SetInitiatorForTesting(content::WebContents* test_initiator);
+
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+  int GetLocalPrinterVersionForTesting() { return local_printer_version_; }
+#endif
 
   mojo::Receiver<crosapi::mojom::PrintServerObserver> receiver_{this};
 
+  mojo::Receiver<crosapi::mojom::LocalPrintersObserver>
+      local_printers_receiver_{this};
+
+  // Used for testing, when `GetInitiator` called and `test_initiator` is set
+  // then it will be returned instead of calling `PrintPreviewDialogController`
+  // to find the initiator.
+  raw_ptr<content::WebContents> test_initiator_ = nullptr;
+
   // Used to transmit mojo interface method calls to ash chrome.
   // Null if the interface is unavailable.
   // Note that this is not propagated to LocalPrinterHandlerLacros.
   // The pointer is constant - if ash crashes and the mojo connection is lost,
   // lacros will automatically be restarted.
-  crosapi::mojom::LocalPrinter* local_printer_ = nullptr;
+  raw_ptr<crosapi::mojom::LocalPrinter, DanglingUntriaged> local_printer_ =
+      nullptr;
+
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+  // Version number of the LocalPrinter mojo service.
+  int local_printer_version_ = 0;
+#endif
 
   base::WeakPtrFactory<PrintPreviewHandlerChromeOS> weak_factory_{this};
 };
