@@ -19,6 +19,10 @@
 #include "printing/print_job_constants.h"
 #include "printing/print_settings.h"
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "chromeos/crosapi/mojom/extension_printer.mojom-shared.h"
+#endif
+
 namespace printing {
 
 namespace {
@@ -195,7 +199,7 @@ void RecordGetPrintersTimeHistogram(mojom::PrinterType printer_type,
       break;
     case mojom::PrinterType::kPrivetDeprecated:
     case mojom::PrinterType::kCloudDeprecated:
-      NOTREACHED_NORETURN();
+      NOTREACHED();
   }
   base::UmaHistogramCustomTimes(
       base::StrCat({"PrintPreview.GetPrintersTime.", printer_type_metric}),
@@ -204,4 +208,12 @@ void RecordGetPrintersTimeHistogram(mojom::PrinterType printer_type,
       /*max=*/base::Minutes(1), /*buckets=*/50);
 }
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+void ReportLacrosExtensionPrintJobStatusFromAshHistogram(
+    crosapi::mojom::StartPrintStatus status) {
+  base::UmaHistogramEnumeration("Printing.LacrosExtensions.FromAsh.Job.Result",
+                                status);
+}
+#endif
+
 }  // namespace printing
