@@ -19,6 +19,7 @@
 #include "chrome/browser/profiles/profile_test_util.h"
 #include "chrome/browser/profiles/profile_window.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_list_observer.h"
 #include "chrome/browser/ui/webui/profile_helper.h"
@@ -31,6 +32,10 @@
 #include "content/public/test/test_utils.h"
 #include "content/public/test/test_web_ui.h"
 
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+#include "chrome/browser/lacros/browser_test_util.h"
+#endif
+
 namespace {
 
 Profile* CreateProfile() {
@@ -109,13 +114,7 @@ class ProfileHelperTest : public InProcessBrowserTest {
   }
 };
 
-// TODO(crbug.com/1486054): Times out consistently on lacros asan builds.
-#if BUILDFLAG(IS_CHROMEOS_LACROS) && defined(ADDRESS_SANITIZER)
-#define MAYBE_OpenNewWindowForProfile DISABLED_OpenNewWindowForProfile
-#else
-#define MAYBE_OpenNewWindowForProfile OpenNewWindowForProfile
-#endif  // BUILDFLAG(IS_CHROMEOS_LACROS) && defined(ADDRESS_SANITIZER)
-IN_PROC_BROWSER_TEST_F(ProfileHelperTest, MAYBE_OpenNewWindowForProfile) {
+IN_PROC_BROWSER_TEST_F(ProfileHelperTest, OpenNewWindowForProfile) {
   BrowserList* browser_list = BrowserList::GetInstance();
 
   Browser* original_browser = browser();
@@ -131,14 +130,19 @@ IN_PROC_BROWSER_TEST_F(ProfileHelperTest, MAYBE_OpenNewWindowForProfile) {
   EXPECT_EQ(1u, browser_list->size());
   EXPECT_EQ(original_browser, browser_list->GetLastActive());
 
-  // Open additional browser will add new window and activates it.
+  // Opening additional browser will add new window and activate it.
   Profile* additional_profile = CreateProfile();
   activation_observer =
       std::make_unique<ExpectBrowserActivationForProfile>(additional_profile);
   webui::OpenNewWindowForProfile(additional_profile);
   EXPECT_EQ(2u, browser_list->size());
   activation_observer->Wait();
-  EXPECT_EQ(additional_profile, browser_list->GetLastActive()->profile());
+  Browser* additional_browser = browser_list->GetLastActive();
+  EXPECT_EQ(additional_profile, additional_browser->profile());
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+  // Await complete window creation to avoid interference with the next steps.
+  ASSERT_TRUE(browser_test_util::WaitForWindowCreation(additional_browser));
+#endif
 
 // On Macs OpenNewWindowForProfile does not activate existing browser
 // while non of the browser windows have focus. BrowserWindowCocoa::Show() got
@@ -208,8 +212,17 @@ IN_PROC_BROWSER_TEST_F(ProfileHelperTest, DeleteActiveProfile) {
   CloseBrowserSynchronously(original_browser);
   EXPECT_EQ(1u, browser_list->size());
   EXPECT_EQ(additional_profile, browser_list->get(0)->profile());
+  // ProfileManager will switch active profile upon observing
+  // BrowserListObserver::OnBrowserSetLastActive(). Wait until the event
+  // is observed if the active profile has not switched to `additional_profile`
+  // yet.
+  bool wait_for_set_last_active_observed =
+      ProfileManager::GetLastUsedProfileIfLoaded() != additional_profile;
+  ui_test_utils::WaitForBrowserSetLastActive(browser_list->get(0),
+                                             wait_for_set_last_active_observed);
+
   // Ensure the last active browser and the`LastUsedProfile` is set.
-  browser_list->get(0)->window()->Show();
+  EXPECT_EQ(chrome::FindLastActive(), browser_list->get(0));
   EXPECT_EQ(g_browser_process->profile_manager()->GetLastUsedProfileDir(),
             additional_profile->GetPath());
 
@@ -251,7 +264,7 @@ class ProfileHelperTestWithDestroyProfile
   base::test::ScopedFeatureList feature_list_;
 };
 
-// TODO(crbug.com/1504677): Fix this flaky test. Probably a timing issue.
+// TODO(crbug.com/40945232): Fix this flaky test. Probably a timing issue.
 IN_PROC_BROWSER_TEST_P(ProfileHelperTestWithDestroyProfile,
                        DISABLED_DeleteInactiveProfile) {
   content::TestWebUI web_ui;
