@@ -5,7 +5,20 @@
 #ifndef CHROME_BROWSER_UI_WEBUI_PROFILE_INTERNALS_PROFILE_INTERNALS_UI_H_
 #define CHROME_BROWSER_UI_WEBUI_PROFILE_INTERNALS_PROFILE_INTERNALS_UI_H_
 
+#include "chrome/common/webui_url_constants.h"
 #include "content/public/browser/web_ui_controller.h"
+#include "content/public/browser/webui_config.h"
+#include "content/public/common/url_constants.h"
+
+class ProfileInternalsUI;
+
+class ProfileInternalsUIConfig
+    : public content::DefaultWebUIConfig<ProfileInternalsUI> {
+ public:
+  ProfileInternalsUIConfig()
+      : DefaultWebUIConfig(content::kChromeUIScheme,
+                           chrome::kChromeUIProfileInternalsHost) {}
+};
 
 // Controller for chrome://profile-internals page.
 class ProfileInternalsUI : public content::WebUIController {
