@@ -6,6 +6,7 @@
 
 #include "base/functional/bind.h"
 #include "base/metrics/histogram_functions.h"
+#include "base/strings/string_number_conversions.h"
 #include "chrome/browser/privacy_sandbox/privacy_sandbox_settings_factory.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/hats/hats_service.h"
@@ -20,6 +21,7 @@
 #include "components/prefs/pref_service.h"
 #include "components/privacy_sandbox/privacy_sandbox_prefs.h"
 #include "components/privacy_sandbox/privacy_sandbox_settings.h"
+#include "components/safe_browsing/core/common/features.h"
 #include "components/version_info/version_info.h"
 #include "content/public/browser/visibility.h"
 #include "content/public/browser/web_contents.h"
@@ -221,7 +223,8 @@ SurveyStringData HatsHandler::GetSecurityPageProductSpecificStringData(
       {"Safe Browsing Setting Before Trigger", safe_browsing_setting_before},
       {"Safe Browsing Setting After Trigger", safe_browsing_setting_current},
       {"Client Channel", client_channel},
-      {"Time On Page", std::to_string(args[2].GetDouble())}};
+      {"Time On Page", base::NumberToString(args[2].GetDouble())},
+  };
 }
 
 void HatsHandler::HandleTrustSafetyInteractionOccurred(
@@ -329,16 +332,6 @@ void HatsHandler::RequestHatsSurvey(TrustSafetyInteraction interaction) {
       product_specific_bits_data = GetAdPrivacyProductSpecificBitsData(profile);
       break;
     }
-#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
-    case TrustSafetyInteraction::OPENED_GET_MOST_CHROME: {
-      trigger = kHatsSurveyTriggerGetMostChrome;
-      timeout_ms = features::kHappinessTrackingSurveysGetMostChromeTime.Get()
-                       .InMilliseconds();
-      navigation_behaviour =
-          HatsService::NavigationBehaviour::REQUIRE_SAME_DOCUMENT;
-      break;
-    }
-#endif
     case TrustSafetyInteraction::OPENED_PASSWORD_MANAGER:
       [[fallthrough]];
     case TrustSafetyInteraction::RAN_PASSWORD_CHECK: {
