@@ -99,6 +99,10 @@ void ClearBrowsingDataHandler::RegisterMessages() {
       "getSyncState",
       base::BindRepeating(&ClearBrowsingDataHandler::HandleGetSyncState,
                           base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "restartClearBrowsingDataCounters",
+      base::BindRepeating(&ClearBrowsingDataHandler::HandleRestartCounters,
+                          base::Unretained(this)));
 }
 
 void ClearBrowsingDataHandler::OnJavascriptAllowed() {
@@ -107,7 +111,9 @@ void ClearBrowsingDataHandler::OnJavascriptAllowed() {
 
   dse_service_observation_.Observe(
       TemplateURLServiceFactory::GetForProfile(profile_));
-  DCHECK(counters_.empty());
+
+  DCHECK(counters_basic_.empty());
+  DCHECK(counters_advanced_.empty());
   for (const std::string& pref : kCounterPrefsBasic) {
     AddCounter(BrowsingDataCounterFactory::GetForProfileAndPref(profile_, pref),
                browsing_data::ClearBrowsingDataTab::BASIC);
@@ -116,26 +122,14 @@ void ClearBrowsingDataHandler::OnJavascriptAllowed() {
     AddCounter(BrowsingDataCounterFactory::GetForProfileAndPref(profile_, pref),
                browsing_data::ClearBrowsingDataTab::ADVANCED);
   }
-  PrefService* prefs = profile_->GetPrefs();
-  period_ = std::make_unique<IntegerPrefMember>();
-  period_->Init(
-      browsing_data::prefs::kDeleteTimePeriod, prefs,
-      base::BindRepeating(&ClearBrowsingDataHandler::HandleTimePeriodChanged,
-                          base::Unretained(this)));
-  periodBasic_ = std::make_unique<IntegerPrefMember>();
-  periodBasic_->Init(
-      browsing_data::prefs::kDeleteTimePeriodBasic, prefs,
-      base::BindRepeating(&ClearBrowsingDataHandler::HandleTimePeriodChanged,
-                          base::Unretained(this)));
 }
 
 void ClearBrowsingDataHandler::OnJavascriptDisallowed() {
   dse_service_observation_.Reset();
   sync_service_observation_.Reset();
   weak_ptr_factory_.InvalidateWeakPtrs();
-  counters_.clear();
-  period_.reset();
-  periodBasic_.reset();
+  counters_basic_.clear();
+  counters_advanced_.clear();
 }
 
 void ClearBrowsingDataHandler::HandleClearBrowsingDataForTest() {
@@ -187,7 +181,7 @@ void ClearBrowsingDataHandler::HandleClearBrowsingData(
       case BrowsingDataType::CACHE:
         remove_mask |= content::BrowsingDataRemover::DATA_TYPE_CACHE;
         break;
-      case BrowsingDataType::COOKIES:
+      case BrowsingDataType::SITE_DATA:
         remove_mask |= chrome_browsing_data_remover::DATA_TYPE_SITE_DATA;
         origin_mask |=
             content::BrowsingDataRemover::ORIGIN_TYPE_UNPROTECTED_WEB;
@@ -207,12 +201,9 @@ void ClearBrowsingDataHandler::HandleClearBrowsingData(
         remove_mask |= chrome_browsing_data_remover::DATA_TYPE_SITE_DATA;
         origin_mask |= content::BrowsingDataRemover::ORIGIN_TYPE_PROTECTED_WEB;
         break;
-      case BrowsingDataType::BOOKMARKS:
-        // Only implemented on Android.
-        NOTREACHED();
-        break;
-      case BrowsingDataType::NUM_TYPES:
-        NOTREACHED();
+      case BrowsingDataType::TABS:
+        // Tab closure is not implemented yet.
+        NOTIMPLEMENTED();
         break;
     }
 
@@ -227,7 +218,7 @@ void ClearBrowsingDataHandler::HandleClearBrowsingData(
   // Record the deletion of cookies and cache.
   content::BrowsingDataRemover::CookieOrCacheDeletionChoice choice =
       content::BrowsingDataRemover::NEITHER_COOKIES_NOR_CACHE;
-  if (data_types.find(BrowsingDataType::COOKIES) != data_types.end()) {
+  if (data_types.find(BrowsingDataType::SITE_DATA) != data_types.end()) {
     choice = data_types.find(BrowsingDataType::CACHE) != data_types.end()
                  ? content::BrowsingDataRemover::BOTH_COOKIES_AND_CACHE
                  : content::BrowsingDataRemover::ONLY_COOKIES;
@@ -324,8 +315,18 @@ void ClearBrowsingDataHandler::HandleInitialize(const base::Value::List& args) {
   RefreshHistoryNotice();
 
   // Restart the counters each time the dialog is reopened.
-  for (const auto& counter : counters_)
-    counter->Restart();
+  //
+  // TODO(crbug.com/331925113): Since each Clear Browsing Data dialog execution
+  // commits the time selection to prefs, we may read it back from there.
+  // However, it would be safer if the "initializeClearBrowsingData" delivered
+  // the actual initial selection from the UI.
+  PrefService* prefs = profile_->GetPrefs();
+  auto initial_period_basic = static_cast<browsing_data::TimePeriod>(
+      prefs->GetInteger(browsing_data::prefs::kDeleteTimePeriodBasic));
+  auto initial_period_advanced = static_cast<browsing_data::TimePeriod>(
+      prefs->GetInteger(browsing_data::prefs::kDeleteTimePeriod));
+  RestartCounters(true /* basic */, initial_period_basic);
+  RestartCounters(false /* basic */, initial_period_advanced);
 
   ResolveJavascriptCallback(callback_id, base::Value() /* Promise<void> */);
 }
@@ -337,6 +338,14 @@ void ClearBrowsingDataHandler::HandleGetSyncState(
   ResolveJavascriptCallback(callback_id, CreateSyncStateEvent());
 }
 
+void ClearBrowsingDataHandler::HandleRestartCounters(
+    const base::Value::List& args) {
+  AllowJavascript();
+  CHECK_EQ(2U, args.size());
+  RestartCounters(args[0].GetBool() /* basic */,
+                  static_cast<browsing_data::TimePeriod>(args[1].GetInt()));
+}
+
 void ClearBrowsingDataHandler::OnStateChanged(syncer::SyncService* sync) {
   UpdateSyncState();
 }
@@ -406,11 +415,14 @@ void ClearBrowsingDataHandler::AddCounter(
     std::unique_ptr<browsing_data::BrowsingDataCounter> counter,
     browsing_data::ClearBrowsingDataTab tab) {
   DCHECK(counter);
-  counter->Init(
-      profile_->GetPrefs(), tab,
+  counter->InitWithoutPeriodPref(
+      profile_->GetPrefs(), tab, base::Time(),
       base::BindRepeating(&ClearBrowsingDataHandler::UpdateCounterText,
                           base::Unretained(this)));
-  counters_.push_back(std::move(counter));
+
+  ((tab == browsing_data::ClearBrowsingDataTab::BASIC) ? counters_basic_
+                                                       : counters_advanced_)
+      .push_back(std::move(counter));
 }
 
 void ClearBrowsingDataHandler::UpdateCounterText(
@@ -421,6 +433,15 @@ void ClearBrowsingDataHandler::UpdateCounterText(
           result.get(), profile_)));
 }
 
+void ClearBrowsingDataHandler::RestartCounters(
+    bool basic,
+    browsing_data::TimePeriod time_period) {
+  // Updating the begin time of a counter automatically forces a restart.
+  for (const auto& counter : (basic ? counters_basic_ : counters_advanced_)) {
+    counter->SetBeginTime(browsing_data::CalculateBeginDeleteTime(time_period));
+  }
+}
+
 void ClearBrowsingDataHandler::HandleTimePeriodChanged(
     const std::string& pref_name) {
   PrefService* prefs = profile_->GetPrefs();
