@@ -14,8 +14,11 @@
 #include "base/functional/callback_helpers.h"
 #include "base/memory/raw_ptr.h"
 #include "base/scoped_observation.h"
+#include "base/strings/stringprintf.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/metrics/user_action_tester.h"
+#include "base/test/task_environment.h"
+#include "base/time/time.h"
 #include "base/values.h"
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/consent_auditor/consent_auditor_factory.h"
@@ -34,9 +37,13 @@
 #include "chrome/test/base/testing_profile.h"
 #include "components/consent_auditor/fake_consent_auditor.h"
 #include "components/signin/public/base/avatar_icon_util.h"
+#include "components/signin/public/base/signin_switches.h"
+#include "components/signin/public/identity_manager/account_capabilities_test_mutator.h"
 #include "content/public/test/browser_task_environment.h"
 #include "content/public/test/test_web_ui.h"
 
+namespace {
+
 const int kExpectedProfileImageSize = 128;
 
 // The dialog needs to be initialized with a height but the actual value doesn't
@@ -76,11 +83,22 @@ class SyncConfirmationHandlerTest : public BrowserWithTestWindowTest,
   static const char kConsentText4[];
   static const char kConsentText5[];
 
+  static bool IsMinorModeEnabled() {
+#if BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_WIN)
+    return true;
+#else
+    return false;
+#endif
+  }
+
   SyncConfirmationHandlerTest()
-      : did_user_explicitly_interact_(false),
+      : BrowserWithTestWindowTest(
+            base::test::TaskEnvironment::TimeSource::MOCK_TIME),
+        did_user_explicitly_interact_(false),
         on_sync_confirmation_ui_closed_called_(false),
         sync_confirmation_ui_closed_result_(LoginUIService::ABORT_SYNC),
-        web_ui_(new content::TestWebUI) {}
+        web_ui_(new content::TestWebUI) {
+  }
 
   SyncConfirmationHandlerTest(const SyncConfirmationHandlerTest&) = delete;
   SyncConfirmationHandlerTest& operator=(const SyncConfirmationHandlerTest&) =
@@ -141,12 +159,11 @@ class SyncConfirmationHandlerTest : public BrowserWithTestWindowTest,
   }
 
   TestingProfile::TestingFactories GetTestingFactories() override {
-    TestingProfile::TestingFactories factories = {
-        {ConsentAuditorFactory::GetInstance(),
-         base::BindRepeating(&BuildFakeConsentAuditor)}};
-    IdentityTestEnvironmentProfileAdaptor::
-        AppendIdentityTestEnvironmentFactories(&factories);
-    return factories;
+    return IdentityTestEnvironmentProfileAdaptor::
+        GetIdentityTestEnvironmentFactoriesWithAppendedFactories(
+            {TestingProfile::TestingFactory{
+                ConsentAuditorFactory::GetInstance(),
+                base::BindRepeating(&BuildFakeConsentAuditor)}});
   }
 
   const std::unordered_map<std::string, int>& GetStringToGrdIdMap() {
@@ -195,6 +212,17 @@ class SyncConfirmationHandlerTest : public BrowserWithTestWindowTest,
     EXPECT_EQ(primary_account.IsManaged(), show_enterprise_badge.value());
   }
 
+  SyncConfirmationScreenMode GetScreenMode(
+      const content::TestWebUI::CallData& call_data) {
+    CHECK(call_data.arg1()->is_string())
+        << "arg1 should be string (callback name)";
+    CHECK(call_data.arg1()->GetString() == "screen-mode-changed")
+        << "Wrong callback name";
+
+    CHECK(call_data.arg2()->is_int()) << "arg2 should be int";
+    return static_cast<SyncConfirmationScreenMode>(call_data.arg2()->GetInt());
+  }
+
  protected:
   bool did_user_explicitly_interact_;
   bool on_sync_confirmation_ui_closed_called_;
@@ -202,6 +230,7 @@ class SyncConfirmationHandlerTest : public BrowserWithTestWindowTest,
       sync_confirmation_ui_closed_result_;
   // Holds information for the account currently logged in.
   AccountInfo account_info_;
+  base::HistogramTester histogram_tester_;
 
  private:
   std::unique_ptr<content::TestWebUI> web_ui_;
@@ -212,9 +241,9 @@ class SyncConfirmationHandlerTest : public BrowserWithTestWindowTest,
   std::unordered_map<std::string, int> string_to_grd_id_map_;
   base::ScopedObservation<LoginUIService, LoginUIService::Observer>
       login_ui_service_observation_{this};
-  base::HistogramTester histogram_tester_;
   std::unique_ptr<IdentityTestEnvironmentProfileAdaptor>
       identity_test_env_adaptor_;
+  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 const char SyncConfirmationHandlerTest::kConsentText1[] = "consentText1";
@@ -223,7 +252,7 @@ const char SyncConfirmationHandlerTest::kConsentText3[] = "consentText3";
 const char SyncConfirmationHandlerTest::kConsentText4[] = "consentText4";
 const char SyncConfirmationHandlerTest::kConsentText5[] = "consentText5";
 
-TEST_F(SyncConfirmationHandlerTest, TestSetAccountInfoIfPrimaryAccountReady) {
+TEST_F(SyncConfirmationHandlerTest, TestAvatarChangeWhenPrimaryAccountReady) {
   identity_test_env()->SimulateSuccessfulFetchOfAccountInfo(
       account_info_.account_id, account_info_.email, account_info_.gaia, "",
       "full_name", "given_name", "locale",
@@ -233,26 +262,102 @@ TEST_F(SyncConfirmationHandlerTest, TestSetAccountInfoIfPrimaryAccountReady) {
   args.Append(kDefaultDialogHeight);
   handler()->HandleInitializedWithSize(args);
 
-  ASSERT_EQ(1U, web_ui()->call_data().size());
+  ASSERT_GE(web_ui()->call_data().size(), 1U);
   ExpectAccountInfoChanged(*web_ui()->call_data()[0]);
+
+  if (IsMinorModeEnabled()) {
+    // When minor mode is effective, screen mode is only sent when the
+    // capability is available.
+    EXPECT_EQ(1U, web_ui()->call_data().size());
+  } else {
+    // Without experiment, expect defaulting to kUnrestricted
+    ASSERT_EQ(2U, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kUnrestricted,
+              GetScreenMode(*web_ui()->call_data()[1]));
+  }
+}
+
+TEST_F(SyncConfirmationHandlerTest, TestScreenModeChangedWhenCapabilityReady) {
+  // Both account info and capability are required to trigger SetAccountInfo.
+  AccountCapabilitiesTestMutator mutator(&account_info_.capabilities);
+  mutator.set_can_show_history_sync_opt_ins_without_minor_mode_restrictions(
+      false);
+  identity_test_env()->UpdateAccountInfoForAccount(account_info_);
+
+  base::Value::List args;
+  args.Append(kDefaultDialogHeight);
+  handler()->HandleInitializedWithSize(args);
+
+  if (IsMinorModeEnabled()) {
+    // In minor mode, capability was set to false, which means restricting.
+    ASSERT_EQ(1U, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kRestricted,
+              GetScreenMode(*web_ui()->call_data()[0]));
+  } else {
+    // Without experiment, expect defaulting to kUnrestricted
+    ASSERT_EQ(1U, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kUnrestricted,
+              GetScreenMode(*web_ui()->call_data()[0]));
+  }
+}
+
+TEST_F(SyncConfirmationHandlerTest, TestScreenModeChangeImmuneToAltering) {
+  // Both account info and capability are required to trigger SetAccountInfo.
+  AccountCapabilitiesTestMutator mutator(&account_info_.capabilities);
+  mutator.set_can_show_history_sync_opt_ins_without_minor_mode_restrictions(
+      false);
+  identity_test_env()->UpdateAccountInfoForAccount(account_info_);
+
+  base::Value::List args;
+  args.Append(kDefaultDialogHeight);
+  handler()->HandleInitializedWithSize(args);
+
+  if (IsMinorModeEnabled()) {
+    // In minor mode, capability was set to false, which means restricting.
+    ASSERT_EQ(1U, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kRestricted,
+              GetScreenMode(*web_ui()->call_data()[0]));
+  } else {
+    // Without experiment, expect defaulting to kUnrestricted
+    ASSERT_EQ(1U, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kUnrestricted,
+              GetScreenMode(*web_ui()->call_data()[0]));
+  }
+
+  // Now attempt flipping capability
+  mutator.set_can_show_history_sync_opt_ins_without_minor_mode_restrictions(
+      true);
+  identity_test_env()->UpdateAccountInfoForAccount(account_info_);
+
+  // The number of calls stays unchanged.
+  EXPECT_EQ(1U, web_ui()->call_data().size());
 }
 
 TEST_F(SyncConfirmationHandlerTest,
-       TestSetAccountInfoIfPrimaryAccountReadyLater) {
+       TestAvatarChangeWhenPrimaryAccountReadyLater) {
   base::Value::List args;
   args.Append(kDefaultDialogHeight);
   handler()->HandleInitializedWithSize(args);
 
-  // No callback called when there's no account image available.
-  ASSERT_EQ(0U, web_ui()->call_data().size());
+  // Tracks the number of calls which is variable due to minor-mode flag
+  // possibly enabled.
+  unsigned call_count = 0;
+
+  if (!IsMinorModeEnabled()) {
+    // The only callback here is defaulting screen mode to kUnrestricted.
+    ASSERT_EQ(++call_count, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kUnrestricted,
+              GetScreenMode(*web_ui()->call_data()[call_count - 1]));
+  }
 
   identity_test_env()->SimulateSuccessfulFetchOfAccountInfo(
       account_info_.account_id, account_info_.email, account_info_.gaia, "",
       "full_name", "given_name", "locale",
       "http://picture.example.com/picture.jpg");
 
-  ASSERT_EQ(1U, web_ui()->call_data().size());
-  ExpectAccountInfoChanged(*web_ui()->call_data()[0]);
+  // AccountInfo proper is being changed
+  ASSERT_EQ(++call_count, web_ui()->call_data().size());
+  ExpectAccountInfoChanged(*web_ui()->call_data()[call_count - 1]);
 }
 
 TEST_F(SyncConfirmationHandlerTest,
@@ -260,7 +365,17 @@ TEST_F(SyncConfirmationHandlerTest,
   base::Value::List args;
   args.Append(kDefaultDialogHeight);
   handler()->HandleInitializedWithSize(args);
-  EXPECT_EQ(0U, web_ui()->call_data().size());
+
+  // Tracks the number of calls which is variable due to minor-mode flag
+  // possibly enabled.
+  unsigned call_count = 0;
+
+  if (!IsMinorModeEnabled()) {
+    // The only callback here is defaulting screen mode to kUnrestricted.
+    ASSERT_EQ(++call_count, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kUnrestricted,
+              GetScreenMode(*web_ui()->call_data()[call_count - 1]));
+  }
 
   AccountInfo account_info =
       identity_test_env()->MakeAccountAvailable("bar@example.com");
@@ -269,9 +384,8 @@ TEST_F(SyncConfirmationHandlerTest,
       "bar_full_name", "bar_given_name", "bar_locale",
       "http://picture.example.com/bar_picture.jpg");
 
-  // Updating the account info of a secondary account should not update the
-  // image of the sync confirmation dialog.
-  EXPECT_EQ(0U, web_ui()->call_data().size());
+  // Account update was ignored so number of calls is unchanged.
+  ASSERT_EQ(call_count, web_ui()->call_data().size());
 
   identity_test_env()->SimulateSuccessfulFetchOfAccountInfo(
       account_info_.account_id, account_info_.email, account_info_.gaia, "",
@@ -280,11 +394,12 @@ TEST_F(SyncConfirmationHandlerTest,
 
   // Updating the account info of the primary account should update the
   // image of the sync confirmation dialog.
-  ASSERT_EQ(1U, web_ui()->call_data().size());
-  ExpectAccountInfoChanged(*web_ui()->call_data()[0]);
+  ASSERT_EQ(++call_count, web_ui()->call_data().size());
+  ExpectAccountInfoChanged(*web_ui()->call_data()[call_count - 1]);
 }
 
-TEST_F(SyncConfirmationHandlerTest, TestSetAccountInfoManaged) {
+TEST_F(SyncConfirmationHandlerTest,
+       TestAvatarChangeManagedWhenPrimaryAccountReady) {
   identity_test_env()->SimulateSuccessfulFetchOfAccountInfo(
       account_info_.account_id, account_info_.email, account_info_.gaia,
       "google.com", "full_name", "given_name", "locale",
@@ -294,12 +409,25 @@ TEST_F(SyncConfirmationHandlerTest, TestSetAccountInfoManaged) {
   args.Append(kDefaultDialogHeight);
   handler()->HandleInitializedWithSize(args);
 
-  ASSERT_EQ(1U, web_ui()->call_data().size());
+  ASSERT_GE(web_ui()->call_data().size(), 1U);
   ExpectAccountInfoChanged(*web_ui()->call_data()[0]);
+
+  if (IsMinorModeEnabled()) {
+    // When minor mode is effective, screen mode is only sent when the
+    // capability is available.
+    ASSERT_EQ(1U, web_ui()->call_data().size());
+  } else {
+    ASSERT_EQ(2U, web_ui()->call_data().size());
+    EXPECT_EQ(SyncConfirmationScreenMode::kUnrestricted,
+              GetScreenMode(*web_ui()->call_data()[1]));
+  }
 }
 
 TEST_F(SyncConfirmationHandlerTest, TestHandleUndo) {
-  handler()->HandleUndo(base::Value::List());
+  base::Value::List args;
+  args.Append(static_cast<int>(SyncConfirmationScreenMode::kRestricted));
+
+  handler()->HandleUndo(args);
   did_user_explicitly_interact_ = true;
 
   EXPECT_TRUE(on_sync_confirmation_ui_closed_called_);
@@ -325,6 +453,7 @@ TEST_F(SyncConfirmationHandlerTest, TestHandleConfirm) {
   base::Value::List args;
   args.Append(std::move(consent_description));
   args.Append(std::move(consent_confirmation));
+  args.Append(static_cast<int>(SyncConfirmationScreenMode::kRestricted));
 
   handler()->HandleConfirm(args);
   did_user_explicitly_interact_ = true;
@@ -363,6 +492,7 @@ TEST_F(SyncConfirmationHandlerTest, TestHandleConfirmWithAdvancedSyncSettings) {
   base::Value::List args;
   args.Append(std::move(consent_description));
   args.Append(std::move(consent_confirmation));
+  args.Append(static_cast<int>(SyncConfirmationScreenMode::kRestricted));
 
   handler()->HandleGoToSettings(args);
   did_user_explicitly_interact_ = true;
@@ -385,3 +515,128 @@ TEST_F(SyncConfirmationHandlerTest, TestHandleConfirmWithAdvancedSyncSettings) {
 
   EXPECT_EQ(account_info_.account_id, consent_auditor()->account_id());
 }
+
+TEST_F(SyncConfirmationHandlerTest, UserVisibleLatencyIsRecordedImmediately) {
+  if (!IsMinorModeEnabled()) {
+    GTEST_SKIP() << "Latency tracking is only implemented in minor mode.";
+  }
+
+  AccountCapabilitiesTestMutator mutator(&account_info_.capabilities);
+  mutator.set_can_show_history_sync_opt_ins_without_minor_mode_restrictions(
+      false);
+  identity_test_env()->UpdateAccountInfoForAccount(account_info_);
+
+  base::Value::List args;
+  args.Append(kDefaultDialogHeight);
+  handler()->HandleInitializedWithSize(args);
+
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              base::BucketsInclude(base::Bucket(/*min=*/0, /*count=*/1)));
+
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.FetchLatency"),
+              ::testing::IsEmpty());
+
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.ImmediatelyAvailable"),
+              base::BucketsInclude(base::Bucket(/*min=*/true, /*count=*/1)));
+}
+
+TEST_F(SyncConfirmationHandlerTest, UserVisibleLatencyIsRecordedLater) {
+  if (!IsMinorModeEnabled()) {
+    GTEST_SKIP() << "Latency tracking is only implemented in minor mode.";
+  }
+
+  base::Value::List args;
+  args.Append(kDefaultDialogHeight);
+  handler()->HandleInitializedWithSize(args);
+
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.ImmediatelyAvailable"),
+              base::BucketsInclude(base::Bucket(/*min=*/false, /*count=*/1)));
+
+  // Latencies are yet to be recorded.
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              ::testing::IsEmpty());
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.FetchLatency"),
+              ::testing::IsEmpty());
+
+  AccountCapabilitiesTestMutator mutator(&account_info_.capabilities);
+  mutator.set_can_show_history_sync_opt_ins_without_minor_mode_restrictions(
+      false);
+  identity_test_env()->UpdateAccountInfoForAccount(account_info_);
+
+  // Latency is finally recorded but let's not assert any specific value to
+  // avoid flakiness.
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              ::testing::SizeIs(1));
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.FetchLatency"),
+              ::testing::SizeIs(1));
+}
+
+TEST_F(SyncConfirmationHandlerTest, UserVisibleLatencyIsNotRecordedTwice) {
+  if (!IsMinorModeEnabled()) {
+    GTEST_SKIP() << "Latency tracking is only implemented in minor mode.";
+  }
+
+  base::Value::List args;
+  args.Append(kDefaultDialogHeight);
+  handler()->HandleInitializedWithSize(args);
+
+  // Verify how many times latency was recorded by looking at UserVisibleLatency
+  // only.
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              ::testing::IsEmpty());
+
+  AccountCapabilitiesTestMutator mutator(&account_info_.capabilities);
+  mutator.set_can_show_history_sync_opt_ins_without_minor_mode_restrictions(
+      false);
+  identity_test_env()->UpdateAccountInfoForAccount(account_info_);
+
+  // After update latency is recorded.
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              ::testing::SizeIs(1));
+
+  // This triggers OnExtendedAccountInfoUpdated again but this time should not
+  // record any latency.
+  identity_test_env()->SimulateSuccessfulFetchOfAccountInfo(
+      account_info_.account_id, account_info_.email, account_info_.gaia, "",
+      "full_name", "given_name", "locale",
+      "http://picture.example.com/picture.jpg");
+
+  // So assert that sample count is unchanged.
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              ::testing::SizeIs(1));
+}
+
+TEST_F(SyncConfirmationHandlerTest, UserVisibleLatencyIsRecordedPastDeadline) {
+  if (!IsMinorModeEnabled()) {
+    GTEST_SKIP() << "Latency tracking is only implemented in minor mode.";
+  }
+
+  base::Value::List args;
+  args.Append(kDefaultDialogHeight);
+  handler()->HandleInitializedWithSize(args);
+
+  // Advance clock by amount of time larger than any sane fetch timeout.
+  task_environment()->FastForwardBy(base::Minutes(1));
+
+  // Even though capability was not received, latency is recorded, because the
+  // minor-safe behaviour was imposed by reaching the fetch deadline.
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.UserVisibleLatency"),
+              ::testing::SizeIs(1));
+  EXPECT_THAT(histogram_tester_.GetAllSamples(
+                  "Signin.AccountCapabilities.FetchLatency"),
+              ::testing::SizeIs(1));
+}
+
+}  // namespace
