@@ -2,13 +2,23 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/ui/webui/sync_internals/sync_internals_ui.h"
 
 #include <memory>
 
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/ui/webui/sync_internals/sync_internals_message_handler.h"
+#include "chrome/browser/signin/identity_manager_factory.h"
+#include "chrome/browser/sync/sync_invalidations_service_factory.h"
+#include "chrome/browser/sync/sync_service_factory.h"
+#include "chrome/browser/sync/user_event_service_factory.h"
+#include "chrome/browser/ui/webui/sync_internals/chrome_sync_internals_message_handler.h"
 #include "chrome/browser/ui/webui/webui_util.h"
+#include "chrome/common/channel_info.h"
 #include "chrome/common/url_constants.h"
 #include "components/grit/sync_service_sync_internals_resources.h"
 #include "components/grit/sync_service_sync_internals_resources_map.h"
@@ -44,7 +54,13 @@ SyncInternalsUI::SyncInternalsUI(content::WebUI* web_ui)
     : WebUIController(web_ui) {
   CreateAndAddSyncInternalsHTMLSource(Profile::FromWebUI(web_ui));
 
-  web_ui->AddMessageHandler(std::make_unique<SyncInternalsMessageHandler>());
+  auto* profile = Profile::FromWebUI(web_ui)->GetOriginalProfile();
+  web_ui->AddMessageHandler(std::make_unique<ChromeSyncInternalsMessageHandler>(
+      IdentityManagerFactory::GetForProfile(profile),
+      SyncServiceFactory::GetForProfile(profile),
+      SyncInvalidationsServiceFactory::GetForProfile(profile),
+      browser_sync::UserEventServiceFactory::GetForProfile(profile),
+      chrome::GetChannelName(chrome::WithExtendedStable(true))));
 }
 
 SyncInternalsUI::~SyncInternalsUI() = default;
