@@ -11,9 +11,11 @@ namespace chromeos {
 // type.
 extern const char kInvalidChromeOSSystemExtensionDeclaration[];
 // Error message returned when a chromeos_system_extension's
-// externally_connectable key contains other than one origin.
+// externally_connectable key contains other than allowed origin when IWA is
+// supported.
 extern const char kInvalidExternallyConnectableDeclaration[];
-
+// Error message returned when extension id is not allowed.
+extern const char kInvalidChromeOSSystemExtensionId[];
 }  // namespace chromeos
 
 #endif  // CHROME_COMMON_CHROMEOS_EXTENSIONS_CHROMEOS_SYSTEM_EXTENSIONS_MANIFEST_CONSTANTS_H_
