@@ -246,7 +246,7 @@ class USBDevicesFormatter : public ChromePermissionMessageFormatter {
             IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE_UNKNOWN_VENDOR);
         break;
       default:
-        NOTREACHED();
+        NOTREACHED_IN_MIGRATION();
     }
     return PermissionMessage(msg, permissions);
   }
@@ -754,6 +754,9 @@ ChromePermissionMessageRule::GetAllRules() {
       {IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_DIAGNOSTICS,
        {APIPermissionID::kChromeOSDiagnostics},
        {}},
+      {IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_DIAGNOSTICS_NETWORK_INFO_FOR_MLAB,
+       {APIPermissionID::kChromeOSDiagnosticsNetworkInfoForMlab},
+       {}},
       {IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_EVENTS,
        {APIPermissionID::kChromeOSEvents},
        {}},
