@@ -2,8 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/common/net/x509_certificate_model.h"
 
+#include <string_view>
+
 #include "net/cert/x509_util.h"
 #include "net/test/cert_builder.h"
 #include "net/test/cert_test_util.h"
@@ -21,7 +28,7 @@ namespace {
 
 std::optional<std::string> FindExtension(
     const std::vector<x509_certificate_model::Extension>& extensions,
-    base::StringPiece name) {
+    std::string_view name) {
   for (const auto& extension : extensions) {
     if (extension.name == name) {
       return extension.value;
@@ -730,7 +737,7 @@ INSTANTIATE_TEST_SUITE_P(All,
                          testing::Values(std::string(),
                                          std::string("nickname")));
 
-// TODO(https://crbug.com/953425): This test suite has "2" at the end of the
+// TODO(crbug.com/41453265): This test suite has "2" at the end of the
 // name to avoid conflicting with x509_certificate_model_nss_unittest. Should
 // rename the test suite in that file to X509CertificateModelNSSTest and remove
 // the 2 from here.
