@@ -2,16 +2,18 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "components/autofill/content/renderer/password_generation_agent.h"
+
 #include <string.h>
 
 #include <memory>
 #include <string>
+#include <string_view>
 
 #include "base/command_line.h"
 #include "base/functional/bind.h"
 #include "base/ranges/algorithm.h"
 #include "base/run_loop.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/metrics/histogram_tester.h"
@@ -22,7 +24,6 @@
 #include "chrome/test/base/chrome_render_view_test.h"
 #include "components/autofill/content/renderer/autofill_agent.h"
 #include "components/autofill/content/renderer/form_autofill_util.h"
-#include "components/autofill/content/renderer/password_generation_agent.h"
 #include "components/autofill/content/renderer/test_password_autofill_agent.h"
 #include "components/autofill/core/common/autofill_switches.h"
 #include "components/autofill/core/common/form_data.h"
@@ -39,6 +40,7 @@
 #include "third_party/blink/public/web/web_document.h"
 #include "third_party/blink/public/web/web_element.h"
 #include "third_party/blink/public/web/web_frame_widget.h"
+#include "third_party/blink/public/web/web_input_element.h"
 #include "third_party/blink/public/web/web_local_frame.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 
@@ -56,15 +58,14 @@ using testing::AtMost;
 using testing::Eq;
 
 namespace autofill {
-
 namespace {
 
 // Utility method that tries to find a field in `form` whose `id_attribute`
 // matches `id`. Returns nullptr if no such field exists.
-const FormFieldData* FindFieldById(const FormData& form, base::StringPiece id) {
-  auto it = base::ranges::find(form.fields, base::UTF8ToUTF16(id),
+const FormFieldData* FindFieldById(const FormData& form, std::string_view id) {
+  auto it = base::ranges::find(form.fields(), base::UTF8ToUTF16(id),
                                &FormFieldData::id_attribute);
-  return it != form.fields.end() ? &*it : nullptr;
+  return it != form.fields().end() ? &*it : nullptr;
 }
 
 class FakeContentAutofillDriver : public mojom::AutofillDriver {
@@ -84,9 +85,6 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
 
  private:
   // mojom::AutofillDriver:
-  void SetFormToBeProbablySubmitted(
-      const std::optional<FormData>& form) override {}
-
   void FormsSeen(const std::vector<FormData>& updated_forms,
                  const std::vector<FormRendererId>& removed_forms) override {
     forms_seen_run_loop_->Quit();
@@ -96,37 +94,37 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
                      bool known_success,
                      mojom::SubmissionSource source) override {}
 
+  void CaretMovedInFormField(const FormData& form,
+                             FieldRendererId field_id,
+                             const gfx::Rect& caret_bounds) override {}
+
   void TextFieldDidChange(const FormData& form,
-                          const FormFieldData& field,
-                          const gfx::RectF& bounding_box,
+                          FieldRendererId field_id,
                           base::TimeTicks timestamp) override {}
 
   void TextFieldDidScroll(const FormData& form,
-                          const FormFieldData& field,
-                          const gfx::RectF& bounding_box) override {}
+                          FieldRendererId field_id) override {}
 
   void SelectControlDidChange(const FormData& form,
-                              const FormFieldData& field,
-                              const gfx::RectF& bounding_box) override {}
+                              FieldRendererId field_id) override {}
 
-  void JavaScriptChangedAutofilledValue(
-      const FormData& form,
-      const FormFieldData& field,
-      const std::u16string& old_value) override {}
+  void JavaScriptChangedAutofilledValue(const FormData& form,
+                                        FieldRendererId field_id,
+                                        const std::u16string& old_value,
+                                        bool formatting_only) override {}
 
   void AskForValuesToFill(
       const FormData& form,
-      const FormFieldData& field,
-      const gfx::RectF& bounding_box,
+      FieldRendererId field_id,
+      const gfx::Rect& caret_bounds,
       AutofillSuggestionTriggerSource trigger_source) override {}
 
   void HidePopup() override {}
 
-  void FocusNoLongerOnForm(bool had_interacted_form) override {}
+  void FocusOnNonFormField() override {}
 
   void FocusOnFormField(const FormData& form,
-                        const FormFieldData& field,
-                        const gfx::RectF& bounding_box) override {}
+                        FieldRendererId field_id) override {}
 
   void DidFillAutofillFormData(const FormData& form,
                                base::TimeTicks timestamp) override {}
@@ -142,8 +140,6 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
   mojo::AssociatedReceiverSet<mojom::AutofillDriver> receivers_;
 };
 
-}  // namespace
-
 constexpr char kSigninFormHTML[] =
     "<FORM name = 'blah' action = 'http://www.random.com/'> "
     "  <INPUT type = 'text' id = 'username'/> "
@@ -259,7 +255,8 @@ class PasswordGenerationAgentTest : public ChromeRenderViewTest {
   void TearDown() override;
 
   void LoadHTMLWithUserGesture(const char* html);
-  WebElement GetElementById(base::StringPiece element_id);
+  WebElement GetElementById(std::string_view element_id);
+  WebInputElement GetInputElementById(std::string_view element_id);
   void FocusField(const char* element_id);
 
   void ExpectAutomaticGenerationAvailable(const char* element_id,
@@ -270,8 +267,10 @@ class PasswordGenerationAgentTest : public ChromeRenderViewTest {
       const std::u16string& expected_generation_element);
   void SelectGenerationFallbackAndExpect(bool available);
   void ExpectAttribute(const WebElement& element,
-                       base::StringPiece attribute,
-                       base::StringPiece expected_value);
+                       std::string_view attribute,
+                       std::string_view expected_value);
+  void CheckPreviewedValue(const WebInputElement& element,
+                           const std::u16string& value);
 
   void BindAutofillDriver(mojo::ScopedInterfaceEndpointHandle handle);
   void BindPasswordManagerDriver(mojo::ScopedInterfaceEndpointHandle handle);
@@ -311,8 +310,8 @@ void PasswordGenerationAgentTest::RegisterMainFrameRemoteInterfaces() {
 void PasswordGenerationAgentTest::SetUp() {
   ChromeRenderViewTest::SetUp();
 
-  // TODO(crbug/862989): Remove workaround preventing non-test classes to bind
-  // fake_driver_ or fake_pw_client_.
+  // TODO(crbug.com/41401202): Remove workaround preventing non-test classes to
+  // bind fake_driver_ or fake_pw_client_.
   password_autofill_agent_->GetPasswordManagerDriver();
   password_generation_->RequestPasswordManagerClientForTesting();
   base::RunLoop().RunUntilIdle();  // Executes binding the interfaces.
@@ -352,18 +351,26 @@ void PasswordGenerationAgentTest::LoadHTMLWithUserGesture(const char* html) {
 }
 
 WebElement PasswordGenerationAgentTest::GetElementById(
-    base::StringPiece element_id) {
+    std::string_view element_id) {
   WebDocument document = GetMainFrame()->GetDocument();
   WebElement element =
       document.GetElementById(blink::WebString::FromUTF8(element_id));
-  CHECK(!element.IsNull());
+  CHECK(element);
   return element;
 }
 
+WebInputElement PasswordGenerationAgentTest::GetInputElementById(
+    std::string_view element_id) {
+  WebInputElement input_element =
+      GetElementById(element_id).To<WebInputElement>();
+  CHECK(input_element);
+  return input_element;
+}
+
 void PasswordGenerationAgentTest::FocusField(const char* element_id) {
   SimulateElementClick(element_id);
 #if BUILDFLAG(IS_ANDROID)
-  // TODO(crbug.com/1293802): On Android, the JS above doesn't trigger the
+  // TODO(crbug.com/40820173): On Android, the JS above doesn't trigger the
   // method below.
   GetMainFrame()->AutofillClient()->DidCompleteFocusChangeInFrame();
 #endif  // BUILDFLAG(IS_ANDROID)
@@ -377,7 +384,7 @@ void PasswordGenerationAgentTest::ExpectAutomaticGenerationAvailable(
   if (status == kNotReported) {
     EXPECT_CALL(fake_pw_client_, AutomaticGenerationAvailable(_)).Times(0);
   } else {
-    // TODO(crbug.com/1473553): Expect the call precisely once.
+    // TODO(crbug.com/40279043): Expect the call precisely once.
     EXPECT_CALL(fake_pw_client_, AutomaticGenerationAvailable(_))
         .Times(testing::AtLeast(1));
   }
@@ -433,14 +440,21 @@ void PasswordGenerationAgentTest::SelectGenerationFallbackAndExpect(
 
 void PasswordGenerationAgentTest::ExpectAttribute(
     const WebElement& element,
-    base::StringPiece attribute,
-    base::StringPiece expected_value) {
+    std::string_view attribute,
+    std::string_view expected_value) {
   WebString actual_value =
       element.GetAttribute(blink::WebString::FromUTF8(attribute));
   ASSERT_FALSE(actual_value.IsNull());
   EXPECT_EQ(expected_value, actual_value.Ascii());
 }
 
+void PasswordGenerationAgentTest::CheckPreviewedValue(
+    const WebInputElement& element,
+    const std::u16string& value) {
+  EXPECT_EQ(value, element.SuggestedValue().Utf16());
+  EXPECT_EQ(element.GetAutofillState(), blink::WebAutofillState::kPreviewed);
+}
+
 void PasswordGenerationAgentTest::BindAutofillDriver(
     mojo::ScopedInterfaceEndpointHandle handle) {
   fake_autofill_driver_.BindReceiver(
@@ -557,13 +571,10 @@ TEST_F(PasswordGenerationAgentTest, FillTest) {
                                     /*confirm_password_id=*/"second_password");
   ExpectAutomaticGenerationAvailable("first_password", kAvailable);
 
-  WebElement element =
-      document.GetElementById(WebString::FromUTF8("first_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement first_password_element = element.To<WebInputElement>();
-  element = document.GetElementById(WebString::FromUTF8("second_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement second_password_element = element.To<WebInputElement>();
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
+  WebInputElement second_password_element =
+      GetInputElementById("second_password");
 
   // Both password fields should be empty.
   EXPECT_TRUE(first_password_element.Value().IsNull());
@@ -589,8 +600,7 @@ TEST_F(PasswordGenerationAgentTest, FillTest) {
   }
 
   // Check that focus returns to previously focused element.
-  element = document.GetElementById(WebString::FromUTF8("address"));
-  ASSERT_FALSE(element.IsNull());
+  WebInputElement element = GetInputElementById("address");
   EXPECT_EQ(element, document.FocusedElement());
 }
 
@@ -602,14 +612,10 @@ TEST_F(PasswordGenerationAgentTest, EditingTest) {
                                     /*confirm_password_id=*/"second_password");
   ExpectAutomaticGenerationAvailable("first_password", kAvailable);
 
-  WebDocument document = GetMainFrame()->GetDocument();
-  WebElement element =
-      document.GetElementById(WebString::FromUTF8("first_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement first_password_element = element.To<WebInputElement>();
-  element = document.GetElementById(WebString::FromUTF8("second_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement second_password_element = element.To<WebInputElement>();
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
+  WebInputElement second_password_element =
+      GetInputElementById("second_password");
 
   std::u16string password = u"random_password";
   EXPECT_CALL(fake_pw_client_, PresaveGeneratedPassword(_, Eq(password)));
@@ -623,14 +629,19 @@ TEST_F(PasswordGenerationAgentTest, EditingTest) {
   EXPECT_EQ(password, second_password_element.Value().Utf16());
 
   // After editing the first field they are still the same.
-  std::string edited_password_ascii = "edited_password";
-  std::u16string edited_password = base::ASCIIToUTF16(edited_password_ascii);
+  std::u16string appended_chars = u"123";
+  std::u16string edited_password = password + appended_chars;
   EXPECT_CALL(fake_pw_client_,
               PresaveGeneratedPassword(_, Eq(edited_password)));
   EXPECT_CALL(fake_pw_client_, ShowPasswordEditingPopup).Times(AtLeast(1));
   FocusField("first_password");
-  SimulateUserInputChangeForElement(&first_password_element,
-                                    edited_password_ascii);
+  SimulateUserTypingASCIICharacter(ui::VKEY_END, /*flush_message_loop=*/false);
+  for (size_t i = 0; i < appended_chars.size(); i++) {
+    SimulateUserTypingASCIICharacter(appended_chars[i],
+                                     /*flush_message_loop=*/false);
+  }
+  base::RunLoop().RunUntilIdle();
+
   EXPECT_EQ(edited_password, first_password_element.Value().Utf16());
   EXPECT_EQ(edited_password, second_password_element.Value().Utf16());
   EXPECT_TRUE(first_password_element.IsAutofilled());
@@ -638,10 +649,10 @@ TEST_F(PasswordGenerationAgentTest, EditingTest) {
   ASSERT_TRUE(fake_driver_.form_data_maybe_submitted().has_value());
   EXPECT_THAT(FindFieldById(*fake_driver_.form_data_maybe_submitted(),
                             "first_password"),
-              testing::Field(&FormFieldData::value, edited_password));
+              testing::Property(&FormFieldData::value, edited_password));
   EXPECT_THAT(FindFieldById(*fake_driver_.form_data_maybe_submitted(),
                             "second_password"),
-              testing::Field(&FormFieldData::value, edited_password));
+              testing::Property(&FormFieldData::value, edited_password));
   base::RunLoop().RunUntilIdle();
   testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
 
@@ -649,18 +660,20 @@ TEST_F(PasswordGenerationAgentTest, EditingTest) {
   // is deleted.
   EXPECT_CALL(fake_pw_client_, PasswordNoLongerGenerated(_));
   EXPECT_CALL(fake_pw_client_, AutomaticGenerationAvailable(_));
-  SimulateUserInputChangeForElement(&first_password_element, std::string());
+  SimulateUserInputChangeForElement(first_password_element, std::string());
   EXPECT_EQ(std::u16string(), first_password_element.Value().Utf16());
   EXPECT_EQ(std::u16string(), second_password_element.Value().Utf16());
   EXPECT_FALSE(first_password_element.IsAutofilled());
   EXPECT_FALSE(second_password_element.IsAutofilled());
+  EXPECT_FALSE(first_password_element.ShouldRevealPassword());
+  EXPECT_FALSE(second_password_element.ShouldRevealPassword());
   ASSERT_TRUE(fake_driver_.form_data_maybe_submitted().has_value());
   EXPECT_THAT(FindFieldById(*fake_driver_.form_data_maybe_submitted(),
                             "first_password"),
-              testing::Field(&FormFieldData::value, std::u16string()));
+              testing::Property(&FormFieldData::value, std::u16string()));
   EXPECT_THAT(FindFieldById(*fake_driver_.form_data_maybe_submitted(),
                             "second_password"),
-              testing::Field(&FormFieldData::value, std::u16string()));
+              testing::Property(&FormFieldData::value, std::u16string()));
 }
 
 TEST_F(PasswordGenerationAgentTest, EditingEventsTest) {
@@ -743,11 +756,8 @@ TEST_F(PasswordGenerationAgentTest, MaximumCharsForGenerationOffer) {
   // There should now be a message to show the UI.
   ExpectAutomaticGenerationAvailable("first_password", kAvailable);
 
-  WebDocument document = GetMainFrame()->GetDocument();
-  WebElement element =
-      document.GetElementById(WebString::FromUTF8("first_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement first_password_element = element.To<WebInputElement>();
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
 
   // Make a password just under maximum offer size.
   // Due to implementation details it's OK to get one more trigger for the
@@ -755,7 +765,7 @@ TEST_F(PasswordGenerationAgentTest, MaximumCharsForGenerationOffer) {
   EXPECT_CALL(fake_pw_client_, AutomaticGenerationAvailable(_))
       .Times(AtMost(1));
   SimulateUserInputChangeForElement(
-      &first_password_element,
+      first_password_element,
       std::string(PasswordGenerationAgent::kMaximumCharsForGenerationOffer,
                   'a'));
   testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
@@ -835,14 +845,10 @@ TEST_F(PasswordGenerationAgentTest, MinimumLengthForEditedPassword) {
 
   // The first password field is still non empty. The second one should be
   // cleared.
-  WebDocument document = GetMainFrame()->GetDocument();
-  WebElement element =
-      document.GetElementById(WebString::FromUTF8("first_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement first_password_element = element.To<WebInputElement>();
-  element = document.GetElementById(WebString::FromUTF8("second_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement second_password_element = element.To<WebInputElement>();
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
+  WebInputElement second_password_element =
+      GetInputElementById("second_password");
   EXPECT_NE(std::u16string(), first_password_element.Value().Utf16());
   EXPECT_EQ(std::u16string(), second_password_element.Value().Utf16());
 }
@@ -921,8 +927,7 @@ TEST_F(PasswordGenerationAgentTest, ChangePasswordFormDetectionTest) {
 #if !BUILDFLAG(IS_ANDROID)
 TEST_F(PasswordGenerationAgentTest, DesktopContextMenuGenerationInFormTest) {
   LoadHTMLWithUserGesture(kSigninFormHTML);
-  WebElement element = GetElementById("password");
-  WebInputElement first_password_element = element.To<WebInputElement>();
+  WebInputElement first_password_element = GetInputElementById("password");
   EXPECT_TRUE(first_password_element.Value().IsNull());
   SimulateElementRightClick("password");
   SelectGenerationFallbackAndExpect(true);
@@ -965,7 +970,7 @@ TEST_F(PasswordGenerationAgentTest, DesktopContextMenuGenerationNoIds) {
       "document.getElementsByClassName('first_password')[0].focus();");
   WebInputElement first_password_element =
       document.FocusedElement().To<WebInputElement>();
-  ASSERT_FALSE(first_password_element.IsNull());
+  ASSERT_TRUE(first_password_element);
   SelectGenerationFallbackAndExpect(true);
 
   // Simulate that the user accepts a generated password.
@@ -984,7 +989,7 @@ TEST_F(PasswordGenerationAgentTest, DesktopContextMenuGenerationNoIds) {
       "document.getElementsByClassName('second_password')[0].focus();");
   WebInputElement second_password_element =
       document.FocusedElement().To<WebInputElement>();
-  ASSERT_FALSE(second_password_element.IsNull());
+  ASSERT_TRUE(second_password_element);
   EXPECT_EQ(password, second_password_element.Value().Utf16());
   EXPECT_TRUE(second_password_element.IsAutofilled());
 }
@@ -1014,6 +1019,32 @@ TEST_F(PasswordGenerationAgentTest,
   SelectGenerationFallbackAndExpect(true);
 }
 
+// Tests that manual generation can be triggered on a text field, if heuristics
+// signal that the field is a password field.
+TEST_F(PasswordGenerationAgentTest,
+       ManualGenerationIsTriggeredOnTextFieldWithHeuristic) {
+  // The field type is text, but has password mention in the id/name attribute.
+  LoadHTML(
+      R"(
+      <input type="text" id="username-field" name="username-field">
+      <input type="text" id="password-field" name="password-field">
+    )");
+
+  SimulateElementRightClick("password-field");
+  SelectGenerationFallbackAndExpect(true);
+
+  // Simulate that the user accepts a generated password.
+  std::u16string password = u"random_password";
+  EXPECT_CALL(fake_pw_client_, PresaveGeneratedPassword(_, Eq(password)));
+  password_generation_->GeneratedPasswordAccepted(password);
+
+  WebInputElement password_element = GetInputElementById("password-field");
+  // Check that the password field is autofilled with the generated
+  // password.
+  EXPECT_EQ(password, password_element.Value().Utf16());
+  EXPECT_TRUE(password_element.IsAutofilled());
+}
+
 #endif  // !BUILDFLAG(IS_ANDROID)
 
 TEST_F(PasswordGenerationAgentTest, PresavingGeneratedPassword) {
@@ -1129,10 +1160,7 @@ TEST_F(PasswordGenerationAgentTest, RevealPassword) {
     FocusField(kGenerationElementId);
     fake_pw_client_.Flush();
 
-    WebDocument document = GetMainFrame()->GetDocument();
-    WebElement element = GetElementById(kGenerationElementId);
-    ASSERT_FALSE(element.IsNull());
-    blink::WebInputElement input = element.To<WebInputElement>();
+    WebInputElement input = GetInputElementById(kGenerationElementId);
     EXPECT_TRUE(input.ShouldRevealPassword());
 
     // Click on another HTML element.
@@ -1235,10 +1263,9 @@ TEST_F(PasswordGenerationAgentTest, AutofillToGenerationField) {
   ExpectAutomaticGenerationAvailable("first_password", kAvailable);
 
   WebDocument document = GetMainFrame()->GetDocument();
-  WebElement element = GetElementById("first_password");
-  const WebInputElement input_element = element.To<WebInputElement>();
+  const WebInputElement input_element = GetInputElementById("first_password");
   // Since password isn't generated (just suitable field was detected),
-  // |OnFieldAutofilled| wouldn't trigger any actions.
+  // `OnFieldAutofilled` wouldn't trigger any actions.
   EXPECT_CALL(fake_pw_client_, PasswordNoLongerGenerated(testing::_)).Times(0);
   password_generation_->OnFieldAutofilled(input_element);
 }
@@ -1268,7 +1295,7 @@ TEST_F(PasswordGenerationAgentTest, PasswordUnmaskedUntilCompleteDeletion) {
   testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
 
   // Delete characters of the generated password until only
-  // |kMinimumLengthForEditedPassword| - 1 chars remain.
+  // `kMinimumLengthForEditedPassword` - 1 chars remain.
   EXPECT_CALL(fake_pw_client_, ShowPasswordEditingPopup).Times(AtLeast(1));
   FocusField(kGenerationElementId);
   SimulateUserTypingASCIICharacter(ui::VKEY_END, false);
@@ -1286,9 +1313,7 @@ TEST_F(PasswordGenerationAgentTest, PasswordUnmaskedUntilCompleteDeletion) {
 
   // Check that the characters remain unmasked.
   WebDocument document = GetMainFrame()->GetDocument();
-  WebElement element = GetElementById(kGenerationElementId);
-  ASSERT_FALSE(element.IsNull());
-  blink::WebInputElement input = element.To<WebInputElement>();
+  WebInputElement input = GetInputElementById(kGenerationElementId);
   EXPECT_TRUE(input.ShouldRevealPassword());
 
   // Delete the rest of the characters. The field should now mask new
@@ -1320,7 +1345,7 @@ TEST_F(PasswordGenerationAgentTest, ShortPasswordMaskedAfterChangingFocus) {
   testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
 
   // Delete characters of the generated password until only
-  // |kMinimumLengthForEditedPassword| - 1 chars remain.
+  // `kMinimumLengthForEditedPassword` - 1 chars remain.
   EXPECT_CALL(fake_pw_client_, ShowPasswordEditingPopup).Times(AtLeast(1));
   FocusField(kGenerationElementId);
   EXPECT_CALL(fake_pw_client_, PasswordNoLongerGenerated(testing::_));
@@ -1337,10 +1362,7 @@ TEST_F(PasswordGenerationAgentTest, ShortPasswordMaskedAfterChangingFocus) {
   testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
 
   // Check that the characters remain unmasked.
-  WebDocument document = GetMainFrame()->GetDocument();
-  WebElement element = GetElementById(kGenerationElementId);
-  ASSERT_FALSE(element.IsNull());
-  blink::WebInputElement input = element.To<WebInputElement>();
+  WebInputElement input = GetInputElementById(kGenerationElementId);
   EXPECT_TRUE(input.ShouldRevealPassword());
 
   // Focus another element on the page. The password should be masked.
@@ -1363,8 +1385,7 @@ TEST_F(PasswordGenerationAgentTest, GenerationAvailableByRendererIds) {
   WebDocument document = GetMainFrame()->GetDocument();
   std::vector<WebInputElement> password_elements;
   for (const char* id : kPasswordElementsIds) {
-    WebElement element = GetElementById(id);
-    WebInputElement input = element.To<WebInputElement>();
+    WebInputElement input = GetInputElementById(id);
     password_elements.push_back(input);
   }
 
@@ -1389,47 +1410,69 @@ TEST_F(PasswordGenerationAgentTest, GenerationAvailableByRendererIds) {
   ExpectAutomaticGenerationAvailable(kPasswordElementsIds[2], kAvailable);
 }
 
-TEST_F(PasswordGenerationAgentTest, SuggestionPreviewTest) {
+TEST_F(PasswordGenerationAgentTest, SuggestionPreviewedAndClearedTest) {
   LoadHTMLWithUserGesture(kAccountCreationFormHTML);
   WebDocument document = GetMainFrame()->GetDocument();
   SetFoundFormEligibleForGeneration(password_generation_,
                                     GetMainFrame()->GetDocument(),
-                                    "first_password" /* new_password_id */,
-                                    "second_password" /* confirm_password_id*/);
+                                    /*new_password_id=*/"first_password",
+                                    /*confirm_password_id=*/"second_password");
   ExpectAutomaticGenerationAvailable("first_password", kAvailable);
 
-  WebElement element =
-      document.GetElementById(WebString::FromUTF8("first_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement first_password_element = element.To<WebInputElement>();
-  element = document.GetElementById(WebString::FromUTF8("second_password"));
-  ASSERT_FALSE(element.IsNull());
-  WebInputElement second_password_element = element.To<WebInputElement>();
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
+  WebInputElement second_password_element =
+      GetInputElementById("second_password");
 
   std::u16string password = u"random_password";
   password_generation_->PreviewGenerationSuggestion(password);
 
   // Both password fields should have suggested values.
-  EXPECT_EQ(password, first_password_element.SuggestedValue().Utf16());
-  EXPECT_EQ(password, second_password_element.SuggestedValue().Utf16());
-  EXPECT_EQ(first_password_element.GetAutofillState(),
-            blink::WebAutofillState::kPreviewed);
-  EXPECT_EQ(second_password_element.GetAutofillState(),
-            blink::WebAutofillState::kPreviewed);
+  CheckPreviewedValue(first_password_element, password);
+  CheckPreviewedValue(second_password_element, password);
 
   // Previewed suggestions should be successfully cleared upon request.
-  EXPECT_TRUE(password_generation_->DidClearGenerationSuggestion(
-      first_password_element));
+  password_generation_->ClearPreviewedForm();
   EXPECT_TRUE(first_password_element.SuggestedValue().IsNull());
   EXPECT_TRUE(second_password_element.SuggestedValue().IsNull());
   EXPECT_EQ(first_password_element.GetAutofillState(),
             blink::WebAutofillState::kNotFilled);
   EXPECT_EQ(second_password_element.GetAutofillState(),
             blink::WebAutofillState::kNotFilled);
+}
 
-  // Clearing should not succeed when there is nothing to clear.
-  EXPECT_FALSE(password_generation_->DidClearGenerationSuggestion(
-      first_password_element));
+TEST_F(PasswordGenerationAgentTest, SuggestionPreviewedAndFilledTest) {
+  LoadHTMLWithUserGesture(kAccountCreationFormHTML);
+  WebDocument document = GetMainFrame()->GetDocument();
+  SetFoundFormEligibleForGeneration(password_generation_,
+                                    GetMainFrame()->GetDocument(),
+                                    /*new_password_id=*/"first_password",
+                                    /*confirm_password_id=*/"second_password");
+  ExpectAutomaticGenerationAvailable("first_password", kAvailable);
+
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
+  WebInputElement second_password_element =
+      GetInputElementById("second_password");
+
+  std::u16string password = u"random_password";
+  password_generation_->PreviewGenerationSuggestion(password);
+
+  // Both password fields should have suggested values.
+  CheckPreviewedValue(first_password_element, password);
+  CheckPreviewedValue(second_password_element, password);
+
+  // Previewed suggestion should be successfully cleared when the
+  // suggestion is accepted.
+  EXPECT_CALL(fake_pw_client_, PresaveGeneratedPassword(_, Eq(password)));
+  password_generation_->GeneratedPasswordAccepted(password);
+
+  EXPECT_TRUE(first_password_element.SuggestedValue().IsNull());
+  EXPECT_TRUE(second_password_element.SuggestedValue().IsNull());
+  EXPECT_EQ(first_password_element.GetAutofillState(),
+            blink::WebAutofillState::kAutofilled);
+  EXPECT_EQ(second_password_element.GetAutofillState(),
+            blink::WebAutofillState::kAutofilled);
 }
 
 TEST_F(PasswordGenerationAgentTest, AdvancesFocusToNextFieldAfterPasswords) {
@@ -1443,8 +1486,76 @@ TEST_F(PasswordGenerationAgentTest, AdvancesFocusToNextFieldAfterPasswords) {
 
   password_generation_->FocusNextFieldAfterPasswords();
   WebDocument document = GetMainFrame()->GetDocument();
-  EXPECT_EQ(document.FocusedElement(),
-            document.GetElementById(WebString::FromUTF8("address")));
+  EXPECT_EQ(document.FocusedElement(), GetElementById("address"));
 }
 
+TEST_F(PasswordGenerationAgentTest,
+       MasksPasswordAfterReplacingTextBySelectingAll) {
+  LoadHTMLWithUserGesture(kAccountCreationFormHTML);
+  SetFoundFormEligibleForGeneration(password_generation_,
+                                    GetMainFrame()->GetDocument(),
+                                    /*new_password_id=*/"first_password",
+                                    /*confirm_password_id=*/"second_password");
+
+  WebInputElement first_password_element =
+      GetInputElementById("first_password");
+  WebInputElement second_password_element =
+      GetInputElementById("second_password");
+
+  // Generate password.
+  ExpectAutomaticGenerationAvailable("first_password", kAvailable);
+  std::u16string password = u"random_password";
+  EXPECT_CALL(fake_pw_client_, PresaveGeneratedPassword(_, Eq(password)));
+  password_generation_->GeneratedPasswordAccepted(password);
+  fake_pw_client_.Flush();
+  testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
+
+  // Verify that values in both fields are mirrored.
+  EXPECT_EQ(first_password_element.Value().Utf16(), password);
+  EXPECT_EQ(second_password_element.Value().Utf16(), password);
+
+  // Focus first password field, select all characters and type a letter.
+  EXPECT_CALL(fake_pw_client_, ShowPasswordEditingPopup).Times(AtLeast(1));
+  FocusField("first_password");
+  first_password_element.SetSelectionRange(0, password.length());
+  fake_pw_client_.Flush();
+  testing::Mock::VerifyAndClearExpectations(&fake_pw_client_);
+  EXPECT_CALL(fake_pw_client_, PasswordNoLongerGenerated);
+  EXPECT_CALL(fake_pw_client_, AutomaticGenerationAvailable).Times(AtLeast(1));
+  SimulateUserTypingASCIICharacter('X', /*flush_message_loop=*/true);
+
+  // First field should contain the typed letter, second field should be empty
+  // since the password is no longer generated. Both fields should be masked.
+  EXPECT_EQ(first_password_element.Value().Utf16(), u"X");
+  EXPECT_TRUE(second_password_element.Value().Utf16().empty());
+  EXPECT_FALSE(first_password_element.ShouldRevealPassword());
+  EXPECT_FALSE(second_password_element.ShouldRevealPassword());
+}
+
+// Tests that automatic generation is suggested after the field has become a
+// password field at least once.
+TEST_F(PasswordGenerationAgentTest, AutomaticSuggestionOnHasBeenPasswordField) {
+  LoadHTML(
+      R"(
+      <input type="text" id="username-field" name="username-field">
+      <input type="password" id="password-field" name="password-field">
+    )");
+  constexpr char kPasswordElementId[] = "password-field";
+
+  WebInputElement password_field = GetInputElementById(kPasswordElementId);
+
+  // Simulate parser finding the field to trigger generation on.
+  SetFoundFormEligibleForGeneration(
+      password_generation_, GetMainFrame()->GetDocument(),
+      /*new_password_id=*/kPasswordElementId, /*confirm_password_id=*/nullptr);
+
+  // User clicks on reveal password button, changing the field type to
+  // text.
+  password_field.SetAttribute("type", "text");
+
+  // Automatic generation should still be available.
+  ExpectAutomaticGenerationAvailable(kPasswordElementId, kAvailable);
+}
+
+}  // namespace
 }  // namespace autofill
