@@ -28,12 +28,13 @@
 #include "base/threading/platform_thread.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
-#include "chrome/common/child_process_logging.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/media/media_resource_provider.h"
 #include "chrome/common/net/net_resource_provider.h"
+#include "chrome/common/privacy_budget/privacy_budget_settings_provider.h"
 #include "chrome/common/renderer_configuration.mojom.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/renderer/process_state.h"
 #include "components/visitedlink/renderer/visitedlink_reader.h"
 #include "content/public/child/child_thread.h"
 #include "content/public/common/content_switches.h"
@@ -47,6 +48,7 @@
 #include "net/base/net_module.h"
 #include "third_party/blink/public/common/associated_interfaces/associated_interface_registry.h"
 #include "third_party/blink/public/common/features.h"
+#include "third_party/blink/public/common/privacy_budget/identifiability_study_settings.h"
 #include "third_party/blink/public/platform/web_string.h"
 #include "third_party/blink/public/platform/web_url.h"
 #include "third_party/blink/public/web/web_document.h"
@@ -61,7 +63,6 @@
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
 #include "chrome/renderer/bound_session_credentials/bound_session_request_throttled_handler_renderer_impl.h"
 #include "chrome/renderer/bound_session_credentials/bound_session_request_throttled_in_renderer_manager.h"
-#include "components/signin/public/base/signin_switches.h"
 #endif
 
 using blink::WebCache;
@@ -83,8 +84,6 @@ scoped_refptr<base::SequencedTaskRunner> GetCallbackGroupTaskRunner() {
 
 }  // namespace
 
-bool ChromeRenderThreadObserver::is_incognito_process_ = false;
-
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 // static
 scoped_refptr<ChromeRenderThreadObserver::ChromeOSListener>
@@ -164,7 +163,6 @@ ChromeRenderThreadObserver::CreateBoundSessionRequestThrottledHandler() const {
     return nullptr;
   }
 
-  CHECK(switches::IsBoundSessionCredentialsEnabled());
   return std::make_unique<BoundSessionRequestThrottledHandlerRendererImpl>(
       bound_session_request_throttled_in_renderer_manager_, io_task_runner_);
 }
@@ -176,12 +174,20 @@ void ChromeRenderThreadObserver::RegisterMojoInterfaces(
       base::BindRepeating(
           &ChromeRenderThreadObserver::OnRendererConfigurationAssociatedRequest,
           base::Unretained(this)));
+  associated_interfaces
+      ->AddInterface<chrome::mojom::IdentifiabilityStudyConfigurator>(
+          base::BindRepeating(
+              &ChromeRenderThreadObserver::
+                  OnIdentifiabilityStudyConfiguratorAssociatedRequest,
+              base::Unretained(this)));
 }
 
 void ChromeRenderThreadObserver::UnregisterMojoInterfaces(
     blink::AssociatedInterfaceRegistry* associated_interfaces) {
   associated_interfaces->RemoveInterface(
       chrome::mojom::RendererConfiguration::Name_);
+  associated_interfaces->RemoveInterface(
+      chrome::mojom::IdentifiabilityStudyConfigurator::Name_);
 }
 
 void ChromeRenderThreadObserver::SetInitialConfiguration(
@@ -194,7 +200,7 @@ void ChromeRenderThreadObserver::SetInitialConfiguration(
         bound_session_request_throttled_handler) {
   if (content_settings_manager)
     content_settings_manager_.Bind(std::move(content_settings_manager));
-  is_incognito_process_ = is_incognito_process;
+  chrome::SetIsIncognitoProcess(is_incognito_process);
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   if (chromeos_listener_receiver) {
     chromeos_listener_ =
@@ -204,7 +210,6 @@ void ChromeRenderThreadObserver::SetInitialConfiguration(
 
 #if BUILDFLAG(ENABLE_BOUND_SESSION_CREDENTIALS)
   if (bound_session_request_throttled_handler) {
-    CHECK(switches::IsBoundSessionCredentialsEnabled());
     bound_session_request_throttled_in_renderer_manager_ =
         BoundSessionRequestThrottledInRendererManager::Create(
             std::move(bound_session_request_throttled_handler));
@@ -219,8 +224,26 @@ void ChromeRenderThreadObserver::SetConfiguration(
   dynamic_params_ = std::move(params);
 }
 
+void ChromeRenderThreadObserver::ConfigureIdentifiabilityStudy(
+    bool meta_experiment_active) {
+  // This is superfluous in single-process mode and triggers a DCHECK
+  if (!base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kSingleProcess)) {
+    blink::IdentifiabilityStudySettings::SetGlobalProvider(
+        std::make_unique<PrivacyBudgetSettingsProvider>(
+            meta_experiment_active));
+  }
+}
+
 void ChromeRenderThreadObserver::OnRendererConfigurationAssociatedRequest(
     mojo::PendingAssociatedReceiver<chrome::mojom::RendererConfiguration>
         receiver) {
   renderer_configuration_receivers_.Add(this, std::move(receiver));
 }
+
+void ChromeRenderThreadObserver::
+    OnIdentifiabilityStudyConfiguratorAssociatedRequest(
+        mojo::PendingAssociatedReceiver<
+            chrome::mojom::IdentifiabilityStudyConfigurator> receiver) {
+  identifiability_study_configurator_receivers_.Add(this, std::move(receiver));
+}
