@@ -2,9 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/test/chromedriver/capabilities.h"
 
 #include <map>
+#include <string_view>
 #include <utility>
 
 #include "base/containers/contains.h"
@@ -14,7 +20,6 @@
 #include "base/json/string_escape.h"
 #include "base/logging.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_tokenizer.h"
 #include "base/strings/string_util.h"
@@ -28,6 +33,7 @@
 #include "chrome/test/chromedriver/chrome/status.h"
 #include "chrome/test/chromedriver/constants/version.h"
 #include "chrome/test/chromedriver/logging.h"
+#include "chrome/test/chromedriver/prompt_behavior.h"
 #include "chrome/test/chromedriver/session.h"
 #include "chrome/test/chromedriver/util.h"
 
@@ -164,8 +170,8 @@ Status ParseMobileEmulation(const base::Value& option,
     }
     mobile_device.user_agent = *user_agent;
 
-    mobile_ua = base::StringPiece{*user_agent}.find("Mobile") !=
-                base::StringPiece::npos;
+    mobile_ua =
+        std::string_view{*user_agent}.find("Mobile") != std::string_view::npos;
   }
 
   if (mobile_emulation->Find("deviceMetrics")) {
@@ -491,23 +497,18 @@ Status ParsePageLoadStrategy(const base::Value& option,
   return Status(kInvalidArgument, "invalid 'pageLoadStrategy'");
 }
 
-Status ParseUnhandledPromptBehavior(const base::Value& option,
+Status ParseUnhandledPromptBehavior(bool w3c_compliant,
+                                    const base::Value& option,
                                     Capabilities* capabilities) {
-  if (!option.is_string()) {
-    return Status(kInvalidArgument,
-                  "'unhandledPromptBehavior' must be a string");
+  PromptBehavior unhandled_prompt_behavior(w3c_compliant);
+  Status status =
+      PromptBehavior::Create(w3c_compliant, option, unhandled_prompt_behavior);
+  if (status.IsError()) {
+    return status;
   }
-  capabilities->unhandled_prompt_behavior = option.GetString();
-  if (capabilities->unhandled_prompt_behavior == prompt_behavior::kDismiss ||
-      capabilities->unhandled_prompt_behavior == prompt_behavior::kAccept ||
-      capabilities->unhandled_prompt_behavior ==
-          prompt_behavior::kDismissAndNotify ||
-      capabilities->unhandled_prompt_behavior ==
-          prompt_behavior::kAcceptAndNotify ||
-      capabilities->unhandled_prompt_behavior == prompt_behavior::kIgnore) {
-    return Status(kOk);
-  }
-  return Status(kInvalidArgument, "invalid 'unhandledPromptBehavior'");
+  capabilities->unhandled_prompt_behavior =
+      std::move(unhandled_prompt_behavior);
+  return Status(kOk);
 }
 
 Status ParseTimeouts(const base::Value& option, Capabilities* capabilities) {
@@ -815,7 +816,7 @@ Status ParseWindowTypes(const base::Value& option, Capabilities* capabilities) {
       return Status(kInvalidArgument, "each window type must be a string");
     }
     WebViewInfo::Type type;
-    Status status = WebViewInfo::ParseType(window_type.GetString(), &type);
+    Status status = WebViewInfo::ParseType(window_type.GetString(), type);
     if (status.IsError())
       return status;
     window_types_tmp.insert(type);
@@ -997,7 +998,7 @@ void Switches::SetFromSwitches(const Switches& switches) {
 }
 
 namespace {
-constexpr auto kMultivaluedSwitches = base::MakeFixedFlatSet<base::StringPiece>(
+constexpr auto kMultivaluedSwitches = base::MakeFixedFlatSet<std::string_view>(
     {"enable-blink-features", "disable-blink-features", "enable-features",
      "disable-features"});
 }  // namespace
@@ -1118,16 +1119,16 @@ Status Capabilities::Parse(const base::Value::Dict& desired_caps,
   parser_map["strictFileInteractability"] =
       base::BindRepeating(&ParseBoolean, &strict_file_interactability);
   parser_map["webSocketUrl"] =
-      base::BindRepeating(&ParseBoolean, &webSocketUrl);
+      base::BindRepeating(&ParseBoolean, &web_socket_url);
   if (!w3c_compliant) {
     // TODO(https://crbug.com/chromedriver/2596): "unexpectedAlertBehaviour" is
     // legacy name of "unhandledPromptBehavior", remove when we stop supporting
     // legacy mode.
     parser_map["unexpectedAlertBehaviour"] =
-        base::BindRepeating(&ParseUnhandledPromptBehavior);
+        base::BindRepeating(&ParseUnhandledPromptBehavior, w3c_compliant);
   }
   parser_map["unhandledPromptBehavior"] =
-      base::BindRepeating(&ParseUnhandledPromptBehavior);
+      base::BindRepeating(&ParseUnhandledPromptBehavior, w3c_compliant);
 
   // W3C defined extension capabilities.
   // See https://w3c.github.io/webauthn/#sctn-automation-webdriver-capability
