@@ -35,7 +35,6 @@
 #include "chrome/test/chromedriver/chrome/devtools_client_impl.h"
 #include "chrome/test/chromedriver/chrome/devtools_event_listener.h"
 #include "chrome/test/chromedriver/chrome/geoposition.h"
-#include "chrome/test/chromedriver/chrome/javascript_dialog_manager.h"
 #include "chrome/test/chromedriver/chrome/status.h"
 #include "chrome/test/chromedriver/chrome/web_view.h"
 #include "chrome/test/chromedriver/chrome_launcher.h"
@@ -44,6 +43,7 @@
 #include "chrome/test/chromedriver/logging.h"
 #include "chrome/test/chromedriver/session.h"
 #include "chrome/test/chromedriver/util.h"
+#include "services/device/public/cpp/generic_sensor/orientation_util.h"
 #include "services/network/public/mojom/url_loader_factory.mojom.h"
 
 namespace {
@@ -71,11 +71,9 @@ Status EvaluateScriptAndIgnoreResult(Session* session,
   Status status = session->GetTargetWindow(&web_view);
   if (status.IsError())
     return status;
-  if (!web_view->IsServiceWorker() &&
-      web_view->GetJavaScriptDialogManager()->IsDialogOpen()) {
+  if (!web_view->IsServiceWorker() && web_view->IsDialogOpen()) {
     std::string alert_text;
-    status =
-        web_view->GetJavaScriptDialogManager()->GetDialogMessage(&alert_text);
+    status = web_view->GetDialogMessage(alert_text);
     if (status.IsError())
       return Status(kUnexpectedAlertOpen);
     return Status(kUnexpectedAlertOpen, "{Alert text : " + alert_text + "}");
@@ -85,6 +83,16 @@ Status EvaluateScriptAndIgnoreResult(Session* session,
   return web_view->EvaluateScript(frame_id, expression, await_promise, &result);
 }
 
+base::RepeatingCallback<Status(bool*)> BidiResponseIsReceivedCallback(
+    Session* session) {
+  return base::BindRepeating(
+      [](Session* session, bool* condition_is_met) {
+        *condition_is_met = !session->awaiting_bidi_response;
+        return Status{kOk};
+      },
+      base::Unretained(session));
+}
+
 }  // namespace
 
 InitSessionParams::InitSessionParams(
@@ -203,7 +211,7 @@ base::Value::Dict CreateCapabilities(Session* session,
   caps.Set("strictFileInteractability", session->strict_file_interactability);
   caps.Set(session->w3c_compliant ? "unhandledPromptBehavior"
                                   : "unexpectedAlertBehaviour",
-           session->unhandled_prompt_behavior);
+           session->unhandled_prompt_behavior.CapabilityView());
 
   // Extensions defined by the W3C.
   // See https://w3c.github.io/webauthn/#sctn-automation-webdriver-capability
@@ -258,7 +266,7 @@ base::Value::Dict CreateCapabilities(Session* session,
     caps.Set("hasTouchScreen", session->chrome->HasTouchScreen());
   }
 
-  if (session->webSocketUrl) {
+  if (session->web_socket_url) {
     caps.Set("webSocketUrl",
              "ws://" + session->host + "/session/" + session->id);
   }
@@ -299,7 +307,7 @@ Status InitSessionHelper(const InitSessionParams& bound_params,
   // |session| will own the |CommandListener|s.
   session->command_listeners.swap(command_listeners);
 
-  if (session->webSocketUrl) {
+  if (session->web_socket_url) {
     // Suffixes used with the client channels.
     std::string client_suffixes[] = {Session::kChannelSuffix,
                                      Session::kNoChannelSuffix,
@@ -349,15 +357,36 @@ Status InitSessionHelper(const InitSessionParams& bound_params,
     *value = std::make_unique<base::Value>(session->capabilities->Clone());
   }
 
-  if (session->webSocketUrl) {
+  if (session->web_socket_url) {
     WebView* web_view = nullptr;
     status = session->GetTargetWindow(&web_view);
     if (status.IsError())
       return status;
     session->bidi_mapper_web_view_id = session->window;
 
+    // Create a new tab because the default one will be occupied by the
+    // mapper. The new tab is activated and focused.
+    std::string web_view_id;
+    status = session->chrome->NewWindow(
+        session->window, Chrome::WindowType::kTab, false, &web_view_id);
+
+    if (status.IsError()) {
+      return status;
+    }
+
+    std::unique_ptr<base::Value> result;
+    base::Value::Dict body;
+    body.Set("handle", web_view_id);
+
+    // Even though the new tab is already activated the explicit switch to the
+    // new tab is needed to update the internal state of ChromeDriver properly.
+    status = ExecuteSwitchToWindow(session, body, &result);
+    if (status.IsError()) {
+      return status;
+    }
+
     // Wait until the default page navigation is over to prevent the mapper
-    // from begin evicted by the navigation.
+    // from being evicted by the navigation.
     status = web_view->WaitForPendingNavigations(
         session->GetCurrentFrameId(), Timeout(session->page_load_timeout),
         true);
@@ -365,6 +394,7 @@ Status InitSessionHelper(const InitSessionParams& bound_params,
       return status;
     }
 
+    // Start the mapper.
     base::CommandLine* cmd_line = base::CommandLine::ForCurrentProcess();
     base::FilePath bidi_mapper_path =
         cmd_line->GetSwitchValuePath("bidi-mapper-path");
@@ -381,31 +411,24 @@ Status InitSessionHelper(const InitSessionParams& bound_params,
       }
     }
 
-    base::Value::Dict mapper_options;
-    mapper_options.Set("acceptInsecureCerts",
-                       capabilities.accept_insecure_certs);
-    mapper_options.Set("sharedIdWithFrame", true);
-    status = web_view->StartBidiServer(mapper_script, mapper_options);
+    status = web_view->StartBidiServer(mapper_script);
     if (status.IsError()) {
       return status;
     }
 
-    {
-      // Create a new tab because the default one is occupied by the BiDiMapper
-      std::string web_view_id;
-      status = session->chrome->NewWindow(
-          session->window, Chrome::WindowType::kTab, &web_view_id);
-
-      if (status.IsError())
-        return status;
-
-      std::unique_ptr<base::Value> result;
-      base::Value::Dict body;
-      body.Set("handle", web_view_id);
-
-      status = ExecuteSwitchToWindow(session, body, &result);
+    // Execute session.new for the newly-created mapper instance.
+    base::Value::Dict bidi_cmd;
+    bidi_cmd.Set("channel", "/init-bidi-session");
+    bidi_cmd.Set("id", 1);
+    bidi_cmd.Set("params", params.Clone());
+    bidi_cmd.Set("method", "session.new");
+    base::Value::Dict bidi_response;
+    status = web_view->SendBidiCommand(
+        std::move(bidi_cmd), Timeout(base::Seconds(20)), bidi_response);
+    if (status.IsError()) {
+      return status;
     }
-  }
+  }  // if (session->web_socket_url)
 
   return status;
 }
@@ -439,16 +462,11 @@ Status ConfigureSession(Session* session,
   if (status.IsError())
     return status;
 
-  if (capabilities->unhandled_prompt_behavior.length() > 0) {
+  if (capabilities->unhandled_prompt_behavior) {
     session->unhandled_prompt_behavior =
-        capabilities->unhandled_prompt_behavior;
+        std::move(capabilities->unhandled_prompt_behavior).value();
   } else {
-    // W3C spec (https://www.w3.org/TR/webdriver/#dfn-handle-any-user-prompts)
-    // shows the default behavior to be dismiss and notify. For backward
-    // compatibility, in legacy mode default behavior is not handling prompt.
-    session->unhandled_prompt_behavior =
-        session->w3c_compliant ? ::prompt_behavior::kDismissAndNotify
-                               : ::prompt_behavior::kIgnore;
+    session->unhandled_prompt_behavior = PromptBehavior(session->w3c_compliant);
   }
 
   session->implicit_wait = capabilities->implicit_wait_timeout;
@@ -456,7 +474,7 @@ Status ConfigureSession(Session* session,
   session->script_timeout = capabilities->script_timeout;
   session->strict_file_interactability =
       capabilities->strict_file_interactability;
-  session->webSocketUrl = capabilities->webSocketUrl;
+  session->web_socket_url = capabilities->web_socket_url;
   Log::Level driver_level = Log::kWarning;
   if (capabilities->logging_prefs.count(WebDriverLog::kDriverType))
     driver_level = capabilities->logging_prefs[WebDriverLog::kDriverType];
@@ -775,7 +793,7 @@ Status ExecuteClose(Session* session,
   if (status.IsError())
     return status;
   bool is_last_web_view = web_view_ids.size() == 1u;
-  if (session->webSocketUrl) {
+  if (session->web_socket_url) {
     is_last_web_view = web_view_ids.size() <= 2u;
   }
   web_view_ids.clear();
@@ -789,34 +807,36 @@ Status ExecuteClose(Session* session,
   if (status.IsError())
     return status;
 
-
-  JavaScriptDialogManager* dialog_manager =
-      web_view->GetJavaScriptDialogManager();
-  if (dialog_manager->IsDialogOpen()) {
+  if (web_view->IsDialogOpen()) {
     std::string alert_text;
-    status = dialog_manager->GetDialogMessage(&alert_text);
+    status = web_view->GetDialogMessage(alert_text);
     if (status.IsError())
       return status;
 
-    // Close the dialog depending on the unexpectedalert behaviour set by user
-    // before returning an error, so that subsequent commands do not fail.
-    const std::string& prompt_behavior = session->unhandled_prompt_behavior;
-
-    if (prompt_behavior == ::prompt_behavior::kAccept ||
-        prompt_behavior == ::prompt_behavior::kAcceptAndNotify) {
-      status = dialog_manager->HandleDialog(true, session->prompt_text.get());
-    } else if (prompt_behavior == ::prompt_behavior::kDismiss ||
-               prompt_behavior == ::prompt_behavior::kDismissAndNotify) {
-      status = dialog_manager->HandleDialog(false, session->prompt_text.get());
+    std::string dialog_type;
+    status = web_view->GetTypeOfDialog(dialog_type);
+    if (status.IsError()) {
+      return status;
     }
-    if (status.IsError())
-      return status;
 
-    // For backward compatibility, in legacy mode we always notify.
-    if (!session->w3c_compliant ||
-        prompt_behavior == ::prompt_behavior::kAcceptAndNotify ||
-        prompt_behavior == ::prompt_behavior::kDismissAndNotify ||
-        prompt_behavior == ::prompt_behavior::kIgnore) {
+    PromptHandlerConfiguration prompt_handler_configuration;
+    status = session->unhandled_prompt_behavior.GetConfiguration(
+        dialog_type, prompt_handler_configuration);
+    if (status.IsError()) {
+      return status;
+    }
+
+    if (prompt_handler_configuration.type == PromptHandlerType::kAccept ||
+        prompt_handler_configuration.type == PromptHandlerType::kDismiss) {
+      status = web_view->HandleDialog(
+          prompt_handler_configuration.type == PromptHandlerType::kAccept,
+          session->prompt_text);
+      if (status.IsError()) {
+        return status;
+      }
+    }
+
+    if (prompt_handler_configuration.notify) {
       return Status(kUnexpectedAlertOpen, "{Alert text : " + alert_text + "}");
     }
   }
@@ -846,10 +866,11 @@ Status ExecuteGetWindowHandles(Session* session,
   std::list<std::string> web_view_ids;
   Status status = session->chrome->GetWebViewIds(&web_view_ids,
                                                  session->w3c_compliant);
-  if (status.IsError())
+  if (status.IsError()) {
     return status;
+  }
 
-  if (session->webSocketUrl) {
+  if (session->web_socket_url) {
     auto it =
         base::ranges::find(web_view_ids, session->bidi_mapper_web_view_id);
     if (it != web_view_ids.end()) {
@@ -906,8 +927,10 @@ Status ExecuteSwitchToWindow(Session* session,
       std::unique_ptr<base::Value> result;
       WebView* web_view;
       status = session->chrome->GetWebViewById(*it, &web_view);
+      // `CallFunction(...)` below may remove web views that detach during this
+      // loop. In that case, continue searching.
       if (status.IsError())
-        return status;
+        continue;
       status = web_view->CallFunction(
           std::string(), kGetWindowNameScript, args, &result);
       if (status.IsError())
@@ -1168,30 +1191,31 @@ bool ParseXYZValue(const base::Value::Dict& params,
   return true;
 }
 
-bool ParseOrientationQuaternion(const base::Value::Dict& params,
-                                base::Value::Dict* out_params) {
-  constexpr size_t kQuaternionListSize = 4;  // x, y, z, w
+bool ParseOrientationEuler(const base::Value::Dict& params,
+                           base::Value::Dict* out_params) {
+  if (!params.contains("alpha") || !params.contains("beta") ||
+      !params.contains("gamma")) {
+    return false;
+  }
 
-  const base::Value::List* value_list = params.FindList("quaternion");
-  if (!value_list || value_list->size() != kQuaternionListSize) {
+  std::optional<double> alpha = params.FindDouble("alpha");
+  if (!alpha.has_value()) {
     return false;
   }
-  std::optional<double> x = (*value_list)[0].GetIfDouble();
-  if (!x.has_value()) {
+  std::optional<double> beta = params.FindDouble("beta");
+  if (!beta.has_value()) {
     return false;
   }
-  std::optional<double> y = (*value_list)[1].GetIfDouble();
-  if (!y.has_value()) {
+  std::optional<double> gamma = params.FindDouble("gamma");
+  if (!gamma.has_value()) {
     return false;
   }
-  std::optional<double> z = (*value_list)[2].GetIfDouble();
-  if (!z.has_value()) {
-    return false;
-  }
-  std::optional<double> w = (*value_list)[3].GetIfDouble();
-  if (!w.has_value()) {
+  device::SensorReading quaternion_readings;
+  if (!device::ComputeQuaternionFromEulerAngles(*alpha, *beta, *gamma,
+                                                &quaternion_readings)) {
     return false;
   }
+
   // Construct a dict that looks like this:
   // {
   //   quaternion: {
@@ -1201,9 +1225,13 @@ bool ParseOrientationQuaternion(const base::Value::Dict& params,
   //     w: VAL4
   //   }
   // }
+  const double x = quaternion_readings.orientation_quat.x;
+  const double y = quaternion_readings.orientation_quat.y;
+  const double z = quaternion_readings.orientation_quat.z;
+  const double w = quaternion_readings.orientation_quat.w;
   out_params->Set(
       "quaternion",
-      base::Value::Dict().Set("x", *x).Set("y", *y).Set("z", *z).Set("w", *w));
+      base::Value::Dict().Set("x", x).Set("y", y).Set("z", z).Set("w", w));
   return true;
 }
 
@@ -1239,9 +1267,10 @@ base::expected<base::Value::Dict, Status> ParseSensorUpdateParams(
     }
   } else if (*type == "absolute-orientation" ||
              *type == "relative-orientation") {
-    if (!ParseOrientationQuaternion(*reading_dict, &reading)) {
-      return base::unexpected(
-          Status(kInvalidArgument, "Could not parse quaternion"));
+    if (!ParseOrientationEuler(*reading_dict, &reading)) {
+      return base::unexpected(Status(
+          kInvalidArgument, "Could not parse " + *type +
+                                " readings. Invalid alpha/beta/gamma values"));
     }
   } else {
     return base::unexpected(Status(
@@ -1256,7 +1285,7 @@ base::expected<base::Value::Dict, Status> ParseSensorUpdateParams(
 
 Status ExecuteUpdateVirtualSensor(Session* session,
                                   const base::Value::Dict& params,
-                                  std::unique_ptr<base::Value>* value) {
+                                  std::unique_ptr<base::Value>*) {
   WebView* web_view = nullptr;
   Status status = session->GetTargetWindow(&web_view);
   if (status.IsError()) {
@@ -1268,8 +1297,8 @@ Status ExecuteUpdateVirtualSensor(Session* session,
     return cdp_params.error();
   }
 
-  return web_view->SendCommandAndGetResult(
-      "Emulation.setSensorOverrideReadings", cdp_params.value(), value);
+  return web_view->SendCommand("Emulation.setSensorOverrideReadings",
+                               cdp_params.value());
 }
 
 Status ExecuteRemoveVirtualSensor(Session* session,
@@ -1640,6 +1669,85 @@ Status ExecuteSetTimeZone(Session* session,
   return Status(kOk);
 }
 
+Status ExecuteCreateVirtualPressureSource(Session* session,
+                                          const base::Value::Dict& params,
+                                          std::unique_ptr<base::Value>* value) {
+  WebView* web_view = nullptr;
+  Status status = session->GetTargetWindow(&web_view);
+  if (status.IsError()) {
+    return status;
+  }
+
+  const std::string* type = params.FindString("type");
+  if (!type) {
+    return Status(kInvalidArgument, "'type' must be a string");
+  }
+
+  base::Value::Dict body;
+  body.Set("enabled", true);
+  body.Set("source", *type);
+
+  base::Value::Dict metadata;
+  metadata.Set("available", true);
+  if (params.contains("supported")) {
+    auto supported = params.FindBool("supported");
+    if (!supported.has_value()) {
+      return Status(kInvalidArgument, "'supported' must be a boolean");
+    }
+    metadata.Set("available", *supported);
+  }
+  body.Set("metadata", std::move(metadata));
+
+  return web_view->SendCommand("Emulation.setPressureSourceOverrideEnabled",
+                               body);
+}
+
+Status ExecuteUpdateVirtualPressureSource(Session* session,
+                                          const base::Value::Dict& params,
+                                          std::unique_ptr<base::Value>* value) {
+  WebView* web_view = nullptr;
+  Status status = session->GetTargetWindow(&web_view);
+  if (status.IsError()) {
+    return status;
+  }
+
+  const std::string* type = params.FindString("type");
+  if (!type) {
+    return Status(kInvalidArgument, "'type' must be a string");
+  }
+
+  const std::string* sample = params.FindString("sample");
+  if (!sample) {
+    return Status(kInvalidArgument, "'sample' must be a string");
+  }
+
+  base::Value::Dict body;
+  body.Set("source", *type);
+  body.Set("state", *sample);
+  return web_view->SendCommand("Emulation.setPressureStateOverride", body);
+}
+
+Status ExecuteRemoveVirtualPressureSource(Session* session,
+                                          const base::Value::Dict& params,
+                                          std::unique_ptr<base::Value>* value) {
+  WebView* web_view = nullptr;
+  Status status = session->GetTargetWindow(&web_view);
+  if (status.IsError()) {
+    return status;
+  }
+
+  const std::string* type = params.FindString("type");
+  if (!type) {
+    return Status(kInvalidArgument, "'type' must be a string");
+  }
+
+  base::Value::Dict body;
+  body.Set("enabled", false);
+  body.Set("source", *type);
+  return web_view->SendCommand("Emulation.setPressureSourceOverrideEnabled",
+                               body);
+}
+
 // Run a BiDi command
 Status ForwardBidiCommand(Session* session,
                           const base::Value::Dict& params,
@@ -1688,20 +1796,14 @@ Status ForwardBidiCommand(Session* session,
     // This simplifies us closing the browser if the last tab was closed.
     session->awaiting_bidi_response = true;
     status = web_view->PostBidiCommand(std::move(bidi_cmd));
-    base::RepeatingCallback<Status(bool*)> bidi_response_is_received =
-        base::BindRepeating(
-            [](Session* session, bool* condition_is_met) {
-              *condition_is_met = !session->awaiting_bidi_response;
-              return Status{kOk};
-            },
-            base::Unretained(session));
     if (status.IsError()) {
       return status;
     }
 
     // The timeout is the same as in ChromeImpl::CloseTarget
-    status = web_view->HandleEventsUntil(std::move(bidi_response_is_received),
-                                         Timeout(base::Seconds(20)));
+    status = web_view->HandleEventsUntil(
+        std::move(BidiResponseIsReceivedCallback(session)),
+        Timeout(base::Seconds(20)));
     if (status.code() == kTimeout) {
       // It looks like something is going wrong with the BiDiMapper.
       // Terminating the session...
@@ -1713,14 +1815,14 @@ Status ForwardBidiCommand(Session* session,
       return status;
     }
 
-    std::list<std::string> web_view_ids;
-    status =
-        session->chrome->GetWebViewIds(&web_view_ids, session->w3c_compliant);
+    size_t web_view_count;
+    status = session->chrome->GetWebViewCount(&web_view_count,
+                                              session->w3c_compliant);
     if (status.IsError()) {
       return status;
     }
 
-    bool is_last_web_view = web_view_ids.size() <= 1u;
+    bool is_last_web_view = web_view_count <= 1u;
     if (is_last_web_view) {
       session->quit = true;
       status = session->chrome->Quit();
